package com.goodt.dremio.connector.asp;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;
import org.springframework.web.util.ContentCachingRequestWrapper;

import javax.servlet.http.HttpServletRequest;
import java.nio.charset.StandardCharsets;
import java.util.Arrays;
import java.util.Set;

import com.goodt.dremio.connector.dto.UserInfo;

@Slf4j
@Aspect
@Component
public class HibernateLoggingAspect {

    @Value("${app.logging.sensitive.enabled:false}")
    private boolean enabled;

    private final ObjectMapper om = new ObjectMapper();

    private static final Set<String> METHODS = Set.of("POST", "PUT", "DELETE");

    @Pointcut("execution(* org.springframework.data.jpa.repository.JpaRepository.save*(..))" +
            "|| execution(* org.springframework.data.jpa.repository.JpaRepository.delete*(..))" +
            "|| execution(* org.springframework.data.repository.CrudRepository.save*(..))" +
            "|| execution(* org.springframework.data.repository.CrudRepository.delete*(..))")
    public void repositoryMethod() {
    }

    @Around("repositoryMethod()")
    public Object logBeforeHibernateMethod(ProceedingJoinPoint joinPoint) throws Throwable {
        if (!enabled) {
            return joinPoint.proceed();
        }

        ServletRequestAttributes requestAttributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        if (requestAttributes == null) {
            return joinPoint.proceed();
        }

        HttpServletRequest request = requestAttributes.getRequest();
        if (!METHODS.contains(request.getMethod())) {
            return joinPoint.proceed();
        }

        final String employeeId = getSessionEmployeeId();
        final String data = new String(((ContentCachingRequestWrapper)request).getContentAsByteArray(), StandardCharsets.UTF_8);
        log.info("Employee: {}, Api: |{}|{}", employeeId, request.getMethod(), request.getRequestURL());
        if (StringUtils.hasText(data)) {
            log.info("Employee: {}, Change data: {}", employeeId, data);
        }

        String targetJson;
        try {
            targetJson = om.writeValueAsString(Arrays.asList(joinPoint.getArgs()));
        } catch (JsonProcessingException e) {
            targetJson = null;
        }

        log.info("Changes to entity: {}", targetJson);

        return joinPoint.proceed();
    }

    protected String getSessionEmployeeId() {
        UserInfo userInfo = (UserInfo) SecurityContextHolder.getContext().getAuthentication().getPrincipal();
        return userInfo.getEmployeeId() != null ? userInfo.getEmployeeId() : null;
    }
}
