package com.goodt.dremio.connector.parser.click_house;

import lombok.extern.slf4j.Slf4j;

import java.sql.SQLSyntaxErrorException;
import java.util.*;
import java.util.concurrent.atomic.AtomicReference;
import java.util.stream.Collectors;

import com.goodt.dremio.connector.dto.err.ConnectorException;
import com.goodt.dremio.connector.entity.query.QuerySchema;
import com.goodt.dremio.connector.enums.OrderType;
import com.goodt.dremio.connector.enums.contracts.IStrOperation;
import com.goodt.dremio.connector.exceptions.ParserException;
import com.goodt.dremio.connector.parser.AbstractQueryCreator;
import com.goodt.dremio.connector.parser.dremio.AggregateFun;
import com.goodt.dremio.connector.parser.dremio.EFilterType;

@Slf4j
public class ClickHouseQueryCreator extends AbstractQueryCreator {
    private static final Set<String> BLACK_LIST = new HashSet<>(Arrays.asList("SELECT", "JOIN"));

    private static final String WHERE = " WHERE ";

    private final QuerySchema querySchema;

    private final String schema;

    private final List<String> whereLst;

    private final List<String> havingLst;

    private final List<String> subLst;

    public ClickHouseQueryCreator(QuerySchema querySchema, String schema) {
        this.querySchema = querySchema;
        this.schema = schema;
        this.whereLst = new ArrayList<>();
        this.havingLst = new ArrayList<>();
        this.subLst = new ArrayList<>();
    }

    public String parse(String permissions) {
        try {
            parseConditions();

            StringBuilder sb = new StringBuilder();
            sb.append(getFrom());

            String where = conditions(whereLst);
            String having = conditions(havingLst);
            List<String> groupColumns = getGroup();
            String order = getOrder();
            if (where != null) {
                sb.append(WHERE).append(where);
                if (permissions != null) {
                    sb.append(" AND ").append(permissions);
                }
            } else {
                if (permissions != null) {
                    sb.append(WHERE).append(permissions);
                }
            }

            if (groupColumns != null && !groupColumns.isEmpty()) {
                sb.append(" GROUP BY ").append(list(groupColumns));
            }
            if (having != null) {
                sb.append(" HAVING ").append(having);
            }
            if (order != null) {
                sb.append(" ").append(order);
            }

            String result = getDiff(sb.toString(), groupColumns, order);
            if (result == null) {
                result = sb.toString();
            }

            log.info(result);
            return result;
        } catch (ConnectorException ex) {
            log.debug(ex.toString());
            throw ex;
        } catch (Exception ex) {
            log.debug(ex.toString());
            throw new ParserException(ex);
        }
    }

    private String getOrder() {
        if (querySchema.getSort() == null) {
            return null;
        }

        List<String> items = querySchema.getSort().stream().map(sortItem -> {
            String name = getFirstKey(sortItem);
            OrderType def = sortItem.get(name);
            return column(name) + " " + def.sql();
        }).collect(Collectors.toList());

        if (!items.isEmpty()) {
            return "ORDER BY " + list(items);
        }

        return null;
    }

    private List<String> getGroup() {
        if (querySchema.getDimensions() == null) {
            return Collections.emptyList();
        }
        return querySchema.getDimensions().stream().map(dimension -> {
            String dimensionName = getFirstKey(dimension);
            String dimensionColumn = getFieldColumn(dimension.get(dimensionName));
            return dimensionColumn != null ? column(dimensionColumn) : null;
        }).filter(x -> x != null).collect(Collectors.toList());
    }

    private String getFrom() throws SQLSyntaxErrorException {
        List<String> select = new ArrayList<>();
        String path = getPathToTable();
        String from = path;

        if (querySchema.getDimensions() != null) {
            querySchema.getDimensions().forEach(dimension -> {
                String name = getFirstKey(dimension);
                String columnName = getFieldColumn(dimension.get(name));
                if (columnName != null) {
                    select.add(column(columnName) + " AS " + column(name));
                }
            });
        }

        if (querySchema.getMetrics() != null) {
            for (Map<String, Map<AggregateFun, String>> metric : querySchema.getMetrics()) {
                String name = getFirstKey(metric);
                checkForSQLInjections(name);
                AggregateFun metricDef = getFirstKey(metric.get(name));
                if (metricDef != AggregateFun.EXPRESSION) {
                    String columnName = getFieldColumn(metric.get(name).get(metricDef));

                    if (metric == null || columnName == null) {
                        continue;
                    }

                    select.add(metricDef.sql(columnName) + " AS " + column(name));
                } else {
                    String exp = metric.get(name).get(AggregateFun.EXPRESSION);
                    checkForSQLInjections(exp);
                    select.add(metricDef.sql(exp) + " AS " + column(name));
                }
            }
        }
        if (subLst.size() > 0) {
            from = "(SELECT * FROM " + path + WHERE + conditions(subLst) + " )";
        }

        return "SELECT " + (select.size() > 0 ? list(select) : "*") + " FROM " + from + " AS t";
    }

    private void parseConditions() {
        if (querySchema.getFilters() == null) {
            return;
        }
        querySchema.getFilters().forEach(filterItem -> {
            String filterName = getFirstKey(filterItem);
            EFilterType filterDef = getFirstKey(filterItem.get(filterName));
            List<String> values = filterItem.get(filterName).get(filterDef);

            Map<String, Map<AggregateFun, String>> metric = findMetric(filterName);
            Map<String, String> dimension = findDimension(filterName);

            if (dimension != null) {
                String dimensionName = getFirstKey(dimension);
                String columnName = getFieldColumn(dimension.get(dimensionName));

                if (columnName != null) {
                    whereLst.add(filterDef.sql(columnName, values, getPathToTable()));
                }
            } else if (metric != null) {
                String metricName = getFirstKey(metric);
                AggregateFun metricType = getFirstKey(metric.get(metricName));
                if (metricType == AggregateFun.VALUE) {
                    String columnName = getFieldColumn(metric.get(metricName).get(metricType));
                    whereLst.add(filterDef.sql(columnName, values, getPathToTable()));
                } else {
                    String columnName = getFieldColumn(metric.get(metricName).get(metricType));
                    final String exp = metricType.sql(columnName);
                    havingLst.add(filterDef.sql(exp, values, getPathToTable(), true));
                }
            } else {
                String columnName = getFieldColumn(filterName);

                if (columnName != null) {
                    subLst.add(filterDef.sql(columnName, values, getPathToTable()));
                }
            }
        });
    }

    private String getFieldColumn(String fieldName) {
        return querySchema.getFields().get(fieldName);
    }

    private Map<String, String> findDimension(String filterName) {
        if (querySchema.getDimensions() == null) {
            return null;
        }

        return querySchema
                .getDimensions()
                .stream()
                .filter(dimension -> dimension.containsKey(filterName))
                .findAny()
                .orElse(null);
    }

    private Map<String, Map<AggregateFun, String>> findMetric(String filterName) {
        if (querySchema.getMetrics() == null) {
            return null;
        }

        return querySchema
                .getMetrics()
                .stream().filter(metric -> metric.containsKey(filterName))
                .findAny()
                .orElse(null);
    }

    private String getPathToTable() {
        final List<String> from = querySchema.getFrom();
        if (from.isEmpty()) {
            throw ConnectorException.of("Click House adapter not found table for select");
        }
        if (from.size() > 1) {
            throw ConnectorException.of("Click House adapter must select from one table but found " + from.size());
        }
        return schema + "." + from.get(0);
    }

    private String getDiff(String query, List<String> groupColumns, String order) throws SQLSyntaxErrorException {
        if (querySchema.getDiff() == null) {
            return null;
        }
        List<String> select = new ArrayList<>();
        List<String> dimensionNames = new ArrayList<>();
        List<String> selectLeftJoin = new ArrayList<>();
        List<String> where = new ArrayList<>();
        AtomicReference<String> filterName = new AtomicReference<>("");
        StringBuilder sb = new StringBuilder();
        filterName.set(querySchema.getDiff().getDate());
        if (querySchema.getDimensions() != null) {
            querySchema.getDimensions().forEach(dimension -> {
                String name = getFirstKey(dimension);
                String dimensionColumnName = column(name);
                if (dimensionColumnName != null) {
                    select.add("t." + dimensionColumnName + " AS " + dimensionColumnName);
                }
                String columnName = getFieldColumn(dimension.get(name));
                if (columnName != null) {
                    if (querySchema.getDiff().getType() == 2) {
                        if (!dimension.get(name).equals(querySchema.getDiff().getDate())) {
                            dimensionNames.add(dimensionColumnName);
                        }
                        int pos = groupColumns.indexOf(column(getFieldColumn(filterName.get())));
                        if (pos >= 0) {
                            groupColumns.remove(pos);
                        }
                        String value = dimension.values().stream().findFirst().orElse(null);
                        if (Objects.equals(value, querySchema.getDiff().getDate())) {
                            filterName.set(columnName);
                        } else {
                            selectLeftJoin.add(column(columnName) + " AS " + column(name));
                        }
                    } else if (querySchema.getDiff().getType() == 1) {
                        dimensionNames.add(dimensionColumnName);
                        selectLeftJoin.add(column(columnName) + " AS " + column(name));
                    } else if (querySchema.getDiff().getType() == 3) {
                        dimensionNames.add(dimensionColumnName);
                        String value = dimension.values().stream().findFirst().orElse(null);
                        if (Objects.equals(value, querySchema.getDiff().getDate())) {
                            filterName.set(columnName);
                        }
                    }
                }
            });
        }
        if (querySchema.getMetrics() != null) {
            for (Map<String, Map<AggregateFun, String>> metric : querySchema.getMetrics()) {
                String name = getFirstKey(metric);
                select.add("COALESCE(" + column(name) + ",0) AS " + column(name));
                select.add("COALESCE(" + column(name + "-prev") + ",0) AS " + column(name + "-prev"));

                AggregateFun metricDef = getFirstKey(metric.get(name));
                if (metricDef != AggregateFun.EXPRESSION) {
                    String columnName = getFieldColumn(metric.get(name).get(metricDef));

                    if (metric == null || columnName == null) {
                        continue;
                    }

                    selectLeftJoin.add(metricDef.sql(columnName) + " AS " + column(name + "-prev"));
                } else {
                    String exp = metric.get(name).get(AggregateFun.EXPRESSION);
                    checkForSQLInjections(name);
                    selectLeftJoin.add(metricDef.sql(exp) + " AS " + column(name + "-prev"));
                }
            }
        }
        Map<String, String> expressions = querySchema.getDiff().getExpressions();
        if (expressions != null) {
            expressions.forEach((key, value) -> select.add(value + " AS " + column(key)));
        }
        Map<EFilterType, List<String>> dateFilter = querySchema.getDiff().getDateFilter();
        if (dateFilter != null) {
            EFilterType filterDef = getFirstKey(dateFilter);
            List<String> values = dateFilter.get(filterDef);

            where.add(filterDef.sql(filterName.toString(), values, getPathToTable()));
        }

        sb.append("SELECT ").append(select.isEmpty() ? "*" : list(select)).append(" FROM (").append(query).append(") AS t")
                .append(" LEFT JOIN (SELECT ").append(selectLeftJoin.isEmpty() ? "*" : list(selectLeftJoin))
                .append(" FROM ").append(getPathToTable()).append(WHERE).append(conditions(where));
        if (querySchema.getDiff().getType() != 3 && groupColumns != null && !groupColumns.isEmpty()) {
            sb.append(" GROUP BY ").append(list(groupColumns));
        }
        sb.append(") AS j").append(querySchema.getDiff().getType() == 3 ? " ON '1'='1'" : getJoinOn(dimensionNames))
                .append(" ").append(order);

        return sb.toString();
    }

    private String getJoinOn(List<String> values) {
        final StringBuilder sb = new StringBuilder();
        for (int i = 0; i < values.size(); i++) {
            if (i == 0) {
                sb.append(" ON");
            }
            sb.append(" j.").append(values.get(i)).append(" = t.").append(values.get(i));
            if (i < values.size() - 1) {
                sb.append(" AND");
            }
        }
        return sb.toString();
    }

    private String column(String vl) {
        IStrOperation strOperation = new IStrOperation() {
        };
        return strOperation._column(vl);
    }

    private String conditions(List<String> vl) {
        IStrOperation strOperation = new IStrOperation() {
        };
        return strOperation._conditions(vl);
    }

    private String list(List<String> vl) {
        IStrOperation strOperation = new IStrOperation() {
        };
        return strOperation._list(vl);
    }

    private <K, V> K getFirstKey(Map<K, V> map) {
        if (map == null) {
            return null;
        }

        return map.keySet().stream().findFirst().orElse(null);
    }

    private void checkForSQLInjections(String sql) throws SQLSyntaxErrorException {
        for (String forbiddenPart : BLACK_LIST) {
            if (sql.toLowerCase().contains(forbiddenPart.toLowerCase())) {
                log.error("Someone trying to hack the system with SQL Injections. SQL string is {}", sql);
                throw new SQLSyntaxErrorException("Invalid input");
            }
        }
    }
}