package com.goodt.dremio.connector.parser.dremio;

import com.fasterxml.jackson.annotation.JsonValue;
import org.springframework.util.Assert;

import java.util.List;
import java.util.function.UnaryOperator;
import java.util.stream.Collectors;

import com.goodt.dremio.connector.enums.contracts.IFilterSqlOperation;

public enum EFilterType implements IFilterSqlOperation {

    EQ("$eq") {
        @Override
        public String sql(String col, List<String> args, String tab, boolean func) {
            Assert.isTrue(args.size() > 0, () -> String.format(ONE_ARG, name(), args.size()));
            final UnaryOperator<String> fn = func ? s -> s : this::_column;
            return args.get(0) == null ? fn.apply(col) + " is null" : fn.apply(col) + " = " + _value(args.get(0));
        }
    },
    EQ_NOT("$eqn") {
        @Override
        public String sql(String col, List<String> args, String tab, boolean func) {
            Assert.isTrue(args.size() > 0, () -> String.format(ONE_ARG, name(), args.size()));
            final UnaryOperator<String> fn = func ? s -> s : this::_column;
            return fn.apply(col) + " != " + _value(args.get(0));
        }
    },
    LESS("$lt") {
        @Override
        public String sql(String col, List<String> args, String tab, boolean func) {
            Assert.isTrue(args.size() > 0, () -> String.format(ONE_ARG, name(), args.size()));
            final UnaryOperator<String> fn = func ? s -> s : this::_column;
            return fn.apply(col) + " < " + _value(args.get(0));
        }
    },
    LESS_EQ("$ltq") {
        @Override
        public String sql(String col, List<String> args, String tab, boolean func) {
            Assert.isTrue(args.size() > 0, () -> String.format(ONE_ARG, name(), args.size()));
            final UnaryOperator<String> fn = func ? s -> s : this::_column;
            return fn.apply(col) + " <= " + _value(args.get(0));
        }
    },
    GREATER("$gt") {
        @Override
        public String sql(String col, List<String> args, String tab, boolean func) {
            Assert.isTrue(args.size() > 0, () -> String.format(ONE_ARG, name(), args.size()));
            final UnaryOperator<String> fn = func ? s -> s : this::_column;
            return fn.apply(col) + " > " + _value(args.get(0));
        }
    },
    GREATER_EQ("$gtq") {
        @Override
        public String sql(String col, List<String> args, String tab, boolean func) {
            Assert.isTrue(args.size() > 0, () -> String.format(ONE_ARG, name(), args.size()));
            final UnaryOperator<String> fn = func ? s -> s : this::_column;
            return fn.apply(col) + " >= " + _value(args.get(0));
        }
    },
    IN("$in") {
        @Override
        public String sql(String col, List<String> args, String tab, boolean func) {
            Assert.isTrue(args.size() > 0, "Условие " + name() + " принимает 1 и более аргументов, передаётся " + args.size());
            final UnaryOperator<String> fn = func ? s -> s : this::_column;
            return fn.apply(col) + " IN " + "(" + _value(args) + ")";
        }
    },
    IN_NOT("$inn") {
        @Override
        public String sql(String col, List<String> args, String tab, boolean func) {
            Assert.isTrue(args.size() > 0, "Условие " + name() + " принимает 1 и более аргументов, передаётся " + args.size());
            final UnaryOperator<String> fn = func ? s -> s : this::_column;
            return fn.apply(col) + " NOT IN " + "(" + _value(args) + ")";
        }
    },
    BETWEEN("$btw") {
        @Override
        public String sql(String col, List<String> args, String tab, boolean func) {
            Assert.isTrue(args.size() > 1, "Условие " + name() + " принимает 2 аргумента, передаётся " + args.size());
            final UnaryOperator<String> fn = func ? s -> s : this::_column;
            return fn.apply(col) + " BETWEEN " + _value(args.get(0)) + " AND " + _value(args.get(1));
        }
    },
    BETWEEN_NOT("$btwn") {
        @Override
        public String sql(String col, List<String> args, String tab, boolean func) {
            Assert.isTrue(args.size() > 1, "Условие " + name() + " принимает 2 аргумента, передаётся " + args.size());
            final UnaryOperator<String> fn = func ? s -> s : this::_column;
            return fn.apply(col) + " NOT BETWEEN " + _value(args.get(0)) + " AND " + _value(args.get(1));
        }
    },
    LIKE("$lk") {
        @Override
        public String sql(String col, List<String> args, String tab, boolean func) {
            Assert.isTrue(args.size() > 0, "Условие " + name() + " принимает 1 и более аргументов, передаётся " + args.size());
            final UnaryOperator<String> fn = func ? s -> s : this::_column;
            List<String> arr = args.stream().map(val -> fn.apply(col) + " LIKE " + _value(val, false)).collect(Collectors.toList());
            return "(" + arr.stream().reduce((str1, str2) -> str1 + " OR " + str2).get() + ")";
        }
    },
    MAX("$max") {
        @Override
        public String sql(String col, List<String> args, String tab, boolean func) {
            Assert.isTrue(args.size() > 0, () -> String.format(ONE_ARG, name(), args.size()));
            final UnaryOperator<String> fn = func ? s -> s : this::_column;
            return fn.apply(col) + " = (SELECT MAX (" + _column(args.get(0)) + ") FROM " + tab + ")";
        }
    },
    MIN("$min") {
        @Override
        public String sql(String col, List<String> args, String tab, boolean func) {
            Assert.isTrue(args.size() > 0, () -> String.format(ONE_ARG, name(), args.size()));
            final UnaryOperator<String> fn = func ? s -> s : this::_column;
            return fn.apply(col) + " = (SELECT MIN (" + _column(args.get(0)) + ") FROM " + tab + ")";
        }
    },
    IS("$is") {
        @Override
        public String sql(String colName, List<String> args, String table, boolean func) {
            Assert.isTrue(args.size() > 0, "Условие " + name() + " принимает 1 аргумент, передаётся " + args.size());
            return _column(colName) + " is " + args.get(0);
        }
    },
    IS_NOT("$isn") {
        @Override
        public String sql(String colName, List<String> args, String table, boolean func) {
            Assert.isTrue(args.size() > 0, "Условие " + name() + " принимает 1 аргумент, передаётся " + args.size());
            return _column(colName) + " is not " + args.get(0);
        }
    };

    private static final String ONE_ARG = "Условие %s принимает 1 аргумент, передаётся %d";

    private final String definition;

    EFilterType(String type) {
        this.definition = type;
    }

    @JsonValue
    public String getDef() {
        return definition;
    }
}
