package com.goodt.dremio.connector.parser.pg;

import com.goodt.dremio.connector.enums.EClientDataType;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.time.StopWatch;
import org.hibernate.boot.model.naming.IllegalIdentifierException;
import org.springframework.jdbc.support.rowset.SqlRowSet;

import java.sql.ResultSet;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;
import java.util.concurrent.atomic.AtomicReference;

import com.goodt.dremio.connector.config.RsModeConfig;
import com.goodt.dremio.connector.dto.ConnectionCtx;
import com.goodt.dremio.connector.dto.err.ConnectorException;
import com.goodt.dremio.connector.entity.query.QueryRequest;
import com.goodt.dremio.connector.entity.query.QueryResult;
import com.goodt.dremio.connector.parser.AbstractDataLoader;
import com.goodt.dremio.connector.parser.PermissionsAppender;
import com.goodt.dremio.connector.parser.QueryCreator;
import com.goodt.dremio.connector.utils.DataUtils;

@Slf4j
public class PgDataLoader extends AbstractDataLoader {

    private final ConnectionCtx ctx;

    public PgDataLoader(UUID logId, QueryRequest query, PermissionsAppender permissions, ConnectionCtx ctx, RsModeConfig rsModeConfig) {
        super(logId, query, permissions, rsModeConfig);
        this.ctx = ctx;
    }

    @Override
    protected QueryResult loadDataSql(String perm) {
        QueryCreator parser = getQueryCreator();
        String sql;
        try {
            sql = parser.parse(perm);
        } catch (ConnectorException ex) {
            log.debug(ex.toString());
            throw ex;
        } catch (Exception e) {
            log.error("dremio parser error", e);
            throw new RuntimeException(e);
        }
        QueryResult rez;
        if (query.getQuerySchema().getFields() == null && !query.isDebug()) {
            throw new IllegalArgumentException("Schema fields must be filled.");
        }
        String initialSql = sql;
        Long schemaLimit = getSchemaLimit(query.getRsmode());
        if (schemaLimit < query.getLimit()) {
            throw new IllegalArgumentException("Query limit cannot exceed schema limit");
        }
        if (query.getLimit() > 0) {
            sql += " LIMIT " + query.getLimit() + " OFFSET " + query.getOffset();
            rez = execute(sql);
            Long rowCount = rez.getRowCount();
            if (query.getLimit() == rowCount || query.getOffset() > 0) {
                rez.setRowCount(getRowCount(initialSql));
            }
        } else {
            sql += " LIMIT " + schemaLimit + " OFFSET " + query.getOffset();
            rez = execute(sql);
            Long rowCount = rez.getRowCount();
            if (schemaLimit.equals(rowCount) || query.getOffset() > 0) {
                throw new IllegalIdentifierException("Result rows count cannot exceed schema limit");
            }
        }
        if (rez.getRowCount() == 0) {
            rez.fillEmptyData();
        }
        if (query.isDebug()) {
            rez.setSql(sql);
        }
        return rez;
    }

    @Override
    protected QueryCreator getQueryCreator() {
        return new PgQueryCreator(query.getQuerySchema());
    }

    private QueryResult execute(String sql) {
        final QueryResult rez = new QueryResult();
        AtomicReference<Long> rowCounter = new AtomicReference<>(0L);
        StopWatch sw = StopWatch.createStarted();
        ctx.template().query(sql, rs -> {
            if (rs.getRow() == 1) {
                DataUtils.getSchema(rez, rs);
            }
            Map<String, String> values = new HashMap<>();
            for (int k = 1; k <= rs.getMetaData().getColumnCount(); k++) {
                if (rez.getSchema().get(k - 1).get("type").equals(EClientDataType.BOOLEAN.name())) {
                    values.put(rs.getMetaData().getColumnName(k), rs.getBoolean(k) ? "true" : "false");
                } else {
                    values.put(rs.getMetaData().getColumnName(k), rs.getString(k));
                }
            }
            rez.addRow(values);
            rowCounter.getAndUpdate(x -> x + 1);
        });
        sw.stop();
        log.info("PgDataLoader.execute time: {}", sw.getTime());
        rez.setRowCount(rowCounter.get());
        return rez;
    }

    public long getRowCount(String sql) {
        String sqlCnt = "select count(1) rowCount from (" + sql + ") t";
        log.debug("getRowCount(): " + sqlCnt);

        SqlRowSet sqlRowSet = ctx.template().queryForRowSet(sqlCnt);
        if (sqlRowSet.next()) {
            return sqlRowSet.getLong("rowCount");
        }

        return 0L;
    }
}
