package com.goodt.dremio.connector.repository.impl;

import com.goodt.dremio.connector.dto.ConnectionCtx;
import com.goodt.dremio.connector.entity.permissions.EmployeePermissions;
import com.goodt.dremio.connector.entity.permissions.TablePermissions;
import com.goodt.dremio.connector.entity.query.QueryResult;
import com.goodt.dremio.connector.repository.IDremioQuery;
import com.goodt.dremio.connector.service.DremioCache;
import com.goodt.dremio.connector.utils.DataUtils;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.jdbc.support.rowset.SqlRowSet;
import org.springframework.jdbc.support.rowset.SqlRowSetMetaData;

import java.util.*;
import java.util.concurrent.atomic.AtomicReference;

@Slf4j
@RequiredArgsConstructor
public class DremioQueryImpl implements IDremioQuery {

    private final ConnectionCtx ctx;

    private final DremioCache cache;

    private final String permissionsCatalog;

    @Override
    public QueryResult execute(String sql, UUID logId) {
        log.info("{}: Execute query({})", logId, sql);
        QueryResult rez = new QueryResult();

        AtomicReference<Long> rowCounter = new AtomicReference<>(0L);

        ctx.template().query(sql, rs -> {
            if (rs.getRow() == 1) {
                DataUtils.getSchema(rez, rs);
            }
            Map<String, String> values = new HashMap<>();
            for (int k = 1; k <= rs.getMetaData().getColumnCount(); k++) {
                values.put(rs.getMetaData().getColumnName(k), rs.getString(k));
            }
            rez.addRow(values);
            rowCounter.getAndUpdate(x -> x + 1);
        });
        log.info("{}: query returned {} rows", logId, rowCounter.get());

        rez.setRowCount(rowCounter.get());
        return rez;
    }

    @Override
    public Long getRowCount(String sql) {
        String sqlCnt = String.format("select count(*) rowCount from (%s)", sql) ;
        log.debug("getRowCount(): {}", sqlCnt);

        SqlRowSet sqlRowSet = ctx.template().queryForRowSet(sqlCnt);
        if (sqlRowSet.next()) {
            return sqlRowSet.getLong("rowCount");
        }

        return 0L;
    }

    @Override
    public EmployeePermissions getEmployeePermissions(String employeeId, List<String> path, UUID logId) {
        log.debug("{}: getEmployeePermissions({})", logId, employeeId);
        EmployeePermissions empPermissions = new EmployeePermissions(employeeId);

        final String cacheKey = path.isEmpty() ? employeeId : getTablePath(path) + "_" + employeeId;
        if (cache.employee().get(cacheKey) != null) {
            log.debug("{}: Get employee permissions data from cache by key \"{}\"", logId, cacheKey);
            empPermissions = cache.employee().get(cacheKey);
        } else {
            log.debug("{}: Get employee permissions data from DB", logId);
            List<TablePermissions> permissions = getTablePermissions(path, logId);
            Set<String> permissionsNames = new HashSet<>();
            for (TablePermissions tablePermissions : permissions) {
                permissionsNames.add(tablePermissions.getPermission());
            }
            String tablePermissionsString = permissionsNames.isEmpty() ? "" : String.join("' ", permissionsNames);

            final String sql = String.format("select distinct %s from \"%s\".\"permission\" where \"$%s\" = '%s'",
                                             tablePermissionsString, permissionsCatalog, ctx.authKey(), employeeId);
            log.debug("{}: \t---> {}", logId, sql);
            SqlRowSet sqlRowSet = ctx.template().queryForRowSet(sql);
            SqlRowSetMetaData metaData = sqlRowSet.getMetaData();
            String[] columnNames = metaData.getColumnNames();
            int rowCounter = 0;
            while (sqlRowSet.next()) {
                for (String colName : columnNames) {
                    if (colName.equals("$" + ctx.authKey())) {
                        continue;
                    }
                    if (!empPermissions.getPermissions().containsKey(colName)) {
                        List<String> values = new ArrayList<>();
                        empPermissions.getPermissions().put(colName, values);
                    }
                    empPermissions.getPermissions().get(colName).add(sqlRowSet.getString(colName));
                }
                rowCounter++;
            }
            empPermissions.setPermissionsCount(rowCounter);
            log.debug("{}: getEmployeePermissions() complete. Select {} rows.", logId, rowCounter);

            cache.employee().put(cacheKey, empPermissions);
        }
        log.debug("{}: we got {} employee permissions", logId, empPermissions.getPermissions().size());

        return empPermissions;
    }

    @Override
    public List<TablePermissions> getTablePermissions(List<String> path, UUID logId) {
        log.debug("{}: getTablePermissions({})", logId, path);
        List<TablePermissions> permissionsList = new ArrayList<>();
        String table = "null";
        if (!path.isEmpty()) {
            table = getTablePath(path);
        }
        if (cache.tablePermission().get(table) != null) {
            log.debug("{}: Get table permissions data from cache by key \"{}\"", logId, table);
            permissionsList = cache.tablePermission().get(table);
        } else {
            log.debug("{}Get table permissions data from DB", logId);
            String sql = path.isEmpty() ? "select \"permission\", \"column\", \"table\" from " + permissionsCatalog + ".\"table-permission\"" :
                    "select \"permission\", \"column\", \"table\" from " + permissionsCatalog + ".\"table-permission\" where \"table\" = '" + table + "'";
            log.debug("{}: \t---> {}", logId, sql);
            SqlRowSet sqlRowSet = ctx.template().queryForRowSet(sql);
            while (sqlRowSet.next()) {
                TablePermissions tablePermissions = new TablePermissions();
                tablePermissions.setPermission(sqlRowSet.getString("permission"));
                tablePermissions.setColumn(sqlRowSet.getString("column"));
                permissionsList.add(tablePermissions);
            }
            log.debug("{}: DB returned {} table permissions", logId, permissionsList.size());
            cache.tablePermission().put(table, permissionsList);
        }

        log.debug("{}: we got {} table permissions", logId, permissionsList.size());
        return permissionsList;
    }

    private String getTablePath(List<String> path) {
        return path.stream().reduce((x1, x2) -> x1 + "." + x2).get();
    }

}
