package com.goodt.dremio.connector.service;

import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.util.CollectionUtils;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.UUID;
import java.util.stream.Collectors;

import com.goodt.dremio.connector.dto.ConnectionCtx;
import com.goodt.dremio.connector.dto.ObjectDto;
import com.goodt.dremio.connector.dto.ObjectFindDto;
import com.goodt.dremio.connector.dto.UserInfo;
import com.goodt.dremio.connector.dto.err.ConnectorException;
import com.goodt.dremio.connector.entity.permissions.AttributeValueInputDto;
import com.goodt.dremio.connector.entity.permissions.AttributeValueResponse;
import com.goodt.dremio.connector.entity.query.QuerySchema;
import com.goodt.dremio.connector.parser.PermissionsAppender;
import com.goodt.dremio.connector.service.clients.PermissionClient;

@Slf4j
public abstract class AbstractPermissionsAppender implements PermissionsAppender {

    @Value("${app-cfg.sur.isEnabled}")
    private boolean isSurEnabled;
    @Autowired
    private PermissionClient permissionClient;

    public String get(ConnectionCtx ctx, QuerySchema schema, UserInfo userInfo, UUID logId) {
        if (!isSurEnabled) {
            log.info("{}: SUR is not enabled! Table permissions are not applied.", logId);
            return null;
        }
        if (userInfo.isAdmin()) {
            log.info("{}: Admin user connected! Table permissions are not applied.", logId);
            return null;
        }
        return getRestrictionQueryPart(userInfo.getEmployeeId(), schema.getFrom(), logId);
    }

    protected String getRestrictionQueryPart(String employeeId, List<String> path, UUID logId) {
        log.info("{}: getRestrictionQueryPart({})", logId, employeeId);

        final String tableName = path.stream().reduce((s1, s2) -> s1 + "." + s2).orElse("");

        List<ObjectDto> objects = permissionClient.getObjects(ObjectFindDto.builder().entityValues(List.of(tableName)).build());
        if (objects.isEmpty()) {
            throw ConnectorException.of("От СУР вернулся пустой список");
        }

        List<String> objectCodes = objects.stream()
                .map(ObjectDto::getCode)
                .collect(Collectors.toList());

        List<String> attributeCodes = objects.stream()
                .flatMap(objectDto -> objectDto.getRestrictions().stream())
                .map(i -> i.getAttribute().getCode())
                .collect(Collectors.toList());

        List<AttributeValueResponse> userAllowedColumns = permissionClient.getDatasetPermissions(
                AttributeValueInputDto.builder()
                        .objectCodes(objectCodes)
                        .attributes(attributeCodes)
                        .build());

        log.info("{}: AttributeValue collection size:{}. objectCodes:{}, attributes:{}", logId, userAllowedColumns.size(), objectCodes.toString(), attributeCodes.toString());

        final StringBuilder rez = new StringBuilder();
        rez.append("(");
        StringBuilder valuesBlock = createValuesBlock(attributeCodes, userAllowedColumns);
        rez.append(valuesBlock);
        rez.append(")");

        log.info("rez: {}", rez);
        return rez.toString();
    }

    protected abstract String quoteColumn(String columnName);

    private StringBuilder createValuesBlock(List<String> columns, List<AttributeValueResponse> attributeValueResponses) {
        final StringBuilder sb = new StringBuilder();
        if (CollectionUtils.isEmpty(attributeValueResponses)) {
            sb.append("1 = 1");
            return sb;
        }
        final Map<String, List<AttributeValueResponse>> permMap = attributeValueResponses.stream().collect(Collectors.groupingBy(AttributeValueResponse::getAttributeCode));
        boolean hasAny = false;
        for (String column : columns) {
            if (hasAny) {
                sb.append(" or ");
            }

            final List<AttributeValueResponse> nvl = permMap.getOrDefault(column, Collections.emptyList());
            if (nvl.stream().anyMatch(n -> n.getValues().contains("_ALL_"))) {
                sb.append("1 = 1");
                break;
            }

            if (!permMap.containsKey(column)) {
                sb.append("(").append(quoteColumn(column)).append(" is null)");
                hasAny = true;
                continue;
            }
            boolean started = false;
            for (AttributeValueResponse nv : nvl) {
                for (String value : nv.getValues()) {
                    sb.append(started ? " or " : "(");
                    started = true;
                    sb.append(quoteColumn(column)).append(" = '").append(value).append("'");
                }
            }
            sb.append(" or ").append(quoteColumn(column)).append(" is null)");
            hasAny = true;

        }
        return sb;
    }
}
