package com.goodt.dremio.connector.service;

import com.zaxxer.hikari.HikariDataSource;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.jdbc.DataSourceBuilder;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import javax.sql.DataSource;
import java.net.URI;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import com.goodt.dremio.connector.dao.ConnectionDao;
import com.goodt.dremio.connector.dto.ConnectionCtx;
import com.goodt.dremio.connector.dto.err.ConnectorException;
import com.goodt.dremio.connector.entities.AdapterCode;
import com.goodt.dremio.connector.entities.Connection;

@Component
public class ClientPull {

    private final Map<String, ConnectionCtx> pull = new ConcurrentHashMap<>();

    @Autowired
    private ConnectionDao connectionDao;

    private ConnectionCtx defaultCtx;

    @PostConstruct
    void init() {
        connectionDao.findAll().forEach(this::add);
    }

    public ConnectionCtx get(String code) {
        if (code == null) {
            if (defaultCtx != null) {
                return defaultCtx;
            } else {
                throw ConnectorException.of("Connection code is null and no default connection");
            }
        }
        if (!pull.containsKey(code)) {
            throw ConnectorException.of("Connection code not exist " + code);
        }
        return pull.get(code);
    }

    public void add(Connection cn) {
        final ConnectionCtx ctx = createJdbc(cn);
        if (cn.isDefault()) {
            defaultCtx = ctx;
        }
        pull.put(cn.getCode(), ctx);
    }

    public void remove(Connection cn) {
        pull.remove(cn.getCode());
    }

    private ConnectionCtx createJdbc(Connection cn) {
        final AdapterCode code = cn.getAdapter().getCode();
        final DataSource ds = DataSourceBuilder.create()
                .driverClassName(code.getDriver())
                .url(createConnectionStr(cn))
                .username(cn.getUsername()).password(cn.getPassword())
                .build();
        if (ds instanceof HikariDataSource) {
            ((HikariDataSource) ds).setMaximumPoolSize(20);
        }
        final ConnectionCtx ctx = new ConnectionCtx()
                .authKey(cn.getUserUid())
                .template(new JdbcTemplate(ds))
                .login(cn.getUsername())
                .provider(code)
                .password(cn.getPassword())
                .code(cn.getCode());
        if (cn.getConnectionApi() != null) {
            ctx.api(URI.create(cn.getConnectionApi()));
        }
        if (code == AdapterCode.PG) {
            ctx.authKey(cn.getDatabase());
            ctx.schema(cn.getSchema() != null ? cn.getSchema() : "public");
        } else if (code == AdapterCode.CLICK_HOUSE) {
            ctx.schema(cn.getSchema() != null ? cn.getSchema() : "default");
        }
        return ctx;
    }

    private String createConnectionStr(Connection cn) {
        final AdapterCode code = cn.getAdapter().getCode();
        final String[] arr = cn.getServer().split(":");
        if (code == AdapterCode.PG) {
            final String schema = cn.getSchema() != null ? cn.getSchema() : "public";
            return String.format(code.getConStrFmt(), arr[0], arr[1], cn.getDatabase(), schema);
        }
        return String.format(code.getConStrFmt(), arr[0], arr[1]);
    }
}
