package com.goodt.dremio.connector.service;

import cdjd.org.apache.commons.lang3.StringUtils;
import lombok.Getter;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Objects;
import java.util.Optional;
import java.util.UUID;

import me.goodt.micro.core.entity.AbstractEntity;
import me.goodt.micro.core.error.RTException;
import me.goodt.micro.core.service.AbstractCrudService;
import me.goodt.micro.core.service.Converter;
import me.goodt.micro.core.service.Updater;
import com.goodt.dremio.connector.dao.AdapterDao;
import com.goodt.dremio.connector.dao.ConnectionDao;
import com.goodt.dremio.connector.dto.ConnectionRes;
import com.goodt.dremio.connector.dto.DrillAdapter;
import com.goodt.dremio.connector.dto.DrillStorage;
import com.goodt.dremio.connector.dto.err.ConnectorException;
import com.goodt.dremio.connector.entities.Adapter;
import com.goodt.dremio.connector.entities.AdapterCode;
import com.goodt.dremio.connector.entities.Connection;

@Slf4j
@Service
@Transactional
public class ConnectionService extends AbstractCrudService<Connection, Long> {

    @Getter
    @Autowired
    private ConnectionDao dao;

    @Autowired
    private AdapterDao adapterDao;

    @Autowired
    private ClientPull clientPull;

    @Autowired
    DrillAdapterService drillAdapterService;

    @Transactional(readOnly = true)
    public ConnectionRes getById(Long id, String ownerId, Converter<Connection, ConnectionRes> converter) {
        Connection entity = this.getDao().findById(id, ownerId)
                .orElseThrow(() -> RTException.of(String.format("Entity %s not found by id %s", this.getEntityClazz(), id)));
        ConnectionRes res = converter.convert(entity);
        if (entity.getAdapter().getCode().equals(AdapterCode.DRILL)) {
            updateConfig(id, res);
        }
        return res;
    }

    @Transactional(readOnly = true)
    public List<Connection> getAll() {
        return getDao().findAll();
    }

    @Transactional(readOnly = true)
    public List<Connection> getAll(String ownerId) {
        return getDao().findAll(ownerId);
    }

    @Override
    public <R> R create(Updater<Connection> creator, Converter<Connection, R> converter) {
        throw new UnsupportedOperationException();
    }

    @Override
    public Connection update(Long id, Updater<Connection> updater) {
        throw new UnsupportedOperationException();
    }

    @Transactional
    public ConnectionRes create(Updater<Connection> creator, Converter<Connection, ConnectionRes> converter, ConnectionRes res) {
        ConnectionRes connectionRes = super.create(creator, converter);
        Connection entity = this.getDao().findById(connectionRes.getId())
                .orElseThrow(() -> RTException.of(String.format("Entity %s not found by id %s", this.getEntityClazz(), connectionRes.getId())));
        if (entity.getAdapter().getCode().equals(AdapterCode.DRILL)) {
            drillAdapterService.createOrUpdate(new DrillAdapter(entity.getId(), res.getConfig()));
            updateConfig(entity.getId(), connectionRes);
        }
        return connectionRes;
    }

    @Transactional
    public ConnectionRes update(Long id, Updater<Connection> updater, Converter<Connection, ConnectionRes> converter, ConnectionRes res) {
        Connection entity = super.update(id, updater);
        updateAdapter(entity);
        ConnectionRes connectionRes = converter.convert(entity);
        if (entity.getAdapter().getCode().equals(AdapterCode.DRILL)) {
            drillAdapterService.createOrUpdate(new DrillAdapter(entity.getId(), res.getConfig()));
            updateConfig(entity.getId(), connectionRes);
        }
        clientPull.add(entity);
        return connectionRes;
    }

    public ConnectionRes clone(Long id, Converter<Connection, ConnectionRes> converter) {
        Connection originalEntity = this.getDao().findById(id)
                .orElseThrow(() -> RTException.of(String.format("Entity %s not found by id %s", this.getEntityClazz(), id)));

        Connection newEntity = this.inst();
        newEntity.setCode(UUID.randomUUID().toString());
        newEntity.setName(originalEntity.getName());
        newEntity.setDescription(originalEntity.getDescription());
        newEntity.setServer(originalEntity.getServer());
        newEntity.setDatabase(originalEntity.getDatabase());
        newEntity.setUsername(originalEntity.getUsername());
        newEntity.setPassword(originalEntity.getPassword());
        newEntity.setUserUid(originalEntity.getUserUid());
        newEntity.setConnectionApi(originalEntity.getConnectionApi());
        newEntity.setSchema(originalEntity.getSchema());
        newEntity.setAdapter(originalEntity.getAdapter());
        newEntity.setOwnerId(originalEntity.getOwnerId());

        Connection connection = this.getDao().save(newEntity);
        afterSaveCreated(connection);

        ConnectionRes connectionRes = converter.convert(connection);
        if (connection.getAdapter().getCode().equals(AdapterCode.DRILL)) {
            DrillStorage drillStorage = drillAdapterService.getByConnectionId(originalEntity.getId());
            if (drillStorage.getConfig() != null) {
                connectionRes.setConfig(drillStorage.getConfig());
                drillAdapterService.createOrUpdate(new DrillAdapter(connection.getId(), connectionRes.getConfig()));
            }
        }
        return connectionRes;
    }

    @Override
    protected void beforeUpdate(Connection entity) {
        validate(entity);
        final Connection oldConn = getDao().findById(entity.getId()).orElseThrow();
        if (!Objects.equals(oldConn.getCode(), entity.getCode())) {
            throw ConnectorException.of("Connection code has changed");
        }
    }

    @Override
    protected void beforeDelete(Long id) {
        Connection entity = this.getDao().findById(id)
                .orElseThrow(() -> RTException.of(String.format("Entity %s not found by id %s", this.getEntityClazz(), id)));
        if (entity.getAdapter().getCode().equals(AdapterCode.DRILL)) {
            drillAdapterService.delete(id);
        }
        clientPull.remove(entity);
    }

    private void updateAdapter(Connection entity) {
        Adapter adapter = adapterDao.findById(entity.getAdapterId())
                .orElseThrow(() -> ConnectorException.of(String.format("Adapter with id \"%d\" not found.", entity.getAdapterId())));
        entity.setAdapter(adapter);
    }

    protected void afterSaveCreated(Connection entity) {
        validate(entity);
        updateAdapter(entity);
        clientPull.add(entity);
    }

    private void updateConfig(Long connectionId, ConnectionRes connectionRes) {
        DrillStorage drillStorage = drillAdapterService.getByConnectionId(connectionId);
        if (drillStorage.getConfig() != null) {
            connectionRes.setConfig(drillStorage.getConfig());
        }
    }

    private static void validate(Connection connection) {
        if (connection.getAdapter() == null) {
            throw ConnectorException.of("Adapter is required");
        }
        if (StringUtils.isBlank(connection.getCode())) {
            throw ConnectorException.of("Code is required");
        }
        connection.setCode(connection.getCode().trim());
        final String cs = connection.getServer();
        if (cs == null || cs.isBlank() || !cs.contains(":")) {
            throw ConnectorException.of("Server must be not empty and must be in the format: server:port");
        }
        final String[] arr = cs.split(":");
        if (!arr[1].matches("\\d+")) {
            throw ConnectorException.of("Server port must contains only digits");
        }
    }
}
