package com.goodt.dremio.connector.service;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ObjectNode;
import lombok.extern.slf4j.Slf4j;
import org.jetbrains.annotations.NotNull;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Base64Utils;

import java.net.URI;

import com.goodt.dremio.connector.dao.ConnectionDao;
import com.goodt.dremio.connector.dto.DrillAdapter;
import com.goodt.dremio.connector.dto.DrillStorage;
import com.goodt.dremio.connector.dto.DrillStorageConfig;
import com.goodt.dremio.connector.dto.DrillStorageResultResp;
import com.goodt.dremio.connector.dto.err.ConnectorException;
import com.goodt.dremio.connector.entities.Connection;
import com.goodt.dremio.connector.exceptions.DrillApiException;
import com.goodt.dremio.connector.security.GlobalDefs;
import com.goodt.dremio.connector.service.clients.DrillClient;

@Slf4j
@Service
@Transactional
public class DrillAdapterService {

    @Autowired
    DrillClient drillClient;
    @Autowired
    ConnectionDao connectionDao;

    @Transactional
    public DrillStorage getByConnectionId(Long connectionId) {
        try {
            Connection entity = getConnection(connectionId);
            DrillStorageConfig storage = drillClient.getStorage(URI.create(entity.getConnectionApi()), entity.getCode(), buildAuthHeader(entity));
            return new DrillStorage(storage.getName(), new ObjectMapper().writeValueAsString(storage.getConfig()));
        } catch (Exception ex) {
            throw new DrillApiException(ex);
        }
    }

    @Transactional
    public DrillStorageResultResp createOrUpdate(DrillAdapter drillAdapter) {
        try {
            Connection entity = getConnection(drillAdapter.getConnectionId());
            DrillStorageConfig drillStorage = new DrillStorageConfig();
            drillStorage.setName(entity.getCode());
            if (drillAdapter.getConfig() != null && !drillAdapter.getConfig().isBlank()) {
                drillStorage.setConfig(getConfigJson(drillAdapter));
            }
            DrillStorageResultResp storage = drillClient.createStorage(URI.create(entity.getConnectionApi()), entity.getCode(), drillStorage, buildAuthHeader(entity));
            checkResult(storage, "Cannot create drill adapter: \"%s\"");
            return storage;
        } catch (Exception ex) {
            throw new DrillApiException(ex);
        }
    }

    @Transactional
    public void delete(Long connectionId) {
        try {
            Connection entity = getConnection(connectionId);
            DrillStorageResultResp storage = drillClient.deleteStorage(URI.create(entity.getConnectionApi()), entity.getCode(), buildAuthHeader(entity));
            checkResult(storage, "Cannot delete drill adapter: \"%s\"");
        } catch (Exception ex) {
            throw new DrillApiException(ex);
        }
    }

    private static Object getConfigJson(DrillAdapter drillAdapter) throws JsonProcessingException {
        ObjectNode config = (ObjectNode) new ObjectMapper().readTree(drillAdapter.getConfig());
        return config;
    }

    private static void checkResult(DrillStorageResultResp storage, String message) {
        if (!storage.getResult().equals(GlobalDefs.DRILL_RESULT_SUCCESS)) {
            throw ConnectorException.of(String.format(message, storage.getResult()));
        }
    }

    private Connection getConnection(Long connectionId) {
        return connectionDao.findById(connectionId)
                .orElseThrow(() -> ConnectorException.of(String.format("Connection with id \"%d\" not found.", connectionId)));
    }

    @NotNull
    private static String buildAuthHeader(Connection entity) {
        if (!entity.getUsername().isBlank() && !entity.getPassword().isBlank()) {
            byte[] encodedBytes = Base64Utils.encode((entity.getUsername() + ":" + entity.getPassword()).getBytes());
            return "Basic " + new String(encodedBytes);
        }
        return "";
    }
}
