package com.goodt.dremio.connector.system;

import lombok.extern.slf4j.Slf4j;

import java.util.Date;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

@Slf4j
public class ExpiredConcurrentHashMap<K, V> extends ConcurrentHashMap<K, V> {

    private static final long serialVersionUID = 1L;

    private Map<K, Long> timeMap = new ConcurrentHashMap<>();

    private long expiryInMillis = 5000;

    private long cleanerInMillis = 1000;

    private String cacheName;

    public ExpiredConcurrentHashMap(String cacheName) {
        this.cacheName = cacheName;
        initialize();
    }

    public ExpiredConcurrentHashMap(String cacheName, long expiryInMillis, long cleanerInMillis) {
        log.info("Initialize cache for \"{}\" with expire time \"{}\" sec and cleaner time \"{}\" sec", cacheName, expiryInMillis / 1000, cleanerInMillis / 1000);
        this.cacheName = cacheName;
        this.expiryInMillis = expiryInMillis;
        this.cleanerInMillis = cleanerInMillis;
        initialize();
    }

    private void initialize() {
        new CleanerThread().start();
    }

    @Override
    public V put(K key, V value) {
        Date date = new Date();
        timeMap.put(key, date.getTime());
        log.debug("Inserting element to \"{}\": {} : {} : {}", ExpiredConcurrentHashMap.this.cacheName, date, key, value);
        return super.put(key, value);
    }

    @Override
    public void putAll(Map<? extends K, ? extends V> m) {
        for (Entry<? extends K, ? extends V> en : m.entrySet()) {
            put(en.getKey(), en.getValue());
        }
    }

    @Override
    public V putIfAbsent(K key, V value) {
        if (!containsKey(key)) {
            return put(key, value);
        } else {
            return get(key);
        }
    }

    private class CleanerThread extends Thread {
        @Override
        public void run() {
            log.info("Initiating Cleaner Thread for \"{}\"", ExpiredConcurrentHashMap.this.cacheName);
            while (true) {
                cleanMap();
                try {
                    Thread.sleep(cleanerInMillis);
                } catch (InterruptedException ex) {
                    log.error("Initiating Cleaner Thread error", ex);
                }
            }
        }

        private void cleanMap() {
            long currentTime = new Date().getTime();
            for (K key : timeMap.keySet()) {
                if (currentTime > (timeMap.get(key) + expiryInMillis)) {
                    V value = remove(key);
                    timeMap.remove(key);
                    log.debug("Removing expired element from \"{}\": {} : {} : {}", ExpiredConcurrentHashMap.this.cacheName, new Date(), key, value);
                }
            }
        }
    }
}