package com.goodt.dremio.connector.controller;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.goodt.dremio.connector.TestSecurityConfig;
import com.goodt.dremio.connector.entity.api.CatalogResult;
import com.goodt.dremio.connector.entity.api.RootEntity;
import com.goodt.dremio.connector.entity.permissions.PermissionResponse;
import com.goodt.dremio.connector.entity.query.QueryRequest;
import com.goodt.dremio.connector.entity.query.QueryResult;
import com.goodt.dremio.connector.entity.query.QuerySchema;
import com.goodt.dremio.connector.exceptions.PermissionInfoException;
import com.goodt.dremio.connector.parser.DataLoadFactory;
import com.goodt.dremio.connector.repository.IDremioQuery;
import com.goodt.dremio.connector.service.*;
import com.goodt.dremio.connector.service.impl.DremioTokenizer;
import org.apache.commons.io.IOUtils;
import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.EnableAutoConfiguration;
import org.springframework.boot.autoconfigure.jdbc.DataSourceAutoConfiguration;
import org.springframework.boot.autoconfigure.jdbc.DataSourceTransactionManagerAutoConfiguration;
import org.springframework.boot.autoconfigure.orm.jpa.HibernateJpaAutoConfiguration;
import org.springframework.boot.test.autoconfigure.web.servlet.WebMvcTest;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.http.MediaType;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.test.web.servlet.MockMvc;

import java.io.IOException;
import java.io.InputStream;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import static org.hamcrest.Matchers.containsString;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.ArgumentMatchers.anyString;
import static org.mockito.Mockito.when;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.post;
import static org.springframework.test.web.servlet.result.MockMvcResultHandlers.print;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.content;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

@Disabled
@SpringBootTest(classes = TestSecurityConfig.class)
@WebMvcTest(DataController.class)
@EnableAutoConfiguration(exclude = {DataSourceAutoConfiguration.class, DataSourceTransactionManagerAutoConfiguration.class, HibernateJpaAutoConfiguration.class})
public class DataControllerTest {

    @Autowired
    private MockMvc mockMvc;
    @MockBean
    JdbcTemplate jdbcTemplate;
    @MockBean
    IDremioQuery dremioQuery;
    @MockBean
    SchemaProviderDremio schemaProvider;
    @MockBean
    DremioTokenizer dremioTokenizer;
    @MockBean
    ClientPull clientPull;
    @MockBean
    SchemaProviderFactory schemaProviderFactory;
    @MockBean
    DataLoadFactory dataLoadFactory;
    @MockBean
    IDremioPermissions permissions;
    @MockBean
    DremioCache cache;

    ObjectMapper objectMapper = new ObjectMapper();

    @Test
    public void getPermissionsInfoSuccessful() throws Exception {
        when(schemaProvider.getPermissionInfo()).thenReturn(Arrays.asList("$divisionId", "$systemRoleId", "$kp"));

        this.mockMvc
                .perform(get("/api/v1/permissions-info"))
                .andDo(print())
                .andExpect(status().isOk())
                .andExpect(content().string(containsString("[{\"name\":\"$divisionId\"},{\"name\":\"$systemRoleId\"},{\"name\":\"$kp\"}]")));
    }

    @Test
    void getPermissionsInfoException() throws Exception {
        when(schemaProvider.getPermissionInfo()).thenThrow(new PermissionInfoException("Test exception interception"));

        mockMvc.perform(get("/api/v1/permissions-info"))
                .andDo(print())
                .andExpect(status().isServiceUnavailable())
                .andExpect(status().is5xxServerError())
                .andExpect(content().string(containsString("Test exception interception")));
    }

    @Test
    void queryTestOkWithCount() throws Exception {
        when(dremioQuery.getRowCount(anyString())).thenReturn(4L);
        when(dremioQuery.execute(anyString(), any())).thenReturn(getQueryResult());

        QueryRequest queryRequest = new QueryRequest();
        queryRequest.setOffset(0);
        queryRequest.setLimit(10);
        queryRequest.setDebug(false);
        queryRequest.setQuerySchema(getQuerySchema());

        mockMvc.perform(post("/api/v1/data")
                                .content(asJsonString(queryRequest))
                                .contentType(MediaType.APPLICATION_JSON)
                                .accept(MediaType.APPLICATION_JSON))
                .andDo(print())
                .andExpect(status().isOk())
                .andExpect(content().string(containsString("\"rowCount\":4")));
    }

    @Test
    void queryTestOkWithOutCount() throws Exception {
        when(dremioQuery.getRowCount(anyString())).thenReturn(4L);
        when(dremioQuery.execute(anyString(), any())).thenReturn(getQueryResult());

        QueryRequest queryRequest = new QueryRequest();
        queryRequest.setOffset(0);
        queryRequest.setLimit(0);
        queryRequest.setDebug(false);
        queryRequest.setQuerySchema(getQuerySchema());

        mockMvc.perform(post("/api/v1/data")
                                .content(asJsonString(queryRequest))
                                .contentType(MediaType.APPLICATION_JSON)
                                .accept(MediaType.APPLICATION_JSON))
                .andDo(print())
                .andExpect(status().isOk())
                .andExpect(content().string(containsString("\"rowCount\":2")));
    }

    @Test
    void queryTestErrorWithException() throws Exception {
        when(dremioQuery.getRowCount(anyString())).thenReturn(4L);
        when(dremioQuery.execute(anyString(), any())).thenReturn(getQueryResult());

        QueryRequest queryRequest = new QueryRequest();
        queryRequest.setOffset(0);
        queryRequest.setLimit(0);
        queryRequest.setDebug(false);
        queryRequest.setQuerySchema(getQuerySchema());

        mockMvc.perform(post("/api/v1/data")
                                .content(asJsonString(queryRequest))
                                .contentType(MediaType.APPLICATION_JSON)
                                .accept(MediaType.APPLICATION_JSON))
                .andDo(print())
                .andExpect(status().isInternalServerError())
                .andExpect(status().is5xxServerError())
                .andExpect(content().string(containsString("Query execution error")));
    }

    private QuerySchema getQuerySchema() {
        InputStream inputStream = getClass().getClassLoader().getResourceAsStream("./test_service/QuerySchema.json");
        try {
            String jsonStr = IOUtils.toString(inputStream, StandardCharsets.UTF_8.name());
            return objectMapper.readValue(jsonStr, QuerySchema.class);
        } catch (IOException e) {
            e.printStackTrace();
            return null;
        }
    }

    private QueryResult getQueryResult() {
        InputStream inputStream = getClass().getClassLoader().getResourceAsStream("./test_service/QueryResult.json");
        try {
            String jsonStr = IOUtils.toString(inputStream, StandardCharsets.UTF_8.name());
            return objectMapper.readValue(jsonStr, QueryResult.class);
        } catch (IOException e) {
            e.printStackTrace();
            return null;
        }
    }

    private String asJsonString(Object _obj) {
        try {
            return objectMapper.writeValueAsString(_obj);
        } catch (JsonProcessingException e) {
            e.printStackTrace();
            return null;
        }
    }

    @Test
    void getPermissionsOk() throws Exception {
        PermissionResponse permissionResponse = new PermissionResponse();
        permissionResponse.setPermissions(new ArrayList<>());
        when(permissions.getPermissionsList(anyString(), any(), any())).thenReturn(permissionResponse);

        mockMvc.perform(get("/api/v1/permissions"))
                .andDo(print())
                .andExpect(status().isOk())
                .andExpect(content().string(containsString("{\"employeeId\":null,\"permissions\":[]}")));
    }

    @Test
    void getPermissionsError() throws Exception {
        when(permissions.getPermissionsList(anyString(), any(), any())).thenThrow(new RuntimeException("User permissions error!"));

        mockMvc.perform(get("/api/v1/permissions"))
                .andDo(print())
                .andExpect(status().isInternalServerError())
                .andExpect(content().string(containsString("User permissions error!")));
    }

    @Test
    void getMainCatalogOk() throws Exception {
        when(schemaProvider.getMainCatalog()).thenReturn(getRootEntityList());

        mockMvc.perform(get("/api/v1/catalog"))
                .andDo(print())
                .andExpect(status().isOk())
                .andExpect(content().string(containsString(objectMapper.writeValueAsString(getRootEntityList()))));
    }

    @Test
    void getMainCatalogError() throws Exception {
        when(schemaProvider.getMainCatalog()).thenThrow(new RuntimeException("Get main catalog exception"));

        mockMvc.perform(get("/api/v1/catalog"))
                .andDo(print())
                .andExpect(status().isServiceUnavailable())
                .andExpect(content().string(containsString("Get main catalog exception")));
    }

    @Test
    void getEntityByPathOk() throws Exception {
        when(schemaProvider.getEntityByPath(any())).thenReturn(getCatalogResult());

        mockMvc.perform(post("/api/v1/catalog/by-path")
                                .content("{ \"path\": [\"_test\", \"permission\"] }")
                                .contentType(MediaType.APPLICATION_JSON)
                                .accept(MediaType.APPLICATION_JSON))
                .andDo(print())
                .andExpect(status().isOk())
                .andExpect(content().string(containsString(objectMapper.writeValueAsString(getCatalogResult()))));
    }

    @Test
    void getEntityByPathError() throws Exception {
        when(schemaProvider.getEntityByPath(any())).thenThrow(new RuntimeException("Get catalog by path exception"));

        mockMvc.perform(post("/api/v1/catalog/by-path")
                                .content("{ \"path\": [\"_test\", \"permission\"] }")
                                .contentType(MediaType.APPLICATION_JSON)
                                .accept(MediaType.APPLICATION_JSON))
                .andDo(print())
                .andExpect(status().isServiceUnavailable())
                .andExpect(content().string(containsString("Get catalog by path exception")));
    }

    private CatalogResult getCatalogResult() {
        InputStream inputStream = getClass().getClassLoader().getResourceAsStream("test_service/CatalogResult.json");
        try {
            String jsonStr = IOUtils.toString(inputStream, StandardCharsets.UTF_8.name());
            return objectMapper.readValue(jsonStr, CatalogResult.class);
        } catch (IOException e) {
            e.printStackTrace();
            return null;
        }
    }

    private List<RootEntity> getRootEntityList() {
        InputStream inputStream = getClass().getClassLoader().getResourceAsStream("test_service/RootEntityList.json");
        try {
            String jsonStr = IOUtils.toString(inputStream, StandardCharsets.UTF_8.name());
            return objectMapper.readValue(jsonStr, new TypeReference<List<RootEntity>>() {
            });
        } catch (IOException e) {
            e.printStackTrace();
            return null;
        }
    }

}
