package com.goodt.drive.fileupload.application.configuration;

import lombok.RequiredArgsConstructor;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.http.HttpHeaders;
import org.springframework.security.config.Customizer;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity;
import org.springframework.security.config.annotation.web.configuration.WebSecurityConfigurerAdapter;
import org.springframework.security.config.http.SessionCreationPolicy;
import org.springframework.security.web.authentication.UsernamePasswordAuthenticationFilter;
import org.springframework.web.cors.CorsConfiguration;
import org.springframework.web.cors.CorsConfigurationSource;
import org.springframework.web.cors.UrlBasedCorsConfigurationSource;

import javax.servlet.http.HttpServletResponse;

import com.goodt.drive.fileupload.security.JwtAuthenticationFilter;

import static com.goodt.drive.fileupload.application.utils.ResponseErrorMessageUtil.setResponseError;

@Configuration
@EnableWebSecurity
@RequiredArgsConstructor
@ConditionalOnProperty(value = "appConfig.keyCloak.isEnabled", havingValue = "false")
public class WebSecurityConfig extends WebSecurityConfigurerAdapter {

    private final JwtAuthenticationFilter jwtAuthenticationFilter;
    private final CorsConfig corsConfig;

    @Override
    protected void configure(final HttpSecurity http) throws Exception {
        http.headers().frameOptions().sameOrigin();
        http.cors(Customizer.withDefaults())
                .csrf().disable()
                .sessionManagement().sessionCreationPolicy(SessionCreationPolicy.STATELESS)
                .and()
                .authorizeRequests().antMatchers("/", "/callback", "/build-info", "/login**", "/webjars/**", "/error**").permitAll()
                .and()
                .authorizeRequests().antMatchers("/api/ping").permitAll()
                .and()
                .authorizeRequests().antMatchers("/api/**").authenticated()
                .and()
                .authorizeRequests().antMatchers("/graphql").authenticated()
                .and()
                .authorizeRequests().antMatchers("/h2-console/**").permitAll()
                .and()
                .authorizeRequests().antMatchers("/swagger-ui.html").permitAll()
                .and()
                .authorizeRequests().antMatchers("/swagger-ui/**").permitAll()
                .and()
                .exceptionHandling()
                .authenticationEntryPoint(
                        (request, response, authException) -> {
                            if (request.getHeader(HttpHeaders.AUTHORIZATION) == null) {
                                setResponseError(response, HttpServletResponse.SC_UNAUTHORIZED, "Authentication Failed: Invalid Authorization Header");
                            } else {
                                setResponseError(response, HttpServletResponse.SC_UNAUTHORIZED, "Authentication Failed: Invalid Json Web Token");
                            }
                        }
                )
                .and()
                .addFilterBefore(jwtAuthenticationFilter, UsernamePasswordAuthenticationFilter.class)
                .logout().permitAll().logoutSuccessUrl("/");
    }

    @Bean
    CorsConfigurationSource corsConfigurationSource() {
        CorsConfiguration configuration = new CorsConfiguration();
        configuration.setAllowedHeaders(corsConfig.getAllowedHeaders());
        configuration.setAllowedMethods(corsConfig.getAllowedMethods());
        configuration.setAllowedOriginPatterns(corsConfig.getAllowedOrigins());
        UrlBasedCorsConfigurationSource source = new UrlBasedCorsConfigurationSource();
        source.registerCorsConfiguration("/graphql", configuration);
        source.registerCorsConfiguration("/api/**", configuration);
        return source;
    }
}
