package com.goodt.drive.fileupload.application.services;

import com.goodt.drive.fileupload.application.configuration.FileStorageConfig;
import com.goodt.drive.fileupload.application.exception.FileNotFoundException;
import com.goodt.drive.fileupload.application.exception.FileStorageException;
import com.goodt.drive.fileupload.application.utils.UploadUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.core.io.Resource;
import org.springframework.core.io.UrlResource;
import org.springframework.http.HttpHeaders;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;

@Slf4j
@Service("fileStorageService")
@ConditionalOnBean(FileStorageConfig.class)
public class FileStorageServiceImpl implements IFileUploadService {

    public FileStorageServiceImpl(FileStorageConfig fileStorageConfig) {
        fileStorageLocation = Paths.get(fileStorageConfig.getUploadDir()).toAbsolutePath().normalize();

        try {
            Files.createDirectories(fileStorageLocation);
        } catch (Exception ex) {
            throw new FileStorageException("Could not create the directory where the uploaded files will be stored.", ex);
        }
    }

    @Override
    public String uploadFile(MultipartFile uploadingFile, HttpServletRequest request) {
        log.debug("start uploading {} file to the server", uploadingFile.getOriginalFilename());
        String newFileName = UploadUtil.getNewFileName(uploadingFile.getOriginalFilename());
        try {
            Path targetLocation = fileStorageLocation.resolve(newFileName);
            uploadingFile.transferTo(targetLocation);
            return String.format("%s/%s", fileStorageConfig.getUrn(), newFileName);
        } catch (IOException | IllegalStateException ex) {
            throw new FileStorageException(String.format("Could not store file %s. Please try again!", uploadingFile.getOriginalFilename()), ex);
        }
    }

    @Override
    public ResponseEntity<?> downloadFile(String fileName, HttpServletRequest request) {
        try {
            Path filePath = fileStorageLocation.resolve(fileName).normalize();
            Resource resource = new UrlResource(filePath.toUri());
            if (!resource.exists()) {
                throw new FileNotFoundException(String.format("File %s is not found", fileName));
            }

            return ResponseEntity.ok()
                    .contentType(UploadUtil.getMediaTypeForFileName(request, resource.getFile().getAbsolutePath()))
                    .header(HttpHeaders.CONTENT_DISPOSITION, String.format("attachment; filename=\"%s\"", resource.getFilename()))
                    .body(resource);
        } catch (Exception ex) {
            throw new FileNotFoundException(String.format("File %s is not found", fileName), ex);
        }
    }

    private final Path fileStorageLocation;

    @Autowired
    private FileStorageConfig fileStorageConfig;
}
