package com.goodt.drive.fileupload.application.services;

import com.goodt.drive.fileupload.application.configuration.MinioConfig;
import com.goodt.drive.fileupload.application.exception.FileDownloadException;
import com.goodt.drive.fileupload.application.exception.FileNotFoundException;
import com.goodt.drive.fileupload.application.exception.FileStorageException;
import com.goodt.drive.fileupload.application.utils.UploadUtil;
import io.minio.*;
import io.minio.errors.*;
import io.minio.http.Method;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.core.io.InputStreamResource;
import org.springframework.http.HttpHeaders;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.io.InputStream;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;

@Service("minioService")
@ConditionalOnBean(MinioConfig.class)
public class MinioStorageServiceImpl implements IFileUploadService {

    @Override
    public String uploadFile(MultipartFile file, HttpServletRequest request) {
        try {
            String newFileName = UploadUtil.getNewFileName(file.getOriginalFilename());
            s3Client.putObject(
                    PutObjectArgs.builder()
                            .bucket(minioConfig.getBucketName())
                            .object(newFileName)
                            .stream(file.getInputStream(), file.getSize(), -1).build()
            );
            return s3Client.getPresignedObjectUrl(
                    GetPresignedObjectUrlArgs.builder()
                            .method(Method.GET)
                            .bucket(minioConfig.getBucketName())
                            .object(newFileName)
    .build());
        } catch (ErrorResponseException | InsufficientDataException | InternalException | InvalidKeyException | InvalidResponseException | IOException | NoSuchAlgorithmException | XmlParserException | ServerException ex) {
            throw new FileStorageException(String.format("Could not store file %s on MinioClient. Message: %s %nPlease try again!", file.getOriginalFilename(), ex.getMessage()), ex);
        }
    }

    @Override
    public ResponseEntity<?> downloadFile(String fileName, HttpServletRequest request) {
        try {
            StatObjectResponse statObject = s3Client.statObject(
                    StatObjectArgs.builder()
                            .bucket(minioConfig.getBucketName())
                            .object(fileName)
                            .build());

            if (statObject == null || statObject.size() == 0)
                throw new FileNotFoundException(String.format("File %s is not found", fileName));

            InputStream is = s3Client.getObject(
                    GetObjectArgs.builder()
                            .bucket(minioConfig.getBucketName())
                            .object(fileName)
                            .build());

            String encode = URLEncoder.encode(fileName, StandardCharsets.UTF_8);
            HttpHeaders headers = new HttpHeaders();
            headers.set(HttpHeaders.CONTENT_DISPOSITION, String.format("attachment; filename=\"%s\"", encode));
            return ResponseEntity.ok()
                    .contentType(UploadUtil.getMediaTypeForFileName(request, encode))
                    .headers(headers)
                    .body(new InputStreamResource(is));
        } catch (ErrorResponseException | InsufficientDataException | InternalException | InvalidKeyException | InvalidResponseException | IOException | NoSuchAlgorithmException | ServerException | XmlParserException ex) {
            throw new FileDownloadException(String.format("Could not download file %s through Minio", fileName), ex);
        }
    }

    @Autowired
    private MinioClient s3Client;

    @Autowired
    private MinioConfig minioConfig;
}
