package com.goodt.drive.fileupload.application.services;

import com.goodt.drive.fileupload.application.configuration.WebDavConfig;
import com.goodt.drive.fileupload.application.exception.FileDownloadException;
import com.goodt.drive.fileupload.application.exception.FileStorageException;
import com.goodt.drive.fileupload.application.utils.UploadUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.hc.client5.http.auth.AuthScope;
import org.apache.hc.client5.http.auth.UsernamePasswordCredentials;
import org.apache.hc.client5.http.classic.methods.HttpGet;
import org.apache.hc.client5.http.classic.methods.HttpPut;
import org.apache.hc.client5.http.entity.mime.MultipartEntityBuilder;
import org.apache.hc.client5.http.impl.auth.BasicCredentialsProvider;
import org.apache.hc.client5.http.impl.classic.CloseableHttpClient;
import org.apache.hc.client5.http.impl.classic.CloseableHttpResponse;
import org.apache.hc.client5.http.impl.classic.HttpClients;
import org.apache.hc.core5.http.ContentType;
import org.apache.hc.core5.http.HttpEntity;
import org.apache.hc.core5.http.io.entity.EntityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.core.io.ByteArrayResource;
import org.springframework.http.HttpHeaders;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;

@Slf4j
@Service
@ConditionalOnBean(WebDavConfig.class)
public class WebDavStorageServiceImpl implements IFileUploadService {

    @Autowired
    private WebDavConfig webDavConfig;

    @Override
    public String uploadFile(MultipartFile file, HttpServletRequest request) {
        String newFileName = UploadUtil.getNewFileName(file.getOriginalFilename());
        try {
            log.info("Connecting to {}", webDavConfig.getUrl());

            final BasicCredentialsProvider credProvider = new BasicCredentialsProvider();
            credProvider.setCredentials(new AuthScope(null, -1),
                                        new UsernamePasswordCredentials(webDavConfig.getUsername(), webDavConfig.getPassword().toCharArray()));

            HttpEntity entity = MultipartEntityBuilder.create()
                    .addBinaryBody("upload_file", file.getInputStream(), ContentType.APPLICATION_OCTET_STREAM, newFileName)
                    .build();

            try (CloseableHttpClient httpclient = HttpClients.custom().setDefaultCredentialsProvider(credProvider).build()) {
                final HttpPut httpPut = new HttpPut(String.format("%s/%s", webDavConfig.getUrl(), newFileName));
                httpPut.setEntity(entity);
                try (CloseableHttpResponse response = httpclient.execute(httpPut)) {
                    log.info("Response code: {}", response.getCode());
                }
            }

            String url = request.getRequestURL().toString();
            return String.format("%s%s%s%s", url.replaceAll(request.getRequestURI(), ""), webDavConfig.getUrn(), UploadUtil.NAME_PARAMETER, newFileName);
        } catch (Exception ex) {
            throw new FileStorageException(String.format("Could not store file %s on WebDav server. Message: %s\nPlease try again!", file.getOriginalFilename(), ex.getMessage()), ex);
        }
    }

    @Override
    public ResponseEntity<?> downloadFile(String fileName, HttpServletRequest request) {
        try {
            log.info("Connecting to {}", webDavConfig.getUrl());
            final BasicCredentialsProvider credProvider = new BasicCredentialsProvider();
            credProvider.setCredentials(new AuthScope(null, -1),
                                        new UsernamePasswordCredentials(webDavConfig.getUsername(), webDavConfig.getPassword().toCharArray()));
            byte[] result;
            try (CloseableHttpClient httpclient = HttpClients.custom()
                    .setDefaultCredentialsProvider(credProvider)
                    .build()) {
                final HttpGet httpget = new HttpGet(String.format("%s/%s", webDavConfig.getUrl(), fileName));
                try (CloseableHttpResponse response = httpclient.execute(httpget)) {
                    result = EntityUtils.toByteArray(response.getEntity());
                }
            }
            String encode = URLEncoder.encode(fileName, StandardCharsets.UTF_8);
            HttpHeaders headers = new HttpHeaders();
            headers.set(HttpHeaders.CONTENT_DISPOSITION, String.format("attachment; filename=\"%s\"", encode));
            return ResponseEntity.ok()
                    .contentType(UploadUtil.getMediaTypeForFileName(request, encode))
                    .headers(headers)
                    .body(new ByteArrayResource(result));
        } catch (IOException ex) {
            throw new FileDownloadException(String.format("Could not download file %s through WebDav server", fileName), ex);
        }
    }
}
