package com.goodt.drive.supeditor.application.authentication;

import javassist.NotFoundException;
import org.apache.commons.lang3.exception.ExceptionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.security.access.AccessDeniedException;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.web.AuthenticationEntryPoint;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;

import com.goodt.drive.supeditor.application.exceptions.BadRequestException;

@ControllerAdvice
public class AppAuthenticationEntryPoint implements AuthenticationEntryPoint {

    @ExceptionHandler(value = {BadRequestException.class})
    public void commence(HttpServletRequest request, HttpServletResponse response, BadRequestException badRequestException) throws IOException {
        logger.debug("Bad request (400): \"{}\" message: \"{}\"", request.getRequestURL(), badRequestException.getMessage());
        setResponseError(response, HttpServletResponse.SC_BAD_REQUEST, String.format("Bad request: \"%s\"", badRequestException.getMessage()));
    }

    @Override
    public void commence(HttpServletRequest request, HttpServletResponse response, AuthenticationException authException) throws IOException, ServletException {
        logger.debug(String.format("Access to resource is denied (401) for request: \"%s\" message: \"%s\"", request.getRequestURL(), authException.getMessage()));
        setResponseError(response, HttpServletResponse.SC_UNAUTHORIZED, "Authentication Failed");
    }
    
    @ExceptionHandler (value = {AccessDeniedException.class})
    public void commence(HttpServletRequest request, HttpServletResponse response, AccessDeniedException accessDeniedException) throws IOException {
        logger.debug(String.format("Access to resource is forbidden (403) for request: \"%s\" message: \"%s\"", request.getRequestURL(), accessDeniedException.getMessage()));
        setResponseError(response, HttpServletResponse.SC_FORBIDDEN, String.format("Access Denies: %s", accessDeniedException.getMessage()));
    }
    
    @ExceptionHandler (value = {NotFoundException.class})
    public void commence(HttpServletRequest request, HttpServletResponse response, NotFoundException notFoundException) throws IOException {
        logger.debug(String.format("Object was not found (404) for request: \"%s\" message: \"%s\"", request.getRequestURL(), notFoundException.getMessage()));
        setResponseError(response, HttpServletResponse.SC_NOT_FOUND, String.format("Not found: %s", notFoundException.getMessage()));
    }

    @ExceptionHandler(value = {OutOfMemoryError.class})
    public void commence(HttpServletRequest request, HttpServletResponse response, OutOfMemoryError outOfMemoryError) throws IOException {
        logger.error("Out of memory error: {}", ExceptionUtils.getStackTrace(outOfMemoryError));
        setResponseError(response, HttpServletResponse.SC_INTERNAL_SERVER_ERROR, String.format("Out of memory: %s", outOfMemoryError.getMessage()));
    }
    
    @ExceptionHandler (value = {Exception.class})
    public void commence(HttpServletRequest request, HttpServletResponse response, Exception exception) throws IOException {
        logger.error(String.format("An error occurred during request: %s %s error message: %s", 
                     request.getMethod(), request.getRequestURL(), exception.getMessage()));
        setResponseError(response, HttpServletResponse.SC_INTERNAL_SERVER_ERROR, String.format("Internal Server Error: %s", exception.getMessage()));
    }
    
    private void setResponseError(HttpServletResponse response, int errorCode, String errorMessage) throws IOException{
        response.setStatus(errorCode);
        response.getWriter().write(errorMessage);
        response.getWriter().flush();
        response.getWriter().close();
    }
    
    private final Logger logger = LoggerFactory.getLogger(this.getClass());
}
