package com.goodt.drive.supeditor.application.controllers.api;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.media.ArraySchema;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.responses.ApiResponses;
import javassist.NotFoundException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.AccessDeniedException;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.stream.Collectors;

import com.goodt.drive.supeditor.application.data.OperationResult;
import com.goodt.drive.supeditor.application.dto.DataConnectorDto;
import com.goodt.drive.supeditor.application.factories.DataConnectorFactory;
import com.goodt.drive.supeditor.application.model.entities.DataConnectorEntity;
import com.goodt.drive.supeditor.application.services.IDataConnectorService;

@CrossOrigin
@RestController
public class DataConnectorController extends BaseController {

    @Autowired
    private IDataConnectorService dataConnectorService;

    @Operation(summary = "Получить информацию о DataConnector", description = "Получить информацию о DataConnector", tags = {"dataconnector"})
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "В случае если объекты в БД были найдены (см. таблицу data_connector) и удалось получить их DTO представление",
                    content = @Content(schema = @Schema(implementation = DataConnectorDto.class))),
            @ApiResponse(responseCode = "401", description = "В случае если запрос отправлен без токена или с недействительным токеном возвращается ошибка 401 и строка с ошибкой в теле запроса",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "404", description = "В случае если событие с указанным идентификатором не существует",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "500", description = "В случае если что-то пошло не так (см. тело ответа с описанием ошибки)",
                    content = @Content(schema = @Schema(implementation = String.class))),
    })
    @GetMapping("/api/dataconnector/{id}")
    public DataConnectorDto get(
            @PathVariable(name = "id")
            @Parameter(name = "id", description = "Идентификатор конфигурации (таблица data_connector).", example = "1") Long id) throws NotFoundException {
        getSessionIdOrThrowException();
        checkPermission(dataConnectorService.find(id));
        DataConnectorEntity entity = dataConnectorService.find(id);
        return buildDto(entity);
    }

    @Operation(summary = "Получение массива DataConnector", description = "Получение массива DataConnector", tags = {"dataconnector"})
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "В случае если объекты в БД были найдены (см. таблицу data_connector) и удалось получить их DTO представление",
                    content = @Content(array = @ArraySchema(schema = @Schema(implementation = DataConnectorDto.class)))),
            @ApiResponse(responseCode = "401", description = "В случае если запрос отправлен без токена или с недействительным токеном возвращается ошибка 401 и строка с ошибкой в теле запроса",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "404", description = "В случае если событие с указанным идентификатором не существует",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "500", description = "В случае если что-то пошло не так (см. тело ответа с описанием ошибки)",
                    content = @Content(schema = @Schema(implementation = String.class))),
    })
    @GetMapping("/api/dataconnector")
    public List<DataConnectorDto> getAll(
            @RequestParam(name = "only_closed", required = false)
            @Parameter(name = "only_closed", description = "Только закрытые.", example = "true") boolean onlyClosed) {
        String sessionId = getSessionIdOrThrowException();
        List<DataConnectorEntity> entities = dataConnectorService.findAll(sessionId, onlyClosed);
        return entities.stream().map(this::buildDto).collect(Collectors.toList());
    }

    @Operation(summary = "Создание DataConnector", description = "Создание DataConnector", tags = {"dataconnector"})
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "В случае если объект был успешно создан или обновлен (см. таблицу data_connector)",
                    content = @Content(schema = @Schema(implementation = DataConnectorDto.class))),
            @ApiResponse(responseCode = "401", description = "В случае если запрос отправлен без токена или с недействительным токеном возвращается ошибка 401 и строка с ошибкой в теле запроса",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "404", description = "В случае если тип события с указанным идентификатором не существует",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "500", description = "В случае если что-то пошло не так (см. тело ответа с описанием ошибки)",
                    content = @Content(schema = @Schema(implementation = String.class))),
    })
    @PostMapping("/api/dataconnector")
    public DataConnectorDto create(
            @io.swagger.v3.oas.annotations.parameters.RequestBody(description = "DTO представление DataConnector (таблица data_connector)")
            @RequestBody DataConnectorDto dto) throws NotFoundException {
        String sessionId = getSessionIdOrThrowException();
        DataConnectorEntity newEntity = dataConnectorService.create(dto, sessionId);
        return buildDto(newEntity);
    }

    @Operation(summary = "Обновление DataConnector", description = "Обновление DataConnector", tags = {"dataconnector"})
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "В случае если объект был успешно создан или обновлен (см. таблицу data_connector)",
                    content = @Content(schema = @Schema(implementation = DataConnectorDto.class))),
            @ApiResponse(responseCode = "401", description = "В случае если запрос отправлен без токена или с недействительным токеном возвращается ошибка 401 и строка с ошибкой в теле запроса",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "404", description = "В случае если тип события с указанным идентификатором не существует",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "500", description = "В случае если что-то пошло не так (см. тело ответа с описанием ошибки)",
                    content = @Content(schema = @Schema(implementation = String.class))),
    })
    @PutMapping("/api/dataconnector/{id}")
    public DataConnectorDto update(
            @PathVariable(name = "id", required = false)
            @Parameter(name = "id", description = "Идентификатор DataConnector (таблица data_connector).", example = "1") Long id,
            @io.swagger.v3.oas.annotations.parameters.RequestBody(description = "DTO представление DataConnector (таблица data_connector)")
            @RequestBody DataConnectorDto dto) throws NotFoundException {
        getSessionIdOrThrowException();
        checkPermission(dataConnectorService.find(id));
        dto.setId(id);
        DataConnectorEntity newEntity = dataConnectorService.update(dto);
        return buildDto(newEntity);
    }

    @Operation(summary = "Удалить DataConnector", description = "Удалить DataConnector", tags = {"dataconnector"})
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "В случае если объекты в БД были найдены (см. таблицу data_connector) и удалось получить их DTO представление",
                    content = @Content(schema = @Schema(implementation = OperationResult.class))),
            @ApiResponse(responseCode = "401", description = "В случае если запрос отправлен без токена или с недействительным токеном возвращается ошибка 401 и строка с ошибкой в теле запроса",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "404", description = "В случае если событие с указанным идентификатором не существует",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "500", description = "В случае если что-то пошло не так (см. тело ответа с описанием ошибки)",
                    content = @Content(schema = @Schema(implementation = String.class))),
    })
    @DeleteMapping("/api/dataconnector/{id}")
    public OperationResult delete(
            @PathVariable(name = "id")
            @Parameter(name = "id", description = "Идентификатор DataConnector.", example = "1") Long id) throws NotFoundException {
        getSessionIdOrThrowException();
        checkPermission(dataConnectorService.find(id));
        dataConnectorService.delete(id);
        return new OperationResult(true, "");
    }

    @Operation(summary = "Восстановить удалённый DataConnector", description = "Восстановить удалённый DataConnector", tags = {"dataconnector"})
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "В случае если объекты в БД были найдены (см. таблицу data_connector) и удалось получить их DTO представление",
                    content = @Content(schema = @Schema(implementation = OperationResult.class))),
            @ApiResponse(responseCode = "401", description = "В случае если запрос отправлен без токена или с недействительным токеном возвращается ошибка 401 и строка с ошибкой в теле запроса",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "404", description = "В случае если событие с указанным идентификатором не существует",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "500", description = "В случае если что-то пошло не так (см. тело ответа с описанием ошибки)",
                    content = @Content(schema = @Schema(implementation = String.class))),
    })
    @PostMapping("/api/dataconnector/{id}/restore")
    public OperationResult restore(
            @PathVariable(name = "id")
            @Parameter(name = "id", description = "Идентификатор DataConnector.", example = "1") Long id) throws NotFoundException {
        getSessionIdOrThrowException();
        checkPermission(dataConnectorService.find(id));
        dataConnectorService.restore(id);
        return new OperationResult(true, "");
    }

    private DataConnectorDto buildDto(DataConnectorEntity entity) {
        DataConnectorDto dto = DataConnectorFactory.create(entity);
        setIsOwner(entity.getOwnerId(), dto);
        return dto;
    }

    private void setIsOwner(String ownerId, DataConnectorDto dto) {
        String sessionId = getSessionId();
        dto.setIsOwner(ownerId.equals(sessionId));
    }

    private String getSessionIdOrThrowException() {
        String sessionId = getSessionId();
        if (sessionId == null) {
            throw new AccessDeniedException("Not authorized.");
        }
        return sessionId;
    }

    private void checkPermission(DataConnectorEntity dataConnectorEntity) {
        String sessionId = getSessionId();
        if (!dataConnectorEntity.getOwnerId().equals(sessionId)) {
            throw new AccessDeniedException(String.format("Access denied Owner \"%s\" for DataConnector with id \"%d\"", sessionId, dataConnectorEntity.getId()));
        }
    }
}
