package com.goodt.drive.supeditor.application.controllers.api;

import com.goodt.drive.supeditor.application.data.OperationResult;
import com.goodt.drive.supeditor.application.dto.FolderDto;
import com.goodt.drive.supeditor.application.dto.SharedGroupPermissionDto;
import com.goodt.drive.supeditor.application.dto.keycloak.GroupDto;
import com.goodt.drive.supeditor.application.dto.keycloak.UserDto;
import com.goodt.drive.supeditor.application.factories.FolderFactory;
import com.goodt.drive.supeditor.application.model.entities.ProjectTypeEntity;
import com.goodt.drive.supeditor.application.model.entities.FolderEntity;
import com.goodt.drive.supeditor.application.services.IProjectTypeService;
import com.goodt.drive.supeditor.application.services.IFolderService;
import com.goodt.drive.supeditor.application.services.clients.IKeyCloakServiceClient;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.media.ArraySchema;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.responses.ApiResponses;
import javassist.NotFoundException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.AccessDeniedException;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

@CrossOrigin
@RestController
public class FolderController extends BaseController {

    @Autowired
    private IFolderService folderService;

    @Autowired
    private IProjectTypeService projectTypeService;

    @Autowired
    private IKeyCloakServiceClient keyCloakServiceClient;

    @Operation(summary = "Получение массива директорий", description = "Получение массива директорий", tags = { "folder" })
    @ApiResponses(value = {
                     @ApiResponse(responseCode = "200", description = "В случае если объекты в БД были найдены (см. таблицу folder) и удалось получить их DTO представление",
                                  content = @Content(array = @ArraySchema(schema = @Schema(implementation = FolderDto.class)))),
                     @ApiResponse(responseCode = "401", description = "В случае если запрос отправлен без токена или с недействительным токеном возвращается ошибка 401 и строка с ошибкой в теле запроса",
                                  content = @Content(schema = @Schema(implementation = String.class))),
                     @ApiResponse(responseCode = "404", description = "В случае если событие с указанным идентификатором не существует",
                                  content = @Content(schema = @Schema(implementation = String.class))),
                     @ApiResponse(responseCode = "500", description = "В случае если что-то пошло не так (см. тело ответа с описанием ошибки)",
                                  content = @Content(schema = @Schema(implementation = String.class))),
    })
    @GetMapping("/api/folder")
    public List<FolderDto> getAll(@RequestParam(name = "parent_id", required = false)
                                      @Parameter(name = "parent_id", description = "Идентификатор родительской директории (таблица folder).", example = "1")
                                      Long parentId,
                                  @RequestParam(name = "type", required = true)
                                      @Parameter(name = "type", description = "Название типа конфигурации (таблица project_type).", example = "app")
                                      String projectTypeName,
                                  @RequestParam(name = "id", required = false)
                                      @Parameter(name = "id", description = "Массив идентификаторов (таблица folder).", example = "[1,2,3]")
                                      List<Long> idList) throws NotFoundException {
        String sessionId = getSessionIdOrThrowException();
        ProjectTypeEntity projectTypeEntity = projectTypeService.findByName(projectTypeName);
        Set<String> groupMembers = getGroupMembers(sessionId);
        List<FolderEntity> folderEntities = folderService.findAll(sessionId, parentId, projectTypeEntity.getId(), idList, groupMembers);
        return folderEntities.stream().map(this::buildFolderDto).collect(Collectors.toList());
    }

    @Operation(summary = "Создавние директории", description = "Создавние директории", tags = { "folder" })
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "В случае если объект был успешно создан или обновлен (см. таблицу folder)",
                    content = @Content(schema = @Schema(implementation = FolderDto.class))),
            @ApiResponse(responseCode = "401", description = "В случае если запрос отправлен без токена или с недействительным токеном возвращается ошибка 401 и строка с ошибкой в теле запроса",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "500", description = "В случае если что-то пошло не так (см. тело ответа с описанием ошибки)",
                    content = @Content(schema = @Schema(implementation = String.class))),
    })
    @PostMapping("/api/folder")
    public FolderDto create(@io.swagger.v3.oas.annotations.parameters.RequestBody(description = "DTO представление директорий (таблица folder)")
                                @RequestBody FolderDto dto) throws NotFoundException {
        String sessionId = getSessionIdOrThrowException();
        if (dto.getParentId() != null) {
            checkPermission(folderService.find(dto.getParentId()));
        }
        FolderEntity folderEntity = folderService.create(dto, sessionId);
        return buildFolderDto(folderEntity);
    }

    @Operation(summary = "Обновление директории", description = "Обновление директории", tags = { "folder" })
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "В случае если объект был успешно создан или обновлен (см. таблицу folder)",
                    content = @Content(schema = @Schema(implementation = FolderDto.class))),
            @ApiResponse(responseCode = "401", description = "В случае если запрос отправлен без токена или с недействительным токеном возвращается ошибка 401 и строка с ошибкой в теле запроса",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "500", description = "В случае если что-то пошло не так (см. тело ответа с описанием ошибки)",
                    content = @Content(schema = @Schema(implementation = String.class))),
    })
    @PutMapping("/api/folder/{id}")
    public FolderDto update(@PathVariable(name = "id", required = false)
                                @Parameter(name = "id", description = "Идентификатор директории (таблица folder).", example = "1")
                                Long id,
                            @io.swagger.v3.oas.annotations.parameters.RequestBody(description = "DTO представление директорий (таблица folder)")
                                @RequestBody FolderDto dto) throws NotFoundException {
        getSessionIdOrThrowException();
        checkPermission(folderService.find(id), new SharedGroupPermissionDto(true));
        if (dto.getParentId() != null) {
            checkPermission(folderService.find(dto.getParentId()));
        }
        dto.setId(id);
        FolderEntity folderEntity = folderService.update(dto);
        return buildFolderDto(folderEntity);
    }

    @Operation(summary = "Удаление директории", description = "Удаление директории", tags = { "folder" })
    @ApiResponses(value = {
            @ApiResponse(responseCode = "204", description = "В случае если объект в БД (см. таблицу folder) был удален (поставлено date_to=NOW())"),
            @ApiResponse(responseCode = "401", description = "В случае если запрос отправлен без токена или с недействительным токеном возвращается ошибка 401 и строка с ошибкой в теле запроса",
                    content = @Content(schema = @Schema(implementation = OperationResult.class))),
            @ApiResponse(responseCode = "404", description = "В случае если тип события с указанным идентификатором не существует",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "500", description = "В случае если что-то пошло не так (см. тело ответа с описанием ошибки)",
                    content = @Content(schema = @Schema(implementation = String.class))),
    })
    @DeleteMapping("/api/folder/{id}")
    public OperationResult delete(@PathVariable(name = "id", required = true)
                                      @Parameter(name = "id", description = "Идентификатор директории (таблица folder).", example = "1")
                                      Long id) throws NotFoundException {
        getSessionIdOrThrowException();
        checkPermission(folderService.find(id), new SharedGroupPermissionDto(true));
        folderService.delete(id);
        return new OperationResult(true, "");
    }

    @Operation(summary = "Добавить шаринг директории для группы", description = "Добавить шаринг директории для группы", tags = { "folder" })
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "В случае если объект был успешно обновлен (см. таблицу folder)",
                    content = @Content(schema = @Schema(implementation = FolderDto.class))),
            @ApiResponse(responseCode = "401", description = "В случае если запрос отправлен без токена или с недействительным токеном возвращается ошибка 401 и строка с ошибкой в теле запроса",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "404", description = "В случае если тип события с указанным идентификатором не существует",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "500", description = "В случае если что-то пошло не так (см. тело ответа с описанием ошибки)",
                    content = @Content(schema = @Schema(implementation = String.class))),
    })
    @PutMapping("/api/folder/{id}/sharegroup")
    public FolderDto shareGroup(@PathVariable(name = "id", required = true)
                                 @Parameter(name = "id", description = "Идентификатор директории (таблица folder).", example = "1")
                                 Long id,
                            @io.swagger.v3.oas.annotations.parameters.RequestBody(description = "DTO прав доступа (таблица folder)")
                                    @RequestBody SharedGroupPermissionDto dto) throws NotFoundException {
        getSessionIdOrThrowException();
        FolderEntity folderEntity = folderService.find(id);
        checkPermissionIsOwner(folderEntity);
        FolderEntity updatedFolderEntity = folderService.shareGroup(id, dto);
        return buildFolderDto(updatedFolderEntity);
    }

    @Operation(summary = "Снять шаринг директории для группы", description = "Снять шаринг директории для группы", tags = { "folder" })
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "В случае если объект был успешно обновлен (см. таблицу folder)",
                    content = @Content(schema = @Schema(implementation = FolderDto.class))),
            @ApiResponse(responseCode = "401", description = "В случае если запрос отправлен без токена или с недействительным токеном возвращается ошибка 401 и строка с ошибкой в теле запроса",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "404", description = "В случае если тип события с указанным идентификатором не существует",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "500", description = "В случае если что-то пошло не так (см. тело ответа с описанием ошибки)",
                    content = @Content(schema = @Schema(implementation = String.class))),
    })
    @PutMapping("/api/folder/{id}/unsharegroup")
    public FolderDto unshareGroup(@PathVariable(name = "id", required = true)
                                   @Parameter(name = "id", description = "Идентификатор директории (таблица folder).", example = "1")
                                   Long id) throws NotFoundException {
        getSessionIdOrThrowException();
        FolderEntity folderEntity = folderService.find(id);
        checkPermissionIsOwner(folderEntity);
        FolderEntity updatedFolderEntity = folderService.unshareGroup(id);
        return buildFolderDto(updatedFolderEntity);
    }

    private FolderDto buildFolderDto(FolderEntity folderEntity) {
        FolderDto folderDto = FolderFactory.create(folderEntity);
        setIsOwner(folderEntity.getOwnerId(), folderDto);
        return folderDto;
    }

    private void setIsOwner(String ownerId, FolderDto dto) {
        String sessionId = getSessionId();
        dto.setIsOwner(ownerId.equals(sessionId));
    }

    private String getSessionIdOrThrowException() {
        String sessionId = getSessionId();
        if (sessionId == null) {
            throw new AccessDeniedException("Not authorized.");
        }
        return sessionId;
    }

    private void checkPermissionIsOwner(FolderEntity folderEntity) {
        String sessionId = getSessionId();
        if (!folderEntity.getOwnerId().equals(sessionId)) {
            throw new AccessDeniedException(String.format("Access denied Owner \"%s\" for Folder with id \"%d\"", sessionId, folderEntity.getId()));
        }
    }

    private void checkPermission(FolderEntity folderEntity) {
        String sessionId = getSessionId();
        if (folderEntity.isSharedGroup()) {
            Set<String> groupMembers = getGroupMembers(sessionId);
            if (!groupMembers.contains(folderEntity.getOwnerId())) {
                throw new AccessDeniedException(String.format("Access denied Owner \"%s\" in shared group for Folder with id \"%d\"", sessionId, folderEntity.getId()));
            }
        } else {
            checkPermissionIsOwner(folderEntity);
        }
    }

    private void checkPermission(FolderEntity folderEntity, SharedGroupPermissionDto sharedGroupPermissionDto) {
        checkPermission(folderEntity);
        String sessionId = getSessionId();
        if (!folderEntity.getOwnerId().equals(sessionId) && folderEntity.isSharedGroup()
                && sharedGroupPermissionDto.isCanSharedGroupEdit() && !folderEntity.isCanSharedGroupEdit()) {
            throw new AccessDeniedException(String.format("Access denied Owner \"%s\" for Folder with id \"%d\" to edit shared group", sessionId, folderEntity.getId()));
        }
    }

    private Set<String> getGroupMembers(String sessionId) {
        Set<String> sessionIdList = new HashSet<>();
        List<GroupDto> groupDtos = keyCloakServiceClient.getGroupAll(sessionId);
        for (GroupDto groupDto : groupDtos) {
            for (UserDto userDto : keyCloakServiceClient.getGroupMemberAll(groupDto.getId())) {
                String userDtoId = userDto.getId();
                sessionIdList.add(userDtoId);
            }
        }
        return sessionIdList;
    }
}
