package com.goodt.drive.supeditor.application.controllers.api;

import com.goodt.drive.supeditor.GlobalDefs;
import com.goodt.drive.supeditor.application.data.OperationResult;
import com.goodt.drive.supeditor.application.data.UserInfo;
import com.goodt.drive.supeditor.application.dto.*;
import com.goodt.drive.supeditor.application.dto.keycloak.GroupDto;
import com.goodt.drive.supeditor.application.dto.keycloak.UserDto;
import com.goodt.drive.supeditor.application.exceptions.BadRequestException;
import com.goodt.drive.supeditor.application.exceptions.git.TagsAlreadyExistsException;
import com.goodt.drive.supeditor.application.factories.ProjectFactory;
import com.goodt.drive.supeditor.application.model.entities.*;
import com.goodt.drive.supeditor.application.services.IFolderService;
import com.goodt.drive.supeditor.application.services.IProjectService;
import com.goodt.drive.supeditor.application.services.IProjectTagService;
import com.goodt.drive.supeditor.application.services.IProjectTypeService;
import com.goodt.drive.supeditor.application.services.clients.IKeyCloakServiceClient;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.media.ArraySchema;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.responses.ApiResponses;
import javassist.NotFoundException;
import org.apache.commons.lang3.StringUtils;
import org.eclipse.jgit.api.errors.GitAPIException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.AccessDeniedException;
import org.springframework.util.AntPathMatcher;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.HandlerMapping;

import javax.servlet.http.HttpServletRequest;
import java.io.*;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

@CrossOrigin
@RestController
public class ProjectController extends BaseController {

    @Autowired
    private IProjectService projectService;
    @Autowired
    private IFolderService folderService;
    @Autowired
    private IProjectTagService projectTagService;
    @Autowired
    private IProjectTypeService projectTypeService;
    @Autowired
    private IKeyCloakServiceClient keyCloakServiceClient;

    @Operation(summary = "Получить файл приложения", description = "Получить файл приложения", tags = {"project"})
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "В случае если объекты в БД были найдены (см. таблицу project) и удалось получить их DTO представление",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "401", description = "В случае если запрос отправлен без токена или с недействительным токеном возвращается ошибка 401 и строка с ошибкой в теле запроса",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "404", description = "В случае если событие с указанным идентификатором не существует",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "500", description = "В случае если что-то пошло не так (см. тело ответа с описанием ошибки)",
                    content = @Content(schema = @Schema(implementation = String.class))),
    })
    @GetMapping("/api/project/{project}/**")
    public ResponseEntity<byte[]> preview(
            @PathVariable(name = "project")
            @Parameter(name = "project", description = "Идентификатор или slug приложения.", example = "1") String project,
            HttpServletRequest request) throws NotFoundException {
        getSessionIdOrThrowException();
        Long projectId;
        if (StringUtils.isNumeric(project)) {
            projectId = Long.valueOf(project);
            checkPermission(projectService.find(projectId));
        } else {
            ProjectEntity projectEntity = projectService.findBySlug(project);
            checkPermission(projectEntity);
            projectId = projectEntity.getId();
        }
        String path = (String) request.getAttribute(HandlerMapping.PATH_WITHIN_HANDLER_MAPPING_ATTRIBUTE);
        String matchPattern = (String) request.getAttribute(HandlerMapping.BEST_MATCHING_PATTERN_ATTRIBUTE);
        String filepath = new AntPathMatcher().extractPathWithinPattern(matchPattern, path);
        ProjectContentEntity projectContentEntity = projectService.findContent(projectId, filepath);
        HttpHeaders headers = new HttpHeaders();
        headers.add(HttpHeaders.CONTENT_TYPE, String.format("%s; charset=utf-8", projectContentEntity.getContentType()));
        headers.setContentLength(projectContentEntity.getLength());
        headers.add(HttpHeaders.CONTENT_DISPOSITION, String.format("attachment; filename=\"%s\"", projectContentEntity.getFilename()));
        return new ResponseEntity<>(projectContentEntity.getByteContent(), headers, HttpStatus.OK);
    }

    @Operation(summary = "Получить информацию о приложении", description = "Получить информацию о приложении", tags = {"project"})
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "В случае если объекты в БД были найдены (см. таблицу project) и удалось получить их DTO представление",
                    content = @Content(schema = @Schema(implementation = ProjectInfoDto.class))),
            @ApiResponse(responseCode = "401", description = "В случае если запрос отправлен без токена или с недействительным токеном возвращается ошибка 401 и строка с ошибкой в теле запроса",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "404", description = "В случае если событие с указанным идентификатором не существует",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "500", description = "В случае если что-то пошло не так (см. тело ответа с описанием ошибки)",
                    content = @Content(schema = @Schema(implementation = String.class))),
    })
    @GetMapping("/api/project/{id}")
    public ProjectInfoDto get(
            @PathVariable(name = "id")
            @Parameter(name = "id", description = "Идентификатор конфигурации (таблица project).", example = "1") Long id) throws NotFoundException {
        getSessionIdOrThrowException();
        checkPermission(projectService.find(id));
        ProjectEntity projectEntity = projectService.find(id);
        ProjectDto projectDto = buildProjectDto(projectEntity);
        byte[] projectData = projectService.findContent(id, GlobalDefs.MAIN_JSON_FILENAME).getByteContent();
        return new ProjectInfoDto(projectDto, new String(projectData));
    }

    @Operation(summary = "Проверить существование приложения", description = "Проверить существование приложения", tags = {"project"})
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "В случае если объекты в БД были найдены (см. таблицу project) и удалось получить их DTO представление",
                    content = @Content(schema = @Schema(implementation = OperationResult.class))),
            @ApiResponse(responseCode = "401", description = "В случае если запрос отправлен без токена или с недействительным токеном возвращается ошибка 401 и строка с ошибкой в теле запроса",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "404", description = "В случае если событие с указанным идентификатором не существует",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "500", description = "В случае если что-то пошло не так (см. тело ответа с описанием ошибки)",
                    content = @Content(schema = @Schema(implementation = String.class))),
    })
    @GetMapping("/api/project/{slug}/exists")
    public OperationResult exists(
            @PathVariable(name = "slug")
            @Parameter(name = "slug", description = "Slug конфигурации (таблица project).", example = "1") String slug,
            @RequestParam(name = "exclude_id", required = false)
            @Parameter(name = "exclude_id", description = "Исключающий идентификатор конфигурации (таблица project).", example = "1") Long excludeId) {
        getSessionIdOrThrowException();
        boolean result = false;
        try {
            ProjectEntity projectEntity = projectService.findBySlug(slug);
            if (excludeId != null) {
                if (!excludeId.equals(projectEntity.getId())) {
                    result = true;
                }
            } else {
                result = true;
            }
        } catch (NotFoundException ignored) {

        }
        return new OperationResult(result, "");
    }

    @Operation(summary = "Получение массива приложений", description = "Получение массива приложений", tags = {"project"})
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "В случае если объекты в БД были найдены (см. таблицу project) и удалось получить их DTO представление",
                    content = @Content(array = @ArraySchema(schema = @Schema(implementation = ProjectDto.class)))),
            @ApiResponse(responseCode = "401", description = "В случае если запрос отправлен без токена или с недействительным токеном возвращается ошибка 401 и строка с ошибкой в теле запроса",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "404", description = "В случае если событие с указанным идентификатором не существует",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "500", description = "В случае если что-то пошло не так (см. тело ответа с описанием ошибки)",
                    content = @Content(schema = @Schema(implementation = String.class))),
    })
    @GetMapping("/api/project")
    public List<ProjectDto> getAll(
            @RequestParam(name = "folder_id", required = false)
            @Parameter(name = "folder_id", description = "Идентификатор директории (таблица folder).", example = "1") Long folderId,
            @RequestParam(name = "type")
            @Parameter(name = "type", description = "Название типа конфигурации (таблица project_type).", example = "app") String projectTypeName,
            @RequestParam(name = "key", required = false)
            @Parameter(name = "key", description = "Ключ доступа.", example = "1") String key) throws NotFoundException {
        boolean checkOwner = key == null || !key.equals(getAppConfig().getAppKey());
        String sessionId = null;
        Set<String> groupMembers = new HashSet<>();
        if (checkOwner) {
            sessionId = getSessionIdOrThrowException();
            groupMembers = getGroupMembers(sessionId);
        }
        ProjectTypeEntity projectTypeEntity = projectTypeService.findByName(projectTypeName);
        List<ProjectEntity> projectEntities = projectService.findAll(sessionId, folderId, projectTypeEntity.getId(), groupMembers);
        return projectEntities.stream().map(this::buildProjectDto).collect(Collectors.toList());
    }

    @Operation(summary = "Получение массива всех приложений по ключу", description = "Получение массива всех приложений по ключу", tags = {"project"})
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "В случае если объекты в БД были найдены (см. таблицу project) и удалось получить их DTO представление",
                    content = @Content(array = @ArraySchema(schema = @Schema(implementation = ProjectDto.class)))),
            @ApiResponse(responseCode = "404", description = "В случае если событие с указанным идентификатором не существует",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "500", description = "В случае если что-то пошло не так (см. тело ответа с описанием ошибки)",
                    content = @Content(schema = @Schema(implementation = String.class))),
    })
    @GetMapping("/api/project/listall")
    public List<ProjectDto> getListAll(
            @RequestParam(name = "key")
            @Parameter(name = "key", description = "Ключ доступа.", example = "1") String key) {
        if (key == null || !key.equals(getAppConfig().getAppKey())) {
            throw new AccessDeniedException("Access Denied: Key is not valid.");
        }
        List<ProjectEntity> projectEntities = projectService.findAll();
        return projectEntities.stream().map(this::buildProjectDto).collect(Collectors.toList());
    }

    @Operation(summary = "Экспортировать приложение", description = "Экспортировать приложение", tags = {"project"})
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "В случае если объекты в БД были найдены (см. таблицу project) и удалось получить их DTO представление",
                    content = @Content(schema = @Schema(implementation = ResponseEntity.class))),
            @ApiResponse(responseCode = "401", description = "В случае если запрос отправлен без токена или с недействительным токеном возвращается ошибка 401 и строка с ошибкой в теле запроса",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "404", description = "В случае если событие с указанным идентификатором не существует",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "500", description = "В случае если что-то пошло не так (см. тело ответа с описанием ошибки)",
                    content = @Content(schema = @Schema(implementation = String.class))),
    })
    @GetMapping("/api/project/{id}/export")
    public ResponseEntity<?> export(
            @PathVariable(name = "id")
            @Parameter(name = "id", description = "Идентификатор конфигурации (таблица project).", example = "1") Long id,
            @RequestParam(name = "ref", required = false)
            @Parameter(name = "ref", description = "Commit ref", example = "1") String ref,
            @RequestParam(name = "key", required = false)
            @Parameter(name = "key", description = "Ключ доступа.", example = "1") String key) throws NotFoundException, IOException {
        ProjectEntity projectEntity = projectService.find(id);
        boolean checkOwner = key == null || !key.equals(getAppConfig().getAppKey());
        if (checkOwner) {
            getSessionIdOrThrowException();
            checkPermission(projectEntity);
        }
        String filename;
        if (ref != null) {
            filename = URLEncoder.encode(String.format("%s_%s.zip", projectEntity.getTitle(), ref), StandardCharsets.UTF_8);
        } else {
            filename = URLEncoder.encode(String.format("%s.zip", projectEntity.getTitle()), StandardCharsets.UTF_8);
        }
        ByteArrayOutputStream data = projectService.exportProject(id, ref);
        HttpHeaders headers = new HttpHeaders();
        headers.setContentLength(data.size());
        headers.add(HttpHeaders.CONTENT_TYPE, "application/zip");
        headers.add(HttpHeaders.CONTENT_DISPOSITION, String.format("attachment; filename=\"%s\"", filename));
        return new ResponseEntity<>(data.toByteArray(), headers, HttpStatus.OK);
    }

    @Operation(summary = "Импортировать приложение", description = "Импортировать приложение", tags = {"project"})
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "В случае если объекты в БД были найдены (см. таблицу project) и удалось получить их DTO представление",
                    content = @Content(schema = @Schema(implementation = ProjectDto.class))),
            @ApiResponse(responseCode = "401", description = "В случае если запрос отправлен без токена или с недействительным токеном возвращается ошибка 401 и строка с ошибкой в теле запроса",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "404", description = "В случае если событие с указанным идентификатором не существует",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "500", description = "В случае если что-то пошло не так (см. тело ответа с описанием ошибки)",
                    content = @Content(schema = @Schema(implementation = String.class))),
    })
    @RequestMapping(value = "/api/project/import", method = RequestMethod.POST, consumes = {MediaType.MULTIPART_FORM_DATA_VALUE})
    public ProjectDto importAction(
            @io.swagger.v3.oas.annotations.parameters.RequestBody(description = "DTO представление приложения (таблица project)")
            @RequestPart(value = "body") ProjectDto dto,
            @RequestPart(name = "archive")
            @Parameter(name = "archive", description = "Файл архив конфигурации.") MultipartFile archive) throws NotFoundException, IOException {
        String sessionId = getSessionIdOrThrowException();
        ProjectEntity newProjectEntity = projectService.importProject(sessionId, dto, archive);
        if (newProjectEntity.getOwnerId().equals(sessionId)) {
            if (dto.isSharedGroup()) {
                newProjectEntity = projectService.shareGroup(newProjectEntity.getId(), new SharedGroupPermissionDto(newProjectEntity.isCanSharedGroupEdit()));
            } else {
                newProjectEntity = projectService.unshareGroup(newProjectEntity.getId());
            }
        }
        return buildProjectDto(newProjectEntity);
    }

    @Operation(summary = "Клонировать приложение", description = "Клонировать приложение", tags = {"project"})
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "В случае если объекты в БД были найдены (см. таблицу project) и удалось получить их DTO представление",
                    content = @Content(schema = @Schema(implementation = ProjectDto.class))),
            @ApiResponse(responseCode = "401", description = "В случае если запрос отправлен без токена или с недействительным токеном возвращается ошибка 401 и строка с ошибкой в теле запроса",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "404", description = "В случае если событие с указанным идентификатором не существует",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "500", description = "В случае если что-то пошло не так (см. тело ответа с описанием ошибки)",
                    content = @Content(schema = @Schema(implementation = String.class))),
    })
    @PostMapping("/api/project/{id}/clone")
    public ProjectDto cloneAction(
            @PathVariable(name = "id")
            @Parameter(name = "id", description = "Идентификатор конфигурации (таблица project).", example = "1") Long id,
            @io.swagger.v3.oas.annotations.parameters.RequestBody(description = "DTO представление приложения (таблица project)")
            @RequestBody ProjectDto dto) throws NotFoundException, IOException {
        String sessionId = getSessionIdOrThrowException();
        ProjectEntity projectEntity = projectService.find(id);
        checkPermission(projectEntity);
        if (dto.getFolderId() != null) {
            checkPermissionFolder(folderService.find(dto.getFolderId()));
        }
        dto.setId(id);
        dto.setSharedGroup(false);
        ProjectEntity newProjectEntity = projectService.cloneProject(dto, sessionId);
        return buildProjectDto(newProjectEntity);
    }

    @Operation(summary = "Создание приложения", description = "Создание приложения", tags = {"project"})
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "В случае если объект был успешно создан или обновлен (см. таблицу project)",
                    content = @Content(schema = @Schema(implementation = ProjectDto.class))),
            @ApiResponse(responseCode = "401", description = "В случае если запрос отправлен без токена или с недействительным токеном возвращается ошибка 401 и строка с ошибкой в теле запроса",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "404", description = "В случае если тип события с указанным идентификатором не существует",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "500", description = "В случае если что-то пошло не так (см. тело ответа с описанием ошибки)",
                    content = @Content(schema = @Schema(implementation = String.class))),
    })
    @PostMapping("/api/project")
    public ProjectDto create(
            @io.swagger.v3.oas.annotations.parameters.RequestBody(description = "DTO представление приложения (таблица project)")
            @RequestBody ProjectDto dto) throws NotFoundException {
        String sessionId = getSessionIdOrThrowException();
        if (dto.getFolderId() != null) {
            checkPermissionFolder(folderService.find(dto.getFolderId()));
        }
        ProjectEntity newProjectEntity = projectService.create(dto, sessionId);
        if (newProjectEntity.getOwnerId().equals(sessionId)) {
            if (dto.isSharedGroup()) {
                newProjectEntity = projectService.shareGroup(newProjectEntity.getId(), new SharedGroupPermissionDto(newProjectEntity.isCanSharedGroupEdit()));
            } else {
                newProjectEntity = projectService.unshareGroup(newProjectEntity.getId());
            }
        }
        return buildProjectDto(newProjectEntity);
    }

    @Operation(summary = "Обновление приложения", description = "Обновление приложения", tags = {"project"})
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "В случае если объект был успешно создан или обновлен (см. таблицу project)",
                    content = @Content(schema = @Schema(implementation = ProjectDto.class))),
            @ApiResponse(responseCode = "401", description = "В случае если запрос отправлен без токена или с недействительным токеном возвращается ошибка 401 и строка с ошибкой в теле запроса",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "404", description = "В случае если тип события с указанным идентификатором не существует",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "500", description = "В случае если что-то пошло не так (см. тело ответа с описанием ошибки)",
                    content = @Content(schema = @Schema(implementation = String.class))),
    })
    @PutMapping("/api/project/{id}")
    public ProjectDto update(
            @PathVariable(name = "id", required = false)
            @Parameter(name = "id", description = "Идентификатор приложения (таблица project).", example = "1") Long id,
            @io.swagger.v3.oas.annotations.parameters.RequestBody(description = "DTO представление приложения (таблица project)")
            @RequestBody ProjectDto dto) throws NotFoundException {
        String sessionId = getSessionIdOrThrowException();
        checkPermission(projectService.find(id), new SharedGroupPermissionDto(true));
        if (dto.getFolderId() != null) {
            checkPermissionFolder(folderService.find(dto.getFolderId()));
        }
        dto.setId(id);
        ProjectEntity projectEntity = projectService.update(dto);
        if (projectEntity.getOwnerId().equals(sessionId)) {
            if (dto.isSharedGroup()) {
                projectEntity = projectService.shareGroup(projectEntity.getId(), new SharedGroupPermissionDto(projectEntity.isCanSharedGroupEdit()));
            } else {
                projectEntity = projectService.unshareGroup(projectEntity.getId());
            }
        }
        return buildProjectDto(projectEntity);
    }

    @Operation(summary = "Удалить приложение", description = "Удалить приложение", tags = {"project"})
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "В случае если объекты в БД были найдены (см. таблицу project) и удалось получить их DTO представление",
                    content = @Content(schema = @Schema(implementation = OperationResult.class))),
            @ApiResponse(responseCode = "401", description = "В случае если запрос отправлен без токена или с недействительным токеном возвращается ошибка 401 и строка с ошибкой в теле запроса",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "404", description = "В случае если событие с указанным идентификатором не существует",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "500", description = "В случае если что-то пошло не так (см. тело ответа с описанием ошибки)",
                    content = @Content(schema = @Schema(implementation = String.class))),
    })
    @DeleteMapping("/api/project/{id}")
    public OperationResult delete(
            @PathVariable(name = "id")
            @Parameter(name = "id", description = "Идентификатор приложения.", example = "1") Long id) throws NotFoundException, IOException {
        getSessionIdOrThrowException();
        checkPermission(projectService.find(id), new SharedGroupPermissionDto(true));
        projectService.delete(id);
        return new OperationResult(true, "");
    }

    @Operation(summary = "Получить информацию о текущем коммите приложения", description = "Получить информацию о текущем коммите приложения", tags = {"project"})
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "В случае если объект был успешно создан или обновлен (см. таблицу project)",
                    content = @Content(schema = @Schema(implementation = ProjectDto.class))),
            @ApiResponse(responseCode = "401", description = "В случае если запрос отправлен без токена или с недействительным токеном возвращается ошибка 401 и строка с ошибкой в теле запроса",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "404", description = "В случае если тип события с указанным идентификатором не существует",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "500", description = "В случае если что-то пошло не так (см. тело ответа с описанием ошибки)",
                    content = @Content(schema = @Schema(implementation = String.class))),
    })
    @GetMapping("/api/project/{id}/commit")
    public List<ProjectCommitDto> getCommitAllProject(
            @PathVariable(name = "id")
            @Parameter(name = "id", description = "Идентификатор приложения (таблица project).", example = "1") Long id) throws NotFoundException, IOException, GitAPIException {
        String sessionId = getSessionIdOrThrowException();
        checkPermission(projectService.find(id));
        return projectService.getCommitAll(sessionId, id);
    }

    @Operation(summary = "Commit приложения", description = "Commit приложения", tags = {"project"})
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "В случае если объект был успешно создан или обновлен (см. таблицу project)",
                    content = @Content(schema = @Schema(implementation = ProjectDto.class))),
            @ApiResponse(responseCode = "401", description = "В случае если запрос отправлен без токена или с недействительным токеном возвращается ошибка 401 и строка с ошибкой в теле запроса",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "404", description = "В случае если тип события с указанным идентификатором не существует",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "500", description = "В случае если что-то пошло не так (см. тело ответа с описанием ошибки)",
                    content = @Content(schema = @Schema(implementation = String.class))),
    })
    @PostMapping("/api/project/{id}/commit")
    public ProjectCommitDto commitProject(
            @PathVariable(name = "id")
            @Parameter(name = "id", description = "Идентификатор приложения (таблица project).", example = "1") Long id,
            @io.swagger.v3.oas.annotations.parameters.RequestBody(description = "DTO представление приложения (таблица project)")
            @RequestBody ProjectCommitDto dto) throws NotFoundException, GitAPIException, IOException {
        String sessionId = getSessionIdOrThrowException();
        checkPermission(projectService.find(id), new SharedGroupPermissionDto(true));
        if (dto.getTags() == null) {
            throw new IllegalArgumentException("Tags cannot be null");
        }
        UserInfo userInfo = getUserInfo();
        try {
            return projectService.commitProject(sessionId, id, dto.getComment(), dto.getTags(),
                                                userInfo.getEmail() != null ? userInfo.getEmail() : "");
        } catch (TagsAlreadyExistsException e) {
            throw new BadRequestException(e.getMessage());
        }
    }

    @Operation(summary = "Обновление темы приложения", description = "Обновление темы приложения", tags = {"project"})
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "В случае если объект был успешно создан или обновлен (см. таблицу project)",
                    content = @Content(schema = @Schema(implementation = ProjectDto.class))),
            @ApiResponse(responseCode = "401", description = "В случае если запрос отправлен без токена или с недействительным токеном возвращается ошибка 401 и строка с ошибкой в теле запроса",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "404", description = "В случае если тип события с указанным идентификатором не существует",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "500", description = "В случае если что-то пошло не так (см. тело ответа с описанием ошибки)",
                    content = @Content(schema = @Schema(implementation = String.class))),
    })
    @PutMapping("/api/project/{id}/updatetheme")
    public ProjectDto updateTheme(
            @PathVariable(name = "id")
            @Parameter(name = "id", description = "Идентификатор конфигурации (таблица project).", example = "1") Long id,
            @io.swagger.v3.oas.annotations.parameters.RequestBody(description = "DTO представление приложения (таблица project)")
            @RequestBody ProjectThemeDto dto) throws NotFoundException, IOException {
        getSessionIdOrThrowException();
        checkPermission(projectService.find(id), new SharedGroupPermissionDto(true));
        dto.setProjectId(id);
        ProjectEntity projectEntity = projectService.updateTheme(dto);
        return buildProjectDto(projectEntity);
    }

    @Operation(summary = "Обновление данных приложения", description = "Обновление данных приложения", tags = {"project"})
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "В случае если объект был успешно создан или обновлен (см. таблицу project)",
                    content = @Content(schema = @Schema(implementation = ProjectDto.class))),
            @ApiResponse(responseCode = "401", description = "В случае если запрос отправлен без токена или с недействительным токеном возвращается ошибка 401 и строка с ошибкой в теле запроса",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "404", description = "В случае если тип события с указанным идентификатором не существует",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "500", description = "В случае если что-то пошло не так (см. тело ответа с описанием ошибки)",
                    content = @Content(schema = @Schema(implementation = String.class))),
    })
    @PutMapping("/api/project/{id}/updatedata")
    public ProjectDto updateData(
            @PathVariable(name = "id")
            @Parameter(name = "id", description = "Идентификатор конфигурации (таблица project).", example = "1") Long id,
            @io.swagger.v3.oas.annotations.parameters.RequestBody(description = "DTO представление приложения (таблица project)")
            @RequestBody ProjectDataDto dto) throws NotFoundException, IOException {
        getSessionIdOrThrowException();
        checkPermission(projectService.find(id), new SharedGroupPermissionDto(true));
        dto.setProjectId(id);
        ProjectEntity projectEntity = projectService.updateData(dto);
        return buildProjectDto(projectEntity);
    }

    @Operation(summary = "Добавить шаринг приложения для группы", description = "Добавить шаринг приложения для группы", tags = {"project"})
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "В случае если объект был успешно обновлен (см. таблицу project)",
                    content = @Content(schema = @Schema(implementation = ProjectDto.class))),
            @ApiResponse(responseCode = "401", description = "В случае если запрос отправлен без токена или с недействительным токеном возвращается ошибка 401 и строка с ошибкой в теле запроса",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "404", description = "В случае если тип события с указанным идентификатором не существует",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "500", description = "В случае если что-то пошло не так (см. тело ответа с описанием ошибки)",
                    content = @Content(schema = @Schema(implementation = String.class))),
    })
    @PutMapping("/api/project/{id}/sharegroup")
    public ProjectDto shareGroup(
            @PathVariable(name = "id")
            @Parameter(name = "id", description = "Идентификатор конфигурации (таблица project).", example = "1") Long id) throws NotFoundException {
        getSessionIdOrThrowException();
        ProjectEntity projectEntity = projectService.find(id);
        checkPermissionIsOwner(projectEntity);
        ProjectEntity updatedProjectEntity = projectService.shareGroup(id, new SharedGroupPermissionDto(true));
        return buildProjectDto(updatedProjectEntity);
    }

    @Operation(summary = "Снять шаринг приложения для группы", description = "Снять шаринг приложения для группы", tags = {"project"})
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "В случае если объект был успешно обновлен (см. таблицу project)",
                    content = @Content(schema = @Schema(implementation = ProjectDto.class))),
            @ApiResponse(responseCode = "401", description = "В случае если запрос отправлен без токена или с недействительным токеном возвращается ошибка 401 и строка с ошибкой в теле запроса",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "404", description = "В случае если тип события с указанным идентификатором не существует",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "500", description = "В случае если что-то пошло не так (см. тело ответа с описанием ошибки)",
                    content = @Content(schema = @Schema(implementation = String.class))),
    })
    @PutMapping("/api/project/{id}/unsharegroup")
    public ProjectDto unshareGroup(
            @PathVariable(name = "id")
            @Parameter(name = "id", description = "Идентификатор конфигурации (таблица project).", example = "1") Long id) throws NotFoundException {
        getSessionIdOrThrowException();
        ProjectEntity projectEntity = projectService.find(id);
        checkPermissionIsOwner(projectEntity);
        ProjectEntity updatedProjectEntity = projectService.unshareGroup(id);
        return buildProjectDto(updatedProjectEntity);
    }

    private ProjectDto buildProjectDto(ProjectEntity projectEntity) {
        ProjectDto projectDto = ProjectFactory.create(projectEntity);
        setTags(projectDto);
        setIsOwner(projectEntity.getOwnerId(), projectDto);
        return projectDto;
    }

    private void setTags(ProjectDto dto) {
        dto.setTags(projectTagService.findAllByProjectId(dto.getId()).stream().map(projectTagEntity -> projectTagEntity.getTag().getName()).collect(Collectors.toList()));
    }

    private void setIsOwner(String ownerId, ProjectDto dto) {
        String sessionId = getSessionId();
        dto.setIsOwner(ownerId.equals(sessionId));
    }

    private String getSessionIdOrThrowException() {
        String sessionId = getSessionId();
        if (sessionId == null) {
            throw new AccessDeniedException("Not authorized.");
        }
        return sessionId;
    }

    private void checkPermissionIsOwner(ProjectEntity projectEntity) {
        String sessionId = getSessionId();
        if (!projectEntity.getOwnerId().equals(sessionId)) {
            throw new AccessDeniedException(String.format("Access denied Owner \"%s\" for Project with id \"%d\"", sessionId, projectEntity.getId()));
        }
    }

    private void checkPermission(ProjectEntity projectEntity) {
        String sessionId = getSessionId();
        if (projectEntity.isSharedGroup()) {
            Set<String> groupMembers = getGroupMembers(sessionId);
            if (!groupMembers.contains(projectEntity.getOwnerId())) {
                throw new AccessDeniedException(String.format("Access denied Owner \"%s\" in shared group for Project with id \"%d\"", sessionId, projectEntity.getId()));
            }
        } else {
            checkPermissionIsOwner(projectEntity);
        }
    }

    private void checkPermission(ProjectEntity projectEntity, SharedGroupPermissionDto sharedGroupPermissionDto) {
        checkPermission(projectEntity);
        String sessionId = getSessionId();
        if (!projectEntity.getOwnerId().equals(sessionId) && projectEntity.isSharedGroup()) {
            if (sharedGroupPermissionDto.isCanSharedGroupEdit() && !projectEntity.isCanSharedGroupEdit()) {
                throw new AccessDeniedException(String.format("Access denied Owner \"%s\" for Project with id \"%d\" to edit shared group", sessionId, projectEntity.getId()));
            }
        }
    }

    private void checkPermissionFolder(FolderEntity folderEntity) {
        String sessionId = getSessionId();
        if (folderEntity.isSharedGroup()) {
            Set<String> groupMembers = getGroupMembers(sessionId);
            if (!groupMembers.contains(folderEntity.getOwnerId())) {
                throw new AccessDeniedException(String.format("Access denied Owner \"%s\" in shared group for Folder with id \"%d\"", sessionId, folderEntity.getId()));
            }
        } else {
            checkPermissionFolderIsOwner(folderEntity);
        }
    }

    private void checkPermissionFolderIsOwner(FolderEntity folderEntity) {
        String sessionId = getSessionId();
        if (!folderEntity.getOwnerId().equals(sessionId)) {
            throw new AccessDeniedException(String.format("Access denied Owner \"%s\" for Folder with id \"%d\"", sessionId, folderEntity.getId()));
        }
    }

    private Set<String> getGroupMembers(String sessionId) {
        Set<String> sessionIdList = new HashSet<>();
        List<GroupDto> groupDtos = keyCloakServiceClient.getGroupAll(sessionId);
        for (GroupDto groupDto : groupDtos) {
            for (UserDto userDto : keyCloakServiceClient.getGroupMemberAll(groupDto.getId())) {
                String userDtoId = userDto.getId();
                sessionIdList.add(userDtoId);
            }
        }
        return sessionIdList;
    }
}
