package com.goodt.drive.supeditor.application.controllers.api;

import com.goodt.drive.supeditor.GlobalDefs;
import com.goodt.drive.supeditor.application.data.OperationResult;
import com.goodt.drive.supeditor.application.dto.SharedGroupPermissionDto;
import com.goodt.drive.supeditor.application.dto.UploadFileDto;
import com.goodt.drive.supeditor.application.dto.keycloak.GroupDto;
import com.goodt.drive.supeditor.application.dto.keycloak.UserDto;
import com.goodt.drive.supeditor.application.model.entities.ProjectEntity;
import com.goodt.drive.supeditor.application.services.IProjectService;
import com.goodt.drive.supeditor.application.services.IProjectUploadService;
import com.goodt.drive.supeditor.application.services.clients.IKeyCloakServiceClient;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.media.ArraySchema;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.responses.ApiResponses;
import javassist.NotFoundException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.AccessDeniedException;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

@CrossOrigin
@RestController
public class ProjectFileController extends BaseController {

    @Autowired
    private IProjectUploadService projectUploadService;
    @Autowired
    private IProjectService projectService;
    @Autowired
    private IKeyCloakServiceClient keyCloakServiceClient;

    @Operation(summary = "Получить список файлов приложения", description = "Получить список файлов приложения", tags = {"project_file"})
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "В случае если объекты в БД были найдены (см. таблицу project) и удалось получить их DTO представление",
                    content = @Content(array = @ArraySchema(schema = @Schema(implementation = UploadFileDto.class)))),
            @ApiResponse(responseCode = "401", description = "В случае если запрос отправлен без токена или с недействительным токеном возвращается ошибка 401 и строка с ошибкой в теле запроса",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "404", description = "В случае если событие с указанным идентификатором не существует",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "500", description = "В случае если что-то пошло не так (см. тело ответа с описанием ошибки)",
                    content = @Content(schema = @Schema(implementation = String.class))),
    })
    @GetMapping("/api/project/{id}/file")
    public List<UploadFileDto> listFile(@PathVariable(name = "id", required = true)
                                        @Parameter(name = "id", description = "Идентификатор приложения.", example = "1")
                                                Long id) throws NotFoundException {
        getSessionIdOrThrowException();
        checkPermission(projectService.find(id));
        return projectUploadService.findAll(id);
    }

    @Operation(summary = "Скачать файлы приложения", description = "Скачать файлы приложения", tags = {"project_file"})
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "В случае если объекты в БД были найдены (см. таблицу project) и удалось получить их DTO представление",
                    content = @Content(schema = @Schema(implementation = ResponseEntity.class))),
            @ApiResponse(responseCode = "401", description = "В случае если запрос отправлен без токена или с недействительным токеном возвращается ошибка 401 и строка с ошибкой в теле запроса",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "404", description = "В случае если событие с указанным идентификатором не существует",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "500", description = "В случае если что-то пошло не так (см. тело ответа с описанием ошибки)",
                    content = @Content(schema = @Schema(implementation = String.class))),
    })
    @GetMapping("/api/project/{id}/file/download")
    public ResponseEntity<byte[]> downloadFiles(@PathVariable(name = "id", required = true)
                                                @Parameter(name = "id", description = "Идентификатор приложения.", example = "1")
                                                        Long id) throws NotFoundException, IOException {
        getSessionIdOrThrowException();
        checkPermission(projectService.find(id));
        ByteArrayOutputStream data = projectUploadService.buildArchive(id);
        HttpHeaders headers = new HttpHeaders();
        headers.setContentLength(data.size());
        headers.add(HttpHeaders.CONTENT_TYPE, "application/zip");
        headers.add(HttpHeaders.CONTENT_DISPOSITION, String.format("attachment; filename=%s", GlobalDefs.DOWNLOAD_FILES_ARCHIVE_NAME));
        return new ResponseEntity<>(data.toByteArray(), headers, HttpStatus.OK);
    }

    @Operation(summary = "Загрузить файл приложения", description = "Загрузить файл приложения", tags = {"project_file"})
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "В случае если объекты в БД были найдены (см. таблицу project) и удалось получить их DTO представление",
                    content = @Content(schema = @Schema(implementation = UploadFileDto.class))),
            @ApiResponse(responseCode = "401", description = "В случае если запрос отправлен без токена или с недействительным токеном возвращается ошибка 401 и строка с ошибкой в теле запроса",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "404", description = "В случае если событие с указанным идентификатором не существует",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "500", description = "В случае если что-то пошло не так (см. тело ответа с описанием ошибки)",
                    content = @Content(schema = @Schema(implementation = String.class))),
    })
    @PostMapping(value = "/api/project/{id}/file", consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
    public UploadFileDto uploadFile(
            @PathVariable(name = "id")
            @Parameter(name = "id", description = "Идентификатор приложения.", example = "1") Long id,
            @RequestPart(name = "file")
            @Parameter(description = "Файл архив конфигурации.") MultipartFile file) throws NotFoundException, IOException {
        getSessionIdOrThrowException();
        checkPermission(projectService.find(id), new SharedGroupPermissionDto(true));
        return projectUploadService.create(id, file);
    }

    @Operation(summary = "Удалить файл приложения", description = "Удалить файл приложения", tags = {"project_file"})
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "В случае если объекты в БД были найдены (см. таблицу project) и удалось получить их DTO представление",
                    content = @Content(schema = @Schema(implementation = OperationResult.class))),
            @ApiResponse(responseCode = "401", description = "В случае если запрос отправлен без токена или с недействительным токеном возвращается ошибка 401 и строка с ошибкой в теле запроса",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "404", description = "В случае если событие с указанным идентификатором не существует",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "500", description = "В случае если что-то пошло не так (см. тело ответа с описанием ошибки)",
                    content = @Content(schema = @Schema(implementation = String.class))),
    })
    @DeleteMapping("/api/project/{id}/file/{hash}")
    public OperationResult deleteFile(@PathVariable(name = "id", required = true)
                                      @Parameter(name = "id", description = "Идентификатор приложения.", example = "1")
                                              Long id,
                                      @PathVariable(name = "hash", required = true)
                                      @Parameter(name = "hash", description = "MD5 hash названия файла.", example = "1")
                                              String hash) throws NotFoundException, IOException {
        getSessionIdOrThrowException();
        checkPermission(projectService.find(id), new SharedGroupPermissionDto(true));
        boolean deleted = projectUploadService.delete(id, hash);
        return new OperationResult(deleted, "");
    }

    private String getSessionIdOrThrowException() {
        String sessionId = getSessionId();
        if (sessionId == null) {
            throw new AccessDeniedException("Not authorized.");
        }
        return sessionId;
    }

    private void checkPermission(ProjectEntity projectEntity) {
        String sessionId = getSessionId();
        if (projectEntity.isSharedGroup()) {
            Set<String> groupMembers = getGroupMembers(sessionId);
            if (!groupMembers.contains(projectEntity.getOwnerId())) {
                throw new AccessDeniedException(String.format("Access denied Owner \"%s\" in shared group for Project with id \"%d\"", sessionId, projectEntity.getId()));
            }
        } else if (!projectEntity.getOwnerId().equals(sessionId)) {
            throw new AccessDeniedException(String.format("Access denied Owner \"%s\" for Project with id \"%d\"", sessionId, projectEntity.getId()));
        }
    }

    private void checkPermission(ProjectEntity projectEntity, SharedGroupPermissionDto sharedGroupPermissionDto) {
        checkPermission(projectEntity);
        String sessionId = getSessionId();
        if (!projectEntity.getOwnerId().equals(sessionId) && projectEntity.isSharedGroup()) {
            if (sharedGroupPermissionDto.isCanSharedGroupEdit() && !projectEntity.isCanSharedGroupEdit()) {
                throw new AccessDeniedException(String.format("Access denied Owner \"%s\" for Project with id \"%d\" to edit shared group", sessionId, projectEntity.getId()));
            }
        }
    }

    private Set<String> getGroupMembers(String sessionId) {
        Set<String> sessionIdList = new HashSet<>();
        List<GroupDto> groupDtos = keyCloakServiceClient.getGroupAll(sessionId);
        for (GroupDto groupDto : groupDtos) {
            for (UserDto userDto : keyCloakServiceClient.getGroupMemberAll(groupDto.getId())) {
                String userDtoId = userDto.getId();
                sessionIdList.add(userDtoId);
            }
        }
        return sessionIdList;
    }
}
