package com.goodt.drive.supeditor.application.controllers.api;

import com.goodt.drive.supeditor.application.data.OperationResult;
import com.goodt.drive.supeditor.application.dto.ProjectPageDto;
import com.goodt.drive.supeditor.application.dto.SharedGroupPermissionDto;
import com.goodt.drive.supeditor.application.dto.keycloak.GroupDto;
import com.goodt.drive.supeditor.application.dto.keycloak.UserDto;
import com.goodt.drive.supeditor.application.factories.ProjectPageFactory;
import com.goodt.drive.supeditor.application.model.entities.ProjectContentEntity;
import com.goodt.drive.supeditor.application.model.entities.ProjectEntity;
import com.goodt.drive.supeditor.application.services.IProjectPageService;
import com.goodt.drive.supeditor.application.services.IProjectService;
import com.goodt.drive.supeditor.application.services.clients.IKeyCloakServiceClient;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.responses.ApiResponses;
import javassist.NotFoundException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.AccessDeniedException;
import org.springframework.web.bind.annotation.*;

import java.io.IOException;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

@CrossOrigin
@RestController
public class ProjectPageController extends BaseController {

    @Autowired
    private IProjectPageService projectPageService;
    @Autowired
    private IProjectService projectService;
    @Autowired
    private IKeyCloakServiceClient keyCloakServiceClient;

    @Operation(summary = "Получение страницы приложения", description = "Получение страницы приложения", tags = { "project_page" })
    @ApiResponses(value = {
                     @ApiResponse(responseCode = "200", description = "В случае если объекты в БД были найдены (см. таблицу project_page) и удалось получить их DTO представление",
                                  content = @Content(schema = @Schema(implementation = ProjectPageDto.class))),
                     @ApiResponse(responseCode = "401", description = "В случае если запрос отправлен без токена или с недействительным токеном возвращается ошибка 401 и строка с ошибкой в теле запроса",
                                  content = @Content(schema = @Schema(implementation = String.class))),
                     @ApiResponse(responseCode = "404", description = "В случае если событие с указанным идентификатором не существует",
                                  content = @Content(schema = @Schema(implementation = String.class))),
                     @ApiResponse(responseCode = "500", description = "В случае если что-то пошло не так (см. тело ответа с описанием ошибки)",
                                  content = @Content(schema = @Schema(implementation = String.class))),
    })
    @GetMapping("/api/project/{project_id}/page/{page_id}")
    public ProjectPageDto get(@PathVariable(name = "project_id", required = true)
                                  @Parameter(name = "project_id", description = "Идентификатор конфигурации (таблица project).", example = "1")
                                  Long projectId,
                              @PathVariable(name = "page_id", required = true)
                                  @Parameter(name = "page_id", description = "Идентификатор страницы.", example = "page-id-1")
                                  String pageId) throws NotFoundException {
        getSessionIdOrThrowException();
        checkPermission(projectService.find(projectId));
        ProjectContentEntity projectContentEntity = projectPageService.find(projectId, pageId);
        return ProjectPageFactory.create(projectContentEntity);
    }

    @Operation(summary = "Создание страницы приложения", description = "Создание страницы приложения", tags = { "project_page" })
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "В случае если объект был успешно создан или обновлен (см. таблицу project_page)",
                    content = @Content(schema = @Schema(implementation = ProjectPageDto.class))),
            @ApiResponse(responseCode = "401", description = "В случае если запрос отправлен без токена или с недействительным токеном возвращается ошибка 401 и строка с ошибкой в теле запроса",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "404", description = "В случае если тип события с указанным идентификатором не существует",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "500", description = "В случае если что-то пошло не так (см. тело ответа с описанием ошибки)",
                    content = @Content(schema = @Schema(implementation = String.class))),
    })
    @PostMapping("/api/project/{project_id}/page")
    public ProjectPageDto create(@PathVariable(name = "project_id", required = true)
                                     @Parameter(name = "project_id", description = "Идентификатор конфигурации (таблица project).", example = "1")
                                     Long projectId,
                                 @io.swagger.v3.oas.annotations.parameters.RequestBody(description = "DTO представление страниц (таблица project_page)")
                                     @RequestBody ProjectPageDto dto) throws NotFoundException, IOException {
        getSessionIdOrThrowException();
        checkPermission(projectService.find(projectId), new SharedGroupPermissionDto(true));
        dto.setProjectId(projectId);
        ProjectContentEntity projectContentEntity = projectPageService.create(dto);
        return ProjectPageFactory.create(projectContentEntity);
    }

    @Operation(summary = "Обновление страницы приложения", description = "Обновление страницы приложения", tags = { "project_page" })
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "В случае если объект был успешно создан или обновлен (см. таблицу project_page)",
                    content = @Content(schema = @Schema(implementation = ProjectPageDto.class))),
            @ApiResponse(responseCode = "401", description = "В случае если запрос отправлен без токена или с недействительным токеном возвращается ошибка 401 и строка с ошибкой в теле запроса",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "404", description = "В случае если тип события с указанным идентификатором не существует",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "500", description = "В случае если что-то пошло не так (см. тело ответа с описанием ошибки)",
                    content = @Content(schema = @Schema(implementation = String.class))),
    })
    @PutMapping("/api/project/{project_id}/page/{page_id}")
    public ProjectPageDto update(@PathVariable(name = "project_id", required = true)
                                     @Parameter(name = "project_id", description = "Идентификатор конфигурации (таблица project).", example = "1")
                                     Long projectId,
                                 @PathVariable(name = "page_id", required = true)
                                     @Parameter(name = "page_id", description = "Идентификатор страницы.", example = "page-id-1")
                                     String pageId,
                                 @io.swagger.v3.oas.annotations.parameters.RequestBody(description = "DTO представление страниц (таблица project_page)")
                                     @RequestBody ProjectPageDto dto) throws NotFoundException, IOException {
        getSessionIdOrThrowException();
        checkPermission(projectService.find(projectId), new SharedGroupPermissionDto(true));
        dto.setProjectId(projectId);
        dto.setPageId(pageId);
        ProjectContentEntity projectContentEntity = projectPageService.update(dto);
        return ProjectPageFactory.create(projectContentEntity);
    }

    @Operation(summary = "Удаление страницы приложения", description = "Удаление страницы приложения", tags = { "project_page" })
    @ApiResponses(value = {
            @ApiResponse(responseCode = "204", description = "В случае если объект в БД (см. таблицу project_page) был удален (поставлено date_to=NOW())"),
            @ApiResponse(responseCode = "401", description = "В случае если запрос отправлен без токена или с недействительным токеном возвращается ошибка 401 и строка с ошибкой в теле запроса",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "404", description = "В случае если тип события с указанным идентификатором не существует",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "500", description = "В случае если что-то пошло не так (см. тело ответа с описанием ошибки)",
                    content = @Content(schema = @Schema(implementation = String.class))),
    })
    @DeleteMapping("/api/project/{project_id}/page/{page_id}")
    public OperationResult delete(@PathVariable(name = "project_id", required = true)
                                      @Parameter(name = "project_id", description = "Идентификатор конфигурации (таблица project).", example = "1")
                                      Long projectId,
                                  @PathVariable(name = "page_id", required = true)
                                      @Parameter(name = "page_id", description = "Идентификатор страницы.", example = "page-id-1")
                                      String pageId) throws IOException, NotFoundException {
        getSessionIdOrThrowException();
        checkPermission(projectService.find(projectId), new SharedGroupPermissionDto(true));
        projectPageService.delete(projectId, pageId);
        return new OperationResult(true, "");
    }

    private String getSessionIdOrThrowException() {
        String sessionId = getSessionId();
        if (sessionId == null) {
            throw new AccessDeniedException("Not authorized.");
        }
        return sessionId;
    }

    private void checkPermission(ProjectEntity projectEntity) {
        String sessionId = getSessionId();
        if (projectEntity.isSharedGroup()) {
            Set<String> groupMembers = getGroupMembers(sessionId);
            if (!groupMembers.contains(projectEntity.getOwnerId())) {
                throw new AccessDeniedException(String.format("Access denied Owner \"%s\" in shared group for Project with id \"%d\"", sessionId, projectEntity.getId()));
            }
        } else if (!projectEntity.getOwnerId().equals(sessionId)) {
            throw new AccessDeniedException(String.format("Access denied Owner \"%s\" for Project with id \"%d\"", sessionId, projectEntity.getId()));
        }
    }

    private void checkPermission(ProjectEntity projectEntity, SharedGroupPermissionDto sharedGroupPermissionDto) {
        checkPermission(projectEntity);
        String sessionId = getSessionId();
        if (!projectEntity.getOwnerId().equals(sessionId) && projectEntity.isSharedGroup()) {
            if (sharedGroupPermissionDto.isCanSharedGroupEdit() && !projectEntity.isCanSharedGroupEdit()) {
                throw new AccessDeniedException(String.format("Access denied Owner \"%s\" for Project with id \"%d\" to edit shared group", sessionId, projectEntity.getId()));
            }
        }
    }

    private Set<String> getGroupMembers(String sessionId) {
        Set<String> sessionIdList = new HashSet<>();
        List<GroupDto> groupDtos = keyCloakServiceClient.getGroupAll(sessionId);
        for (GroupDto groupDto : groupDtos) {
            for (UserDto userDto : keyCloakServiceClient.getGroupMemberAll(groupDto.getId())) {
                String userDtoId = userDto.getId();
                sessionIdList.add(userDtoId);
            }
        }
        return sessionIdList;
    }
}
