package com.goodt.drive.supeditor.application.dao;


import com.goodt.drive.supeditor.application.configuration.ProjectConfig;
import com.goodt.drive.supeditor.application.model.entities.ProjectContentEntity;
import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import javax.activation.MimetypesFileTypeMap;
import java.io.File;
import java.io.IOException;
import java.net.URLConnection;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

@Repository
public class ProjectContentDao {

    @Autowired
    private ProjectConfig projectConfig;

    public Optional<ProjectContentEntity> find(Long projectId, String filepath) {
        File baseDir = getBaseDir(projectId);
        File file = new File(baseDir, filepath);
        if (!file.exists()) {
            return Optional.empty();
        }
        return Optional.of(buildEntity(projectId, file));
    }

    public Optional<ProjectContentEntity> find(Long projectId, String subfolder, String filepath) {
        return find(projectId, String.join(File.separator, StringUtils.strip(subfolder, File.separator), StringUtils.strip(filepath, File.separator)));
    }

    public boolean exists(Long projectId, String filepath) {
        File baseDir = getBaseDir(projectId);
        File file = new File(baseDir, filepath);
        return file.exists();
    }

    public boolean exists(Long projectId, String subfolder, String filepath) {
        return exists(projectId, String.join(File.separator, StringUtils.strip(subfolder, File.separator), StringUtils.strip(filepath, File.separator)));
    }

    public List<ProjectContentEntity> findAll(Long projectId) {
        File dir = getBaseDir(projectId);
        return findAllFiles(projectId, dir);
    }

    public List<ProjectContentEntity> findAll(Long projectId, String subfolder) {
        File dir = getBaseDir(projectId);
        return findAllFiles(projectId, new File(dir, subfolder));
    }

    public List<ProjectContentEntity> findAllRecursive(Long projectId) {
        File dir = getBaseDir(projectId);
        return findAllFilesRecursive(projectId, dir);
    }

    public List<ProjectContentEntity> findAllRecursive(Long projectId, String subfolder) {
        File dir = getBaseDir(projectId);
        return findAllFilesRecursive(projectId, new File(dir, subfolder));
    }

    public ProjectContentEntity save(ProjectContentEntity projectContentEntity, byte[] content) throws IOException {
        File baseDir = getBaseDir(projectContentEntity.getProjectId());
        StringJoiner filepath = new StringJoiner(File.separator);
        if (projectContentEntity.getFolder() != null) {
            filepath.add(projectContentEntity.getFolder());
        }
        filepath.add(projectContentEntity.getFilename());
        File file = new File(baseDir, filepath.toString());
        if (!file.getParentFile().exists()) {
            Files.createDirectories(file.getParentFile().toPath());
        }
        Files.write(file.toPath(), content);
        return buildEntity(projectContentEntity.getProjectId(), file);
    }

    public boolean delete(Long projectId, String filepath) throws IOException {
        File baseDir = getBaseDir(projectId);
        File file = new File(baseDir, filepath);
        return Files.deleteIfExists(file.toPath());
    }

    public boolean delete(Long projectId, String subfolder, String filepath) throws IOException {
        String stripSubfolder = StringUtils.strip(subfolder, File.separator);
        String stripFilename = StringUtils.strip(filepath, File.separator);
        String path = String.join(File.separator, stripSubfolder, stripFilename);
        return delete(projectId, path);
    }

    public boolean deleteAll(Long projectId) throws IOException {
        File baseDir = getBaseDir(projectId);
        FileUtils.deleteDirectory(baseDir);
        return true;
    }

    public boolean deleteAll(Long projectId, String subfolder) throws IOException {
        File baseDir = getBaseDir(projectId);
        FileUtils.deleteDirectory(new File(baseDir, subfolder));
        return true;
    }

    public File getBaseDir(Long projectId) {
        return new File(projectConfig.getPath(), projectId.toString());
    }

    private List<ProjectContentEntity> findAllFiles(Long projectId, File dir) {
        if (!dir.exists()) {
            return new ArrayList<>();
        }
        try (Stream<Path> walk = Files.walk(dir.toPath(), 1)) {
            return getEntities(projectId, walk);
        } catch (IOException e) {
            return new ArrayList<>();
        }
    }

    private List<ProjectContentEntity> findAllFilesRecursive(Long projectId, File dir) {
        if (!dir.exists()) {
            return new ArrayList<>();
        }
        try (Stream<Path> walk = Files.walk(dir.toPath())) {
            return getEntities(projectId, walk);
        } catch (IOException e) {
            return new ArrayList<>();
        }
    }

    private List<ProjectContentEntity> getEntities(Long projectId, Stream<Path> walk) {
        return walk
                .filter(file -> !Files.isDirectory(file))
                .map(Path::toFile)
                .map(file -> buildEntity(projectId, file))
                .collect(Collectors.toList());
    }

    private ProjectContentEntity buildEntity(Long projectId, File file) {
        ProjectContentEntity projectContentEntity = new ProjectContentEntity();
        projectContentEntity.setFile(file);
        projectContentEntity.setProjectId(projectId);
        String folder = getSubfolder(projectId, file);
        projectContentEntity.setFolder(folder);
        projectContentEntity.setFilename(file.getName());
        String contentType = URLConnection.guessContentTypeFromName(file.getName());
        if (contentType == null) {
            contentType = new MimetypesFileTypeMap().getContentType(file);
        }
        projectContentEntity.setContentType(contentType);
        projectContentEntity.setLength(file.length());
        return projectContentEntity;
    }

    private String getSubfolder(Long projectId, File file) {
        File baseDir = getBaseDir(projectId);
        String folder = file
                .getParentFile()
                .getAbsolutePath()
                .replace(baseDir.getAbsolutePath(), "");
        return StringUtils.strip(folder, File.separator);
    }
}