package com.goodt.drive.supeditor.application.services;

import javassist.NotFoundException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.transaction.Transactional;
import java.util.Date;
import java.util.List;
import java.util.Set;

import com.goodt.drive.supeditor.application.dao.FolderDao;
import com.goodt.drive.supeditor.application.dao.ProjectDao;
import com.goodt.drive.supeditor.application.dto.FolderDto;
import com.goodt.drive.supeditor.application.dto.SharedGroupPermissionDto;
import com.goodt.drive.supeditor.application.model.entities.FolderEntity;
import com.goodt.drive.supeditor.application.model.entities.ProjectEntity;
import com.goodt.drive.supeditor.application.model.entities.ProjectTypeEntity;

@Service
public class FolderServiceImpl implements IFolderService {

    @Autowired
    private FolderDao folderDao;

    @Autowired
    private ProjectDao projectDao;

    @Autowired
    private IProjectTypeService projectTypeService;

    @Override
    public FolderEntity find(Long id) throws NotFoundException {
        return folderDao.findById(id).orElseThrow(() ->
                new NotFoundException(String.format("FolderEntity with id \"%d\" not found", id)));
    }

    @Override
    public List<FolderEntity> findAll(String ownerId, Long parentId, Long projectTypeId, List<Long> idList, Set<String> ownerIdList) {
        return folderDao.findAll(ownerId, parentId, projectTypeId, idList, ownerIdList);
    }

    @Override
    public FolderEntity create(FolderDto dto, String ownerId) throws NotFoundException {
        FolderEntity folderEntity = new FolderEntity();
        ProjectTypeEntity projectTypeEntity = projectTypeService.findByName(dto.getType());
        folderEntity.setProjectType(projectTypeEntity);
        if (dto.getParentId() != null) {
            FolderEntity parentFolderEntity = find(dto.getParentId());
            folderEntity.setParent(parentFolderEntity);
        }
        folderEntity.setName(dto.getName());
        folderEntity.setOwnerId(ownerId);
        folderEntity.setDateFrom(new Date());
        folderEntity.setSharedGroup(dto.isSharedGroup());
        folderEntity.setCanSharedGroupEdit(dto.getCanSharedGroupEdit() != null ? dto.getCanSharedGroupEdit() : true);
        return folderDao.save(folderEntity);
    }

    @Override
    @Transactional
    public FolderEntity update(FolderDto dto) throws NotFoundException {
        FolderEntity folderEntity = find( dto.getId());
        ProjectTypeEntity projectTypeEntity = projectTypeService.findByName(dto.getType());
        folderEntity.setProjectType(projectTypeEntity);
        if (dto.getParentId() != null) {
            FolderEntity parentFolderEntity = find(dto.getParentId());
            folderEntity.setParent(parentFolderEntity);
        } else {
            folderEntity.setParent(null);
        }
        folderEntity.setName(dto.getName());
        folderEntity.setSharedGroup(dto.isSharedGroup());
        folderEntity.setCanSharedGroupEdit(dto.getCanSharedGroupEdit() != null ? dto.getCanSharedGroupEdit() : true);
        return folderDao.save(folderEntity);
    }

    @Override
    public FolderEntity shareGroup(Long id, SharedGroupPermissionDto sharedGroupPermissionDto) throws NotFoundException {
        FolderEntity folderEntity = find(id);
        folderEntity.setSharedGroup(true);
        folderEntity.setCanSharedGroupEdit(sharedGroupPermissionDto.isCanSharedGroupEdit());
        return folderDao.save(folderEntity);
    }

    @Override
    public FolderEntity unshareGroup(Long id) throws NotFoundException {
        FolderEntity folderEntity = find(id);
        folderEntity.setSharedGroup(false);
        return folderDao.save(folderEntity);
    }

    @Override
    @Transactional
    public void delete(Long id) throws NotFoundException {
        List<ProjectEntity> projectEntities = projectDao.findAllByFolderId(id);
        for (ProjectEntity projectEntity : projectEntities) {
            projectEntity.setFolder(null);
            projectDao.save(projectEntity);
        }
        folderDao.deleteById(id);
    }

}
