package com.goodt.drive.supeditor.application.services;

import com.google.gson.Gson;
import javassist.NotFoundException;
import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.StringUtils;
import org.eclipse.jgit.api.Git;
import org.eclipse.jgit.api.ResetCommand;
import org.eclipse.jgit.api.errors.CheckoutConflictException;
import org.eclipse.jgit.api.errors.GitAPIException;
import org.eclipse.jgit.lib.AnyObjectId;
import org.eclipse.jgit.lib.ObjectIdRef;
import org.eclipse.jgit.lib.Ref;
import org.eclipse.jgit.lib.Repository;
import org.eclipse.jgit.revwalk.RevCommit;
import org.eclipse.jgit.storage.file.FileRepositoryBuilder;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;
import java.util.zip.ZipOutputStream;

import com.goodt.drive.supeditor.GlobalDefs;
import com.goodt.drive.supeditor.application.configuration.ProjectConfig;
import com.goodt.drive.supeditor.application.dao.ProjectContentDao;
import com.goodt.drive.supeditor.application.dao.ProjectDao;
import com.goodt.drive.supeditor.application.dao.ProjectTagDao;
import com.goodt.drive.supeditor.application.dao.TagDao;
import com.goodt.drive.supeditor.application.dto.*;
import com.goodt.drive.supeditor.application.exceptions.BadRequestException;
import com.goodt.drive.supeditor.application.exceptions.git.TagsAlreadyExistsException;
import com.goodt.drive.supeditor.application.model.entities.*;

@Service
public class ProjectServiceImpl implements IProjectService {

    @Autowired
    private ProjectConfig projectConfig;

    @Autowired
    private ITagService tagService;

    @Autowired
    private IFolderService folderService;

    @Autowired
    private IProjectTypeService projectTypeService;

    @Autowired
    private ProjectContentDao projectContentDao;

    @Autowired
    private ProjectDao projectDao;

    @Autowired
    private ProjectTagDao projectTagDao;

    @Autowired
    private TagDao tagDao;

    private final Logger logger = LoggerFactory.getLogger(this.getClass());

    @Override
    public ProjectEntity find(Long id) throws NotFoundException {
        return projectDao.findById(id).orElseThrow(() ->
                new NotFoundException(String.format("ProjectEntity with id \"%d\" not found", id)));
    }

    @Override
    public ProjectEntity findBySlug(String slug) throws NotFoundException {
        return projectDao.findBySlug(slug).orElseThrow(() ->
                new NotFoundException(String.format("ProjectEntity with slug \"%s\" not found", slug)));
    }

    @Override
    public ProjectContentEntity findContent(Long projectId, String filepath) throws NotFoundException {
        return projectContentDao.find(projectId, filepath)
                .orElseThrow(() -> new NotFoundException(String.format("Project file \"%d\" \"%s\" not found", projectId, filepath)));
    }

    @Override
    @Transactional(readOnly = true)
    public List<ProjectEntity> findAll(String ownerId, Long folderId, Long projectTypeId, Set<String> ownerIdList) {
        return projectDao.findAll(ownerId, folderId, projectTypeId, ownerIdList);
    }

    @Override
    public List<ProjectEntity> findAll() {
        return projectDao.findAll();
    }

    @Override
    @Transactional
    public ProjectEntity create(ProjectDto dto, String ownerId) throws NotFoundException {
        ProjectEntity projectEntity = new ProjectEntity();
        if (dto.getFolderId() != null) {
            FolderEntity folderEntity = folderService.find(dto.getFolderId());
            projectEntity.setFolder(folderEntity);
        }
        ProjectTypeEntity projectTypeEntity = projectTypeService.findByName(dto.getType());
        projectEntity.setProjectType(projectTypeEntity);
        if (dto.getSlug() != null) {
            projectEntity.setSlug(dto.getSlug().equals("") ? null : dto.getSlug());
        } else {
            projectEntity.setSlug(null);
        }
        projectEntity.setTitle(dto.getTitle());
        projectEntity.setDescription(dto.getDescription());
        projectEntity.setOwnerId(ownerId);
        projectEntity.setDateFrom(new Date());
        projectEntity.setSharedGroup(dto.isSharedGroup());
        projectEntity.setCanSharedGroupEdit(dto.getCanSharedGroupEdit() != null ? dto.getCanSharedGroupEdit() : true);
        ProjectEntity newProjectEntity = projectDao.save(projectEntity);
        if (dto.getTags() != null) {
            updateTags(newProjectEntity, buildTags(dto.getTags()));
        }
        return newProjectEntity;
    }

    @Override
    @Transactional
    public ProjectEntity update(ProjectDto dto) throws NotFoundException {
        ProjectEntity projectEntity = find(dto.getId());
        if (dto.getFolderId() != null) {
            FolderEntity folderEntity = folderService.find(dto.getFolderId());
            projectEntity.setFolder(folderEntity);
        } else {
            projectEntity.setFolder(null);
        }
        ProjectTypeEntity projectTypeEntity = projectTypeService.findByName(dto.getType());
        projectEntity.setProjectType(projectTypeEntity);
        if (dto.getSlug() != null) {
            projectEntity.setSlug(dto.getSlug().equals("") ? null : dto.getSlug());
        } else {
            projectEntity.setSlug(null);
        }
        projectEntity.setTitle(dto.getTitle());
        projectEntity.setDescription(dto.getDescription());
        projectEntity.setSharedGroup(dto.isSharedGroup());
        projectEntity.setCanSharedGroupEdit(dto.getCanSharedGroupEdit() != null ? dto.getCanSharedGroupEdit() : true);
        ProjectEntity newProjectEntity = projectDao.save(projectEntity);
        if (dto.getTags() != null) {
            updateTags(newProjectEntity, buildTags(dto.getTags()));
        }
        return newProjectEntity;
    }

    @Override
    public ProjectCommitDto commitProject(String ownerId, Long id, String comment, List<String> tags, String email) throws NotFoundException, IOException, GitAPIException, TagsAlreadyExistsException {
        ProjectEntity projectEntity = find(id);
        File projectDir = new File(getProjectDir(projectEntity.getId()), GlobalDefs.GIT_DIRNAME);
        if (!projectDir.exists()) {
            Files.createDirectories(projectDir.toPath());
            try (Repository repository = FileRepositoryBuilder.create(projectDir)) {
                repository.create();
            }
        }
        try (Git git = Git.open(projectDir)) {
            if (tags != null) {
                validateTags(tags, git.getRepository().getAllRefsByPeeledObjectId());
            }
            git.add().addFilepattern(".").call();
            RevCommit revCommit = git.commit()
                    .setMessage(comment)
                    .setAuthor(ownerId, email)
                    .call();
            if (tags != null) {
                for (String tag : tags) {
                    git.tag().setName(tag).call();
                }
            }
            return buildProjectCommitDto(git.getRepository().getAllRefsByPeeledObjectId(), revCommit);
        }
    }

    @Override
    public Optional<ProjectCommitDto> getLastCommitInfo(String ownerId, Long id) throws NotFoundException, IOException, GitAPIException {
        ProjectEntity projectEntity = find(id);
        File projectDir = new File(getProjectDir(projectEntity.getId()), GlobalDefs.GIT_DIRNAME);
        if (!projectDir.exists()) {
            logger.debug("Git is not available for project with id \"{}}\".", id);
            return Optional.empty();
        }
        try (Git git = Git.open(projectDir)) {
            Map<AnyObjectId, Set<Ref>> refs = git.getRepository().getAllRefsByPeeledObjectId();
            Iterator<RevCommit> revCommits = git.log().call().iterator();
            if (revCommits.hasNext()) {
                RevCommit revCommit = revCommits.next();
                ProjectCommitDto projectCommitDto = buildProjectCommitDto(refs, revCommit);
                return Optional.of(projectCommitDto);
            }
            return Optional.empty();
        }
    }

    @Override
    public List<ProjectCommitDto> getCommitAll(String ownerId, Long id) throws NotFoundException, IOException, GitAPIException {
        ProjectEntity projectEntity = find(id);
        File projectDir = new File(getProjectDir(projectEntity.getId()), GlobalDefs.GIT_DIRNAME);
        List<ProjectCommitDto> projectCommitDtos = new ArrayList<>();
        if (!projectDir.exists()) {
            return projectCommitDtos;
        }
        try (Git git = Git.open(projectDir)) {
            Map<AnyObjectId, Set<Ref>> refs = git.getRepository().getAllRefsByPeeledObjectId();
            for (RevCommit revCommit : git.log().all().call()) {
                ProjectCommitDto projectCommitDto = buildProjectCommitDto(refs, revCommit);
                projectCommitDtos.add(projectCommitDto);
            }
        }
        return projectCommitDtos;
    }

    @Override
    @Transactional
    public ProjectEntity updateData(ProjectDataDto dto) throws NotFoundException, IOException {
        ProjectEntity projectEntity = find(dto.getProjectId());
        projectContentDao.save(new ProjectContentEntity(projectEntity.getId(), null, GlobalDefs.MAIN_JSON_FILENAME), dto.getData().getBytes());
        return projectEntity;
    }

    @Override
    @Transactional
    public ProjectEntity updateTheme(ProjectThemeDto dto) throws NotFoundException, IOException {
        Long projectId = dto.getProjectId();
        ProjectEntity projectEntity = find(projectId);
        if (dto.getCss() != null) {
            projectContentDao.save(new ProjectContentEntity(projectId, null, GlobalDefs.STYLE_CSS_FILENAME), dto.getCss().getBytes());
        } else {
            projectContentDao.delete(projectId, GlobalDefs.STYLE_CSS_FILENAME);
        }
        if (dto.getLess() != null) {
            projectContentDao.save(new ProjectContentEntity(projectId, null, GlobalDefs.STYLE_LESS_FILENAME), dto.getLess().getBytes());
        } else {
            projectContentDao.delete(projectId, GlobalDefs.STYLE_LESS_FILENAME);
        }
        if (dto.getTheme() != null) {
            projectContentDao.save(new ProjectContentEntity(projectId, null, GlobalDefs.STYLE_THEME_FILENAME), dto.getTheme().getBytes());
        } else {
            projectContentDao.delete(projectId, GlobalDefs.STYLE_THEME_FILENAME);
        }
        return projectEntity;
    }

    @Override
    public ProjectEntity shareGroup(Long id, SharedGroupPermissionDto sharedGroupPermissionDto) throws NotFoundException {
        ProjectEntity projectEntity = find(id);
        projectEntity.setSharedGroup(true);
        projectEntity.setCanSharedGroupEdit(sharedGroupPermissionDto.isCanSharedGroupEdit());
        return projectDao.save(projectEntity);
    }

    @Override
    public ProjectEntity unshareGroup(Long id) throws NotFoundException {
        ProjectEntity projectEntity = find(id);
        projectEntity.setSharedGroup(false);
        return projectDao.save(projectEntity);
    }

    @Override
    public ByteArrayOutputStream exportProject(Long id, String ref) throws IOException {
        if (ref != null) {
            Path tempDirectory = Files.createTempDirectory(String.format("%d_%s_%s", id, ref, UUID.randomUUID()));
            FileUtils.copyDirectory(projectContentDao.getBaseDir(id), tempDirectory.toFile());
            try (ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
                 ZipOutputStream zipOutputStream = new ZipOutputStream(outputStream);
                 Git git = Git.open(tempDirectory.toFile())) {

                git.reset().setMode(ResetCommand.ResetType.HARD).setRef(ref).call();
                try (Stream<Path> walk = Files.walk(tempDirectory)) {
                    walk
                        .filter(file -> !Files.isDirectory(file))
                        .filter(file -> !String.valueOf(file).startsWith(tempDirectory + "/.git/"))
                        .map(Path::toFile)
                        .forEach(file -> {
                            StringBuilder filepathBuilder = new StringBuilder();
                            String filedir = StringUtils.strip(file.getParentFile().getAbsolutePath().replace(String.valueOf(tempDirectory), ""), "/");
                            if (!filedir.isEmpty()) {
                                filepathBuilder.append(filedir).append("/");
                            }
                            filepathBuilder.append(file.getName());
                            String filepath = String.valueOf(filepathBuilder);
                            try {
                                zipOutputStream.putNextEntry(new ZipEntry(filepath));
                                zipOutputStream.write(Files.readAllBytes(file.toPath()));
                                zipOutputStream.closeEntry();
                                if (filepath.startsWith(GlobalDefs.UPLOAD_DIRNAME + "/")) {
                                    zipOutputStream.putNextEntry(new ZipEntry(String.format("%s/%d/%s", filedir, id, file.getName())));
                                    zipOutputStream.write(Files.readAllBytes(file.toPath()));
                                    zipOutputStream.closeEntry();
                                }
                            } catch (IOException e) {
                                logger.error("An error occurred during put entry \"{}\" to zipArchive: {}", filepath, e.getMessage());
                                throw new BadRequestException(String.format("An error occurred during put entry \"%s\" to zipArchive", filepath));
                            }
                        });
                }
                return outputStream;
            } catch (GitAPIException e) {
                logger.error("Git is not available or damaged for project with id \"{}\": {}", id, e.getMessage());
                throw new BadRequestException(String.format("Git is not available or damaged for project with id \"%d\"", id));
            } finally {
                FileUtils.deleteDirectory(tempDirectory.toFile());
            }
        } else {
            try (ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
                 ZipOutputStream zipOutputStream = new ZipOutputStream(outputStream)) {
                List<ProjectContentEntity> projectContentEntities = projectContentDao.findAllRecursive(id);
                for (ProjectContentEntity projectContentEntity : projectContentEntities) {
                    zipOutputStream.putNextEntry(new ZipEntry(projectContentEntity.getRelativeFilepath()));
                    zipOutputStream.write(projectContentEntity.getByteContent());
                    zipOutputStream.closeEntry();
                }
                List<ProjectContentEntity> projectUploadEntities = projectContentDao.findAllRecursive(id, GlobalDefs.UPLOAD_DIRNAME);
                for (ProjectContentEntity projectUploadEntity : projectUploadEntities) {
                    zipOutputStream.putNextEntry(new ZipEntry(String.format("%s/%d/%s", projectUploadEntity.getFolder(), id, projectUploadEntity.getFilename())));
                    zipOutputStream.write(projectUploadEntity.getByteContent());
                    zipOutputStream.closeEntry();
                }
                return outputStream;
            }
        }
    }

    @Override
    @Transactional
    public ProjectEntity importProject(String ownerId, ProjectDto dto, MultipartFile archive) throws NotFoundException, IOException {
        ProjectEntity newProjectEntity = create(dto, ownerId);
        Long newProjectEntityId = newProjectEntity.getId();
        Long oldProjectId = null;
        try (ZipInputStream zipInputStream = new ZipInputStream(archive.getInputStream())) {
            boolean hasNext = true;
            while (hasNext) {
                ZipEntry nextEntry = zipInputStream.getNextEntry();
                if (nextEntry == null) {
                    hasNext = false;
                } else {
                    byte[] bytes = zipInputStream.readAllBytes();
                    if (nextEntry.getName().equals(GlobalDefs.EXIM_META_FILENAME)) {
                        String json = new String(bytes, StandardCharsets.UTF_8);
                        ProjectExImMetaDto projectMeta = new Gson().fromJson(json, ProjectExImMetaDto.class);
                        oldProjectId = projectMeta.getId();
                    }
                    if (!nextEntry.isDirectory()) {
                        projectContentDao.save(new ProjectContentEntity(newProjectEntityId, null, nextEntry.getName()), bytes);
                    }
                }
            }
        }
        if (oldProjectId != null && !oldProjectId.equals(newProjectEntityId)) {
            fixUploadFolder(oldProjectId, newProjectEntityId);
        }
        replaceProjectIdByProject(newProjectEntityId);
        return newProjectEntity;
    }

    @Override
    @Transactional
    public ProjectEntity cloneProject(ProjectDto dto, String ownerId) throws NotFoundException, IOException {
        ProjectEntity newProjectEntity = create(dto, ownerId);
        Long newProjectEntityId = newProjectEntity.getId();
        File srcDir = getProjectDir(dto.getId());
        if (srcDir.isDirectory() && srcDir.exists()) {
            File destDir = getProjectDir(newProjectEntityId);
            FileUtils.copyDirectory(srcDir, destDir);
        }
        replaceProjectIdByProject(newProjectEntityId);
        return newProjectEntity;
    }

    @Override
    @Transactional
    public void delete(Long id) throws NotFoundException, IOException {
        List<ProjectTagEntity> projectTagEntities = projectTagDao.findAllByProjectId(id);
        projectTagDao.deleteAll(projectTagEntities);
        projectDao.deleteById(id);
        projectContentDao.deleteAll(id);
    }

    public String replaceProjectId(String data, Long projectId) {
        return data.replaceAll("(\"upload/)[0-9]+(/[^\"]+\")", String.format("$1%d$2", projectId));
    }

    public void replaceProjectIdByProject(Long projectId) throws IOException {
        Optional<ProjectContentEntity> mainJsonContentEntityOptional = projectContentDao.find(projectId, GlobalDefs.MAIN_JSON_FILENAME);
        if (mainJsonContentEntityOptional.isPresent()) {
            ProjectContentEntity projectContentEntity = mainJsonContentEntityOptional.get();
            projectContentDao.save(projectContentEntity, replaceProjectId(projectContentEntity.getContent(), projectId).getBytes());
        }
        List<ProjectContentEntity> pagesContentEntities = projectContentDao.findAll(projectId, GlobalDefs.PAGES_DIRNAME);
        for (ProjectContentEntity pagesContentEntity : pagesContentEntities) {
            String content = pagesContentEntity.getContent();
            projectContentDao.save(pagesContentEntity, replaceProjectId(content, projectId).getBytes());
        }
    }

    private ProjectCommitDto buildProjectCommitDto(Map<AnyObjectId, Set<Ref>> refs, RevCommit revCommit) {
        ProjectCommitDto projectCommitDto = new ProjectCommitDto();
        projectCommitDto.setHash(revCommit.getName());
        projectCommitDto.setComment(revCommit.getFullMessage());
        projectCommitDto.setDate(revCommit.getAuthorIdent().getWhen());
        if (refs.containsKey(revCommit.getId())) {
            projectCommitDto.setTags(refs.get(revCommit.getId())
                                             .stream()
                                             .filter(ref -> ref instanceof ObjectIdRef.PeeledTag)
                                             .filter(ref -> ref.getName().lastIndexOf("/") != -1)
                                             .map(ref -> ref.getName().substring(ref.getName().lastIndexOf("/") + 1))
                                             .collect(Collectors.toList()));
        }
        return projectCommitDto;
    }

    private void fixUploadFolder(Long oldProjectId, Long newProjectId) throws IOException {
        Optional<ProjectContentEntity> projectContentEntityOptional = projectContentDao.find(newProjectId, GlobalDefs.UPLOAD_DIRNAME, String.valueOf(oldProjectId));
        if (projectContentEntityOptional.isPresent()) {
            List<ProjectContentEntity> projectContentEntities = projectContentDao.findAllRecursive(newProjectId, String.format("%s/%d", GlobalDefs.UPLOAD_DIRNAME, oldProjectId));
            for (ProjectContentEntity projectContentEntity : projectContentEntities) {
                projectContentEntity.setFolder(projectContentEntity.getFolder().replace(String.format("%s/%d", GlobalDefs.UPLOAD_DIRNAME, oldProjectId), GlobalDefs.UPLOAD_DIRNAME));
                projectContentDao.save(projectContentEntity, projectContentEntity.getByteContent());
                projectContentDao.delete(newProjectId, String.format("%s/%s/%s", GlobalDefs.UPLOAD_DIRNAME, oldProjectId, projectContentEntity.getFilename()));
            }
            projectContentDao.deleteAll(newProjectId, String.format("%s/%d", GlobalDefs.UPLOAD_DIRNAME, oldProjectId));
        }
    }

    private File getProjectDir(Long id) {
        return new File(projectConfig.getPath(), String.valueOf(id));
    }

    private void validateTags(List<String> tags, Map<AnyObjectId, Set<Ref>> refs) throws TagsAlreadyExistsException {
        List<String> tagsAlreadyExists = refs.values().stream().flatMap(Collection::stream)
                .filter(ref -> ref instanceof ObjectIdRef.PeeledTag)
                .filter(ref -> ref.getName().lastIndexOf("/") != -1)
                .map(ref -> ref.getName().substring(ref.getName().lastIndexOf("/") + 1))
                .filter(tags::contains)
                .collect(Collectors.toList());
        if (!tagsAlreadyExists.isEmpty()) {
            throw new TagsAlreadyExistsException(tags);
        }
    }

    private void updateTags(ProjectEntity project, List<Long> tags) {
        List<ProjectTagEntity> projectTagEntities = projectTagDao.findAllByProjectId(project.getId());
        projectTagDao.deleteAll(projectTagEntities);
        List<ProjectTagEntity> projectTagEntityList = tagDao.findAllById(tags)
                .stream()
                .map(tagEntity -> new ProjectTagEntity(project, tagEntity))
                .collect(Collectors.toList());
        projectTagDao.saveAll(projectTagEntityList);
    }

    private List<Long> buildTags(List<String> tags) throws NotFoundException {
        List<TagEntity> tagEntities = tagService.findAllByNameList(tags);
        for (String tag : tags) {
            if (tagEntities.stream().noneMatch(tagEntity -> tagEntity.getName().equals(tag))) {
                TagEntity tagEntity = tagService.create(tag);
                tagEntities.add(tagEntity);
            }
        }
        return tagEntities.stream().map(BaseEntity::getId).collect(Collectors.toList());
    }
}
