package com.goodt.drive.supeditor.application.services;

import com.goodt.drive.supeditor.GlobalDefs;
import com.goodt.drive.supeditor.application.dto.UploadFileDto;
import com.goodt.drive.supeditor.application.model.entities.ProjectContentEntity;
import com.goodt.drive.supeditor.application.dao.ProjectContentDao;

import javassist.NotFoundException;
import org.apache.commons.codec.digest.DigestUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.List;
import java.util.stream.Collectors;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

@Service
public class ProjectUploadServiceImpl implements IProjectUploadService {

    @Override
    public List<UploadFileDto> findAll(Long projectId) {
        return projectContentDao.findAll(projectId, GlobalDefs.UPLOAD_DIRNAME)
                .stream().map(projectContentEntity -> buildUploadFileDto(projectContentEntity))
                .collect(Collectors.toList());
    }

    @Override
    public ByteArrayOutputStream buildArchive(Long projectId) throws IOException {
        try (ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
             ZipOutputStream zipOutputStream = new ZipOutputStream(outputStream)) {

            List<ProjectContentEntity> projectContentEntities = projectContentDao.findAll(projectId, GlobalDefs.UPLOAD_DIRNAME);
            for (ProjectContentEntity projectContentEntity : projectContentEntities) {
                zipOutputStream.putNextEntry(new ZipEntry(projectContentEntity.getFilename()));
                zipOutputStream.write(projectContentEntity.getByteContent());
                zipOutputStream.closeEntry();
            }
            return outputStream;
        }
    }

    @Override
    public UploadFileDto create(Long projectId, MultipartFile file) throws NotFoundException, IOException {
        String filePath = String.format("%s/%s", GlobalDefs.UPLOAD_DIRNAME, file.getOriginalFilename());
        ProjectContentEntity projectContentEntity = projectContentDao.save(new ProjectContentEntity(projectId, null, filePath), file.getBytes());
        return buildUploadFileDto(projectContentEntity);
    }

    @Override
    public boolean delete(Long projectId, String hash) throws NotFoundException, IOException {
        List<ProjectContentEntity> projectContentEntities = projectContentDao.findAll(projectId, GlobalDefs.UPLOAD_DIRNAME);
        ProjectContentEntity projectContentEntity = projectContentEntities.stream()
                .filter(entity -> getHash(entity).equals(hash))
                .findFirst()
                .orElseThrow(() -> new NotFoundException(String.format("ProjectUpload with project id \"%d\" and hash \"%s\" not found", projectId, hash)));
        return projectContentDao.delete(projectId, GlobalDefs.UPLOAD_DIRNAME, projectContentEntity.getFilename());
    }

    private UploadFileDto buildUploadFileDto(ProjectContentEntity uploadFile) {
        String hash = getHash(uploadFile);
        String url = String.format("%s/%d/%s", GlobalDefs.UPLOAD_DIRNAME, uploadFile.getProjectId(), uploadFile.getFilename());
        String type = uploadFile.getContentType();
        long length = uploadFile.getLength();
        long lastModified = uploadFile.getFile().lastModified();
        return new UploadFileDto(hash, url, uploadFile.getFilename(), type, length, lastModified);
    }

    private String getHash(ProjectContentEntity uploadFile) {
        return DigestUtils.md5Hex(uploadFile.getFilename());
    }

    @Autowired
    private ProjectContentDao projectContentDao;

    private final Logger logger = LoggerFactory.getLogger(this.getClass());

}
