package com.goodt.drive.supeditor.application.model.services;

import com.goodt.drive.supeditor.application.dao.FolderDao;
import com.goodt.drive.supeditor.application.dao.ProjectDao;
import com.goodt.drive.supeditor.application.dto.FolderDto;
import com.goodt.drive.supeditor.application.dto.SharedGroupPermissionDto;
import com.goodt.drive.supeditor.application.model.entities.FolderEntity;
import com.goodt.drive.supeditor.application.services.IFolderService;
import com.goodt.drive.supeditor.utils.FunctionalTestBase;
import javassist.NotFoundException;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.ActiveProfiles;

import java.util.*;

@SpringBootTest
@ActiveProfiles("functests")
public class TestFolderService extends FunctionalTestBase  {

    @Test
    public void testFind() throws NotFoundException {
        FolderEntity folderEntity = folderService.find(1L);
        Assertions.assertNotNull(folderEntity);
    }

    @Test
    public void testFind_NotFound() {
        Assertions.assertThrows(NotFoundException.class, () -> folderService.find(9999L));
    }

    @Test
    public void testFindAll() {
        List<FolderEntity> entities = folderService.findAll("test-owner-id", null, 1L, null, new HashSet<>());
        Assertions.assertTrue(entities.size() > 0);
        Assertions.assertTrue(entities.stream().allMatch(folderEntity -> folderEntity.getOwnerId().equals("test-owner-id")));
        Assertions.assertTrue(entities.stream().allMatch(folderEntity -> folderEntity.getParent() == null));
    }

    @Test
    public void testFindAll_ByParent() {
        List<FolderEntity> entities = folderService.findAll("test-owner-id", 1L, 1L, null, new HashSet<>());
        Assertions.assertTrue(entities.size() > 0);
        Assertions.assertTrue(entities.stream().allMatch(folderEntity -> folderEntity.getOwnerId().equals("test-owner-id")));
        Assertions.assertTrue(entities.stream().allMatch(folderEntity -> folderEntity.getParent().getId().equals(1L)));
    }

    @Test
    public void testFindAll_ByOwner() {
        List<FolderEntity> entities = folderService.findAll("test-owner-id-2", null, 1L, null, new HashSet<>());
        Assertions.assertTrue(entities.size() > 0);
        Assertions.assertTrue(entities.stream().allMatch(folderEntity -> folderEntity.getOwnerId().equals("test-owner-id-2")));
        Assertions.assertTrue(entities.stream().allMatch(folderEntity -> folderEntity.getParent() == null));
    }

    @Test
    public void testFindAll_ByOwnerList() {
        List<FolderEntity> entities = folderService.findAll(null, null, 1L, null, Set.of("test-owner-id-2"));
        Assertions.assertTrue(entities.size() > 0);
        Assertions.assertTrue(entities.stream().allMatch(folderEntity -> folderEntity.getOwnerId().equals("test-owner-id-2")));
        Assertions.assertTrue(entities.stream().allMatch(folderEntity -> folderEntity.isSharedGroup() == true));
        Assertions.assertTrue(entities.stream().allMatch(folderEntity -> folderEntity.getParent() == null));
    }

    @Test
    public void testFindAll_ByIdList() {
        List<FolderEntity> entities = folderService.findAll("test-owner-id", null, 1L, Arrays.asList(1L, 2L, 3L, 4L), new HashSet<>());
        Assertions.assertEquals(2, entities.size());
        Assertions.assertTrue(entities.stream().anyMatch(folderEntity -> folderEntity.getId().equals(1L)));
        Assertions.assertTrue(entities.stream().anyMatch(folderEntity -> folderEntity.getId().equals(2L)));
        Assertions.assertTrue(entities.stream().allMatch(folderEntity -> folderEntity.getOwnerId().equals("test-owner-id")));
    }

    @Test
    public void testFindAll_ByIdList_Empty() {
        List<FolderEntity> entities = folderService.findAll("test-owner-id", null, 1L, new ArrayList<>(), new HashSet<>());
        Assertions.assertEquals(0, entities.size());
    }

    @Test
    public void testCreate() throws NotFoundException {
        FolderDto dto = new FolderDto(null, null, "app", "new folder", null, false, false);
        FolderEntity folderEntity = folderService.create(dto, "test-owner-id");
        Assertions.assertNotNull(folderEntity);
    }

    @Test
    public void testUpdate() throws NotFoundException {
        FolderDto dto = new FolderDto(2L, 2L, "template", "new folder", null, false, false);
        FolderEntity folderEntity = folderService.update(dto);
        Assertions.assertNotNull(folderEntity);
        Assertions.assertEquals(2L, folderEntity.getParent().getId());
        Assertions.assertEquals(2L, folderEntity.getProjectType().getId());
        Assertions.assertEquals("new folder", folderEntity.getName());
    }

    @Test
    public void testUpdate_RemoveParent() throws NotFoundException {
        FolderDto dto = new FolderDto(2L, null, "template", "new folder", null, false, false);
        FolderEntity folderEntity = folderService.update(dto);
        Assertions.assertNotNull(folderEntity);
        Assertions.assertNull(folderEntity.getParent());
    }

    @Test
    public void testShareGroup() throws NotFoundException {
        long folderId = 2L;
        FolderEntity entity = folderService.shareGroup(folderId, new SharedGroupPermissionDto(true));
        Assertions.assertTrue(entity.isSharedGroup());
        Assertions.assertTrue(entity.isCanSharedGroupEdit());
    }

    @Test
    public void testUnshareGroup() throws NotFoundException {
        long folderId = 2L;
        FolderEntity entity = folderDao.findById(folderId).get();
        entity.setSharedGroup(true);
        folderDao.save(entity);
        FolderEntity updatedFolderEntity = folderService.unshareGroup(folderId);
        Assertions.assertFalse(updatedFolderEntity.isSharedGroup());
    }

    @Test
    public void testDelete() throws NotFoundException {
        folderService.delete(6L);
        Assertions.assertTrue(folderDao.findById(6L).isEmpty());
        folderService.delete(9L);
        Assertions.assertTrue(folderDao.findById(9L).isEmpty());
        Assertions.assertNull(projectDao.findById(10L).get().getFolder());
    }

    @Autowired
    private IFolderService folderService;

    @Autowired
    private FolderDao folderDao;

    @Autowired
    private ProjectDao projectDao;
}
