package com.goodt.drive.supeditor.application.model.services;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import javassist.NotFoundException;
import org.apache.commons.io.FileUtils;
import org.eclipse.jgit.api.errors.GitAPIException;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.http.MediaType;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.web.multipart.MultipartFile;

import java.io.*;
import java.nio.charset.Charset;
import java.nio.file.Files;
import java.util.*;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;
import java.util.zip.ZipOutputStream;

import com.goodt.drive.supeditor.GlobalDefs;
import com.goodt.drive.supeditor.application.configuration.ProjectConfig;
import com.goodt.drive.supeditor.application.dao.ProjectContentDao;
import com.goodt.drive.supeditor.application.dao.ProjectDao;
import com.goodt.drive.supeditor.application.dao.ProjectTagDao;
import com.goodt.drive.supeditor.application.dto.*;
import com.goodt.drive.supeditor.application.exceptions.git.TagsAlreadyExistsException;
import com.goodt.drive.supeditor.application.model.entities.ProjectContentEntity;
import com.goodt.drive.supeditor.application.model.entities.ProjectEntity;
import com.goodt.drive.supeditor.application.model.entities.ProjectTagEntity;
import com.goodt.drive.supeditor.application.services.IProjectPageService;
import com.goodt.drive.supeditor.application.services.IProjectService;
import com.goodt.drive.supeditor.utils.FunctionalTestBase;

@SpringBootTest
@ActiveProfiles("functests")
class TestProjectService extends FunctionalTestBase {

    @Autowired
    private ProjectConfig projectConfig;

    @Autowired
    private IProjectService projectService;

    @Autowired
    private ProjectDao projectDao;

    @Autowired
    private IProjectPageService projectPageService;

    @Autowired
    private ProjectTagDao projectTagDao;

    @Autowired
    private ProjectContentDao projectContentDao;

    @BeforeEach
    void setupContext() {
        try {
            projectContentDao.save(new ProjectContentEntity(1L, null, GlobalDefs.MAIN_JSON_FILENAME), "{\"json\":\"data\",\"picture\":\"upload/1/filename.jpg\"}".getBytes());
            projectContentDao.save(new ProjectContentEntity(1L, null, GlobalDefs.STYLE_CSS_FILENAME), "theme-css".getBytes());
            projectContentDao.save(new ProjectContentEntity(1L, null, GlobalDefs.STYLE_LESS_FILENAME), "theme-less".getBytes());
            projectContentDao.save(new ProjectContentEntity(1L, null, GlobalDefs.STYLE_THEME_FILENAME), "theme-data".getBytes());
            projectContentDao.save(new ProjectContentEntity(2L, null, GlobalDefs.MAIN_JSON_FILENAME), "{\"json\":\"data\"}".getBytes());
            projectContentDao.save(new ProjectContentEntity(3L, null, "test-1.txt"), "test-file-1.txt".getBytes());
            projectContentDao.save(new ProjectContentEntity(1L, GlobalDefs.PAGES_DIRNAME, "conf1-page1.json"), "{\"json-p-1\":\"data-p-1\",\"picture-p-1\":\"upload/1/filename-p-1.jpg\"}".getBytes());
            projectContentDao.save(new ProjectContentEntity(6L, null, GlobalDefs.MAIN_JSON_FILENAME), "{\"test\":\"test-1\",\"filename\":\"upload/123/file.jpg\",\"filename2\":\"upload/345/file2.jpg\"}".getBytes());
            projectContentDao.save(new ProjectContentEntity(7L, null, GlobalDefs.MAIN_JSON_FILENAME), "{\"json\":\"data\"}".getBytes());
            projectContentDao.save(new ProjectContentEntity(8L, null, GlobalDefs.MAIN_JSON_FILENAME), "{\"json\":\"data\"}".getBytes());
            projectContentDao.save(new ProjectContentEntity(1L, GlobalDefs.PAGES_DIRNAME, "conf1-page2.json"), "conf page data 2".getBytes());
            projectContentDao.save(new ProjectContentEntity(1L, GlobalDefs.PAGES_DIRNAME, "conf1-page3.json"), "conf page data 3".getBytes());
            projectContentDao.save(new ProjectContentEntity(1L, GlobalDefs.PAGES_DIRNAME, "conf1-page4.json"), "conf page data 4".getBytes());
            projectContentDao.save(new ProjectContentEntity(2L, GlobalDefs.PAGES_DIRNAME, "conf2-page1.json"), "".getBytes());
            projectContentDao.save(new ProjectContentEntity(6L, GlobalDefs.PAGES_DIRNAME, "conf6-page1.json"), "{\"test\":\"test-1\",\"filename\":\"upload/123/file.jpg\",\"filename2\":\"upload/123/file2.jpg\"}".getBytes());
            projectContentDao.save(new ProjectContentEntity(1L, GlobalDefs.UPLOAD_DIRNAME, "test.jpg"), "test-file.jpg".getBytes());
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    @AfterEach
    void downContext() throws IOException {
        FileUtils.deleteDirectory(new File(projectConfig.getPath()));
    }

    @Test
    void testFind() throws NotFoundException {
        ProjectEntity projectEntity = projectService.find(1L);
        Assertions.assertNotNull(projectEntity);
    }

    @Test
    void testFind_NotFound() {
        Assertions.assertThrows(NotFoundException.class, () -> projectService.find(9999L));
    }

    @Test
    void testFindBySlug() throws NotFoundException {
        ProjectEntity projectEntity = projectService.findBySlug("conf-1-slug");
        Assertions.assertNotNull(projectEntity);
    }

    @Test
    void testFindListAll() {
        List<ProjectEntity> entities = projectService.findAll();
        Assertions.assertEquals(13, entities.size());
    }

    @Test
    void testFindAll() {
        List<ProjectEntity> entities = projectService.findAll("test-owner-id", null, 1L, new HashSet<>());
        Assertions.assertTrue(entities.size() > 0);
        Assertions.assertTrue(entities.stream().allMatch(folderEntity -> folderEntity.getOwnerId().equals("test-owner-id")));
        Assertions.assertTrue(entities.stream().allMatch(folderEntity -> folderEntity.getFolder() == null));
        Assertions.assertEquals(6L, entities.get(0).getId());
        Assertions.assertEquals(1L, entities.get(1).getId());
    }

    @Test
    void testFindAll_ByFolder() {
        List<ProjectEntity> entities = projectService.findAll("test-owner-id", 1L, 1L, new HashSet<>());
        Assertions.assertTrue(entities.size() > 0);
        Assertions.assertTrue(entities.stream().allMatch(projectEntity -> projectEntity.getOwnerId().equals("test-owner-id")));
        Assertions.assertTrue(entities.stream().allMatch(projectEntity -> projectEntity.getFolder().getId().equals(1L)));
    }

    @Test
    void testFindAll_ByOwner() {
        List<ProjectEntity> entities = projectService.findAll("test-owner-id-2", null, 1L, new HashSet<>());
        Assertions.assertTrue(entities.size() > 0);
        Assertions.assertTrue(entities.stream().allMatch(projectEntity -> projectEntity.getOwnerId().equals("test-owner-id-2")));
        Assertions.assertTrue(entities.stream().allMatch(projectEntity -> projectEntity.getFolder() == null));
    }

    @Test
    void testFindAll_ByOwnerList() {
        List<ProjectEntity> entities = projectService.findAll(null, null, 1L, Set.of("test-owner-id-2"));
        Assertions.assertTrue(entities.size() > 0);
        Assertions.assertTrue(entities.stream().allMatch(projectEntity -> projectEntity.getOwnerId().equals("test-owner-id-2")));
        Assertions.assertTrue(entities.stream().allMatch(projectEntity -> projectEntity.isSharedGroup() == true));
        Assertions.assertTrue(entities.stream().allMatch(projectEntity -> projectEntity.getFolder() == null));
    }

    @Test
    void testCreate() throws NotFoundException {
        ProjectDto dto = new ProjectDto(null, "new-folder-test-1", "new folder test 1", "", "app", null, Arrays.asList("tag 1 name", "tag 3 name"), null, false, false);
        ProjectEntity projectEntity = projectService.create(dto, "test-owner-id");
        Assertions.assertNotNull(projectEntity);

        List<ProjectTagEntity> projectTagEntities = projectTagDao.findAllByProjectId(projectEntity.getId());
        Assertions.assertEquals(2, projectTagEntities.size());
        Assertions.assertTrue(projectTagEntities.stream().anyMatch(projectTagEntity -> projectTagEntity.getTag().getName().equals("tag 1 name")));
        Assertions.assertTrue(projectTagEntities.stream().anyMatch(projectTagEntity -> projectTagEntity.getTag().getName().equals("tag 3 name")));

    }

    @Test
    void testUpdate() throws NotFoundException {
        ProjectDto dto = new ProjectDto(2L, "upd-slug-test-1", "upd title 1", "upd descr 1", "template", 2L, Arrays.asList("tag 1 name", "tag 3 name"), null, false, false);
        ProjectEntity projectEntity = projectService.update(dto);
        Assertions.assertNotNull(projectEntity);
        Assertions.assertEquals(2L, projectEntity.getFolder().getId());
        Assertions.assertEquals("upd-slug-test-1", projectEntity.getSlug());
        Assertions.assertEquals("upd title 1", projectEntity.getTitle());
        Assertions.assertEquals("upd descr 1", projectEntity.getDescription());
        Assertions.assertEquals(2L, projectEntity.getProjectType().getId());

        List<ProjectTagEntity> projectTagEntities = projectTagDao.findAllByProjectId(projectEntity.getId());
        Assertions.assertEquals(2, projectTagEntities.size());
        Assertions.assertTrue(projectTagEntities.stream().anyMatch(projectTagEntity -> projectTagEntity.getTag().getName().equals("tag 1 name")));
        Assertions.assertTrue(projectTagEntities.stream().anyMatch(projectTagEntity -> projectTagEntity.getTag().getName().equals("tag 3 name")));
    }

    @Test
    void testUpdateTheme() throws NotFoundException, IOException {
        ProjectThemeDto dto = new ProjectThemeDto(2L, "upd css style", "upd less style", "upd theme css");
        ProjectEntity projectEntity = projectService.updateTheme(dto);
        Assertions.assertEquals(2L, projectEntity.getId());
        ProjectContentEntity styleCssEntity = projectContentDao.find(projectEntity.getId(), GlobalDefs.STYLE_CSS_FILENAME).get();
        Assertions.assertEquals("upd css style", styleCssEntity.getContent());
        ProjectContentEntity styleLessEntity = projectContentDao.find(projectEntity.getId(), GlobalDefs.STYLE_LESS_FILENAME).get();
        Assertions.assertEquals("upd less style", styleLessEntity.getContent());
        ProjectContentEntity themeCssEntity = projectContentDao.find(projectEntity.getId(), GlobalDefs.STYLE_THEME_FILENAME).get();
        Assertions.assertEquals("upd theme css", themeCssEntity.getContent());
    }

    @Test
    void testUpdateData() throws NotFoundException, IOException {
        ProjectDataDto dto = new ProjectDataDto(2L, "upd project data");
        ProjectEntity projectEntity = projectService.updateData(dto);
        Assertions.assertEquals(2L, projectEntity.getId());
    }

    @Test
    void testShareGroup() throws NotFoundException {
        long projectId = 2L;
        ProjectEntity projectEntity = projectService.shareGroup(projectId, new SharedGroupPermissionDto(true));
        Assertions.assertTrue(projectEntity.isSharedGroup());
        Assertions.assertTrue(projectEntity.isCanSharedGroupEdit());
    }

    @Test
    void testUnshareGroup() throws NotFoundException {
        long projectId = 2L;
        ProjectEntity projectEntity = projectDao.findById(projectId).get();
        projectEntity.setSharedGroup(true);
        projectDao.save(projectEntity);
        ProjectEntity updatedProjectEntity = projectService.unshareGroup(projectId);
        Assertions.assertFalse(updatedProjectEntity.isSharedGroup());
    }

    @Test
    void testUpdate_RemoveFolder() throws NotFoundException {
        ProjectDto dto = new ProjectDto(2L, "upd-slug-test-1", "upd title 1", "upd descr 1", "template", null, new ArrayList<>(), null, false, false);
        ProjectEntity projectEntity = projectService.update(dto);
        Assertions.assertNotNull(projectEntity);
        Assertions.assertNull(projectEntity.getFolder());
    }

    @Test
    void testGetCommitAll() throws NotFoundException, GitAPIException, IOException, TagsAlreadyExistsException {
        Long projectId = 2L;
        File gitFolder = new File(projectConfig.getPath(), String.format("%d/.git", projectId));
        Assertions.assertFalse(gitFolder.exists());
        projectService.commitProject("test-owner-id", projectId, "commit comment", List.of("tagName"), "");
        projectService.commitProject("test-owner-id", projectId, "commit comment 2", List.of("tagName2"), "");
        projectService.commitProject("test-owner-id", projectId, "commit comment 3", List.of("tagName3", "tagName4"), "");
        Assertions.assertTrue(gitFolder.exists());
        List<ProjectCommitDto> projectCommitDtos = projectService.getCommitAll("test-owner-id", projectId);
        Assertions.assertEquals(3, projectCommitDtos.size());
        ProjectCommitDto projectCommitDto = projectCommitDtos.get(0);
        Assertions.assertEquals("commit comment 3", projectCommitDto.getComment());
        Assertions.assertNotNull(projectCommitDto.getHash());
        Assertions.assertTrue(projectCommitDto.getTags().stream().allMatch(tagName -> tagName.equals("tagName3") || tagName.equals("tagName4")));
        Assertions.assertNotNull(projectCommitDto.getDate());
    }

    @Test
    void testGetCommitAll_GitFolderNotExists() throws NotFoundException, GitAPIException, IOException {
        Long projectId = 2L;
        File gitFolder = new File(projectConfig.getPath(), String.format("%d/.git", projectId));
        Assertions.assertFalse(gitFolder.exists());
        List<ProjectCommitDto> projectCommitDtos = projectService.getCommitAll("test-owner-id", projectId);
        Assertions.assertEquals(0, projectCommitDtos.size());
    }

    @Test
    void testCommit() throws NotFoundException, GitAPIException, IOException, TagsAlreadyExistsException {
        Long projectId = 2L;
        File gitFolder = new File(projectConfig.getPath(), String.format("%d/.git", projectId));
        Assertions.assertFalse(gitFolder.exists());
        ProjectCommitDto newProjectCommitDto = projectService.commitProject("test-owner-id", projectId, "commit comment", List.of("tagName"), "");
        Assertions.assertNotNull(newProjectCommitDto);
        Assertions.assertTrue(gitFolder.exists());
        Assertions.assertNotNull(newProjectCommitDto.getHash());
        Assertions.assertEquals("commit comment", newProjectCommitDto.getComment());
        Assertions.assertNotNull(newProjectCommitDto.getDate());
        Assertions.assertEquals("tagName", newProjectCommitDto.getTags().get(0));
    }

    @Test
    void testCommit_NoTag() throws NotFoundException, GitAPIException, IOException, TagsAlreadyExistsException {
        Long projectId = 2L;
        ProjectCommitDto projectCommitDto = projectService.commitProject("test-owner-id", projectId, "commit comment", null, "");
        Assertions.assertNotNull(projectCommitDto.getHash());
        Assertions.assertEquals(0, projectCommitDto.getTags().size());
    }

    @Test
    void testCommit_WithTwoTags() throws GitAPIException, NotFoundException, IOException, TagsAlreadyExistsException {
        Long projectId = 2L;
        File gitFolder = new File(projectConfig.getPath(), String.format("%d/.git", projectId));
        Assertions.assertFalse(gitFolder.exists());
        projectService.commitProject("test-owner-id", projectId, "commit comment", List.of("tag1", "tag2"), "");
        Optional<ProjectCommitDto> projectCommitDtoOptional = projectService.getLastCommitInfo("test-owner-id", projectId);
        Assertions.assertEquals(2, projectCommitDtoOptional.get().getTags().size());
    }

    @Test
    void testCommit_TagAlreadyExists() throws GitAPIException, NotFoundException, IOException, TagsAlreadyExistsException {
        Long projectId = 2L;
        File gitFolder = new File(projectConfig.getPath(), String.format("%d/.git", projectId));
        Assertions.assertFalse(gitFolder.exists());
        projectService.commitProject("test-owner-id", projectId, "commit comment", List.of("tag1"), "");
        Assertions.assertThrows(TagsAlreadyExistsException.class, () -> projectService.commitProject("test-owner-id", projectId, "commit comment 2", List.of("tag1", "tag2"), ""));
    }

    @Test
    void testFindContent() throws NotFoundException {
        ProjectContentEntity content = projectService.findContent(1L, GlobalDefs.MAIN_JSON_FILENAME);
        Assertions.assertEquals(GlobalDefs.MAIN_JSON_FILENAME, content.getFilename());
        Assertions.assertEquals("{\"json\":\"data\",\"picture\":\"upload/1/filename.jpg\"}", new String(content.getByteContent()));
    }

    @Test
    void testDelete() throws NotFoundException, IOException {
        Long projectId = 3L;
        File projectDir = new File(projectConfig.getPath(), projectId.toString());
        Assertions.assertTrue(projectDir.exists());
        projectService.delete(projectId);
        Assertions.assertTrue(projectDao.findById(projectId).isEmpty());
        Assertions.assertFalse(projectDir.exists());
    }

    @Test
    void testProjectExImMeatDto() {
        String json = "{\"id\":152,\"folder_id\":4,\"title\":\"title\",\"description\":\"\",\"tags\":[{}]}";
        ProjectExImMetaDto projectMeta = new Gson().fromJson(json, ProjectExImMetaDto.class);
        Assertions.assertEquals(152L, projectMeta.getId());
    }

    @Test
    void testExport() throws IOException {
        Long originalProjectId = 1L;
        try (ByteArrayOutputStream outputStream = projectService.exportProject(originalProjectId, null);
             ZipInputStream zipInputStream = new ZipInputStream(new ByteArrayInputStream(outputStream.toByteArray()))) {
            List<ZipEntry> zipEntries = new ArrayList<>();
            String testData = "";
            boolean hasNext = true;
            while (hasNext) {
                ZipEntry nextEntry = zipInputStream.getNextEntry();
                if (nextEntry == null) {
                    hasNext = false;
                } else {
                    zipEntries.add(nextEntry);
                    if (nextEntry.getName().equals(GlobalDefs.PAGES_DIRNAME + "/conf1-page1.json")) {
                        byte[] bytes = zipInputStream.readAllBytes();
                        testData = new String(bytes);
                    }
                }
            }
            Assertions.assertEquals(10, zipEntries.size());
            Assertions.assertTrue(zipEntries.stream().anyMatch(zipEntry -> zipEntry.getName().equals(GlobalDefs.MAIN_JSON_FILENAME)));
            Assertions.assertTrue(zipEntries.stream().anyMatch(zipEntry -> zipEntry.getName().equals(GlobalDefs.STYLE_CSS_FILENAME)));
            Assertions.assertTrue(zipEntries.stream().anyMatch(zipEntry -> zipEntry.getName().equals(GlobalDefs.STYLE_LESS_FILENAME)));
            Assertions.assertTrue(zipEntries.stream().anyMatch(zipEntry -> zipEntry.getName().equals(GlobalDefs.STYLE_THEME_FILENAME)));
            Assertions.assertTrue(zipEntries.stream().anyMatch(zipEntry -> zipEntry.getName().equals(GlobalDefs.PAGES_DIRNAME + "/conf1-page1.json")));
            Assertions.assertEquals("{\"json-p-1\":\"data-p-1\",\"picture-p-1\":\"upload/1/filename-p-1.jpg\"}", testData);
            Assertions.assertTrue(zipEntries.stream().anyMatch(zipEntry -> zipEntry.getName().equals(GlobalDefs.PAGES_DIRNAME + "/conf1-page2.json")));
            Assertions.assertTrue(zipEntries.stream().anyMatch(zipEntry -> zipEntry.getName().equals(GlobalDefs.PAGES_DIRNAME + "/conf1-page3.json")));
            Assertions.assertTrue(zipEntries.stream().anyMatch(zipEntry -> zipEntry.getName().equals(GlobalDefs.PAGES_DIRNAME + "/conf1-page4.json")));
            Assertions.assertTrue(zipEntries.stream().anyMatch(zipEntry -> zipEntry.getName().equals(GlobalDefs.UPLOAD_DIRNAME + "/test.jpg")));
            Assertions.assertTrue(zipEntries.stream().anyMatch(zipEntry -> zipEntry.getName().equals(GlobalDefs.UPLOAD_DIRNAME + "/1/test.jpg")));
        }
    }

    @Test
    void testExport_Ref() throws IOException, NotFoundException, GitAPIException, TagsAlreadyExistsException {
        Long originalProjectId = 13L;

        projectContentDao.save(new ProjectContentEntity(originalProjectId, null, GlobalDefs.MAIN_JSON_FILENAME), "{\"json\":\"data\",\"picture\":\"upload/1/filename.jpg\"}".getBytes());
        projectContentDao.save(new ProjectContentEntity(originalProjectId, GlobalDefs.UPLOAD_DIRNAME, "test.jpg"), "test-file.jpg".getBytes());
        projectService.commitProject("test-owner-id", originalProjectId, "init commit", new ArrayList<>(), "");

        projectContentDao.save(new ProjectContentEntity(originalProjectId, null, GlobalDefs.STYLE_CSS_FILENAME), "theme-css".getBytes());
        projectService.commitProject("test-owner-id", originalProjectId, "add style css", new ArrayList<>(), "");

        List<ProjectCommitDto> commitAll = projectService.getCommitAll("test-owner-id", originalProjectId);
        ProjectCommitDto firstCommit = commitAll.stream().filter(projectCommitDto -> projectCommitDto.getComment().equals("init commit")).findFirst().get();

        try (ByteArrayOutputStream outputStream = projectService.exportProject(originalProjectId, firstCommit.getHash());
             ZipInputStream zipInputStream = new ZipInputStream(new ByteArrayInputStream(outputStream.toByteArray()))) {
            List<ZipEntry> zipEntries = new ArrayList<>();
            boolean hasNext = true;
            while (hasNext) {
                ZipEntry nextEntry = zipInputStream.getNextEntry();
                if (nextEntry == null) {
                    hasNext = false;
                } else {
                    zipEntries.add(nextEntry);
                }
            }
            Assertions.assertTrue(zipEntries.stream().anyMatch(zipEntry -> zipEntry.getName().equals(GlobalDefs.MAIN_JSON_FILENAME)));
            Assertions.assertTrue(zipEntries.stream().anyMatch(zipEntry -> zipEntry.getName().equals(GlobalDefs.UPLOAD_DIRNAME + "/test.jpg")));
            Assertions.assertTrue(zipEntries.stream().anyMatch(zipEntry -> zipEntry.getName().equals(GlobalDefs.UPLOAD_DIRNAME + "/13/test.jpg")));
            Assertions.assertFalse(zipEntries.stream().anyMatch(zipEntry -> zipEntry.getName().equals(GlobalDefs.STYLE_CSS_FILENAME)));
            Assertions.assertTrue(zipEntries.stream().noneMatch(zipEntry -> zipEntry.getName().startsWith(".git")));
        }
    }

    @Test
    void testImport() throws NotFoundException, IOException {
        ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
        ZipOutputStream zipOutputStream = new ZipOutputStream(outputStream);

        Map<String, Object> meta = new HashMap<>();
        meta.put("id", 223344);
        meta.put("folder_id", 1);
        meta.put("title", "import title zip");
        meta.put("description", "import description zip");
        List<String> tags = new ArrayList<>();
        meta.put("tags", tags);
        zipOutputStream.putNextEntry(new ZipEntry(GlobalDefs.EXIM_META_FILENAME));
        zipOutputStream.write(new GsonBuilder().serializeNulls().create().toJson(meta).getBytes());

        zipOutputStream.putNextEntry(new ZipEntry(GlobalDefs.MAIN_JSON_FILENAME));
        zipOutputStream.write("{\"json\":\"data\",\"picture\":\"upload/1/filename.jpg\"}".getBytes());

        zipOutputStream.putNextEntry(new ZipEntry(GlobalDefs.STYLE_CSS_FILENAME));
        zipOutputStream.write("import style css".getBytes());
        zipOutputStream.putNextEntry(new ZipEntry(GlobalDefs.STYLE_LESS_FILENAME));
        zipOutputStream.write("import style less".getBytes());
        zipOutputStream.putNextEntry(new ZipEntry(GlobalDefs.STYLE_THEME_FILENAME));
        zipOutputStream.write("import style theme".getBytes());

        zipOutputStream.putNextEntry(new ZipEntry(GlobalDefs.UPLOAD_DIRNAME + "/file-txt-2.txt"));
        zipOutputStream.write("file txt 2".getBytes());
        zipOutputStream.putNextEntry(new ZipEntry(GlobalDefs.UPLOAD_DIRNAME + "/223344/file-txt-3.txt"));
        zipOutputStream.write("file txt 3".getBytes());

        zipOutputStream.putNextEntry(new ZipEntry(GlobalDefs.PAGES_DIRNAME + "/import1-page1.json"));
        zipOutputStream.write("{\"json-p-1\":\"data-p-1\",\"picture-p-1\":\"upload/1/filename-p-1.jpg\"}".getBytes());
        zipOutputStream.putNextEntry(new ZipEntry(GlobalDefs.PAGES_DIRNAME + "/import1-page2.json"));
        zipOutputStream.write("import 1 page 2".getBytes());

        zipOutputStream.close();
        outputStream.close();

        InputStream inputStream = new ByteArrayInputStream(outputStream.toByteArray());
        MultipartFile archive = new MockMultipartFile("archive.zip", "archive.zip", MediaType.APPLICATION_OCTET_STREAM_VALUE, inputStream);
        ProjectDto dto = new ProjectDto(null, "import-slug-2", "import title-2", "import description-2", "app", 1L, new ArrayList<>(), null, false, false);
        ProjectEntity importProjectEntity = projectService.importProject("test-owner-id", dto, archive);
        Assertions.assertNotNull(importProjectEntity);

        Optional<ProjectEntity> projectEntityOptional = projectDao.findById(importProjectEntity.getId());
        Assertions.assertTrue(projectEntityOptional.isPresent());
        ProjectEntity newProjectEntity = projectEntityOptional.get();
        Assertions.assertEquals(1L, newProjectEntity.getFolder().getId());
        Assertions.assertEquals("import-slug-2", newProjectEntity.getSlug());
        Assertions.assertEquals("import title-2", newProjectEntity.getTitle());
        Assertions.assertEquals("import description-2", newProjectEntity.getDescription());
        Assertions.assertEquals(1L, newProjectEntity.getProjectType().getId());
        Assertions.assertEquals("import style css", new String(projectService.findContent(newProjectEntity.getId(), GlobalDefs.STYLE_CSS_FILENAME).getByteContent()));
        Assertions.assertEquals("import style less", new String(projectService.findContent(newProjectEntity.getId(), GlobalDefs.STYLE_LESS_FILENAME).getByteContent()));
        Assertions.assertEquals("import style theme", new String(projectService.findContent(newProjectEntity.getId(), GlobalDefs.STYLE_THEME_FILENAME).getByteContent()));
        Assertions.assertEquals(String.format("{\"json\":\"data\",\"picture\":\"upload/%d/filename.jpg\"}", newProjectEntity.getId()), new String(projectService.findContent(newProjectEntity.getId(), GlobalDefs.MAIN_JSON_FILENAME).getByteContent()));

        ProjectContentEntity projectContentEntity = projectPageService.find(newProjectEntity.getId(), "import1-page1");
        Assertions.assertEquals(String.format("{\"json-p-1\":\"data-p-1\",\"picture-p-1\":\"upload/%d/filename-p-1.jpg\"}", newProjectEntity.getId()), new String(projectContentEntity.getByteContent()));
        Assertions.assertEquals(2, projectPageService.findAll(newProjectEntity.getId()).size());

        File fileTxt2Txt = new File(String.format("%s/%d/%s/file-txt-2.txt", projectConfig.getPath(), newProjectEntity.getId(), GlobalDefs.UPLOAD_DIRNAME));
        Assertions.assertTrue(fileTxt2Txt.exists());
        Assertions.assertTrue(fileTxt2Txt.length() > 0);
        File fileTxt3Txt = new File(String.format("%s/%d/%s/file-txt-3.txt", projectConfig.getPath(), newProjectEntity.getId(), GlobalDefs.UPLOAD_DIRNAME));
        Assertions.assertTrue(fileTxt3Txt.exists());
        Assertions.assertTrue(fileTxt3Txt.length() > 0);
    }

    @Test
    void testExportImportZip() throws NotFoundException, IOException {
        ByteArrayOutputStream outputStream = projectService.exportProject(1L, null);
        InputStream inputStream = new ByteArrayInputStream(outputStream.toByteArray());
        MultipartFile archive = new MockMultipartFile("archive.zip", "archive.zip", MediaType.APPLICATION_OCTET_STREAM_VALUE, inputStream);
        ProjectDto dto = new ProjectDto(null, "import-slug", "import title", "import description", "app", 1L, new ArrayList<>(), null, false, false);
        ProjectEntity importProjectEntity = projectService.importProject("test-owner-id", dto, archive);
        Assertions.assertNotNull(importProjectEntity);

        Optional<ProjectEntity> projectEntityOptional = projectDao.findById(importProjectEntity.getId());
        Assertions.assertTrue(projectEntityOptional.isPresent());
        ProjectEntity newProjectEntity = projectEntityOptional.get();
        Assertions.assertEquals(1L, newProjectEntity.getFolder().getId());
        Assertions.assertEquals("import-slug", newProjectEntity.getSlug());
        Assertions.assertEquals("import title", newProjectEntity.getTitle());
        Assertions.assertEquals("import description", newProjectEntity.getDescription());
        Assertions.assertEquals(1L, newProjectEntity.getProjectType().getId());
        Assertions.assertEquals("theme-css", new String(projectService.findContent(newProjectEntity.getId(), GlobalDefs.STYLE_CSS_FILENAME).getByteContent()));
        Assertions.assertEquals("theme-less", new String(projectService.findContent(newProjectEntity.getId(), GlobalDefs.STYLE_LESS_FILENAME).getByteContent()));
        Assertions.assertEquals("theme-data", new String(projectService.findContent(newProjectEntity.getId(), GlobalDefs.STYLE_THEME_FILENAME).getByteContent()));
        Assertions.assertEquals(String.format("{\"json\":\"data\",\"picture\":\"upload/%d/filename.jpg\"}", newProjectEntity.getId()), new String(projectService.findContent(newProjectEntity.getId(), GlobalDefs.MAIN_JSON_FILENAME).getByteContent()));

        ProjectContentEntity projectContentEntity = projectPageService.find(newProjectEntity.getId(), "conf1-page1");
        Assertions.assertEquals(String.format("{\"json-p-1\":\"data-p-1\",\"picture-p-1\":\"upload/%d/filename-p-1.jpg\"}", newProjectEntity.getId()), new String(projectContentEntity.getByteContent()));
    }

    @Test
    void testClone() throws NotFoundException, IOException {
        Long originalProjectId = 1L;

        File originalUploadDir = new File(projectConfig.getPath(), String.format("%d/%s", originalProjectId, GlobalDefs.UPLOAD_DIRNAME));
        Files.createDirectories(originalUploadDir.toPath());
        try (FileOutputStream fileOutputStream = new FileOutputStream(String.format("%s/%s", originalUploadDir, "test.jpg"))) {
            fileOutputStream.write("test-file.jpg".getBytes());
        }

        ProjectDto dto = new ProjectDto(originalProjectId, "cloned-1-slug-1", "cloned 1 title 1", "cloned 1 description 1", "app", 1L, new ArrayList<>(), null, false, false);
        ProjectEntity newProjectEntity = projectService.cloneProject(dto, "test-owner-id");
        Assertions.assertNotNull(newProjectEntity);
        Assertions.assertEquals(1L, newProjectEntity.getFolder().getId());
        Assertions.assertEquals("cloned-1-slug-1", newProjectEntity.getSlug());
        Assertions.assertEquals("cloned 1 title 1", newProjectEntity.getTitle());
        Assertions.assertEquals("cloned 1 description 1", newProjectEntity.getDescription());
        Assertions.assertEquals(String.format("{\"json\":\"data\",\"picture\":\"upload/%d/filename.jpg\"}", newProjectEntity.getId()), new String(projectService.findContent(newProjectEntity.getId(), GlobalDefs.MAIN_JSON_FILENAME).getByteContent()));
        Assertions.assertEquals("theme-css", new String(projectService.findContent(newProjectEntity.getId(), GlobalDefs.STYLE_CSS_FILENAME).getByteContent()));
        Assertions.assertEquals("theme-less", new String(projectService.findContent(newProjectEntity.getId(), GlobalDefs.STYLE_LESS_FILENAME).getByteContent()));
        Assertions.assertEquals("theme-data", new String(projectService.findContent(newProjectEntity.getId(), GlobalDefs.STYLE_THEME_FILENAME).getByteContent()));

        List<ProjectContentEntity> entities = projectPageService.findAll(newProjectEntity.getId());
        Assertions.assertEquals(4, entities.size());
        Assertions.assertTrue(entities
                .stream()
                .anyMatch(
                        projectContentEntity -> new String(projectContentEntity.getByteContent()).equals(String.format("{\"json-p-1\":\"data-p-1\",\"picture-p-1\":\"upload/%d/filename-p-1.jpg\"}", newProjectEntity.getId()))
                )
        );

        File uploadedFile = new File(String.format("%s/%d/%s/%s", projectConfig.getPath(), newProjectEntity.getId(), GlobalDefs.UPLOAD_DIRNAME, "test.jpg"));
        Assertions.assertTrue(uploadedFile.exists());
        Assertions.assertTrue(uploadedFile.length() > 0);
    }

    @Test
    void testReplaceProject() {
        String replaceProjectId = projectService.replaceProjectId("{\"test\":\"test-1\",\"filename\":\"upload/123/file.jpg\",\"filename2\":\"upload/345/file2.jpg\"}", 456L);
        Assertions.assertEquals("{\"test\":\"test-1\",\"filename\":\"upload/456/file.jpg\",\"filename2\":\"upload/456/file2.jpg\"}", replaceProjectId);
    }

    @Test
    void testReplaceProjectByProject() throws NotFoundException, IOException {
        Long projectId = 6L;
        File projectDir = new File(projectConfig.getPath(), projectId.toString());
        File mainJsonFile = new File(projectDir, GlobalDefs.MAIN_JSON_FILENAME);
        File pageFile = new File(projectDir, GlobalDefs.PAGES_DIRNAME + "/conf6-page1.json");
        projectService.replaceProjectIdByProject(projectId);
        Assertions.assertEquals("{\"test\":\"test-1\",\"filename\":\"upload/6/file.jpg\",\"filename2\":\"upload/6/file2.jpg\"}", FileUtils.readFileToString(mainJsonFile, Charset.defaultCharset()));
        Assertions.assertEquals("{\"test\":\"test-1\",\"filename\":\"upload/6/file.jpg\",\"filename2\":\"upload/6/file2.jpg\"}", FileUtils.readFileToString(pageFile, Charset.defaultCharset()));
    }
}
