package com.goodt.drive.supeditor.application.model.services;

import com.goodt.drive.supeditor.GlobalDefs;
import com.goodt.drive.supeditor.application.configuration.ProjectConfig;
import com.goodt.drive.supeditor.application.model.entities.ProjectContentEntity;
import com.goodt.drive.supeditor.application.dto.UploadFileDto;
import com.goodt.drive.supeditor.application.dao.ProjectContentDao;
import com.goodt.drive.supeditor.application.services.IProjectUploadService;
import com.goodt.drive.supeditor.utils.FunctionalTestBase;
import javassist.NotFoundException;
import org.apache.commons.io.FileUtils;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.http.MediaType;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.web.multipart.MultipartFile;

import java.io.*;
import java.util.ArrayList;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;

@SpringBootTest
@ActiveProfiles("functests")
public class TestProjectUploadService extends FunctionalTestBase  {

    @BeforeEach
    public void setupContext() {
        try {
            projectContentDao.save(new ProjectContentEntity(1L, GlobalDefs.UPLOAD_DIRNAME, "test-1.txt"), "test-file-1.txt".getBytes());
            projectContentDao.save(new ProjectContentEntity(1L, GlobalDefs.UPLOAD_DIRNAME, "test-2.txt"), "test-file-2.txt".getBytes());
            projectContentDao.save(new ProjectContentEntity(1L, GlobalDefs.UPLOAD_DIRNAME, "child/test-3.txt"), "test-file-3.txt".getBytes());
            projectContentDao.save(new ProjectContentEntity(1L, GlobalDefs.UPLOAD_DIRNAME, "test.jpg"), "test-file.jpg".getBytes());
            projectContentDao.save(new ProjectContentEntity(1L, GlobalDefs.UPLOAD_DIRNAME, "test-3.txt"), "test-file-3.txt".getBytes());
            projectContentDao.save(new ProjectContentEntity(1L, GlobalDefs.UPLOAD_DIRNAME, "test-4.txt"), "test-file-4.txt".getBytes());
            projectContentDao.save(new ProjectContentEntity(1L, GlobalDefs.UPLOAD_DIRNAME, "delete-test.jpg"), "delete-test.jpg".getBytes());
            projectContentDao.save(new ProjectContentEntity(1L, GlobalDefs.UPLOAD_DIRNAME, "test-11.svg"), "".getBytes());
            projectContentDao.save(new ProjectContentEntity(4L, GlobalDefs.UPLOAD_DIRNAME, "test-4-project.txt"), "test".getBytes());
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    @AfterEach
    public void downContext() throws IOException {
        FileUtils.deleteDirectory(new File(projectConfig.getPath()));
    }

    @Test
    public void testListFile() throws NotFoundException {
        Long projectId = 1L;
        List<UploadFileDto> uploadFileDtos = projectUploadService.findAll(projectId);
        Assertions.assertTrue(uploadFileDtos.size() > 0);
        Assertions.assertTrue(uploadFileDtos.stream().noneMatch(dto -> dto.getName().equals("test-4-project.txt")));
        UploadFileDto uploadSvgDto = uploadFileDtos.stream().filter(dto -> dto.getName().equals("test-11.svg")).findFirst().get();
        Assertions.assertEquals("image/svg+xml", uploadSvgDto.getType());
        UploadFileDto uploadFileDto = uploadFileDtos.stream().filter(dto -> dto.getName().equals("test-1.txt")).findFirst().get();
        Assertions.assertEquals("3a037826842d8d64fd41c067d14f02da", uploadFileDto.getHash());
        Assertions.assertEquals("upload/1/test-1.txt", uploadFileDto.getUrl());
        Assertions.assertEquals("test-1.txt", uploadFileDto.getName());
        Assertions.assertEquals("text/plain", uploadFileDto.getType());
        Assertions.assertTrue(uploadFileDto.getLastModified() > 0);
        Assertions.assertTrue(uploadFileDto.getSize() > 0);
    }

    @Test
    public void testBuildUploadArchive() throws NotFoundException, IOException {
        Long projectId = 1L;
        ByteArrayOutputStream outputStream = projectUploadService.buildArchive(projectId);
        Assertions.assertNotNull(outputStream);
        ZipInputStream zipInputStream = new ZipInputStream(new ByteArrayInputStream(outputStream.toByteArray()));
        List<ZipEntry> zipEntries = new ArrayList<>();
        boolean hasNext = true;
        while (hasNext) {
            ZipEntry nextEntry = zipInputStream.getNextEntry();
            if (nextEntry == null) {
                hasNext = false;
            } else {
                zipEntries.add(nextEntry);
            }
        }
        Assertions.assertEquals(7, zipEntries.size());
        Assertions.assertTrue(zipEntries.stream().anyMatch(zipEntry -> zipEntry.getName().equals("test.jpg")));
        Assertions.assertTrue(zipEntries.stream().anyMatch(zipEntry -> zipEntry.getName().equals("test-3.txt")));
        Assertions.assertTrue(zipEntries.stream().anyMatch(zipEntry -> zipEntry.getName().equals("test-4.txt")));
        Assertions.assertTrue(zipEntries.stream().noneMatch(zipEntry -> zipEntry.getName().equals("child/test-3.txt")));
    }

    @Test
    public void testDeleteFile() throws IOException, NotFoundException {
        Long projectId = 1L;
        Assertions.assertTrue(projectContentDao.exists(projectId, GlobalDefs.UPLOAD_DIRNAME, "delete-test.jpg"));
        boolean deleted = projectUploadService.delete(projectId, "41f0d0bc50266499d7752da992f3ca91");
        Assertions.assertTrue(deleted);
        Assertions.assertFalse(projectContentDao.exists(projectId, GlobalDefs.UPLOAD_DIRNAME, "delete-test.jpg"));
    }

    @Test
    public void testUploadFile() throws NotFoundException, IOException {
        Long projectId = 1L;
        InputStream inputStream = new ByteArrayInputStream("text file data".getBytes());
        MultipartFile file = new MockMultipartFile("text-file.txt", "text-file.txt", MediaType.APPLICATION_OCTET_STREAM_VALUE, inputStream);
        UploadFileDto uploadFileDto = projectUploadService.create(projectId, file);
        Assertions.assertEquals("df0169362237d8783ea50e9129019fe7", uploadFileDto.getHash());
        Assertions.assertEquals("upload/1/text-file.txt", uploadFileDto.getUrl());
        Assertions.assertEquals("text-file.txt", uploadFileDto.getName());
        Assertions.assertEquals("text/plain", uploadFileDto.getType());
        Assertions.assertTrue(uploadFileDto.getSize() > 0);
    }

    @Autowired
    private ProjectConfig projectConfig;

    @Autowired
    private IProjectUploadService projectUploadService;
    
    @Autowired
    private ProjectContentDao projectContentDao;
}
