
package com.goodt.drive.supeditor.application.rest;

import org.junit.Assert;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestInstance;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.web.client.TestRestTemplate;
import org.springframework.boot.web.server.LocalServerPort;
import org.springframework.http.*;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.web.reactive.function.BodyInserters;
import org.springframework.web.reactive.function.client.WebClient;

import java.util.List;
import java.util.stream.Collectors;

import com.goodt.drive.supeditor.application.configuration.AppConfig;
import com.goodt.drive.supeditor.application.dao.DataConnectorDao;
import com.goodt.drive.supeditor.application.data.OperationResult;
import com.goodt.drive.supeditor.application.dto.DataConnectorDto;
import com.goodt.drive.supeditor.application.dto.TokenResponseDto;
import com.goodt.drive.supeditor.application.dto.UserDto;
import com.goodt.drive.supeditor.application.model.entities.DataConnectorEntity;
import com.goodt.drive.supeditor.utils.IntegrationalTestBase;

@SpringBootTest(webEnvironment = SpringBootTest.WebEnvironment.RANDOM_PORT)
@ActiveProfiles("integrationaltests")
@TestInstance(TestInstance.Lifecycle.PER_CLASS)
public class TestDataConnectorController extends IntegrationalTestBase {

    @LocalServerPort
    private int port;

    @Autowired
    private TestRestTemplate restTemplate;

    private String hostName = "http://localhost:" + port;
    private HttpHeaders requestHeaders;

    @BeforeAll
    public void setup() {
        WebClient client = WebClient.builder()
                .baseUrl(appConfig.getKeyCloakConfig().getBaseUrl())
                .build();
        TokenResponseDto result = client.post().uri(appConfig.getKeyCloakConfig().getTokenUrl())
                .header(HttpHeaders.CONTENT_TYPE, MediaType.APPLICATION_FORM_URLENCODED_VALUE)
                .body(BodyInserters.fromFormData("client_id", appConfig.getKeyCloakConfig().getClientId())
                        .with("client_secret", appConfig.getKeyCloakConfig().getClientSecret())
                        .with("grant_type", "password")
.with("username", appConfig.getKeyCloakConfig().getUsername())
                        .with("password", appConfig.getKeyCloakConfig().getPassword()))
                .retrieve().bodyToMono(TokenResponseDto.class).block();

        hostName = "http://localhost:" + port;
        requestHeaders = new HttpHeaders();
        requestHeaders.add("authorization", String.format("%s %s", result.getTokenType(), result.getAccessToken()));
    }

    @BeforeEach
    public void initUser() {
        HttpEntity request = new HttpEntity<>(requestHeaders);
        ResponseEntity<UserDto> response = restTemplate.exchange(
                hostName + "/api/user/info",
                HttpMethod.GET,
                request,
                UserDto.class
        );
        List<DataConnectorEntity> dataConnectorEntities = dataConnectorDao.findAll();
        dataConnectorDao.saveAll(dataConnectorEntities.stream()
                .filter(dataConnectorEntity -> dataConnectorEntity.getOwnerId() != null && dataConnectorEntity.getOwnerId().equals("test-owner-id"))
                .map(dataConnectorEntity -> {
                    dataConnectorEntity.setOwnerId(response.getBody().getId());
                    return dataConnectorEntity;
                })
                .collect(Collectors.toList()));
    }

    @Test
    public void testGetById() {
        HttpEntity request = new HttpEntity<>(requestHeaders);

        ResponseEntity<DataConnectorDto> response = restTemplate.exchange(
                hostName + "/api/dataconnector/{id}",
                HttpMethod.GET,
                request,
                DataConnectorDto.class,
                1L
        );
        Assert.assertEquals(HttpStatus.OK, response.getStatusCode());
        Assert.assertEquals(1L, response.getBody().getId().longValue());
    }

    @Test
    public void testGetAll() {
        HttpEntity request = new HttpEntity<>(requestHeaders);

        ResponseEntity<DataConnectorDto[]> response = restTemplate.exchange(
                hostName + "/api/dataconnector",
                HttpMethod.GET,
                request,
                DataConnectorDto[].class
        );
        Assert.assertEquals(HttpStatus.OK, response.getStatusCode());
        List<DataConnectorDto> dtos = List.of(response.getBody());
        Assert.assertEquals(3, dtos.size());
        Assert.assertTrue(dtos.stream().allMatch(dto -> dto.getIsOwner() != null));
    }

    @Test
    public void testGetAll_Closed() {
        HttpEntity request = new HttpEntity<>(requestHeaders);

        ResponseEntity<DataConnectorDto[]> response = restTemplate.exchange(
                hostName + "/api/dataconnector?only_closed=true",
                HttpMethod.GET,
                request,
                DataConnectorDto[].class
        );
        Assert.assertEquals(HttpStatus.OK, response.getStatusCode());
        List<DataConnectorDto> dtos = List.of(response.getBody());
        Assert.assertEquals(2, dtos.size());
        Assert.assertTrue(dtos.stream().allMatch(dto -> dto.getIsOwner() != null));
    }

    @Test
    public void testCreate() {
        DataConnectorDto dto = new DataConnectorDto(null, "new name", "", "", "", null);
        HttpEntity request = new HttpEntity<>(dto, requestHeaders);

        ResponseEntity<DataConnectorDto> response = restTemplate.exchange(
                hostName + "/api/dataconnector",
                HttpMethod.POST,
                request,
                DataConnectorDto.class
        );
        Assert.assertEquals(HttpStatus.OK, response.getStatusCode());
        Assert.assertNotNull(response.getBody().getId());
    }

    @Test
    public void testUpdate() {
        DataConnectorDto dto = new DataConnectorDto(null, "new name", "", "", "", null);
        HttpEntity request = new HttpEntity<>(dto, requestHeaders);

        ResponseEntity<DataConnectorDto> response = restTemplate.exchange(
                hostName + "/api/dataconnector/{id}",
                HttpMethod.PUT,
                request,
                DataConnectorDto.class,
                2L
        );
        Assert.assertEquals(HttpStatus.OK, response.getStatusCode());
        Assert.assertNotNull(response.getBody().getId());
    }

    @Test
    public void testDelete() {
        HttpEntity request = new HttpEntity<>(requestHeaders);

        ResponseEntity<OperationResult> response = restTemplate.exchange(
                hostName + "/api/dataconnector/{id}",
                HttpMethod.DELETE,
                request,
                OperationResult.class,
                3L
        );
        Assert.assertEquals(HttpStatus.OK, response.getStatusCode());
        Assert.assertEquals(true, response.getBody().getResult());
    }

    @Test
    public void testRestore() {
        HttpEntity request = new HttpEntity<>(requestHeaders);

        ResponseEntity<OperationResult> response = restTemplate.exchange(
                hostName + "/api/dataconnector/{id}/restore",
                HttpMethod.POST,
                request,
                OperationResult.class,
                5L
        );
        Assert.assertEquals(HttpStatus.OK, response.getStatusCode());
        Assert.assertEquals(true, response.getBody().getResult());
    }

    @Autowired
    DataConnectorDao dataConnectorDao;

    @Autowired
    AppConfig appConfig;
}
