package com.goodt.drive.supeditor.application.rest;

import org.junit.Assert;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestInstance;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.boot.test.web.client.TestRestTemplate;
import org.springframework.boot.web.server.LocalServerPort;
import org.springframework.http.*;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.web.reactive.function.BodyInserters;
import org.springframework.web.reactive.function.client.WebClient;

import java.util.List;
import java.util.stream.Collectors;

import com.goodt.drive.supeditor.application.configuration.AppConfig;
import com.goodt.drive.supeditor.application.dao.FolderDao;
import com.goodt.drive.supeditor.application.dao.ProjectDao;
import com.goodt.drive.supeditor.application.data.OperationResult;
import com.goodt.drive.supeditor.application.dto.*;
import com.goodt.drive.supeditor.application.dto.keycloak.GroupDto;
import com.goodt.drive.supeditor.application.model.entities.FolderEntity;
import com.goodt.drive.supeditor.application.model.entities.ProjectEntity;
import com.goodt.drive.supeditor.application.services.clients.IKeyCloakServiceClient;
import com.goodt.drive.supeditor.utils.IntegrationalTestBase;

import static org.mockito.ArgumentMatchers.any;

@SpringBootTest(webEnvironment = SpringBootTest.WebEnvironment.RANDOM_PORT)
@ActiveProfiles("integrationaltests")
@TestInstance(TestInstance.Lifecycle.PER_CLASS)
public class TestFolderController extends IntegrationalTestBase {

    @LocalServerPort
    private int port;

    @Autowired
    private TestRestTemplate restTemplate;

    private String hostName = "http://localhost:" + port;
    private HttpHeaders requestHeaders;

    @Autowired
    private ProjectDao projectDao;

    @Autowired
    private FolderDao folderDao;

    @Autowired
    private AppConfig appConfig;

    @MockBean
    private IKeyCloakServiceClient keyCloakServiceClient;

    @BeforeAll
    public void setup() {
        WebClient client = WebClient.builder()
                .baseUrl(appConfig.getKeyCloakConfig().getBaseUrl())
                .build();
        TokenResponseDto result = client.post().uri(appConfig.getKeyCloakConfig().getTokenUrl())
                .header(HttpHeaders.CONTENT_TYPE, MediaType.APPLICATION_FORM_URLENCODED_VALUE)
                .body(BodyInserters.fromFormData("client_id", appConfig.getKeyCloakConfig().getClientId())
                        .with("client_secret", appConfig.getKeyCloakConfig().getClientSecret())
                        .with("grant_type", "password")
.with("username", appConfig.getKeyCloakConfig().getUsername())
                        .with("password", appConfig.getKeyCloakConfig().getPassword()))
                .retrieve().bodyToMono(TokenResponseDto.class).block();

        hostName = "http://localhost:" + port;
        requestHeaders = new HttpHeaders();
        requestHeaders.add("authorization", String.format("%s %s", result.getTokenType(), result.getAccessToken()));
    }

    @BeforeEach
    public void initUser() {
        HttpEntity request = new HttpEntity<>(requestHeaders);
        ResponseEntity<UserDto> response = restTemplate.exchange(
                hostName + "/api/user/info",
                HttpMethod.GET,
                request,
                UserDto.class
        );
        List<FolderEntity> folderEntities = folderDao.findAll();
        folderDao.saveAll(folderEntities.stream()
                .filter(folderEntity -> folderEntity.getOwnerId() != null && folderEntity.getOwnerId().equals("test-owner-id"))
                .map(folderEntity -> {
                    folderEntity.setOwnerId(response.getBody().getId());
                    return folderEntity;
                })
                .collect(Collectors.toList()));
        List<ProjectEntity> projectEntities = projectDao.findAll();
        projectDao.saveAll(projectEntities.stream()
                .filter(projectEntity -> projectEntity.getOwnerId() != null && projectEntity.getOwnerId().equals("test-owner-id"))
                .map(projectEntity -> {
                    projectEntity.setOwnerId(response.getBody().getId());
                    return projectEntity;
                })
                .collect(Collectors.toList()));
    }

    @Test
    public void testGetAll() {
        HttpEntity request = new HttpEntity<>(requestHeaders);

        ResponseEntity<FolderDto[]> response = restTemplate.exchange(
                hostName + "/api/folder?type={type}",
                HttpMethod.GET,
                request,
                FolderDto[].class,
                "app"
        );
        Assert.assertEquals(HttpStatus.OK, response.getStatusCode());
        List<FolderDto> dtos = List.of(response.getBody());
        Assert.assertEquals(2, dtos.size());
        Assert.assertTrue(dtos.stream().allMatch(dto -> dto.getParentId() == null));
        Assert.assertTrue(dtos.stream().allMatch(dto -> dto.getIsOwner() != null));
    }

    @Test
    public void testGetAll_ByIdList() {
        HttpEntity request = new HttpEntity<>(requestHeaders);

        ResponseEntity<FolderDto[]> response = restTemplate.exchange(
                hostName + "/api/folder?type={type}&id={id1},{id2},{id3},{id4}",
                HttpMethod.GET,
                request,
                FolderDto[].class,
                "app",
                1L,
                2L,
                3L,
                4L
        );
        Assert.assertEquals(HttpStatus.OK, response.getStatusCode());
        List<FolderDto> dtos = List.of(response.getBody());
        Assert.assertEquals(2, dtos.size());
        Assert.assertTrue(dtos.stream().anyMatch(dto -> dto.getId().equals(1L)));
        Assert.assertTrue(dtos.stream().anyMatch(dto -> dto.getId().equals(2L)));
        Assert.assertTrue(dtos.stream().noneMatch(dto -> dto.getId().equals(4L)));
    }

    @Test
    public void testGetAll_WithParent() {
        HttpEntity request = new HttpEntity<>(requestHeaders);

        ResponseEntity<FolderDto[]> response = restTemplate.exchange(
                hostName + "/api/folder?type={type}&parent_id={parent_id}",
                HttpMethod.GET,
                request,
                FolderDto[].class,
                "app",
                1L
        );
        Assert.assertEquals(HttpStatus.OK, response.getStatusCode());
        List<FolderDto> dtos = List.of(response.getBody());
        Assert.assertEquals(2, dtos.size());
        Assert.assertTrue(dtos.stream().anyMatch(dto -> dto.getId().equals(2L)));
        Assert.assertTrue(dtos.stream().anyMatch(dto -> dto.getId().equals(7L)));
    }

    @Test
    public void testCreate() {
        FolderDto dto = new FolderDto(null, null, "app", "new folder created name", null, false, false);
        HttpEntity request = new HttpEntity<>(dto, requestHeaders);

        ResponseEntity<FolderDto> response = restTemplate.exchange(
                hostName + "/api/folder",
                HttpMethod.POST,
                request,
                FolderDto.class
        );
        Assert.assertEquals(HttpStatus.OK, response.getStatusCode());
        Assert.assertNotNull(response.getBody().getId());
    }

    @Test
    public void testUpdate() {
        FolderDto dto = new FolderDto(null, 3L, "app", "new folder update name", null, false, false);
        HttpEntity request = new HttpEntity<>(dto, requestHeaders);

        ResponseEntity<FolderDto> response = restTemplate.exchange(
                hostName + "/api/folder/{id}",
                HttpMethod.PUT,
                request,
                FolderDto.class,
                2L
        );
        Assert.assertEquals(HttpStatus.OK, response.getStatusCode());
        Assert.assertNotNull(response.getBody().getId());
    }

    @Test
    public void testUpdate_SharedGroup_Success() {
        Mockito.when(keyCloakServiceClient.getGroupAll(any()))
                .thenReturn(List.of(new GroupDto("group-id", "test", "path")));
        Mockito.when(keyCloakServiceClient.getGroupMemberAll(any()))
                .thenReturn(List.of(new com.goodt.drive.supeditor.application.dto.keycloak.UserDto("test-owner-id-2", "", true, true)));

        FolderDto dto = new FolderDto(null, null, "app", "new folder update name", null, false, false);
        HttpEntity request = new HttpEntity<>(dto, requestHeaders);

        ResponseEntity<FolderDto> response = restTemplate.exchange(
                hostName + "/api/folder/{id}",
                HttpMethod.PUT,
                request,
                FolderDto.class,
                11L
        );
        Assert.assertEquals(HttpStatus.OK, response.getStatusCode());
    }

    @Test
    public void testUpdate_SharedGroup_Fail() {
        Mockito.when(keyCloakServiceClient.getGroupAll(any()))
                .thenReturn(List.of(new GroupDto("group-id", "test", "path")));
        Mockito.when(keyCloakServiceClient.getGroupMemberAll(any()))
                .thenReturn(List.of(new com.goodt.drive.supeditor.application.dto.keycloak.UserDto("test-owner-id-2", "", true, true)));

        FolderDto dto = new FolderDto(null, null, "app", "new folder update name", null, false, false);
        HttpEntity request = new HttpEntity<>(dto, requestHeaders);

        ResponseEntity<String> response = restTemplate.exchange(
                hostName + "/api/folder/{id}",
                HttpMethod.PUT,
                request,
                String.class,
                10L
        );
        Assert.assertEquals(HttpStatus.FORBIDDEN, response.getStatusCode());
    }

    @Test
    public void testShareGroup() {
        SharedGroupPermissionDto dto = new SharedGroupPermissionDto(true);
        HttpEntity request = new HttpEntity<>(dto, requestHeaders);

        Long folderId = 2L;
        ResponseEntity<ProjectDto> response = restTemplate.exchange(
                hostName + "/api/folder/{id}/sharegroup",
                HttpMethod.PUT,
                request,
                ProjectDto.class,
                folderId
        );
        Assert.assertEquals(HttpStatus.OK, response.getStatusCode());
        Assert.assertNotNull(response.getBody().getId());
        Assert.assertTrue(response.getBody().isSharedGroup());
        Assert.assertTrue(response.getBody().getCanSharedGroupEdit());
    }

    @Test
    public void testShareGroup_SharedGroup_Fail() {
        SharedGroupPermissionDto dto = new SharedGroupPermissionDto(true);
        HttpEntity request = new HttpEntity<>(dto, requestHeaders);

        Long folderId = 11L;
        ResponseEntity<String> response = restTemplate.exchange(
                hostName + "/api/folder/{id}/sharegroup",
                HttpMethod.PUT,
                request,
                String.class,
                folderId
        );
        Assert.assertEquals(HttpStatus.FORBIDDEN, response.getStatusCode());
    }

    @Test
    public void testUnshareGroup() {
        HttpEntity request = new HttpEntity<>(requestHeaders);

        Long folderId = 2L;
        ResponseEntity<ProjectDto> response = restTemplate.exchange(
                hostName + "/api/folder/{id}/unsharegroup",
                HttpMethod.PUT,
                request,
                ProjectDto.class,
                folderId
        );
        Assert.assertEquals(HttpStatus.OK, response.getStatusCode());
        Assert.assertNotNull(response.getBody().getId());
    }

    @Test
    public void testUnshareGroup_SharedGroup_Fail() {
        HttpEntity request = new HttpEntity<>(requestHeaders);

        Long folderId = 11L;
        ResponseEntity<String> response = restTemplate.exchange(
                hostName + "/api/folder/{id}/unsharegroup",
                HttpMethod.PUT,
                request,
                String.class,
                folderId
        );
        Assert.assertEquals(HttpStatus.FORBIDDEN, response.getStatusCode());
    }

    @Test
    public void testDelete() {
        HttpEntity request = new HttpEntity<>(requestHeaders);

        ResponseEntity<OperationResult> response = restTemplate.exchange(
                hostName + "/api/folder/{id}",
                HttpMethod.DELETE,
                request,
                OperationResult.class,
                3L
        );
        Assert.assertEquals(HttpStatus.OK, response.getStatusCode());
        Assert.assertEquals(true, response.getBody().getResult());
    }
}
