package com.goodt.drive.supeditor.application.rest;

import javassist.NotFoundException;
import org.apache.commons.io.FileUtils;
import org.eclipse.jgit.api.errors.GitAPIException;
import org.junit.Assert;
import org.junit.jupiter.api.*;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.boot.test.web.client.TestRestTemplate;
import org.springframework.boot.web.server.LocalServerPort;
import org.springframework.http.*;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.reactive.function.BodyInserters;
import org.springframework.web.reactive.function.client.WebClient;

import java.io.*;
import java.nio.charset.Charset;
import java.nio.file.Files;
import java.util.*;
import java.util.stream.Collectors;

import com.goodt.drive.supeditor.GlobalDefs;
import com.goodt.drive.supeditor.application.configuration.AppConfig;
import com.goodt.drive.supeditor.application.dao.FolderDao;
import com.goodt.drive.supeditor.application.dao.ProjectContentDao;
import com.goodt.drive.supeditor.application.dao.ProjectDao;
import com.goodt.drive.supeditor.application.data.OperationResult;
import com.goodt.drive.supeditor.application.dto.*;
import com.goodt.drive.supeditor.application.dto.keycloak.GroupDto;
import com.goodt.drive.supeditor.application.exceptions.git.TagsAlreadyExistsException;
import com.goodt.drive.supeditor.application.model.entities.FolderEntity;
import com.goodt.drive.supeditor.application.model.entities.ProjectContentEntity;
import com.goodt.drive.supeditor.application.model.entities.ProjectEntity;
import com.goodt.drive.supeditor.application.services.IProjectService;
import com.goodt.drive.supeditor.application.services.clients.IKeyCloakServiceClient;
import com.goodt.drive.supeditor.utils.IntegrationalTestBase;

import static org.mockito.ArgumentMatchers.any;

@SpringBootTest(webEnvironment = SpringBootTest.WebEnvironment.RANDOM_PORT)
@ActiveProfiles("integrationaltests")
@TestInstance(TestInstance.Lifecycle.PER_CLASS)
public class TestProjectController extends IntegrationalTestBase {

    @LocalServerPort
    private int port;

    @Autowired
    private TestRestTemplate restTemplate;

    private String hostName = "http://localhost:" + port;
    private HttpHeaders requestHeaders;

    @Autowired
    private ProjectContentDao projectContentDao;

    @Autowired
    private IProjectService projectService;

    @Autowired
    private ProjectDao projectDao;

    @Autowired
    private FolderDao folderDao;

    @Autowired
    private AppConfig appConfig;

    @MockBean
    private IKeyCloakServiceClient keyCloakServiceClient;

    @BeforeAll
    public void setup() {
        WebClient client = WebClient.builder()
                .baseUrl(appConfig.getKeyCloakConfig().getBaseUrl())
                .build();
        TokenResponseDto result = client.post().uri(appConfig.getKeyCloakConfig().getTokenUrl())
                .header(HttpHeaders.CONTENT_TYPE, MediaType.APPLICATION_FORM_URLENCODED_VALUE)
                .body(BodyInserters.fromFormData("client_id", appConfig.getKeyCloakConfig().getClientId())
                        .with("client_secret", appConfig.getKeyCloakConfig().getClientSecret())
                        .with("grant_type", "password")
.with("username", appConfig.getKeyCloakConfig().getUsername())
                        .with("password", appConfig.getKeyCloakConfig().getPassword()))
                .retrieve().bodyToMono(TokenResponseDto.class).block();

        hostName = "http://localhost:" + port;
        requestHeaders = new HttpHeaders();
        requestHeaders.add("authorization", String.format("%s %s", result.getTokenType(), result.getAccessToken()));
    }

    @BeforeEach
    public void initUser() {
        HttpEntity request = new HttpEntity<>(requestHeaders);
        ResponseEntity<UserDto> response = restTemplate.exchange(
                hostName + "/api/user/info",
                HttpMethod.GET,
                request,
                UserDto.class
        );
        assert Objects.requireNonNull(response.getBody()).getId() != null;
        List<FolderEntity> folderEntities = folderDao.findAll();
        folderDao.saveAll(folderEntities.stream()
                .filter(folderEntity -> folderEntity.getOwnerId() != null && folderEntity.getOwnerId().equals("test-owner-id"))
                .peek(folderEntity -> folderEntity.setOwnerId(response.getBody().getId()))
                .collect(Collectors.toList()));
        List<ProjectEntity> projectEntities = projectDao.findAll();
        projectDao.saveAll(projectEntities.stream()
                .filter(projectEntity -> projectEntity.getOwnerId() != null && projectEntity.getOwnerId().equals("test-owner-id"))
                .peek(projectEntity -> projectEntity.setOwnerId(response.getBody().getId()))
                .collect(Collectors.toList()));
    }

    @BeforeEach
    public void setupContext() {
        try {
            projectContentDao.save(new ProjectContentEntity(1L, null, GlobalDefs.MAIN_JSON_FILENAME), "{\"json\":\"data\",\"picture\":\"upload/1/filename.jpg\"}".getBytes());
            projectContentDao.save(new ProjectContentEntity(6L, null, GlobalDefs.MAIN_JSON_FILENAME), "{\"json\":\"data\"}".getBytes());
            projectContentDao.save(new ProjectContentEntity(7L, null, GlobalDefs.MAIN_JSON_FILENAME), "{\"json\":\"data\"}".getBytes());
            projectContentDao.save(new ProjectContentEntity(8L, null, GlobalDefs.MAIN_JSON_FILENAME), "{\"json\":\"data\"}".getBytes());
            projectContentDao.save(new ProjectContentEntity(1L, null, GlobalDefs.STYLE_CSS_FILENAME), "theme-css".getBytes());
            projectContentDao.save(new ProjectContentEntity(1L, null, GlobalDefs.STYLE_LESS_FILENAME), "theme-less".getBytes());
            projectContentDao.save(new ProjectContentEntity(1L, null, GlobalDefs.STYLE_THEME_FILENAME), "theme css".getBytes());
            projectContentDao.save(new ProjectContentEntity(1L, GlobalDefs.UPLOAD_DIRNAME, "upload-file.txt"), "upload-file text".getBytes());
            projectContentDao.save(new ProjectContentEntity(1L, GlobalDefs.UPLOAD_DIRNAME, "1/upload-file-2.txt"), "1/upload-file-2 text".getBytes());
            projectContentDao.save(new ProjectContentEntity(1L, GlobalDefs.PAGES_DIRNAME, "conf1-page1.json"), "{\"json-p-1\":\"data-p-1\",\"picture-p-1\":\"upload/1/filename-p-1.jpg\"}".getBytes());
            projectContentDao.save(new ProjectContentEntity(1L, GlobalDefs.PAGES_DIRNAME, "conf1-page2.json"), "conf page data 2".getBytes());
            projectContentDao.save(new ProjectContentEntity(1L, GlobalDefs.PAGES_DIRNAME, "conf1-page3.json"), "conf page data 3".getBytes());
            projectContentDao.save(new ProjectContentEntity(1L, GlobalDefs.PAGES_DIRNAME, "conf1-page4.json"), "conf page data 4".getBytes());
            projectContentDao.save(new ProjectContentEntity(2L, GlobalDefs.PAGES_DIRNAME, "conf2-page1.json"), "".getBytes());
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    @AfterEach
    public void downContext() throws IOException {
        FileUtils.deleteDirectory(new File(appConfig.getProjectConfig().getPath()));
    }

    @Test
    public void testGetById() {
        HttpEntity request = new HttpEntity<>(requestHeaders);

        ResponseEntity<ProjectInfoDto> response = restTemplate.exchange(
                hostName + "/api/project/{id}",
                HttpMethod.GET,
                request,
                ProjectInfoDto.class,
                1L
        );
        Assert.assertEquals(HttpStatus.OK, response.getStatusCode());
        ProjectInfoDto dto = response.getBody();
        Assert.assertTrue(dto.getMeta().getId().equals(1L));
        Assert.assertTrue(dto.getMeta().getSlug().equals("conf-1-slug"));
        Assert.assertTrue(dto.getMeta().getTitle().equals("conf 1 title"));
        Assert.assertTrue(dto.getMeta().getTags().size() > 0);
        Assert.assertTrue(dto.getData().length() > 0);
        Assert.assertEquals("{\"json\":\"data\",\"picture\":\"upload/1/filename.jpg\"}", dto.getData());
        Assert.assertTrue(dto.getMeta().getIsOwner());
    }

    @Test
    public void testPreview() {
        HttpEntity request = new HttpEntity<>(requestHeaders);

        ResponseEntity<byte[]> response = restTemplate.exchange(
                hostName + "/api/project/{project}/{filename}",
                HttpMethod.GET,
                request,
                byte[].class,
                1L,
                GlobalDefs.MAIN_JSON_FILENAME
        );
        Assert.assertEquals(HttpStatus.OK, response.getStatusCode());
        Assert.assertTrue(response.getBody().length > 0);
    }

    @Test
    public void testPreview_BySlug() {
        HttpEntity request = new HttpEntity<>(requestHeaders);

        ResponseEntity<byte[]> response = restTemplate.exchange(
                hostName + "/api/project/{project}/{filename}",
                HttpMethod.GET,
                request,
                byte[].class,
                "conf-1-slug",
                GlobalDefs.MAIN_JSON_FILENAME
        );
        Assert.assertEquals(HttpStatus.OK, response.getStatusCode());
        Assert.assertTrue(response.getBody().length > 0);
    }

    @Test
    public void testThemeCss() {
        HttpEntity request = new HttpEntity<>(requestHeaders);

        ResponseEntity<String> response = restTemplate.exchange(
                hostName + "/api/project/{project}/" + GlobalDefs.STYLE_CSS_FILENAME,
                HttpMethod.GET,
                request,
                String.class,
                1L
        );
        Assert.assertEquals(HttpStatus.OK, response.getStatusCode());
        Assert.assertTrue(response.getBody().length() > 0);
    }

    @Test
    public void testThemeLess() {
        HttpEntity request = new HttpEntity<>(requestHeaders);

        ResponseEntity<String> response = restTemplate.exchange(
                hostName + "/api/project/{project}/" + GlobalDefs.STYLE_LESS_FILENAME,
                HttpMethod.GET,
                request,
                String.class,
                1L
        );
        Assert.assertEquals(HttpStatus.OK, response.getStatusCode());
        Assert.assertTrue(response.getBody().length() > 0);
    }

    @Test
    public void testTheme() {
        HttpEntity request = new HttpEntity<>(requestHeaders);

        ResponseEntity<String> response = restTemplate.exchange(
                hostName + "/api/project/{project}/" + GlobalDefs.STYLE_THEME_FILENAME,
                HttpMethod.GET,
                request,
                String.class,
                1L
        );
        Assert.assertEquals(HttpStatus.OK, response.getStatusCode());
        Assert.assertTrue(response.getBody().length() > 0);
    }

    @Test
    public void testUploadFile() {
        HttpEntity request = new HttpEntity<>(requestHeaders);

        ResponseEntity<String> response = restTemplate.exchange(
                hostName + "/api/project/{project}/" + GlobalDefs.UPLOAD_DIRNAME + "/upload-file.txt",
                HttpMethod.GET,
                request,
                String.class,
                1L
        );
        Assert.assertEquals(HttpStatus.OK, response.getStatusCode());
        Assert.assertTrue(response.getBody().length() > 0);
    }

    @Test
    public void testUploadFile_Subdir() {
        HttpEntity request = new HttpEntity<>(requestHeaders);

        ResponseEntity<String> response = restTemplate.exchange(
                hostName + "/api/project/{project}/" + GlobalDefs.UPLOAD_DIRNAME + "/1/upload-file-2.txt",
                HttpMethod.GET,
                request,
                String.class,
                1L
        );
        Assert.assertEquals(HttpStatus.OK, response.getStatusCode());
        Assert.assertTrue(response.getBody().length() > 0);
    }

    @Test
    public void testPage() {
        HttpEntity request = new HttpEntity<>(requestHeaders);

        ResponseEntity<String> response = restTemplate.exchange(
                hostName + "/api/project/{project}/" + GlobalDefs.PAGES_DIRNAME + "/conf1-page1.json",
                HttpMethod.GET,
                request,
                String.class,
                1L
        );
        Assert.assertEquals(HttpStatus.OK, response.getStatusCode());
        Assert.assertTrue(response.getBody().length() > 0);
    }

    @Test
    public void testPage_NotFound() {
        HttpEntity request = new HttpEntity<>(requestHeaders);

        ResponseEntity<String> response = restTemplate.exchange(
                hostName + "/api/project/{project}/" + GlobalDefs.PAGES_DIRNAME + "/conf999-page999.json",
                HttpMethod.GET,
                request,
                String.class,
                1L
        );
        Assert.assertEquals(HttpStatus.NOT_FOUND, response.getStatusCode());
    }

    @Test
    public void testExists() {
        HttpEntity request = new HttpEntity<>(requestHeaders);

        ResponseEntity<OperationResult> response = restTemplate.exchange(
                hostName + "/api/project/{slug}/exists",
                HttpMethod.GET,
                request,
                OperationResult.class,
                "conf-1-slug"
        );
        Assert.assertEquals(HttpStatus.OK, response.getStatusCode());
        Assert.assertEquals(true, response.getBody().getResult());
    }

    @Test
    public void testExists_Excluded() {
        HttpEntity request = new HttpEntity<>(requestHeaders);

        ResponseEntity<OperationResult> response = restTemplate.exchange(
                hostName + "/api/project/{slug}/exists?exclude_id={exclude_id}",
                HttpMethod.GET,
                request,
                OperationResult.class,
                "conf-1-slug",
                1L
        );
        Assert.assertEquals(HttpStatus.OK, response.getStatusCode());
        Assert.assertEquals(false, response.getBody().getResult());
    }

    @Test
    public void testExists_NotFound() {
        HttpEntity request = new HttpEntity<>(requestHeaders);

        ResponseEntity<OperationResult> response = restTemplate.exchange(
                hostName + "/api/project/{slug}/exists",
                HttpMethod.GET,
                request,
                OperationResult.class,
                "conf-1-slug-not-found"
        );
        Assert.assertEquals(HttpStatus.OK, response.getStatusCode());
        Assert.assertEquals(false, response.getBody().getResult());
    }

    @Test
    public void testExport() {
        HttpEntity request = new HttpEntity<>(requestHeaders);

        ResponseEntity<byte[]> response = restTemplate.exchange(
                hostName + "/api/project/{id}/export",
                HttpMethod.GET,
                request,
                byte[].class,
                1L
        );
        Assert.assertEquals(HttpStatus.OK, response.getStatusCode());
        Assert.assertTrue(response.getBody().length > 0);
    }

    @Test
    public void testExport_AccessDenied() {
        HttpEntity request = new HttpEntity<>(requestHeaders);

        ResponseEntity<byte[]> response = restTemplate.exchange(
                hostName + "/api/project/{id}/export",
                HttpMethod.GET,
                request,
                byte[].class,
                4L
        );
        Assert.assertEquals(HttpStatus.FORBIDDEN, response.getStatusCode());
        Assert.assertTrue(response.getBody().length > 0);
    }

    @Test
    public void testExport_ByAppKey() {
        HttpEntity request = new HttpEntity<>(new HttpHeaders());

        ResponseEntity<byte[]> response = restTemplate.exchange(
                hostName + "/api/project/{id}/export?&key={key}",
                HttpMethod.GET,
                request,
                byte[].class,
                4L,
                appConfig.getAppKey()
        );
        Assert.assertEquals(HttpStatus.OK, response.getStatusCode());
        Assert.assertTrue(response.getBody().length > 0);
    }

    @Test
    public void testImport() throws IOException {
        InputStream inputStream = new ByteArrayInputStream("text file data".getBytes());
        MultipartFile file = new MockMultipartFile("text-file.txt", "text-file.txt", MediaType.APPLICATION_OCTET_STREAM_VALUE, inputStream);
        HttpEntity archive = new HttpEntity<>(file.getResource());

        ProjectDto dto = new ProjectDto(null, "new-project-import-slug", "new-project-import-title", "new-project-import-description", "app", null, new ArrayList<>(), null, false, false);
        HttpHeaders bodyPartHeaders = new HttpHeaders();
        bodyPartHeaders.setContentType(MediaType.APPLICATION_JSON);
        HttpEntity body = new HttpEntity<>(dto, bodyPartHeaders);

        MultiValueMap<String, Object> requestBody = new LinkedMultiValueMap<>();
        requestBody.add("body", body);
        requestBody.add("archive", archive);

        HttpHeaders currentHeaders = new HttpHeaders();
        currentHeaders.putAll(requestHeaders);
        currentHeaders.setContentType(MediaType.MULTIPART_FORM_DATA);
        HttpEntity<MultiValueMap> requestExtra = new HttpEntity<>(requestBody, currentHeaders);

        ResponseEntity<ProjectDto> response = restTemplate.exchange(
                hostName + "/api/project/import",
                HttpMethod.POST,
                requestExtra,
                ProjectDto.class
        );
        Assert.assertEquals(HttpStatus.OK, response.getStatusCode());
        Assert.assertNotNull(response.getBody().getId());
    }

    @Test
    public void testExportImport() throws IOException {
        Long originalProjectId = 1L;

        File originalUploadDir = new File(appConfig.getProjectConfig().getPath(), String.format("%d/%s", originalProjectId, GlobalDefs.UPLOAD_DIRNAME));
        Files.createDirectories(originalUploadDir.toPath());
        try (FileOutputStream fileOutputStream = new FileOutputStream(String.format("%s/%s", originalUploadDir, "test.jpg"))) {
            fileOutputStream.write("test-file.jpg".getBytes());
        }

        HttpEntity request = new HttpEntity<>(requestHeaders);
        ResponseEntity<byte[]> responseExport = restTemplate.exchange(
                hostName + "/api/project/{id}/export",
                HttpMethod.GET,
                request,
                byte[].class,
                originalProjectId
        );

        InputStream inputStream = new ByteArrayInputStream(responseExport.getBody());
        MultipartFile file = new MockMultipartFile("text-file.txt", "text-file.txt", MediaType.APPLICATION_OCTET_STREAM_VALUE, inputStream);
        HttpEntity archive = new HttpEntity<>(file.getResource());

        ProjectDto dto = new ProjectDto(null, "new-project-import-slug", "new-project-import-title", "new-project-import-description", "app", null, new ArrayList<>(), null, false, false);
        HttpHeaders bodyPartHeaders = new HttpHeaders();
        bodyPartHeaders.setContentType(MediaType.APPLICATION_JSON);
        HttpEntity body = new HttpEntity<>(dto, bodyPartHeaders);

        MultiValueMap<String, Object> requestBody = new LinkedMultiValueMap<>();
        requestBody.add("body", body);
        requestBody.add("archive", archive);

        HttpHeaders currentHeaders = new HttpHeaders();
        currentHeaders.putAll(requestHeaders);
        currentHeaders.setContentType(MediaType.MULTIPART_FORM_DATA);
        HttpEntity<MultiValueMap> requestExtra = new HttpEntity<>(requestBody, currentHeaders);

        ResponseEntity<ProjectDto> response = restTemplate.exchange(
                hostName + "/api/project/import",
                HttpMethod.POST,
                requestExtra,
                ProjectDto.class
        );
        Assert.assertEquals(HttpStatus.OK, response.getStatusCode());
        Object projectId = response.getBody().getId();
        Assert.assertNotNull(projectId);
        File uploadedFileTestJpg = new File(String.format("%s/%d/%s/%s", appConfig.getProjectConfig().getPath(), projectId, GlobalDefs.UPLOAD_DIRNAME, "test.jpg"));
        Assert.assertTrue(uploadedFileTestJpg.exists());
    }

    @Test
    public void testClone() {
        ProjectDto dto = new ProjectDto(null, "new-clone-slug", "new clone title", "new clone description", "app", 1L, Arrays.asList("tag 1 name", "new tag name 1"), null, true, true);
        HttpEntity request = new HttpEntity<>(dto, requestHeaders);

        ResponseEntity<ProjectDto> response = restTemplate.exchange(
                hostName + "/api/project/{id}/clone",
                HttpMethod.POST,
                request,
                ProjectDto.class,
                1L
        );
        Assert.assertEquals(HttpStatus.OK, response.getStatusCode());
        Assert.assertNotNull(response.getBody().getId());
        ProjectEntity updatedProjectEntity = projectDao.findById(response.getBody().getId()).get();
        Assert.assertFalse(updatedProjectEntity.isSharedGroup());
    }

    @Test
    public void testClone_SharedFolder() {
        Mockito.when(keyCloakServiceClient.getGroupAll(any()))
                .thenReturn(List.of(new GroupDto("group-id", "test", "path")));
        Mockito.when(keyCloakServiceClient.getGroupMemberAll(any()))
                .thenReturn(List.of(new com.goodt.drive.supeditor.application.dto.keycloak.UserDto("test-owner-id-2", "", true, true)));

        ProjectDto dto = new ProjectDto(null, "new-clone-slug", "new clone title", "new clone description", "app", 10L, Arrays.asList("tag 1 name", "new tag name 1"), null, true, true);
        HttpEntity request = new HttpEntity<>(dto, requestHeaders);

        ResponseEntity<ProjectDto> response = restTemplate.exchange(
                hostName + "/api/project/{id}/clone",
                HttpMethod.POST,
                request,
                ProjectDto.class,
                1L
        );
        Assert.assertEquals(HttpStatus.OK, response.getStatusCode());
        Assert.assertNotNull(response.getBody().getId());
    }

    @Test
    public void testList() {
        HttpEntity request = new HttpEntity<>(requestHeaders);

        ResponseEntity<ProjectDto[]> response = restTemplate.exchange(
                hostName + "/api/project?type={type}",
                HttpMethod.GET,
                request,
                ProjectDto[].class,
                "app"
        );
        Assert.assertEquals(HttpStatus.OK, response.getStatusCode());
        Assert.assertNotNull(response.getBody().length > 0);
        Assert.assertTrue(Arrays.stream(response.getBody()).allMatch(projectDto -> projectDto.getType().equals("app")));
        Assert.assertTrue(Arrays.stream(response.getBody()).allMatch(projectDto -> projectDto.getFolderId() == null));
        List<Long> projectIdList = Arrays.stream(response.getBody()).map(projectDto -> projectDto.getId()).collect(Collectors.toList());
        List<ProjectEntity> projectEntities = projectDao.findAllById(projectIdList);
        String ownerId = projectEntities.get(0).getOwnerId();
        Assert.assertTrue(projectEntities.stream().allMatch(projectEntity -> projectEntity.getOwnerId().equals(ownerId)));
    }

    @Test
    public void testList_ByFolder() {
        HttpEntity request = new HttpEntity<>(requestHeaders);

        ResponseEntity<ProjectDto[]> response = restTemplate.exchange(
                hostName + "/api/project?type={type}&folder_id={folder_id}",
                HttpMethod.GET,
                request,
                ProjectDto[].class,
                "app",
                1L
        );
        Assert.assertEquals(HttpStatus.OK, response.getStatusCode());
        Assert.assertNotNull(response.getBody().length > 0);
        Assert.assertTrue(Arrays.stream(response.getBody()).allMatch(projectDto -> projectDto.getType().equals("app")));
        Assert.assertTrue(Arrays.stream(response.getBody()).allMatch(projectDto -> projectDto.getFolderId().equals(1L)));
    }

    @Test
    public void testList_ByKey() {
        HttpEntity request = new HttpEntity<>(new HttpHeaders());

        ResponseEntity<ProjectDto[]> response = restTemplate.exchange(
                hostName + "/api/project?type={type}&key={key}",
                HttpMethod.GET,
                request,
                ProjectDto[].class,
                "app",
                appConfig.getAppKey()
        );
        Assert.assertEquals(HttpStatus.OK, response.getStatusCode());
        Assert.assertNotNull(response.getBody().length > 0);
    }

    @Test
    public void testList_AccessDenied() {
        HttpEntity request = new HttpEntity<>(new HttpHeaders());

        ResponseEntity<String> response = restTemplate.exchange(
                hostName + "/api/project?type={type}&key=",
                HttpMethod.GET,
                request,
                String.class,
                "app"
        );
        Assert.assertEquals(HttpStatus.FORBIDDEN, response.getStatusCode());
    }

    @Test
    public void testListAll_ByKey() {
        HttpEntity request = new HttpEntity<>(new HttpHeaders());

        ResponseEntity<ProjectDto[]> response = restTemplate.exchange(
                hostName + "/api/project/listall?key={key}",
                HttpMethod.GET,
                request,
                ProjectDto[].class,
                appConfig.getAppKey()
        );
        Assert.assertEquals(HttpStatus.OK, response.getStatusCode());
        Assert.assertEquals(13, response.getBody().length);
    }

    @Test
    public void testListAllByKey_AccessDenied() {
        HttpEntity request = new HttpEntity<>(new HttpHeaders());

        ResponseEntity<String> response = restTemplate.exchange(
                hostName + "/api/project/listall?key=",
                HttpMethod.GET,
                request,
                String.class
        );
        Assert.assertEquals(HttpStatus.FORBIDDEN, response.getStatusCode());
    }

    @Test
    public void testUpdateTheme() throws IOException {
        ProjectThemeDto dto = new ProjectThemeDto(null, "new upd css", "new upd less", "new upd theme");
        HttpEntity request = new HttpEntity<>(dto, requestHeaders);

        Long projectId = 2L;
        ResponseEntity<ProjectDto> response = restTemplate.exchange(
                hostName + "/api/project/{project_id}/updatetheme",
                HttpMethod.PUT,
                request,
                ProjectDto.class,
                projectId
        );
        Assert.assertEquals(HttpStatus.OK, response.getStatusCode());
        Assert.assertNotNull(response.getBody().getId());
        String cssTheme = FileUtils.readFileToString(
                new File(String.format("%s/%d/%s", appConfig.getProjectConfig().getPath(), projectId, GlobalDefs.STYLE_CSS_FILENAME)), Charset.defaultCharset());
        Assert.assertEquals("new upd css", cssTheme);
        String lessTheme = FileUtils.readFileToString(
                new File(String.format("%s/%d/%s", appConfig.getProjectConfig().getPath(), projectId, GlobalDefs.STYLE_LESS_FILENAME)), Charset.defaultCharset());
        Assert.assertEquals("new upd less", lessTheme);
        String themeCss = FileUtils.readFileToString(
                new File(String.format("%s/%d/%s", appConfig.getProjectConfig().getPath(), projectId, GlobalDefs.STYLE_THEME_FILENAME)), Charset.defaultCharset());
        Assert.assertEquals("new upd theme", themeCss);
    }

    @Test
    public void testUpdateTheme_SharedGroup_Success() {
        Mockito.when(keyCloakServiceClient.getGroupAll(any()))
                .thenReturn(List.of(new GroupDto("group-id", "test", "path")));
        Mockito.when(keyCloakServiceClient.getGroupMemberAll(any()))
                .thenReturn(List.of(new com.goodt.drive.supeditor.application.dto.keycloak.UserDto("test-owner-id-2", "", true, true)));

        ProjectThemeDto dto = new ProjectThemeDto(null, "new upd css", "new upd less", "new upd theme");
        HttpEntity request = new HttpEntity<>(dto, requestHeaders);

        Long projectId = 12L;
        ResponseEntity<ProjectDto> response = restTemplate.exchange(
                hostName + "/api/project/{project_id}/updatetheme",
                HttpMethod.PUT,
                request,
                ProjectDto.class,
                projectId
        );
        Assert.assertEquals(HttpStatus.OK, response.getStatusCode());
    }

    @Test
    public void testUpdateTheme_SharedGroup_Fail() {
        Mockito.when(keyCloakServiceClient.getGroupAll(any()))
                .thenReturn(List.of(new GroupDto("group-id", "test", "path")));
        Mockito.when(keyCloakServiceClient.getGroupMemberAll(any()))
                .thenReturn(List.of(new com.goodt.drive.supeditor.application.dto.keycloak.UserDto("test-owner-id-2", "", true, true)));

        ProjectThemeDto dto = new ProjectThemeDto(null, "new upd css", "new upd less", "new upd theme");
        HttpEntity request = new HttpEntity<>(dto, requestHeaders);

        Long projectId = 11L;
        ResponseEntity<String> response = restTemplate.exchange(
                hostName + "/api/project/{project_id}/updatetheme",
                HttpMethod.PUT,
                request,
                String.class,
                projectId
        );
        Assert.assertEquals(HttpStatus.FORBIDDEN, response.getStatusCode());
    }

    @Test
    public void testCreate() {
        ProjectDto dto = new ProjectDto(null, "new-clone-slug-33", "new clone title-33", "new clone description-33", "app", null, Arrays.asList("tag 1 name", "new tag name 33"), null, false, false);
        HttpEntity request = new HttpEntity<>(dto, requestHeaders);

        ResponseEntity<ProjectDto> response = restTemplate.exchange(
                hostName + "/api/project",
                HttpMethod.POST,
                request,
                ProjectDto.class
        );
        Assert.assertEquals(HttpStatus.OK, response.getStatusCode());
        Assert.assertNotNull(response.getBody().getId());
        Assert.assertTrue(response.getBody().getTags().size() > 0);
    }

    @Test
    public void testUpdate() {
        ProjectDto dto = new ProjectDto(null, "new-clone-slug-2", "new upd title-2", "new upd description-2", "app", null, Arrays.asList("tag 1 name", "new tag name 2"), null, false, false);
        HttpEntity request = new HttpEntity<>(dto, requestHeaders);

        ResponseEntity<ProjectDto> response = restTemplate.exchange(
                hostName + "/api/project/{id}",
                HttpMethod.PUT,
                request,
                ProjectDto.class,
                2L
        );
        Assert.assertEquals(HttpStatus.OK, response.getStatusCode());
        Assert.assertNotNull(response.getBody().getId());
    }

    @Test
    public void testUpdate_SharedGroup_Success() {
        Mockito.when(keyCloakServiceClient.getGroupAll(any()))
                .thenReturn(List.of(new GroupDto("group-id", "test", "path")));
        Mockito.when(keyCloakServiceClient.getGroupMemberAll(any()))
                .thenReturn(List.of(new com.goodt.drive.supeditor.application.dto.keycloak.UserDto("test-owner-id-2", "", true, true)));

        ProjectDto dto = new ProjectDto(null, "new-clone-slug-2", "new upd title-2", "new upd description-2", "app", null, Arrays.asList("tag 1 name", "new tag name 2"), null, false, false);
        HttpEntity request = new HttpEntity<>(dto, requestHeaders);

        ResponseEntity<String> response = restTemplate.exchange(
                hostName + "/api/project/{id}",
                HttpMethod.PUT,
                request,
                String.class,
                12L
        );
        Assert.assertEquals(HttpStatus.OK, response.getStatusCode());
    }

    @Test
    public void testUpdate_SharedGroup_Fail() {
        Mockito.when(keyCloakServiceClient.getGroupAll(any()))
                .thenReturn(List.of(new GroupDto("group-id", "test", "path")));
        Mockito.when(keyCloakServiceClient.getGroupMemberAll(any()))
                .thenReturn(List.of(new com.goodt.drive.supeditor.application.dto.keycloak.UserDto("test-owner-id-2", "", true, true)));

        ProjectDto dto = new ProjectDto(null, "new-clone-slug-2", "new upd title-2", "new upd description-2", "app", null, Arrays.asList("tag 1 name", "new tag name 2"), null, false, false);
        HttpEntity request = new HttpEntity<>(dto, requestHeaders);

        ResponseEntity<String> response = restTemplate.exchange(
                hostName + "/api/project/{id}",
                HttpMethod.PUT,
                request,
                String.class,
                11L
        );
        Assert.assertEquals(HttpStatus.FORBIDDEN, response.getStatusCode());
    }

    @Test
    public void testGetCommit() throws GitAPIException, NotFoundException, IOException, TagsAlreadyExistsException {
        projectService.commitProject("test-owner-id", 2L, "commit comment", List.of("tag2"), "");

        HttpEntity request = new HttpEntity<>(requestHeaders);

        ResponseEntity<ProjectCommitDto[]> response = restTemplate.exchange(
                hostName + "/api/project/{id}/commit",
                HttpMethod.GET,
                request,
                ProjectCommitDto[].class,
                2L
        );
        Assert.assertEquals(HttpStatus.OK, response.getStatusCode());
        ProjectCommitDto[] body = response.getBody();
        Assert.assertEquals("commit comment", body[0].getComment());
        Assert.assertEquals("tag2", body[0].getTags().get(0));
    }

    @Test
    public void testPostCommit() {
        ProjectCommitDto dto = new ProjectCommitDto("comment", List.of("tag2"));
        HttpEntity request = new HttpEntity<>(dto, requestHeaders);

        ResponseEntity<ProjectCommitDto> response = restTemplate.exchange(
                hostName + "/api/project/{id}/commit",
                HttpMethod.POST,
                request,
                ProjectCommitDto.class,
                2L
        );
        Assert.assertEquals(HttpStatus.OK, response.getStatusCode());
        Assert.assertNotNull(response.getBody().getHash());
    }

    @Test
    public void testPostCommit_TagAlreadyExists() throws GitAPIException, TagsAlreadyExistsException, NotFoundException, IOException {
        projectService.commitProject("test-owner-id", 2L, "commit comment", List.of("tag2"), "");

        ProjectCommitDto dto = new ProjectCommitDto("comment", List.of("tag2"));
        HttpEntity request = new HttpEntity<>(dto, requestHeaders);

        ResponseEntity<String> response = restTemplate.exchange(
                hostName + "/api/project/{id}/commit",
                HttpMethod.POST,
                request,
                String.class,
                2L
        );
        Assert.assertEquals(HttpStatus.BAD_REQUEST, response.getStatusCode());
        Assert.assertNotNull(response.getBody());
    }

    @Test
    public void testPostCommit_SharedGroup_Success() {
        Mockito.when(keyCloakServiceClient.getGroupAll(any()))
                .thenReturn(List.of(new GroupDto("group-id", "test", "path")));
        Mockito.when(keyCloakServiceClient.getGroupMemberAll(any()))
                .thenReturn(List.of(new com.goodt.drive.supeditor.application.dto.keycloak.UserDto("test-owner-id-2", "", true, true)));

        ProjectCommitDto dto = new ProjectCommitDto("comment", List.of("tag2"));
        HttpEntity request = new HttpEntity<>(dto, requestHeaders);

        ResponseEntity<ProjectCommitDto> response = restTemplate.exchange(
                hostName + "/api/project/{id}/commit",
                HttpMethod.POST,
                request,
                ProjectCommitDto.class,
                12L
        );
        Assert.assertEquals(HttpStatus.OK, response.getStatusCode());
    }

    @Test
    public void testPostCommit_SharedGroup_Fail() {
        Mockito.when(keyCloakServiceClient.getGroupAll(any()))
                .thenReturn(List.of(new GroupDto("group-id", "test", "path")));
        Mockito.when(keyCloakServiceClient.getGroupMemberAll(any()))
                .thenReturn(List.of(new com.goodt.drive.supeditor.application.dto.keycloak.UserDto("test-owner-id-2", "", true, true)));

        ProjectCommitDto dto = new ProjectCommitDto("comment", List.of("tag2"));
        HttpEntity request = new HttpEntity<>(dto, requestHeaders);

        ResponseEntity<String> response = restTemplate.exchange(
                hostName + "/api/project/{id}/commit",
                HttpMethod.POST,
                request,
                String.class,
                11L
        );
        Assert.assertEquals(HttpStatus.FORBIDDEN, response.getStatusCode());
    }

    @Test
    public void testUpdateData() throws IOException {
        ProjectDataDto dto = new ProjectDataDto(null, "new upd data");
        HttpEntity request = new HttpEntity<>(dto, requestHeaders);

        Long projectId = 2L;
        ResponseEntity<ProjectDto> response = restTemplate.exchange(
                hostName + "/api/project/{id}/updatedata",
                HttpMethod.PUT,
                request,
                ProjectDto.class,
                projectId
        );
        Assert.assertEquals(HttpStatus.OK, response.getStatusCode());
        Assert.assertNotNull(response.getBody().getId());
        String data = FileUtils.readFileToString(new File(String.format("%s/%d/%s", appConfig.getProjectConfig().getPath(), projectId, GlobalDefs.MAIN_JSON_FILENAME)), Charset.defaultCharset());
        Assert.assertEquals("new upd data", data);
    }

    @Test
    public void testUpdateData_SharedGroup_Success() {
        Mockito.when(keyCloakServiceClient.getGroupAll(any()))
                .thenReturn(List.of(new GroupDto("group-id", "test", "path")));
        Mockito.when(keyCloakServiceClient.getGroupMemberAll(any()))
                .thenReturn(List.of(new com.goodt.drive.supeditor.application.dto.keycloak.UserDto("test-owner-id-2", "", true, true)));

        ProjectDataDto dto = new ProjectDataDto(null, "new upd data");
        HttpEntity request = new HttpEntity<>(dto, requestHeaders);

        Long projectId = 12L;
        ResponseEntity<ProjectDto> response = restTemplate.exchange(
                hostName + "/api/project/{id}/updatedata",
                HttpMethod.PUT,
                request,
                ProjectDto.class,
                projectId
        );
        Assert.assertEquals(HttpStatus.OK, response.getStatusCode());
    }

    @Test
    public void testUpdateData_SharedGroup_Fail() {
        Mockito.when(keyCloakServiceClient.getGroupAll(any()))
                .thenReturn(List.of(new GroupDto("group-id", "test", "path")));
        Mockito.when(keyCloakServiceClient.getGroupMemberAll(any()))
                .thenReturn(List.of(new com.goodt.drive.supeditor.application.dto.keycloak.UserDto("test-owner-id-2", "", true, true)));

        ProjectDataDto dto = new ProjectDataDto(null, "new upd data");
        HttpEntity request = new HttpEntity<>(dto, requestHeaders);

        Long projectId = 11L;
        ResponseEntity<String> response = restTemplate.exchange(
                hostName + "/api/project/{id}/updatedata",
                HttpMethod.PUT,
                request,
                String.class,
                projectId
        );
        Assert.assertEquals(HttpStatus.FORBIDDEN, response.getStatusCode());
    }

    @Test
    public void testShareGroup() {
        SharedGroupPermissionDto dto = new SharedGroupPermissionDto(true);
        HttpEntity request = new HttpEntity<>(dto, requestHeaders);

        Long projectId = 2L;
        ResponseEntity<ProjectDto> response = restTemplate.exchange(
                hostName + "/api/project/{id}/sharegroup",
                HttpMethod.PUT,
                request,
                ProjectDto.class,
                projectId
        );
        Assert.assertEquals(HttpStatus.OK, response.getStatusCode());
        Assert.assertNotNull(response.getBody().getId());
        Assert.assertNotNull(response.getBody().isSharedGroup());
        Assert.assertNotNull(response.getBody().getCanSharedGroupEdit());
    }

    @Test
    public void testShareGroup_SharedGroup_Fail() {
        SharedGroupPermissionDto dto = new SharedGroupPermissionDto(true);
        HttpEntity request = new HttpEntity<>(dto, requestHeaders);

        Long projectId = 12L;
        ResponseEntity<String> response = restTemplate.exchange(
                hostName + "/api/project/{id}/sharegroup",
                HttpMethod.PUT,
                request,
                String.class,
                projectId
        );
        Assert.assertEquals(HttpStatus.FORBIDDEN, response.getStatusCode());
    }

    @Test
    public void testUnshareGroup() {
        HttpEntity request = new HttpEntity<>(requestHeaders);

        Long projectId = 2L;
        ResponseEntity<ProjectDto> response = restTemplate.exchange(
                hostName + "/api/project/{id}/unsharegroup",
                HttpMethod.PUT,
                request,
                ProjectDto.class,
                projectId
        );
        Assert.assertEquals(HttpStatus.OK, response.getStatusCode());
        Assert.assertNotNull(response.getBody().getId());
    }

    @Test
    public void testUnshareGroup_SharedGroup_Fail() {
        HttpEntity request = new HttpEntity<>(requestHeaders);

        Long projectId = 12L;
        ResponseEntity<String> response = restTemplate.exchange(
                hostName + "/api/project/{id}/unsharegroup",
                HttpMethod.PUT,
                request,
                String.class,
                projectId
        );
        Assert.assertEquals(HttpStatus.FORBIDDEN, response.getStatusCode());
    }

    @Test
    public void testDelete() {
        HttpEntity request = new HttpEntity<>(requestHeaders);

        ResponseEntity<OperationResult> response = restTemplate.exchange(
                hostName + "/api/project/{id}",
                HttpMethod.DELETE,
                request,
                OperationResult.class,
                3L
        );
        Assert.assertEquals(HttpStatus.OK, response.getStatusCode());
        Assert.assertEquals(true, response.getBody().getResult());
    }

    @Test
    public void testDelete_SharedGroup_Success() {
        Mockito.when(keyCloakServiceClient.getGroupAll(any()))
                .thenReturn(List.of(new GroupDto("group-id", "test", "path")));
        Mockito.when(keyCloakServiceClient.getGroupMemberAll(any()))
                .thenReturn(List.of(new com.goodt.drive.supeditor.application.dto.keycloak.UserDto("test-owner-id-2", "", true, true)));

        HttpEntity request = new HttpEntity<>(requestHeaders);

        ResponseEntity<OperationResult> response = restTemplate.exchange(
                hostName + "/api/project/{id}",
                HttpMethod.DELETE,
                request,
                OperationResult.class,
                12L
        );
        Assert.assertEquals(HttpStatus.OK, response.getStatusCode());
    }

    @Test
    public void testDelete_SharedGroup_Fail() {
        Mockito.when(keyCloakServiceClient.getGroupAll(any()))
                .thenReturn(List.of(new GroupDto("group-id", "test", "path")));
        Mockito.when(keyCloakServiceClient.getGroupMemberAll(any()))
                .thenReturn(List.of(new com.goodt.drive.supeditor.application.dto.keycloak.UserDto("test-owner-id-2", "", true, true)));

        HttpEntity request = new HttpEntity<>(requestHeaders);

        ResponseEntity<String> response = restTemplate.exchange(
                hostName + "/api/project/{id}",
                HttpMethod.DELETE,
                request,
                String.class,
                11L
        );
        Assert.assertEquals(HttpStatus.FORBIDDEN, response.getStatusCode());
    }
}
