package com.goodt.drive.supeditor.application.rest;

import org.apache.commons.io.FileUtils;
import org.junit.Assert;
import org.junit.jupiter.api.*;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.boot.test.web.client.TestRestTemplate;
import org.springframework.boot.web.server.LocalServerPort;
import org.springframework.http.*;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.reactive.function.BodyInserters;
import org.springframework.web.reactive.function.client.WebClient;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

import com.goodt.drive.supeditor.GlobalDefs;
import com.goodt.drive.supeditor.application.configuration.AppConfig;
import com.goodt.drive.supeditor.application.dao.FolderDao;
import com.goodt.drive.supeditor.application.dao.ProjectDao;
import com.goodt.drive.supeditor.application.data.OperationResult;
import com.goodt.drive.supeditor.application.dto.TokenResponseDto;
import com.goodt.drive.supeditor.application.dto.UploadFileDto;
import com.goodt.drive.supeditor.application.dto.UserDto;
import com.goodt.drive.supeditor.application.dto.keycloak.GroupDto;
import com.goodt.drive.supeditor.application.model.entities.FolderEntity;
import com.goodt.drive.supeditor.application.model.entities.ProjectContentEntity;
import com.goodt.drive.supeditor.application.model.entities.ProjectEntity;
import com.goodt.drive.supeditor.application.dao.ProjectContentDao;
import com.goodt.drive.supeditor.application.services.clients.IKeyCloakServiceClient;
import com.goodt.drive.supeditor.utils.IntegrationalTestBase;

import static org.mockito.ArgumentMatchers.any;

@SpringBootTest(webEnvironment = SpringBootTest.WebEnvironment.RANDOM_PORT)
@ActiveProfiles("integrationaltests")
@TestInstance(TestInstance.Lifecycle.PER_CLASS)
public class TestProjectFileController extends IntegrationalTestBase {

    @LocalServerPort
    private int port;

    @Autowired
    private TestRestTemplate restTemplate;

    private String hostName = "http://localhost:" + port;
    private HttpHeaders requestHeaders;

    @Autowired
    private ProjectContentDao projectContentDao;

    @Autowired
    private ProjectDao projectDao;

    @Autowired
    private FolderDao folderDao;

    @Autowired
    private AppConfig appConfig;

    @MockBean
    private IKeyCloakServiceClient keyCloakServiceClient;

    @BeforeAll
    public void setup() {
        WebClient client = WebClient.builder()
                .baseUrl(appConfig.getKeyCloakConfig().getBaseUrl())
                .build();
        TokenResponseDto result = client.post().uri(appConfig.getKeyCloakConfig().getTokenUrl())
                .header(HttpHeaders.CONTENT_TYPE, MediaType.APPLICATION_FORM_URLENCODED_VALUE)
                .body(BodyInserters.fromFormData("client_id", appConfig.getKeyCloakConfig().getClientId())
                        .with("client_secret", appConfig.getKeyCloakConfig().getClientSecret())
                        .with("grant_type", "password")
.with("username", appConfig.getKeyCloakConfig().getUsername())
                        .with("password", appConfig.getKeyCloakConfig().getPassword()))
                .retrieve().bodyToMono(TokenResponseDto.class).block();

        hostName = "http://localhost:" + port;
        requestHeaders = new HttpHeaders();
        requestHeaders.add("authorization", String.format("%s %s", result.getTokenType(), result.getAccessToken()));
    }

    @BeforeEach
    public void initUser() {
        HttpEntity request = new HttpEntity<>(requestHeaders);
        ResponseEntity<UserDto> response = restTemplate.exchange(
                hostName + "/api/user/info",
                HttpMethod.GET,
                request,
                UserDto.class
        );
        List<FolderEntity> folderEntities = folderDao.findAll();
        folderDao.saveAll(folderEntities.stream()
                .filter(folderEntity -> folderEntity.getOwnerId() != null && folderEntity.getOwnerId().equals("test-owner-id"))
                .map(folderEntity -> {
                    folderEntity.setOwnerId(response.getBody().getId());
                    return folderEntity;
                })
                .collect(Collectors.toList()));
        List<ProjectEntity> projectEntities = projectDao.findAll();
        projectDao.saveAll(projectEntities.stream()
                .filter(projectEntity -> projectEntity.getOwnerId() != null && projectEntity.getOwnerId().equals("test-owner-id"))
                .map(projectEntity -> {
                    projectEntity.setOwnerId(response.getBody().getId());
                    return projectEntity;
                })
                .collect(Collectors.toList()));
    }

    @BeforeEach
    public void setupContext() {
        try {
            projectContentDao.save(new ProjectContentEntity(1L, GlobalDefs.UPLOAD_DIRNAME, "test-hash.jpg"), "test-hash-file".getBytes());
            projectContentDao.save(new ProjectContentEntity(11L, GlobalDefs.UPLOAD_DIRNAME, "test-hash.jpg"), "test-hash-file".getBytes());
            projectContentDao.save(new ProjectContentEntity(12L, GlobalDefs.UPLOAD_DIRNAME, "test-hash.jpg"), "test-hash-file".getBytes());
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    @AfterEach
    public void downContext() throws IOException {
        FileUtils.deleteDirectory(new File(appConfig.getProjectConfig().getPath()));
    }

    @Test
    public void testListFile() {
        HttpEntity request = new HttpEntity<>(requestHeaders);

        ResponseEntity<UploadFileDto[]> response = restTemplate.exchange(
                hostName + "/api/project/{id}/file",
                HttpMethod.GET,
                request,
                UploadFileDto[].class,
                1L
        );
        Assert.assertEquals(HttpStatus.OK, response.getStatusCode());
        Assert.assertTrue(Arrays.stream(response.getBody()).allMatch(uploadFileDto -> uploadFileDto.getLastModified() > 0));
    }

    @Test
    public void testDownloadFiles() {
        HttpEntity request = new HttpEntity<>(requestHeaders);

        ResponseEntity<byte[]> response = restTemplate.exchange(
                hostName + "/api/project/{id}/file/download",
                HttpMethod.GET,
                request,
                byte[].class,
                1L
        );
        Assert.assertEquals(HttpStatus.OK, response.getStatusCode());
        Assert.assertTrue(response.getBody().length > 0);
    }

    @Test
    public void testUploadFile() throws IOException {
        Long projectId = 1L;
        InputStream inputStream = new ByteArrayInputStream("text file data".getBytes());
        MultipartFile file = new MockMultipartFile("text-file.txt", "text-file.txt", MediaType.MULTIPART_FORM_DATA_VALUE, inputStream);
        HttpEntity<?> archive = new HttpEntity<>(file.getResource());

        MultiValueMap<String, Object> requestBody = new LinkedMultiValueMap<>();

        requestBody.add("file", archive);

        HttpHeaders currentHeaders = new HttpHeaders();
        currentHeaders.putAll(requestHeaders);
        currentHeaders.setContentType(MediaType.MULTIPART_FORM_DATA);
        HttpEntity<MultiValueMap> requestExtra = new HttpEntity<>(requestBody, currentHeaders);

        ResponseEntity<UploadFileDto> response = restTemplate.exchange(
                hostName + "/api/project/{id}/file",
                HttpMethod.POST,
                requestExtra,
                UploadFileDto.class,
                projectId
        );
        Assert.assertEquals(HttpStatus.OK, response.getStatusCode());
        File uploadedFile = new File(String.format("%s/%d/%s/%s", appConfig.getProjectConfig().getPath(), projectId, GlobalDefs.UPLOAD_DIRNAME, file.getOriginalFilename()));
        Assert.assertTrue(uploadedFile.exists());
    }

    @Test
    public void testUploadFile_SharedGroup_Success() throws IOException {
        Mockito.when(keyCloakServiceClient.getGroupAll(any()))
                .thenReturn(List.of(new GroupDto("group-id", "test", "path")));
        Mockito.when(keyCloakServiceClient.getGroupMemberAll(any()))
                .thenReturn(List.of(new com.goodt.drive.supeditor.application.dto.keycloak.UserDto("test-owner-id-2", "", true, true)));

        Long projectId = 12L;
        InputStream inputStream = new ByteArrayInputStream("text file data".getBytes());
        MultipartFile file = new MockMultipartFile("text-file.txt", "text-file.txt", MediaType.MULTIPART_FORM_DATA_VALUE, inputStream);
        HttpEntity<?> archive = new HttpEntity<>(file.getResource());

        MultiValueMap<String, Object> requestBody = new LinkedMultiValueMap<>();

        requestBody.add("file", archive);

        HttpHeaders currentHeaders = new HttpHeaders();
        currentHeaders.putAll(requestHeaders);
        currentHeaders.setContentType(MediaType.MULTIPART_FORM_DATA);
        HttpEntity<MultiValueMap> requestExtra = new HttpEntity<>(requestBody, currentHeaders);

        ResponseEntity<UploadFileDto> response = restTemplate.exchange(
                hostName + "/api/project/{id}/file",
                HttpMethod.POST,
                requestExtra,
                UploadFileDto.class,
                projectId
        );
        Assert.assertEquals(HttpStatus.OK, response.getStatusCode());
    }

    @Test
    public void testUploadFile_SharedGroup_Fail() throws IOException {
        Mockito.when(keyCloakServiceClient.getGroupAll(any()))
                .thenReturn(List.of(new GroupDto("group-id", "test", "path")));
        Mockito.when(keyCloakServiceClient.getGroupMemberAll(any()))
                .thenReturn(List.of(new com.goodt.drive.supeditor.application.dto.keycloak.UserDto("test-owner-id-2", "", true, true)));

        Long projectId = 11L;
        InputStream inputStream = new ByteArrayInputStream("text file data".getBytes());
        MultipartFile file = new MockMultipartFile("text-file.txt", "text-file.txt", MediaType.MULTIPART_FORM_DATA_VALUE, inputStream);
        HttpEntity<?> archive = new HttpEntity<>(file.getResource());

        MultiValueMap<String, Object> requestBody = new LinkedMultiValueMap<>();

        requestBody.add("file", archive);

        HttpHeaders currentHeaders = new HttpHeaders();
        currentHeaders.putAll(requestHeaders);
        currentHeaders.setContentType(MediaType.MULTIPART_FORM_DATA);
        HttpEntity<MultiValueMap> requestExtra = new HttpEntity<>(requestBody, currentHeaders);

        ResponseEntity<String> response = restTemplate.exchange(
                hostName + "/api/project/{id}/file",
                HttpMethod.POST,
                requestExtra,
                String.class,
                projectId
        );
        Assert.assertEquals(HttpStatus.FORBIDDEN, response.getStatusCode());
    }

    @Test
    public void testDeletefile() {

        HttpEntity request = new HttpEntity<>(requestHeaders);

        ResponseEntity<OperationResult> response = restTemplate.exchange(
                hostName + "/api/project/{id}/file/{hash}",
                HttpMethod.DELETE,
                request,
                OperationResult.class,
                1L,
                "e8897331d79df297cb4a291872d674c6"
        );
        Assert.assertEquals(HttpStatus.OK, response.getStatusCode());
        Assert.assertEquals(true, response.getBody().getResult());
    }

    @Test
    public void testDeletefile_SharedGroup_Success() {
        Mockito.when(keyCloakServiceClient.getGroupAll(any()))
                .thenReturn(List.of(new GroupDto("group-id", "test", "path")));
        Mockito.when(keyCloakServiceClient.getGroupMemberAll(any()))
                .thenReturn(List.of(new com.goodt.drive.supeditor.application.dto.keycloak.UserDto("test-owner-id-2", "", true, true)));

        HttpEntity request = new HttpEntity<>(requestHeaders);

        ResponseEntity<OperationResult> response = restTemplate.exchange(
                hostName + "/api/project/{id}/file/{hash}",
                HttpMethod.DELETE,
                request,
                OperationResult.class,
                12L,
                "e8897331d79df297cb4a291872d674c6"
        );
        Assert.assertEquals(HttpStatus.OK, response.getStatusCode());
    }

    @Test
    public void testDeletefile_SharedGroup_Fail() {
        Mockito.when(keyCloakServiceClient.getGroupAll(any()))
                .thenReturn(List.of(new GroupDto("group-id", "test", "path")));
        Mockito.when(keyCloakServiceClient.getGroupMemberAll(any()))
                .thenReturn(List.of(new com.goodt.drive.supeditor.application.dto.keycloak.UserDto("test-owner-id-2", "", true, true)));

        HttpEntity request = new HttpEntity<>(requestHeaders);

        ResponseEntity<String> response = restTemplate.exchange(
                hostName + "/api/project/{id}/file/{hash}",
                HttpMethod.DELETE,
                request,
                String.class,
                11L,
                "e8897331d79df297cb4a291872d674c6"
        );
        Assert.assertEquals(HttpStatus.FORBIDDEN, response.getStatusCode());
    }
}
