package com.goodt.drive.supeditor.application.rest;

import org.apache.commons.io.FileUtils;
import org.junit.Assert;
import org.junit.jupiter.api.*;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.boot.test.web.client.TestRestTemplate;
import org.springframework.boot.web.server.LocalServerPort;
import org.springframework.http.*;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.web.reactive.function.BodyInserters;
import org.springframework.web.reactive.function.client.WebClient;

import java.io.File;
import java.io.IOException;
import java.util.List;
import java.util.stream.Collectors;

import com.goodt.drive.supeditor.GlobalDefs;
import com.goodt.drive.supeditor.application.configuration.AppConfig;
import com.goodt.drive.supeditor.application.dao.FolderDao;
import com.goodt.drive.supeditor.application.dao.ProjectDao;
import com.goodt.drive.supeditor.application.data.OperationResult;
import com.goodt.drive.supeditor.application.dto.ProjectPageDto;
import com.goodt.drive.supeditor.application.dto.TokenResponseDto;
import com.goodt.drive.supeditor.application.dto.UserDto;
import com.goodt.drive.supeditor.application.dto.keycloak.GroupDto;
import com.goodt.drive.supeditor.application.model.entities.FolderEntity;
import com.goodt.drive.supeditor.application.model.entities.ProjectContentEntity;
import com.goodt.drive.supeditor.application.model.entities.ProjectEntity;
import com.goodt.drive.supeditor.application.dao.ProjectContentDao;
import com.goodt.drive.supeditor.application.services.clients.IKeyCloakServiceClient;
import com.goodt.drive.supeditor.utils.IntegrationalTestBase;

import static org.mockito.ArgumentMatchers.any;

@SpringBootTest(webEnvironment = SpringBootTest.WebEnvironment.RANDOM_PORT)
@ActiveProfiles("integrationaltests")
@TestInstance(TestInstance.Lifecycle.PER_CLASS)
public class TestProjectPageController extends IntegrationalTestBase {

    @LocalServerPort
    private int port;

    @Autowired
    private TestRestTemplate restTemplate;

    private String hostName = "http://localhost:" + port;
    private HttpHeaders requestHeaders;

    @Autowired
    private ProjectContentDao projectContentDao;

    @Autowired
    private ProjectDao projectDao;

    @Autowired
    private FolderDao folderDao;

    @Autowired
    private AppConfig appConfig;

    @MockBean
    private IKeyCloakServiceClient keyCloakServiceClient;

    @BeforeAll
    public void setup() {
        WebClient client = WebClient.builder()
                .baseUrl(appConfig.getKeyCloakConfig().getBaseUrl())
                .build();
        TokenResponseDto result = client.post().uri(appConfig.getKeyCloakConfig().getTokenUrl())
                .header(HttpHeaders.CONTENT_TYPE, MediaType.APPLICATION_FORM_URLENCODED_VALUE)
                .body(BodyInserters.fromFormData("client_id", appConfig.getKeyCloakConfig().getClientId())
                        .with("client_secret", appConfig.getKeyCloakConfig().getClientSecret())
                        .with("grant_type", "password")
.with("username", appConfig.getKeyCloakConfig().getUsername())
                        .with("password", appConfig.getKeyCloakConfig().getPassword()))
                .retrieve().bodyToMono(TokenResponseDto.class).block();

        hostName = "http://localhost:" + port;
        requestHeaders = new HttpHeaders();
        requestHeaders.add("authorization", String.format("%s %s", result.getTokenType(), result.getAccessToken()));
    }

    @BeforeEach
    public void setupContext() {
        try {
            projectContentDao.save(new ProjectContentEntity(1L, GlobalDefs.PAGES_DIRNAME, "conf1-page1.json"), "conf page data 1".getBytes());
            projectContentDao.save(new ProjectContentEntity(1L, GlobalDefs.PAGES_DIRNAME, "conf1-page2.json"), "conf page data 2".getBytes());
            projectContentDao.save(new ProjectContentEntity(1L, GlobalDefs.PAGES_DIRNAME, "conf1-page3.json"), "conf page data 3".getBytes());
            projectContentDao.save(new ProjectContentEntity(11L, GlobalDefs.PAGES_DIRNAME, "conf1-page3.json"), "conf page data 3".getBytes());
            projectContentDao.save(new ProjectContentEntity(12L, GlobalDefs.PAGES_DIRNAME, "conf1-page3.json"), "conf page data 3".getBytes());
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    @AfterEach
    public void downContext() throws IOException {
        FileUtils.deleteDirectory(new File(appConfig.getProjectConfig().getPath()));
    }

    @BeforeEach
    public void initUser() {
        HttpEntity request = new HttpEntity<>(requestHeaders);
        ResponseEntity<UserDto> response = restTemplate.exchange(
                hostName + "/api/user/info",
                HttpMethod.GET,
                request,
                UserDto.class
        );
        List<FolderEntity> folderEntities = folderDao.findAll();
        folderDao.saveAll(folderEntities.stream()
                .filter(folderEntity -> folderEntity.getOwnerId() != null && folderEntity.getOwnerId().equals("test-owner-id"))
                .map(folderEntity -> {
                    folderEntity.setOwnerId(response.getBody().getId());
                    return folderEntity;
                })
                .collect(Collectors.toList()));
        List<ProjectEntity> projectEntities = projectDao.findAll();
        projectDao.saveAll(projectEntities.stream()
                .filter(projectEntity -> projectEntity.getOwnerId() != null && projectEntity.getOwnerId().equals("test-owner-id"))
                .map(projectEntity -> {
                    projectEntity.setOwnerId(response.getBody().getId());
                    return projectEntity;
                })
                .collect(Collectors.toList()));
    }

    @Test
    public void testGetById() {
        HttpEntity request = new HttpEntity<>(requestHeaders);

        ResponseEntity<ProjectPageDto> response = restTemplate.exchange(
                hostName + "/api/project/{project_id}/page/{page_id}",
                HttpMethod.GET,
                request,
                ProjectPageDto.class,
                1L,
                "conf1-page1"
        );
        Assert.assertEquals(HttpStatus.OK, response.getStatusCode());
        ProjectPageDto dto = response.getBody();
        Assert.assertTrue(dto.getProjectId().equals(1L));
        Assert.assertTrue(dto.getPageId().equals("conf1-page1"));
    }

    @Test
    public void testPreview_NotFound() {
        HttpEntity request = new HttpEntity<>(requestHeaders);

        ResponseEntity<String> response = restTemplate.exchange(
                "http://localhost:" + port + "/api/project/{project_id}/page/{page_id}.json",
                HttpMethod.GET,
                request,
                String.class,
                1L,
                "conf9999-page999"
        );
        Assert.assertEquals(HttpStatus.NOT_FOUND, response.getStatusCode());
    }

    @Test
    public void testCreate() {
        ProjectPageDto dto = new ProjectPageDto(null, "conf1-page1-test-new", "new test data create");
        HttpEntity request = new HttpEntity<>(dto, requestHeaders);

        ResponseEntity<ProjectPageDto> response = restTemplate.exchange(
                hostName + "/api/project/{project_id}/page",
                HttpMethod.POST,
                request,
                ProjectPageDto.class,
                2L
        );
        Assert.assertEquals(HttpStatus.OK, response.getStatusCode());
        Assert.assertEquals("new test data create", response.getBody().getData());
    }

    @Test
    public void testCreate_SharedGroup_Success() {
        Mockito.when(keyCloakServiceClient.getGroupAll(any()))
                .thenReturn(List.of(new GroupDto("group-id", "test", "path")));
        Mockito.when(keyCloakServiceClient.getGroupMemberAll(any()))
                .thenReturn(List.of(new com.goodt.drive.supeditor.application.dto.keycloak.UserDto("test-owner-id-2", "", true, true)));

        ProjectPageDto dto = new ProjectPageDto(null, "conf1-page1-test-new", "new test data create");
        HttpEntity request = new HttpEntity<>(dto, requestHeaders);

        ResponseEntity<ProjectPageDto> response = restTemplate.exchange(
                hostName + "/api/project/{project_id}/page",
                HttpMethod.POST,
                request,
                ProjectPageDto.class,
                12L
        );
        Assert.assertEquals(HttpStatus.OK, response.getStatusCode());
    }

    @Test
    public void testCreate_SharedGroup_Fail() {
        Mockito.when(keyCloakServiceClient.getGroupAll(any()))
                .thenReturn(List.of(new GroupDto("group-id", "test", "path")));
        Mockito.when(keyCloakServiceClient.getGroupMemberAll(any()))
                .thenReturn(List.of(new com.goodt.drive.supeditor.application.dto.keycloak.UserDto("test-owner-id-2", "", true, true)));

        ProjectPageDto dto = new ProjectPageDto(null, "conf1-page1-test-new", "new test data create");
        HttpEntity request = new HttpEntity<>(dto, requestHeaders);

        ResponseEntity<String> response = restTemplate.exchange(
                hostName + "/api/project/{project_id}/page",
                HttpMethod.POST,
                request,
                String.class,
                11L
        );
        Assert.assertEquals(HttpStatus.FORBIDDEN, response.getStatusCode());
    }

    @Test
    public void testUpdate() {
        ProjectPageDto dto = new ProjectPageDto(null, null, "new test data");
        HttpEntity request = new HttpEntity<>(dto, requestHeaders);

        ResponseEntity<ProjectPageDto> response = restTemplate.exchange(
                hostName + "/api/project/{project_id}/page/{page_id}",
                HttpMethod.PUT,
                request,
                ProjectPageDto.class,
                1L,
                "conf1-page2"
        );
        Assert.assertEquals(HttpStatus.OK, response.getStatusCode());
        Assert.assertEquals("new test data", response.getBody().getData());
    }

    @Test
    public void testUpdate_SharedGroup_Success() {
        Mockito.when(keyCloakServiceClient.getGroupAll(any()))
                .thenReturn(List.of(new GroupDto("group-id", "test", "path")));
        Mockito.when(keyCloakServiceClient.getGroupMemberAll(any()))
                .thenReturn(List.of(new com.goodt.drive.supeditor.application.dto.keycloak.UserDto("test-owner-id-2", "", true, true)));

        ProjectPageDto dto = new ProjectPageDto(null, null, "new test data");
        HttpEntity request = new HttpEntity<>(dto, requestHeaders);

        ResponseEntity<ProjectPageDto> response = restTemplate.exchange(
                hostName + "/api/project/{project_id}/page/{page_id}",
                HttpMethod.PUT,
                request,
                ProjectPageDto.class,
                12L,
                "conf1-page2"
        );
        Assert.assertEquals(HttpStatus.OK, response.getStatusCode());
    }

    @Test
    public void testUpdate_SharedGroup_Fail() {
        Mockito.when(keyCloakServiceClient.getGroupAll(any()))
                .thenReturn(List.of(new GroupDto("group-id", "test", "path")));
        Mockito.when(keyCloakServiceClient.getGroupMemberAll(any()))
                .thenReturn(List.of(new com.goodt.drive.supeditor.application.dto.keycloak.UserDto("test-owner-id-2", "", true, true)));

        ProjectPageDto dto = new ProjectPageDto(null, null, "new test data");
        HttpEntity request = new HttpEntity<>(dto, requestHeaders);

        ResponseEntity<String> response = restTemplate.exchange(
                hostName + "/api/project/{project_id}/page/{page_id}",
                HttpMethod.PUT,
                request,
                String.class,
                11L,
                "conf1-page2"
        );
        Assert.assertEquals(HttpStatus.FORBIDDEN, response.getStatusCode());
    }

    @Test
    public void testDelete() {
        HttpEntity request = new HttpEntity<>(requestHeaders);

        ResponseEntity<OperationResult> response = restTemplate.exchange(
                hostName + "/api/project/{project_id}/page/{page_id}",
                HttpMethod.DELETE,
                request,
                OperationResult.class,
                1L,
                "conf1-page3"
        );
        Assert.assertEquals(HttpStatus.OK, response.getStatusCode());
        Assert.assertNotNull(response.getBody().getResult());
    }

    @Test
    public void testDeleteSharedGroup_Success() {
        Mockito.when(keyCloakServiceClient.getGroupAll(any()))
                .thenReturn(List.of(new GroupDto("group-id", "test", "path")));
        Mockito.when(keyCloakServiceClient.getGroupMemberAll(any()))
                .thenReturn(List.of(new com.goodt.drive.supeditor.application.dto.keycloak.UserDto("test-owner-id-2", "", true, true)));

        HttpEntity request = new HttpEntity<>(requestHeaders);

        ResponseEntity<OperationResult> response = restTemplate.exchange(
                hostName + "/api/project/{project_id}/page/{page_id}",
                HttpMethod.DELETE,
                request,
                OperationResult.class,
                12L,
                "conf1-page3"
        );
        Assert.assertEquals(HttpStatus.OK, response.getStatusCode());
    }

    @Test
    public void testDeleteSharedGroup_Fail() {
        Mockito.when(keyCloakServiceClient.getGroupAll(any()))
                .thenReturn(List.of(new GroupDto("group-id", "test", "path")));
        Mockito.when(keyCloakServiceClient.getGroupMemberAll(any()))
                .thenReturn(List.of(new com.goodt.drive.supeditor.application.dto.keycloak.UserDto("test-owner-id-2", "", true, true)));

        HttpEntity request = new HttpEntity<>(requestHeaders);

        ResponseEntity<String> response = restTemplate.exchange(
                hostName + "/api/project/{project_id}/page/{page_id}",
                HttpMethod.DELETE,
                request,
                String.class,
                11L,
                "conf1-page3"
        );
        Assert.assertEquals(HttpStatus.FORBIDDEN, response.getStatusCode());
    }
}
