/**
 * @typedef {Object} QueryOptions
 * @property {IQuerySchema} query       query
 * @property {Object} dimensionList     dimensionList
 */
export type QueryOptions = {
    /**
     * query
     */
    query: IQuerySchema;
    /**
     * dimensionList
     */
    dimensionList: Record<string, any>;
};

/**
 * @typedef {Object} MetricOptions
 * @param {String} name?        название metric
 * @param {String} type?        тип metric {@link Query#METRIC_TYPE}
 * @param {String} field?       название field
 */
export type MetricOptions = {
    name?: string;
    type?: `${Query.METRIC_TYPE}`;
    field?: string;
};
/**
 * @typedef {Object} DimensionOptions
 * @param {String} name?        название metric
 * @param {String} field?       название field
 */
export type DimensionOptions = {
    name?: string;
    field?: string;
};
/**
 * @typedef {Object} FilterOptions
 * @param {String} name?        название metric/dimension/field
 * @param {String} type?        тип filter {@link Query#FILTER_TYPE}
 * @param {Array} value?       массив значений
 */
export type FilterOptions = {
    name?: string;
    type?: `${Query.FILTER_TYPE}`;
    value?: any[];
};
/**
 * @typedef {Object} SortOptions
 * @param {String} name?       название metric/dimension
 * @param {String} type?       тип sort {@link Query#SORT_TYPE}
 */
export type SortOptions = {
    name?: string;
    type?: `${Query.SORT_TYPE}`;
};

type Filter = Record<string, any>;
type Sort = Record<string, any>;
type Dimension = Record<string, any>;
type Metric = Record<string, any>;

export type IQuerySchema = {
    [field in Query.KEY]?: string;
};

/**
 * @class
 */
export class Query {
    /**
     * Клонирует json-like obj
     * @param {Object} obj
     * @return {Object}
     */
    static clone(obj: any): any;
    /**
     * Создает/валидирует query
     * @param {IQuerySchema} [query=null]    query для валидации
     */
    static createQuery(query?: IQuerySchema): IQuerySchema;
    /**
     * Создает/мутирует metric
     * @param {MetricOptions} options   опции
     * @param {Object} [metric=null]           metric для мутации
     */
    static createMetric({ name, type, field }: MetricOptions, metric?: Metric | null): Metric;
    /**
     * Создает/мутирует dimension
     * @param {DimensionOptions} options    опции
     * @param {Object} [dimension=null]            dimension для мутации
     */
    static createDimension(
        { name, field }: DimensionOptions,
        dimension?: Dimension | null
    ): Dimension;
    /**
     * Создает/мутирует filter
     * @param {FilterOptions} options    опции
     * @param {Object} [filter=null]            filter для мутации
     */
    static createFilter({ name, type, value }: FilterOptions, filter?: Filter | null): Filter;
    /**
     * @typedef {Object} SortOptions
     * @param {String} name?       название metric/dimension
     * @param {String} type?       тип sort {@link Query#SORT_TYPE}
     */
    /**
     * Создает/мутирует sort
     * @param {SortOptions} options     опции
     * @param {Object} [sort=null]             sort для мутации
     */
    static createSort({ name, type }: SortOptions, sort?: Sort | null): any;
    /**
     * Возвращает имя metric
     * @param {Object} metric
     * @return {String}
     */
    static getMetricName(metric: any): string;
    /**
     * Возвращает тип metric
     * @param {Object} metric
     * @return {String}
     */
    static getMetricType(metric: any): string;
    /**
     * Возвращает field metric
     * @param {Object} metric
     * @return {String}
     */
    static getMetricField(metric: any): string;
    /**
     * Возвращает имя dimension
     * @param {Object} dimension
     * @return {String}
     */
    static getDimensionName(dimension: any): string;
    /**
     * Возвращает field dimension
     * @param {Object} dimension
     * @return {String}
     */
    static getDimensionField(dimension: any): string;
    /**
     * Возвращает имя filter
     * @param {Object} filter
     * @return {String}
     */
    static getFilterName(filter: any): string;
    /**
     * Возвращает тип filter
     * @param {Object} filter
     * @return {String}
     */
    static getFilterType(filter: any): string;
    /**
     * Возвращает value filter
     * @param {Object} filter
     * @return {String}
     */
    static getFilterValue(filter: any): string;
    /**
     * Возвращает имя sort
     * @param {Object} sort
     * @return {String}
     */
    static getSortName(sort: any): string;
    /**
     * Возвращает тип sort
     * @param {Object} sort
     * @return {String}
     */
    static getSortType(sort: any): string;
    /**
     * Добавляет/заменяет filter в query
     * @param {IQuerySchema} query      query
     * @param {Object} filter           filter
     * @return {IQuerySchema}
     */
    static queryInsertUpdateFilter(query: IQuerySchema, filter: any): any;
    /**
     * Удаляет filter из query
     * @param {IQuerySchema} query      query
     * @param {String} name             название metric/dimension
     * @return {IQuerySchema}
     */
    static queryRemoveFilter(query: any, name: string): any;
    /**
     * Удаляет все filter из query
     * @param {IQuerySchema} query    query
     * @return {IQuerySchema}
     */
    static queryRemoveAllFilters(query: any): any;
    /**
     * Заменяет Query.FILTER_VALUE value всех фильтров
     * @param {IQuerySchema} query
     * @return {IQuerySchema}
     */
    static queryReplaceAllFilterValues(query: IQuerySchema): IQuerySchema;
    /**
     * Возвращает field name dimension
     * @param {IQuerySchema} query      query
     * @param {String} name             dimension name
     * @return {String}                 field name или null если dimension нет
     */
    static queryGetDimensionField(query: IQuerySchema, name: string): string;
    /**
     * Добавляет/заменяет dimension в query
     * @param {IQuerySchema} query      query
     * @param {Object} dimension        dimension
     * @return {IQuerySchema}
     */
    static queryInsertUpdateDimension(query: IQuerySchema, dimension: any): any;
    /**
     * Возвращает массив имен field
     * @param {IQuerySchema} query      query
     * @return {Array}                  массив имен field
     */
    static queryFieldNames(query: IQuerySchema): any[];
    /**
     * Возвращает массив имен metric в query
     * @param {IQuerySchema} query      query
     * @return {Array}                  массив имен metric
     */
    static queryMetricNames(query: IQuerySchema): any[];
    /**
     * Возвращает массив имен dimension в query
     * @param {IQuerySchema} query      query
     * @return {Array}                  массив имен dimension
     */
    static queryDimensionNames(query: IQuerySchema): any[];
    /**
     * Валидирует query
     * @param {IQuerySchema} query
     * @return {Boolean}
     */
    static validateQuery(query: IQuerySchema): boolean;
    /**
     * Валидирует from
     * @param {IQuerySchema} query
     * @return {Boolean}
     */
    static validateFrom(query: IQuerySchema): boolean;
    /**
     * Валидирует metric
     * @param {Object} metric
     * @return {Boolean}
     */
    static validateMetric(metric: IQuerySchema): boolean;
    /**
     * Валидирует dimension
     * @param {Object} dimension
     * @return {Boolean}
     */
    static validateDimension(dimension: any): boolean;
    /**
     * Валидирует filter
     * @param {Object} filter
     * @return {Boolean}
     */
    static validateFilter(filter: any): boolean;
    /**
     * Валидирует sort
     * @param {Object} sort
     * @return {Boolean}
     */
    static validateSort(sort: any): boolean;
    /**
     * @private Валидирует metric/dimension/filter/sort like obj
     * @param {Object} obj
     * @return {Boolean}
     */
    private static _validateObj;
    /**
     * @typedef {Object} QueryOptions
     * @property {IQuerySchema} query       query
     * @property {Object} dimensionList     dimensionList
     */
    /**
     * Constructor
     * @param {QueryOptions} options
     */
    constructor({ query, dimensionList }: QueryOptions);
    query: IQuerySchema;
    dimensionList: any;
    states: any[];
    /**
     * Активирует dimension из списка dimensionList
     * @param {Array} names     массив названий dimension
     */
    enableDimensions(names: string[]): void;
    /**
     * Формирует query с текущим state всех активных dimension
     * @return {IQuerySchema}     query
     */
    buildQuery(): IQuerySchema;
    /**
     * Проверяет, сущ. ли state для dimensionName
     * @param {String} dimensionName    название dimension
     * @return {Boolean}
     */
    dimensionStateExists(dimensionName: string): boolean;
    /**
     * Возвращает true, если dimension state первый
     * @param {String} dimensionName    название dimension
     * @return {Boolean}
     */
    dimensionStateIsFirst(dimensionName: string): boolean;
    /**
     * Возвращает true, если dimension state последний
     * @param {String} dimensionName    название dimension
     * @return {Boolean}
     */
    dimensionStateIsLast(dimensionName: string): boolean;
    /**
     * Сдвигает тек. dimension state на шаг вперед
     * @param {String} dimensionName    название dimension
     * @param {Array} filterValue      значения filter
     * @param {String} filterType       тип filter {@link Query#FILTER_TYPE}
     * @return {Boolean}                true если удалось; false нет
     */
    dimensionStateGoNext(dimensionName: string, filterValue: any[], filterType: string): boolean;
    /**
     * Сдвигает тек. dimension state на шаг назад
     * @param {String} dimensionName    название dimension
     * @return {Boolean}                true если удалось; false нет
     */
    dimensionStateGoPrev(dimensionName: string): boolean;
}

export namespace Query {
    namespace FORMAT {
        const DATE: string;
        const TIME: string;
        const TIMESTAMP: string;
    }
    enum KEY {
        FIELDS = '$fields',
        FROM = '$from',
        METRICS = '$metrics',
        DIMENSIONS = '$dimensions',
        FILTERS = '$filters',
        SORT = '$sort'
    }
    enum SORT_TYPE {
        ASC = 'asc',
        DESC = 'desc'
    }
    enum METRIC_TYPE {
        VALUE = '$v',
        EXPRESSION = '$exp',
        AVG = '$avg',
        COUNT = '$count',
        MAX = '$max',
        MIN = '$min',
        SUM = '$sum',
        GROUP_CONCAT = '$gc',
        GROUP_CONCAT_UNIQ = '$gcu'
    }
    enum FILTER_TYPE {
        IS = '$is',
        IS_NOT = '$isn',
        EQ = '$eq',
        EQ_NOT = '$eqn',
        LESS = '$lt',
        LESS_EQ = '$ltq',
        GREATER = '$gt',
        GREATER_EQ = '$gtq',
        IN = '$in',
        IN_NOT = '$inn',
        BETWEEN = '$btw',
        BETWEEN_NOT = '$btwn',
        LIKE = '$lk'
    }
    namespace FILTER_VALUE {
        const YEAR_ROLLING: string;
        const YEAR_START: string;
        const YEAR_END: string;
        const YEAR_PREV_START: string;
        const YEAR_PREV_END: string;
        const QUARTER_START: string;
        const QUARTER_END: string;
        const QUARTER_PY_START: string;
        const QUARTER_PY_END: string;
        const QUARTER_PREV_START: string;
        const QUARTER_PREV_END: string;
        const MONTH_ROLLING: string;
        const MONTH_START: string;
        const MONTH_END: string;
        const MONTH_PY_START: string;
        const MONTH_PY_END: string;
        const MONTH_PREV_START: string;
        const MONTH_PREV_END: string;
        const WEEK_ROLLING: string;
        const WEEK_START: string;
        const WEEK_END: string;
        const WEEK_PY_START: string;
        const WEEK_PY_END: string;
        const WEEK_PREV_START: string;
        const WEEK_PREV_END: string;
        const TODAY: string;
        const YESTERDAY: string;
    }
    type FILTER_VALUE = string;
    namespace FILTER_VALUE_HANDLER {
        export function YEAR_ROLLING_1(): any;
        export { YEAR_ROLLING_1 as YEAR_ROLLING };
        export function YEAR_START_1(): any;
        export { YEAR_START_1 as YEAR_START };
        export function YEAR_END_1(): any;
        export { YEAR_END_1 as YEAR_END };
        export function YEAR_PREV_START_1(): any;
        export { YEAR_PREV_START_1 as YEAR_PREV_START };
        export function YEAR_PREV_END_1(): any;
        export { YEAR_PREV_END_1 as YEAR_PREV_END };
        export function QUARTER_START_1(): any;
        export { QUARTER_START_1 as QUARTER_START };
        export function QUARTER_END_1(): any;
        export { QUARTER_END_1 as QUARTER_END };
        export function QUARTER_PY_START_1(): any;
        export { QUARTER_PY_START_1 as QUARTER_PY_START };
        export function QUARTER_PY_END_1(): any;
        export { QUARTER_PY_END_1 as QUARTER_PY_END };
        export function QUARTER_PREV_START_1(): any;
        export { QUARTER_PREV_START_1 as QUARTER_PREV_START };
        export function QUARTER_PREV_END_1(): any;
        export { QUARTER_PREV_END_1 as QUARTER_PREV_END };
        export function MONTH_ROLLING_1(): any;
        export { MONTH_ROLLING_1 as MONTH_ROLLING };
        export function MONTH_START_1(): any;
        export { MONTH_START_1 as MONTH_START };
        export function MONTH_END_1(): any;
        export { MONTH_END_1 as MONTH_END };
        export function MONTH_PY_START_1(): any;
        export { MONTH_PY_START_1 as MONTH_PY_START };
        export function MONTH_PY_END_1(): any;
        export { MONTH_PY_END_1 as MONTH_PY_END };
        export function MONTH_PREV_START_1(): any;
        export { MONTH_PREV_START_1 as MONTH_PREV_START };
        export function MONTH_PREV_END_1(): any;
        export { MONTH_PREV_END_1 as MONTH_PREV_END };
        export function WEEK_ROLLING_1(): any;
        export { WEEK_ROLLING_1 as WEEK_ROLLING };
        export function WEEK_START_1(): any;
        export { WEEK_START_1 as WEEK_START };
        export function WEEK_END_1(): any;
        export { WEEK_END_1 as WEEK_END };
        export function WEEK_PY_START_1(): any;
        export { WEEK_PY_START_1 as WEEK_PY_START };
        export function WEEK_PY_END_1(): any;
        export { WEEK_PY_END_1 as WEEK_PY_END };
        export function WEEK_PREV_START_1(): any;
        export { WEEK_PREV_START_1 as WEEK_PREV_START };
        export function WEEK_PREV_END_1(): any;
        export { WEEK_PREV_END_1 as WEEK_PREV_END };
        export function TODAY_1(): any;
        export { TODAY_1 as TODAY };
        export function YESTERDAY_1(): any;
        export { YESTERDAY_1 as YESTERDAY };
    }
    type FILTER_VALUE_HANDLER = string;
    const FILTER_TYPE_VALUE_VALIDATION: {
        [x: string]: (v: any) => boolean;
    };
}
