// @ts-ignore
import { AxiosInstance, AxiosRequestConfig, CancelTokenSource } from 'axios';
import { Http } from './Http';

export { Dremio } from './constants';
export * as Errors from './errors';
export * from './errors';
export * from './Query';
/**
 * SDKConfig
 */
export type SDKConfig = {
    /**
     */
    host?: string;
    /**
     * нужно ли заменять константы в значениях фильтров
     */
    replaceFilterValues?: boolean;
    /**
     * auth token
     */
    authToken?: string;
    /**
     * admin mode
     */
    admin?: boolean;

    connection?: string;

    beforeRequest?: () => Promise<void>;

    processResponseError?: (error: Error) => Error;

    processResponseResult?: (result: DremioResult) => DremioResult;
};
/**
 * IQuerySchema
 */
export type IQuerySchema = {
    $from: any[];
    $fields: any;
    $metrics: any[];
    $dimensions: any[];
    $filters: any[];
    $sort: any[];
};
/**
 * DremioResult
 */
export type DremioResult = {
    /**
     * кол-во записей
     */
    rowCount: number;
    /**
     * схема данных
     */
    schema: any[];
    /**
     * данные
     */
    rows: Record<string, any>[] | null;
};

/**
 * DremioRootEntity
 */
export type DremioRootEntity = {
    id: string;
    path: any[];
    type: string;
    containerType: string;
};
/**
 * DremioContainerInfo
 */
export type DremioContainerInfo = {
    id: string;
    path: any[];
    name: string;
    entityType: string;
    children: DremioContainerChildInfo[];
};
/**
 * DremioContainerChildInfo
 */
export type DremioContainerChildInfo = {
    id: string;
    path: any[];
    type: string;
    datasetType: string;
    containerType: string;
};
/**
 * DremioEntityInfo
 */
export type DremioEntityInfo = {
    id: string;
    path: any[];
    type: string;
    entityType: string;
    fields: any[];
};
export type DremioPermissions = {
    employeeId: string;
    permissions: Array<any>;
};
export type DremioPermissionInfo = {
    name: string;
};
export type PromiseCancelable<T = void> = Promise<T>;

/**
 * @class
 */
export class SDK {
    /**
     * @type {Http}
     */
    public transport: Http;

    /**
     * @type {SDKConfig}
     */
    config: SDKConfig;

    /**
     * @property {Function}
     */
    readonly processResponseResult?: (result: DremioResult) => DremioResult;

    /**
     * Constructor
     * @param {SDKConfig} [config=null]   config
     */
    constructor(config?: SDKConfig);

    /**
     * Возвращает ссылку на dremio инстанс
     * @return {PromiseCancelable<string>}
     */
    getDremioUrl();

    /**
     * Возвращает список прав текущего пользователя (если задан authToken {@link SDK#config} и он валиден)
     * @return {PromiseCancelable<DremioPermissions>}
     */
    getPermissions(): PromiseCancelable<DremioPermissions>;

    /**
     * Возвращает информацию о правах
     * @return {PromiseCancelable<DremioPermissionInfo[]>}
     */
    getPermissionsInfo(): PromiseCancelable<DremioPermissionInfo[]>;

    /**
     * Возрвщает root сущности dremio
     * @return {PromiseCancelable<DremioRootEntity[]>}
     */
    getRootEntities(): PromiseCancelable<DremioRootEntity[]>;

    /**
     * Возвращает сущность dremio.CatalogEntity по path
     * @param {Array} [path=null]    путь
     * @return {PromiseCancelable<DremioContainerInfo | DremioEntityInfo>}
     */
    getEntityByPath(path?: any[]): PromiseCancelable<DremioContainerInfo | DremioEntityInfo>;

    /**
     * Выполняет запрос
     * @param {IQuerySchema} query      query
     * @param {Number} [offset=0]       offset
     * @param {Number} [limit=10]       limit
     * @param {Boolean} [debug=false]   debug
     * @return {PromiseCancelable<DremioResult>}
     */
    getData(
        query: IQuerySchema,
        offset?: number,
        limit?: number,
        debug?: boolean
    ): PromiseCancelable<DremioResult>;

    /**
     *
     * @param {{ url: string, options?: { method?: string } }} action
     * @param {Record<string, string|number>} [pathParams]
     * @param {Record<string, any>} [params]
     * @param {Record<string, string|number|boolean|null>} [queryParams]
     * @param {Record<string, any>} [options]
     * @return {PromiseCancelable}
     */
    requestAction({
        action,
        pathParams,
        params,
        queryParams,
        options
    }: {
        action: { url: string, options?: { method?: string } },
        pathParams?: Record<string, string>,
        params?: Record<string, any>,
        queryParams?: Record<string, string | number | boolean | null>,
        options?: AxiosRequestConfig
    }): PromiseCancelable<any>;

    /**
     * Отменяет все активные запросы
     */
    cancelActiveRequests(): void;
}
