import { BaseApiService, ApiServiceError } from '@goodt-common/api';

import { IApiServiceMixinInstance, IApiServiceMixinOptions } from '@goodt-common/api';
import { SafeResult } from '@goodt-common/utils';

import {
    DivisionTeamAssignmentDto,
    DivisionPositionDto,
    LegalEntityTeamAssignmentDto,
    LegalEntityAssignmentInfoDto,
    RoleInfoDto,
    EmployeeExtendedInfoDto,
    EmployeeConditionInfoDto,
    DivisionInfoDto,
    DivisionShortInfoDto,
    DivisionTeamRoleContainerDto,
    DivisionTeamRoleRawDto,
    DivisionTeamRoleDto,
    DivisionTeamSuccessorDto,
    AssignmentReadinessDto,
    AssignmentRotationDto,
    PositionExtendedDto,
    LegalEntityDto,
    EmployeeInfoDto,
    PositionRankDto,
    PositionCategoryDto,
    ReasonDto,
    ImportanceCriteriaGroupDto,
    PositionImportanceCriteriaDto,
    RecalculatePositionImportanceCriteriaDto,
    PositionSuccessorDto,
    PositionSuccessorReadinessDto,
    CreatePositionImportanceCriteriaInputDto,
    UpdatePositionImportanceCriteriaInputDto,
    CreatePositionSuccessorInputDto,
    UpdatePositionSuccessorInputDto,
    CreatePositionSuccessorReadinessInputDto,
    UpdatePositionSuccessorReadinessInputDto,
    PositionImportanceDto,
    PositionImportanceBindingDto,
    CreatePositionImportanceBindingInputDto,
    ImportanceCriteriaDto,
    PositionAssignmentDto,
    DivisionGroupDto
} from './dto';

import { IEmployeeContext } from './EmployeeContext';
import { IDivisionContext } from './DivisionContext';
import { ITeamContext } from './TeamContext';

export interface IOrgStructureApiServiceMixinOptions extends IApiServiceMixinOptions {}

export interface IOrgStructureApiServiceMixinInstance extends IApiServiceMixinInstance<TOrgStructureApiService> {}

/**
 *
 */
export class OrgStructureApiService extends BaseApiService {
    /**
     *
     * @param {?number} [employeeId]
     * @return {Promise<SafeResult<EmployeeExtendedInfoDto, Error>>}
     */
    getEmployeeById(employeeId?: number): Promise<SafeResult<EmployeeExtendedInfoDto, ApiServiceError>>;

    /**
     *
     * @param {number[]} employeeIds
     * @return {Promise<SafeResult<EmployeeExtendedInfoDto[], Error>>}
     */
    getEmployeesByIds(employeeIds: number[]): Promise<SafeResult<DivisionTeamAssignmentDto[], ApiServiceError>>;

    /**
     * @description Получение информации о сотрудниках
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/employee/findInfoPost
     *
     *
     * @param {Object} filters
     * @param {number[]} [filters.employeeIds]
     * @param {number} [filters.divisionId]
     * @param {string} [filters.searchToken]
     * @param {string} [filters.legalEntityId]
     * @param {boolean} [filters.withPatronymic]
     * @param {number} [filters.jobTitleId]
     * @param {Object} outputOptions
     * @param {number} [outputOptions.page]
     * @param {number} [outputOptions.size]
     * @param {boolean} [outputOptions.hasPositionAssignment]
     * @param {Object} extraOutputData
     * @param {Function} [extraOutputData.onResult] - function that takes as an argument all information except employees
     * @return {Promise<SafeResult<EmployeeExtendedInfoDto[], Error>>}
     */
    getEmployeesByFilter(
        {
            employeeIds,
            divisionId,
            searchToken,
            legalEntityId,
            withPatronymic,
            jobTitleId
        }: {
            employeeIds?: number[];
            divisionId?: number;
            searchToken?: string;
            legalEntityId?: number;
            withPatronymic?: boolean;
            jobTitleId?: number;
        },
        {
            page,
            size,
            hasPositionAssignment
        }: {
            page?: number;
            size?: number;
            hasPositionAssignment?: boolean;
        },
        {
            onResult
        }: {
            onResult?: ({
                page,
                totalPages,
                totalElements
            }: {
                page: number;
                totalPages: number;
                totalElements: number;
            }) => void;
        }
    ): Promise<SafeResult<EmployeeExtendedInfoDto[], ApiServiceError>>;

    /**
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/employee/getDivisionTeamAssignments
     * @description Получение информации о назначениях в команду подразделения
     *
     * @param {Object} employee
     * @param {number} [employee.employeeId]
     * @param {number} [employee.divisionTeamId]
     *
     * @param {Object} filters
     * @param {boolean} [filters.withClosed]
     *
     * @return {Promise<SafeResult<DivisionTeamAssignmentDto[], Error>>}
     */
    getEmployeeDivisionTeamAssignments(
        {
            employeeId,
            divisionTeamId
        }?: {
            employeeId?: number;
            divisionTeamId?: number;
        },
        { withClosed }?: { withClosed?: boolean }
    ): Promise<SafeResult<DivisionTeamAssignmentDto[], ApiServiceError>>;

    /**
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/employee/getDivisionTeamAssignments
     * @description Получение информации о назначениях в команду подразделения
     *
     * @param {number} [ids]
     * @param {Object} [filters]
     * @param {boolean} [filters.withClosed]
     *
     * @return {Promise<SafeResult<DivisionTeamAssignmentDto[], Error>>}
     */
    getDivisionTeamAssignmentsByIds(
        ids: number[],
        { withClosed }?: { withClosed: boolean }
    ): Promise<SafeResult<DivisionTeamAssignmentDto[], ApiServiceError>>;

    /**
     *
     * @param {number} employeeId
     * @param {number} divisionTeamId
     * @param {boolean} [withChildren=false]
     * @return {Promise<SafeResult<DivisionTeamAssignmentDto[], Error>>}
     */
    async getSubordinatesDivisionsTeamAssignments(
        {
            employeeId,
            divisionTeamId
        }?: {
            employeeId?: number;
            divisionTeamId?: number;
        },
        { withChildren = false }: { withChildren: boolean } = {}
    ): Promise<SafeResult<DivisionTeamAssignmentDto[], ApiServiceError>>;

    /**
     *
     * @param {number} employeeId
     * @param {number} divisionTeamId
     * @return {Promise<SafeResult<DivisionTeamAssignmentDto, Error>>}
     */
    getTeamHeadDivisionTeamAssignment({
        employeeId,
        divisionTeamId
    }?: {
        employeeId?: number;
        divisionTeamId?: number;
    }): Promise<SafeResult<DivisionTeamAssignmentDto, ApiServiceError>>;

    /**
     *
     * @param {number} divisionTeamId
     * @return {Promise<SafeResult<DivisionTeamAssignmentDto[], Error>>}
     */
    getTeamDivisionTeamAssignmentsByTeamId(
        divisionTeamId: number
    ): Promise<SafeResult<DivisionTeamAssignmentDto[], ApiServiceError>>;

    /**
     *
     * @param {number} employeeId
     * @return {Promise<SafeResult<EmployeeConditionInfoDto, Error>>}
     */
    getEmployeeConditionById(employeeId: number): Promise<SafeResult<EmployeeConditionInfoDto, ApiServiceError>>;

    /**
     *
     * @param {number} employeeId
     * @param {number} teamId
     * @return {Promise<SafeResult<DivisionTeamAssignmentDto, Error>>}
     */
    getHeadOfTeamHeadDivisionTeamAssignment({
        employeeId,
        divisionTeamId
    }?: {
        employeeId?: number;
        divisionTeamId?: number;
    }): Promise<SafeResult<DivisionTeamAssignmentDto, ApiServiceError>>;

    /**
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/employee/getLegalEntityTeamAssignments
     *
     * @param {number} assignmentId
     * @param {number} employeeId
     * @param {number} legalEntityTeamId
     * @return {Promise<SafeResult<LegalEntityTeamAssignmentDto[], Error>>}
     */
    getEmployeesLegalEntityTeamAssignments({
        assignmentId,
        employeeId,
        legalEntityTeamId
    }: {
        assignmentId?: number;
        employeeId?: number;
        legalEntityTeamId?: number;
    }): Promise<SafeResult<LegalEntityTeamAssignmentDto[], ApiServiceError>>;

    /**
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/employee/getTeamDivisionAssignmentsByLegalEntity
     * @todo check if return type is array or single value
     * @param {number} legalEntityTeamAssignmentId
     * @return {Promise<SafeResult<LegalEntityTeamAssignmentDto[], Error>>}
     */
    getLegalEntityTeamAssignmentsById(
        legalEntityTeamAssignmentId: number
    ): Promise<SafeResult<LegalEntityTeamAssignmentDto[], ApiServiceError>>;

    /**
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/employee/getTeamDivisionAssignmentsByLegalEntity
     * @param {number} legalEntityId - Идентификаторов организации
     * @param {boolean} [isHead] - Флаг, означающий будет возвращать метод только назначения руководителей или все назначения
     * @param {boolean} [withAssignments] - Отключение поля position_assignments
     * @param {boolean} [withRotation] - Отлючение поля division_team_assignment_rotations
     * @param {boolean} [withEmployee] - Отлючение поля division_team_assignment_employee
     * @param {boolean} [withDtr] - Отлючение поля division_team_role
     * @return {Promise<SafeResult<DivisionTeamAssignmentDto[], Error>>}
     */
    getDivisionTeamAssignmentsByLegalEntityId(
        legalEntityId: number,
        {
            isHead,
            withAssignments,
            withRotation,
            withEmployee,
            withDtr
        }?: {
            isHead?: boolean;
            withAssignments?: boolean;
            withRotation?: boolean;
            withEmployee?: boolean;
            withDtr?: boolean;
        }
    ): Promise<SafeResult<DivisionTeamAssignmentDto[], ApiServiceError>>;

    /**
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/division/getPath
     * @param {number} divisionId
     * @return {Promise<SafeResult<DivisionShortInfoDto[], Error>>}
     */
    getDivisionPathById(divisionId: number): Promise<SafeResult<DivisionShortInfoDto[], ApiServiceError>>;

    /**
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/division/getInfo_1
     * @param {number} divisionId
     * @return {Promise<SafeResult<DivisionInfoDto, Error>>}
     */
    getDivisionInfoById(divisionId: number): Promise<SafeResult<DivisionInfoDto, ApiServiceError>>;

    /**
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/division/listInfo
     * @param {number | number[]} [divisionIds]
     * @param {number} [legalEntityId]
     * @param {number} [parentDivisionId]
     * @param {number[]} [groupIds]
     * @param {boolean} [withChildren=false]
     * @return {Promise<SafeResult<DivisionInfoDto[], Error>>}
     */
    getDivisionInfosByFilter(
        {
            divisionIds,
            legalEntityId,
            groupIds,
            parentDivisionId
        }: {
            divisionIds: number | number[];
            legalEntityId: number;
            groupIds: number[];
            parentDivisionId: number;
        },
        { withChildren = false } = {}
    ): Promise<SafeResult<DivisionInfoDto[], ApiServiceError>>;

    /**
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/division/GetDivisionPositions
     * @param {number[]} divisionId
     * @param {number[]} positionIds
     * @param {number} [legalEntityId]
     * @param {boolean} [hasPositionAssignment]
     * @return {Promise<SafeResult<DivisionPositionDto[], Error>>}
     */
    getDivisionPositions(
        {
            divisionId,
            positionIds,
            legalEntityId
        }?: {
            divisionId?: number[];
            positionIds?: number[];
            legalEntityId?: number;
        },
        {
            hasPositionAssignment
        }?: {
            hasPositionAssignment?: boolean;
        }
    ): Promise<SafeResult<PositionExtendedDto[], ApiServiceError>>;

    /**
     * @return {Promise<SafeResult<RoleInfoDto[], Error>>}
     */
    getRolesAll(): Promise<SafeResult<RoleInfoDto[], ApiServiceError>>;

    /**
     * @description - Получение массива информации о назначениях в организацию
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/role/getLegalEntityAssignments
     *
     * @param {Object} filters
     * @param {number|number[]} [filters.roleId]
     * @param {number|number[]} [filters.legalEntityId]
     * @param {number|number[]} [filters.employeeId]
     * @return {Promise<SafeResult<LegalEntityTeamAssignmentDto[], Error>>}
     */
    getRoleLegalEntityTeamAssignments({
        roleId,
        legalEntityId,
        employeeId
    }: {
        roleId?: number | number[];
        legalEntityId?: number | number[];
        employeeId?: number | number[];
    }): Promise<SafeResult<LegalEntityAssignmentInfoDto[], ApiServiceError>>;

    /**
     *
     * @param {number} reasonIdExclusion
     * @param {string} commentExclusion
     * @param {string} documentUrlExclusion
     * @param {number} id
     * @return {Promise<SafeResult<boolean, Error>>}
     */
    async deleteDivisionTeamSuccessorById(
        reasonIdExclusion: number,
        commentExclusion: string,
        documentUrlExclusion: string,
        id: number
    ): Promise<SafeResult<boolean, Error>>;

    /**
     * Добавление заданного сотрудника в преемники на заданную роль в команде подразделения
     *
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/employee/createDivisionTeamSuccessorEntity
     * @param {number} employeeId
     * @param {number} divisionTeamRoleId
     * @param {string} datePriority
     * @param {number} reasonIdInclusion
     * @param {string} commentInclusion
     * @param {string} documentUrlInclusion
     * @return {Promise<SafeResult<DivisionTeamSuccessorDto, Error>>}
     */
    createDivisionTeamSuccessor(
        employeeId: number,
        divisionTeamRoleId: number,
        datePriority: string,
        reasonIdInclusion: number,
        commentInclusion: string,
        documentUrlInclusion: string
    ): Promise<SafeResult<InstanceType<DivisionTeamSuccessorDto>, ApiServiceError>>;

    /**
     * Добавление готовности заданного преемника к заданной ротации
     *
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/employee/createDivisionTeamSuccessorReadinessEntity
     * @param {number} divisionTeamSuccessorId
     * @param {number} assignmentReadinessId
     * @return {Promise<SafeResult<true>>|Error}
     */
    createDivisionTeamSuccessorReadiness(
        divisionTeamSuccessorId: number,
        assignmentReadinessId: number
    ): Promise<SafeResult<boolean, Error>>;

    /**
     * Добавление или обнуление записи подтверждения преемника hr-ом
     *
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/employee/divisionTeamSuccessorUpdateHr
     * @param {number} divisionTeamSuccessorId
     * @param {string} dateCommitHr
     * @return {Promise<SafeResult<boolean, Error>>;}
     */
    updateDivisionTeamSuccessorDateHr(
        divisionTeamSuccessorId: number,
        dateCommitHr: string
    ): Promise<SafeResult<boolean, Error>>;

    /**
     * @return {Promise<SafeResult<AssignmentReadinessDto[], Error>>}
     */
    getLibraryAssignmentReadinesses(): Promise<SafeResult<AssignmentReadinessDto[], Error>>;

    /**
     /**
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/division_team_role/update
     * @param {number} divisionTeamRoleId
     * @param {number} divisionTeamId
     * @param {number} importanceId
     * @return {Promise<SafeResult<DivisionTeamRoleRawDto, Error>>}
     */
    updateDivisionTeamRoleById(
        divisionTeamRoleId,
        { divisionTeamId, importanceId }
    ): Promise<SafeResult<DivisionTeamRoleRawDto, Error>>;

    /**
     * @param {Object} divisionTeamRoleFilter
     * @param {number} divisionTeamRoleFilter.page
     * @param {number} divisionTeamRoleFilter.size
     * @param {boolean} divisionTeamRoleFilter.successorReadinessDateFromPlusYear
     * @param {string} divisionTeamRoleFilter.successorReadinessDateFromStart
     * @param {string} divisionTeamRoleFilter.successorReadinessDateFromEnd
     * @param {string} divisionTeamRoleFilter.assignmentRotationDateFromStart
     * @param {string} divisionTeamRoleFilter.assignmentRotationDateFromEnd
     * @param {string} divisionTeamRoleFilter.successorDateFrom
     * @param {number[]} divisionTeamRoleFilter.divisionTeamRoleId
     * @param {number[]} divisionTeamRoleFilter.divisionTeamIds
     * @param {number} divisionTeamRoleFilter.divisionId
     * @param {string} divisionTeamRoleFilter.search
     * @param {number} divisionTeamRoleFilter.employeeSuccessor
     * @param {number[]} divisionTeamRoleFilter.legalEntityIds
     * @return {Promise<SafeResult<DivisionTeamRoleContainerDto[], Error>>}
     */
    getDivisionTeamRolesByFilter({
        page,
        size,
        successorReadinessDateFromPlusYear,
        successorReadinessDateFromStart,
        successorReadinessDateFromEnd,
        assignmentRotationDateFromStart,
        assignmentRotationDateFromEnd,
        successorDateFrom,
        divisionTeamRoleId,
        divisionTeamIds,
        divisionId,
        search,
        employeeSuccessor,
        legalEntityIds
    }: {
        page: number;
        size: number;
        successorReadinessDateFromPlusYear: boolean;
        successorReadinessDateFromStart: string;
        successorReadinessDateFromEnd: string;
        assignmentRotationDateFromStart: string;
        assignmentRotationDateFromEnd: string;
        successorDateFrom: string;
        divisionTeamRoleId: number[];
        divisionTeamIds: number[];
        divisionId: number;
        search: string;
        employeeSuccessor: number;
        legalEntityIds: number[];
    }): Promise<SafeResult<DivisionTeamRoleContainerDto[], Error>>;

    /**
     *
     * @param {number} divisionTeamId
     * @return {Promise<SafeResult<DivisionTeamRoleContainerDto[], Error>>}
     */
    async getDivisionTeamRoleInfosByDivisionTeamId(
        divisionTeamId: number
    ): Promise<SafeResult<DivisionTeamRoleContainerDto[], Error>>;

    /**
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/division_team_role
     * @return {Promise<SafeResult<DivisionTeamRoleRawDto[], Error>>}
     */
    async getDivisionTeamRoles(): Promise<SafeResult<DivisionTeamRoleRawDto[], Error>>;

    /**
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/division_team_role
     * @return {Promise<SafeResult<DivisionTeamRoleRawDto, Error>>}
     */
    async getDivisionTeamRoleById(id: number): Promise<SafeResult<DivisionTeamRoleRawDto, Error>>;

    /**
     * @param {Record<string, number|string|undefined>} divisionTeamRoleDto
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/division_team_role/create
     * @return {Promise<SafeResult<DivisionTeamRoleDto, Error>>}
     */
    async createDivisionTeamRole(
        divisionTeamRoleDto: Record<string, number | string | undefined | null>
    ): Promise<SafeResult<DivisionTeamRoleDto, Error>>;

    /**
     * @param {number} employeeId
     * @param {number} roleId
     * @param {number} legalEntityId
     * @return {Promise<SafeResult<Boolean, Error>>}
     */
    createOrUpdateEmployeeRole({
        employeeId,
        roleId,
        legalEntityId
    }: {
        employeeId: number;
        roleId: number;
        legalEntityId: number;
    }): Promise<SafeResult<boolean, ApiServiceError>>;

    /**
     * @return {Promise<SafeResult<Boolean, Error>>}
     */
    async deleteEmployeeRole({
        employeeId,
        roleId,
        legalEntityId
    }: {
        employeeId: string;
        roleId: string;
        legalEntityId: string;
    }): Promise<SafeResult<boolean, ApiServiceError>>;

    /**
     * Получение всех ротаций назначений
     *
     * @return {Promise<SafeResult<AssignmentRotationDto[], Error>>}
     */
    async getAssignmentRotations(): Promise<SafeResult<AssignmentRotationDto[], ApiServiceError>>;

    /**
     * Получение информации о всех организациях
     *
     * @return {Promise<SafeResult<LegalEntityDto[], Error>>}
     */
    async getLegalEntities(): Promise<SafeResult<LegalEntityDto[], ApiServiceError>>;

    /**
     * Получение информации об организации
     *
     * @param {number} legalEntityId
     * @return {Promise<SafeResult<LegalEntityDto, Error>>}
     */
    getLegalEntityById(legalEntityId: number): Promise<SafeResult<LegalEntityDto, ApiServiceError>>;

    /**
     * Получение информации о сотруднике по назначению
     *
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/employee/getEmployeeInfoByAssignment
     * @param {number} divisionTeamAssignmentId
     * @return {Promise<SafeResult<EmployeeInfoDto, Error>>}
     */
    getEmployeeByDivisionTeamAssignmentId(
        divisionTeamAssignmentId: number
    ): Promise<SafeResult<EmployeeInfoDto, Error>>;

    /**
     * @description Получение информации о всех рангах позиций
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/position/getPositionRankist
     *
     * @return {Promise<SafeResult<PositionRankDto[], Error>>}
     */
    getPositionRanks(): Promise<SafeResult<PositionRankDto[], Error>>;

    /**
     * @description Получение информации о всех категориях позиций
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/position/getPositionCategoryList
     *
     * @return {Promise<SafeResult<PositionCategoryDto[], Error>>}
     */
    getPositionCategories(): Promise<SafeResult<PositionCategoryDto[], Error>>;

    /**
     * @description Получение информации о всех типах причин
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/reason/getReasonList
     *
     * @return {Promise<SafeResult<ReasonDto[], Error>>}
     */
    getReasons(): Promise<SafeResult<ReasonDto[], Error>>;

    /**
     * @description Получение идентификатора сотрудника из сессии
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/employee/getEmployeeId
     *
     * @return {Promise<SafeResult<number, Error>>}
     */
    getSessionEmployeeId(): Promise<SafeResult<number, Error>>;

    /**
     * @description Получение информации о всех группах критериев важности
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/importancecriteria/findImportanceCriteriaGroupAll
     *
     * @return {Promise<SafeResult<ImportanceCriteriaGroupDto[], Error>>}
     */
    getImportanceCriteriaGroups(): Promise<SafeResult<ImportanceCriteriaGroupDto[], Error>>;

    /**
     * @description Получение информации о всех позициях критериев важности
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/importancecriteria/getPositionImportanceCriteriaList
     *
     * @return {Promise<SafeResult<PositionImportanceCriteriaDto[], Error>>}
     */
    getPositionImportanceCriteria(): Promise<SafeResult<PositionImportanceCriteriaDto[], Error>>;

    /**
     * @description Получение информации о всех критериях важности
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/importancecriteria/getImportanceCriteriaList
     *
     * @return {Promise<SafeResult<ImportanceCriteriaDto[], Error>>}
     */
    getImportanceCriteria(): Promise<SafeResult<ImportanceCriteriaDto[], Error>>;

    /**
     * @description Обновление типа причин
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/importancecriteria/recalculatePositionImportanceCriteria
     *
     * @param {RecalculatePositionImportanceCriteriaDto} recalculatePositionImportanceCriteria
     *
     * @returns {Promise<SafeResult<PositionImportanceCriteriaDto, Error>>}
     */
    recalculatePositionImportanceCriteria(
        recalculatePositionImportanceCriteria
    ): Promise<SafeResult<PositionImportanceCriteriaDto, Error>>;

    /**
     * @description Получение информации о position successor
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/position/getPositionSuccessorList
     *
     * @returns {Promise<SafeResult<PositionSuccessorDto[], Error>>}
     */
    getPositionSuccessors(): Promise<SafeResult<PositionSuccessorDto[], Error>>;

    /**
     * @description Создание позиции критериев важности
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/importancecriteria/createPositionImportanceCriteria
     *
     * @param {CreatePositionImportanceCriteriaInputDto} positionImportanceCriteria
     * @returns {Promise<SafeResult<PositionImportanceCriteriaDto, Error>>}
     */
    createPositionImportanceCriteria({
        weight,
        value,
        position_id,
        importance_criteria_id
    }: {
        weight?: number;
        value?: number;
        position_id?: number;
        importance_criteria_id?: number;
    }): Promise<SafeResult<PositionImportanceCriteriaDto, Error>>;

    /**
     * @description Обновление позиции критериев важности
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/importancecriteria/updatePositionImportanceCriteria
     *
     * @param {number} id
     * @param {UpdatePositionImportanceCriteriaInputDto} positionImportanceCriteria
     * @returns {Promise<SafeResult<PositionImportanceCriteriaDto, Error>>}
     */
    updatePositionImportanceCriteria(
        id: number,
        {
            weight,
            value,
            position_id,
            importance_criteria_id
        }: {
            weight?: number;
            value?: number;
            position_id?: number;
            importance_criteria_id?: number;
        }
    ): Promise<SafeResult<PositionImportanceCriteriaDto, Error>>;

    /**
     * @description Удаление позиции критериев важности
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/importancecriteria/deleteReasonType
     *
     * @param {number} id
     * @returns {Promise<SafeResult<Boolean, Error>>}
     */
    deletePositionImportanceCriteria(id: number): Promise<SafeResult<Boolean, Error>>;

    /**
     * @description Создание position successor
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/position/createPositionSuccessor
     *
     *
     * @param {CreatePositionSuccessorInputDto} positionSuccessor
     *
     * @returns {Promise<SafeResult<PositionSuccessorDto, Error>>}
     */
    createPositionSuccessor({
        date_commit_hr,
        date_priority,
        employee_id,
        position_id,
        position_group_id,
        reason_id_inclusion,
        reason_id_exclusion,
        comment_inclusion,
        comment_exclusion,
        document_url_inclusion,
        document_url_exclusion
    }: {
        date_commit_hr?: string;
        date_priority?: string;
        employee_id?: number;
        position_id?: number;
        position_group_id?: number;
        reason_id_inclusion?: number;
        reason_id_exclusion?: number;
        comment_inclusion?: string;
        comment_exclusion?: string;
        document_url_inclusion?: string;
        document_url_exclusion?: string;
    }): Promise<SafeResult<PositionSuccessorDto, Error>>;

    /**
     * @description Обновление position successor
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/position/updatePositionSuccessor
     *
     *
     * @param {number} id
     * @param {UpdatePositionSuccessorInputDto} positionSuccessor
     *
     * @returns {Promise<SafeResult<PositionSuccessorDto, Error>>}
     */
    updatePositionSuccessor(
        id: number,
        {
            date_commit_hr,
            date_priority,
            employee_id,
            position_id,
            position_group_id,
            reason_id_inclusion,
            reason_id_exclusion,
            comment_inclusion,
            comment_exclusion,
            document_url_inclusion,
            document_url_exclusion
        }: {
            date_commit_hr?: string;
            date_priority?: string;
            employee_id?: number;
            position_id?: number;
            position_group_id?: number;
            reason_id_inclusion?: number;
            reason_id_exclusion?: number;
            comment_inclusion?: string;
            comment_exclusion?: string;
            document_url_inclusion?: string;
            document_url_exclusion?: string;
        }
    ): Promise<SafeResult<PositionSuccessorDto, Error>>;

    /**
     * @description Удаление position successor
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/position/deletePositionSuccessor
     *
     *
     * @param {number} id
     *
     * @returns {Promise<SafeResult<Boolean, Error>>}
     */
    deletePositionSuccessor(id: number): Promise<SafeResult<Boolean, Error>>;

    /**
     * @description Создание position successor readiness
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/position/createPositionSuccessorReadiness
     *
     * @param {CreatePositionSuccessorReadinessInputDto} positionSuccessorReadiness
     *
     * @returns {Promise<SafeResult<PositionSuccessorReadinessDto, Error>>}
     */
    createPositionSuccessorReadiness({
        position_successor_id,
        readiness_id
    }: {
        position_successor_id?: number;
        readiness_id?: number;
    }): Promise<SafeResult<PositionSuccessorReadinessDto, Error>>;

    /**
     * @description Обновление position successor readiness
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/position/updatePositionSuccessorReadiness
     *
     * @param {number} id
     * @param {UpdatePositionSuccessorReadinessInputDto} positionSuccessorReadiness
     *
     * @returns {Promise<SafeResult<PositionSuccessorReadinessDto, Error>>}
     */
    updatePositionSuccessorReadiness(
        id: number,
        { positionSuccessorId, readinessId }: { positionSuccessorId?: number; readinessId?: number }
    ): Promise<SafeResult<PositionSuccessorReadinessDto, Error>>;

    /**
     * @description Удаление position successor readiness
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/position/deletePositionSuccessorReadiness
     *
     * @param {number} id
     *
     * @returns {Promise<SafeResult<Boolean, Error>>}
     */
    deletePositionSuccessorReadiness(id: number): Promise<SafeResult<Boolean, Error>>;

    /**
     * @description Закрывает назначение сотрудника в команду по указанным параметрам
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/role/clearEmployeeRole
     *
     * @param {number} legalEntityId
     * @param {number} roleId
     * @param {number} employeeId
     *
     * @returns {Promise<SafeResult<Boolean, Error>>}
     */
    clearEmployeeRole({
        legalEntityId,
        roleId,
        employeeId
    }: {
        legalEntityId: number;
        roleId: number;
        employeeId: number;
    }): Promise<SafeResult<boolean, Error>>;

    /**
     * @description Получение информации о всех значениях позиций
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/position/getPositionImportanceList
     *
     * @returns {Promise<SafeResult<PositionImportanceDto[], Error>>}
     */
    getPositionImportances(): Promise<SafeResult<PositionImportanceDto[], Error>>;

    /**
     * @description Получение всех связок позиции и критичности позиции.
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/position/getAllPositionPositionImportance
     *
     * @param {Object} params
     * @param {number[]} params.divisionTeamIds Фильтрация записей по division_team
     * @param {number[]} params.divisionTeamAssignmentIds Фильтрация записей по division_team_assigment
     * @param {number[]} params.positionIds Массив идентификаторов должностей.
     *
     * @param {Object} filterParams
     * @param {boolean} filterParams.withClosed Запрашиваются ли закрытые записи.
     *
     * @returns {Promise<SafeResult<PositionImportanceBindingDto[], Error>>}
     */
    getPositionImportanceBindingsByFilter(
        {
            divisionTeamIds,
            divisionTeamAssignmentIds,
            positionIds
        }: {
            divisionTeamIds?: number[];
            divisionTeamAssignmentIds?: number[];
            positionIds: number[];
        } = {},
        { withClosed }: { withClosed?: boolean } = {}
    ): Promise<SafeResult<PositionImportanceBindingDto[], Error>>;

    /**
     * @description Создание связки позиции и критичности позиции.
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/position/createPositionPositionImportance
     *
     * @param {CreatePositionImportanceBindingInputDto} positionImportanceBindingInput
     * @returns {Promise<SafeResult<PositionImportanceBindingDto[], Error>>}
     */
    createPositionImportanceBinding(
        positionImportanceBindingInput
    ): Promise<SafeResult<PositionImportanceBindingDto[], Error>>;

    /**
     * @description удаление связки позиции и критичности позиции
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/position/deletePositionPositionImportance
     *
     * @param {number} id
     * @returns {Promise<SafeResult<Object, Error>>}
     */
    deletePositionPositionImportanceBinding(id: number): Promise<SafeResult<Object, Error>>;

    /**
     * @description Получение списка Position по идентификатору сотрудника и идентификатору подразделения
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/position/getPositionAssignmentByEmployeeId
     *
     * @param {number} employeeId
     * @param {number[]} divisionIds
     * @returns {Promise<SafeResult<PositionAssignmentDto[], Error>>}
     */
    getPositionAssignmentsByFilter({
        employeeId,
        divisionIds
    }: {
        employeeId: number;
        divisionIds: number[];
    }): Promise<SafeResult<PositionAssignmentDto[], Error>>;

    /**
     * @description Получение информации о списке групп подразделений
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/division/divisionGroupInfo
     *
     * @param {number[]} groupIds
     * @returns {Promise<SafeResult<DivisionGroupDto[], Error>>}
     */
    getDivisionGroups({ groupIds }?: { groupIds?: number[] }): Promise<SafeResult<DivisionGroupDto[], Error>>;

    /**
     * @description Получение информации о подчиненных в команде подразделения по ИД команды
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/employee/getTeamDivisionSubordinatesTeam
     *
     * @param {number} divisionTeamId - Идентификатор команды подразделения
     *
     * @param {Object} [filters]
     * @param {boolean} [filters.withChildren] - Необходимо ли получать подчиненных от подчиненных
     * @returns {Promise<SafeResult<DivisionTeamAssignmentDto[], Error>>}
     */
    getEmployeeDivisionTeamSubordinatesByDivisionTeamId(
        divisionTeamId: number,
        { withChildren }?: { withChildren: boolean } = {}
    ): Promise<SafeResult<DivisionTeamAssignmentDto[], Error>>;

    /**
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/work_experience_type/getAll
     * @description Получение информации о всех типах опыта работы
     *
     * @returns {Promise<SafeResult<DivisionInfoDto, Error>>}
     */
    getWorkExperienceTypes(): Promise<SafeResult<DivisionInfoDto, Error>>;

    /**
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/work_experience_type/create_1
     * @description Создание типа опыта работы
     *
     * @param {string} name
     * @param {string} description
     * @returns {Promise<SafeResult<DivisionTeamRoleDto, Error>>}
     */
    createWorkExperienceType({
        name,
        description
    }: {
        name: string;
        description: string;
    }): Promise<SafeResult<DivisionTeamRoleDto, Error>>;

    /**
     *
     */
    employee: IEmployeeContext;

    /**
     *
     */
    division: IDivisionContext;

    /**
     *
     */
    divisionTeam: ITeamContext;
}
