/**
 * @typedef {import('@goodt-common/infra').BaseDto} BaseDto
 */
/* eslint-disable camelcase */

import { ApiClientMethod, BaseApiService, processRequestResult, ApiServiceError, buildRequest } from '@goodt-common/api';

import { ApiEndpointPaths as Paths, ServiceAction } from './config';
import {
    DivisionTeamAssignmentDto,
    DivisionTeamRoleContainerDto,
    EmployeeConditionInfoDto,
    EmployeeExtendedInfoDto,
    LegalEntityTeamAssignmentDto,
    DivisionInfoDto,
    PositionExtendedDto,
    DivisionShortInfoDto,
    RoleInfoDto,
    DivisionTeamRoleRawDto,
    DivisionTeamRoleDto,
    DivisionTeamAssignmentRotationDto,
    DivisionTeamSuccessorDto,
    AssignmentReadinessDto,
    DivisionTeamSuccessorReadinessDto,
    AssignmentRotationDto,
    LegalEntityDto,
    EmployeeInfoDto,
    PositionRankDto,
    PositionCategoryDto,
    ReasonDto,
    ImportanceCriteriaGroupDto,
    PositionImportanceCriteriaDto,
    ImportanceCriteriaDto,
    RecalculatePositionImportanceCriteriaDto,
    PositionSuccessorDto,
    PositionSuccessorReadinessDto,
    CreatePositionImportanceCriteriaInputDto,
    UpdatePositionImportanceCriteriaInputDto,
    CreatePositionSuccessorInputDto,
    UpdatePositionSuccessorInputDto,
    CreatePositionSuccessorReadinessInputDto,
    UpdatePositionSuccessorReadinessInputDto,
    PositionImportanceDto,
    PositionImportanceBindingDto,
    PositionAssignmentDto,
    DivisionGroupDto,
    CreatePositionImportanceBindingInputDto
} from './dto';

import { withEmployeeContext, withEmployeeIdContext } from './EmployeeContext';
import { withDivisionContext, withDivisionIdContext } from './DivisionContext';
import { withTeamContext, withTeamIdContext } from './TeamContext';

/**
 *
 */
class OrgStructureApiService extends BaseApiService {
    /**
     *
     * @param {import('@goodt-common/api').IApiServiceRequest | import('@goodt-common/api').IApiServiceRequestOptions} apiServiceRequest
     * @param {function(...args?: any[]): any} [DtoConstructor=BaseDto]
     * @return {typeof ReturnType<processRequestResult>}
     */
    async request(apiServiceRequest, DtoConstructor) {
        if ('action' in apiServiceRequest) {
            apiServiceRequest = buildRequest(apiServiceRequest);
        }
        return processRequestResult(await super.request(apiServiceRequest), DtoConstructor);
    }

    /**
     *
     * @param {?number} [employeeId]
     * @return {Promise<SafeResult<EmployeeExtendedInfoDto, Error>>}
     */
    async getEmployeeById(employeeId) {
        // prettier-ignore
        return this.request({
            url: Paths.EMPLOYEE_INFO,
            params: {
                id: employeeId
            }
        }, EmployeeExtendedInfoDto);
    }

    /**
     *
     * @param {number[]} employeeIds
     * @return {Promise<SafeResult<EmployeeExtendedInfoDto[], Error>>}
     */
    async getEmployeesByIds(employeeIds) {
        return this.request(
            {
                url: Paths.EMPLOYEE_LIST,
                params: {
                    employees: employeeIds
                }
            },
            EmployeeExtendedInfoDto
        );
    }

    /**
     * @description Получение информации о сотрудниках
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/employee/findInfoPost
     *
     *
     * @param {Object} filters
     * @param {number[]} [filters.employeeIds]
     * @param {number} [filters.divisionId]
     * @param {string} [filters.searchToken]
     * @param {string} [filters.legalEntityId]
     * @param {boolean} [filters.withPatronymic]
     * @param {number} [filters.jobTitleId]
     * @param {Object} outputOptions
     * @param {number} [outputOptions.page]
     * @param {number} [outputOptions.size]
     * @param {boolean} [outputOptions.hasPositionAssignment]
     * @param {Object} extraOutputData
     * @param {Function} [extraOutputData.onResult] - function that takes as an argument all information except employees
     * @return {Promise<SafeResult<EmployeeExtendedInfoDto[], Error>>}
     */
    async getEmployeesByFilter(
        { employeeIds, divisionId, searchToken, legalEntityId, withPatronymic, jobTitleId } = {},
        { page, size, hasPositionAssignment } = {},
        { onResult } = {}
    ) {
        return processRequestResult(
            await this.request({
                action: ServiceAction.EMPLOYEE_FIND,
                queryParams: {
                    ...(employeeIds && { id: employeeIds }),
                    ...(divisionId && { division: divisionId }),
                    ...(searchToken && { search: searchToken }),
                    ...((page || page === 0) && { page }),
                    ...(legalEntityId && { legal_entity_id: legalEntityId }),
                    ...(size && { size }),
                    ...(withPatronymic != null && { with_patronymic: withPatronymic }),
                    ...(hasPositionAssignment != null && { has_position_assignment: hasPositionAssignment }),
                    ...(jobTitleId != null && { job_title_id: jobTitleId })
                }
            }),
            EmployeeExtendedInfoDto,
            {
                resultTransformer: ({ data, ...meta }) => {
                    if (typeof onResult === 'function') {
                        onResult(meta);
                    }
                    return data;
                }
            }
        );
    }

    /**
     *
     * @param {number} employeeId
     * @return {Promise<SafeResult<EmployeeConditionInfoDto, Error>>}
     */
    async getEmployeeConditionById(employeeId) {
        // prettier-ignore
        return this.request({
            url: Paths.EMPLOYEE_CONDITION,
            params: {
                id: employeeId
            }
        }, EmployeeConditionInfoDto);
    }

    /**
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/employee/getDivisionTeamAssignments
     * @description Получение информации о назначениях в команду подразделения
     *
     * @param {Object} employee
     * @param {number} [employee.employeeId]
     * @param {number} [employee.divisionTeamId]
     *
     * @param {Object} filters
     * @param {boolean} [filters.withClosed]
     *
     * @return {Promise<SafeResult<DivisionTeamAssignmentDto[], Error>>}
     */
    async getEmployeeDivisionTeamAssignments({ employeeId, divisionTeamId } = {}, { withClosed = false } = {}) {
        // prettier-ignore
        return this.request({
            action: ServiceAction.EMPLOYEE_TEAM_DIVISION_ASSIGNMENTS_GET,
            params: {
                ...(employeeId && { employee: employeeId }),
                ...(divisionTeamId && { team: divisionTeamId }),
                with_closed: withClosed
            }
        }, DivisionTeamAssignmentDto);
    }

    /**
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/employee/getDivisionTeamAssignments
     * @description Получение информации о назначениях в команду подразделения
     *
     * @param {number} [ids]
     * @param {Object} [filters]
     * @param {boolean} [filters.withClosed]
     *
     * @return {Promise<SafeResult<DivisionTeamAssignmentDto[], Error>>}
     */
    async getDivisionTeamAssignmentsByIds(ids, { withClosed = false } = {}) {
        // prettier-ignore
        return this.request({
            action: ServiceAction.EMPLOYEE_TEAM_DIVISION_ASSIGNMENTS_GET,
            params: {
                id: ids,
                with_closed: withClosed
            }
        }, DivisionTeamAssignmentDto);
    }

    /**
     *
     * @param {number} employeeId
     * @param {number} divisionTeamId
     * @param {boolean} [withChildren=false]
     * @return {Promise<SafeResult<DivisionTeamAssignmentDto[], Error>>}
     */
    async getSubordinatesDivisionsTeamAssignments({ employeeId, divisionTeamId }, { withChildren = false } = {}) {
        // prettier-ignore
        return this.request({
            url: Paths.EMPLOYEE_TEAM_DIVISION_SUBORDINATES,
            params: {
                id: employeeId,
                team: divisionTeamId,
                withchilds: withChildren
            }
        }, DivisionTeamAssignmentDto);
    }

    /**
     *
     * @param {number} employeeId
     * @param {number} divisionTeamId
     * @return {Promise<SafeResult<DivisionTeamAssignmentDto, Error>>}
     */
    async getTeamHeadDivisionTeamAssignment({ employeeId, divisionTeamId }) {
        // prettier-ignore
        return this.request({
            url: Paths.EMPLOYEE_TEAM_DIVISION_HEAD,
            params: {
                employee: employeeId,
                team: divisionTeamId
            }
        }, DivisionTeamAssignmentDto);
    }

    /**
     *
     * @param {number} divisionTeamId
     * @return {Promise<SafeResult<DivisionTeamRoleContainerDto[], Error>>}
     */
    async getDivisionTeamRoleInfosByDivisionTeamId(divisionTeamId) {
        // prettier-ignore
        return this.request({
            url: Paths.EMPLOYEE_TEAM_DIVISION_ROLES_TEAM,
            params: {
                team: divisionTeamId
            }
        }, DivisionTeamRoleContainerDto);
    }

    /**
     * @param {number} reasonIdExclusion
     * @param {string} commentExclusion
     * @param {string} documentUrlExclusion
     * @param {number} id
     * @return {Promise<SafeResult<boolean, Error>>}
     */
    async deleteDivisionTeamSuccessorById(reasonIdExclusion, commentExclusion, documentUrlExclusion, id) {
        // prettier-ignore
        return this.request({
            url: Paths.EMPLOYEE_DELETE_TEAM_DIVISION_SUCCESSOR,
            options: {
                method: ApiClientMethod.POST,
                params: {
                    reason_id_exclusion: reasonIdExclusion,
                    comment_exclusion: commentExclusion,
                    document_url_exclusion: documentUrlExclusion,
                    division_team_successor_id: id
                }
            }
        }, Boolean);
    }

    /**
     * Добавление преемника
     *
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/employee/createDivisionTeamSuccessorEntity
     * @param {number} employeeId
     * @param {number} divisionTeamRoleId
     * @param {string} datePriority
     * @param {number} reasonIdInclusion
     * @param {string} commentInclusion
     * @param {string} documentUrlInclusion
     * @return {Promise<SafeResult<DivisionTeamSuccessorDto, Error>>}
     */
    createDivisionTeamSuccessor(
        employeeId,
        divisionTeamRoleId,
        datePriority,
        reasonIdInclusion,
        commentInclusion,
        documentUrlInclusion
    ) {
        // prettier-ignore
        return this.request({
            action: ServiceAction.EMPLOYEE_DIVISION_TEAM_SUCCESSOR_ADD,
            queryParams: {
                employee_id: employeeId,
                division_team_role_id: divisionTeamRoleId,
                date_priority: datePriority,
                reason_id_inclusion: reasonIdInclusion,
                comment_inclusion: commentInclusion,
                document_url_inclusion: documentUrlInclusion
            }
        }, DivisionTeamSuccessorDto);
    }

    /**
     * Добавление готовности заданного преемника к заданной ротации
     *
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/employee/createDivisionTeamSuccessorReadinessEntity
     * @param {number} divisionTeamSuccessorId
     * @param {number} assignmentReadinessId
     * @return {Promise<SafeResult<DivisionTeamSuccessorReadinessDto, Error>>}
     */
    createDivisionTeamSuccessorReadiness(divisionTeamSuccessorId, assignmentReadinessId) {
        // prettier-ignore
        return this.request({
            action: ServiceAction.EMPLOYEE_DIVISION_TEAM_SUCCESSOR_READINESS_SET,
            queryParams: {
                division_team_successor_id: divisionTeamSuccessorId,
                assignment_readiness_id: assignmentReadinessId
            }
        }, DivisionTeamSuccessorReadinessDto);
    }

    /**
     * Добавление или обнуление записи подтверждения преемника hr-ом
     *
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/employee/divisionTeamSuccessorUpdateHr
     * @param {number} divisionTeamSuccessorId
     * @param {string} dateCommitHr
     * @return {Promise<SafeResult<boolean, Error>>}
     */
    updateDivisionTeamSuccessorDateHr(divisionTeamSuccessorId, dateCommitHr) {
        // prettier-ignore
        return this.request({
            action: ServiceAction.EMPLOYEE_DIVISION_TEAM_SUCCESSOR_DATE_HR_UPDATE,
            queryParams: {
                division_team_successor_id: divisionTeamSuccessorId,
                date_commit_hr: dateCommitHr
            }
        }, Boolean);
    }

    /**
     * Получение всех готовностей к назначениям
     *
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/library/getAllAssignmentReadiness
     * @return {Promise<SafeResult<AssignmentReadinessDto[], Error>>}
     */
    getAssignmentReadinesses() {
        // prettier-ignore
        return this.request({
            action: ServiceAction.ASSIGNMENT_READINESS_GET
        }, AssignmentReadinessDto);
    }

    /**
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/division_team_role/update
     * @param {number} divisionTeamRoleId
     * @param {number} divisionTeamId
     * @param {number} importanceId
     * @return {Promise<SafeResult<DivisionTeamRoleRawDto, Error>>}
     */
    updateDivisionTeamRoleById(divisionTeamRoleId, { divisionTeamId, importanceId }) {
        // prettier-ignore
        return this.request({
             action: ServiceAction.DIVISION_TEAM_ROLE_SET_BY_ID,
             pathParams: { id: divisionTeamRoleId },
             params: {
                 division_team_id: divisionTeamId,
                 position_importance_id: importanceId
             }
         }, DivisionTeamRoleRawDto);
    }

    /**
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/division_team_role/findPost
     * Получение информации о ролях, соответствующих заданным фильтрам.
     *
     * @param {Object} divisionTeamRoleFilter
     * @param {number} divisionTeamRoleFilter.page
     * @param {number} divisionTeamRoleFilter.size
     * @param {boolean} divisionTeamRoleFilter.successorReadinessDateFromPlusYear
     * @param {string} divisionTeamRoleFilter.successorReadinessDateFromStart
     * @param {string} divisionTeamRoleFilter.successorReadinessDateFromEnd
     * @param {string} divisionTeamRoleFilter.assignmentRotationDateFromStart
     * @param {string} divisionTeamRoleFilter.assignmentRotationDateFromEnd
     * @param {string} divisionTeamRoleFilter.successorDateFrom
     * @param {number[]} divisionTeamRoleFilter.divisionTeamRoleId
     * @param {number[]} divisionTeamRoleFilter.divisionTeamIds
     * @param {number} divisionTeamRoleFilter.divisionId
     * @param {string} divisionTeamRoleFilter.search
     * @param {number} divisionTeamRoleFilter.employeeSuccessor
     * @param {number[]} divisionTeamRoleFilter.legalEntityIds
     * @return {Promise<SafeResult<DivisionTeamRoleContainerDto[], Error>>}
     */
    getDivisionTeamRolesByFilter({
        page,
        size,
        successorReadinessDateFromPlusYear,
        successorReadinessDateFromStart,
        successorReadinessDateFromEnd,
        assignmentRotationDateFromStart,
        assignmentRotationDateFromEnd,
        successorDateFrom,
        divisionTeamRoleId,
        divisionTeamIds,
        divisionId,
        search,
        employeeSuccessor,
        legalEntityIds
    } = {}) {
        // prettier-ignore
        return this.request({
            action: ServiceAction.DIVISION_TEAM_ROLE_FIND,
            queryParams: {
                ...(page != null && { page }),
                ...(size != null && { size }),
                ...(successorReadinessDateFromPlusYear != null && {successor_readiness_date_from_plus_year: successorReadinessDateFromPlusYear}),
                ...(successorReadinessDateFromStart != null && {successor_readiness_date_from_start: successorReadinessDateFromStart }),
                ...(successorReadinessDateFromEnd != null && { successor_readiness_date_from_end: successorReadinessDateFromEnd }),
                ...(assignmentRotationDateFromStart != null && { assignment_rotation_date_from_start: assignmentRotationDateFromStart }),
                ...(assignmentRotationDateFromEnd != null && { assignment_rotation_date_from_end: assignmentRotationDateFromEnd }),
                ...(successorDateFrom != null && { successor_date_from: successorDateFrom }),
                ...(divisionTeamRoleId != null && { division_team_role_id: divisionTeamRoleId }),
                ...(divisionTeamIds != null && { division_team_id: divisionTeamIds }),
                ...(divisionId != null && { division_id: divisionId }),
                ...(search != null && { search }),
                ...(employeeSuccessor != null && { employee_successor: employeeSuccessor }),
                ...(legalEntityIds != null && { legal_entity_id: legalEntityIds }),
            }
        }, DivisionTeamRoleContainerDto);
    }

    /**
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/division_team_role/findPost     * @param {number} divisionTeamId
     * @return {Promise<SafeResult<DivisionTeamAssignmentDto[], Error>>}
     */
    async getTeamDivisionTeamAssignmentsByTeamId(divisionTeamId) {
        // prettier-ignore
        return this.request({
            url: Paths.EMPLOYEE_TEAM_DIVISION_ASSIGNMENTS_TEAM,
            params: {
                team: divisionTeamId
            }
        }, DivisionTeamAssignmentDto);
    }

    /**
     *
     * @param {number} employeeId
     * @param {number} divisionTeamId
     * @return {Promise<SafeResult<DivisionTeamAssignmentDto, Error>>}
     */
    async getHeadOfTeamHeadDivisionTeamAssignment({ employeeId, divisionTeamId }) {
        // prettier-ignore
        return this.request({
            url: Paths.EMPLOYEE_TEAM_DIVISION_HEAD_HEAD,
            params: {
                employee: employeeId,
                team: divisionTeamId
            }
        }, DivisionTeamAssignmentDto);
    }

    /**
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/employee/getLegalEntityTeamAssignments
     *
     * @param {number} employeeId
     * @param {number} legalEntityTeamId
     * @return {Promise<SafeResult<LegalEntityTeamAssignmentDto[], Error>>}
     */
    async getEmployeesLegalEntityTeamAssignments({ employeeId, legalEntityTeamId }) {
        // prettier-ignore
        return this.request({
            action: ServiceAction.EMPLOYEE_TEAM_LEGAL_ENTITY_ASSIGNMENTS_GET,
            params: {
                ...(employeeId && { employee: employeeId }),
                ...(legalEntityTeamId && { team: legalEntityTeamId })
            }
        }, LegalEntityTeamAssignmentDto);
    }

    /**
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/employee/getLegalEntityTeamAssignments
     *
     * @param {number} assignmentId
     * @return {Promise<SafeResult<LegalEntityTeamAssignmentDto[], Error>>}
     */
    async getLegalEntityTeamAssignmentsById(assignmentId) {
        // prettier-ignore
        return this.request({
            action: ServiceAction.EMPLOYEE_TEAM_LEGAL_ENTITY_ASSIGNMENTS_GET,
            params: {
                id: assignmentId
            }
        }, LegalEntityTeamAssignmentDto);
    }

    /**
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/employee/getTeamDivisionAssignmentsByLegalEntity
     * @param {number} legalEntityId - Идентификаторов организации
     * @param {boolean} [isHead] - Флаг, означающий будет возвращать метод только назначения руководителей или все назначения
     * @param {boolean} [withAssignments] - Отключение поля position_assignments
     * @param {boolean} [withRotation] - Отлючение поля division_team_assignment_rotations
     * @param {boolean} [withEmployee] - Отлючение поля division_team_assignment_employee
     * @param {boolean} [withDtr] - Отлючение поля division_team_role
     * @return {Promise<SafeResult<DivisionTeamAssignmentDto[], Error>>}
     */
    async getDivisionTeamAssignmentsByLegalEntityId(
        legalEntityId,
        { isHead, withAssignments, withRotation, withEmployee, withDtr } = {}
    ) {
        // prettier-ignore
        return this.request({
            action: ServiceAction.EMPLOYEE_TEAM_DIVISION_LEGAL_ENTITY_ASSIGNMENTS_GET,
            params: {
                legal_entity: legalEntityId,
                is_head: isHead,
                with_assignments: withAssignments,
                with_rotation: withRotation,
                with_employee: withEmployee,
                with_dtr: withDtr
            }
        }, DivisionTeamAssignmentDto);
    }

    /**
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/division/getPath
     * @param {number} divisionId
     * @return {Promise<SafeResult<DivisionShortInfoDto[], Error>>}
     */
    async getDivisionPathById(divisionId) {
        // prettier-ignore
        return this.request({
            action: ServiceAction.DIVISION_PATH_GET,
            params: {
                id: divisionId
            }
        }, DivisionShortInfoDto);
    }

    /**
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/division/getInfo_1
     * @param {number} divisionId
     * @return {Promise<SafeResult<DivisionInfoDto, Error>>}
     */
    async getDivisionInfoById(divisionId) {
        // prettier-ignore
        return this.request({
            action: ServiceAction.DIVISION_INFO_GET,
            params: {
                id: divisionId
            }
        }, DivisionInfoDto);
    }

    /**
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/division/listInfo
     * @param {number | number[]} [divisionIds]
     * @param {number} [legalEntityId]
     * @param {number} [parentDivisionId]
     * @param {number[]} [groupIds]
     * @param {boolean} [withChildren=false]
     * @return {Promise<SafeResult<DivisionInfoDto[], Error>>}
     */
    async getDivisionInfosByFilter(
        { divisionIds, legalEntityId, groupIds, parentDivisionId },
        { withChildren = false } = {}
    ) {
        // prettier-ignore
        return this.request({
            action: ServiceAction.DIVISION_LIST_GET,
            params: {
                ...(divisionIds && { division_id: divisionIds }),
                ...(parentDivisionId && { parent: parentDivisionId }),
                ...(legalEntityId && { legalentity: legalEntityId }),
                ...(groupIds && { group_id: groupIds }),
                withchilds: withChildren
            }
        }, DivisionInfoDto);
    }

    /**
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/division/GetDivisionPositions
     * @param {number[]} [divisionId]
     * @param {number[]} [positionIds]
     * @param {number} [legalEntityId]
     * @param {boolean} [hasPositionAssignment]
     * @return {Promise<SafeResult<PositionExtendedDto[], Error>>}
     */
    async getDivisionPositions({ divisionId, positionIds, legalEntityId } = {}, { hasPositionAssignment } = {}) {
        // prettier-ignore
        return this.request({
            url: Paths.DIVISION_POSITION,
            params: {
                ...(divisionId && {division: divisionId}),
                ...(positionIds && { position_id: positionIds }),
                ...(legalEntityId && { legal_entity: legalEntityId }),
                ...(hasPositionAssignment != null && { has_position_assignment: hasPositionAssignment }),
            }
        }, PositionExtendedDto);
    }

    /**
     * @return {Promise<SafeResult<RoleInfoDto[], Error>>}
     */
    async getRolesAll() {
        // prettier-ignore
        return this.request({
            url: Paths.ROLE_LIST
        }, RoleInfoDto);
    }

    /**
     * @description - Получение массива информации о назначениях в организацию
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/role/getLegalEntityAssignments
     *
     * @param {Object} filters
     * @param {number|number[]} [filters.roleId]
     * @param {number|number[]} [filters.legalEntityId]
     * @param {number|number[]} [filters.employeeId]
     * @return {Promise<SafeResult<LegalEntityTeamAssignmentDto[], Error>>}
     */
    async getRoleLegalEntityTeamAssignments({ roleId, legalEntityId, employeeId } = {}) {
        // prettier-ignore
        return this.request({
            action: ServiceAction.ROLE_ASSIGNMENTS_LIST_GET,
            params: {
                role_id: roleId,
                legal_entity_id: legalEntityId,
                employee_id: employeeId
            }
        }, LegalEntityTeamAssignmentDto);
    }

    /**
     *
     * @param {number} employeeId
     * @param {number} roleId
     * @param {number} legalEntityId
     * @return {Promise<SafeResult<Boolean, Error>>}
     */
    async createOrUpdateEmployeeRole({ employeeId, roleId, legalEntityId }) {
        // prettier-ignore
        return this.request({
            action: ServiceAction.ROLE_SET_EMPLOYEE,
            queryParams: {
                employee_id: employeeId,
                ...(roleId && { role_id: roleId }),
                ...(legalEntityId && { legal_entity_id: legalEntityId })
            }
        }, Boolean);
    }

    /**
     * @return {Promise<SafeResult<Boolean, Error>>}
     */
    async deleteEmployeeRole({ employeeId, roleId, legalEntityId }) {
        // prettier-ignore
        return this.request({
            url: Paths.ROLE_LIST,
            params: {
                employee_id: employeeId,
                role_id: roleId,
                legal_entity_id: legalEntityId
            },
            options: {
                method: ApiClientMethod.POST
            }
        }, Boolean);
    }

    /**
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/division_team_role
     * @return {Promise<SafeResult<DivisionTeamRoleRawDto[], Error | null>>}
     */
    async getDivisionTeamRoles() {
        // prettier-ignore
        return this.request({
            url: Paths.DIVISION_TEAM_ROLE
        }, DivisionTeamRoleRawDto);
    }

    /**
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/division_team_role
     * @return {Promise<SafeResult<DivisionTeamRoleRawDto, Error>>}
     */
    async getDivisionTeamRoleById(id) {
        if (id == null) {
            throw new ApiServiceError(`'OrgStructureApiService.getDivisionTeamRoleById' method invalid arguments`, {
                code: ApiServiceError.Code.INTERNAL
            });
        }
        // prettier-ignore
        return this.request({
            url: Paths.DIVISION_TEAM_ROLE_GET_BY_ID.replace(':id', id)
        }, DivisionTeamRoleRawDto);
    }

    /**
     * @param {Record<string, number|string|undefined>} divisionTeamRoleDto
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/division_team_role/create
     * @return {Promise<SafeResult<DivisionTeamRoleDto, Error>>}
     */
    async createDivisionTeamRole(divisionTeamRoleDto) {
        // prettier-ignore
        return this.request({
            url: Paths.DIVISION_TEAM_ROLE,
            params: divisionTeamRoleDto,
            options: {
                method: ApiClientMethod.POST
            }
        }, DivisionTeamRoleDto);
    }

    /**
     * employee/teamdivisionassignmentrotationcommit
     * @param {number|string} id DivisionTeamAssignmentRotation id
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/division_team_role/create
     * @return {Promise<SafeResult<true, Error>>}
     */
    async commitDivisionTeamAssignmentRotation(id) {
        // prettier-ignore
        return this.request({
            url: Paths.DIVISION_TEAM_ASSIGNMENT_ROTATION_COMMIT,
            options: {
                method: ApiClientMethod.POST,
                params: { id }
            }
        }, Boolean);
    }

    /**
     * employee//teamdivisionassignmentrotationwithdraw
     * @param {number|string} id DivisionTeamAssignmentRotation id
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/division_team_role/create
     * @return {Promise<SafeResult<true, Error>>}
     */
    async withdrawDivisionTeamAssignmentRotation(id) {
        // prettier-ignore
        return this.request({
            url: Paths.DIVISION_TEAM_ASSIGNMENT_ROTATION_WITHDRAW,
            options: {
                method: ApiClientMethod.POST,
                params: { id }
            }
        }, Boolean);
    }

    /**
     * employee/teamdivisionassignmentrotationcomment
     * @param {number|string} divisionTeamAssignmentId DivisionTeamAssignment id
     * @param {number|string} assignmentRotationId AssignmentRotation id
     * @return {Promise<SafeResult<DivisionTeamAssignmentRotationDto[], Error>>}
     */
    async createDivisionTeamAssignmentRotation(divisionTeamAssignmentId, assignmentRotationId) {
        // prettier-ignore
        return this.request({
            url: Paths.DIVISION_TEAM_ASSIGNMENT_ROTATION_CREATE,
            options: {
                method: ApiClientMethod.POST,
                params: {
                    division_team_assignment_id: divisionTeamAssignmentId,
                    assignment_rotation_id: assignmentRotationId
                }
            }
        }, DivisionTeamAssignmentRotationDto);
    }

    /**
     * employee/teamdivisionassignmentrotationcomment
     * @param {number|string} divisionTeamAssignmentRotationId DivisionTeamAssignmentRotation id
     * @param {string} employeeComment
     * @param {string} hrComment
     * @return {Promise<SafeResult<true, Error>>}
     */
    async updateDivisionTeamAssignmentRotationComment(
        divisionTeamAssignmentRotationId,
        { hrComment, employeeComment }
    ) {
        return this.request(
            {
                action: ServiceAction.DIVISION_TEAM_ASSIGNMENT_ROTATION_UPDATE_COMMENT_PUT,
                queryParams: {
                    id: divisionTeamAssignmentRotationId,
                    ...(hrComment != null && { comment_hr: hrComment }),
                    ...(employeeComment != null && { comment_employee: employeeComment })
                }
            },
            Boolean
        );
    }

    /**
     * Получение всех ротаций назначений
     *
     * @return {Promise<SafeResult<AssignmentRotationDto[], Error>>}
     */
    async getAssignmentRotations() {
        // prettier-ignore
        return this.request({
            action: ServiceAction.LIBRARY_ASSIGNMENT_ROTATIONS_GET
        }, AssignmentRotationDto);
    }

    /**
     * Employee context
     */
    get employee() {
        const employeeContext = withEmployeeContext(this);
        return {
            /**
             * @param {number|number[]} id
             */
            withId: withEmployeeIdContext(employeeContext),
            ...employeeContext
        };
    }

    /**
     * Division context
     */
    get division() {
        const divisionContext = withDivisionContext(this);
        return {
            withId: withDivisionIdContext(divisionContext),
            ...divisionContext
        };
    }

    /**
     * Team context
     */
    get divisionTeam() {
        const teamContext = withTeamContext(this);
        return {
            withId: withTeamIdContext(teamContext),
            ...teamContext
        };
    }

    /**
     * Получение информации о всех организациях
     *
     * @return {Promise<SafeResult<LegalEntityDto[], Error>>}
     */
    getLegalEntities() {
        return this.request(
            {
                action: ServiceAction.LEGAl_ENTITY_LIST
            },
            LegalEntityDto
        );
    }

    /**
     * Получение информации об организации
     *
     * @param {number} legalEntityId
     * @return {Promise<SafeResult<LegalEntityDto, Error>>}
     */
    getLegalEntityById(legalEntityId) {
        return this.request(
            {
                action: ServiceAction.LEGAL_ENTITY_INFO,
                params: {
                    id: legalEntityId
                }
            },
            LegalEntityDto
        );
    }

    /**
     * Получение информации о сотруднике по назначению
     *
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/employee/getEmployeeInfoByAssignment
     * @param {number} divisionTeamAssignmentId
     * @return {Promise<SafeResult<EmployeeInfoDto, Error>>}
     */
    getEmployeeByDivisionTeamAssignmentId(divisionTeamAssignmentId) {
        return this.request(
            {
                action: ServiceAction.EMPLOYEE_INFO_BY_ASSIGNMENT,
                params: {
                    division_team_assignment_id: divisionTeamAssignmentId
                }
            },
            EmployeeInfoDto
        );
    }

    /**
     * @description Получение информации о всех рангах позиций
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/position/getPositionRankist
     *
     * @return {Promise<SafeResult<PositionRankDto[], Error>>}
     */
    getPositionRanks() {
        return this.request(
            {
                action: ServiceAction.POSITION_RANKS_GET
            },
            PositionRankDto
        );
    }

    /**
     * @description Получение информации о всех категориях позиций
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/position/getPositionCategoryList
     *
     * @return {Promise<SafeResult<PositionCategoryDto[], Error>>}
     */
    getPositionCategories() {
        return this.request(
            {
                action: ServiceAction.POSITION_CATEGORIES_GET
            },
            PositionCategoryDto
        );
    }

    /**
     * @description Получение информации о всех типах причин
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/reason/getReasonList
     *
     * @return {Promise<SafeResult<ReasonDto[], Error>>}
     */
    getReasons() {
        return this.request(
            {
                action: ServiceAction.REASON_LIST_GET
            },
            ReasonDto
        );
    }

    /**
     * @description Получение идентификатора сотрудника из сессии
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/employee/getEmployeeId
     *
     * @return {Promise<SafeResult<number, Error>>}
     */
    getSessionEmployeeId() {
        return this.request(
            {
                action: ServiceAction.EMPLOYEE_ME_GET
            },
            Number
        );
    }

    /**
     * @description Получение информации о всех группах критериев важности
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/importancecriteria/findImportanceCriteriaGroupAll
     *
     * @return {Promise<SafeResult<ImportanceCriteriaGroupDto[], Error>>}
     */
    getImportanceCriteriaGroups() {
        return this.request(
            {
                action: ServiceAction.IMPORTANCE_CRITERIA_GROUP_LIST_GET
            },
            ImportanceCriteriaGroupDto
        );
    }

    /**
     * @description Получение информации о всех позициях критериев важности
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/importancecriteria/getPositionImportanceCriteriaList
     *
     * @return {Promise<SafeResult<PositionImportanceCriteriaDto[], Error>>}
     */
    getPositionImportanceCriteria() {
        return this.request(
            {
                action: ServiceAction.POSITION_IMPORTANCE_CRITERIA_LIST_GET
            },
            PositionImportanceCriteriaDto
        );
    }

    /**
     * @description Получение информации о всех критериях важности
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/importancecriteria/getImportanceCriteriaList
     *
     * @return {Promise<SafeResult<ImportanceCriteriaDto[], Error>>}
     */
    getImportanceCriteria() {
        return this.request(
            {
                action: ServiceAction.IMPORTANCE_CRITERIA_LIST
            },
            ImportanceCriteriaDto
        );
    }

    /**
     * @description Обновление типа причин
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/importancecriteria/recalculatePositionImportanceCriteria
     *
     * @param {RecalculatePositionImportanceCriteriaDto} recalculatePositionImportanceCriteria
     *
     * @returns {Promise<SafeResult<PositionImportanceCriteriaDto, Error>>}
     */
    recalculatePositionImportanceCriteria(recalculatePositionImportanceCriteria) {
        return this.request(
            {
                action: ServiceAction.POSITION_RECALCULATE_IMPORTANCE_CRITERIA_PUT,
                params: {
                    ...recalculatePositionImportanceCriteria
                }
            },
            PositionImportanceCriteriaDto
        );
    }

    /**
     * @description Получение информации о position successor
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/position/getPositionSuccessorList
     *
     * @returns {Promise<SafeResult<PositionSuccessorDto[], Error>>}
     */
    getPositionSuccessors() {
        return this.request(
            {
                action: ServiceAction.POSITION_SUCCESSOR_LIST_GET
            },
            PositionSuccessorDto
        );
    }

    /**
     * @description Создание позиции критериев важности
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/importancecriteria/createPositionImportanceCriteria
     *
     * @param {CreatePositionImportanceCriteriaInputDto} positionImportanceCriteria
     * @returns {Promise<SafeResult<PositionImportanceCriteriaDto, Error>>}
     */
    createPositionImportanceCriteria({ weight, value, position_id, importance_criteria_id }) {
        return this.request(
            {
                action: ServiceAction.POSITION_IMPORTANCE_CRITERIA_CREATE,
                params: {
                    weight,
                    value,
                    position_id,
                    importance_criteria_id
                }
            },
            PositionImportanceCriteriaDto
        );
    }

    /**
     * @description Обновление позиции критериев важности
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/importancecriteria/updatePositionImportanceCriteria
     *
     * @param {number} id
     * @param {UpdatePositionImportanceCriteriaInputDto} positionImportanceCriteria
     * @returns {Promise<SafeResult<PositionImportanceCriteriaDto, Error>>}
     */
    updatePositionImportanceCriteria(id, { weight, value, position_id, importance_criteria_id }) {
        return this.request(
            {
                action: ServiceAction.POSITION_IMPORTANCE_CRITERIA_UPDATE,
                pathParams: { id },
                params: {
                    weight,
                    value,
                    position_id,
                    importance_criteria_id
                }
            },
            PositionImportanceCriteriaDto
        );
    }

    /**
     * @description Удаление позиции критериев важности
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/importancecriteria/deleteReasonType
     *
     * @param {number} id
     * @returns {Promise<SafeResult<Boolean, Error>>}
     */
    deletePositionImportanceCriteria(id) {
        return this.request(
            {
                action: ServiceAction.POSITION_IMPORTANCE_CRITERIA_DELETE,
                pathParams: { id }
            },
            Boolean
        );
    }

    /**
     * @description Создание position successor
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/position/createPositionSuccessor
     *
     *
     * @param {CreatePositionSuccessorInputDto} positionSuccessor
     *
     * @returns {Promise<SafeResult<PositionSuccessorDto, Error>>}
     */
    createPositionSuccessor({
        date_commit_hr,
        date_priority,
        employee_id,
        position_id,
        position_group_id,
        reason_id_inclusion,
        reason_id_exclusion,
        comment_inclusion,
        comment_exclusion,
        document_url_inclusion,
        document_url_exclusion
    }) {
        return this.request(
            {
                action: ServiceAction.POSITION_SUCCESSOR_CREATE,
                params: {
                    date_commit_hr,
                    date_priority,
                    employee_id,
                    position_id,
                    position_group_id,
                    reason_id_inclusion,
                    reason_id_exclusion,
                    comment_inclusion,
                    comment_exclusion,
                    document_url_inclusion,
                    document_url_exclusion
                }
            },
            PositionSuccessorDto
        );
    }

    /**
     * @description Обновление position successor
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/position/updatePositionSuccessor
     *
     *
     * @param {number} id
     * @param {UpdatePositionSuccessorInputDto} positionSuccessor
     *
     * @returns {Promise<SafeResult<PositionSuccessorDto, Error>>}
     */
    updatePositionSuccessor(
        id,
        {
            date_commit_hr,
            date_priority,
            employee_id,
            position_id,
            position_group_id,
            reason_id_inclusion,
            reason_id_exclusion,
            comment_inclusion,
            comment_exclusion,
            document_url_inclusion,
            document_url_exclusion
        }
    ) {
        return this.request(
            {
                action: ServiceAction.POSITION_SUCCESSOR_UPDATE,
                pathParams: { id },
                params: {
                    date_commit_hr,
                    date_priority,
                    employee_id,
                    position_id,
                    position_group_id,
                    reason_id_inclusion,
                    reason_id_exclusion,
                    comment_inclusion,
                    comment_exclusion,
                    document_url_inclusion,
                    document_url_exclusion
                }
            },
            PositionSuccessorDto
        );
    }

    /**
     * @description Удаление position successor
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/position/deletePositionSuccessor
     *
     *
     * @param {number} id
     *
     * @returns {Promise<SafeResult<Boolean, Error>>}
     */
    deletePositionSuccessor(id) {
        return this.request(
            {
                action: ServiceAction.POSITION_SUCCESSOR_DELETE,
                pathParams: { id }
            },
            Boolean
        );
    }

    /**
     * @description Создание position successor readiness
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/position/createPositionSuccessorReadiness
     *
     * @param {CreatePositionSuccessorReadinessInputDto} positionSuccessorReadiness
     *
     * @returns {Promise<SafeResult<PositionSuccessorReadinessDto, Error>>}
     */
    createPositionSuccessorReadiness({ position_successor_id, readiness_id }) {
        return this.request(
            {
                action: ServiceAction.POSITION_SUCCESSOR_READINESS_CREATE,
                params: {
                    position_successor_id,
                    readiness_id
                }
            },
            PositionSuccessorReadinessDto
        );
    }

    /**
     * @description Обновление position successor readiness
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/position/updatePositionSuccessorReadiness
     *
     * @param {number} id
     * @param {UpdatePositionSuccessorReadinessInputDto} positionSuccessorReadiness
     *
     * @returns {Promise<SafeResult<PositionSuccessorReadinessDto, Error>>}
     */
    updatePositionSuccessorReadiness(id, { position_successor_id, readiness_id }) {
        return this.request(
            {
                action: ServiceAction.POSITION_SUCCESSOR_READINESS_UPDATE,
                pathParams: { id },
                params: {
                    position_successor_id,
                    readiness_id
                }
            },
            PositionSuccessorReadinessDto
        );
    }

    /**
     * @description Удаление position successor readiness
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/position/deletePositionSuccessorReadiness
     *
     * @param {number} id
     *
     * @returns {Promise<SafeResult<Boolean, Error>>}
     */
    deletePositionSuccessorReadiness(id) {
        return this.request(
            {
                action: ServiceAction.POSITION_SUCCESSOR_READINESS_DELETE,
                pathParams: { id }
            },
            Boolean
        );
    }

    /**
     * @description Закрывает назначение сотрудника в команду по указанным параметрам
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/role/clearEmployeeRole
     *
     * @param {number} legalEntityId
     * @param {number} roleId
     * @param {number} employeeId
     *
     * @returns {Promise<SafeResult<Boolean, Error>>}
     */
    clearEmployeeRole({ legalEntityId, roleId, employeeId }) {
        return this.request(
            {
                action: ServiceAction.CLEAR_EMPLOYEE_ROLE,
                queryParams: {
                    legal_entity_id: legalEntityId,
                    role_id: roleId,
                    employee_id: employeeId
                }
            },
            Boolean
        );
    }

    /**
     * @description Получение информации о всех значениях позиций
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/position/getPositionImportanceList
     *
     * @returns {Promise<SafeResult<PositionImportanceDto[], Error>>}
     */
    getPositionImportances() {
        return this.request(
            {
                action: ServiceAction.POSITION_IMPORTANCE_LIST_GET
            },
            PositionImportanceDto
        );
    }

    /**
     * @description Получение всех связок позиции и критичности позиции.
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/position/getAllPositionPositionImportance
     *
     * @param {Object} params
     * @param {number[]} params.divisionTeamIds Фильтрация записей по division_team
     * @param {number[]} params.divisionTeamAssignmentIds Фильтрация записей по division_team_assigment
     * @param {number[]} params.positionIds Массив идентификаторов должностей.
     *
     * @param {Object} filterParams
     * @param {boolean} filterParams.withClosed Запрашиваются ли закрытые записи.
     *
     * @returns {Promise<SafeResult<PositionImportanceBindingDto[], Error>>}
     */
    getPositionImportanceBindingsByFilter(
        { divisionTeamIds, divisionTeamAssignmentIds, positionIds } = {},
        { withClosed } = {}
    ) {
        return this.request(
            {
                action: ServiceAction.POSITION_POSITION_IMPORTANCE_GET,
                params: {
                    with_closed: withClosed,
                    division_team: divisionTeamIds,
                    users: divisionTeamAssignmentIds,
                    position_id: positionIds
                }
            },
            PositionImportanceBindingDto
        );
    }

    /**
     * @description Создание связки позиции и критичности позиции.
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/position/createPositionPositionImportance
     *
     * @param {CreatePositionImportanceBindingInputDto} positionImportanceBindingInput
     * @returns {Promise<SafeResult<PositionImportanceBindingDto[], Error>>}
     */
    createPositionImportanceBinding(positionImportanceBindingInput) {
        return this.request(
            {
                action: ServiceAction.POSITION_POSITION_IMPORTANCE_CREATE_POST,
                params: positionImportanceBindingInput
            },
            PositionImportanceBindingDto
        );
    }

    /**
     * @description удаление связки позиции и критичности позиции
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/position/deletePositionPositionImportance
     *
     * @param {number} id
     * @returns {Promise<SafeResult<Object, Error>>}
     */
    deletePositionPositionImportanceBinding(id) {
        return this.request(
            {
                action: ServiceAction.POSITION_POSITION_IMPORTANCE_DELETE,
                pathParams: {
                    id
                }
            },
            Object
        );
    }

    /**
     * @description Получение списка Position по идентификатору сотрудника и идентификатору подразделения
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/position/getPositionAssignmentByEmployeeId
     *
     * @param {number} employeeId
     * @param {number[]} divisionIds
     * @returns {Promise<SafeResult<PositionAssignmentDto[], Error>>}
     */
    getPositionAssignmentsByFilter({ employeeId, divisionIds } = {}) {
        return this.request(
            {
                action: ServiceAction.POSITION_FIND_GET,
                params: {
                    employee_id: employeeId,
                    division_id: divisionIds
                }
            },
            PositionAssignmentDto
        );
    }

    /**
     * @description Получение информации о списке групп подразделений
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/division/divisionGroupInfo
     *
     * @param {number[]} [groupIds]
     * @returns {Promise<SafeResult<DivisionGroupDto[], Error>>}
     */
    getDivisionGroups({ groupIds } = {}) {
        return this.request(
            {
                action: ServiceAction.DIVISION_GROUP_GET,
                params: {
                    id: groupIds
                }
            },
            DivisionGroupDto
        );
    }

    /**
     * @description Получение информации о подчиненных в команде подразделения по ИД команды
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/employee/getTeamDivisionSubordinatesTeam
     *
     * @param {number} divisionTeamId - Идентификатор команды подразделения
     *
     * @param {Object} [filters]
     * @param {boolean} [filters.withChildren] - Необходимо ли получать подчиненных от подчиненных
     * @returns {Promise<SafeResult<DivisionTeamAssignmentDto[], Error>>}
     */
    getEmployeeDivisionTeamSubordinatesByDivisionTeamId(divisionTeamId, { withChildren = false } = {}) {
        return this.request(
            {
                action: ServiceAction.EMPLOYEE_TEAM_DIVISION_SUBORDINATES_TEAM_GET,
                params: {
                    team: divisionTeamId,
                    withchilds: withChildren
                }
            },
            DivisionTeamAssignmentDto
        );
    }

    /**
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/work_experience_type/getAll
     * @description Получение информации о всех типах опыта работы
     *
     * @returns {Promise<SafeResult<DivisionInfoDto, Error>>}
     */
    getWorkExperienceTypes() {
        return this.request(
            {
                action: ServiceAction.WORK_EXPERIENCE_TYPE_GET
            },
            DivisionInfoDto
        );
    }

    /**
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/work_experience_type/create_1
     * @description Создание типа опыта работы
     *
     * @param {string} name
     * @param {string} description
     * @returns {Promise<SafeResult<DivisionTeamRoleDto, Error>>}
     */
    createWorkExperienceType({ name, description }) {
        return this.request(
            {
                action: ServiceAction.WORK_EXPERIENCE_TYPE_CREATE_POST,
                params: {
                    name,
                    description
                }
            },
            DivisionTeamRoleDto
        );
    }
}

/**
 *
 */
export { OrgStructureApiService };
