/**
 * @typedef {import('@goodt-common/infra').BaseDto} BaseDto
 */
/* eslint-disable camelcase */

import { buildRequest, BaseApiService, processRequestResult } from '@goodt-common/api';

import { ServiceAction } from './config';

import {
    KpiGroupRawDto,
    KpiRawDto,
    KpiDataRawDto,
    KpiMetricDto,
    EmployeeExperienceDto,
    EmployeeProjectDto,
    EmployeeProjectActivityDto,
    EmployeeProjectRoleDto,
    OperationResultDto,
    CompetenceFindDto,
    CompetenceCatalogDto,
    ScaleTypeDto,
    ScaleLevelDto,
    CompetenceDto,
    DivisionTeamCompetenceDto,
    DevelopmentGoalDto,
    EvaluationEventDto,
    DevelopmentFormDto,
    DevelopmentFormCatalogDto,
    EvaluationEventTypeEvaluationRoleDto,
    EvaluationEventTypeDto,
    CompetenceProfileLinkDto,
    CompetenceProfileLinkTypeDto,
    CompetenceProfilePositionDto,
    CompetenceProfileDto,
    ReasonDto,
    AssignmentAccordanceDto,
    DevelopmentGoalCompetenceDto,
    EventDto,
    ReceiverSystemNotificationDto,
    NotificationDynamicRecipientDto,
    NotificationTokenDto,
    SubstituteNotificationContentTemplateDto,
    NotificationContentTemplateDto,
    NotificationLogDto
} from './dto';

/**
 *
 */
class RostalentApiService extends BaseApiService {
    /**
     *
     * @param {import('@goodt-common/api').IApiServiceRequest | import('@goodt-common/api').IApiServiceRequestOptions} apiServiceRequest
     * @param {function(...args?: any[]): any} [DtoConstructor=BaseDto]
     * @return {typeof ReturnType<processRequestResult>}
     */
    async request(apiServiceRequest, DtoConstructor) {
        if ('action' in apiServiceRequest) {
            apiServiceRequest = buildRequest(apiServiceRequest);
        }
        return processRequestResult(await super.request(apiServiceRequest), DtoConstructor);
    }

    /**
     * @link https://goodt-dev.goodt.me:8486/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/ueppc/updateProgress
     * @description Обновление прогресса задачи по его идентификатору. Берется первый попавшийся field с params: {"type":"progress"}
     *
     *
     * @param {number} goalId - Идентификатор цели, для которой необходимо установить прогресс.
     *
     * @param {Object} options - Объект с параметрами для установки прогресса.
     * @param {string} options.progress - Прогресс, который необходимо установить для цели.
     *
     * @returns {Promise<SafeResult<String, Error>>}
     */
    setGoalProgress(goalId, { progress }) {
        return this.request(
            {
                action: ServiceAction.UEPPC_SET_PROGRESS_POST,
                params: {
                    goal_id: goalId,
                    progress
                }
            },
            String
        );
    }

    /**
     * @link https://goodt-dev.goodt.me:8486/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/kpi/getGroupList
     * @description Получение массива групп Kpi по идентификатору типа (type_id)
     *
     * @param {number|string} typeId - идентификатор типа.
     *
     * @return {Promise<SafeResult<KpiGroupRawDto[], Error>>}
     */
    getKpiGroupsByTypeId(typeId) {
        return this.request(
            {
                action: ServiceAction.KPI_GROUP_LIST_GET,
                params: {
                    type_id: typeId
                }
            },
            KpiGroupRawDto
        );
    }

    /**
     * @link https://goodt-dev.goodt.me:8486/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/kpi/getListByGroupId
     * @description Получение массива Kpi по идентификатору группы (group_id)
     *
     * @param {number|string} groupId - идентификатор группы
     * @return {Promise<SafeResult<KpiRawDto[], Error>>}
     */
    getKpiByGroupId(groupId) {
        return this.request(
            {
                action: ServiceAction.KPI_LIST_GET,
                params: {
                    group_id: groupId
                }
            },
            KpiRawDto
        );
    }

    /**
     * @link https://goodt-dev.goodt.me:8486/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/kpi/getData
     * @description Получение массива данных Kpi по идентификатору Kpi (kpi_id)
     *
     * @param {number|string} kpiId - идентификатор KPI
     * @return {Promise<SafeResult<KpiDataRawDto[], Error>>}
     */
    getKpiData(kpiId) {
        return this.request(
            {
                action: ServiceAction.KPI_DATA_LIST_GET,
                params: {
                    kpi_id: kpiId
                }
            },
            KpiDataRawDto
        );
    }

    /**
     * @link https://goodt-dev.goodt.me:8486/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/kpi_metric/getList_4
     * @description Получение массива Kpi метрик
     *
     * @return {Promise<SafeResult<KpiMetricDto[], Error>>}
     */
    getKpiMetrics() {
        return this.request(
            {
                action: ServiceAction.KPI_METRIC_LIST_GET
            },
            KpiMetricDto
        );
    }

    /**
     * @link https://goodt-dev.goodt.me:8486/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/employee/find
     * @description Получение информации о стаже работы сотрудника
     *
     * @param {[number|string]} employeeId - идентификатор работника
     * @return {Promise<SafeResult<EmployeeExperienceDto[], Error>>}
     */
    getEmployeeExperiences(employeeId) {
        return this.request(
            {
                action: ServiceAction.EMPLOYEE_WORK_EXPERIENCE_FIND_GET,
                params: {
                    employee_id: employeeId
                }
            },
            EmployeeExperienceDto
        );
    }

    /**
     * @link https://goodt-dev.goodt.me:8486/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/worker_project/getList_3
     * @description Получение массива WorkerProject по набору параметров
     *
     * @return {Promise<SafeResult<EmployeeProjectDto[], Error>>}
     */
    getEmployeesProjects() {
        return this.request(
            {
                action: ServiceAction.WORKER_PROJECT_LIST_GET
            },
            EmployeeProjectDto
        );
    }

    /**
     * @link https://goodt-dev.goodt.me:8486/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/worker_project_activity/getList_2
     * @description Получение массива WorkerProjectActivity по набору параметров
     *
     * @return {Promise<SafeResult<EmployeeProjectActivityDto[], Error>>}
     */
    getEmployeesProjectsActivities() {
        return this.request(
            {
                action: ServiceAction.WORKER_PROJECT_LIST_GET
            },
            EmployeeProjectActivityDto
        );
    }

    /**
     * @link https://goodt-dev.goodt.me:8486/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/worker_project_role/getList
     * @description Получение массива WorkerProjectRole по набору параметров
     *
     * @return {Promise<SafeResult<EmployeeProjectRoleDto[], Error>>}
     */
    getEmployeesProjectsRoles() {
        return this.request(
            {
                action: ServiceAction.WORKER_PROJECT_ROLE_LIST_GET
            },
            EmployeeProjectRoleDto
        );
    }

    /**
     * @link https://goodt-dev.goodt.me:8486/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/worker_project/create_3
     * @description Создание нового WorkerProject
     *
     * @param {EmployeeProjectInput} employeeProjectInput
     *
     * @return {Promise<SafeResult<Number, Error>>}
     */
    createEmployeeProject(employeeProjectInput) {
        return this.request(
            {
                action: ServiceAction.WORKER_PROJECT_UPDATE_POST,
                params: employeeProjectInput
            },
            Number
        );
    }

    /**
     * @link https://goodt-dev.goodt.me:8486/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/worker_project_activity/create_2
     * @description Создание нового WorkerProjectActivity
     *
     * @param {EmployeeProjectActivityInput} employeeProjectActivityInput
     *
     * @return {Promise<SafeResult<Number, Error>>}
     */
    createEmployeeProjectActivity(employeeProjectActivityInput) {
        return this.request(
            {
                action: ServiceAction.WORKER_PROJECT_ACTIVITY_UPDATE_POST,
                params: employeeProjectActivityInput
            },
            Number
        );
    }

    /**
     * @link https://goodt-dev.goodt.me:8486/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/worker_project_role/create
     * @description Создание нового WorkerProjectRole
     *
     * @param {EmployeeProjectRoleInput} employeeProjectRoleInput
     *
     * @return {Promise<SafeResult<Number, Error>>}
     */
    createEmployeeProjectRole(employeeProjectRoleInput) {
        return this.request(
            {
                action: ServiceAction.WORKER_PROJECT_ROLE_UPDATE_POST,
                params: employeeProjectRoleInput
            },
            Number
        );
    }

    /**
     * @link https://goodt-dev.goodt.me:8486/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/event/customEmailEventCode
     * @description Метод по отправке сообщений в кафку (кастомные уведомления). Логика разная в зависимости от переданного кода
     *
     * @param {CustomEmailEventCodeInput} event
     * @return {Promise<SafeResult<OperationResultDto, Error>>}
     */
    triggerCustomEmailEventCode(event) {
        return this.request(
            {
                action: ServiceAction.EVENT_CUSTOM_EMAIL_EVENT_CODE_POST,
                params: event
            },
            OperationResultDto
        );
    }

    /**
     * @description Создание новой компетенции
     * @link https://goodt-dev.goodt.me:8486/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/competence/createCompetence
     *
     * @param {CompetenceInput} competenceInput
     * @return {Promise<SafeResult<Number, Error>>}
     */
    createCompetence(competenceInput) {
        return this.request(
            {
                action: ServiceAction.COMPETENCE_UPDATE_POST,
                params: competenceInput
            },
            Number
        );
    }

    /**
     * @link https://goodt-dev.goodt.me:8486/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/competence/updateCompetence,
     * @description Обновление компетенции по его идентификатору
     *
     * @param {number} id - идентификатор кометенции
     * @param {CompetenceInput} competenceInput
     *
     * @return {Promise<SafeResult<Number, Error>>}
     */
    updateCompetence(id, competenceInput) {
        return this.request(
            {
                action: ServiceAction.COMPETENCE_UPDATE_PUT,
                params: competence,
                pathParams: { id }
            },
            Number
        );
    }

    /**
     * @link https://goodt-dev.goodt.me:8486/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/competence/competenceFindNew
     * @description По умолчанию флаг is_ids_exclude = false, найдет ID переданные в массиве ids, если флаг = true то найдет все кроме переданных в ids.
     *
     * @param {Object} filter
     * @param {number[]} filter.ids
     * @param {string} filter.name
     * @param {number[]} filter.catalogIds
     * @param {number} filter.masterId
     *
     * @param {Object} [options]
     * @param {boolean} [options.withClosed]
     * @param {boolean} [options.isIdsExclude]
     *
     * @return {Promise<SafeResult<CompetenceFindDto, Error>>}
     */
    getCompetencesByFilterPost({ ids, name, catalogIds, masterId }, { withClosed, isIdsExclude } = {}) {
        return this.request(
            {
                action: ServiceAction.COMPETENCE_FIND_NEW_POST,
                params: {
                    ids,
                    name,
                    catalog_ids: catalogIds,
                    is_ids_exclude: isIdsExclude,
                    master_id: masterId,
                    with_closed: withClosed
                }
            },
            CompetenceFindDto
        );
    }

    /**
     * @link https://goodt-dev.goodt.me:8486/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/competence/divisionList
     * @description Получение массива рекомендованных компетенции для подразделения по идентификатору подразделения (team)
     *
     * @param {number} divisionTeamId
     * @param {Object} [options]
     * @param {boolean} [options.withClosed]
     *
     * @return {Promise<SafeResult<DivisionTeamCompetenceDto[], Error>>}
     */
    getDivisionTeamCompetences(divisionTeamId, { withClosed = true } = {}) {
        return this.request(
            {
                action: ServiceAction.COMPETENCE_DIVISION_LIST_GET,
                params: {
                    team: divisionTeamId,
                    with_closed: withClosed
                }
            },
            DivisionTeamCompetenceDto
        );
    }

    /**
     * @link https://goodt-dev.goodt.me:8486/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/competence/divisionDelete
     * @description Удаление рекомендованной компетенции для подразделения по идентификатору
     *
     * @param {DivisionTeamCompetenceDataDeleteInput} deleteCompetenceInput
     *
     * @return {Promise<SafeResult<Number, Error>>}
     */
    deleteDivisionTeamCompetence(deleteCompetenceInput) {
        return this.request(
            {
                action: ServiceAction.COMPETENCE_DIVISION_DELETE_POST,
                params: deleteCompetenceInput
            },
            Number
        );
    }

    /**
     * @link https://goodt-dev.goodt.me:8486/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/competence/catalogList_1
     * @description Получение массива каталога компетенций
     *
     * @return {Promise<SafeResult<CompetenceCatalogDto[], Error>>}
     */
    getCompetenceCatalogs() {
        return this.request(
            {
                action: ServiceAction.COMPETENCE_CATALOG_LIST_GET
            },
            CompetenceCatalogDto
        );
    }

    /**
     * @link https://goodt-dev.goodt.me:8486/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/development_goal/list_4
     * @description Получение массива целей развития
     *
     * @return {Promise<SafeResult<DevelopmentGoalDto[], Error>>}
     */
    getDevelopmentGoals() {
        return this.request(
            {
                action: ServiceAction.DEVELOPMENT_GOAL_LIST_GET
            },
            DevelopmentGoalDto
        );
    }

    /**
     * @link https://goodt-dev.goodt.me:8486/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/scale/getScaleType
     * @description Получение информации о типе шкалы
     *
     * @param {number|string} id - идентификатор типа шкалы
     *
     * @return {Promise<SafeResult<ScaleTypeDto, Error>>}
     */
    getScaleTypeById(id) {
        return this.request(
            {
                action: ServiceAction.SCALE_TYPE_GET,
                pathParams: { id }
            },
            ScaleTypeDto
        );
    }

    /**
     * @link https://goodt-dev.goodt.me:8486/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/evaluation/getEvaluationEvent
     * @description Получение мероприятия оценки
     *
     * @param {number|string} id
     * @return {Promise<SafeResult<EvaluationEventDto, Error>>}
     */
    getEvaluationEventById(id) {
        return this.request(
            {
                action: ServiceAction.EVALUATION_EVENT_GET,
                pathParams: { id }
            },
            EvaluationEventDto
        );
    }

    /**
     * @link https://goodt-dev.goodt.me:8486/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/competence/getCompetencesByCompetenceProfile
     * @description Получение информации о компетенциях по идентификатору профиля компетенции
     *
     * @param {number|string} competenceProfileId - Идентификатор профиля компетенции
     *
     * @return {Promise<SafeResult<CompetenceDto[], Error>>}
     */
    getCompetencesByCompetenceProfileId(competenceProfileId) {
        return this.request(
            {
                action: ServiceAction.COMPETENCE_PROFILE_COMPETENCE_COMPETENCE_LIST_GET,
                params: {
                    competence_profile_id: competenceProfileId
                }
            },
            CompetenceDto
        );
    }

    /**
     * @link https://goodt-dev.goodt.me:8486/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/competence/catalogList_1
     * @description Получение массива компетенций по имени (name) или идентификатору каталога (catalog_id)
     *
     * @param {Object} filter
     * @param {number[]|string[]} filter.ids
     * @param {number|string} filter.catalogId
     * @param {string} filter.name
     * @param {Object} options
     * @param {boolean} options.withClosed
     *
     * @return {Promise<SafeResult<CompetenceDto[], Error>>}
     */
    getCompetencesByFilter({ ids, catalogId, name }, { withClosed }) {
        return this.request(
            {
                action: ServiceAction.COMPETENCE_LIST_GET,
                params: {
                    id: ids,
                    name,
                    catalog_id: catalogId,
                    with_closed: withClosed
                }
            },
            CompetenceDto
        );
    }

    /**
     * @link https://goodt-dev.goodt.me:8486/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/development_form/list_8
     * @description Получение массива форм развития по идентификатору каталога (catalog_id)
     *
     * @param {number|string} catalogId
     *
     * @return {Promise<SafeResult<DevelopmentFormDto[], Error>>}
     */
    getDevelopmentForms(catalogId) {
        return this.request(
            {
                action: ServiceAction.DEVELOPMENT_FORM_LIST_GET,
                params: {
                    catalog_id: catalogId
                }
            },
            DevelopmentFormDto
        );
    }

    /**
     * @link https://goodt-dev.goodt.me:8486/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/development_form/catalogList
     * @description Получение массива каталога форм развития
     *
     * @return {Promise<SafeResult<DevelopmentFormCatalogDto[], Error>>}
     */
    getDevelopmentFormCatalogs() {
        return this.request(
            {
                action: ServiceAction.DEVELOPMENT_FORM_CATALOG_LIST_GET
            },
            DevelopmentFormCatalogDto
        );
    }

    /**
     * @link https://goodt-dev.goodt.me:8486/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/evaluation/getAllEvaluationEventTypeEvaluationRole
     * @description Получение актуальных связок роли оценщика и типа мероприятия оценки
     *
     * @param {number | string} evaluationEventTypeId - Идентификатор типа мероприятия оценки
     *
     * @return {Promise<SafeResult<EvaluationEventTypeEvaluationRoleDto[], Error>>}
     */
    getEvaluationEventTypeRoleBindings({ evaluationEventTypeId }) {
        return this.request(
            {
                action: ServiceAction.EVALUATION_EVENT_TYPE_EVALUATION_ROLE_GET,
                params: {
                    evaluation_event_type_id: evaluationEventTypeId
                }
            },
            EvaluationEventTypeEvaluationRoleDto
        );
    }

    /**
     * @link https://goodt-dev.goodt.me:8486/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/evaluation/getEvaluationEventType
     * @description Получение справочника типов мероприятий оценки
     *
     * @param {number | string } id - Идентификатор справочника
     *
     * @return {Promise<SafeResult<EvaluationEventTypeDto[], Error>>}
     */
    getEvaluationEventTypes(id) {
        return this.request(
            {
                action: ServiceAction.EVALUATION_EVENT_TYPE_GET,
                pathParams: { id }
            },
            EvaluationEventTypeDto
        );
    }

    /**
     * @link https://goodt-dev.goodt.me:8486/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/onboarding/downloadOnboardingTaskFile
     * @description Скачивание файла
     *
     * @param {number | string} id - Идентификатор файла
     *
     * @return {Promise<SafeResult<Object, Error>>}
     */
    downloadTaskFile(id) {
        return this.request(
            {
                action: ServiceAction.ONBOARDING_TASK_FILE_DOWNLOAD_GET,
                pathParams: { id }
            },
            Object
        );
    }

    /**
     * @link https://goodt-dev.goodt.me:8486/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/competence_profile_link/getCompetenceProfileLink
     * @description Получение записи о связи пары профилей должностей и типа их связи по ID
     *
     * @param {number | string} id - Идентификатор
     *
     * @return {Promise<SafeResult<CompetenceProfileLinkDto, Error>>}
     */
    getCompetenceProfileLinkById(id) {
        return this.request(
            {
                action: ServiceAction.COMPETENCE_PROFILE_LINK_GET,
                pathParams: { id }
            },
            CompetenceProfileLinkDto
        );
    }

    /**
     * @link https://goodt-dev.goodt.me:8486/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/competence_profile_link/getAllCompetenceProfileLinkType
     * @description Получение списка типов связей профилей должностей
     *
     * @return {Promise<SafeResult<CompetenceProfileLinkTypeDto, Error>>}
     */
    getCompetenceProfileLinkTypes() {
        return this.request(
            {
                action: ServiceAction.COMPETENCE_PROFILE_LINK_TYPE_GET
            },
            CompetenceProfileLinkTypeDto
        );
    }

    /**
     * @link https://goodt-dev.goodt.me:8486/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/competence/getCompetenceProfilePosition
     * @description Получение информации о связи профиля компетенции и позиции
     *
     * @param {number |string} id - Идентификатор связи
     *
     * @return {Promise<SafeResult<CompetenceProfilePositionDto, Error>>}
     */
    getCompetenceProfilePositionById(id) {
        return this.request(
            {
                action: ServiceAction.COMPETENCE_PROFILE_POSITION_BY_ID_GET,
                pathParams: { id }
            },
            CompetenceProfilePositionDto
        );
    }

    /**
     * @link https://goodt-dev.goodt.me:8486/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/competence/getCompetenceProfile
     * @description Получение информации о профиле компетенций
     *
     * @param {number | string} id - Идентификатор профиля компетенций
     *
     * @return {Promise<SafeResult<CompetenceProfileDto, Error>>}
     */
    getCompetenceProfileById(id) {
        return this.request(
            {
                action: ServiceAction.COMPETENCE_PROFILE_BY_ID_GET,
                pathParams: { id }
            },
            CompetenceProfileDto
        );
    }

    /**
     * @link https://goodt-dev.goodt.me:8486/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/scale/getScaleLevels
     * @description Получение информации об актуальных баллах шкал
     *
     * @return {Promise<SafeResult<ScaleLevelDto[], Error>>}
     */
    getScaleLevels() {
        return this.request(
            {
                action: ServiceAction.SCALE_LEVEL_GET
            },
            ScaleLevelDto
        );
    }

    /**
     * @link https://goodt-dev.goodt.me:8486/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/reason/getReasonList
     * @description Получение информации о всех типах причин
     *
     * @return {Promise<SafeResult<ReasonDto[], Error>>}
     */
    getReasons() {
        return this.request(
            {
                action: ServiceAction.REASON_LIST_GET
            },
            ReasonDto
        );
    }

    /**
     * @link https://goodt-dev.goodt.me:8486/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/competence/assignmentAccordance
     * @description Получение процента соответствия назначений по компетенциям
     *
     * @param {Object} options
     * @param {number[] | string[]} options.assignmentFrom
     * @param {number[] | string[]} [options.assignmentTo]
     *
     * @return {Promise<SafeResult<AssignmentAccordanceDto[], Error>>}
     */
    getCompetenceAssignmentsAccordances({ assignmentFrom, assignmentTo }) {
        return this.request(
            {
                action: ServiceAction.COMPETENCE_ASSIGNMENT_ACCORDANCE_GET,
                params: {
                    assignments_from: assignmentFrom,
                    assignments_to: assignmentTo
                }
            },
            AssignmentAccordanceDto
        );
    }

    /**
     * @link https://goodt-dev.goodt.me:8486/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/development_goal/list_3
     * @description Получение массива DevelopmentGoalCompetence
     *
     * @return {Promise<SafeResult<DevelopmentGoalCompetenceDto[], Error>>}
     */
    getDevelopmentGoalCompetences() {
        return this.request(
            {
                action: ServiceAction.DEVELOPMENT_GOAL_COMPETENCE_LIST_GET
            },
            DevelopmentGoalCompetenceDto
        );
    }

    /**
     * @link https://goodt-dev.goodt.me:8486/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/competence/getCompetence
     * @description Получение информации о компетенции, её индикаторах и уровнях индикаторов
     *
     * @param {number | string} id - Идентификатор компетенции
     *
     * @return {Promise<SafeResult<CompetenceDto, Error>>}
     */
    getCompetenceById(id) {
        return this.request(
            {
                action: ServiceAction.COMPETENCE_BY_ID_GET,
                pathParams: { id }
            },
            CompetenceDto
        );
    }

    /**
     * @link https://goodt-dev.goodt.me:8486/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/competence/deleteCompetenceEvaluationEvent
     * @description Удаление связки компетенции и мероприятия оценки
     *
     * @param {Object} options
     * @param {number | string} options.evaluationEventId
     * @param {number | string} options.competenceId
     *
     * @return {Promise<SafeResult<OperationResultDto, Error>>}
     */
    deleteCompetenceEvaluationEventBinding({ evaluationEventId, competenceId }) {
        return this.request(
            {
                action: ServiceAction.COMPETENCE_EVALUATION_EVENT_DELETE,
                pathParams: {
                    evaluation_event_id: evaluationEventId,
                    competence_id: competenceId
                }
            },
            OperationResultDto
        );
    }

    /**
     * @link https://goodt-dev.goodt.me:8486/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/event/updateDtaRotation
     * @description Создание уведомления для работника о том, что ему нужно подтвердить статус ротации. Обязательные поля: "author_name", "button_link", "division_team_assignment_id"
     *
     * @param {EventInput} eventInput
     *
     * @return {Promise<SafeResult<EventDto, Error>>}
     */
    createEventUpdatedTaRotation(eventInput) {
        return this.request(
            {
                action: ServiceAction.EVENT_CREATE_UPDATED_TA_ROTATION_POST,
                params: eventInput
            },
            EventDto
        );
    }

    /**
     * @link https://goodt-dev.goodt.me:8486/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/evaluation/createTaskByParams
     * @description Метод ищет по task_id дочерний с task_type = 172 и заполненным task_type_field_id = 217. Если не находит дочерний таск или пустой филд - ошибка. Входные параметры для метода. Варианты использования: 1) task_id, employee_id, indicator_level_id, comment; 2) task_id, employee_id, competence_id, scale_level_id, comment
     *
     * @param {ParamsForBffPollResultInput} pollKeyResultInput
     *
     * @return {Promise<SafeResult<OperationResultDto, Error>>}
     */
    createEvaluationPollKeyResult(pollKeyResultInput) {
        return this.request(
            {
                action: ServiceAction.EVALUATION_BFF_POLL_RESULT_POST,
                params: pollKeyResultInput
            },
            OperationResultDto
        );
    }

    /**
     * @link https://goodt-dev.goodt.me:8486/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/evaluation/deleteEvaluationEvent
     * @description Удаление мероприятия оценки
     *
     * @param {number | string} id - Идентификатор мероприятия оценки
     *
     * @return {Promise<SafeResult<OperationResultDto, Error>>}
     */
    deleteEvaluationEvent(id) {
        return this.request(
            {
                action: ServiceAction.EVALUATION_EVENT_DELETE,
                pathParams: { id }
            },
            OperationResultDto
        );
    }

    /**
     * @link https://goodt-dev.goodt.me:8486/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/competence_profile_link/createCompetenceProfileLink
     * @description Создание записи о типе связи пары профилей должностей
     *
     * @param {CompetenceProfileLinkInput} competenceProfileLinkInput
     *
     * @return {Promise<SafeResult<CompetenceProfileLinkDto, Error>>}
     */
    createCompetenceProfileLink(competenceProfileLinkInput) {
        return this.request(
            {
                action: ServiceAction.COMPETENCE_PROFILE_LINK_POST,
                params: competenceProfileLinkInput
            },
            CompetenceProfileLinkDto
        );
    }

    /**
     * @link https://goodt-dev.goodt.me:8486/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/competence_profile_link/deleteCompetenceProfileLink
     * @description Закрытие (date_to != null) записи о типе связи пары профилей должностей по ID
     *
     * @param {number | string} id - Идентификатор
     *
     * @return {Promise<SafeResult<OperationResultDto, Error>>}
     */
    deleteCompetenceProfileLink(id) {
        return this.request(
            {
                action: ServiceAction.COMPETENCE_PROFILE_LINK_DELETE,
                pathParams: { id }
            },
            OperationResultDto
        );
    }

    /**
     * @link https://goodt-dev.goodt.me:8486/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/competence/updateCompetenceProfilePosition
     * @description Обновление связи профиля компетенции и позиции
     *
     * @param {number | string} id - Идентификатор связи
     * @param {CompetenceProfilePositionInput} profilePosition
     *
     * @return {Promise<SafeResult<CompetenceProfilePositionDto, Error>>}
     */
    updateCompetenceProfilePosition(id, profilePosition) {
        return this.request(
            {
                action: ServiceAction.COMPETENCE_PROFILE_POSITION_UPDATE_PUT,
                pathParams: { id },
                params: profilePosition
            },
            CompetenceProfilePositionDto
        );
    }

    /**
     * @link https://goodt-dev.goodt.me:8486/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/competence/update_18
     * @description Обновление профиля компетенций
     *
     * @param {number | string} id - Идентификатор профиля компетенций
     * @param {CompetenceProfileInput} competenceProfileInput
     *
     * @return {Promise<SafeResult<CompetenceProfileDto, Error>>}
     */
    updateCompetenceProfile(id, competenceProfileInput) {
        return this.request(
            {
                action: ServiceAction.COMPETENCE_PROFILE_PUT,
                pathParams: { id },
                params: competenceProfileInput
            },
            CompetenceProfileDto
        );
    }

    /**
     * @link https://goodt-dev.goodt.me:8486/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/event/updateDtaRotationCompleted
     * @description Создание уведомления работника о том, что он актуализировал. Обязательные поля: "author_name", "division_team_assignment_id"
     *
     * @param {EventInput} eventInput
     *
     * @return {Promise<SafeResult<EventDto, Error>>}
     */
    createEventRotationCompleted(eventInput) {
        return this.request(
            {
                action: ServiceAction.EVENT_CREATE_UPDATED_TA_ROTATION_COMPLETED_POST,
                params: eventInput
            },
            EventDto
        );
    }

    /**
     * @link https://goodt-dev.goodt.me:8486/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/event/createTaskAchievement
     * @description Создание факта фиксации достижения цели
     *
     * @param {TaskAchievementInput} taskAchievementInput
     *
     * @return {Promise<SafeResult<EventDto, Error>>}
     */
    createEventTaskAchievement(taskAchievementInput) {
        return this.request(
            {
                action: ServiceAction.EVENT_CREATE_TASK_ACHIEVEMENT_POST,
                params: taskAchievementInput
            },
            EventDto
        );
    }

    /**
     * @link https://goodt-dev.goodt.me:8486/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/event/createKrAchievementEvent
     * @description Фиксация факта фиксации достижения ключевого результата. Метод ожидает во входных данных task_type = 4
     *
     * @param {KrAchievementInput} keyResultAchievementInput
     *
     * @return {Promise<SafeResult<EventDto, Error>>}
     */
    createEventKeyResultAchievement(keyResultAchievementInput) {
        return this.request(
            {
                action: ServiceAction.EVENT_CREATE_KR_ACHIEVEMENT_POST,
                params: keyResultAchievementInput
            },
            EventDto
        );
    }

    /**
     * @link https://goodt-dev.goodt.me:8486/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/event/createKrProgress
     * @description Фиксация прогресса в достижении ключевого результата
     *
     * @param {KrProgressInput} keyResultProgressInput
     *
     * @return {Promise<SafeResult<EventDto, Error>>}
     */
    createKeyResultProgress(keyResultProgressInput) {
        return this.request(
            {
                action: ServiceAction.EVENT_CREATE_KR_PROGRESS_POST,
                params: keyResultProgressInput
            },
            EventDto
        );
    }

    /**
     * @link https://goodt-dev.goodt.me:8486/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/notification_receiver_system/getAllNotificationReceiverSystem
     * @description Получение массива notification_receiver_system
     *
     * @return {Promise<SafeResult<ReceiverSystemNotificationDto[], Error>>}
     */
    getReceiverSystemNotifications() {
        return this.request(
            {
                action: ServiceAction.RECEIVER_SYSTEM_NOTIFICATION_GET
            },
            ReceiverSystemNotificationDto
        );
    }

    /**
     * @link https://goodt-dev.goodt.me:8486/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/notification_recipient/getDynamicRecipients
     * @description Получение записей из таблицы notification_recipient, исключая static_employee и static_division
     *
     * @return {Promise<SafeResult<NotificationDynamicRecipientDto[], Error>>}
     */
    getNotificationDynamicRecipients() {
        return this.request(
            {
                action: ServiceAction.NOTIFICATION_DYNAMIC_RECIPIENT_GET
            },
            NotificationDynamicRecipientDto
        );
    }

    /**
     * @link https://goodt-dev.goodt.me:8486/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/notification_token/getAll
     * @description Получение массива notification_token
     *
     * @return {Promise<SafeResult<NotificationTokenDto[], Error>>}
     */
    getNotificationTokens() {
        return this.request(
            {
                action: ServiceAction.NOTIFICATION_TOKEN_GET
            },
            NotificationTokenDto
        );
    }

    /**
     * @link https://goodt-dev.goodt.me:8486/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/notification_template_content/findSubstitutes
     * @description Получение краткой информации из таблицы notification_template_content. При полученном notificationTemplateContentId шаблон с данным идентификатором исключается
     *
     * @param {import('./dto.d.ts').SubstituteNotificationContentTemplateInput} substituteNotificationContentTemplateInput
     * @return {Promise<SafeResult<SubstituteNotificationContentTemplateDto[], Error>>}
     */
    getSubstituteNotificationContentTemplates(substituteNotificationContentTemplateInput) {
        return this.request(
            {
                action: ServiceAction.SUBSTITUTE_NOTIFICATION_CONTENT_TEMPLATE_GET,
                params: substituteNotificationContentTemplateInput
            },
            SubstituteNotificationContentTemplateDto
        );
    }

    /**
     * @link https://goodt-dev.goodt.me:8486/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/notification_template_content/getById_1
     * @description Получение notification_template_content
     *
     * @param {number} id
     * @return {Promise<SafeResult<NotificationContentTemplateDto, Error>>}
     */
    getNotificationContentTemplateById(id) {
        return this.request(
            {
                action: ServiceAction.NOTIFICATION_TEMPLATE_CONTENT_BY_ID_GET,
                pathParams: { id }
            },
            NotificationContentTemplateDto
        );
    }

    /**
     * @link https://goodt-dev.goodt.me:8486/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/notification_log/load_2
     * @description Загрузка карточки в Журнале уведомлений
     *
     * @param {number} id
     * @return {Promise<SafeResult<NotificationLogDto, Error>>}
     */
    getNotificationLogById(id) {
        return this.request(
            {
                action: ServiceAction.NOTIFICATION_LOG_BY_ID_GET,
                pathParams: { id }
            },
            NotificationLogDto
        );
    }

    /**
     * @link https://goodt-dev.goodt.me:8486/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/notification_template_content/create_7
     * @description Создание notification_template_content
     *
     * @param {import('dto.d.ts').NotificationContentTemplateInput} notificationContentTemplateInput
     * @return {Promise<SafeResult<NotificationContentTemplateDto, Error>>}
     */
    createNotificationContentTemplate(notificationContentTemplateInput) {
        return this.request(
            {
                action: ServiceAction.NOTIFICATION_TEMPLATE_CONTENT_CREATE,
                params: notificationContentTemplateInput
            },
            NotificationContentTemplateDto
        );
    }

    /**
     * @link https://goodt-dev.goodt.me:8486/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/notification_template_content/update_7
     * @description Изменение notification_template_content
     *
     * @param {number} id
     * @param {import('dto.d.ts').NotificationContentTemplateInput} notificationContentTemplateInput
     * @return {Promise<SafeResult<NotificationContentTemplateDto, Error>>}
     */
    updateNotificationContentTemplate(id, notificationContentTemplateInput) {
        return this.request(
            {
                action: ServiceAction.NOTIFICATION_TEMPLATE_CONTENT_PUT,
                pathParams: { id },
                params: notificationContentTemplateInput
            },
            NotificationContentTemplateDto
        );
    }
}

/**
 *
 */
export { RostalentApiService };
