import { BaseApiService } from '@goodt-common/api';

import { TaskSettingsApiService } from '../TaskSettingsApiService/v1';
import { OrgStructureApiService } from '../OrgStructureApiService';

import {
    TaskFieldFilter,
    KpiGoalCheckNumbersDto,
    KpiGoalTasksDto,
    ManagedPositionAssignmentDto,
    OperationResultDto,
    UpdateTaskStatusFilter,
    EmployeeFilter,
    EmployeeFilterOutputOptions,
    EmployeeInfoDto,
    PositionDto,
    EmployeeDeputyDto,
    DivisionDto,
    TaskLinksFilter,
    PositionAssignmentDto
} from './dto';
import { TaskDto, TaskLinkDto } from '../TaskSettingsApiService/v1/dto';

export { EmployeeDeputyInput } from './type';

export class TaskOrgApiService extends Mixed<BaseApiService, TaskSettingsApiService, OrgStructureApiService> {
    /**
     * @description Метод массового изменения статуса целей/задач
     * @link https://goodt-dev.goodt.me:8487/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/task/statusChangeBulk
     *
     * @param {UpdateTaskStatusFilter} updateTaskStatusFilter
     * @param {number[]|number} targetTaskStatusId - Массив идентификаторов новых статусов для обновления
     * @return {Promise<SafeResult<Object, Error>>}
     */
    updateTasksStatuses(
        updateTaskStatusFilter: UpdateTaskStatusFilter,
        targetTaskStatusId: number[] | number
    ): Promise<SafeResult<Object, Error>>;

    /**
     * @description Получение PositionAssignment для менеджера по division_team
     * @link https://dev.goodt.me:8590/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/position/getPositionAssignmentForManager
     *
     * @param {number} divisionTeamId
     * @param {boolean} withChildren
     * @return {Promise<SafeResult<ManagedPositionAssignmentDto[], Error>>}
     */
    getManagedPositionAssignments(
        { divisionTeamId }: { divisionTeamId: number },
        { withChildren }: { withChildren: boolean }
    ): Promise<SafeResult<ManagedPositionAssignmentDto[], Error>>;

    /**
     * @description Создание тасков по списку из excel файла
     * @link https://dev.goodt.me:8590/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/task/CreateTasksByExcel
     *
     * @param {Object} formData
     * @return {Promise<SafeResult<OperationResultDto, Error>>}
     */
    createKpiGoalTasks(formData: Object): Promise<SafeResult<OperationResultDto, Error>>;

    /**
     * @description Получение данных
     * @link https://dev.goodt.me:8590/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/kpigoal/kpiGoalTaskFind
     *
     * @param {number} year
     * @param {number[]} roles
     * @return {Promise<SafeResult<KpiGoalTasksDto[], Error>>}
     */
    getKpiGoalTasksByFilter({
        year,
        roles
    }: {
        year: number;
        roles: number[];
    }): Promise<SafeResult<KpiGoalTasksDto[], Error>>;

    /**
     * @description Загрузка пользовательской формы excel файла для проверки табельных номеров сотрудников
     * @link https://dev.goodt.me:8590/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/kpigoal/checkAllNumbers
     *
     * @param {Object} formData
     * @return {Promise<SafeResult<KpiGoalCheckNumbersDto, Error>>}
     */
    checkKpiGoalEmployeesNumbers(formData: Object): Promise<SafeResult<KpiGoalCheckNumbersDto, Error>>;

    /**
     * @description Загрузка пользовательской формы excel файла для проверки табельных номеров сотрудников и получения в ответе excel файла с неподтвержденными номерами.
     * @link https://dev.goodt.me:8590/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/kpigoal/checkAllNumbersExcel
     *
     * @param {Object} formData
     * @return {Promise<SafeResult<Blob, Error>>}
     */
    checkKpiGoalEmployeesNumbersExcel(formData: Object): Promise<SafeResult<Blob, Error>>;

    /**
     * @description Поиск таска по массиву значений филдов
     * @link https://dev.goodt.me:8590/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/task/taskFindByField
     *
     * @param {TaskFieldFilter[]} taskFieldFilter
     * @return {Promise<SafeResult<TaskDto[], Error>>}
     */
    getTasksByTaskFieldFilter(taskFieldFilter: TaskFieldFilter[]): Promise<SafeResult<TaskDto[], Error>>;

    /**
     * @override
     * @description Получение информации о сотрудниках
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/employee/findInfoPost
     *
     * @param {EmployeeFilter} employeeFilter
     * @param {EmployeeFilterOutputOptions} outputOptions
     * @param {Object} extraOutputData
     * @param {Function} [extraOutputData.onResult] - function that takes as an argument all information except employees
     * @return {Promise<SafeResult<EmployeeInfoDto[], Error>>}
     */
    getEmployeesByFilter(
        employeeFilter: EmployeeFilter,
        outputOptions: EmployeeFilterOutputOptions,
        {
            onResult
        }: {
            onResult?: ({
                page,
                totalPages,
                totalElements
            }: {
                page: number;
                totalPages: number;
                totalElements: number;
            }) => void;
        }
    ): Promise<SafeResult<EmployeeInfoDto[], ApiServiceError>>;

    /**
     * @link https://dev.goodt.me:8590/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/position/getPositionAssignmentByEmployeeId
     * @description Получение списка Position по идентификатору сотрудника и идентификатору подразделения
     *
     * @param {number | string} employeeId
     * @param {number[] | string[]} divisionIds
     *
     * @return {Promise<SafeResult<PositionDto[], Error>>}
     */
    async getPositionsByFilter({
        employeeId,
        divisionIds
    }: {
        employeeId: number | string;
        divisionIds: number[] | string[];
    }): Promise<SafeResult<PositionDto[], Error>>;

    /**
     * @link https://dev.goodt.me:8590/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/division/findPost_1
     * @description Получение информации о подразделениях в зависимости от переданных параметров.
     *
     * @param {Object} filters
     * @param {number[]} filters.divisionIds
     * @param {number[]} filters.legalEntityIds
     * @param {boolean} filters.withClosed
     * @param {string} filters.search
     *
     * @param {Object} pagination
     * @param {number} pagination.page
     * @param {number} pagination.size
     *
     * @return {Promise<SafeResult<DivisionDto[], Error>>}
     */
    async getDivisionsByFilter(
        { divisionIds, legalEntityIds, search }: { divisionIds?: number[]; legalEntityIds?: number[]; search?: string },
        { withClosed }?: { withClosed?: boolean },
        { page, size }?: { page: number; size: number }
    ): Promise<SafeResult<DivisionDto[], Error>>;

    /**
     * @link https://goodt-dev.goodt.me:8490/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/employee_deputy/getAll_3
     * @description Получение информации о всехобъектах employee_deputy
     *
     * @param {Object} filters
     * @param {number} filters.employeeId
     * @param {string} filters.date
     *
     * @param {Object} [pagination]
     * @param {number} [pagination.page]
     * @param {number} [pagination.size]
     * @param {string[]} [pagination.sort]
     *
     * @return {Promise<SafeResult<EmployeeDeputyDto[], Error>>}
     */
    async getEmployeeDeputies(
        { employeeId, date }: { employeeId: number; date: string },
        { page, size, sort }?: { page: number; size: number; sort: string[] }
    ): Promise<SafeResult<EmployeeDeputyDto[], Error>>;

    /**
     * @link https://goodt-dev.goodt.me:8490/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/employee_deputy/create_5
     * @description Созданиеобъекта employee_deputy
     *
     * @param {EmployeeDeputyInput} employeeDeputy
     *
     * @return {Promise<SafeResult<EmployeeDeputyDto, Error>>}
     */
    async createEmployeeDeputy(employeeDeputy: EmployeeDeputyInput): Promise<SafeResult<EmployeeDeputyDto, Error>>;

    /**
     * @link https://goodt-dev.goodt.me:8490/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/employee_deputy/getById_4
     * @description Получение информации об объектe employee_deputy
     *
     * @param {number | string} id
     *
     * @return {Promise<SafeResult<EmployeeDeputyDto, Error>>}
     */
    async getEmployeeDeputyById(id: number | string): Promise<SafeResult<EmployeeDeputyDto, Error>>;

    /**
     * @link https://goodt-dev.goodt.me:8490/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/employee_deputy/update_5
     * @description Изменение объекта employee_deputy
     *
     * @param {number | string} id
     * @param {EmployeeDeputyInput} employeeDeputy
     *
     * @return {Promise<SafeResult<EmployeeDeputyDto, Error>>}
     */
    async updateEmployeeDeputy(
        id: number | string,
        employeeDeputy: EmployeeDeputyInput
    ): Promise<SafeResult<EmployeeDeputyDto, Error>>;

    /**
     * @link https://goodt-dev.goodt.me:8490/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/employee_deputy/delete_4
     * @description Удаление объекта employee_deputy
     *
     * @param {number | string} id
     *
     * @return {Promise<SafeResult<OperationResultDto, Error>>}
     */
    async deleteEmployeeDeputy(id: number | string): Promise<SafeResult<OperationResultDto, Error>>;

    /**
     * @override
     * @link https://dev.goodt.me:8590/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/task/getTaskLinkListWithArray
     * @description Получение списка связей целей с возможностью передачей массивов для параметров task_user_id_from, task_user_id_to, task_id_from и task_id_to.
     *
     * @param {TaskLinksFilter} taskLinksFilter
     * @return {Promise<SafeResult<TaskLinkDto[], Error>>}
     */
    async getTaskLinksByFilter({
        userIds,
        taskLinkTypeIds,
        taskFromId,
        taskToId,
        withTaskFromClosed,
        withTaskToClosed,
        withClosedTaskLink,
        taskFromUserId,
        taskToUserId,
        taskLinkId
    }: TaskLinksFilter): Promise<SafeResult<TaskLinkDto[], Error>>;

    /**
     * @link https://goodt-dev.goodt.me:8590/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/position/getPositionAssignmentByPositionId
     * @description Получение PositionAssignment по идентификатору Position
     *
     * @param {number} positionId
     * @return {Promise<SafeResult<PositionAssignmentDto, Error>>}
     */
    async getPositionAssignmentByPositionId(positionId: number): Promise<SafeResult<PositionAssignmentDto, Error>>;
}
