import { mixin } from 'class-mixins';
import { buildRequest, BaseApiService, processRequestResult } from '@goodt-common/api';

import { TaskSettingsApiService } from '../TaskSettingsApiService/v1';
import { OrgStructureApiService } from '../OrgStructureApiService';
import { ServiceAction as TaskSettingsServiceAction } from '../TaskSettingsApiService/v1/config';
import { ServiceAction as OrgStructureServiceAction } from '../OrgStructureApiService/config';
import { resolveNonEmptyValueArray } from './utils';
import { ServiceAction } from './config';
import {
    DivisionDto,
    EmployeeDeputyDto,
    EmployeeInfoDto,
    KpiGoalCheckNumbersDto,
    KpiGoalTasksDto,
    ManagedPositionAssignmentDto,
    OperationResultDto,
    PositionDto,
    PositionAssignmentDto
} from './dto';
import { TaskDto, TaskLinkDto } from '../TaskSettingsApiService/v1/dto';

export class TaskOrgApiService extends BaseApiService {
    /**
     *
     * @param {IApiServiceRequestOptions} request
     * @param [DtoConstructor]
     * @param [processOptions]
     * @return {Promise<SafeResult<*, Error>>}
     */
    async request(request, DtoConstructor, processOptions) {
        if ('action' in request) {
            request = buildRequest(request);
        }
        return processRequestResult(await super.request(request), DtoConstructor, processOptions);
    }

    /**
     * @override
     * @description Метод массового изменения статуса целей/задач
     * @link https://goodt-dev.goodt.me:8487/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/task/statusChangeBulk
     *
     * @param {import('./dto.d.ts').UpdateTaskStatusFilter} updateTaskStatusFilter
     * @param {number[]|number} targetTaskStatusId - Массив идентификаторов новых статусов для обновления
     * @return {Promise<SafeResult<Object, Error>>}
     */
    updateTasksStatuses(
        { taskIds, taskTypeFields, userIds, taskTypeIds, statusIds, processIds, comment } = {},
        targetTaskStatusId
    ) {
        return this.request(
            {
                action: TaskSettingsServiceAction.TASK_STATUS_CHANGE_BULK_POST,
                params: {
                    task_ids: resolveNonEmptyValueArray(taskIds),
                    users: resolveNonEmptyValueArray(userIds),
                    task_type: resolveNonEmptyValueArray(taskTypeIds),
                    current_status: resolveNonEmptyValueArray(statusIds),
                    process_id: resolveNonEmptyValueArray(processIds),
                    status: resolveNonEmptyValueArray(targetTaskStatusId),
                    task_type_fields: resolveNonEmptyValueArray(taskTypeFields),
                    comment
                }
            },
            Object
        );
    }

    /**
     * @description Получение PositionAssignment для менеджера по division_team
     * @link https://dev.goodt.me:8590/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/position/getPositionAssignmentForManager
     *
     * @param {number} divisionTeamId
     * @param {boolean} withChildren
     * @return {Promise<SafeResult<ManagedPositionAssignmentDto[], Error>>}
     */
    getManagedPositionAssignments({ divisionTeamId }, { withChildren = false } = {}) {
        return this.request(
            {
                action: ServiceAction.POSITION_ASSIGNMENTS_MANAGED_GET,
                queryParams: { division_team: divisionTeamId, with_children: withChildren }
            },
            ManagedPositionAssignmentDto
        );
    }

    /**
     * @description Создание тасков по списку из excel файла
     * @link https://dev.goodt.me:8590/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/task/CreateTasksByExcel
     *
     * @param {Object} formData
     * @return {Promise<SafeResult<OperationResultDto, Error>>}
     */
    createKpiGoalTasks(formData) {
        return this.request(
            {
                action: ServiceAction.KPI_GOAL_TASK_CREATE,
                params: formData
            },
            OperationResultDto
        );
    }

    /**
     * @description Получение данных
     * @link https://dev.goodt.me:8590/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/kpigoal/kpiGoalTaskFind
     *
     * @param {number} year
     * @param {number[]} roles
     * @return {Promise<SafeResult<KpiGoalTasksDto[], Error>>}
     */
    getKpiGoalTasksByFilter({ year, roles }) {
        return this.request(
            {
                action: ServiceAction.KPI_GOAL_TASK_FIND_GET,
                queryParams: { year, roles }
            },
            KpiGoalTasksDto
        );
    }

    /**
     * @description Загрузка пользовательской формы excel файла для проверки табельных номеров сотрудников
     * @link https://dev.goodt.me:8590/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/kpigoal/checkAllNumbers
     *
     * @param {Object} formData
     * @return {Promise<SafeResult<KpiGoalCheckNumbersDto, Error>>}
     */
    checkKpiGoalEmployeesNumbers(formData) {
        return this.request(
            {
                action: ServiceAction.KPI_GOAL_CHECK_NUMBERS_POST,
                params: formData
            },
            KpiGoalCheckNumbersDto
        );
    }

    /**
     * @description Загрузка пользовательской формы excel файла для проверки табельных номеров сотрудников и получения в ответе excel файла с неподтвержденными номерами.
     * @link https://dev.goodt.me:8590/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/kpigoal/checkAllNumbersExcel
     *
     * @param {Object} formData
     * @return {Promise<SafeResult<Blob, Error>>}
     */
    checkKpiGoalEmployeesNumbersExcel(formData) {
        return this.request(
            {
                action: ServiceAction.KPI_GOAL_CHECK_NUMBERS_EXCEL_POST,
                params: formData
            },
            Blob
        );
    }

    /**
     * @description Поиск таска по массиву значений филдов
     * @link https://dev.goodt.me:8590/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/task/taskFindByField
     *
     * @param {import('./dto.d.ts').TaskFieldFilter[]} taskFieldFilter
     * @return {Promise<SafeResult<TaskDto[], Error>>}
     */
    getTasksByTaskFieldFilter(taskFieldFilter) {
        return this.request(
            {
                action: ServiceAction.TASK_FIND_BY_FIELD_POST,
                params: taskFieldFilter
            },
            TaskDto
        );
    }

    /**
     * @override
     * @description Получение информации о сотрудниках
     * @link https://goodt-dev.goodt.me:8480/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/employee/findInfoPost
     *
     * @param {import('./dto.d.ts').EmployeeFilter} employeeFilter
     * @param {import('./dto.d.ts').EmployeeFilterOutputOptions} outputOptions
     * @param {Object} extraOutputData
     * @param {Function} [extraOutputData.onResult] - function that takes as an argument all information except employees
     * @return {Promise<SafeResult<EmployeeInfoDto[], Error>>}
     */
    async getEmployeesByFilter(
        { employeeIds, divisionId, searchToken, legalEntityId, employeeNumbers, withPatronymic, jobTitleId } = {},
        { page, size, hasPositionAssignment } = {},
        { onResult } = {}
    ) {
        return processRequestResult(
            await this.request({
                action: OrgStructureServiceAction.EMPLOYEE_FIND,
                queryParams: {
                    ...(employeeIds && { id: employeeIds }),
                    ...(divisionId && { division: divisionId }),
                    ...(searchToken && { search: searchToken }),
                    ...((page || page === 0) && { page }),
                    ...(legalEntityId && { legal_entity_id: legalEntityId }),
                    ...(employeeNumbers && { employee_number: employeeNumbers }),
                    ...(size && { size }),
                    ...(withPatronymic != null && { with_patronymic: withPatronymic }),
                    ...(hasPositionAssignment != null && { has_position_assignment: hasPositionAssignment }),
                    ...(jobTitleId != null && { job_title_id: jobTitleId })
                }
            }),
            EmployeeInfoDto,
            {
                resultTransformer: ({ data, ...meta }) => {
                    if (typeof onResult === 'function') {
                        onResult(meta);
                    }
                    return data;
                }
            }
        );
    }

    /**
     * @link https://dev.goodt.me:8590/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/position/getPositionAssignmentByEmployeeId
     * @description Получение списка Position по идентификатору сотрудника и идентификатору подразделения
     *
     * @param {number | string} employeeId
     * @param {number[] | string[]} divisionIds
     *
     * @return {Promise<SafeResult<PositionDto[], Error>>}
     */
    async getPositionsByFilter({ employeeId, divisionIds }) {
        return this.request(
            {
                action: ServiceAction.POSITION_FIND_GET,
                params: {
                    employee_id: employeeId,
                    division_id: divisionIds
                }
            },
            PositionDto
        );
    }

    /**
     * @link https://dev.goodt.me:8590/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/division/findPost_1
     * @description Получение информации о подразделениях в зависимости от переданных параметров.
     *
     * @param {Object} filters
     * @param {number[]} filters.divisionIds
     * @param {number[]} filters.legalEntityIds
     * @param {boolean} filters.withClosed
     * @param {string} filters.search
     *
     * @param {Object} pagination
     * @param {number} pagination.page
     * @param {number} pagination.size
     *
     * @return {Promise<SafeResult<DivisionDto[], Error>>}
     */
    async getDivisionsByFilter({ divisionIds, legalEntityIds, search } = {}, { withClosed } = {}, { page, size } = {}) {
        return this.request(
            {
                action: ServiceAction.DIVISION_FIND_POST,
                params: {
                    ...(divisionIds && { id: divisionIds }),
                    ...(legalEntityIds && { legalEntityId: legalEntityIds }),
                    ...(withClosed != null && { withClosed }),
                    ...(search != null && { search }),
                    ...(page != null && { page }),
                    ...(size != null && { size })
                }
            },
            DivisionDto
        );
    }

    /**
     * @link https://goodt-dev.goodt.me:8490/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/employee_deputy/getAll_3
     * @description Получение информации о всехобъектах employee_deputy
     *
     * @param {Object} filters
     * @param {number} filters.employeeId
     * @param {string} filters.date
     *
     * @param {Object} [pagination]
     * @param {number} [pagination.page]
     * @param {number} [pagination.size]
     * @param {string[]} [pagination.sort]
     *
     * @return {Promise<SafeResult<EmployeeDeputyDto[], Error>>}
     */
    async getEmployeeDeputies({ employeeId, date }, { page, size, sort } = {}) {
        return this.request(
            {
                action: ServiceAction.EMPLOYEE_DEPUTY_GET,
                params: {
                    employeeId,
                    date,
                    page,
                    size,
                    sort
                }
            },
            EmployeeDeputyDto
        );
    }

    /**
     * @link https://goodt-dev.goodt.me:8490/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/employee_deputy/create_5
     * @description Созданиеобъекта employee_deputy
     *
     * @param {EmployeeDeputyInput} employeeDeputy
     *
     * @return {Promise<SafeResult<EmployeeDeputyDto, Error>>}
     */
    async createEmployeeDeputy(employeeDeputy) {
        return this.request(
            {
                action: ServiceAction.EMPLOYEE_DEPUTY_POST,
                params: employeeDeputy
            },
            EmployeeDeputyDto
        );
    }

    /**
     * @link https://goodt-dev.goodt.me:8490/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/employee_deputy/getById_4
     * @description Получение информации об объектe employee_deputy
     *
     * @param {number | string} id
     *
     * @return {Promise<SafeResult<EmployeeDeputyDto, Error>>}
     */
    async getEmployeeDeputyById(id) {
        return this.request(
            {
                action: ServiceAction.EMPLOYEE_DEPUTY_BY_ID_GET,
                pathParams: {
                    id
                }
            },
            EmployeeDeputyDto
        );
    }

    /**
     * @link https://goodt-dev.goodt.me:8490/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/employee_deputy/update_5
     * @description Изменение объекта employee_deputy
     *
     * @param {number | string} id
     * @param {EmployeeDeputyInput} employeeDeputy
     *
     * @return {Promise<SafeResult<EmployeeDeputyDto, Error>>}
     */
    async updateEmployeeDeputy(id, employeeDeputy) {
        return this.request(
            {
                action: ServiceAction.EMPLOYEE_DEPUTY_PUT,
                pathParams: {
                    id
                },
                params: employeeDeputy
            },
            EmployeeDeputyDto
        );
    }

    /**
     * @link https://goodt-dev.goodt.me:8490/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/employee_deputy/delete_4
     * @description Удаление объекта employee_deputy
     *
     * @param {number | string} id
     *
     * @return {Promise<SafeResult<OperationResultDto, Error>>}
     */
    async deleteEmployeeDeputy(id) {
        return this.request(
            {
                action: ServiceAction.EMPLOYEE_DEPUTY_DELETE,
                pathParams: {
                    id
                }
            },
            OperationResultDto
        );
    }

    /**
     * @override
     * @link https://dev.goodt.me:8590/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/task/getTaskLinkListWithArray
     * @description Получение списка связей целей с возможностью передачей массивов для параметров task_user_id_from, task_user_id_to, task_id_from и task_id_to.
     *
     * @param {import('./dto.d.ts').TaskLinksFilter} taskLinksFilter
     * @return {Promise<SafeResult<TaskLinkDto[], Error>>}
     */
    async getTaskLinksByFilter({
        userIds,
        taskLinkTypeIds,
        taskFromId,
        taskToId,
        withTaskFromClosed,
        withTaskToClosed,
        withClosedTaskLink,
        taskFromUserId,
        taskToUserId,
        taskLinkId
    }) {
        // prettier-ignore
        return this.request({
            action: TaskSettingsServiceAction.TASK_LINKLIST_GET,
            params: {
                ...(userIds && { users: [userIds].flat().filter((id) => id != null) }),
                ...(taskLinkTypeIds && { task_link_type_id: [taskLinkTypeIds].flat().filter((id) => id != null) }),
                ...(taskFromId && { task_id_from: [taskFromId].flat().filter((id) => id != null) }),
                ...(taskToId && { task_id_to: [taskToId].flat().filter((id) => id != null) }),
                ...(withTaskFromClosed && { task_with_closed_from: withTaskFromClosed }),
                ...(withTaskToClosed && { task_with_closed_to: withTaskToClosed }),
                ...(withClosedTaskLink && {with_closed_task_link: withClosedTaskLink}),
                ...(taskFromUserId && { task_user_id_from: [taskFromUserId].flat().filter((id) => id != null) }),
                ...(taskToUserId && { task_user_id_to: [taskToUserId].flat().filter((id) => id != null) }),
                ...(taskLinkId && { id: [taskLinkId].flat().filter((id) => id != null) })
            }
        }, TaskLinkDto);
    }

    /**
     * @link https://goodt-dev.goodt.me:8590/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/position/getPositionAssignmentByPositionId
     * @description Получение PositionAssignment по идентификатору Position
     *
     * @param {number} positionId
     * @return {Promise<SafeResult<PositionAssignmentDto, Error>>}
     */
    async getPositionAssignmentByPositionId(positionId) {
        return this.request(
            {
                action: ServiceAction.POSITION_ASSIGNMENT_GET,
                params: { position_id: positionId }
            },
            PositionAssignmentDto
        );
    }
}

mixin(TaskOrgApiService, TaskSettingsApiService, OrgStructureApiService);
