/* eslint-disable camelcase */

import { BaseApiService, processRequestResult, buildRequest } from '@goodt-common/api';
import {
    TaskConditionsDto,
    TaskDto,
    TaskHistoryResultDto,
    TaskLinkDto,
    TaskLinkTypeDto,
    TaskTypeDto,
    UserTaskTreeDto,
    ProcessDto,
    ProcessInfoDto,
    ProcessStageShortDto,
    ProcessStageDto,
    ProcessTypeDto,
    ProcessStageTypeDto,
    ProcessTaskBindingDto,
    TaskBatchUpdateInput,
    ModuleDto,
    AccessTypeDto,
    UserTypeDto,
    TaskTypeFieldGroupDto,
    TaskTypeFieldTypeDto,
    ComponentTaskTypeFieldDto,
    CompetenceCommentDto
} from './dto';
import { ServiceAction } from './config';

/**
 * @typedef {import('@goodt-common/utils').ISafeResult} SafeResult
 */
/**
 *
 */
export class TaskSettingsApiService extends BaseApiService {
    /**
     *
     * @param {IApiServiceRequestOptions} request
     * @param [DtoConstructor]
     * @param [processOptions]
     * @return {Promise<SafeResult<*, Error>>}
     */
    async request(request, DtoConstructor, processOptions) {
        return processRequestResult(await super.request(buildRequest(request)), DtoConstructor, processOptions);
    }

    /**
     * Получение списка сущностей "пользовательское дерево задач" для списка заданных идентификаторов
     * пользователей и заданного типа задачи
     *
     * @param {number|number[]} userIds - Массив идентификаторов пользователей
     * @param {number} typeId - Идентификатор типа цели
     * @param {Object} [options={}]
     * @param {number} options.processTypeId - Идентификатор типа процесса
     * @param {number} options.level - Идентификатор уровня
     * @param {boolean} options.history - Этот флаг устанавливает следующее: если true - то выведутся все поля цели, если false - выведутся только незавершённые поля цели
     * @param {number[]} options.processIds - Массив идентификаторов процессов
     * @param {boolean} options.withClosedTask - Этот флаг устанавливает следующее: если true - то выведутся все цели, если false - выведутся только незавершённые цели
     * @param {boolean} options.withClosedTaskField - Этот флаг устанавливает следующее: если true - то выведутся все поля цели, если false - выведутся только незавершённые поля цели
     * @param {boolean} options.rootOnly - Этот флаг устанавливает следующее: если true - то выведутся только рутовые цели, если false - выведутся все имеющиеся цели
     * @return {Promise<SafeResult<UserTaskTreeDto[], Error>>}
     */

    getUserTaskTrees(
        userIds,
        typeId,
        { processTypeId, level, history, processIds, withClosedTask, withClosedTaskField, rootOnly } = {}
    ) {
        // prettier-ignore
        return this.request({
            action: ServiceAction.TASK_TREES_GET,
            queryParams: {
                ...(typeId != null && { type: typeId }),
                ...(userIds !=null && { users: [userIds].flat().filter((id) => id != null) }),
                ...(processTypeId != null && { process_type: processTypeId }),
                ...(level != null && { level }),
                ...(history != null && { history }),
                ...(processIds != null && { process_id: [processIds].flat().filter(id => id != null) }),
                ...(withClosedTask != null && { with_closed_task: withClosedTask }),
                ...(withClosedTaskField != null && { with_closed_task_field: withClosedTaskField }),
                ...(rootOnly != null && { root_only: rootOnly }),
            }
        }, UserTaskTreeDto);
    }

    /**
     * Получение списка сущностей "задача" (одного "дерева" задач) для заданного пользователя
     * и заданного типа задачи
     *
     * @param {number} userId - - Идентификатор пользователя
     * @param {number} typeId  - Идентификатор типа цели
     * @param {Object} [options={}]
     * @param {number} options.processTypeId - Идентификатор типа процесса
     * @param {number} options.level - Идентификатор уровня
     * @param {boolean} options.history - Этот флаг устанавливает следующее: если true - то выведутся все поля цели, если false - выведутся только незавершённые поля цели
     * @param {number[]} options.processIds - Массив идентификаторов процессов
     * @param {boolean} options.withClosedTask - Этот флаг устанавливает следующее: если true - то выведутся все цели, если false - выведутся только незавершённые цели
     * @param {boolean} options.withClosedTaskField - Этот флаг устанавливает следующее: если true - то выведутся все поля цели, если false - выведутся только незавершённые поля цели
     * @param {boolean} options.rootOnly - Этот флаг устанавливает следующее: если true - то выведутся только рутовые цели, если false - выведутся все имеющиеся цели
     * @return {Promise<SafeResult<TaskDto[], Error>>}
     */
    async getTaskTree(
        userId,
        typeId,
        { processTypeId, level, history, processIds, withClosedTask, withClosedTaskField, rootOnly } = {}
    ) {
        return this.request(
            {
                action: ServiceAction.TASK_TREES_GET,
                queryParams: {
                    ...(typeId != null && { type: typeId }),
                    ...(userId != null && { users: [userId].flat().filter((id) => id != null) }),
                    ...(processTypeId != null && { process_type: processTypeId }),
                    ...(level != null && { level }),
                    ...(history != null && { history }),
                    ...(processIds != null && { process_id: [processIds].flat().filter((id) => id != null) }),
                    ...(withClosedTask != null && { with_closed_task: withClosedTask }),
                    ...(withClosedTaskField != null && { with_closed_task_field: withClosedTaskField }),
                    ...(rootOnly != null && { root_only: rootOnly })
                }
            },
            TaskDto,
            { resultTransformer: ServiceAction.TASK_TREES_GET.resultTransformer }
        );
    }

    /**
     *
     * @param {number|number[]} [userIds]
     * @param {number|number[]} [taskLinkTypeIds]
     * @param {number} [taskFromId]
     * @param {number} [taskToId]
     * @param {boolean} [withTaskFromClosed]
     * @param {boolean} [withTaskToClosed]
     * @param {number} [taskFromUserId]
     * @param {number} [taskToUserId]
     * @return {Promise<SafeResult<TaskLinkDto[], Error>>}
     */
    async getTaskLinks({
        userIds,
        taskLinkTypeIds,
        taskFromId,
        taskToId,
        withTaskFromClosed,
        withTaskToClosed,
        taskFromUserId,
        taskToUserId
    }) {
        // prettier-ignore
        return this.getTaskLinksByFilter({
            userIds,
            taskFromId,
            taskToId,
            withTaskFromClosed,
            withTaskToClosed,
            taskFromUserId,
            taskToUserId,
            taskLinkTypeIds
        });
    }

    /**
     * @link https://goodt-dev.goodt.me:8487/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/
     * @description Получение списка связей целей с возможностью передачей массивов для параметров task_user_id_from, task_user_id_to, task_id_from и task_id_to.
     *
     * @param {Object} params
     * @param {number|number[]} params.userIds - Массив идентификаторов пользователей
     * @param {number|number[]} [params.taskLinkTypeIds] - Массив идентификаторов типов ссылки на цели
     * @param {number|number[]} [params.taskFromId] - Массив идентификаторов начальной цели
     * @param {number|number[]} [params.taskToId] - Массив идентификаторов конечной цели
     * @param {boolean} [params.withTaskFromClosed] - false - в результате получим связи только с открытыми начальными целями, true - будем получать связи даже с закрытими начальными целями
     * @param {boolean} [params.withTaskToClosed] - false - в результате получим связи только с открытыми конечными целями, true - будем получать связи даже с закрытими конечными целями
     * @param {number|number[]} [params.taskFromUserId] - Массив идентификаторов пользователей начальной цели
     * @param {number|number[]} [params.taskToUserId] - Массив идентификаторов пользователей конечной цели
     * @param {number} [params.taskLinkId] - Идентификатор связи целей
     * @return {Promise<SafeResult<TaskLinkDto[], Error>>}
     */
    async getTaskLinksByFilter({
        userIds,
        taskLinkTypeIds,
        taskFromId,
        taskToId,
        withTaskFromClosed,
        withTaskToClosed,
        taskFromUserId,
        taskToUserId,
        taskLinkId
    }) {
        // prettier-ignore
        return this.request({
            action: ServiceAction.TASK_LINKLIST_GET,
            params: {
                ...(userIds && { users: [userIds].flat().filter((id) => id != null) }),
                ...(taskLinkTypeIds && { task_link_type_id: [taskLinkTypeIds].flat().filter((id) => id != null) }),
                ...(taskFromId && { task_id_from: [taskFromId].flat().filter((id) => id != null) }),
                ...(taskToId && { task_id_to: [taskToId].flat().filter((id) => id != null) }),
                ...(withTaskFromClosed && { task_with_closed_from: withTaskFromClosed }),
                ...(withTaskToClosed && { task_with_closed_to: withTaskToClosed }),
                ...(taskFromUserId && { task_user_id_from: [taskFromUserId].flat().filter((id) => id != null) }),
                ...(taskToUserId && { task_user_id_to: [taskToUserId].flat().filter((id) => id != null) }),
                ...(taskLinkId && { id: [taskLinkId].flat().filter((id) => id != null) }),
            }
        }, TaskLinkDto);
    }

    /**
     * Изменение связи цели
     *
     * @param {number} linkId
     * @param {number} task_id_from
     * @param {number} task_link_type_id
     * @param {number} task_id_to
     * @param {string} date_from
     * @param {string} date_to
     * @return {Promise<SafeResult<Number, Error>>}
     */
    async updateTaskLink(linkId, { task_id_from, task_link_type_id, task_id_to, date_from, date_to }) {
        return this.request(
            {
                action: ServiceAction.TASK_LINK_UPDATE,
                params: {
                    id: linkId,
                    task_id_from,
                    task_link_type_id,
                    task_id_to,
                    date_from,
                    date_to
                }
            },
            Number
        );
    }

    /**
     * Получение списка типов связей
     * @link https://goodt-dev.goodt.me:8487/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/task/getTaskLinkTypeByIds
     *
     * @param {number} linkTypeId
     * @return {Promise<SafeResult<TaskLinkTypeDto[], Error>>}
     */
    async getTaskLinkTypes(linkTypeId) {
        return this.request(
            {
                action: ServiceAction.TASK_LINK_TYPE,
                params: {
                    id: linkTypeId
                }
            },
            TaskLinkTypeDto
        );
    }

    /**
     * Создание связи цели
     *
     * @param {number} task_id_from
     * @param {number} task_link_type_id
     * @param {number} task_id_to
     * @param {string}date_from
     * @param {string} date_to
     * @return {Promise<SafeResult<Number, Error>>}
     */
    async createTaskLink({ task_id_from, task_link_type_id, task_id_to, date_from, date_to }) {
        return this.request(
            {
                action: ServiceAction.TASK_LINK_UPDATE,
                params: {
                    task_id_from,
                    task_link_type_id,
                    task_id_to,
                    date_from,
                    date_to
                }
            },
            Number
        );
    }

    /**
     *
     * @param {number} id
     * @return {Promise<SafeResult<TaskLinkDto, Error>>}
     */
    async getTaskLinkById(id) {
        // prettier-ignore
        return this.request({
            action: ServiceAction.TASK_LINKLIST_GET,
            params: {
                id
            }
        }, TaskLinkDto, {
            resultTransformer: (taskLinks) => taskLinks[0]
        });
    }

    /**
     * Получение сущности "задача" по её идентификатору
     *
     * @param {number} taskId
     * @return {Promise<SafeResult<TaskDto, Error>>}
     */
    async getTaskById(taskId) {
        // prettier-ignore
        return this.request({
            action: ServiceAction.TASK_INFO_GET,
            params: {
                id: taskId
            }
        }, TaskDto);
    }

    /**
     *
     * @param {number} taskId
     *
     * @param {number} type type id
     * @param {number} user user id
     * @param {Record<string, string>[]} [fields]
     * @param {string} [comment]
     * @param {number} [parent] parent id
     * @param {number} [status]
     * @return {Promise<SafeResult<number, Error>>}
     */
    async replaceTask(taskId, { fields, comment, parent, type, user, status }) {
        return this.request(
            {
                action: ServiceAction.TASK_REPLACE,
                params: { id: taskId, parent, type, user, status, fields, comment },
                pathParams: { id: taskId }
            },
            Number
        );
    }

    /**
     *
     * @param {number} taskId
     *
     * @param {Record<string, string>[]} [fields]
     * @param {string} [comment]
     * @param {number} [parent] parent id
     * @param {number} [type] type id
     * @param {number} [user] user id
     * @param {number} [status]
     * @return {Promise<SafeResult<number, Error>>}
     */
    async updateTask(taskId, { fields, comment, parent, type, user, status }) {
        return this.request(
            {
                action: ServiceAction.TASK_UPDATE,
                params: { id: taskId, parent, type, user, status, fields, comment }
            },
            Number
        );
    }

    /**
     **
     * @param {number} processId process id
     * @param {number} type type id
     * @param {number} user user id
     * @param {Record<string, string>[]} [fields]
     * @param {string} [comment]
     * @param {number} [parent] parent id
     * @param {number} [status]
     * @return {Promise<SafeResult<number, Error>>}
     */
    async createTask(processId, { fields, comment, parent, type, user, status }) {
        // prettier-ignore
        return this.request({
            action: ServiceAction.TASK_CREATE,
            queryParams: { process_id: processId },
            params: { parent, type, user, status, fields, comment }
        }, Number);
    }

    /**
     **
     * @param {number} taskId task id
     * @param {number[]} ids
     * @param {number} status status id
     * @param {string} [comment] comment
     * @return {Promise<SafeResult<boolean, Error>>}
     */
    async updateTaskStatus(taskId, { ids, status, comment }) {
        // prettier-ignore
        return this.request({
            action: ServiceAction.TASK_STATUS_UPDATE,
            params: {
                id: taskId,
                status,
                comment,
                ids
            }
        }, Boolean);
    }

    /**
     **
     * @description Удаление цели
     * @link https://goodt-dev.goodt.me:8487/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/task/deleteTask
     *
     * @param {number} taskId task id
     * @return {Promise<SafeResult<boolean, Error>>}
     */
    async deleteTask(taskId) {
        // prettier-ignore
        return this.request({
            action: ServiceAction.TASK_DELETE,
            queryParams: {
                // eslint-disable-next-line no-restricted-syntax
                id: [taskId]
            }
        }, Boolean);
    }

    /**
     **
     * @description Удаление целей по массиву идентификаторов
     * @link https://goodt-dev.goodt.me:8487/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/task/deleteTask
     *
     * @param {number[]} taskIds task ids
     * @return {Promise<SafeResult<boolean, Error>>}
     */
    async deleteTasks(taskIds) {
        // prettier-ignore
        return this.request({
            action: ServiceAction.TASK_DELETE,
            queryParams: {
                id: taskIds
            }
        }, Boolean);
    }

    /**
     *
     * @param {number} taskLinkId
     * @return {Promise<SafeResult<boolean, Error>>}
     */
    async deleteTaskLink(taskLinkId) {
        // prettier-ignore
        return this.request({
            action: ServiceAction.DELETE_LINK,
            pathParams: { id: taskLinkId }
        }, Boolean);
    }

    /**
     * Task History
     *
     * @param {number} tasksId
     * @return {Promise<SafeResult<TaskHistoryResultDto, Error>>}
     */
    async getTaskHistory(tasksId) {
        // prettier-ignore
        return this.request({
            action: ServiceAction.TASK_HISTORY_GET,
            params: {
                id: tasksId
            }
        }, TaskHistoryResultDto);
    }

    /**
     * Find tasks (exactly filter tasks)
     *
     * @param {import("./types").TaskFilterInput} filter
     * @return {Promise<SafeResult<TaskDto[], Error>>} result
     */
    async getTasksByFilter({
        taskIds,
        taskTypeIds,
        userIds,
        parentIds,
        processId,
        processTypeId,
        taskFieldTypeId,
        taskFieldValue,
        taskField = {},
        rootId
    }) {
        // eslint-disable-next-line no-underscore-dangle
        const _taskFieldTypeId = taskField.type_id ?? taskFieldTypeId;
        // eslint-disable-next-line no-underscore-dangle
        const _taskFieldValue = taskField.value ?? taskFieldValue;
        // prettier-ignore
        return this.request({
            action: ServiceAction.TASK_FIND,
            params: {
                ...(taskIds && { ids: taskIds }),
                ...(userIds && { users: userIds }),
                ...(taskTypeIds && { task_type: taskTypeIds }),
                ...(processTypeId && { process_type_id: processTypeId }),
                ...(parentIds && { parent_id: parentIds }),
                ...(processId && { process_id: processId }),
                ...(_taskFieldTypeId && { task_field_type_id: _taskFieldTypeId }),
                ...(_taskFieldValue && { task_field_value: _taskFieldValue }),
                ...(rootId && { root_id: rootId }),
            }
        }, TaskDto);
    }

    /**
     * Retrieves process-task bindings by filter
     *
     * @param {import("./types").ProcessTaskBindingFilterInput} filter
     * @return {Promise<SafeResult<ProcessTaskBindingDto[], Error>>} result
     */
    async getProcessTaskBindingsByFilter({ taskId, processId }) {
        // prettier-ignore
        return this.request({
            action: ServiceAction.PROCESS_TASK_BINDINGS_GET,
            params: {
                task_id: taskId,
                process_id: processId
            }
        }, ProcessTaskBindingDto);
    }

    /**
     *
     * @param {number} taskId
     * @param {number[]|number} processIds
     * @return {Promise<SafeResult<TaskConditionsDto[], Error>>}
     */
    async getTaskConditions(taskId, { processIds } = {}) {
        // prettier-ignore
        return this.request({
            action: ServiceAction.TASK_CONDITIONS_GET,
            params: {
                id: taskId,
                process_id: [processIds].flat().filter((id) => id != null)
            }
        }, TaskConditionsDto);
    }

    /**
     *
     * @param {number|string} processId
     * @return {Promise<SafeResult<*, Error>|SafeResult<any, Error>|*>}
     */
    async getProcessById(processId) {
        // prettier-ignore
        return this.request({
            action: ServiceAction.PROCESS_GET_BY_ID,
            pathParams: {
                id: processId
            }
        }, ProcessDto);
    }

    /**
     *
     * @param {number|string} typeId
     * @param {string} date
     * @return {Promise<SafeResult<ProcessInfoDto, Error>>}
     */
    async getProcessInfos({ typeId, date } = {}) {
        // prettier-ignore
        return this.request({
            action: ServiceAction.PROCESSES_GET_BY_FILTER,
            params: {
                type_id: typeId,
                date
            }
        }, ProcessInfoDto);
    }

    /**
     *
     * @param {number} typeId
     * @param {string} date
     * @return {Promise<SafeResult<ProcessDto[], Error>>}
     */
    async getProcesses({ typeId, date } = {}) {
        const resultTransformer = (processInfos) => processInfos.map(({ process }) => process);

        // prettier-ignore
        return this.request({
            action: ServiceAction.PROCESSES_GET_BY_FILTER,
            params: {
                type_id: typeId,
                date
            }
        }, ProcessDto, { resultTransformer });
    }

    /**
     * @param {number} processTypeId
     * @param {string} processDate
     * @return {Promise<SafeResult<ProcessStageShortDto[], Error>>}
     */
    async getProcessStages({ processTypeId, processDate } = {}) {
        const resultTransformer = (processInfos) =>
            processInfos.flatMap(({ process_stages: processStages }) => processStages);

        // prettier-ignore
        return this.request({
            action: ServiceAction.PROCESSES_GET_BY_FILTER,
            params: {
                type_id: processTypeId,
                date: processDate
            }
        }, ProcessStageShortDto, { resultTransformer });
    }

    /**
     * @description Создание процесса
     * @link https://goodt-dev.goodt.me:8487/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/process/createProcessNew
     *
     * @param {string} name
     * @param {string} params
     * @param {number} type_id
     * @param {number} is_startable
     * @param {string} date_to
     * @param {string} date_block
     * @param {string} date_end
     * @param {string} date_start
     * @param {string} date_end_confirm
     * @param {string} date_start_confirm
     * @return {Promise<SafeResult<number, Error>>}
     */
    async createProcess({
        name,
        params,
        type_id,
        is_startable,
        date_to,
        date_block,
        date_end,
        date_start,
        date_end_confirm,
        date_start_confirm
    }) {
        // prettier-ignore
        return this.request({
            action: ServiceAction.PROCESS_CREATE,
            params: {
                name,
                params,
                type_id,
                is_startable,
                date_to,
                date_block,
                date_end,
                date_start,
                date_end_confirm,
                date_start_confirm
            }
        }, Number);
    }

    /**
     * @description Изменение процесса
     * @link https://goodt-dev.goodt.me:8487/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/process/updateProcessNew
     *
     * @param {number} processId
     * @param {string} name
     * @param {string} params
     * @param {number} type_id
     * @param {number} is_startable
     * @param {string} date_to
     * @param {string} date_block
     * @param {string} date_start_confirm
     * @param {string} date_end_confirm
     * @param {string} date_start
     * @param {string} date_end
     * @return {Promise<SafeResult<boolean, Error>>}
     */
    async updateProcess(
        processId,
        {
            name,
            params,
            type_id,
            is_startable,
            date_to,
            date_block,
            date_start_confirm,
            date_end_confirm,
            date_start,
            date_end
        }
    ) {
        // prettier-ignore
        return this.request({
            action: ServiceAction.PROCESS_UPDATE,
            pathParams: {
                id: processId
            },
            params: {
                name,
                params,
                type_id,
                is_startable,
                date_to,
                date_block,
                date_start_confirm,
                date_end_confirm,
                date_start,
                date_end
            }
        }, Boolean);
    }

    /**
     **
     * @param {number} processId
     * @return {Promise<SafeResult<boolean, Error>>}
     */
    deleteProcess(processId) {
        // prettier-ignore
        return this.request({
            action: ServiceAction.PROCESS_DELETE,
            pathParams: {
                id: processId
            }
        }, Boolean);
    }

    /**
     * @param {number} processStageId
     * @return {Promise<SafeResult<ProcessStageDto, Error>>}
     */
    async getProcessStageById(processStageId) {
        // prettier-ignore
        return this.request({
            action: ServiceAction.PROCESS_STAGE_GET_BY_ID,
            pathParams: {
                id: processStageId
            }
        }, ProcessStageDto);
    }

    /**
     * @description Создание этапа процесса
     * @link https://goodt-dev.goodt.me:8487/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/process/createProcessStage
     *
     * @param {string} params
     * @param {number} process_id
     * @param {number} stage_type_id
     * @param {string} date_to
     * @param {string} date_start
     * @param {string} date_end
     * @param {string} date_start_confirm
     * @param {string} date_end_confirm
     * @return {Promise<SafeResult<ProcessStageDto, Error>>}
     */
    createProcessStage({
        params,
        process_id,
        stage_type_id,
        date_to,
        date_start,
        date_end,
        date_start_confirm,
        date_end_confirm
    } = {}) {
        // prettier-ignore
        return this.request({
            action: ServiceAction.PROCESS_STAGE_CREATE,
            params: {
                params,
                process_id,
                stage_type_id,
                date_to,
                date_start,
                date_end,
                date_start_confirm,
                date_end_confirm
            }
        }, ProcessStageDto);
    }

    /**
     * @description Обновление этапа процесса
     * @link https://goodt-dev.goodt.me:8487/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/process/updateProcessStage
     *
     * @param {number} processStageId
     * @param {string} params
     * @param {number} process_id
     * @param {number} stage_type_id
     * @param {string} date_to
     * @param {string} date_start
     * @param {string} date_end
     * @param {string} date_start_confirm
     * @param {string} date_end_confirm
     * @return {Promise<SafeResult<ProcessStageDto, Error>>}
     */
    updateProcessStage(
        processStageId,
        { params, process_id, stage_type_id, date_to, date_start, date_end, date_start_confirm, date_end_confirm }
    ) {
        return this.request(
            {
                action: ServiceAction.PROCESS_STAGE_UPDATE,
                pathParams: {
                    id: processStageId
                },
                params: {
                    params,
                    process_id,
                    stage_type_id,
                    date_to,
                    date_start,
                    date_end,
                    date_start_confirm,
                    date_end_confirm
                }
            },
            ProcessStageDto
        );
    }

    /**
     **
     * @param {number} processStageId
     * @return {Promise<SafeResult<boolean, Error>>}
     */
    deleteProcessStage(processStageId) {
        // prettier-ignore
        return this.request({
            action: ServiceAction.PROCESS_STAGE_DELETE,
            pathParams: {
                id: processStageId
            }
        }, Boolean);
    }

    /**
     *
     * @return {Promise<SafeResult<ProcessTypeDto[], Error>>}
     */
    async getProcessTypes() {
        // prettier-ignore
        return this.request({
            action: ServiceAction.PROCESS_GET_TYPES
        }, ProcessTypeDto);
    }

    /**
     *
     * @return {Promise<SafeResult<ProcessStageTypeDto[], Error>>}
     */
    async getProcessStageTypes() {
        // prettier-ignore
        return this.request({
            action: ServiceAction.PROCESS_STAGE_GET_TYPES
        }, ProcessStageTypeDto);
    }

    /**
     * Получение типа задачи `TaskTypeDto` по id
     *
     * @link https://goodt-dev.goodt.me:8487/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/task/getTaskType
     * @return {Promise<SafeResult<TaskTypeDto, Error>>}
     */
    async getTaskTypeById(taskTypeId) {
        // prettier-ignore
        return this.request({
            action: ServiceAction.TASK_TYPE_GET,
            pathParams: {
                id: taskTypeId
            }
        }, TaskTypeDto);
    }

    /**
     * @description Метод загрузки файла на сервер.
     * @link https://goodt-dev.goodt.me:8487/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/file/uploadFile
     *
     * @param {Object} taskParams
     * @param {number} taskParams.taskId
     * @param {number} taskParams.taskTypeFieldId
     * @param {Object} formData
     *
     * @return {Promise<SafeResult<Object, Error>>}
     */
    uploadFile({ taskId, taskTypeFieldId }, formData) {
        return this.request(
            {
                action: ServiceAction.FILE_UPLOAD_POST,
                queryParams: {
                    task_id: taskId,
                    task_type_field_id: taskTypeFieldId
                },
                params: formData
            },
            Object
        );
    }

    /**
     * @descriptionМетод получения файла
     * @link https://goodt-dev.goodt.me:8487/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/file/downloadFile
     *
     * @param {number} taskFieldId
     * @return {Promise<SafeResult<Object, Error>>}
     */
    downloadFileByTaskFieldId(taskFieldId) {
        return this.request(
            {
                action: ServiceAction.FILE_DOWNLOAD_GET,
                pathParams: {
                    taskFieldId
                }
            },
            Object
        );
    }

    /**
     * @description Метод массового изменения статуса целей/задач
     * @link https://goodt-dev.goodt.me:8487/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/task/statusChangeBulk
     *
     * @param {Object} params
     * @param {number[]|number} params.userIds - Массив идентификаторов пользователей для фильтрации
     * @param {number[]|number} params.taskTypeIds - Массив идентификаторов типов цели для фильтрации
     * @param {number[]} [params.statusIds] - Массив идентификаторовтекущих статусов для фильтрации
     * @param {number[]} [params.processIds] - Массив идентификаторов процессов для фильтрации
     * @param {number[]|number} targetStatusId - Массив идентификаторов новых статусов для обновления
     *
     * @return {Promise<SafeResult<Object, Error>>}
     */
    updateTasksStatuses({ userIds, taskTypeIds, statusIds, processIds }, targetStatusId) {
        return this.request(
            {
                action: ServiceAction.TASK_STATUS_CHANGE_BULK_POST,
                params: {
                    users: [userIds].flat(),
                    task_type: [taskTypeIds].flat(),
                    current_status: [statusIds].flat(),
                    process_id: [processIds].flat(),
                    status: [targetStatusId].flat()
                }
            },
            Object
        );
    }

    /**
     * @description Изменение целей
     * @link https://goodt-dev.goodt.me:8487/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/task/updateTask
     *
     * @param { TaskBatchUpdateInput[] } tasks
     * @return {Promise<SafeResult<number[], Error>>}
     */
    updateTasks(tasks) {
        return this.request(
            {
                action: ServiceAction.TASK_UPDATE_PUT,
                params: tasks
            },
            Number
        );
    }

    /**
     *
     * @param {number} processId
     * @param {Object} [filters]
     * @param {string} [filters.externalId]
     * @param {string} [filters.dateStartStart]
     * @param {string} [filters.dateStartEnd]
     * @param {string} [filters.dateEndStart]
     * @param {string} [filters.dateEndEnd]
     * @param {Object} [options]
     * @param {boolean} [options.withClosed]
     * @return {Promise<SafeResult<ProcessStageDto[], Error>>}
     */
    getProcessStagesByFilter(
        processId,
        { externalId, dateStartStart, dateStartEnd, dateEndStart, dateEndEnd } = {},
        { withClosed } = {}
    ) {
        return this.request(
            {
                action: ServiceAction.PROCESS_STAGE_GET,
                params: {
                    process_id: processId,
                    external_id: externalId,
                    date_start_start: dateStartStart,
                    date_start_end: dateStartEnd,
                    date_end_start: dateEndStart,
                    date_end_end: dateEndEnd,
                    with_closed: withClosed
                }
            },
            ProcessStageDto
        );
    }

    /**
     * @description поиск типа таска
     * @link https://goodt-dev.goodt.me:8487/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/task_type/taskTypeFind
     *
     * @param {number[]} taskTypeIds
     * @param {Object} [filters]
     * @param {number} filters.parentId
     * @param {boolean} filters.useParentFilter - флаг, который отвечает за передачу параметра parentId
     * @return {Promise<SafeResult<TaskTypeDto[], Error>>}
     */
    getTaskTypesByIds(taskTypeIds, { parentId, useParentFilter = true } = {}) {
        return this.request(
            {
                action: ServiceAction.TASK_TYPE_FIND_POST,
                queryParams: {
                    parent_id: useParentFilter === true ? parentId : -1
                },
                params: taskTypeIds
            },
            TaskTypeDto
        );
    }

    /**
     * @description Получение информации о модулях
     * @link https://goodt-dev.goodt.me:8487/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/module/getModuleAll
     *
     * @return {Promise<SafeResult<ModuleDto[], Error>>}
     */
    getModules() {
        return this.request(
            {
                action: ServiceAction.MODULE_GET
            },
            ModuleDto
        );
    }

    /**
     * @description Получение информации о типах доступа
     * @link https://goodt-dev.goodt.me:8487/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/access_type/getAccessType
     *
     * @return {Promise<SafeResult<AccessTypeDto[], Error>>}
     */
    getAccessTypes() {
        return this.request(
            {
                action: ServiceAction.ACCESS_TYPE_GET
            },
            AccessTypeDto
        );
    }

    /**
     * @description Получение информации о типах пользователя
     * @link https://goodt-dev.goodt.me:8487/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/user_type/getUserTypeAll
     *
     * @return {Promise<SafeResult<UserTypeDto[], Error>>}
     */
    getUserTypes() {
        return this.request(
            {
                action: ServiceAction.USER_TYPE_GET
            },
            UserTypeDto
        );
    }

    /**
     * @description Получение информации о группах полей таска
     * @link https://goodt-dev.goodt.me:8487/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/task_type_field_group/getTaskTypeFieldGroupAll
     *
     * @return {Promise<SafeResult<TaskTypeFieldGroupDto[], Error>>}
     */
    getTaskTypeFieldGroups() {
        return this.request(
            {
                action: ServiceAction.TASK_TYPE_FIELD_GROUP_GET
            },
            TaskTypeFieldGroupDto
        );
    }

    /**
     * @description Получение информации о типах полей таска
     * @link https://goodt-dev.goodt.me:8487/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/task_type_field_type/getTaskTypeFieldTypeAll
     *
     * @return {Promise<SafeResult<TaskTypeFieldTypeDto[], Error>>}
     */
    getTaskTypeFieldTypes() {
        return this.request(
            {
                action: ServiceAction.TASK_TYPE_FIELD_TYPE_GET
            },
            TaskTypeFieldTypeDto
        );
    }

    /**
     * @description Создание типа цели
     * @link https://goodt-dev.goodt.me:8487/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/task_type/createTaskType
     *
     * @param {TaskTypeInputDto} taskType
     *
     * @return {Promise<SafeResult<number, Error>>}
     */
    createTaskType(taskType) {
        return this.request(
            {
                action: ServiceAction.TASK_TYPE_UPDATE_POST,
                params: taskType
            },
            Number
        );
    }

    /**
     * @description Изменение типа цели
     * @link https://goodt-dev.goodt.me:8487/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/task_type/updateTaskType
     *
     * @param {number} id
     * @param {TaskTypeInputDto} taskType
     *
     * @return {Promise<SafeResult<number, Error>>}
     */
    updateTaskType(id, taskType) {
        return this.request(
            {
                action: ServiceAction.TASK_TYPE_UPDATE_POST,
                params: { id, ...taskType }
            },
            Number
        );
    }

    /**
     * @description Закрытие полей типа цели
     * @link https://goodt-dev.goodt.me:8487/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/task_type_field/closeTaskTypeField
     *
     * @param {number[]} ids
     *
     * @return {Promise<SafeResult<number, Error>>}
     */
    deleteTaskTypeField(ids) {
        return this.request(
            {
                action: ServiceAction.TASK_TYPE_FIELD_DELETE_POST,
                queryParams: {
                    id: ids
                }
            },
            Number
        );
    }

    /**
     * @description Получение всех связей компонентов и полей типа таска (цели)
     * @link https://goodt-dev.goodt.me:8487/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/component_task_type_field/getAllComponentTaskTypeField
     *
     *
     * @return {Promise<SafeResult<ComponentTaskTypeFieldDto[], Error>>}
     */
    getComponentTaskTypeFields() {
        return this.request(
            {
                action: ServiceAction.COMPONENT_TASK_TYPE_FIELD_GET
            },
            ComponentTaskTypeFieldDto
        );
    }

    /**
     * @description Удаление типов целей по массиву идентификаторов
     * @link https://goodt-dev.goodt.me:8487/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/task_type/deleteTask
     *
     * @param {number|string} id
     * @return {Promise<SafeResult<Object, Error>>}
     */
    deleteTaskType(id) {
        return this.request(
            {
                action: ServiceAction.TASK_TYPE_DELETE_POST,
                queryParams: {
                    id
                }
            },
            Object
        );
    }

    get competence() {
        return {
            /**
             * @description Добавление и отображение комментария к уровню развития компетенции
             * @link https://goodt-dev.goodt.me:8487/swagger-ui/index.html?configUrl=/v3/api-docs/swagger-config#/BFF/getOrCreateTaskFieldCommentForBff
             *
             * @param {number} taskId - Идентификатор задачи
             * @param {Object} options - Объект с данными комментария
             * @param {string} options.comment - Текст комментария
             * @param {boolean} options.commentHidden - Флаг скрытости комментария
             *
             * @returns {Promise<SafeResult<CompetenceCommentDto, Error>>}
             */
             addComment: async (taskId, { comment, isCommentHidden }) => {
                return this.request(
                    {
                        action: ServiceAction.BFF_COMPETENCE_ADD_COMMENT_POST,
                        params: {
                            comment,
                            task_id: taskId,
                            comment_hidden: isCommentHidden
                        }
                    },
                    CompetenceCommentDto
                );
            }
        }
    }
}
