import { Url } from '@goodt-common/utils';
import { ConstManager } from '@goodt-wcore/managers';

export const processUrlWithConstants = (constName) => {
    if (constName.includes('%') === false) {
        return constName;
    }

    return Url.join(
        ...constName
            .match(/(.+)?(%.+%)(.+)?/)?.slice(1)
            .filter(Boolean)
            .map((part) => part.startsWith('%') && part.endsWith('%')
                ? ConstManager.instance.getConstValue(part)
                : part
            )
    );
};

export class ApiCacheManager {
    /**
     * @type {ConstManager}
     * @private
     */
    _constManager;

    /**
     * @type {import('@goodt/editor-common').IApiCacheService}
     * @private
     */
    _cacheService;

    /**
     * Cache service provider
     * @type {import('@goodt/editor-common').ApiCacheServiceProvider}
     * @private
     */
    _cacheServiceProvider;

    /**
     * @type {Map<string, UrlInfoType>}
     */
    _cachingUrlsMap = new Map();

    /**
     * Caching is enabled
     * @type {boolean}
     */
    isEnabled = false;

    /**
     * Debug mode is enabled
     * @type {boolean}
     */
    isDebugMode = false;

    /**
     * Cache service cache lifetime
     * @type {number}
     */
    cacheLifetime = 1000;

    /**
     * Cache service is enable lifetime
     * @type {number}
     */
    routeLifetime = 10000;

    /**
     *
     * @return {UrlInfoType[]}
     */
    get cachingUrls() {
        return [...this._cachingUrlsMap.values()];
    }

    /**
     *
     * @param {ConstManager} constManager
     * @param {import('@goodt/editor-common').ApiCacheServiceProvider} cacheServiceProvider
     */
    constructor({ constManager, cacheServiceProvider }) {
        this._constManager = constManager;
        this._cacheServiceProvider = cacheServiceProvider;
    }

    /**
     *
     * @param {string} urlWithConstants
     * @return {string}
     * @private
     */
    _$c(urlWithConstants) {
        return processUrlWithConstants(urlWithConstants);
    }

    /**
     *
     * @param {string} cacheServiceId
     * @return {Promise<void>}
     */
    async init({ cacheServiceId } = {}) {
        if (cacheServiceId != null) {
            await this.setCacheServiceById(cacheServiceId);
        }
    }

    /**
     *
     * @param {'http'|'sw'} cacheServiceId
     * @return {Promise<void>}
     */
    async setCacheServiceById(cacheServiceId) {
        this._cacheService = await this._cacheServiceProvider.get(cacheServiceId);
    }

    /**
     *
     * @param {boolean} isDebugMode
     */
    setDebugMode(isDebugMode) {
        this.isDebugMode = isDebugMode;
        this._cacheService?.setDebugMode(isDebugMode);
    }

    /**
     *
     * @param {boolean} enabled
     * @param {number} lifetime
     * @param {Array} urls
     * @param {string} cacheServiceId
     */
    async setConfig({ enabled = false, cacheLifetime, routeLifetime, urls = [], cacheServiceId } = {}) {
        await this.setCacheServiceById(cacheServiceId);
        if (cacheLifetime != null) {
            this.cacheLifetime = cacheLifetime;
        }
        if (routeLifetime != null) {
            this.routeLifetime = routeLifetime;
        }
        this.isEnabled = enabled;
        this._cachingUrlsMap = new Map(
            urls.map(({ url, enabled, options }) => [
                url, { url, enabled, ...options }
            ])
        );

        this.setupUrlsInCacheService();
    }

    /**
     *
     * @param {boolean} isEnabled
     */
    setupUrlsInCacheService(isEnabled = this.isEnabled) {
        // очищаем кеш, т.к. там могли остаться запросы от неиспользуемых урлов
        this.clearCache();
        // очищаем все кешируемые урлы
        this.clearUrlsInCacheService();

        if (isEnabled) {
            this._cacheService.lifetime = this.cacheLifetime;
            this.setDebugMode(this.isDebugMode);
            this.cachingUrls
                .filter(({ enabled }) => Boolean(enabled))
                .forEach(({ url }) => {
                    this._cacheService.addUrl(this._$c(url));
                });
        }

        this._cacheService.isEnabled = isEnabled;
    }

    clearUrlsInCacheService() {
        this._cacheService.clearUrls();
    }

    /**
     * @public
     */
    clearCache() {
        this._cacheService.clearCache();
    }

    toggle(isEnabled) {
        this.isEnabled = isEnabled;
    }

    /**
     * @param {{ url: string, enabled: boolean }[]} urlInfos
     * @return {void}
     */
    setUrls(urlInfos) {
        this._cachingUrlsMap.clear();
        urlInfos.forEach(({ url, ...options }) => {
            this.addUrl(url, options);
        })
    }

    /**
     *
     * @param {string} url
     * @param {boolean} enabled
     * @param {Record<string, any>} options
     * @return {void}
     */
    addUrl(url, { enabled, ...options }) {
        this._cachingUrlsMap.set(url, { url, enabled, ...options });
    }

    /**
     *
     * @param {string} url
     * @return {void}
     */
    removeUrl(url) {
        this._cachingUrlsMap.delete(url);
    }

    /**
     * Enables cache service for specified timeout/lifetime, then disable it
     *
     * @param {number} lifetime
     */
    startCacheServiceLifecycle(lifetime = this.routeLifetime) {
        if (this.isEnabled === false) {
            return;
        }
        this._cacheService.isEnabled = true;
        setTimeout(() => this._cacheService.isEnabled = false, lifetime);
    }

    /**
     * @return {void}
     */
    save() {
        this.setupUrlsInCacheService();
    }
}
