(() => {
    const query = new URLSearchParams(location.search);
    const isDebugMode = query.get('debug') === '1';
    const CACHE_VERSION = query.get('appVersion');

    const ASSETS_CACHE_NAME_PREFIX = 'Goodt.Assets';

    const LOG_LABEL = `ASSETS CACHE`;
    const LOG_STYLE = `background:#4d9864;color:black;font-weight:bold;border-radius:999px;padding: 1px 5px;margin-right;5px`;

    const CACHE_DESTINATIONS = [
        'script',
        'style',
        'font',
        'image'
    ];
    const MAX_FETCH_TRIES_COUNT = 3;

    // UTILS
    const filterLabelArgs = (args, isLabelArg = true) => {
        return args.filter((value) => {
            const { label } = value ?? {};
            return [null, undefined, ''].includes(label) === !isLabelArg;
        })
    };

    const log = (...args) => {
        if (isDebugMode === false) {
            return;
        }

        const { labels, styles } = filterLabelArgs(args, true).reduce((acc, {
            label,
            style = ''
        }) => {
            const { labels, styles } = acc;
            labels.push(`%c${label}`);
            styles.push(LOG_STYLE.concat(';', style));
            return {
                labels,
                styles
            };
        }, {
            labels: [ `%c${LOG_LABEL}` ],
            styles: [ LOG_STYLE ]
        });
        const nonLabelArgs = filterLabelArgs(args, false);

        console.log(labels.join(''), ...styles, ...nonLabelArgs);
    };

    const clearCache = async (isAll = false) => {
        return caches.keys().then((keys) => {
            return Promise.all(keys
                .filter((key) => key.startsWith(ASSETS_CACHE_NAME_PREFIX) && (isAll || key !== `${ASSETS_CACHE_NAME_PREFIX}.${CACHE_VERSION}`))
                .map((key) => {
                    log({ label: 'CLEAR' }, `deleting cache "${key}"...`);
                    return caches.delete(key);
                })
            );
        });
    };

    self.AssetsCache = {};
    self.AssetsCache.use = ({ isStandalone = false, isEnabled = false } = {}) => {
        let IS_ENABLED_DYNAMIC = null;

        const isCacheEnabled = () => IS_ENABLED_DYNAMIC ?? isEnabled;

        /**
         *
         * @param {ReadableStreamDefaultReader} reader
         * @return {Promise<boolean>}
         */
        const isStreamHasContent = async (reader) => {
            try {
                const { value } = await reader.read();
                return value !== undefined;
            } catch {
                return false;
            } finally {
                await reader.cancel();
            }
        };

        /**
         *
         * @param {Response} response
         * @return {Promise<boolean>}
         */

        const isResponseValid = async (response) => {
            if (response.status === 304) {
                if (response.body == null) {
                    log({ label: '304 – NULL' }, response.url);
                    return false;
                }
                const reader = response.clone().body.getReader();
                if (await isStreamHasContent(reader)) {
                    return true;
                }
                log({ label: '304 – EMPTY' }, response.url);
                return false;
            }
            return true;
        };

        /**
         *
         * @param {Request} request
         * @return {boolean}
         */
        const isAssetsRequest = (request) => {
            const { url, method } = request;
            const conditions = [
                method === 'GET',
                url.startsWith(location.origin)
            ];

            return conditions.every(Boolean);
        };

        const WorkerEventHandlers = {
            'install': async (event) => {
                event.waitUntil(
                    clearCache(isEnabled === false).then(() => {
                        if (isEnabled === false && isStandalone) {
                            return self.clients.matchAll().then(
                                (clients) =>
                                    clients
                                        .forEach((client) => client.navigate(client.url))
                            );
                        }

                    })
                );
                self.skipWaiting();
            },
            'fetch': (event) => {
                /**
                 *  @NOTE if used by importScript
                 */
                // for dynamic switching
                if (isCacheEnabled() === false) {
                    return;
                }

                const { request } = event;

                if (isAssetsRequest(request) === false) {
                    return;
                }
                // force cache reload to refresh index.html root js scripts
                if (request.destination === 'document') {
                    return fetch(request, {
                        cache: 'no-cache'
                    });
                }
                if (CACHE_DESTINATIONS.includes(request.destination) === false) {
                    return;
                }
                const makeRequest = async () => {
                    const requestUrl = isStandalone
                        ? request.url
                        : request.url.replace(/^(.+\/editor\/player\/)\d+(\/.+)$/, '$1:id$2');
                    const cache = await self.caches.open(`${ASSETS_CACHE_NAME_PREFIX}.${CACHE_VERSION}`);
                    const cachedResponse = await cache.match(requestUrl);
                    // Return a cached response if we have one
                    if (cachedResponse) {
                        log({ label: 'FROM_CACHE', style: 'background:#46f56d' }, cachedResponse.url);
                        return cachedResponse;
                    }

                    const fetchResponse = async () => {
                        log({ label: 'NETWORK', style: 'background:#0080ff' }, request.url);
                        const networkResponse = await fetch(request, {
                            cache: 'reload'
                        });
                        if (await isResponseValid(networkResponse)) {
                            cache.put(requestUrl, networkResponse.clone()).then(() => {
                                log({ label: 'TO_CACHE', style: 'background:#a87def' }, networkResponse.url);
                            });

                            return networkResponse;
                        }

                        return null;
                    };

                    let tries = 0;
                    let networkResponse = null;
                    do {
                        networkResponse = await fetchResponse();
                        tries += 1;
                        if (networkResponse == null) {
                            log({ label: 'FAILED', style: 'background:#0080ff' }, request.url);
                        }
                    } while (tries < MAX_FETCH_TRIES_COUNT && networkResponse == null);

                    if (networkResponse == null) {
                        throw new Error(`File has no content: "${request.url}"`);
                    }

                    return networkResponse;
                };
                event.respondWith(makeRequest());
            }
        };

        self.addEventListener('install', WorkerEventHandlers.install);

        if (isEnabled === false) {
            // clearCache(true);
            return;
        }

        // EVENT HANDLERS
        self.addEventListener('fetch', WorkerEventHandlers.fetch);

        const MessageEventName = {
            SET_CACHE_ENABLED: 'setAssetsCacheEnabled',
            CLEAR_CACHE: 'clearAssetsCache'
        };

        const MessageEventHandlers = {
            /**
             * dynamic cache is enabled switching in editors mode
             * @param {EventHandlerOpts} options
             * @param {(...*) => void} log
             */
            [MessageEventName.SET_CACHE_ENABLED]({ details, source }, { event }) {
                IS_ENABLED_DYNAMIC = details.isEnabled;
                event.waitUntil(clearCache(true));
            },
            [MessageEventName.CLEAR_CACHE](_, { event }) {
                clearCache(true)
                    .then(() => self.clients.matchAll()
                        .then(
                            (clients) => {
                                clients.forEach((client) => client.navigate(client.url));
                            }
                        ));
            }
        };

        self.addEventListener('message', (event) => {
            const { data, source } = event;
            const { type, details } = data;
            const handler = MessageEventHandlers[type];
            if (handler) {
                log({ label: 'MESSAGE' }, type, details);
                handler({ type, details, source }, { event });
            }
        });
    };
})();