import { getElemFullType } from '@goodt-wcore/compat';
import { getElemTypeMeta } from './elem';

/**
 * Y combinator
 *
 * @template T
 * @param {function(f: T): T} f
 * @return T
 */
const Y = (f) => ((x) => x(x))((x) => f((y) => x(x)(y)));

/**
 * @typedef {function(projectPageElem: import('@goodt/editor-common').IProjectPageElem): import('@goodt/editor-common').IProjectPageElem} PageDataElemBuilder
 */

const createPageDataWithElemTypesByMapBuilder = (TypesMap) => {
    return Y((process) => {
        /**
         * process fn
         * @type {PageDataElemBuilder}
         */
        return (projectPageElem) => ({
            ...projectPageElem,
            type: TypesMap.get(projectPageElem.type) ?? getElemFullType(getElemTypeMeta(projectPageElem.type)),
            children: projectPageElem.children.map(process)
        });
    })
}

/**
 *
 * @return {*}
 * @param elements
 */
const buildTypeHashes = (elements) => {
    return elements.reduce(
        ({ typeHash, typeHashInverted }, record) => {
            const { type } = record;
            const meta = getElemTypeMeta(type);
            const newType = getElemFullType(meta);
            if (type !== newType) {
                typeHash.set(type, newType);
                typeHashInverted.set(newType, type);
            } else {
                console.warn(`app config.elements patch failed (package replacement not found)`, record);
            }
            return { typeHash, typeHashInverted };
        },
        { typeHash: new Map(), typeHashInverted: new Map() }
    );
};

export class ElemCompatHelper {
    static TypesMap = new Map();
    static TypesInvertedMap = new Map();

    static init({ elements }) {
        const { typeHash, typeHashInverted } = buildTypeHashes(elements);
        this.TypesMap = typeHash;
        this.TypesInvertedMap = typeHashInverted;
    }

    static buildAppConfigWithNewElemTypes({ elements, tools, ...rest }) {
        const processType = (type) => this.TypesMap.get(type) ?? type;
        const elementsProcessed = elements.map(({ type, ...rest }) => ({ type: processType(type), ...rest }));
        const toolsProcessed = Object.entries(tools).reduce(
            (acc, [groupName, groupItems]) => ({
                ...acc,
                [groupName]: groupItems.map(({ elements, ...rest }) => ({
                    ...rest,
                    elements: elements.map(processType)
                }))
            }),
            {}
        );
        return {
            ...rest,
            elements: elementsProcessed,
            tools: toolsProcessed
        };
    }

    /**
     *
     * @param {import('@goodt/editor-common').IProjectPageData['content']} pageDataElems
     * @return {import('@goodt/editor-common').IProjectPageData['content']}
     */
    static buildPageDataWithOldElemTypes(pageDataElems) {
        const buildWithOldElemTypes = createPageDataWithElemTypesByMapBuilder(ElemCompatHelper.TypesInvertedMap);
        return [buildWithOldElemTypes(pageDataElems[0])];
    }

    /**
     * @return {import('@goodt/editor-common').IProjectPageElem[]}
     * @param pageDataElems
     */
    static buildPageDataWithNewElemTypes = (pageDataElems) => {
        const buildWithNewElemTypes = createPageDataWithElemTypesByMapBuilder(ElemCompatHelper.TypesMap);
        return [buildWithNewElemTypes(pageDataElems[0])];
    };

    /**
     * @param {string} elemType
     * @return {string}
     */
    static getOriginalElemType(elemType) {
        return this.TypesMap.get(elemType) ?? 'unknown';
    }
}
