import Vue from 'vue';
import VueRouter from 'vue-router';
import { get } from 'lodash';

import { ProjectDataProviderRepository } from '@goodt-wcore/core/dremio';
import { buildConstantsHash, ApiCacheManager, ApiCacheServiceProvider } from '@goodt/editor-common';

import { Const } from '@goodt-wcore/core';
import { AuthManager, ConstManager, StoreManager, RouteManager } from '@goodt-wcore/managers';
import { ServiceProvider } from '@goodt-common/services';
import { success, fail } from '@goodt-common/utils';

import { NAMESPACES } from '@/store/constants';
import Mutations from '@/store/mutations';
import Actions from '@/store/modules/editor/actions';

import AppEntity from '@/entities/App';
import Config from '@/utils/Config';
import ElemRegistry from '@/utils/ElemRegistry';
import wcoreEventBus from '@/utils/WCoreEventBus';

import { initWAuthManager } from '@/app/auth';
import { init as initErrorService, startErrorHandle, startErrorTracking } from '@/services/error-service';
import ProjectService from '@/services/AppService';
import { useKeycloakCustom } from '@/services/KeycloakAuth';
import { setDebugMode } from '@/utils/DevMode';
import { ElemCompatHelper } from '@goodt/editor-common/utils';

/**
 *
 */
export class Application {
    config = {};

    /** @type {import('vue/types/vue').Vue} */
    vueApp;

    /**
     * Vuex store instance
     * @type {import('vuex').Store}
     */
    vuex;

    /**
     * Widget's global state store
     * @type {import('@goodt-wcore/managers').Store}
     */
    store;

    /**
     * App router instance
     * @type {import('vue-router')}
     */
    router;

    /**
     * App route monitor instance
     * @type {RouteManager}
     */
    routeManager;

    /**
     * App route monitor instance
     * @type {ConstManager}
     */
    constManager;

    /**
     * App route monitor instance
     * @type {AuthManager}
     */
    authManager;

    /**
     * @type {import('vue-i18n').IVueI18n}
     */
    i18n;

    /**
     * @type {import('@goodt-wcore/managers/EventBus').EventBusWrapper}
     */
    eventBus;

    /**
     * @type {WorkerApi}
     */
    workerApi;

    /**
     * Service to cache api requests
     * @type {ApiCacheManager}
     */
    apiServiceCacheManager;

    constructor({
        i18n,
        vuex,
        router,
        store = StoreManager.store,
        authManager = AuthManager.instance,
        routeManager = RouteManager.instance,
        constManager = ConstManager.instance,
        eventBus = wcoreEventBus,
        workerApi
    }) {
        this.constManager = constManager;
        this.authManager = authManager;
        this.routeManager = routeManager;
        this.i18n = i18n;
        this.store = store;
        this.vuex = vuex;
        this.router = router;
        this.eventBus = eventBus;
        this.workerApi = workerApi;
        this.apiServiceCacheManager = new ApiCacheManager({
            constManager,
            cacheServiceProvider: new ApiCacheServiceProvider()
        });

        this._initErrorHandle();
        this._patchVuePlugins({ vuex, i18n, router });
    }

    get editorSettings() {
        return this.vuex.state[NAMESPACES.EDITOR].editorSettings;
    }

    /**
     *
     * @param {import('vue').Component} App
     * @param {import('vue/types/options').ComponentOptions} AppComponentOptions
     * @param {Record<string, string>} ConstMap
     * @param {string} version
     *
     * @return {Promise<Application>}
     */
    async init(App, AppComponentOptions, { ConstMap, version }) {
        ServiceProvider.setup();

        const { vuex, i18n } = this;

        this._setIsLoading(true);

        const appConfig = await this._loadAppConfig();
        const { elements, tools, services, api, errorTracking: isErrorTrackingEnabled } = appConfig;

        this._initErrorTracking(isErrorTrackingEnabled);
        this._initAppConstants({ appConfig, ConstMap });
        this._initAppState({ tools, locale: i18n.locale, version });
        await this._initElemRegistry({ elements });
        this.initServices({ services });
        this.initServiceCacheManager();
        await this.initKeycloak();

        this.projectService = new ProjectService({ transportOptions: api.http });

        // store specific
        await vuex.dispatch(`${NAMESPACES.EDITOR}/${Actions.LOAD_EDITOR_SETTINGS}`);
        await vuex.dispatch(`${NAMESPACES.EDITOR}/${Actions.AUTH_INIT}`);

        this.vueApp = this.createVueApp(App, AppComponentOptions);

        this._setIsLoading(false);

        this._afterVueAppCreated();

        return this;
    }

    /**
     *
     * @private
     */
    _afterVueAppCreated() {
        this.vueApp.$watch(
            () => this.editorSettings,
            this._onEditorSettingsChanged.bind(this),
            { deep: true, immediate: true }
        );
    }

    /**
     * Patches vue plugins with shared instances
     *
     * @param vuex
     * @param router
     * @param i18n
     * @private
     */
    _patchVuePlugins({ vuex, router, i18n }) {
        /*
         * Temporary solution
         * Adds reference to self App in Vuex instance
         */
        // eslint-disable-next-line no-param-reassign
        vuex.$app = this;
        // eslint-disable-next-line no-param-reassign
        router.i18n = i18n;
    }

    /**
     * @param {boolean} isLoading
     * @private
     */
    _setIsLoading(isLoading) {
        this.vuex.commit(Mutations.SET_LOADING, isLoading);
    }

    /**
     * @private
     */
    _initErrorHandle() {
        const { errorService } = initErrorService();
        const { appErrorManager } = startErrorHandle({ errorService });
        this.errorService = errorService;
        this.errorManager = appErrorManager;
    }

    /**
     * @private
     */
    _initErrorTracking(isEnabled) {
        if (isEnabled === true) {
            startErrorTracking({ errorService: this.errorService });
        }
    }

    /**
     *
     * @param {{ constants:import('@goodt/editor-common').IProjectConstGroup[] }} options
     */
    initConstManager({ constants = [] } = {}) {
        this.constManager.setConstantsHash(buildConstantsHash(constants));
    }

    /**
     *
     * @param adapter
     * @return {Promise<AppSession>}
     */
    initAuthManager({ adapter }) {
        return initWAuthManager(adapter, this.authManager);
    }

    /**
     *
     * @param services
     */
    initServices({ services = [] }) {
        ServiceProvider.init(services);
    }

    /**
     * @param {{ dataProviders: import('@goodt/editor-common').IProjectDataProvider[] }} options
     */
    initDataProviders({ dataProviders = [] } = {}) {
        ProjectDataProviderRepository.set(dataProviders);
    }

    /**
     *
     *
     */
    async initKeycloak() {
        await useKeycloakCustom();
    }

    /**
     */
    initServiceCacheManager() {
        this.apiServiceCacheManager.init();
    }

    /**
     *
     * @param {{ removeLocalState:boolean }} options
     */
    initStore({ removeLocalState = false } = {}) {
        if (removeLocalState) {
            const { state } = this.store;
            const stateNew = Object.entries(state).reduce((obj, [key, val]) => {
                const isVO = val instanceof StoreManager.ValueObject;
                const { meta } = val;
                if (isVO && meta?.global) {
                    obj[key] = val;
                }
                return obj;
            }, {});
            this.store.replace(stateNew);
            return;
        }
        this.store.replace({});
    }

    /**
     *
     */
    initEventBus() {
        this.eventBus.reset();
    }

    /**
     *
     * @return {Promise<null>}
     * @private
     */
    async _loadAppConfig() {
        const config = await Config.init();
        ElemCompatHelper.init(config);
        this.config = ElemCompatHelper.buildAppConfigWithNewElemTypes(config);
        return this.config;
    }

    /**
     * App Vuex-store dependent state values
     *
     * @private
     * @param {string} locale
     * @param {Record<string, any>} tools
     * @param {string} version
     */
    _initAppState({ locale, tools, version }) {
        this.vuex.commit(Mutations.SET_LOCALE, locale);
        this.vuex.commit(Mutations.SET_TOOLS, tools);
        this.vuex.commit(Mutations.SET_VERSION, version);
        this.vuex.commit(Mutations.SET_INIT, true);
    }

    /**
     * Creates Vue Instance
     *
     * @private
     * @param {import('vue').Component} App
     * @param {import('vue/types/options').ComponentOptions} AppComponentOptions
     *
     * @return {import('vue').Vue}
     */
    createVueApp(App, { store = this.vuex, router = this.router, i18n = this.i18n, ...restComponentOptions }) {
        const { errorManager, errorService } = this;
        const vueApp = new Vue({
            name: 'Facade',
            store,
            router,
            i18n,
            shadowRoot: document.head,
            beforeCreate() {
                // to be accessed via app.$errorService in vue dev-tools
                this.$errorService = errorService;
                this.$errorManager = errorManager;
            },
            provide: () => ({
                $errorManager: errorManager,
                $errorService: errorService,
                $app: this
            }),
            realmAppOptions: {
                router: new VueRouter(),
                i18n
            },
            render: (h) => h(App),
            ...restComponentOptions
        });
        vueApp.$mount('#app');

        return vueApp;
    }

    /**
     *
     * @param {Record<string, any>} appConfig
     * @param {Record<string, any>} ConstMap
     * @private
     */
    _initAppConstants({ appConfig, ConstMap }) {
        // init wcore const
        Object.entries(ConstMap).forEach(([keyConst, keyConfig]) => {
            Const[keyConst] = get(appConfig, keyConfig, '');
        });
    }

    /**
     *
     * @param {Record<string, any>[]} elements
     * @private
     */
    _initElemRegistry({ elements }) {
        // register elems meta
        elements.forEach(({ type, name, icon }) => ElemRegistry.add({ type, name, icon }));
        // try to register default elems as they might not be present in config
        ElemRegistry.addDefaults();
        return ElemRegistry.ready();
    }

    /**
     * Loads project app json config and re-init base managers and services
     * @param {string} appId
     */
    async initProjectApp(appId) {
        try {
            const app = await this.projectService.appInfo(appId);
            app.data.constants = this.buildConstsGroups(app.data.constants);

            const {
                data: { constants, authAdapter, services, cache: { services: servicesCacheConfig = {} } = {} }
            } = app;

            this.initEventBus();
            this.initStore();
            this.initConstManager({ constants });
            this.initServices({ services });
            this._onEditorSettingsChanged();

            const session = await this.initAuthManager({ adapter: authAdapter });

            await this.apiServiceCacheManager.setConfig(servicesCacheConfig);
            return success({ app, session });
        } catch (error) {
            return fail(error);
        }
    }

    /**
     * Builds constant groups
     * @param {import('@goodt/editor-common').IProjectConsts} constants
     * @return {import('@goodt/editor-common').IProjectConstGroup[]}
     */
    buildConstsGroups(constants) {
        if (Array.isArray(constants)) {
            return constants;
        }
        const group = AppEntity.constGroup({ name: 'default', hash: constants });
        return [group];
    }

    /**
     * @private
     */
    _onEditorSettingsChanged() {
        const { isDeveloperMode } = this.editorSettings;
        this._toggleDevMode(isDeveloperMode);
    }

    /**
     * Toggle dev mode logging
     * @param {boolean} isEnabled
     * @private
     */
    _toggleDevMode(isEnabled) {
        this.apiServiceCacheManager.setDebugMode(isEnabled);
        setDebugMode(isEnabled);
    }
}
