const { ColorVariant } = require('goodt-framework-css/scripts/theme-color');
const { Parser } = require('goodt-framework-css/scripts/theme-parser');

/**
 * @typedef {object} IThemeParserCssRule
 * @property {string} type
 * @property {string} kind
 * @property {string} source
 * @property {string?} selector
 * @property {Record.<string, string>} style
 */
/**
 * @typedef {object} ITheme
 * @property {Record.<string, string>} root
 * @property {Record.<string, string>[]} fonts
 * @property {string} custom
 */
const ThemeParser = {
    Kind: {
        FONT_FACE: 'fontface'
    },
    Selector: {
        ROOT: ':root',
        FONT_FACE: '@font-face'
    },
    Comment: {
        ROOT: 'root',
        FONTS: 'custom fonts',
        CUSTOM: 'custom css'
    },
    /**
     * @param {string} cssSource
     * @returns {Promise.<ITheme[]>}
     */
    async parse(cssSource) {
        const { Kind, Selector } = this;
        const el = new CSSStyleSheet();
        const { cssRules } = await el.replace(cssSource);
        /** @type {IThemeParserCssRule[]} */
        const rules = [...cssRules].map((rule) => {
            const { cssText: source, selectorText: selector = null, style = [] } = rule;
            const type = rule.constructor.name;
            return {
                type,
                kind: type.replace(/^CSS(.+)Rule$/, '$1').toLowerCase(),
                source,
                selector,
                style: [...style].reduce(
                    (acc, name) => ({
                        ...acc,
                        [name]: style.getPropertyValue(name)
                    }),
                    {}
                )
            };
        });
        const root = rules.find(({ selector }) => selector === Selector.ROOT);
        const fonts = rules.filter(({ kind }) => kind === Kind.FONT_FACE).map(({ style }) => style);
        const custom = rules
            .filter(({ kind, selector }) => kind !== Kind.FONT_FACE && selector !== Selector.ROOT)
            .map(({ source }) => source)
            .join('\n');

        return {
            root: root?.style ?? {},
            fonts,
            custom
        };
    },
    /**
     * @param {ITheme} theme
     * @return {string}
     */
    toCss({ root, fonts, custom }) {
        const { wrapWithComment: comment, wrapWithSelector: selector, styleToStr, Comment, Selector } = this;
        const rootDef = selector(styleToStr(root), Selector.ROOT);
        const fontDefs = fonts.map((fontStyle) => selector(styleToStr(fontStyle), Selector.FONT_FACE));
        return [
            comment(rootDef, Comment.ROOT),
            comment(fontDefs, Comment.FONTS),
            comment(`${custom}\n`, Comment.CUSTOM)
        ].join('\n\n');
    },
    /**
     * @param {Record.<string, string>} props
     * @return {string}
     */
    styleToStr(style) {
        return Object.entries(style)
            .map(([key, val]) => `${key}: ${val};`)
            .join('\n');
    },
    /**
     * @param {string} source
     * @param {string} comment
     * @return {string}
     */
    wrapWithComment(source, comment) {
        return `/* ---- ${comment} ---- */\n${source}`;
    },
    /**
     * @param {string} source
     * @param {string} selector
     * @return {string}
     */
    wrapWithSelector(source, selector) {
        return `${selector} {\n${source}\n}`;
    }
};

module.exports = { ColorVariant, Parser, ThemeParser };
