import Vue from 'vue';
import Router from 'vue-router';
import { upperFirst } from 'lodash';
import { Url } from '@goodt-common/utils';
import store from '@/store';
import { NAMESPACES } from '@/store/constants';
import Const from '@/utils/Const';

Vue.use(Router);

let redirectPath = null;
const redirectUrl = localStorage.getItem('redirect');

const Middlewares = {
    auth(to, from, next, { store }) {
        const { authenticated } = store.state[NAMESPACES.EDITOR].editorSession;
        if (authenticated === false) {
            redirectPath = to.path;
            return next({ name: 'auth' });
        }
        if (redirectUrl != null) {
            localStorage.removeItem('redirect');
            window.location.href = redirectUrl;
            return;
        }
        if (redirectPath != null) {
            const path = redirectPath;
            redirectPath = null;
            return next({ path });
        }
        return next();
    },
    guest(to, from, next, { store }) {
        const { authenticated } = store.state[NAMESPACES.EDITOR].editorSession;

        if (authenticated === true) {
            return next({ path: '/apps' });
        }
        return next();
    },
    title(to, from, next, { i18n }) {
        document.title = upperFirst(i18n.t(to.meta.title));
        next();
    },
    appTitle(to, from, next, { store, i18n }) {
        const dispose = store.subscribe((mutation, state) => {
            const { app } = state[NAMESPACES.APP];
            if (app != null) {
                document.title = upperFirst(i18n.t(app.meta.title));
                dispose();
            }
        });
        next();
    }
};

const router = new Router({
    routes: [
        {
            path: '/',
            redirect: '/auth',
            meta: {
                title: '',
                menuIcon: '',
                showMenu: true,
                showInMenu: false,
                middlewares: [Middlewares.title, Middlewares.guest]
            }
        },
        {
            path: '/apps',
            components: {
                default: () => import(/* webpackChunkName: "apps" */ '@/views/Apps.vue')
            },
            meta: {
                title: 'projects',
                menuIcon: 'mdi-folder',
                showMenu: true,
                showInMenu: true,
                middlewares: [Middlewares.title, Middlewares.auth]
            },
            children: [
                {
                    path: '',
                    name: 'app-list',
                    components: {
                        default: () => import(/* webpackChunkName: "app-list" */ '@/views/AppList.vue')
                    },
                    meta: {
                        title: 'projects list',
                        menuIcon: '',
                        showMenu: true,
                        showInMenu: false,
                        middlewares: [Middlewares.title, Middlewares.auth]
                    },
                    props: {
                        default: {
                            appType: Const.APP_TYPE_APP
                        }
                    }
                },
                {
                    path: 'edit/:id',
                    name: 'app-editor',
                    components: {
                        default: () => import(/* webpackChunkName: "app-editor" */ '@/views/AppEditor.vue')
                    },
                    meta: {
                        title: 'project editor',
                        menuIcon: '',
                        showMenu: true,
                        showInMenu: false,
                        middlewares: [Middlewares.auth, Middlewares.appTitle]
                    }
                }
            ]
        },
        {
            path: '/templates',
            components: {
                default: () => import(/* webpackChunkName: "apps" */ '@/views/Apps.vue')
            },
            meta: {
                title: 'templates',
                menuIcon: 'mdi-file',
                showMenu: true,
                showInMenu: true,
                middlewares: [Middlewares.title, Middlewares.auth]
            },
            children: [
                {
                    path: '',
                    name: 'template-list',
                    components: {
                        default: () => import(/* webpackChunkName: "app-list" */ '@/views/AppList.vue')
                    },
                    meta: {
                        title: 'templates list',
                        menuIcon: '',
                        showMenu: true,
                        showInMenu: false,
                        middlewares: [Middlewares.title, Middlewares.auth]
                    },
                    props: {
                        default: {
                            appType: Const.APP_TYPE_TEMPLATE
                        }
                    }
                },
                {
                    path: 'edit/:id',
                    name: 'template-editor',
                    components: {
                        default: () => import(/* webpackChunkName: "app-editor" */ '@/views/AppEditor.vue')
                    },
                    meta: {
                        title: 'template editor',
                        menuIcon: '',
                        showMenu: true,
                        showInMenu: false,
                        middlewares: [Middlewares.auth, Middlewares.appTitle]
                    }
                }
            ]
        },
        {
            path: '/data-providers',
            name: 'data-providers',
            components: {
                default: () => import(/* webpackChunkName: "data-providers" */ '@/views/DataProviders.vue')
            },
            meta: {
                title: 'data providers',
                menuIcon: 'mdi-source-branch',
                showMenu: true,
                showInMenu: true,
                middlewares: [Middlewares.title, Middlewares.auth]
            }
        },
        {
            path: '/metrics',
            name: 'metrics',
            components: {
                default: () => import(/* webpackChunkName: "metrics" */ '@/views/Metrics.vue')
            },
            meta: {
                title: 'metrics',
                menuIcon: 'mdi-gauge',
                showMenu: true,
                showInMenu: false,
                middlewares: [Middlewares.title, Middlewares.auth]
            }
        },
        {
            path: '/api',
            name: 'api',
            components: {
                default: () => import(/* webpackChunkName: "api" */ '@/views/Api.vue')
            },
            meta: {
                title: 'api',
                menuIcon: 'mdi-api',
                showMenu: true,
                showInMenu: false,
                middlewares: [Middlewares.title, Middlewares.auth]
            }
        },
        {
            path: '/about',
            name: 'about',
            components: {
                default: () => import(/* webpackChunkName: "index" */ '@/views/About.vue')
            },
            meta: {
                title: 'about',
                menuIcon: 'mdi-information-outline',
                showMenu: true,
                showInMenu: true,
                middlewares: [Middlewares.title, Middlewares.auth]
            }
        },
        {
            path: '/auth',
            name: 'auth',
            components: {
                default: () => import(/* webpackChunkName: "auth" */ '@/views/Auth.vue')
            },
            meta: {
                title: 'authorization',
                menuIcon: '',
                showMenu: false,
                showInMenu: false,
                middlewares: [Middlewares.title, Middlewares.guest]
            }
        },
        {
            path: '*',
            name: '404',
            components: {
                default: () => import('@/views/NotFound.vue')
            },
            meta: {
                title: 'page not found',
                menuIcon: '',
                showMenu: false,
                showInMenu: false,
                middlewares: [Middlewares.title]
            }
        }
    ]
});

router.beforeEach((to, from, next) => {
    if (to != null && from != null && to.path === from.path) {
        // same route
        return next();
    }

    const { middlewares: middlewaresToRun } = to.meta;

    if (middlewaresToRun == null) {
        return next();
    }

    const runMiddlewares = (middlewares, i = 0) => {
        const fn = middlewares[i];
        if (fn == null) {
            return next();
        }
        const fnNext = (...params) => {
            // call original 'router.next'
            if (params.length > 0) {
                next.apply(this, params);
            }
            // call next middlewares
            else {
                runMiddlewares(middlewares, i + 1);
            }
        };
        // console.log(`middlewares[${i}]`, fn);
        fn(to, from, fnNext, { store, i18n: router.i18n });
    };
    return runMiddlewares(middlewaresToRun);
});

export default router;
