import Service from './Service';
import { ElemCompatHelper } from '@goodt/editor-common/utils';

export default class AppService extends Service {
    /**
     * Возвращает список приложений @see AppEntity.appMeta()
     * @param {String} type
     * @param {String} [folder_id=null]
     * @return {Promise} ~ AppEntity.appMeta[]
     */
    appsList({ type, folder_id = null }) {
        let params = {
            type
        };
        if (folder_id != null) {
            params.folder_id = folder_id;
        }
        return this.transport.request('project', params, (response) => response.data);
    }
    /**
     * Получение приложения по id @see AppEntity.appInfo()
     * @param {String} id   id проекта
     * @return {Promise<import('@goodt/editor-common').IProject>} ~ AppEntity.appInfo[]
     */
    appInfo(id) {
        return this.transport
            .request(`project/${id}`, {}, (response) => response.data)
            .then(({ meta, data }) => ({ meta, data: JSON.parse(data) }));
    }
    /**
     * Удаляет приложение
     * @param {String} id   id проекта
     * @return {Promise} ~ data boolean
     */
    appDelete(id) {
        return this.transport.request(`project/${id}`, {}, (response) => response.data.result, 'delete');
    }
    /**
     * Создает приложение
     * @param {object} appMeta
     * @param {object} appData
     * @return {Promise} data ~ id
     */
    appCreate(appMeta, appData) {
        return new Promise((resolve, reject) => {
            this.appCreateMeta(appMeta)
                .then((id) => {
                    this.appUpdateData(id, appData).then(() => resolve(id));
                })
                .catch((e) => reject(e));
        });
    }
    /**
     * Клонирует приложение
     * @param {object} appMeta
     * @return {Promise} data ~ id
     */
    appClone(appMeta) {
        const params = {
            ...appMeta
        };
        return this.transport.request(`project/${appMeta.id}/clone`, params, (response) => response.data.id, 'post');
    }
    /**
     * Создает приложение на основе загруженного json-файла
     * @param {object} appMeta
     * @param {Blob} appData
     * @return {Promise} data ~ id
     */
    appImport(appMeta, appData) {
        const opts = {
            headers: {
                'Content-Type': 'multipart/form-data'
            }
        };
        const params = new FormData();
        const blob = new Blob([JSON.stringify(appMeta)], {
            type: 'application/json'
        });

        params.append('body', blob);
        params.append('archive', appData);

        return this.transport.request('project/import', params, (response) => response.data.id, 'post', opts);
    }
    /**
     * Создает meta приложение
     * @param {object} appMeta
     * @return {Promise} data ~ id
     */
    appCreateMeta({ id = null, folder_id = null, ...meta }) {
        const params = { ...meta };
        if (id) {
            params.id = id;
        }
        if (folder_id) {
            params.folder_id = folder_id;
        }
        return this.transport.request('project', params, (response) => response.data.id, 'post');
    }
    /**
     * Обновляет meta приложение
     * @param {object} appMeta
     * @return {Promise} data ~ id
     */
    appUpdateMeta({ id, folder_id = null, ...meta }) {
        const params = { ...meta };

        if (folder_id) {
            params.folder_id = folder_id;
        }
        return this.transport.request(`project/${id}`, params, (response) => response.data.id, 'put');
    }
    /**
     * Обновляет data приложение
     * @param {String} id       id проекта
     * @param {object} appData
     * @return {Promise}
     */
    appUpdateData(id, appData) {
        const params = {
            data: JSON.stringify(appData)
        };
        return this.transport.request(`project/${id}/updatedata`, params, (response) => response.data, 'put');
    }
    /**
     * Возвращает ссылку для экспорта приложения по id
     * @param {String} id   id проекта
     */
    appExport({ id }) {
        window.location.href = `./api/project/${id}/export`;
    }
    appExistsBySlug({ id: exclude_id, slug }) {
        const params = { exclude_id };
        return this.transport.request(`/project/${slug}/exists`, params, (response) => response.data.result);
    }
    /**
     * Возвращает ссылку на preview приложения по id/slug
     * @param {{ id:string, slug:string }} meta   id/slug проекта
     * @return {string}
     */
    appPreviewUrl({ id, slug }) {
        const base = location.href.replace(location.hash, '').replace(location.search, '');
        return `${base}player/${slug ? slug : id}/`;
    }
    /**
     * Получение темы (css) приложения по id
     * @param {String} id   id проекта
     * @return {Promise}
     */
    appTheme(id) {
        const request = this.transport.request(`project/${id}/css/theme.css`, {}, (response) => response.data);
        return request.catch((e) => '');
    }
    /**
     * Обновляет тему (less) приложения
     * @param {String} id       id проекта
     * @param {String} thmee    тема проекта
     * @return {Promise}
     */
    appUpdateTheme(id, theme) {
        const params = {
            id,
            theme
        };
        return this.transport.request(`project/${id}/updatetheme`, params, (response) => response.data, 'put');
    }
    /**
     * Добавляет шаринг приложения для группы
     * @param {String} id         id проекта
     * @return {Promise}
     */
    appShareGroup(id) {
        return this.transport.request(`project/${id}/sharegroup`, {}, (response) => response.data, 'put');
    }
    /**
     * Удаляет шаринг приложения для группы
     * @param {String} id         id проекта
     * @return {Promise}
     */
    appUnshareGroup(id) {
        return this.transport.request(`project/${id}/unsharegroup`, {}, (response) => response.data, 'put');
    }
    /**
     * Возвращает список файлов (upload) проекта
     * @param {String} id   id проекта
     * @return {Promise.<import('@/entities/FileInfo').FileInfo[]>}
     */
    appFilesList(id) {
        return this.transport.request(`project/${id}/file`, {}, (response) =>
            response.data.sort((a, b) => a.lastModified - b.lastModified)
        );
    }
    /**
     * Возвращает все файлы (upload) проекта zip архивом
     * @param {String} id
     */
    appFilesDownload(id) {
        window.location.href = `./api/project/${id}/file/download`;
    }
    /**
     * Закачивает файл (upload) проекта
     * @param {String} id   id проекта
     * @param {File} file   файл
     * @param {Function} [onUploadProgress=null]       handler(e:ProgressEvent)
     * @return {Promise}
     */
    appFileUpload(id, file, onUploadProgress = null) {
        const params = new FormData();
        params.append('file', file);
        const options = {
            headers: {
                'Content-Type': 'multipart/form-data'
            },
            onUploadProgress
        };
        return this.transport.request(`project/${id}/file`, params, (response) => response.data, 'post', options);
    }
    /**
     * Удаляет файл (upload) проекта
     * @param {String} id           id проекта
     * @param {String} fileHash     хеш файла
     */
    appFileDelete(id, fileHash) {
        return this.transport.request(
            `project/${id}/file/${fileHash}`,
            {},
            (response) => response.data.result,
            'delete'
        );
    }

    /**
     * Получает информацию о странице приложения
     * @param {String} appId            id приложения
     * @param {String} pageId           id страницы
     * @return {Promise.<import('@goodt/editor-common').IProjectPageElem[]>}
     */
    appPageInfo(appId, pageId) {
        return this.transport.request(`project/${appId}/page/${pageId}`, {}, ({ data: { data: dataStr } }) =>
            ElemCompatHelper.buildPageDataWithNewElemTypes(JSON.parse(dataStr))
        );
    }

    /**
     * Создает новую страницу
     * @param {String} appId         id проекта
     * @param {String} pageId        id страницы проекта
     * @param {Object} content       контент страницы проекта
     * @return {Promise}
     */
    appCreatePage(appId, pageId, content) {
        content = ElemCompatHelper.buildPageDataWithOldElemTypes(content);
        const params = {
            data: JSON.stringify(content),
            page_id: pageId
        };
        return this.transport.request(`project/${appId}/page`, params, (response) => response.data, 'post');
    }
    /**
     * Сохраняет изменения на странице
     * @param {String} appId         id проекта
     * @param {String} pageId        id страницы проекта
     * @param {Object} content       контент страницы проекта
     * @return {Promise}
     */
    appUpdatePage(appId, pageId, content) {
        content = ElemCompatHelper.buildPageDataWithOldElemTypes(content);
        const params = {
            data: JSON.stringify(content)
        };
        return this.transport.request(`project/${appId}/page/${pageId}`, params, (response) => response.data, 'put');
    }

    /**
     * Удаляет страницу из проекта
     * @param {String} appId         id проекта
     * @param {String} pageId        id страницы проекта
     * @return {Promise}
     */
    appDeletePage(appId, pageId) {
        return this.transport.request(
            `project/${appId}/page/${pageId}`,
            {},
            (response) => response.data.result,
            'delete'
        );
    }
}
