import { Url } from '@goodt-common/utils';
import Service from './Service';

const responseHandler = ({ data }) => data;

/**
 * @typedef {object} IConnection
 * @property {number|string} id
 * @property {boolean} byDefault
 * @property {string} name
 * @property {string} description
 * @property {string} server
 * @property {string} username
 * @property {string} password
 * @property {string} database
 * @property {string} schema
 * @property {string} connectionApi
 * @property {string} userUid
 * @property {string} config
 * @property {string|number} adapterId
 */
/**
 * @typedef {object} IAdapter
 * @property {number|string} id
 * @property {string} code
 * @property {string} name
 * @property {string} description
 */

/**
 * @param {IConnection} [options]
 * @return {IConnection}
 */
export const IConnectionFactory = ({
    id = null,
    byDefault = true,
    name = '',
    description = '',
    server = '',
    username = '',
    password = '',
    database = '',
    schema = '',
    connectionApi = '',
    userUid = '',
    config = '',
    adapterId = null
} = {}) => ({
    id,
    byDefault,
    name,
    description,
    server,
    username,
    password,
    database,
    schema,
    connectionApi,
    userUid,
    config,
    adapterId
});

export const AdapterCode = {
    DREMIO: 'DREMIO',
    DRILL: 'DRILL',
    POSTGRE: 'PG'
};

export const AdapterMeta = {
    [AdapterCode.DREMIO]: {
        icon: './i/ds/dremio.svg'
    },
    [AdapterCode.DRILL]: {
        icon: './i/ds/manager.svg'
    },
    [AdapterCode.POSTGRE]: {
        icon: './i/ds/pg.svg'
    }
};

export const DrillAdapterSchemas = [
    {
        name: 'PostgreSQL',
        icon: './i/ds/pg.svg',
        schema: {
            type: 'jdbc',
            enabled: true,
            driver: 'org.postgresql.Driver',
            url: 'jdbc:postgresql://1.2.3.4/mydatabase',
            username: 'user',
            password: 'password'
        }
    },
    {
        name: 'MySQL',
        icon: './i/ds/mysql.svg',
        schema: {
            type: 'jdbc',
            driver: 'com.mysql.jdbc.Driver',
            url: 'jdbc:mysql://localhost:3306',
            username: 'root',
            password: 'mypassword',
            enabled: true
        }
    },
    {
        name: 'MSSQL',
        icon: './i/ds/mssql.svg',
        schema: {
            type: 'jdbc',
            enabled: true,
            driver: 'com.microsoft.sqlserver.jdbc.SQLServerDriver',
            url: 'jdbc:sqlserver://1.2.3.4:1433;databaseName=mydatabase',
            username: 'user',
            password: 'password'
        }
    },
    {
        name: 'Oracle',
        icon: './i/ds/oracle.svg',
        schema: {
            type: 'jdbc',
            enabled: true,
            driver: 'oracle.jdbc.OracleDriver',
            url: 'jdbc:oracle:thin:user/password@1.2.3.4:1521/ORCL'
        }
    },
    {
        name: 'ClickHouse',
        icon: './i/ds/clickhouse.svg',
        schema: {
            type: 'jdbc',
            enabled: true,
            driver: 'ru.yandex.clickhouse.ClickHouseDriver',
            url: 'jdbc:clickhouse://1.2.3.4:8123.default',
            username: 'user',
            password: 'password'
        }
    },
    {
        name: 'Hive',
        icon: './i/ds/hive.svg',
        schema: {
            type: 'hive',
            enabled: false,
            configProps: {
                'hive.metastore.uris': '',
                'javax.jdo.option.ConnectionURL': 'jdbc:derby:;databaseName=../sample-data/drill_hive_db;create=true',
                'hive.metastore.warehouse.dir': '/tmp/drill_hive_wh',
                'fs.default.name': 'file:///',
                'hive.metastore.sasl.enabled': 'false'
            }
        }
    },
    {
        name: 'Mongo',
        icon: './i/ds/mongo.svg',
        schema: {
            type: 'mongo',
            connection: 'mongodb://localhost:27017/',
            enabled: false
        }
    },
    {
        name: 'HTTP',
        icon: './i/ds/http.svg',
        schema: {
            type: 'http',
            cacheResults: true,
            connections: {},
            timeout: 0,
            proxyHost: null,
            proxyPort: 0,
            proxyType: null,
            proxyUsername: null,
            proxyPassword: null,
            enabled: true
        }
    }
];

export class ConnectorApiService extends Service {
    /**
     * @param {{ url }} options
     * @return {ConnectorApiService}
     */
    context({ url }) {
        const baseURL = Url.join(url, 'api/v1').toString();
        return new this.constructor({ transportOptions: { baseURL } });
    }

    /**
     * @return {Promise.<IConnection[], Error>}
     */
    connectionList() {
        return this.transport.request('connections', {}, responseHandler);
    }

    /**
     * @param {string|number} id
     * @return {Promise.<IConnection, Error>}
     */
    connectionGet(id) {
        return this.transport.request(`connections/${id}`, {}, responseHandler);
    }

    /**
     * @param {IConnection} connection
     * @return {Promise.<IConnection,Error>}
     */
    connectionCreate(connection) {
        return this.transport.request('connections', connection, responseHandler, 'post');
    }

    /**
     * @param {IConnection} connection
     * @return {Promise.<IConnection,Error>}
     */
    connectionUpdate(connection) {
        return this.transport.request(`connections/${connection.id}`, connection, responseHandler, 'put');
    }

    /**
     * @param {IConnection} connection
     * @return {Promise}
     */
    connectionDelete(id) {
        return this.transport.request(`connections/${id}`, {}, responseHandler, 'delete');
    }

    /**
     * @param {IConnection} connection
     * @return {Promise}
     */
    connectionRestore(id) {
        throw new Error(`${this.name} method not yet implemented`);
    }

    /**
     * @return {Promise.<IAdapter[], Error>}
     */
    adapterList() {
        return this.transport.request('adapters', {}, responseHandler);
    }
}
