/* eslint-disable max-classes-per-file */
import HttpBase from '@goodt-wcore/core/net/HttpBase';
import { useKeycloakCustom } from '@/services/KeycloakAuth';
import { SessionExpiredError, InvalidTokenError } from './errors';

/**
 * Http transport
 * @param {object} options дефолтные настройки @default {} @see https://www.npmjs.com/package/axios
 */
export default class ServiceTransport extends HttpBase {
    constructor(options) {
        super(options);
        this.initHooks();
    }

    /**
     *
     */
    initHooks() {
        this.axios.interceptors.request.use(async (config = {}) => {
            try {
                const {
                    details: { token }
                } = await (await useKeycloakCustom()).token();
                const configFinal = { ...config };
                configFinal.headers.Authorization = `Bearer ${token}`;
                return configFinal;
            } catch (e) {
                throw new InvalidTokenError('Invalid token');
            }
        });
    }

    /**
     * Создает новый http request
     * @param {String} url                  ссылка api метода @example '/user/edit'
     * @param {object} params               params @default {}
     * @param {function} responseHandler    custom response handler,который орабатывает response @default null @example (response) => { return response.data; }
     * @param {String} method               method @default 'get'
     * @param {object} options              request options @default {} @see https://www.npmjs.com/package/axios
     * @return {Promise}                    promise, имеет параметр 'id' для отмены request @see this.cancelRequest()
     */
    request(url, params = {}, responseHandler = null, method = 'get', options = {}) {
        return super.request({ url, method, params, options, responseHandler });
    }

    /**
     * @param {Error} error
     */
    _processTransportError(error) {
        const sessionError = new SessionExpiredError('Session expired');
        if (error instanceof InvalidTokenError) {
            return sessionError;
        }

        if (this.constructor.isTransportError(error)) {
            const { status = 0 } = error.response ?? {};
            const isSessionError = status > 400 && status <= 405 && status !== 404;

            if (isSessionError) {
                return sessionError;
            }
        }

        const errorProcessed = super._processTransportError(error);
        return errorProcessed;
    }
}
