import {
    ErrorManager,
    ErrorService,
    createErrorInfoHandleStrategy,
    muteLogger
} from '@goodt-wcore/managers/ErrorManager';
import store from '@/store';
import { NAMESPACES } from '@/store/constants';
import Mutations from '@/store/modules/editor/mutations';
import { SessionExpiredError } from './errors';
import { WidgetDebugMessage } from '@goodt-wcore/errors';
import { DremioPresentableError } from '@goodt-common/services';

const UiNotifiableErrorTypes = [ErrorManager.ErrorInfoType.DremioError];

/**
 *
 * @param {import('@goodt-wcore/managers/ErrorManager').ErrorInfo} errorInfo
 * @this {import('@goodt-wcore/managers/ErrorManager').ErrorInfoHandleStrategy}
 * @property  {import('@goodt-wcore/managers/ErrorManager').ErrorInfoHandleStrategy}
 * @return {null|ErrorInfo}
 * @constructor
 */
const errorInfoHandler = function errorInfoHandler(errorInfo) {
    const { type, error, scope } = errorInfo;

    /**
     * Webpack loading chunks error processing when chunk file missing during artifacts update
     */
    if (error.name === 'ChunkLoadError') {
        const twoMinutesTimestamp = (Date.now() / (1000 * 2 * 60)).toFixed();
        const search = new URLSearchParams(window.location.search);
        search.set('r', twoMinutesTimestamp);
        window.location.search = search.toString();
        return null;
    }

    if (error instanceof SessionExpiredError) {
        store.commit(`${NAMESPACES.EDITOR}/${Mutations.SET_EDITOR_SESSION}`, false);
        return null;
    }

    if (error.isCancel) {
        return null;
    }

    if (error instanceof DremioPresentableError) {
        error.setContext(context);
    }

    if (error instanceof WidgetDebugMessage) {
        if (error.type === 'info') {
            this.logger.info(error);
            return null;
        }
        if (error.type === 'warning') {
            this.logger.warn(error);
            return null;
        }
        this.logger.error(error);
        return null;
    }

    if (type === ErrorManager.ErrorInfoType.CoreWarning) {
        this.logger.warn(error);
        return null;
    }

    // if error was created for presentation purposes but drops in error handler
    if (type === ErrorManager.ErrorInfoType.SilentError) {
        return null;
    }

    if (UiNotifiableErrorTypes.includes(type)) {
        return errorInfo;
    }

    // log Vue.globalHandler silent errors
    if (scope !== ErrorService.Scope.GLOBAL) {
        // непойманная никем ошибка, выброшенная ранее глобально
        if (error.scope === ErrorService.Scope.GLOBAL) {
            return null;
        }
        this.logger.error(error);
    }

    return null;
};

/**
 *
 */
export const init = ({ errorService }) => {
    const errorManager = ErrorManager.instance;

    const strategy = createErrorInfoHandleStrategy({ handler: errorInfoHandler });
    errorManager.setStrategy(strategy);
    errorManager.setLogger(muteLogger(console, ['error']));

    errorService.subscribe((e) => errorManager.handle(e));

    return {
        errorManager
    };
};
