import Vue from 'vue';
import { omit } from 'lodash';
import { init as initSentry, withScope, captureException } from '@sentry/vue';

import {
    ErrorManager,
    createErrorInfoHandleStrategy
} from '@goodt-wcore/managers/ErrorManager';

import Config from '@/utils/Config';
import { getElemTypeMeta } from '@goodt/editor-common/utils';

const IgnoringErrorNames = [
    'ChunkLoadError', 'WidgetDebugMessage'
];

const IgnoringErrorTypes = [
    ErrorManager.ErrorInfoType.CoreWarning,
    ErrorManager.ErrorInfoType.ApiServiceError,
    ErrorManager.ErrorInfoType.DremioError,
    ErrorManager.ErrorInfoType.SilentError
];

/**
 *
 * @param {import('@goodt-wcore/managers/ErrorManager').ErrorInfo} errorInfo
 * @this {import('@goodt-wcore/managers/ErrorManager').ErrorInfoHandleStrategy}
 * @property  {import('@goodt-wcore/managers/ErrorManager').ErrorInfoHandleStrategy}
 * @return {null|ErrorInfo}
 * @constructor
 */
const errorInfoHandler = function errorInfoHandler(errorInfo) {
    const { type, error } = errorInfo;

    if (error.isCancel) {
        return null;
    }

    if (IgnoringErrorNames.includes(error.name)) {
        return null;
    }

    if (IgnoringErrorTypes.includes(type)) {
        return null;
    }

    return errorInfo;
};

const prepareContext = (context) => {
    if (context.type.includes('Elem')) {
        const $computed = Object.entries(context._computedWatchers).reduce((acc, [ key, { value } ]) => ({
            ...acc,
            [key]: value
        }), {});

        const ElemComputed = Object.keys(context.$options.extends.computed);
        const ElemData = Object.keys(context.$options.extends.data());
        const { id, type, initProps, $data, $storeState, $routeCurrent } = context;

        return {
            id,
            type,
            'props[descriptor]': JSON.stringify(initProps),
            data: omit($data, ElemData),
            computed: omit($computed, ElemComputed),
            $storeState: $storeState,
            $routeCurrent: $routeCurrent
        };
    }

    return context;
};

const extractPackageVersion = (type) => {
    const { packageName } = getElemTypeMeta(type);
    const npmName = `@goodt-widgets-package/${packageName}`;
    const npmVersion = Config.report?.dependencies[npmName] ?? 'unknown';

    return `${npmName}@${npmVersion}`;
};

/**
 *
 */
export const init = ({ errorService }) => {
    initSentry({
        Vue,
        dsn: process.env.VUE_APP_SENTRY_DSN,
        defaultIntegrations: false,
        // Set tracesSampleRate to 1.0 to capture 100%
        // of transactions for performance monitoring.
        // We recommend adjusting this value in production
        tracesSampleRate: 1.0
    });

    const strategy = createErrorInfoHandleStrategy({ handler: errorInfoHandler });
    const errorManager = ErrorManager.create();
    errorManager.setStrategy(strategy);
    errorService.subscribe((...args) => errorManager.handle(...args));

    errorManager.subscribe(({ error, type, context, scope: errorScope }) => {
        withScope(function(scope) {
            // scope.setLevel('info');
            if (context != null) {
                const elemContext = prepareContext(context);
                scope.setContext('Elem', elemContext);
                scope.setExtras({
                    package: extractPackageVersion(elemContext.type)
                });
            }
            scope.setExtras({
                type,
                scope: errorScope
            });

            // The exception has the event level set by the scope (info).
            captureException(error);
        });
    });

    return {
        errorManager
    };
};
