import Vue from 'vue';
import cloneDeep from 'lodash/cloneDeep';
import { traverse, findNode, findParentNode, deleteNode } from '@goodt-common/utils';

import Actions from './actions';
import Mutations from './mutations';
import Getters from './getters';
import { handleApiError } from '@/store';
import AppService from '@/services/AppService';
import AppEntity from '@/entities/App';
import Config from '@/utils/Config';
import Const from '@/utils/Const';
import { AppPagesCache } from '@/utils/AppPagesCache';
import { AppHistoryManager } from '@/utils/AppHistoryManager';
import Messages from '@/utils/Messages';
import { getWAuthManagerSession, getWAuthManagerAdapter } from '@/app/auth';

const { instance: PagesCacheInstance } = AppPagesCache;
const { instance: HistoryManagerInstance } = AppHistoryManager;

const cloneElem = (elem) => {
    let clone = cloneDeep(elem);
    clone.id = AppEntity.uid();
    generateElemsIds([clone]);
    return clone;
};
const findElem = (elems, elemId) => {
    return findNode(elems, 'children', (i) => i.id === elemId);
};
const findElemParent = (elems, elemId) => {
    return findParentNode(elems, 'children', (i) => i.id === elemId);
};
const eachElem = (elems, func) => {
    traverse(elems, 'children', func);
};
const deleteElem = (elems, elemId) => {
    return deleteNode(elems, 'children', (i) => i.id === elemId);
};
const generateElemsIds = (elems) => {
    traverse(elems, 'children', (i, p) => {
        i.id = AppEntity.uid();
        if (p) {
            i.pid = p.id;
        }
    });
};
const generateElemParentHash = (pageContent) => {
    let hash = {};
    eachElem(pageContent, (i) => (hash[i.id] = i.pid));
    return hash;
};

const patchContent = (pageContent = [], { find, replace }) => {
    const patchedContent = { operationsCount: 0, content: null };
    const contentString = JSON.stringify(pageContent);
    const regEscape = (str) => str.replace(/[-[\]{}()*+?.,\\^$|#]/g, '\\$&');
    const regExp = new RegExp(regEscape(find), 'g');
    const matches = contentString.match(regExp);

    if (matches == null) {
        return patchedContent;
    }

    patchedContent.operationsCount += matches.length;
    patchedContent.content = JSON.parse(contentString.replace(regExp, replace));

    return patchedContent;
};
/**
 * @typedef {import('packages/editor-common').IProject} IProject
 */
export default {
    namespaced: true,
    state: () => ({
        /** @type {IProject} */
        app: null,
        appPageIds: [],
        appSession: {
            authenticated: false,
            profile: {}
        },
        appList: [],
        appListFirstLoad: true,
        appTemplateList: [],
        appTemplateListFirstLoad: true,
        pageSelected: null,
        elemSelectedId: null,
        bufferElem: null,
        pageElemParentHash: {}
    }),
    mutations: {
        [Mutations.PATCH_PAGE_SELECTED_CONTENT](state, { page, content }) {
            page.content = cloneDeep(content);
            PagesCacheInstance.setPage(page);
            state.pageSelected = PagesCacheInstance.getPage(page.id);
        },
        [Mutations.SET_BUFFER_ELEM](state, val) {
            state.bufferElem = cloneDeep(val);
        },
        [Mutations.SET_APP_SESSION](state, session) {
            state.appSession = session;
        },
        [Mutations.SET_APP_LIST](state, appList) {
            state.appList = appList;
            if (state.appListFirstLoad) {
                state.appListFirstLoad = false;
            }
        },
        [Mutations.SET_APP_TEMPLATE_LIST](state, appTemplateList) {
            state.appTemplateList = appTemplateList;
            if (state.appTemplateListFirstLoad) {
                state.appTemplateListFirstLoad = false;
            }
        },
        [Mutations.CREATE_APP](state, appMeta) {
            if (appMeta.type == Const.APP_TYPE_APP) {
                state.appList.push(appMeta);
            } else if (appMeta.type == Const.APP_TYPE_TEMPLATE) {
                state.appTemplateList.push(appMeta);
            }
        },
        [Mutations.UPDATE_APP](state, appMeta) {
            let apps = [];
            if (appMeta.type == Const.APP_TYPE_APP) {
                apps = state.appList;
            } else if (appMeta.type == Const.APP_TYPE_TEMPLATE) {
                apps = state.appTemplateList;
            }
            let i = apps.findIndex((e) => e.id === appMeta.id);
            if (i >= 0) {
                apps.splice(i, 1, appMeta);
            }
        },
        [Mutations.DELETE_APP](state, appId) {
            let i = state.appList.findIndex((app) => app.id === appId);
            if (i >= 0) {
                state.appList.splice(i, 1);
                return;
            }
            i = state.appTemplateList.findIndex((app) => app.id === appId);
            if (i >= 0) {
                state.appTemplateList.splice(i, 1);
            }
        },
        [Mutations.SET_APP](state, app) {
            state.elemSelectedId = null;
            state.app = app;
            state.appPageIds = state.app.data.pages.map(({ id }) => id);
            this.$app.initDataProviders({ dataProviders: app.data.dataProviders ?? [] });
        },
        [Mutations.RESET_APP](state) {
            state.app = null;
            state.pageSelected = null;
            state.appPageIds = [];
        },
        [Mutations.SET_PAGES](state, pages) {
            state.app.data.pages = pages;
        },
        [Mutations.SET_ROUTES](state, routes) {
            state.app.data.routes = routes;
        },
        [Mutations.SET_DATA_PROVIDERS](state, dataProviders) {
            /** @type {IProject} */
            const app = state.app;
            Vue.set(app.data, 'dataProviders', cloneDeep(dataProviders));
        },
        [Mutations.CREATE_ROUTE_ITEM](state, route) {
            state.app.data.routes.push(route);
        },
        [Mutations.UPDATE_ROUTE_ITEM](state, route) {
            let i = state.app.data.routes.findIndex((elem) => elem.id == route.id);
            if (i >= 0) {
                state.app.data.routes.splice(i, 1, route);
            }
        },
        [Mutations.DELETE_ROUTE_ITEM](state, routeId) {
            let routes = state.app.data.routes;
            let i = routes.findIndex((route) => route.id === routeId);
            if (i >= 0) {
                routes.splice(i, 1);
            }
        },
        [Mutations.SET_PAGE_SELECTED](state, page) {
            state.elemSelectedId = null;

            if (page == null) {
                state.pageElemParentHash = null;
                state.pageSelected = null;
                return;
            }

            state.pageElemParentHash = generateElemParentHash(page.content);
            state.pageSelected = cloneDeep(page);
        },
        [Mutations.DELETE_PAGE](state, pageId) {
            const { pages } = state.app.data;
            const i = pages.findIndex((p) => p.id == pageId);
            if (i >= 0) {
                if (pageId == state.pageSelected.id) {
                    const pageToDelete = pages[i];
                    const pagesSameType = pages.filter((p) => p.isTemplate === pageToDelete.isTemplate);
                    const li = pagesSameType.findIndex((p) => p.id == pageId);
                    const n = pagesSameType.length;
                    if (n > 1) {
                        const pageSelectedId = li + 1 < n ? pagesSameType[li + 1].id : pagesSameType[li - 1].id;
                        state.pageSelected = cloneDeep(PagesCacheInstance.getPage(pageSelectedId));
                    } else {
                        state.pageSelected = null;
                    }
                    state.elemSelectedId = null;
                }
                pages.splice(i, 1);
            }
        },
        [Mutations.CREATE_PAGE](state, { page, insertAfterPageId = null }) {
            const { content, ...rest } = cloneDeep(page);
            const { pages } = state.app.data;
            let i = insertAfterPageId ? pages.findIndex((elem) => elem.id == insertAfterPageId) : -1;
            i = i < 0 ? pages.length : i + 1;
            pages.splice(i, 0, rest);
        },
        [Mutations.UPDATE_PAGE](state, page) {
            let pages = state.app.data.pages;
            let i = pages.findIndex((elem) => elem.id == page.id);
            if (i >= 0) {
                pages.splice(i, 1, page);
            }
        },
        [Mutations.SET_ELEM_SELECTED_ID](state, elemId) {
            state.elemSelectedId = elemId;
        },
        [Mutations.SET_ELEM_NAME](state, { id, name }) {
            const { pageSelected: page } = state;
            const elem = page != null ? findElem(page.content, id) : null;
            if (elem == null) {
                return;
            }
            if (name.length > 0) {
                Vue.set(elem, 'name', name);
            } else {
                Vue.delete(elem, 'name');
            }

            page.unsavedChanges++;
            PagesCacheInstance.setPage(page);
        },
        [Mutations.SET_ELEM_PROPS](state, { id, props, propNames = [], page }) {
            const elem = page != null ? findElem(page.content, id) : null;
            if (elem == null) {
                return;
            }
            if (propNames.length > 0) {
                propNames.forEach((name) => {
                    props.hasOwnProperty(name) ? Vue.set(elem.props, name, props[name]) : Vue.delete(elem.props, name);
                });
            } else {
                Vue.set(elem, 'props', cloneDeep(props));
            }

            PagesCacheInstance.setPage(page);
            state.pageSelected = PagesCacheInstance.getPage(page.id);
        },
        [Mutations.SET_ELEM_CHILDREN](state, { id, children }) {
            const page = state.pageSelected;

            if (page == null) {
                return;
            }

            const elem = findElem(page.content, id);

            if (elem == null) {
                return;
            }

            elem.children = children;
            generateElemsIds([elem]);
            state.pageElemParentHash = generateElemParentHash(page.content);
            PagesCacheInstance.setPage(page);
        },
        [Mutations.CHECK_ELEM_CHILDREN_SLOTS](state, { id, slotNames }) {
            if (slotNames.length === 0) {
                return;
            }
            const { pageSelected: page } = state;
            const elem = page != null ? findElem(page.content, id) : null;
            if (elem == null) {
                return;
            }
            let defaultSlotIdx = slotNames.findIndex((name) => ['', 'default'].includes(name));
            defaultSlotIdx = defaultSlotIdx >= 0 ? defaultSlotIdx : 0;
            elem.children.forEach((child) => {
                let childSlotName = child.props?.slot ?? '';
                if (childSlotName !== '' && !slotNames.includes(childSlotName)) {
                    Vue.set(child.props, 'slot', slotNames[defaultSlotIdx]);
                    PagesCacheInstance.setPage(page);
                }
            });
        },
        [Mutations.SET_ELEM_VISIBILITY](state, { id, visible, page }) {
            if (page == null) {
                return;
            }

            const elem = findElem(page.content, id);
            if (elem) {
                elem.visible = visible;
                eachElem(elem.children, (i) => (i.visible = visible));
                PagesCacheInstance.setPage(page);
                state.pageSelected = PagesCacheInstance.getPage(page.id);
            }
        },
        [Mutations.DELETE_ELEM](state, { elemId, page }) {
            if (page != null) {
                deleteElem(page.content, elemId);
                PagesCacheInstance.setPage(page);
                state.pageElemParentHash = generateElemParentHash(page.content);
                state.pageSelected = PagesCacheInstance.getPage(page.id);
            }
        },
        [Mutations.MOVE_ELEM](state, { id, toId, index, slot, page }) {
            if (page == null) {
                return;
            }

            let oldParentElem = findElemParent(page.content, id);
            let newParentElem = findElem(page.content, toId);

            if (oldParentElem == null && newParentElem == null) {
                return;
            }

            let elem = findElem(page.content, id);
            // проверим, может newParentElem является ребенком elem
            if (findElem(elem.children, toId) != null) {
                return;
            }
            elem.pid = toId; // @NOTE set pid
            elem.props.slot = slot; // @NOTE set slot
            let i = oldParentElem.children.findIndex((child) => child.id == id);
            if (i >= 0) {
                // это нужно из-за splice() тк у нас стек сдвигается на -1 индекс
                if (oldParentElem === newParentElem) {
                    index -= i <= index ? 1 : 0;
                }
                oldParentElem.children.splice(i, 1);
                let n = newParentElem.children.length;
                if (index >= 0 && index < n) {
                    newParentElem.children.splice(index, 0, elem);
                } else {
                    newParentElem.children.push(elem);
                }
            }
            PagesCacheInstance.setPage(page);
            state.pageElemParentHash = generateElemParentHash(page.content);
            state.pageSelected = PagesCacheInstance.getPage(page.id);
        },
        [Mutations.CREATE_ELEM](state, { elem, parentId, index, slot, page }) {
            if (page == null) {
                return;
            }

            let parentElem = findElem(page.content, parentId);
            let n = parentElem.children.length;
            elem.pid = parentId; // @NOTE set pid
            elem.props.slot = slot; // @NOTE set slot
            if (index >= 0 && index < n) {
                parentElem.children.splice(index, 0, cloneDeep(elem));
            } else {
                parentElem.children.push(cloneDeep(elem));
            }
            PagesCacheInstance.setPage(page);
            state.pageElemParentHash = generateElemParentHash(page.content);
            state.pageSelected = PagesCacheInstance.getPage(page.id);
        },
        [Mutations.SET_CONSTS](state, { constants }) {
            Vue.set(state.app.data, 'constants', cloneDeep(constants));
        },
        [Mutations.SET_AUTH_ADAPTER](state, authAdapter) {
            Vue.set(state.app.data, 'authAdapter', cloneDeep(authAdapter));
        },
        [Mutations.REMOVE_NON_GLOBAL_STATE]() {
            this.$app.initStore({ removeLocalState: true });
        },
        [Mutations.SET_SERVICES](state, services) {
            const { data } = state.app;
            const key = 'services';
            if (services.length === 0) {
                Vue.delete(data, key);
                return;
            }
            Vue.set(data, key, services);
        },
        [Mutations.SET_API_CACHE_SETTINGS](state, apiCacheSettings) {
            const { data } = state.app;
            const key = 'cache';
            Vue.set(data, key, {
                ...data[key],
                services: apiCacheSettings
            });
        }
    },
    actions: {
        [Actions.PATCH_PAGE_SELECTED_CONTENT]({ state, commit }, { find, replace }) {
            const { pageSelected: page } = state;
            const mutationName = Mutations.PATCH_PAGE_SELECTED_CONTENT;
            const command = AppHistoryManager.createCommand({
                type: Actions.PATCH_PAGE_SELECTED_CONTENT,
                redo: (resolve, reject) => {
                    if (page == null) {
                        return reject(Messages.AN_ERROR_OCCURRED_WHILE_PATCHING_PAGE_CONTENT);
                    }

                    AppHistoryManager.goBackToPage(page, state.pageSelected.id, commit);
                    const { operationsCount, content } = patchContent(page.content, { find, replace });

                    if (operationsCount === 0) {
                        return reject(Messages.NO_REPLACEMENTS_TO_APPLY);
                    }

                    commit(mutationName, { page, content });
                    page.unsavedChanges += 1;

                    resolve(operationsCount);
                },
                undo: (resolve, reject) => {
                    if (page == null) {
                        return reject(Messages.AN_ERROR_OCCURRED_WHILE_PATCHING_PAGE_CONTENT);
                    }

                    AppHistoryManager.goBackToPage(page, state.pageSelected.id, commit);
                    const { operationsCount, content } = patchContent(page.content, { find: replace, replace: find });

                    if (operationsCount === 0) {
                        return reject(Messages.NO_REPLACEMENTS_TO_APPLY);
                    }

                    commit(mutationName, { page, content });
                    page.unsavedChanges -= 1;

                    resolve(operationsCount);
                }
            });

            HistoryManagerInstance.register(command);

            return command.redo();
        },
        [Actions.SET_PAGES]({ commit, state }, pages) {
            const mutationName = Mutations.SET_PAGES;
            const pagesOld = cloneDeep(state.app.data.pages);
            const pagesNew = cloneDeep(pages);
            const command = AppHistoryManager.createCommand({
                type: Actions.SET_PAGES,
                undo: (resolve) => resolve(commit(mutationName, cloneDeep(pagesOld))),
                redo: (resolve) => resolve(commit(mutationName, cloneDeep(pagesNew)))
            });
            HistoryManagerInstance.register(command);
            return command.redo();
        },
        [Actions.CREATE_ROUTE_ITEM]({ commit, state }, route) {
            const { id: routeId } = route;
            const routeNew = cloneDeep(route);
            const command = AppHistoryManager.createCommand({
                type: Actions.CREATE_ROUTE_ITEM,
                undo: (resolve) => resolve(commit(Mutations.DELETE_ROUTE_ITEM, routeId)),
                redo: (resolve) => resolve(commit(Mutations.CREATE_ROUTE_ITEM, cloneDeep(routeNew)))
            });
            HistoryManagerInstance.register(command);
            return command.redo();
        },
        [Actions.UPDATE_ROUTE_ITEM]({ commit, state }, route) {
            const mutationName = Mutations.UPDATE_ROUTE_ITEM;
            const routeOld = cloneDeep(state.app.data.routes.find(({ id }) => id === route.id));
            const routeNew = cloneDeep(route);
            const command = AppHistoryManager.createCommand({
                type: Actions.UPDATE_ROUTE_ITEM,
                undo: (resolve) => resolve(commit(mutationName, cloneDeep(routeOld))),
                redo: (resolve) => resolve(commit(mutationName, cloneDeep(routeNew)))
            });
            HistoryManagerInstance.register(command);
            return command.redo();
        },
        [Actions.DELETE_ROUTE_ITEM]({ commit, state }, routeId) {
            const routeOld = cloneDeep(state.app.data.routes.find(({ id }) => id === routeId));
            const command = AppHistoryManager.createCommand({
                type: Actions.DELETE_ROUTE_ITEM,
                undo: (resolve) => resolve(commit(Mutations.CREATE_ROUTE_ITEM, cloneDeep(routeOld))),
                redo: (resolve) => resolve(commit(Mutations.DELETE_ROUTE_ITEM, routeId))
            });
            HistoryManagerInstance.register(command);
            return command.redo();
        },
        [Actions.SET_ROUTES]({ commit, state }, routes) {
            const mutationName = Mutations.SET_ROUTES;
            const routesOld = cloneDeep(state.app.data.routes);
            const routesNew = cloneDeep(routes);
            const command = AppHistoryManager.createCommand({
                type: Actions.SET_ROUTES,
                undo: (resolve) => resolve(commit(mutationName, cloneDeep(routesOld))),
                redo: (resolve) => resolve(commit(mutationName, cloneDeep(routesNew)))
            });
            HistoryManagerInstance.register(command);
            return command.redo();
        },
        [Actions.SET_DATA_PROVIDERS]({ commit, getters }, dataProviders) {
            /** @type {IProject} */
            const mutationName = Mutations.SET_DATA_PROVIDERS;
            const dataProvidersOld = cloneDeep(getters[Getters.GET_DATA_PROVIDERS]);
            const dataProvidersNew = cloneDeep(dataProviders);
            const command = AppHistoryManager.createCommand({
                type: Actions.SET_DATA_PROVIDERS,
                undo: (resolve) => {
                    this.$app.initDataProviders({ dataProviders: dataProvidersOld });
                    resolve(commit(mutationName, cloneDeep(dataProvidersOld)));
                },
                redo: (resolve) => {
                    this.$app.initDataProviders({ dataProviders: dataProvidersNew });
                    resolve(commit(mutationName, cloneDeep(dataProvidersNew)));
                }
            });
            HistoryManagerInstance.register(command);
            return command.redo();
        },
        [Actions.LOAD_APP_LIST]({ commit }, folderId) {
            return new Promise((resolve, reject) => {
                let p = new AppService().appsList({ type: Const.APP_TYPE_APP, folder_id: folderId }).then((appList) => {
                    commit(Mutations.SET_APP_LIST, appList);
                    resolve();
                });
                handleApiError(p, reject, commit);
            });
        },
        [Actions.LOAD_APP_TEMPLATE_LIST]({ commit, state }, folderId) {
            return new Promise((resolve, reject) => {
                let p = new AppService()
                    .appsList({ type: Const.APP_TYPE_TEMPLATE, folder_id: folderId })
                    .then((appList) => {
                        commit(Mutations.SET_APP_TEMPLATE_LIST, appList);
                        resolve();
                    });
                handleApiError(p, reject, commit);
            });
        },
        async [Actions.LOAD_APP]({ commit, dispatch }, appId) {
            const { isError, error, result } = await this.$app.initProjectApp(appId);
            if (isError) {
                console.error(error);
                return;
            }
            const { app, session } = result;
            const hasPagesInLegacyFormat = app.data.pages.some(({ content }) => content != null);
            // invoke page format convertion <app-pages> => <app-chunk-pages>
            if (hasPagesInLegacyFormat) {
                await dispatch(Actions.CONVERT_APP, { appId, data: app.data });
            }
            commit(Mutations.SET_APP, app);
            commit(Mutations.SET_APP_SESSION, session);
        },
        [Actions.SAVE_APP]({ dispatch, commit, state }) {
            const {
                data,
                data: { pages: appPages },
                meta: { id: appId }
            } = state.app;
            const { appPageIds } = state;

            const pagesToUpdate = PagesCacheInstance.unsavedPages.filter(({ id: pageId }) =>
                appPages.some(({ id }) => id === pageId && appPageIds.includes(pageId))
            );
            const pagesToCreate = PagesCacheInstance.unsavedPages.filter(
                ({ id: pageId }) => appPages.some(({ id }) => id === pageId && appPageIds.includes(pageId)) === false
            );
            const pagesToDelete = PagesCacheInstance.pages.filter(
                ({ id: pageId }) => appPages.some(({ id }) => id === pageId) === false
            );
            const { http: transportOptions } = Config.data.api;
            const timeout =
                transportOptions.timeout * (pagesToCreate.length + pagesToUpdate.length + pagesToDelete.length);
            const appService = new AppService({ transportOptions: { ...transportOptions, timeout } });

            const promises = [
                ...pagesToCreate.map(({ id: pageId, content }) => appService.appCreatePage(appId, pageId, content)),
                ...pagesToUpdate.map(({ id: pageId, content }) => appService.appUpdatePage(appId, pageId, content)),
                ...pagesToDelete.map(({ id: pageId }) => appService.appDeletePage(appId, pageId))
            ];

            return new Promise(async (resolve, reject) => {
                promises.forEach((promise) => {
                    handleApiError(promise, reject, commit);
                });
                try {
                    await Promise.all(promises);
                    await dispatch(Actions.UPDATE_APP_DATA, { appId, data });
                    PagesCacheInstance.resetUnsavedChanges();
                    resolve();
                } catch (e) {
                    reject(e);
                }
            });
        },
        [Actions.RESET_APP]({ commit }) {
            this.$app.initEventBus();
            this.$app.initStore();
            this.$app.initConstManager();
            HistoryManagerInstance.reset();
            PagesCacheInstance.reset();
            commit(Mutations.RESET_APP);
        },
        [Actions.DELETE_APP]({ commit }, appId) {
            return new Promise((resolve, reject) => {
                let p = new AppService().appDelete(appId).then(() => {
                    commit(Mutations.DELETE_APP, appId);
                    resolve();
                });
                handleApiError(p, reject, commit);
            });
        },
        [Actions.CREATE_APP]({ commit }, { meta, data }) {
            return new Promise((resolve, reject) => {
                let p = new AppService().appCreate(meta, data).then((id) => {
                    meta.id = id;
                    commit(Mutations.CREATE_APP, meta);
                    resolve(id);
                });
                handleApiError(p, reject, commit);
            });
        },
        [Actions.CLONE_APP]({ commit }, meta) {
            return new Promise((resolve, reject) => {
                const promise = new AppService().appClone(meta).then(resolve);
                handleApiError(promise, reject, commit);
            });
        },
        [Actions.IMPORT_APP]({ commit }, { meta, data }) {
            return new Promise((resolve, reject) => {
                let p = new AppService().appImport(meta, data).then((id) => {
                    meta.id = id;
                    commit(Mutations.CREATE_APP, meta);
                    resolve(id);
                });
                handleApiError(p, reject, commit);
            });
        },
        [Actions.CONVERT_APP]({ dispatch, commit }, { appId, data }) {
            return new Promise(async (resolve, reject) => {
                PagesCacheInstance.convertApp(data.pages);
                data.pages = data.pages.map(({ content, ...page }) => page);

                const { pages } = PagesCacheInstance;
                const { http: transportOptions } = Config.data.api;
                const timeout = transportOptions.timeout * pages.length;
                const appService = new AppService({ transportOptions: { ...transportOptions, timeout } });
                const promises = pages.map(({ id, content }) => appService.appUpdatePage(appId, id, content));
                promises.forEach((promise) => {
                    handleApiError(promise, reject, commit);
                });

                try {
                    await Promise.all(promises);
                    await dispatch(Actions.UPDATE_APP_DATA, { appId, data });
                } catch (e) {
                    console.error(`Application conversion failed: ${e}`);
                } finally {
                    resolve();
                }
            });
        },
        [Actions.UPDATE_APP_DATA]({ commit, state }, { appId, data }) {
            return new Promise((resolve, reject) => {
                let p = new AppService().appUpdateData(appId, data).then(() => {
                    // we need to override app pages with the latest ones in 'data.pages'
                    // appPageIds - holds page ids which were originally loadeded when 'Mutations.SET_APP' was called
                    // then appPageIds is used to dif actions with pages ie to determine whether a created/updated
                    // page needs to be sent to the server @see Actions.SAVE_APP handler
                    state.appPageIds = data.pages.map(({ id }) => id);
                    resolve();
                });
                handleApiError(p, reject, commit);
            });
        },
        [Actions.UPDATE_APP_META]({ commit }, meta) {
            return new Promise((resolve, reject) => {
                let p = new AppService().appUpdateMeta(meta).then(() => {
                    commit(Mutations.UPDATE_APP, meta);
                    resolve();
                });
                handleApiError(p, reject, commit);
            });
        },
        [Actions.SET_PAGE_SELECTED]({ commit, state }, { appId, pageId }) {
            commit(Mutations.REMOVE_NON_GLOBAL_STATE);

            const cachedPage = cloneDeep(PagesCacheInstance.getPage(pageId));

            if (cachedPage != null) {
                commit(Mutations.SET_PAGE_SELECTED, cachedPage);
                return Promise.resolve();
            }

            return new Promise(async (resolve, reject) => {
                const promise = new AppService().appPageInfo(appId, pageId);
                handleApiError(promise, reject, commit);
                const content = await promise;
                PagesCacheInstance.setPage({ id: pageId, content });
                const page = cloneDeep(PagesCacheInstance.getPage(pageId));
                // @NOTE compatibility
                // fixPageCompatibility(page);
                commit(Mutations.SET_PAGE_SELECTED, page);
                resolve();
            });
        },
        [Actions.CREATE_PAGE]({ commit, state }, { page, pageContent = null, insertAfterPageId = null }) {
            const { id, isTemplate } = page;
            const pageNew = cloneDeep(page);
            let content = pageContent;

            if (content == null) {
                const pageData = AppEntity.pageData({ id, isTemplate });
                content = pageData.content;
            }
            const command = AppHistoryManager.createCommand({
                type: Actions.CREATE_PAGE,
                redo: (resolve) => {
                    PagesCacheInstance.setPage({ id, content, unsavedChanges: 1 });
                    commit(Mutations.CREATE_PAGE, { page: pageNew, insertAfterPageId });
                    resolve();
                },
                undo: (resolve) => resolve(commit(Mutations.DELETE_PAGE, id))
            });

            HistoryManagerInstance.register(command);

            return command.redo();
        },
        async [Actions.DELETE_PAGE]({ commit, state }, { appId, pageId }) {
            const pages = state.app.data.pages;
            const pageIdx = pages.findIndex(({ id }) => id === pageId);
            const page = cloneDeep(pages[pageIdx]);
            const insertAfterPageId = pageIdx === 0 ? pageIdx : pageIdx - 1;

            if (PagesCacheInstance.getPage(pageId) == null) {
                const NOT_FOUND_STATUS_CODE = 404;

                try {
                    const content = await new AppService().appPageInfo(appId, pageId);
                    PagesCacheInstance.setPage({ id: pageId, content });
                } catch ({ response }) {
                    if (response.status === NOT_FOUND_STATUS_CODE) {
                        commit(Mutations.DELETE_PAGE, pageId);
                        return Promise.resolve();
                    }

                    commit(Mutations.SET_API_ERROR, response, { root: true });
                    return Promise.reject();
                }
            }

            const command = AppHistoryManager.createCommand({
                type: Actions.DELETE_PAGE,
                redo: async (resolve, reject) => {
                    if (PagesCacheInstance.getPage(pageId) == null) {
                        const promise = new AppService().appPageInfo(appId, pageId);
                        handleApiError(promise, reject, commit);
                        const content = await promise;
                        PagesCacheInstance.setPage({ id: pageId, content });
                    }
                    commit(Mutations.DELETE_PAGE, pageId);
                    resolve();
                },
                undo: (resolve) => {
                    commit(Mutations.CREATE_PAGE, { page, insertAfterPageId });
                    const { id, content } = PagesCacheInstance.getPage(page.id);
                    PagesCacheInstance.setPage({ id, content, unsavedChanges: 1 });
                    resolve();
                }
            });

            HistoryManagerInstance.register(command);

            return command.redo();
        },
        [Actions.SET_SERVICES]({ commit }, services) {
            commit(Mutations.SET_SERVICES, services);
            this.$app.initServices({ services });
        },
        [Actions.SET_API_CACHE_SETTINGS]({ commit }, cacheSettings) {
            commit(Mutations.SET_API_CACHE_SETTINGS, cacheSettings);
        },
        [Actions.SET_ELEM_PROPS]({ commit, state }, { id, props, propNames = [] }) {
            const page = state.pageSelected;
            const elem = findElem(page.content, id);
            const propsNew = cloneDeep(props);
            const propsOld = cloneDeep(elem.props);

            const command = AppHistoryManager.createCommand({
                type: Actions.SET_ELEM_PROPS,
                redo: (resolve) => {
                    AppHistoryManager.goBackToPage(page, state.pageSelected.id, commit);
                    page.unsavedChanges += 1;
                    commit(Mutations.SET_ELEM_PROPS, { id, props: propsNew, propNames, page });
                    resolve();
                },
                undo: (resolve) => {
                    AppHistoryManager.goBackToPage(page, state.pageSelected.id, commit);
                    page.unsavedChanges -= 1;
                    commit(Mutations.SET_ELEM_PROPS, { id, props: propsOld, page });
                    resolve();
                }
            });

            HistoryManagerInstance.register(command);

            return command.redo();
        },
        [Actions.SET_ELEM_VISIBILITY]({ commit, state }, { id, visible }) {
            const { pageSelected: page } = state;
            const visibleOld = !visible;

            const command = AppHistoryManager.createCommand({
                type: Actions.SET_ELEM_VISIBILITY,
                redo: (resolve) => {
                    AppHistoryManager.goBackToPage(page, state.pageSelected.id, commit);
                    page.unsavedChanges += 1;
                    commit(Mutations.SET_ELEM_VISIBILITY, { id, visible, page });
                    resolve();
                },
                undo: (resolve) => {
                    AppHistoryManager.goBackToPage(page, state.pageSelected.id, commit);
                    page.unsavedChanges -= 1;
                    commit(Mutations.SET_ELEM_VISIBILITY, { id, visible: visibleOld, page });
                    resolve();
                }
            });

            HistoryManagerInstance.register(command);

            return command.redo();
        },
        [Actions.DELETE_ELEM]({ commit, state }, elemId) {
            const { pageSelected: page } = state;
            const elem = cloneDeep(findElem(page.content, elemId));
            const parentElem = findElemParent(page.content, elemId);
            const { slot = 'default' } = elem.props;
            const { id: parentId } = parentElem;
            const index = parentElem.children.findIndex(({ id }) => id === elemId);

            const command = AppHistoryManager.createCommand({
                type: Actions.DELETE_ELEM,
                redo: (resolve) => {
                    AppHistoryManager.goBackToPage(page, state.pageSelected.id, commit);
                    page.unsavedChanges += 1;
                    commit(Mutations.DELETE_ELEM, { elemId, page });
                    resolve();
                },
                undo: (resolve) => {
                    AppHistoryManager.goBackToPage(page, state.pageSelected.id, commit);
                    page.unsavedChanges -= 1;
                    commit(Mutations.CREATE_ELEM, { elem, parentId, index, slot, page });
                    resolve();
                }
            });

            HistoryManagerInstance.register(command);

            return command.redo();
        },
        [Actions.MOVE_ELEM]({ commit, state }, { id, toId, index, slot }) {
            const { pageSelected: page } = state;
            const elem = cloneDeep(findElem(page.content, id));
            const parentElem = findElemParent(page.content, id);
            const { slot: prevSlot } = elem.props;
            const { id: parentId } = parentElem;
            const prevIndex = parentElem.children.findIndex(({ id: elemId }) => elemId === id);

            const command = AppHistoryManager.createCommand({
                type: Actions.MOVE_ELEM,
                redo: (resolve) => {
                    AppHistoryManager.goBackToPage(page, state.pageSelected.id, commit);
                    page.unsavedChanges += 1;
                    commit(Mutations.MOVE_ELEM, { id, toId, index, slot, page });
                    resolve();
                },
                undo: (resolve) => {
                    AppHistoryManager.goBackToPage(page, state.pageSelected.id, commit);
                    page.unsavedChanges -= 1;
                    commit(Mutations.MOVE_ELEM, { id, toId: parentId, index: prevIndex, slot: prevSlot, page });
                    resolve();
                }
            });

            HistoryManagerInstance.register(command);

            return command.redo();
        },
        [Actions.CREATE_ELEM]({ commit, state }, { elem, parentId, index, slot }) {
            const { pageSelected: page } = state;

            const command = AppHistoryManager.createCommand({
                type: Actions.CREATE_ELEM,
                redo: (resolve) => {
                    AppHistoryManager.goBackToPage(page, state.pageSelected.id, commit);
                    page.unsavedChanges += 1;
                    commit(Mutations.CREATE_ELEM, { elem, parentId, index, slot, page });
                    resolve();
                },
                undo: (resolve) => {
                    AppHistoryManager.goBackToPage(page, state.pageSelected.id, commit);
                    page.unsavedChanges -= 1;
                    commit(Mutations.DELETE_ELEM, { elemId: elem.id, page });
                    resolve();
                }
            });

            HistoryManagerInstance.register(command);

            return command.redo();
        },
        [Actions.CLONE_ELEM]({ commit, state }, { elem }) {
            const { pageSelected: page } = state;
            const elemClone = cloneElem(elem);
            const parent = findElemParent(page.content, elem.id);
            const index = parent.children.findIndex(({ id }) => id == elem.id);
            const { slot = 'default' } = elemClone.props;

            const command = AppHistoryManager.createCommand({
                type: Actions.CLONE_ELEM,
                redo: (resolve) => {
                    AppHistoryManager.goBackToPage(page, state.pageSelected.id, commit);
                    page.unsavedChanges += 1;
                    commit(Mutations.CREATE_ELEM, { elem: elemClone, parentId: parent.id, index, slot, page });
                    resolve();
                },
                undo: (resolve) => {
                    AppHistoryManager.goBackToPage(page, state.pageSelected.id, commit);
                    page.unsavedChanges -= 1;
                    commit(Mutations.DELETE_ELEM, { elemId: elemClone.id, page });
                    resolve();
                }
            });

            HistoryManagerInstance.register(command);

            return command.redo();
        },
        [Actions.SET_CONSTS]({ commit, getters }, { constants }) {
            const constantsOld = cloneDeep(getters[Getters.GET_CONSTANTS]);

            const command = AppHistoryManager.createCommand({
                type: Actions.SET_CONSTS,
                redo: (resolve) => {
                    commit(Mutations.SET_CONSTS, { constants });
                    this.$app.initConstManager({ constants });
                    resolve();
                },
                undo: (resolve) => {
                    commit(Mutations.SET_CONSTS, { constants: constantsOld });
                    this.$app.initConstManager({ constants: constantsOld });
                    resolve();
                }
            });

            HistoryManagerInstance.register(command);

            return command.redo();
        },
        [Actions.SET_AUTH_ADAPTER]({ commit, state }, authAdapter) {
            const authAdapterOld = cloneDeep(state.app.data.authAdapter) ?? null;
            const command = AppHistoryManager.createCommand({
                type: Actions.SET_AUTH_ADAPTER,
                redo: (resolve) => {
                    commit(Mutations.SET_AUTH_ADAPTER, authAdapter);
                    this.$app.initAuthManager({ adapter: authAdapter }).then((session) => {
                        commit(Mutations.SET_APP_SESSION, session);
                        resolve();
                    });
                },
                undo: (resolve) => {
                    commit(Mutations.SET_AUTH_ADAPTER, authAdapterOld);
                    this.$app.initAuthManager({ adapter: authAdapterOld }).then((session) => {
                        commit(Mutations.SET_APP_SESSION, session);
                        resolve();
                    });
                }
            });

            HistoryManagerInstance.register(command);

            return command.redo();
        },
        [Actions.LOGIN_APP_AUTH_ADAPTER]({ commit }, credentials = {}) {
            const adapter = getWAuthManagerAdapter();
            if (!adapter) {
                return;
            }
            adapter
                .login(credentials)
                .then(() => getWAuthManagerSession())
                .then((session) => commit(Mutations.SET_APP_SESSION, session))
                .catch(() => console.error('authorization failed: adapter config is invalid'));
        },
        [Actions.LOGOUT_APP_AUTH_ADAPTER]({ commit }) {
            const adapter = getWAuthManagerAdapter();
            if (!adapter) {
                return;
            }
            adapter
                .logout()
                .then(() => getWAuthManagerSession())
                .then((session) => commit(Mutations.SET_APP_SESSION, session));
        },
        [Actions.EXPORT_APP_PAGE]({ state, commit }, { pageId }) {
            const { id: appId, title: appTitle } = state.app.meta;
            const page = state.app.data.pages.find(({ id }) => id === pageId);
            const { title: pageTitle } = page;

            return new Promise(async (resolve, reject) => {
                let cachedPage = cloneDeep(PagesCacheInstance.getPage(pageId));

                if (cachedPage == null) {
                    const promise = new AppService().appPageInfo(appId, pageId);
                    handleApiError(promise, reject, commit);
                    const content = await promise;
                    PagesCacheInstance.setPage({ id: pageId, content });
                    cachedPage = cloneDeep(PagesCacheInstance.getPage(pageId));
                }
                // @NOTE compatibility
                // fixPageCompatibility(cachedPage);
                const appMeta = AppEntity.appMeta({
                    title: pageTitle,
                    description: appTitle,
                    type: Const.APP_TYPE_TEMPLATE
                });
                const appData = AppEntity.appDataByType(Const.APP_TYPE_TEMPLATE);
                const { id: appPageId } = appData.pages[0];
                const service = new AppService();
                service
                    .appCreate(appMeta, appData)
                    .then((newAppId) => service.appCreatePage(newAppId, appPageId, cachedPage.content))
                    .then(resolve)
                    .catch(reject);
            });
        }
    },
    getters: {
        [Getters.GET_BUFFER_ELEM](state) {
            return () => {
                if (!state.bufferElem) {
                    return null;
                }
                return cloneElem(state.bufferElem);
            };
        },
        [Getters.GET_PAGE](state) {
            return (pageId) => {
                if (!state.app) {
                    return null;
                }
                return state.app.data.pages.find((page) => page.id == pageId);
            };
        },
        [Getters.GET_PAGES](state) {
            if (!state.app) {
                return [];
            }
            return state.app.data.pages;
        },
        [Getters.GET_ROUTES](state) {
            if (!state.app) {
                return null;
            }
            return state.app.data.routes;
        },
        [Getters.GET_CONSTANTS](state) {
            if (!state.app) {
                return null;
            }
            return state.app.data.constants;
        },
        [Getters.GET_DATA_PROVIDERS](state) {
            /** @type {IProject} */
            const app = state.app;
            if (!app) {
                return [];
            }
            return app.data.dataProviders ?? [];
        },
        [Getters.GET_AUTH_ADAPTER](state) {
            if (!state.app) {
                return null;
            }
            return state.app.data.authAdapter;
        },
        [Getters.ELEM_SELECTED](state) {
            if (state.app == null || state.pageSelected == null) {
                return null;
            }
            const { content } = state.pageSelected;
            return content != null ? findElem(content, state.elemSelectedId) : null;
        },
        [Getters.GET_LAYERS](state) {
            if (state.app == null || state.pageSelected == null) {
                return [];
            }

            const content = cloneDeep(state.pageSelected.content);

            if (content == null) {
                return [];
            }

            eachElem(content, (elem) => (elem.props = elem.props.slot != null ? { slot: elem.props.slot } : {}));

            return content;
        }
    }
};
