import Mutations from '@/store/modules/app/mutations';
import cloneDeep from 'lodash/cloneDeep';

/**
 * @typedef {import('./AppPagesCache').Page} Page
 */

/**
 * @typedef {object} Command
 * @property {string} type          type
 * @property {redo} redo            redo
 * @property {undo} undo            undo
 */

/**
 * @callback redo
 * @return {Promise}
 */

/**
 * @callback undo
 * @return {Promise}
 */

let appHistoryManager = null;
const appHistoryManagerEnforcer = Symbol('appHistoryManagerEnforcer');
const HISTORY_MAX_LENGTH = 300;

export class AppHistoryManager {
    /**
     * Constructor
     * @param {symbol} enforcer  singleton enforcer
     */
    constructor(enforcer) {
        if (enforcer !== appHistoryManagerEnforcer) {
            throw new Error(`Instantiation failed: use AppHistoryManager.instance`);
        }
        this._history = [];
        this._head = 0;
    }
    
    /**
     * @return {AppHistoryManager}
     */
    static get instance() {
        if (appHistoryManager == null) {
            appHistoryManager = new AppHistoryManager(appHistoryManagerEnforcer);
        }
        return appHistoryManager;
    }
    
    /**
     * creates command from properties
     * @return {Command}
     */
    static createCommand({ type, redo, undo }) {
        return {
            type,
            redo: () => typeof redo.then === 'function' ? redo : new Promise((resolve, reject) => {
                redo(resolve, reject);
            }),
            undo: () => typeof undo.then === 'function' ? undo : new Promise((resolve, reject) => {
                undo(resolve, reject);
            })
        };
    }
    
    /**
     * sets page containing changeable element if page has been changed
     * @param {Page} page
     * @param {string} currentPageId
     * @param {Function} commit
     */
    static goBackToPage(page, currentPageId, commit) {
        if (page.id !== currentPageId) {
            commit(Mutations.REMOVE_NON_GLOBAL_STATE);
            commit(Mutations.SET_PAGE_SELECTED, cloneDeep(page));
        }
    }
    
    /**
     * @return {Command[]}
     */
    get history() {
        return this._history;
    }
    
    /**
     * sets commands to history
     * @param {Command[]} commands
     */
    set history(commands) {
        this._history = commands;
    }
    
    /**
     * @return {number}
     */
    get head() {
        return this._head;
    }
    
    /**
     * sets history head
     * @param {number} num
     */
    set head(num) {
        this._head = num;
    }
    
    /**
     * @return {number}
     */
    get possibleUndoCount() {
        return this.head;
    }
    
    /**
     * @return {number}
     */
    get possibleRedoCount() {
        return this.history.length - this.head;
    }
    
    /**
     * pushes command to registry
     * @param {Command} command
     */
    register(command) {
        const { history, head } = this;
    
        if (head !== history.length) {
            history.splice(head, history.length);
        }
        
        this.history.push(command);
        
        if (history.length > HISTORY_MAX_LENGTH) {
            this.history.shift();
        }
        
        this.head = history.length;
    }
    
    /**
     * executes undo or redo
     * @param {number} index
     */
    async undoRedoGoto(index) {
        const { head } = this;
        
        if (index >= head) {
            await this.redo(index - head + 1);
            return;
        }
        await this.undo(head - index - 1);
    }
    
    /**
     * undoes some count of commands, contain in history
     * @param {number} [steps = 1]
     */
    async undo(steps = 1) {
        const { head, history } = this;
        for (let i = head - 1; i >= head - steps; i--) {
            await history[i].undo();
        }
        this.head -= steps;
    }
    
    /**
     * redoes some count of commands, contain in history
     * @param {number} [steps = 1]
     */
    async redo(steps = 1) {
        const { head, history } = this;
        for (let i = head; i < head + steps; i++) {
            await history[i].redo();
        }
        this.head += steps;
    }
    
    /**
     * resets all commands, contain in history
     */
    reset() {
        this.history = [];
        this.head = 0;
    }
}
