/**
 * @typedef {object} Page
 * @property {string} id                        id
 * @property {Elem[]} content                   content
 * @property {number} [unsavedChanges=0]        unsaved changes count
 */

/**
 * @typedef {object} Elem
 * @property {string} id                        id
 * @property {string|null} pid                  parent id
 * @property {string} type                      type
 * @property {Record<string, any>} props        props
 * @property {boolean} visible                  visibility
 * @property {Elem[]} children                  children
 */
let appPagesCache = null;
const appPagesCacheEnforcer = Symbol('appPagesCacheEnforcer');

export class AppPagesCache {
    /**
     * Constructor
     * @param {symbol} enforcer  singleton enforcer
     */
    constructor(enforcer) {
        if (enforcer !== appPagesCacheEnforcer) {
            throw new Error(`Instantiation failed: use AppPagesCache.instance`);
        }
        this._pagesCache = {};
    }
    
    /**
     * @return {AppPagesCache}
     */
    static get instance() {
        if (appPagesCache == null) {
            appPagesCache = new AppPagesCache(appPagesCacheEnforcer);
        }
        return appPagesCache;
    }
    
    /**
    * sets page to cache
    * @param {Page}
    */
    setPage({ id, content, unsavedChanges = 0 }) {
        this._pagesCache[id] = {
            id,
            content,
            unsavedChanges,
        };
    }
    
    /**
     * returns page or null if page not found
     * @param {Page.id} pageId
     * @return {Page|null}
     */
    getPage(pageId) {
        return this._pagesCache[pageId] ?? null;
    }
    
    /**
     * sets all pages to cache, that happens when app loads in old format
     * @param {Page[]} pages
     */
    convertApp(pages) {
        pages.forEach((page) => {
            this.setPage(page);
        });
    }
    
    /*
    * clears app pages cache
    */
    reset() {
        this._pagesCache = {};
    }
    
    /**
     * resets counter of unsaved changes for every page
     */
    resetUnsavedChanges() {
        Object.values(this._pagesCache).forEach(({ unsavedChanges, ...page }) => {
            this.setPage({ ...page, unsavedChanges: 0 });
        });
    }
    
    /**
     * returns pages saved in cache
     * @return {Page[]}
     */
    get pages() {
        return Object.values(this._pagesCache);
    }
    
    /**
     * returns pages saved in cache that have unsaved changes
     * @return {Page[]}
     */
    get unsavedPages() {
        return this.pages.filter(({ unsavedChanges }) => unsavedChanges > 0);
    }
}
