import ElemError from '@/components/app-editor/canvas/ElemError.vue';
import { getElemTypeMeta } from '@goodt/editor-common/utils';
import { providePackageWidgetLoader } from '@goodt/package-loader';

import Const from './Const';

const records = Symbol('records');

const PackageWidgetLoaders = new Map();

/**
 * @typedef {object} ElemTypeMeta
 * @property {string} packageName
 * @property {string} type
 * @property {string} path
 */
/**
 * @typedef {object} ElemRegistryRecord
 * @property {string} resolvedType
 * @property {string} type
 * @property {string} name
 * @property {string} icon
 * @property {string} path
 * @property {() => Promise} factory
 */
class ElemRegistry {
    constructor() {
        /** @type {Object.<string, ElemRegistryRecord>} */
        this[records] = {};
    }

    /**
     * Registers default widgets that are mandatory to work in editor
     * (if those are not already registered)
     */
    addDefaults() {
        this.add({ type: Const.ELEM_TYPE_ROOT, name: 'root' });
        this.add({ type: Const.ELEM_TYPE_TEMPLATE_CONTENT, name: 'template content', icon: 'mdi-iframe-outline' });
    }

    /**
     * Registers elem type
     * @param {{ type:string, name:string, icon:string }} options
     */
    add({ type, name, icon = '' }) {
        if (this[records][type]) {
            return;
        }
        const { path } = getElemTypeMeta(type);
        const factory = this._createComponentFactory(type);

        this[records][type] = { resolvedType: type, type, name, icon, path, factory };
    }

    ready() {
        return Promise.allSettled([...PackageWidgetLoaders.values()]);
    }

    /**
     * Returns registry record by elem type or null
     * @param {string} elemType
     * @return {?ElemRegistryRecord}
     */
    get(elemType) {
        return this[records][elemType] ?? null;
    }

    /**
     * @private Creates component factory method for spec. widget type
     * @param {string} type
     * @return {import('vue').AsyncComponent}
     */
    _createComponentFactory(type) {
        if (type === Const.ELEM_TYPE_TEMPLATE_CONTENT) {
            return () => ({ component: import('@/components/app-editor/canvas/ElemTemplateContent.vue') });
        }
        if (type === Const.ELEM_TYPE_ROOT) {
            return () => ({ component: import('@goodt-wcore/core/Elem/ElemRoot.vue') });
        }

        const { packageName, type: packageWidgetType } = getElemTypeMeta(type);

        let widgetLoadersPromise = PackageWidgetLoaders.get(packageName);
        if (widgetLoadersPromise == null) {
            widgetLoadersPromise = providePackageWidgetLoader(packageName).catch((reason) => {
                const throwError = () => { throw reason };
                setTimeout(throwError);
                return throwError;
            });
            PackageWidgetLoaders.set(packageName, widgetLoadersPromise);
        }

        return () => {
            const component = widgetLoadersPromise.then((load) => load(packageWidgetType)).catch(() => {
                setTimeout(() => {
                    throw new Error(`Failed to load widget "${type}"`);
                });
                return ElemError;
            });
            return {
                component,
                error: ElemError
            }
        };
    }
}

const ElemRegistryInstance = new ElemRegistry();

export default ElemRegistryInstance;
