importScripts(
    './workbox-sw.js'
);

const CACHE_NAME = 'ROSTALENT_API_CACHE';
let isDebugMode = (new URLSearchParams(location.search)).get('debug') === '1';

workbox.setConfig({
    debug: isDebugMode
});

// <@ATTENTION> https://developer.chrome.com/docs/workbox/modules/workbox-sw/#avoid-async-imports
const { registerRoute } = workbox.routing;
const { StaleWhileRevalidate } = workbox.strategies;
const { ExpirationPlugin } = workbox.expiration;
// </@ATTENTION>

const CACHE_LIFETIME_DEFAULT = 10;
const CachingUrls = new Set();

const log = (...args) => {
    if (isDebugMode === false) {
        return;
    }
    console.log.apply(console.log, ['%cAPI CACHE SW', 'background:#1b73e8;font-weight:bold;border-radius:999px;padding: 1px 5px;', ...args]);
};

// <@ATTENTION> https://developer.chrome.com/docs/workbox/modules/workbox-routing/#how-to-register-a-navigation-route
// this can be placed only in root of SW body or in `install` SW LC handler
registerRoute(
    ({ url }) => [...CachingUrls].some((cachingUrl) => url.href.startsWith(cachingUrl)),
    new StaleWhileRevalidate({
        cacheName: CACHE_NAME,
        plugins: [
            new ExpirationPlugin({
                maxAgeSeconds: CACHE_LIFETIME_DEFAULT,
            }),
        ],
    })
);
// </@ATTENTION>

self.addEventListener('install', async (e) => {
    await self.skipWaiting();
    log(`installing...`);
});

const addUrl = (registeringUrl, { lifetime } = {}) => {
    CachingUrls.add(registeringUrl);
    if (lifetime != null) {
        CacheLifetimesByUrlMap.set(registeringUrl, {
            lifetime
        })
    }
}

const clearUrls = () => {
    CachingUrls.clear();
}

const removeUrl = (registeringUrl) => {
    CachingUrls.has(registeringUrl) && CachingUrls.delete(registeringUrl);
    CacheLifetimesByUrlMap.has(registeringUrl) && CacheLifetimesByUrlMap.delete(registeringUrl);
}

self.addEventListener('message', (event) => {
    const { data } = event;
    const { action, payload } = data;

    switch (action) {
        case 'api/cache/SET_DEBUG_MODE': {
            const { debug } = payload;
            isDebugMode = debug;
            log(`setting debug mode = ${debug}`);
            self.__WB_DISABLE_DEV_LOGS = isDebugMode === false;
            return;
        }
        case 'api/cache/ADD_CACHE_URL': {
            const { url, lifetime } = payload;
            log(`registering url: "${url}"`);
            addUrl(url, { lifetime });
            return;
        }
        case 'api/cache/REMOVE_CACHE_URL': {
            const { url } = payload;
            log(`unregistering url: "${url}"`);
            removeUrl(url);
            return;
        }
        case 'api/cache/CLEAR_CACHE_URLS': {
            log(`clearing ALL registered caching urls`);
            clearUrls();
            return;
        }
        case 'api/cache/SKIP_WAITING': {
            self.skipWaiting();
            return;
        }
        case 'api/cache/SET_DEFAULT_LIFETIME': {
            const { lifetime } = payload;
            CACHE_LIFETIME_DEFAULT = lifetime;
            return;
        }
        case 'api/cache/CACHE_CLEAR': {
            if (caches.has(CACHE_NAME)) {
                log(`clearing [${CACHE_NAME}] cache`);
                caches.delete(CACHE_NAME)
            }
            return;
        }
        default:
            return;
    }
});
