import Vue from 'vue';

const isDebugMode = () => Vue.config.silent === false;

let SERVICE_WORKER_REGISTRATION;

const Actions = {
    SKIP_WAITING: 'api/cache/SKIP_WAITING',
    ADD_CACHE_URL: 'api/cache/ADD_CACHE_URL',
    REMOVE_CACHE_URL: 'api/cache/REMOVE_CACHE_URL',
    CLEAR_CACHE: 'api/cache/CACHE_CLEAR',
    SET_DEFAULT_LIFETIME: 'api/cache/SET_DEFAULT_LIFETIME',
    SET_CACHE_NAME: 'api/cache/SET_CACHE_NAME',
    CLEAR_CACHE_URLS: 'api/cache/CLEAR_CACHE_URLS',
    SET_DEBUG_MODE: 'api/cache/SET_DEBUG_MODE'
};

function ensureServiceWorkerUpdating(oldServiceWorker) {
    if (oldServiceWorker == null) {
        return;
    }
    // window.console.info('New version of the app is available refreshing?');
    oldServiceWorker.postMessage({ action: Actions.SKIP_WAITING });
    // oldServiceWorker.showNotification('New version of the app is available refreshing?', {
    //     actions: ['yes', 'no'],
    // });
    // notification.show('New version of the app is available. Refresh now?');
    // notification.addEventListener('click', () => {
    //     if (registration.waiting) {
    //         // let waiting Service Worker know it should became active
    //         registration.waiting.postMessage({ action: Actions.SKIP_WAITING });
    //     }
    // });
}

const registerCachingServiceWorker = async () => {
    const swQueryParams = new URLSearchParams({
        debug: isDebugMode() ? 1 : 0
    });
    const registration = await navigator.serviceWorker.register(`./api-cache-sw.js?${swQueryParams}`, { scope: process.env.SW_API_CACHE_SCOPE });

    const activeServiceWorker = registration.active;
    if (activeServiceWorker != null) {
        activeServiceWorker.postMessage({ action: Actions.CLEAR_CACHE });
    }

    // ensure the case when the updatefound event was missed is also handled
    // by re-invoking the prompt when there's a waiting Service Worker
    ensureServiceWorkerUpdating(registration.waiting);


    // detect Service Worker update available and wait for it to become installed
    registration.addEventListener('updatefound', () => {
        registration.installing && registration.installing.addEventListener('statechange', () => {
            // if there's an existing controller (previous Service Worker), show the prompt
            if (navigator.serviceWorker.controller != null) {
                ensureServiceWorkerUpdating(registration.waiting);
            }
        });
    });

    let refreshing = false;

    // detect controller change and refresh the page
    navigator.serviceWorker.addEventListener('controllerchange', () => {
        if (refreshing === false) {
            window.location.reload();
            refreshing = true;
        }
    });
    return registration;
}

export const createServiceWorkerRegistration = async () => {
// check if the browser supports serviceWorker at all
    if ('serviceWorker' in window.navigator) {
        // Ожидаем полной загрузки страницы
        return new Promise((resolve) => {
            const doRegistration = () => registerCachingServiceWorker().then((registration) => {
                resolve(registration);
            });
            // wait for the page to load
            window.document.readyState === 'complete'
                ? doRegistration()
                : window.addEventListener('load', doRegistration);
        })
    }
    return Promise.reject();
}

const getServiceWorkerRegistration = async () => {
    if (SERVICE_WORKER_REGISTRATION == null) {
        SERVICE_WORKER_REGISTRATION = await createServiceWorkerRegistration();
    }
    return SERVICE_WORKER_REGISTRATION;
}

export const useCachingServiceWorker = async () => {
    const registration = await getServiceWorkerRegistration();

    const setDebugMode = (debug) => {
        registration.active.postMessage({ action: Actions.SET_DEBUG_MODE, payload: { debug } });
    };
    const addUrl = (url) => {
        registration.active.postMessage({ action: Actions.ADD_CACHE_URL, payload: { url } });
    };
    const removeUrl = (url) => {
        registration.active.postMessage({ action: Actions.REMOVE_CACHE_URL, payload: { url } });
    };
    const clearCache = () => {
        registration.active.postMessage({ action: Actions.CLEAR_CACHE });
    };
    const clearUrls = () => {
        registration.active.postMessage({ action: Actions.CLEAR_CACHE_URLS });
    };
    const setDefaultLifetime = () => {
        registration.active.postMessage({ action: Actions.SET_DEFAULT_LIFETIME });
    };
    const setCacheName = () => {
        registration.active.postMessage({ action: Actions.SET_CACHE_NAME });
    };
    return {
        setDebugMode,
        addUrl,
        removeUrl,
        clearCache,
        clearUrls
    }
}


