import { get, mergeWith } from 'lodash';
import { Url } from '@goodt-common/utils';

class Config {
    _configPublicPath;

    _config;

    get config() {
        return this._config;
    }

    /**
     * constructor
     */
    constructor({ configPublicPath = '/', fileTemplate = 'sandbox{postfix}.json' } = {}) {
        this._configPublicPath = configPublicPath;
        /** @type {function(postfix: string): string} */
        this._configPath = (paramsInput = {})  => Object.entries(paramsInput).reduce(
            (acc, [name, value]) => acc.replace(`{${name}}`, value),
            fileTemplate
        );
    }

    /**
     * Loads config files
     * @return {Promise<object>}
     */
    async load() {
        // await loading for base and local sandbox configs
        // prettier-ignore
        const [base, local] = await Promise.all([
            this._loadConfig(),
            this._loadConfig({ postfix: '.local' })
        ]);
        this._config = this._mergeConfigs(base, local);
        // await loading top-level json refs from another files
        // prettier-ignore
        await Promise.all(
            Object.entries(this._config)
                .filter(([ , propValue]) => typeof propValue?.$ref === 'string')
                .map(([property,  { $ref }]) => this.ensureProp(property, $ref))
        );

        return this._config;
    }

    /**
     *
     * @param {string} $ref
     * @param {string} property
     * @returns {Promise<void>}
     */
    async ensureProp(property, $ref) {
        const [file, accessor = ''] = $ref.split('#/')
        await this.loadProp(property, file, accessor);
    }

    /**
     * Load prop file and assign value to specified config property
     * @param {string} property
     * @param {string} file
     * @param {string} accessor
     * @returns {Promise<void>}
     */
    async loadProp(property, file, accessor = '') {
        const json = await fetch(Url.join(this._configPublicPath, file)).then((r) => r.json());
        const targetPropValue = accessor.length > 0 ? get(json, accessor) : json;
        this._config[property] = targetPropValue;
    }

    /**
     * @private Load concreate config file
     * @param {string} postfix  file name postfix
     * @return {Promise<object>}
     */
    _loadConfig({ postfix = '' } = {}) {
        const url = this._configPath({ postfix });
        return fetch(Url.join(this._configPublicPath, url))
            .then((r) => r.json())
            .catch(() => ({}));
    }

    /**
     * @private Performs deep merge of base, local configs
     * @param {object} base     base config
     * @param {object} local    local config to extend the base one
     * @return {object}     merged config
     */
    // eslint-disable-next-line class-methods-use-this
    _mergeConfigs(base, local) {
        return mergeWith(base, local, (objVal, srcVal) => (Array.isArray(srcVal) ? srcVal : undefined));
    }
}

export { Config };
