/**
 * HTTP Transport Client implementation
 * with Axios HTTP Client underhood
 * and extra service-specific behaviour
 *
 * @implements {import('./types').ITransportConstructor}
 * @param {ITransportOptions} [options={}]  transport config
 */
import {
    HttpClient,
    HttpClientError,
    HttpClientCancelTokenSource,
    ITransport,
    ITransportExtraOptions,
    ITransportOptions,
    ITransportRequest,
    ITransportResponse
} from './types';

import { HttpTransportError, HttpTransportRequestCancel } from './errors';

export class HttpBase implements ITransport {
    private _requests: RequestRecord[];

    private _options: Record<string, any>;

    /**
     * Checks if specified error is transport-related
     *
     * @param {Error|HttpClientError} error
     * @return {boolean}
     */
    static isTransportError(error: Error | HttpClientError): boolean;

    /**
     * Checks if specified error is cancel request-related
     * @param {Error| HttpClientError} error
     * @return {boolean}
     */
    static isCancel(error: Error | HttpClientError): boolean;

    /**
     *
     * @param {Error} error
     * @return {HttpTransportError | HttpTransportError}
     */
    static processTransportError(error: HttpClientError): Error | HttpTransportError | HttpTransportRequestCancel;

    /** @type {HttpClient} */
    axios: HttpClient;

    baseURL: string;

    /**
     * Constructor
     *
     * @param {ITransportOptions} [transportOptions={}] Transport config
     * @param {ITransportExtraOptions} [extraOptions={}] Transport options
     */
    constructor(transportOptions?: ITransportOptions, extraOptions?: ITransportExtraOptions);

    /**
     * Creates a new request
     *
     * @param {RequestConfig} config per-request config
     * @return {RequestPromise}
     */
    request(config: ITransportRequest): Promise<ITransportResponse>;

    /**
     * Disposes http transport related resources
     */
    dispose(): void;

    /**
     * Cancel active RequestPromise
     *
     * @param {number} id   request id
     */
    cancelRequest(id: number): void;

    /**
     * Cancel all active RequestPromises
     */
    cancelAllRequests(): void;

    /**
     * Returns the base url
     *
     * @return {string}
     */
    getBaseUrl(): string;

    /**
     * Registers a new cancel source
     *
     * @private
     * @param {number} id   id
     * @param {CancelTokenSource} cancelTokenSource cancel Token Source
     */
    private _registerRequest(id: number, cancelTokenSource: HttpClientCancelTokenSource);

    /**
     * @param {number} id   id
     * @private
     */
    private _unregisterRequest(id: number);

    /**
     * @protected
     * @param {Error} error
     * @return {HttpTransportError | HttpTransportRequestCancel}
     */
    protected _processTransportError<T extends Error>(error: T): T | HttpTransportError | HttpTransportRequestCancel;
}

export type RequestRecord = {
    id: number;
    source: CancelTokenSource;
};

export { HttpBase as default };
