import axios from 'axios';
import { HttpTransportError, HttpTransportRequestCancel } from './errors';

// @private
let REQUEST_ID = 0;
/**
 * @typedef {Promise} RequestPromise
 * @property {number} id
 */
/**
 * @typedef {object} RequestRecord
 * @property {number} id
 * @property {import('axios').CancelTokenSource} source
 */

/**
 * @typedef {object} RequestConfig
 * @property {string} url               url
 * @property {string} [method='get']    request method
 * @property {Record<string, any>} [params={}]       params
 * @property {Record<string, any>} [options={}]      request options (axios)
 * @property {Function} [responseHandler=null]     response handler
 */
/**
 * HTTP Transport Client implementation
 * with Axios HTTP Client underhood
 * and extra service-specific behaviour
 *
 * @implements {import('./types').ITransportConstructor}
 */
export class HttpBase {
    /**
     *
     * @type {RequestRecord[]}
     * @private
     */
    _requests = [];

    /**
     *
     * @type {Record<string, any>}
     * @private
     */
    _options = {};

    /**
     * Checks if specified error is transport-related
     *
     * @param {Error} error
     * @return {boolean}
     */
    static isTransportError(error) {
        return axios.isAxiosError(error);
    }

    /**
     * Checks if specified error is cancel request-related
     * @param {Error} error
     * @return {boolean}
     */
    static isCancel(error) {
        return axios.isCancel(error);
    }

    /**
     * Обрабатывает ошибку от транспорта и трансформирует в инфраструктурную ошибку
     * Особенность: знает формат ошибки сервера, какие статус коды являются ошибочными
     *
     * @param {Error} error
     * @return {HttpTransportError|HttpTransportRequestCancel}
     */
    static processTransportError(error) {
        if (error instanceof HttpTransportError) {
            return error;
        }

        // mute cancel error
        if (this.isCancel(error)) {
            return new HttpTransportRequestCancel();
        }

        const { message } = error;
        // If non-transport error
        if (this.isTransportError(error) === false) {
            return new HttpTransportError(message, {
                code: HttpTransportError.Code.INTERNAL,
                reason: error
            });
        }

        // The request was made and the server responded with a status code
        // that falls out of the range of 2xx
        const {
            response,
            request,
            config: { url, data: requestData, method }
        } = error;

        const reason = { url, data: requestData, method };

        if (response != null) {
            const { data, status } = response;
            const errorMessage = typeof data === 'string' ? data : message;

            return new HttpTransportError(errorMessage, {
                data,
                code: status,
                reason
            });
        }

        // The request was made but no response was received
        // `error.request` is an instance of XMLHttpRequest in the browser
        if (request != null) {
            return new HttpTransportError(message, {
                code: HttpTransportError.Code.INTERNAL,
                reason
            });
        }

        // Something happened in setting up the request that triggered an Error
        return new HttpTransportError(message, {
            code: HttpTransportError.Code.INTERNAL,
            reason: error
        });
    }

    /**
     * Constructor
     *
     * @param {import('axios').AxiosRequestConfig} [transportOptions={}]
     * @param {Record<string, any>} [extraOptions={}]
     */
    constructor(transportOptions = {}, extraOptions = {}) {
        this._options = { ...this._options, ...extraOptions };
        this.axios = axios.create(transportOptions);
    }

    /**
     *
     * @param url
     * @param method
     * @param params
     * @param options
     * @param responseHandler
     * @return {{request: import('axios').AxiosRequestConfig, cancelTokenController: CancelTokenSource}}
     * @private
     */
    _buildRequestConfig({ url, method = 'get', params = {}, options = {} }) {
        const cancelTokenController = axios.CancelToken.source();
        const request = {
            method,
            url,
            cancelToken: cancelTokenController.token,
            [method.toLocaleLowerCase() === 'get' ? 'params' : 'data']: params,
            ...options
        };

        // axios 0.x.x AxiosRequestConfig.paramsSerializer back compatibility
        if (typeof request.paramsSerializer !== 'object') {
            request.paramsSerializer =
                typeof request.paramsSerializer === 'function'
                    ? { serialize: request.paramsSerializer }
                    : { indexes: null };
        }

        return { request, cancelTokenController };
    }

    /**
     * Creates a new request
     *
     * @param {RequestConfig} config per-request config
     * @return {RequestPromise}
     */
    request({ url, method = 'get', params = {}, options = {}, responseHandler = null }) {
        const requestId = REQUEST_ID++;

        const { request, cancelTokenController } = this._buildRequestConfig({ url, method, params, options });

        /** @type {RequestPromise} */
        // eslint-disable-next-line no-async-promise-executor
        const promise = new Promise(async (resolve, reject) => {
            this._registerRequest(requestId, cancelTokenController);
            try {
                const response = await this.axios.request(request);
                const processedResponse = this._processSuccessResponse(response, { responseHandler });
                resolve(processedResponse);
            } catch (error) {
                const processedError = this._processTransportError(error);
                reject(processedError);
            } finally {
                this._unregisterRequest(requestId);
            }
        });
        // @ts-ignore
        promise.id = requestId;
        return promise;
    }

    /**
     * Disposes http transport related resources
     */
    dispose() {
        this.cancelAllRequests();
    }

    /**
     * Cancel active RequestPromise
     *
     * @param {number} id   request id
     */
    cancelRequest(id) {
        const i = this._requests.findIndex((item) => item.id === id);
        if (i < 0) {
            return;
        }
        const item = this._requests[i];
        item.source.cancel('canceled');
        this._requests.splice(i, 1);
    }

    /**
     * Cancel all active RequestPromises
     */
    cancelAllRequests() {
        this._requests.forEach((item) => item.source.cancel('canceled'));
        this._requests = [];
    }

    /**
     *
     * @return {string}
     */
    get baseURL() {
        // eslint-disable-next-line goodt-rules/no-long-prop-chains
        return this.axios.defaults.baseURL;
    }

    /**
     *
     * @param baseURL
     */
    set baseURL(baseURL) {
        // eslint-disable-next-line goodt-rules/no-long-prop-chains
        this.axios.defaults.baseURL = baseURL;
    }

    /**
     * Returns the base url
     *
     * @return {string}
     */
    getBaseUrl() {
        return this.baseURL;
    }

    /**
     *
     * @param response
     * @param responseHandler
     * @return {*}
     * @private
     */
    _processSuccessResponse(response, { responseHandler }) {
        if (typeof response.data === 'object' && response.data.error != null) {
            const { error } = response.data;
            const message = typeof error === 'string' ? error : error.message ?? 'Transport response error';

            throw new HttpTransportError(message, {
                code: 1,
                data: error,
                reason: response.request
            });
        }

        return responseHandler ? responseHandler(response) : response;
    }

    /**
     * Registers a new cancel source
     *
     * @private
     * @param {number} id   id
     * @param {import('axios').CancelTokenSource} cancelTokenSource cancel Token Source
     */
    _registerRequest(id, cancelTokenSource) {
        this._requests.push({ id, source: cancelTokenSource });
    }

    /**
     * @private
     */
    _unregisterRequest(id) {
        this._requests = this._requests.filter((item) => item.id !== id);
    }

    /**
     *
     * @param {Error} error
     * @return {Error|HttpTransportError|HttpTransportRequestCancel}
     * @protected
     */
    _processTransportError(error) {
        return this.constructor.processTransportError(error);
    }
}

export { HttpBase as default };
