import EventBusEvent from './EventBusEvent';

export default EventBus;
export type EventHandler = (event: any, data: any) => any;
export type ListenerInfo = {
    event: any;
    handler: EventHandler;
    once: boolean;
};
/**
 * @callback EventHandler
 * @param {EventBusEvent} event
 * @param {any} data
 */

/**
 * @typedef {object} ListenerInfo
 * @property {EventBusEvent} event
 * @property {EventHandler} handler
 * @property {boolean} once
 */
/**
 * EventBus class
 */
export class EventBus {
    /**
     * @private
     * @type {Record<string, ListenerInfo[]>}
     */
    private _listeners;
    /**
     * @private
     * @type {Set.<EventHandler>}
     */
    private _hooks;

    /**
     * Registers trigger hook
     * @param {EventHandler} handler
     * @param {boolean} [once=false]
     * @returns {Function}
     */
    hook(handler, once = false): Function;

    /**
     * Registers an event handler
     *
     * @param {EventBusEvent} event     event
     * @param {EventHandler} handler    handler
     * @param {boolean} [once=false]    if set true event handler will be auto-unregistered when invoked
     * @return {Function}               dispose function to unregister the handler i.e. dispose() === unlisten(event, handler)
     */
    listen(event: EventBusEvent, handler: EventHandler, once?: boolean): Function;

    /**
     * Unregisters an event handler
     *
     * @param {EventBusEvent} event     event
     * @param {EventHandler} handler    handler
     * @return {boolean}                true if ayn handler unregistered
     */
    unlisten(event: EventBusEvent, handler: EventHandler): boolean;

    /**
     * Returns true if event handler is registered
     *
     * @param {EventBusEvent} theEvent     event
     * @param {EventHandler} theHandler    handler
     * @return {boolean}
     */
    has(theEvent: any, theHandler: EventHandler): boolean;

    /**
     * Returns count of registered event listener handlers
     *
     * @param {string|EventBusEvent} typeOrEvent     event
     * @return {number}
     */
    count(typeOrEvent): number;

    /**
     * Triggers event
     *
     * @param {EventBusEvent} event     event
     * @param {*} [data]                data
     */
    trigger(event: EventBusEvent, data?: any): void;

    /**
     * Remove all listeners
     */
    resetListeners(): void;

    /**
     * Removes all hooks
     * @param {boolean} [resetHooks=false]
     */
    resetHooks(resetHooks = false): void;

    /**
     * Alias for 'resetListeners'
     */
    reset(): void;

    /**
     * Call event handler
     *
     * @param {EventHandler} handler        handler
     * @param {EventBusEvent} event     event
     * @param {any} data                  data
     */
    callEventHandler(handler: EventHandler, event: any, data: any): void;
}
