import { EventHandler } from './EventBus';
import { EventBus } from './EventBus';
import { StoreElemState } from '@goodt-wcore/managers';

export default EventBusWrapper;

/**
 * EventBusWrapper class
 * used by components
 */
declare class EventBusWrapper {
    /**
     * Constructor
     * @param {EventBus} eventBus
     * @param context
     * @param [listenStateChange]
     * @param [listenRouteNavigate]
     * @param [triggerStateChange]
     * @param [triggerRouteNavigate]
     */
    constructor(
        eventBus: EventBus,
        {
            context,
            listenStateChange,
            listenRouteNavigate,
            triggerStateChange,
            triggerRouteNavigate
        }?: {
            context: { id: string; type: string };
            listenStateChange: (handler: (state: StoreElemState) => void) => () => void;
            listenRouteNavigate: (handler: (...args: IArguments) => void) => () => void;
            triggerStateChange: (state: StoreElemState) => void;
            triggerRouteNavigate: ({ url, params }: { url: string; params: Record<string, any> }) => void;
        }
    );

    /**
     * @type {EventBus}
     */
    private _eb: EventBus;
    /**
     * @type {Function[]}
     */
    private _disposals: Function[];

    private _eventTypeResolvers: Map<string, () => string>;

    /**
     * Listen for @see EventBusEvent.EVENT_NAVIGATE event
     *
     * @param {EventHandler} handler    handler
     * @param {boolean} [once=false]    once
     * @return {Function}               handler ref
     */
    listenNavigate(handler: EventHandler, once?: boolean): Function;

    /**
     * Unlisten @see EventBusEvent.EVENT_NAVIGATE event
     *
     * @param {EventHandler} handler    handler
     */
    unlistenNavigate(handler: EventHandler): void;

    /**
     * Trigger @see EventBusEvent.EVENT_NAVIGATE event (for relative urls; change window location for other url schemas)
     *
     * @param {object} info     nav info object { url:{String}, params:{Object} }
     * @param info.url
     * @param info.params
     * @return {boolean}        whether the event was triggered or not
     */
    triggerNavigate({ url, params }: { url: any; params: any }): boolean;

    /**
     * Listen for @see EventBusEvent.EVENT_STATE_CHANGE event
     *
     * @param {EventHandler} handler    handler
     * @param {boolean} [once=false]    once
     * @return {EventHandler}           decorated handler
     */
    listenStateChange(handler: EventHandler, once?: boolean): any;

    /**
     * Unlisten @see EventBusEvent.EVENT_STATE_CHANGE event
     *
     * @param {EventHandler} decoratedHandler    decorated handler
     */
    unlistenStateChange(decoratedHandler: EventHandler): void;

    /**
     * Trigger @see EventBusEvent.EVENT_STATE_CHANGE event
     *
     * @param {object} stateChange     state change object { '<key>': '<value>' }
     */
    triggerStateChange(stateChange: object): void;

    /**
     * Listen
     *
     * @param {string | EventBusEvent} eventType      event type
     * @param {EventHandler} handler                handler
     * @param {boolean} [once=false]                once? @default false
     * @return {Function}                           dispose handler
     */
    listen(eventType: string | EventBusEvent, handler: EventHandler, once?: boolean): Function;

    /**
     * Unlisten
     *
     * @param {string | EventBusEvent} eventType      event type
     * @param {EventHandler} handler                handler
     */
    unlisten(eventType: string | EventBusEvent, handler: EventHandler): void;

    /**
     * Trigger
     *
     * @param {string | EventBusEvent} eventType    event type
     * @param {object} [data] custom data
     */
    trigger(eventType: string | EventBusEvent, data?: object): void;

    /**
     * Has
     *
     * @param {string | EventBusEvent} eventType      event type
     * @param {EventHandler} handler                handler
     * @return {boolean}
     */
    has(eventType: string | EventBusEvent, handler: EventHandler): boolean;

    /**
     * Count
     *
     * @param {string | EventBusEvent} eventType      event type
     * @return {number}
     */
    count(eventType): number;

    /**
     * Dispose all handlers that refer to eventBus
     */
    dispose(): void;

    /**
     * Dispose all handlers that refer to eventBus
     * @deprecated
     */
    destroy(): void;

    /**
     *
     * @param {string} eventType
     * @return {string}
     * @private
     */
    resolveEventType(eventType: string): string;

    /**
     *
     * @param {string} eventType
     * @param {function(): string} resolver
     */
    setEventTypeResolver(eventType: string, resolver: () => string);

    /**
     *
     * @param {string} eventType
     * @return {function(): string}
     */
    getEventTypeResolver(eventType: string): () => string;
}

import EventBusEvent from './EventBusEvent';
