import { Socket } from "socket.io";
import Joi from 'joi';
import { ClientEvents, ServerEvents, Response } from '../../events.ts';
import { ServerEventEnum, ErrorsEnum, mapErrorDetails } from '../../utils.ts';
import { app, appId, user, token } from './index.ts';

const tokenSchema = Joi.string();
const idSchema = Joi.string();
const userSchema = Joi.object({
    sessionId: idSchema.required(),
    id: idSchema.forbidden(),
    canUserEdit: Joi.boolean().required(),
    name: Joi.string().forbidden(),
    hasModeratorRole: Joi.boolean().forbidden()
});

let userName: string = '';
let userId: string = '';
const userMap = new Map<string, app>();
const defineUserIsModerator = (users: user[], user: user) => {
    if (users.length === 0) {
        return user.canUserEdit;
    }

    const foundUser = users.find(({ sessionId }) => sessionId === user.sessionId);
    if (foundUser != null) {
        return foundUser.hasModeratorRole;
    }

    if (users.some(({ hasModeratorRole }) => hasModeratorRole) === false) {
        return user.canUserEdit;
    }

    return false;
};
const setAppNextModerators = (appId: appId, userSessionId: user['sessionId']) => {
    const app = userMap.get(appId)!;
    const users = [...app.values()];
    if (users.some(({ sessionId }) => sessionId === userSessionId)) {
        return;
    }

    const foundUser = users.find(({ canUserEdit }) => canUserEdit);
    if (foundUser != null) {
        // @ts-ignore
        userMap.set(appId, new Map([...app.entries()].map(([id, user]) => {
            if (foundUser.sessionId === user.sessionId) {
                return [id, { ...user, hasModeratorRole: true }];
            }
            return [id, user];
        })));
    }
};

const removeApp = (appId: appId) => {
    userMap.delete(appId);
};

// @ts-ignore
export function createAppHandlers(socket: Socket<ClientEvents, ServerEvents>) {
    const emitAppJoined = (appId: appId, user: user) => {
        const app: app = userMap.get(appId)!;
        socket.to(appId).emit(ServerEventEnum.APP_JOINED, Object.fromEntries(app.entries()), user);
    };

    const emitAppLeft = (appId: appId, user: user) => {
        const app: app = userMap.get(appId)!;
        socket.to(appId).emit(ServerEventEnum.APP_LEFT, Object.fromEntries(app.entries()), user);
    };

    return {
        handshake(token: token, callback: (res: Response<boolean>) => void) {
            const { error: tokenError } = tokenSchema.validate(token);

            if (tokenError != null) {
                callback({
                    error: ErrorsEnum.INVALID_PAYLOAD,
                    errorDetails: mapErrorDetails(tokenError.details),
                });
                return;
            }

            const profile = JSON.parse(Buffer.from(token.split('.')[1], 'base64').toString('binary'));
            userName = profile.preferred_username;
            userId = profile.sub;
            callback({ data: true });
        },
        joinApp(
            appId: appId,
            user: Omit<user, 'id' | 'name' | 'hasModeratorRole'>,
            callback: (res: Response<Record<string, user>>) => void
        ) {
            const { error: idError } = idSchema.validate(appId);

            if (idError != null) {
                callback({
                    error: ErrorsEnum.INVALID_PAYLOAD,
                    errorDetails: mapErrorDetails(idError.details),
                });
                return;
            }

            const { error: userError, value: userData } = userSchema.validate(user, {
                abortEarly: false,
                stripUnknown: true,
            });

            if (userError != null) {
                callback({
                    error: ErrorsEnum.INVALID_PAYLOAD,
                    errorDetails: mapErrorDetails(userError.details),
                });
                return;
            }

            socket.join(appId);

            const isAppExist = userMap.has(appId);

            if (isAppExist === false) {
                userMap.set(appId, new Map());
            }

            const app = userMap.get(appId)!;
            const joinedUser = { ...userData, id: userId, name: userName };

            joinedUser.hasModeratorRole = defineUserIsModerator([...app.values()], joinedUser);
            app.set(socket.id, joinedUser);

            callback({ data: Object.fromEntries(app.entries()) });
            emitAppJoined(appId, joinedUser);
        },
        leaveApp(appId: appId, callback: (res: Response<boolean>) => void) {
            const { error: idError } = idSchema.validate(appId);

            if (idError != null) {
                callback({
                    error: ErrorsEnum.INVALID_PAYLOAD,
                    errorDetails: mapErrorDetails(idError.details),
                });
                return;
            }

            socket.leave(appId);

            if (userMap.has(appId) === false) {
                callback({ data: true });
                return;
            }

            const app = userMap.get(appId)!;
            const user = app.get(socket.id)!
            app.delete(socket.id);

            if (app.size === 0) {
                removeApp(appId);
                callback({ data: true });
                return;
            }

            if (user.hasModeratorRole) {
                setAppNextModerators(appId, user.sessionId);
            }

            callback({ data: true });
            emitAppLeft(appId, user);
        },
        disconnect() {
            userMap.forEach((app, appId) => {
                if (app.has(socket.id) === false) {
                    return;
                }

                const user = app.get(socket.id)!;
                app.delete(socket.id);

                if (app.size === 0) {
                    removeApp(appId);
                    return;
                }

                if (user.hasModeratorRole) {
                    setAppNextModerators(appId, user.sessionId);
                }

                emitAppLeft(appId, user);
            });
        }
    }
}
