import Vue from 'vue';

// wcore
import { Const as WCoreConst } from '@goodt-wcore/core';
import { Const as WCoreConst1 } from 'goodt-wcore';
import { Elem } from '@goodt-wcore/elem';
import { Panel } from '@goodt-wcore/panel';
import { AuthManager, ConstManager, RouteManager, StoreManager } from '@goodt-wcore/managers';
import { ServiceProvider, ServiceSetupConfigs } from '@goodt-common/services';
import { Adapters } from '@goodt-common/auth';
import { ProjectDataProviderRepository } from '@goodt-common/dremio';
import { ApiCacheManager, ApiCacheServiceProvider } from '@goodt/editor-common';
import ElemRegistry from './ElemRegistry';

// app related
import RouterBuilder from '@/core/RouterBuilder';
import { Action } from '@/store';
import { fetchText } from '@/utils/fetch';
import { Config } from '@/utils';
import { useRouteMonitor } from '@/log/RouteMonitor';
import { init as initErrorHandle } from '@/error-manage';

import { isDeveloperMode } from '@/utils/DevMode';

console.log({ WCoreConst1, Elem, Panel });

export default class Application {
    config = {};

    /**
     * @type {import('vue/types/vue').Vue}
     */
    vueApp;

    /**
     * Vuex store instance
     * @type {import('vuex').Store}
     */
    vuex;

    /**
     * Widget's global state store
     * @type {import('@goodt-wcore/managers').Store}
     */
    store;

    /**
     * App router instance
     * @type {import('vue-router')}
     */
    router;

    /**
     * @type {import('vue-i18n')}
     */
    i18n;
    /**
     * App route monitor instance
     * @type {import('@/log/RouteMonitor').RouteMonitor?}
     */
    routeManager;

    authManager;

    constManager;

    /**
     * Service to cache api requests
     * @type {ApiCacheManager}
     */
    apiServiceCacheManager;

    routeMonitor;

    /**
     *
     */
    workerApi;

    constructor({
                    vuex,
                    store = StoreManager.store,
                    i18n,
                    authManager = AuthManager.instance,
                    routeManager = RouteManager.instance,
                    constManager = ConstManager.instance,
                    workerApi
                }) {
        /*
         * Error Management init should happen first
         * due to possible error manager specific strategies
         * which behaviour (like error tracking, or some UI-less actions)
         * should occurs regardless of the success App instantiation
         */
        this.vuex = vuex;
        this.store = store;
        this.i18n = i18n;
        this.authManager = authManager;
        this.routeManager = routeManager;
        this.constManager = constManager;
        this.apiServiceCacheManager = new ApiCacheManager({
            constManager,
            cacheServiceProvider: new ApiCacheServiceProvider()
        });
        this.workerApi = workerApi;
        this._initErrorHandle();
        this._patchVuePlugins({ vuex });
        this._setupServices();
    }

    /**
     * Init
     * @param {import('vue').Component} App
     * @param {import('vue').ComponentOptions} AppComponentOptions
     * @return {Promise<Application>}
     */
    async init(App, AppComponentOptions) {
        this.config = await Config.init();
        const { api, locale = WCoreConst.LOCALE, auth } = this.config;

        // eslint-disable-next-line no-param-reassign
        WCoreConst.WFM_DATAPROVIDER_API_URL = api.wfm.baseURL;
        // eslint-disable-next-line no-param-reassign
        this.i18n.locale = locale;

        if (Config.isStandaloneMode === false) {
            if (await this._initEditorAuth({ auth }) === false) {
                return this;
            }
        }

        this._injectThemeCss();
        await this.vuex.dispatch(Action.LOAD_APP);
        const {
            cache: { services: servicesCacheConfig } = {},
            dataProviders
        } = this.vuex.state.app;

        // @WARNING
        // should be called FIRST!
        // due to this._initWidgetsAuthManager() (at least,
        // and possible some other in future) depends on
        // service provider should be already initialized)
        this.initServices({ force: true });

        await Promise.all([
            this._initStyle(),
            this._initDataProviders({ dataProviders }),
            this._initWidgetsAuthManager(),
            this._initRouteMonitor(),
            this._initRouter(),
            this._attachEvents(),
            this._initWidgetsConstManager(),
            this._setupElemRegistry()
        ]);
        if (servicesCacheConfig != null) {
            await this._initServiceCacheManager({ config: servicesCacheConfig });
        }

        this.vueApp = this._createVueApp(App, AppComponentOptions);

        return this;
    }

    /**
     * Init style (for compat mode in prod with back-end 1.0)
     */
    _initStyle() {
        const { baseURL = '' } = Config.data.assets ?? {};
        if (baseURL === '') {
            return;
        }
        const el = document.createElement('link');
        el.setAttribute('rel', 'stylesheet');
        el.setAttribute('href', [baseURL, 'css/style.css'].join('/'));
        document.head.append(el);
    }

    /**
     * Init route monitor
     */
    _initRouteMonitor() {
        const { log: { routeMonitor = {} } = {} } = this.config;
        const { url, enabled = false } = routeMonitor;

        if (enabled === false) {
            return;
        }

        this.routeMonitor = useRouteMonitor({
            url,
            getToken: () => {
                const { adapter } = AuthManager.instance;
                return adapter ? adapter.token : '';
            }
        });
    }

    /**
     * Init router
     */
    _initRouter() {
        const { vuex, store, routeManager, routeMonitor, authManager, apiServiceCacheManager } = this;
        const routerBuilder = new RouterBuilder({
            vuex,
            store,
            routeManager,
            routeMonitor,
            authManager,
            apiServiceCacheManager
        });
        this.router = routerBuilder.build();
    }

    /**
     ** @private
     */
    async _setupElemRegistry() {
        ElemRegistry.setup();
    }


    /**
     *
     */
    initServices({ force = true } = {}) {
        const { services: configServices } = this.config;
        const { services: appServices } = this.vuex.state.app;

        const services = [...configServices, ...(appServices ?? [])].filter(
            ({ id: idToCheck }, indexToCheck, array) => {
                const nextIndex = array.findIndex(({ id }, index) => id === idToCheck && index > indexToCheck);
                return nextIndex < 0;
            }
        );
        ServiceProvider.init(services, { force });
    }

    _setupServices() {
        ServiceProvider.setup({ configs: ServiceSetupConfigs });
    }

    /**
     * @param {{ dataProviders:import('@goodt/editor-common').IProjectDataProvider[] }} options
     */
    _initDataProviders({ dataProviders = [] }) {
        ProjectDataProviderRepository.set(dataProviders ?? []);
    }

    /**
     */
    async _initServiceCacheManager({ config }) {
        await this.apiServiceCacheManager.init();
        this.apiServiceCacheManager.setDebugMode(isDeveloperMode);
        await this.apiServiceCacheManager.setConfig(config);
    }

    /**
     * Attach required handlers
     */
    _attachEvents() {}

    /**
     * Inits widgets AuthManager
     * @return {Promise}
     */
    _initWidgetsAuthManager() {
        this.authManager.setAdapters(Adapters);
        const { authAdapter } = this.vuex.state.app;
        if (authAdapter) {
            const { name, config } = authAdapter;
            return this.authManager.init(name, config);
        }

        return Promise.resolve();
    }

    /**
     * Inits widgets ConstManager
     */
    _initWidgetsConstManager() {
        const { constants } = this.vuex.state.app;
        this.constManager.setConstantsHash(constants);
    }

    /**
     *
     */
    _initErrorHandle() {
        const { errorManager, errorService } = initErrorHandle();
        this.errorManager = errorManager;
        this.errorService = errorService;
    }

    /**
     * @param {{ url:string, redirectUrl:string}} auth
     * @return {Promise<boolean>}
     */
    async _initEditorAuth({ auth = {} }) {
        // standalone mode
        try {
            const { details: { token } } = await this.workerApi.auth.token();
            // unauthorized
            if (token == null) {
                throw new Response('401 Unauthorized', { status: 401, statusText: '401 Unauthorized' });
            }
        } catch (error) {
            this.errorService.handleError({ error, context: this, scope: 'Application' });
            return this.workerApi.common.redirect();
        }
        return true;
    }

    /**
     * Injects css theme <link>
     */
    _injectThemeCss() {
        fetchText('css/theme.css').then((css) => {
            const el = document.createElement('style');
            el.innerHTML = css;
            el.dataset.theme = 1;
            document.head.append(el);
        });
    }

    /**
     * Create vue app
     *
     * @param App
     * @param {import('vuex').Store} store
     * @param {import('vue-router')} router
     * @param {import('vue-i18n')} i18n
     * @param {import('vue').ComponentOptions} restComponentOptions
     */
    _createVueApp(App, { store = this.vuex, router = this.router, i18n = this.i18n, ...restComponentOptions }) {
        const { errorService } = this;
        const vueApp = new Vue({
            store,
            router,
            i18n,
            beforeCreate() {
                this.$errorService = errorService;
            },
            provide: () => ({
                $errorService: errorService
            }),
            render: (h) => h(App),
            ...restComponentOptions
        }).$mount('#app');

        return vueApp;
    }

    _patchVuePlugins({ vuex }) {
        /*
         * Temporary solution
         * Adds reference to self App in Vuex instance
         */
        // eslint-disable-next-line no-param-reassign
        vuex.$app = this;
    }
}
