import ElemError from '@goodt/editor-common/components/ElemError.vue';
import { Const } from '@/utils';
import { Transition } from '@/components/Transition';

/**
 * @typedef {object} PageEntity
 * @property {string} id
 * @property {string} title
 * @property {boolean} isTemplate
 * @property {string} templatePageId
 * @property {PageElemEntity[]} content
 */
/**
 * @typedef {object} PageElemEntity
 * @property {string} id
 * @property {string} type
 * @property {object} props
 * @property {PageElemEntity[]} children
 */

/**
 * @typedef {object} ElemInfo
 * @property {string} id            id
 * @property {string} type          type
 * @property {object} props         props
 * @property {import('vue/types/options').AsyncComponentFactory} component     component def
 * @property {ElemInfo[]} children  children
 */

/**
 *
 */
class RenderTreeBuilder {
    static Types = {
        ELEM_TYPE_TEMPLATE_CONTENT: Const.ELEM_TYPE_TEMPLATE_CONTENT
    };

    constructor({ registry }) {
        this.registry = registry;
    }

    build({ page, template }) {
        return this._buildNodeRenderTree({ page, template });
    }

    /**
     * Resolves which info to build: page or template
     *
     * @param template
     * @param page
     * @return {ElemInfo[]}
     * @private
     */
    _buildNodeRenderTree({ template, page }) {
        if (template) {
            return this._buildTemplateRenderTree({ page, template });
        }

        return this._buildPageRenderTree({ page });
    }

    /**
     * Builds PAGE render tree
     *
     * @param page
     * @return {ElemInfo[]}
     * @private
     */
    _buildPageRenderTree({ page }) {
        return this._buildNodeElemInfo(page.content[0]);
    }

    /**
     * Builds TEMPLATE render tree
     *
     * @param page
     * @param template
     * @return {ElemInfo[]}
     * @private
     */
    _buildTemplateRenderTree({ page, template }) {
        return this._buildNodeElemInfo(template.content[0], page.content[0]);
    }

    /**
     * Builds elem info for the renderElem fn
     * Merges recursively 'pageElem' into 'templateElem', by ELEM_TYPE_TEMPLATE_CONTENT type
     *
     * @return {ElemInfo[]}
     * @param args
     */
    _buildNodeElemInfo(...args) {
        if (this._isPage(...args)) {
            return this._buildPageElemInfo(...args);
        }
        return this._buildWidgetElemInfo(...args);
    }

    /**
     * Builds PAGE render info
     *
     * @param id
     * @param type
     * @param props
     * @param children
     * @return {ElemInfo[]}
     * @private
     */
    _buildPageElemInfo({ id, props, children }, pageElem) {
        return {
            id,
            type: 'transition',
            props,
            component: Transition,
            children: [this._buildNodeElemInfo(pageElem)]
        };
    }

    /**
     * Check if node is page
     *
     * @private
     * @param type
     * @param props
     * @param pageElem
     * @return {boolean}
     */
    _isPage({ type }, pageElem) {
        const { Types } = this.constructor;
        return type === Types.ELEM_TYPE_TEMPLATE_CONTENT && pageElem;
    }

    /**
     * Builds elem info for the renderElem fn
     * Merges recursively 'pageElem' into 'templateElem', by ELEM_TYPE_TEMPLATE_CONTENT type
     *
     * @param {PageElemEntity} templateElem page elem entity
     * @param {?PageElemEntity} pageElem
     * @return {ElemInfo[]}
     */
    _buildWidgetElemInfo({ id, type, props, children }, pageElem) {
        const registryRecord = this.registry.get(type);
        const nodeChildrenElemInfo = children.map((child) => this._buildNodeElemInfo(child, pageElem));
        // merge 'pageElem' into 'templateElem'
        return {
            id,
            type,
            props,
            children: nodeChildrenElemInfo,
            component: registryRecord?.factory ?? ElemError
        };
    }
}

export { RenderTreeBuilder, RenderTreeBuilder as default };
