import Vue from 'vue';
import VueRouter from 'vue-router';
import { RouteManager, AuthManager, StoreManager, PopupManager, ConstManager, EB } from '@goodt-wcore/managers';
import { Const, getClientUniqueId, createLog } from '@/utils';
import { Action, Getter } from '@/store';
import Page from '@/components/Page';

const { store: globalStateStore, ValueObject } = StoreManager;
const { useEventBus } = EB;

Vue.use(VueRouter);

/**
 *
 * @param routes
 * @param pages
 * @return {*}
 */
const buildRoutes = ({ routes: initialRoutes, pages }) => {
    const getPage = (pageId) => pages.find(({ id }) => id === pageId);

    const routes = initialRoutes.reduce((accumRoutes, route) => {
        const { name, slug: path, pageId, enabled } = route;
        if (!enabled) {
            return accumRoutes;
        }

        const page = getPage(pageId);
        if (!page) {
            // eslint-disable-next-line no-console
            console.warn(`skipping route for page: ${pageId}, page not found`, 'route:', route);
            return accumRoutes;
        }
        const template = getPage(page.templatePageId);
        const routeToAdd = {
            name,
            path,
            component: Page,
            meta: { route, page, template }
        };

        return [...accumRoutes, routeToAdd];
    }, []);

    // 404 -> redirect '/'
    routes.push({ path: '*', redirect: '/' });
    routes.push({ path: Const.LOGOUT_PATH });

    return routes;
};

/**
 * remove non global keys from store state
 * @param root0
 * @param root0.store
 */
const clearStoreState = ({ store }) => {
    const { state } = store;

    const stateNew = Object.entries(state).reduce((acc, [key, val]) => {
        const meta = val instanceof ValueObject ? val.meta : null;
        if (meta && meta.global) {
            acc[key] = val;
        }
        return acc;
    }, {});

    store.replace(stateNew);
};

/**
 * @param {Record<string, any>} obj
 * @return {Record<string, ValueObject>}
 */
const buildStoreState = (obj) =>
    Object.entries(obj).reduce(
        (acc, [key, val]) => ({ ...acc, [key]: val instanceof ValueObject ? val : new ValueObject(val) }),
        {}
    );

/**
 *
 */
class RouterBuilder {
    /**
     * Vue router instance
     * @private
     */
    _router;

    // @todo delete
    // _currentRoute;
    /**
     *
     */
    vuex;

    /**
     *
     */
    store;

    /**
     * @type {RouteManager}
     */
    routeManager;

    /**
     * @type {PopupManager}
     */
    popupManager;

    /**
     * @type {import('@/log/RouteMonitor').RouteMonitor}
     */
    routeMonitor;

    /**
     * @type {AuthManager}
     */
    authManager;

    /**
     * @type {number}
     */
    numRoutesResolved = 0;

    /**
     * @type {EB.EventBus}
     */
    eventBus;

    /**
     * @type {Set<() => void>}
     */
    routeLeaveDisposals = new Set();
    
    /**
     * generated client id
     * @type {string}
     */
    cid = getClientUniqueId();
    
    constructor({
        vuex,
        store = globalStateStore,
        routeManager = RouteManager.instance,
        routeMonitor = null,
        authManager = AuthManager.instance
    }) {
        const { eventBus } = useEventBus();
        this.vuex = vuex;
        this.store = store;
        this.routeManager = routeManager;
        this.routeMonitor = routeMonitor;
        this.authManager = authManager;
        this.popupManager = PopupManager.instance;
        this.eventBus = eventBus;
    }

    /**
     *
     * @return {*}
     */
    get router() {
        return this._router;
    }

    /**
     *
     * @return {*}
     */
    build() {
        this._router = this._buildRouter();
        this._initPopupManager();
        this._initRouteManager();

        return this._router;
    }

    /**
     * Inits widgets RouteManager
     * @private
     */
    _initRouteManager() {
        this.routeManager.onNavigate(({ path, query }) => {
            this._router.push({ path, query });
        });
    }

    /**
     * Inits PopupManager
     * @private
     */
    _initPopupManager() {
        const getCurrentPopupPageId = () => this.vuex.getters[Getter.CURRENT_PAGE_POPUP];

        this.popupManager.onOpen(async ({ pageId, resolve, closeEvent }) => {
            await this.vuex.dispatch(Action.SET_PAGE_POPUP, pageId);
            if (closeEvent) {
                closeEvent = ConstManager.instance.getConstValue(closeEvent);
                const unsubscribe = this.eventBus.listen(closeEvent, () => this.popupManager.close(), true);
                this.routeLeaveDisposals.add(unsubscribe);
            };

            resolve();
        });

        this.popupManager.onClose(async ({ resolve }) => {
            const pageId = getCurrentPopupPageId();
            await this.vuex.dispatch(Action.SET_PAGE_POPUP, null);
            resolve({ pageId });
        });

        this.popupManager.setIsOpenResolver(getCurrentPopupPageId);
    }

    /**
     *
     * @private
     * @return {VueRouter}
     */
    _buildRouter() {
        const routes = this._buildRoutes();
        const router = new VueRouter({ routes });
        router.beforeEach(this._routerBeforeEach.bind(this));

        return router;
    }

    /**
     *
     * @return {*}
     * @private
     */
    _buildRoutes() {
        const { pages, routes } = this.vuex.state.app;
        return buildRoutes({ pages, routes });
    }

    /**
     * Before each route guard. Decides whether to render a page/popup
     *
     * @private
     * @param {[ import('vue-router').Route, import('vue-router').Route, Function ]} args
     */
    async _routerBeforeEach(...args) {
        this.routeLeaveDisposals.forEach((dispose) => dispose());
        this.routeLeaveDisposals.clear();

        const [to, from] = args;
        const { path, meta } = to;
        const { page, template } = meta;

        if (path === Const.LOGOUT_PATH) {
            this._handleLogout(...args);
            return;
        }
    
        const { vuex, cid, authManager } = this;
        const pageIds = template ? [page.id, template.id] : [page.id];
        const [pageContent, templateContent] = await vuex.dispatch(Action.GET_PAGE_CONTENT, pageIds);
        meta.page = { ...page, content: pageContent };

        if (template) {
            meta.template = { ...template, content: templateContent };
        }

        if (this.routeMonitor) {
            const { adapter } = authManager;
            this.routeMonitor.log(await createLog({ to, from, adapter, cid }));
        }

        if (meta.route.popup) {
            // handle 'popup' currentRoute
            this._handlePopup(...args);
            return;
        }

        // handle 'page' route
        // eslint-disable-next-line prefer-rest-params
        this._handlePage(...args);
    }

    /**
     * @private
     * @param {Route} to
     * @param {Route} from
     * @param {Function} next
     */
    _handleLogout(to, from, next) {
        const { adapter } = this.authManager;
        if (adapter) {
            adapter.logout();
        }
        next(false);
    }

    /**
     * @private
     * @param {Route} to
     * @param {Route} from
     * @param {Function} next
     */
    async _handlePopup(to, from, next) {
        const {
            query,
            meta: {
                route: {
                    pageId,
                    meta: {
                        popupCloseEvent
                    } = {}
                }
            }
        } = to;
        // if first route --> redirect
        if (this.numRoutesResolved === 0) {
            next({ path: '/' });
            return;
        }

        this.store.commit(buildStoreState(query));
        this.popupManager.open({ pageId, closeEvent: popupCloseEvent });
    }

    /**
     * @private
     * @param {Route} to
     * @param {Route} from
     * @param {Function} next
     */
    _handlePage(to, from, next) {
        const {
            path,
            query,
            meta: { route }
        } = to;

        // title
        document.title = route.title;
        this.numRoutesResolved += 1;

        this.vuex.dispatch(Action.SET_PAGE_POPUP, null);

        clearStoreState({ store: this.store });
        this.store.commit(buildStoreState(query));

        this.routeManager.setRoute({
            path,
            query,
            meta: route
        });

        next();
    }
}

export default RouterBuilder;
