import {
    ErrorManager,
    ErrorService,
    createErrorInfoHandleStrategy,
    muteLogger,
    DefaultErrorHandlers
} from '@goodt-common/error-service';
import { WidgetDebugMessage } from '@goodt-wcore/errors';
import { DremioPresentableError } from '@goodt-common/dremio';

const UiNotifiableErrorTypes = [ErrorManager.ErrorInfoType.DremioError];

/**
 *
 * @param {import('@goodt-wcore/managers/ErrorManager').ErrorInfo} errorInfo
 * @this {import('@goodt-wcore/managers/ErrorManager').ErrorInfoHandleStrategy}
 * @property  {import('@goodt-wcore/managers/ErrorManager').ErrorInfoHandleStrategy}
 * @return {null|ErrorInfo}
 * @constructor
 */
const errorInfoHandler = function errorInfoHandler(errorInfo) {
    const { type, error, scope, context } = errorInfo;

    /**
     * Webpack loading chunks error processing when chunk file missing during artifacts update
     */
    if (error.name === 'ChunkLoadError') {
        const twoMinutesTimestamp = (Date.now() / (1000 * 2 * 60)).toFixed();
        const search = new URLSearchParams(window.location.search);
        search.set('r', twoMinutesTimestamp);
        window.location.search = search.toString();
        return null;
    }

    if (error.isCancel) {
        return null;
    }

    if (error instanceof DremioPresentableError) {
        error.setContext(context);
    }

    if (error instanceof WidgetDebugMessage) {
        if (error.type === 'info') {
            this.logger.info(error);
            return null;
        }
        if (error.type === 'warning') {
            this.logger.warn(error);
            return null;
        }
        this.logger.error(error);
        return null;
    }

    if (type === ErrorManager.ErrorInfoType.CoreWarning) {
        this.logger.warn(error);
        return null;
    }

    // if error was created for presentation purposes but drops in error handler
    if (type === ErrorManager.ErrorInfoType.SilentError) {
        return null;
    }

    if (UiNotifiableErrorTypes.includes(type)) {
        return errorInfo;
    }

    if (scope !== ErrorService.Scope.GLOBAL) {
        // непойманная никем ошибка, выброшенная ранее глобально
        if (error.scope === ErrorService.Scope.GLOBAL) {
            return null;
        }
        this.logger.error(error);
        this.logger.error({ scope });
    }

    return null;
};

/**
 *
 */
export const init = () => {
    const [{ instance: errorService }, { instance: errorManager }] = [ErrorService, ErrorManager];

    const strategy = createErrorInfoHandleStrategy({ handler: errorInfoHandler });
    errorManager.setStrategy(strategy);
    errorManager.setLogger(muteLogger(console, ['error']));

    errorService.subscribe((e) => errorManager.handle(e));

    DefaultErrorHandlers.forEach((h) => errorService.addHandler(h));

    return {
        errorManager,
        errorService
    }
};
