import Vue from 'vue';
import Vuex from 'vuex';

import { buildConstantsHash } from '@goodt/editor-common';
import { traverse } from '@goodt-common/utils';
import { Const, Config, changeLocationWithRedirectParam } from '@/utils';
import { fetchJson, NetworkAuthError } from '@/utils/fetch';
import { ElemCompatHelper } from '@goodt/editor-common/utils';

import Action from './actions';
import Mutation from './mutations';
import Getter from './getters';
import ElemRegistry from '@/core/ElemRegistry';

Vue.use(Vuex);

/**
 * Add a cache key suffix to the url
 * @param {string} url
 * @return {string}
 */
const buildUrlWithCacheKey = (url) => {
    const { cacheKey } = Config.data?.assets ?? {};
    return cacheKey ? `${url}?${cacheKey}` : url;
};

/**
 * @param {Error} error
 */
const handleNetworkError = (error) => {
    const { isStandaloneMode, data } = Config;
    const { redirectUrl } = data?.auth ?? {};
    const isAuthError = error instanceof NetworkAuthError;

    if (isAuthError === false) {
        return;
    }
    // handle auth error => only if 'non-standalone' + redirect defined
    if (isStandaloneMode === false && redirectUrl != null) {
        changeLocationWithRedirectParam(redirectUrl);
    }
};

/**
 * Register widget factories
 * @param {import('@/components/Page').PageElemEntity[]} content
 * @return {Promise}
 */
const registerWidgets = (content) => {
    const types = new Set();
    traverse(content, 'children', ({ type }) => types.add(type));
    [...types].forEach((type) => ElemRegistry.add({ type }));

    return ElemRegistry.ready();
};
const isAppPageLegacyFormat = (page) => Array.isArray(page.content) && page.content.length > 0;

const store = new Vuex.Store({
    strict: true,
    state: {
        app: null,
        pageContentHash: {},
        pagePopup: null,
        isPageContentLoading: false
    },
    mutations: {
        [Mutation.SET_APP](state, app) {
            state.app = app;
        },
        [Mutation.SET_PAGE_POPUP](state, pagePopup) {
            state.pagePopup = pagePopup;
        },
        [Mutation.SET_PAGE_CONTENT](state, { pageId, content }) {
            state.pageContentHash[pageId] = content;
        },
        [Mutation.IS_PAGE_CONTENT_LOADING](state, loading) {
            state.isPageContentLoading = loading;
        }
    },
    actions: {
        async [Action.LOAD_APP]({ commit }) {
            const url = buildUrlWithCacheKey(Const.PATH_APP_JSON);
            let app = null;

            try {
                app = await fetchJson(url);
            } catch (error) {
                handleNetworkError(error);
                return;
            }

            app.constants = buildConstantsHash(app.constants);
            // merge app data with optional addons
            if (Config?.data?.app != null) {
                Config.merge(app, Config.data.app);
            }
            // patch pages if legacy format
            app.pages.forEach((page) => {
                if (isAppPageLegacyFormat(page)) {
                    page.content = ElemCompatHelper.buildPageDataWithNewElemTypes(page.content);
                }
            });
            commit(Mutation.SET_APP, Object.freeze(app));
        },
        async [Action.SET_PAGE_POPUP]({ commit, dispatch, getters }, pageId) {
            if (pageId == null) {
                commit(Mutation.SET_PAGE_POPUP, null);
                return;
            }
            const pages = getters[Getter.PAGES];
            const page = pages.find(({ id }) => id === pageId);
            const template = page ? pages.find(({ id }) => id === page.templatePageId) : null;
            const pageIds = template ? [page.id, template.id] : [page.id];
            const [pageContent, templateContent] = await dispatch(Action.GET_PAGE_CONTENT, pageIds);
            page.content = pageContent;

            if (template != null) {
                template.content = templateContent;
            }
            commit(Mutation.SET_PAGE_POPUP, { page, template });
        },
        [Action.GET_PAGE_CONTENT]({ state, commit, getters }, pageIds) {
            const getPageContent = async (pageId) => {
                const foundPage = getters[Getter.PAGES].find(({ id }) => id === pageId);
                let { content: pageContent = [] } = foundPage;
                // legacy format => page.content is stored along the page meta
                if (pageContent.length > 0) {
                    // no hash => we haven't registered widgets yet
                    if (state.pageContentHash[pageId] == null) {
                        await registerWidgets(pageContent);
                        // re-init services, due to loaded page could involve new widgets package load
                        // that could append new ServiceProvider config
                        this.$app.initServices({ force: false });
                        commit(Mutation.SET_PAGE_CONTENT, { pageId, content: null });
                    }
                    return pageContent;
                }

                if (state.pageContentHash[pageId]) {
                    return state.pageContentHash[pageId];
                }
                const url = buildUrlWithCacheKey(Const.PATH_PAGE_JSON(pageId));
                let content = null;
                try {
                    content = await fetchJson(url);
                    content = ElemCompatHelper.buildPageDataWithNewElemTypes(content);
                } catch (error) {
                    handleNetworkError(error);
                }
                Object.freeze(content);
                await registerWidgets(content);
                // re-init services, due to loaded page could involve new widgets package load
                // that could append new ServiceProvider config
                this.$app.initServices({ force: false });
                commit(Mutation.SET_PAGE_CONTENT, { pageId, content });
                return content;
            };
            commit(Mutation.IS_PAGE_CONTENT_LOADING, true);
            const promises = Promise.all(pageIds.map(getPageContent));
            promises.then(() => commit(Mutation.IS_PAGE_CONTENT_LOADING, false));
            return promises;
        }
    },
    getters: {
        [Getter.CURRENT_PAGE_POPUP](state) {
            return state.pagePopup;
        },
        [Getter.PAGES](state) {
            const { app: { pages } = {} } = state;
            return pages ?? [];
        }
    }
});

export { store, Action, Mutation, Getter };

export default store;
