/* eslint-disable id-length */
import { dayjs } from '@goodt-common/utils';
import html2canvas from 'html2canvas';
import { GridPlace } from './const';

/**
 * @param {array} arr
 * @param {number} size
 * @param {string} defaultValue
 * @param {array}
 */
export const setArraySize = (arr, size, defaultValue) => {
    const { length } = arr;
    if (length === size) {
        return arr;
    }
    if (length > size) {
        return arr.slice(0, size);
    }
    const newArr = [...new Array(size - length)].map(() =>
        typeof defaultValue == 'function' ? defaultValue() : defaultValue
    );
    return [...arr, ...newArr];
};
/**
 * @param {array[]} arr
 * @param {number} rows
 * @param {number} cols
 * @param {string} defaultValue
 * @return {array[]}
 */
export const setArray2DSize = (arr, rows, cols, defaultValue) => {
    const newArr = setArraySize(arr, rows, () => [...new Array(cols)].map(() => defaultValue));
    return newArr.map((arrSub) => setArraySize(arrSub, cols, defaultValue));
};
/**
 * @param {array[]} arr
 * @param {string} value
 * @return {{ x:number, y:number, w:number, h:number, value }} intersection
 */
export const findArray2DRect = (arr, value) => {
    const info = { x: -1, y: -1, w: 0, h: 0, value };
    const w = [];
    if (!arr.flat().includes(value)) {
        return null;
    }
    for (let y = 0; y < arr.length; y++) {
        let xw = 0;
        for (let x = 0; x < arr[y].length; x++) {
            const v = arr[y][x];
            if (v !== value) {
                // eslint-disable-next-line
                continue;
            }
            if (info.x < 0) {
                info.x = x;
                info.y = y;
            }
            if (x - info.x - xw <= 1) {
                xw++;
            } else {
                break;
            }
        }
        if (xw > 0) {
            info.h++;
            w.push(xw);
        } else if (info.x >= 0) {
            break;
        }
    }
    info.w = Math.min.apply(this, w);
    return info;
};

/**
 * @param {DOMRect} rect
 * @param {{ left:number, top:number }} position
 * @return {boolean}
 */
export const rectIntersect = (rect, { left, top }) =>
    rect.left <= left && rect.left + rect.width >= left && rect.top <= top && rect.top + rect.height >= top;

/**
 * @param {Element} node
 * @param {string} name
 * @param {(value:any) => {}} func
 * @return {any}
 */
export const getDomAttrValue = (node, name, func) => func(node.getAttribute(name));

/**
 * @return {string[]}
 */
export const gridPlaceFactory = () => [GridPlace.NORMAL, GridPlace.NORMAL];

/**
 * @typedef {import('./constants').ImageFormat} ImageFormat
 */

/**
 * @typedef {Object} CanvasOptions
 * @property {boolean} logging
 * @property {boolean} useCORS
 * @property {number} width
 * @property {number} height
 * @property {string} [backgroundColor]
 */

/**
 * @param {HTMLElement} widgetElement
 * @param {boolean} isFormatPNG
 * @return {CanvasOptions}
 */
export const buildCanvasOptions = (widgetElement, isFormatPNG) => ({
    logging: false,
    useCORS: true,
    width: widgetElement.scrollWidth,
    height: widgetElement.scrollHeight,
    ...(isFormatPNG && { backgroundColor: null })
});

/**
 * @param {{ title: string, format: ImageFormat, withDate: boolean }} options
 * @return {string}
 */
export const resolveFilename = ({ title, extension, withDate }) => {
    if (withDate) {
        return `${title}_${dayjs().format('DD.MM.YYYY')}.${extension}`;
    }

    return `${title}.${extension}`;
};

/**
 * @param {HTMLElement} element
 * @param {CanvasOptions} options
 * @return {HTMLCanvasElement}
 */
export const makeScreenshot = async (element, options) => {
    const objectUrls = [];

    /**
     * user agent uses chromium engine
     * @see https://bugs.chromium.org/p/chromium/issues/detail?id=880768
     */
    if (/Chrome|Chromium/.test(navigator.userAgent)) {
        options.onclone = async (clonedDocument) => {
            const clonedElement = clonedDocument.getElementById(element.id);
            const images = [...clonedElement.querySelectorAll('img')].filter(({ src }) =>
                src.includes(window.location.origin)
            );
            const imagePromises = images.map((image) =>
                fetch(image.src)
                    .then((response) => response.blob())
                    .then(
                        (blob) =>
                            new Promise((resolve, reject) => {
                                const objectUrl = URL.createObjectURL(blob);
                                objectUrls.push(objectUrl);
                                image.onload = resolve;
                                image.onerror = reject;
                                image.src = objectUrl;
                            })
                    )
            );
            await Promise.allSettled(imagePromises);
        };
    }

    const canvas = await html2canvas(element, options);
    objectUrls.forEach(URL.revokeObjectURL);

    return canvas;
};
