import JSPDF from 'jspdf';
import { PageOrientation, IMAGE_FORMAT, A4_HEIGHT, A4_WIDTH, PAGE_X_OFFSET, PAGE_Y_OFFSET } from './constants';

/**
 * @param {number} imageWidth
 * @param {number} imageHeight
 * @param {number} pageWidth
 * @param {number} pageHeight
 * @param {number} aspectRatio
 * @return {[number, number]}
 */
const calcImageDimensions = (imageWidth, imageHeight, pageWidth, pageHeight, aspectRatio) => {
    const [maxWidth, maxHeight] = [pageWidth - PAGE_X_OFFSET, pageHeight - PAGE_Y_OFFSET];

    if (imageWidth < maxWidth && imageHeight < maxHeight) {
        return [imageWidth, imageHeight];
    }

    return aspectRatio > 1 ? [maxWidth, maxWidth / aspectRatio] : [maxHeight, maxHeight * aspectRatio];
};

/**
 * @param{{blob: Blob, filename: string}[]} screenshots
 */
export const printPDF = async (screenshots) => {
    let doc = null;
    const [{ filename: docName }] = screenshots;

    const onImageLoadEnd = (img) => {
        const { width: imageWidth, height: imageHeight } = img;
        const aspectRatio = imageWidth / imageHeight;
        const isLandscape = aspectRatio > 1;
        const [pageWidth, pageHeight] = isLandscape ? [A4_HEIGHT, A4_WIDTH] : [A4_WIDTH, A4_HEIGHT];
        const [width, height] = calcImageDimensions(imageWidth, imageHeight, pageWidth, pageHeight, aspectRatio);
        const [offsetX, offsetY] = [(pageWidth - width) / 2, (pageHeight - height) / 2];
        const options = {
            orientation: isLandscape ? PageOrientation.LANDSCAPE : PageOrientation.PORTRAIT,
            // eslint-disable-next-line no-restricted-syntax
            format: [pageWidth, pageHeight]
        };

        if (doc == null) {
            doc = new JSPDF(options);
        } else {
            doc.addPage(options);
        }

        doc.addImage(img, IMAGE_FORMAT, offsetX, offsetY, width, height);
    };

    const promises = screenshots.map(
        ({ blob }) =>
            new Promise((resolve, reject) => {
                const reader = new FileReader();
                reader.onloadend = () => {
                    const img = new Image();
                    img.onload = () => {
                        onImageLoadEnd(img);
                        resolve();
                    };
                    img.onerror = reject;
                    img.src = reader.result;
                };
                reader.readAsDataURL(blob);
            })
    );

    await Promise.allSettled(promises);

    if (doc != null) {
        doc.save(docName);
    }
};
