import { reactive, useEventHook } from '@goodt-common/utils';
import isEmpty from 'lodash/isEmpty';
import uniqBy from 'lodash/uniqBy';
import {
    hexToRgb,
    resolveCoordinatesForCircles,
    resolveCoordinatesForLineByCircles,
    resolveCoordinatesForSegmentLines,
    resolveCoordinatesForSegmentTitles,
    resolveCoordinatesForSegmentValues
} from '../utils';
import {
    CIRCLE_OFFSET_RADIUS_FOR_FACT_EQ_PLAN,
    CIRCLE_OPACITY_FOR_THE_FACT_LESS_PLAN,
    CIRCLE_STROKE_WIDTH_FOR_FACT_EQ_PLAN,
    CIRCLE_STROKE_WIDTH_FOR_PLAN
} from '../constants';

export class CompetenceDiagramViewModel {
    _state = reactive({
        numberOfLaps: 0,
        isLoading: false,
        circles: [],
        competenceCircles: [],
        catalogIds: [],
        segmentLines: [],
        segmentTitles: [],
        segmentCharacteristics: [],
        competences: [],
        currentLineByHoveredCircles: [],
        segmentValues: [],
        tooltip: null
    });

    /**
     * @type {typeof import('../api/mixins').ApiMixinsTypeDescriptor['rostalentApiService']}
     * @private
     */
    _rostalentApiService;

    /**
     * @type {import('../api/mixins').ApiMixinsTypeDescriptor['taskSettingsApiService']}
     * @private
     */
    _taskSettingsApiService;

    _widget;

    /**
     *
     * @type {{on(callback: () => any): {off(): void}, off(callback: () => any): void, trigger(param: any): void}}
     * @private
     */
    _errorHook = useEventHook();

    constructor({ rostalentApiService, taskSettingsApiService, widget }) {
        this._rostalentApiService = rostalentApiService;
        this._taskSettingsApiService = taskSettingsApiService;
        this._widget = widget;
    }

    get numberOfLaps() {
        return this._state.numberOfLaps;
    }

    get isLoading() {
        return this._state.isLoading;
    }

    get competences() {
        return this._state.competences;
    }

    get circles() {
        return this._state.circles;
    }

    get competenceCircles() {
        return this._state.competenceCircles;
    }

    get segmentLines() {
        return this._state.segmentLines;
    }

    get segmentTitles() {
        return this._state.segmentTitles;
    }

    get currentLineByHoveredCircles() {
        return this._state.currentLineByHoveredCircles;
    }

    get segmentValues() {
        return this._state.segmentValues;
    }

    get tooltip() {
        return this._state.tooltip;
    }

    /**
     *
     * @param safeResult
     * @return {Promise<*|null>}
     * @private
     */
    async _processSafeResult(safeResult) {
        if (safeResult instanceof Promise) {
            // eslint-disable-next-line no-param-reassign
            safeResult = await safeResult;
        }
        const { isError, error, result } = safeResult;
        if (isError) {
            this._onApiError(error);
            return null;
        }

        return result;
    }

    /**
     *
     * @param {Error} error
     * @private
     */
    _onApiError(error) {
        this._errorHook.trigger(error);
    }

    /**
     *
     * @param errorCallback
     * @return {*}
     */
    onError(errorCallback) {
        return this._errorHook.on(errorCallback);
    }

    // API CALLS
    async fetchTaskInfoById(id) {
        return this._processSafeResult(this._taskSettingsApiService.getTaskById(id));
    }

    async fetchCompetenceProfileCompetenceListById(id) {
        return this._processSafeResult(this._rostalentApiService.getCompetenceProfileCompetenceList(id));
    }

    // Builders
    /**
     * @param {object} settings - custom settings from widget and param panels
     */
    async build(settings) {
        await this.resolveDataByMode(settings);
        this.resolveSegmentCharacteristics();
        this.buildDiagram(settings);
    }

    /**
     * @param {object} diagramSettings - custom settings from param panel
     * @param {number} titleOffset - offset title coefficient
     *
     * @description rebuilds chart with new diagram settings
     * */
    rebuildDiagram({ diagramSettings, titleOffset }) {
        const { length } = this._state.competences;
        if (length === 0) {
            return;
        }
        this.buildDiagram({ diagramSettings, titleOffset });
    }

    /**
     * @description destroys diagram
     * */
    destroyDiagram() {
        this._state.circles = [];
        this._state.competenceCircles = [];
        this._state.segmentLines = [];
        this._state.segmentTitles = [];
        this._state.currentLineByHoveredCircles = [];
        this._state.segmentValues = [];
    }

    /**
     * @param {number} taskId
     * @param {number} taskTypeFieldId
     */
    async buildByApiMode({ taskId, taskTypeFieldId }) {
        if (taskId == null || taskTypeFieldId == null) {
            this._state.isLoading = false;
            return;
        }

        const taskInfo = await this.fetchTaskInfoById(taskId);
        if (taskInfo === null || Object.hasOwnProperty.call(taskInfo, 'fields') === false) {
            this._state.isLoading = false;
            return;
        }

        const task = taskInfo.fields.find(({ type: { id: typeId } }) => taskTypeFieldId === typeId);
        if (task == null) {
            return;
        }

        const competenceProfileId = task.value;
        if (competenceProfileId !== null) {
            const competenceCatalogList = await this.fetchCompetenceProfileCompetenceListById(competenceProfileId);
            const segments = new Set();
            competenceCatalogList
                .filter(({ competence_catalog: { parent_id: parentId } }) => parentId === null)
                .forEach(({ competence_catalog: competenceCatalog }) => segments.add(competenceCatalog.id));

            const segmentsKeys = [...segments.keys()];

            if (segmentsKeys.length === 0) {
                this._state.isLoading = false;
                return;
            }

            this._state.competences = competenceCatalogList.map(
                ({
                    competence_catalog: { id: segment, name: segmentTitle, params },
                    name: competenceName,
                    indicators,
                    scale_type: { level_count: diagramNumberOfLaps, scale_levels: scaleLevels }
                }) => ({
                    segment: segmentsKeys.indexOf(segment),
                    plan: this.resolvePlanValueFromScaleLevels(scaleLevels),
                    segmentTitle,
                    competenceName,
                    diagramNumberOfLaps,
                    fact: this.resolveFactValueFromIndicators(indicators),
                    segmentColor: JSON.stringify(params)?.color || '#4272d4',
                    tooltipText: competenceName
                })
            );
            this._state.competences = this._state.competences.map(({ segment, ...values }) => ({
                ...values,
                segment,
                segmentValue: this.resolveSegmentValue(segment)
            }));
            this._state.numberOfLaps = Math.max(
                ...this._state.competences.map(({ diagramNumberOfLaps }) => diagramNumberOfLaps)
            );
        } else {
            this._state.competences = [];
        }
    }

    /**
     * @param {array} catalogIds
     * @param {object} dremioSettings - metric names from param panels
     * @param {array} dremioDatasets - dremio datasets
     * @param {boolean} mergeDatasets
     * @param {number} numberOfLaps
     * @param {boolean} showSegmentsByMetrics
     */
    buildByDremioMode({
        dremioDatasets,
        dremioSettings,
        catalogIds,
        mergeDatasets,
        numberOfLaps,
        showSegmentsByMetrics
    }) {
        if (
            dremioDatasets == null ||
            dremioDatasets.length === 0 ||
            dremioSettings == null ||
            dremioSettings.length === 0
        ) {
            this._state.isLoading = false;
            return;
        }
        this._state.competences =
            mergeDatasets === true
                ? this.mergeDatasets({ dremioDatasets, dremioSettings, catalogIds, showSegmentsByMetrics })
                : this.reduceDatasets({ dremioDatasets, dremioSettings, catalogIds, showSegmentsByMetrics });
        this._state.numberOfLaps = mergeDatasets
            ? numberOfLaps
            : Math.max(...this._state.competences.map(({ diagramNumberOfLaps }) => diagramNumberOfLaps));
    }

    mergeDatasets({ dremioDatasets, dremioSettings, catalogIds, showSegmentsByMetrics }) {
        const segments = dremioDatasets[0].rows
            .map((row) => ({
                segmentId: Number(row[dremioSettings[0].segmentId]),
                segmentTitle: row[dremioSettings[0].segmentTitle],
                segmentColor: JSON.parse(row[dremioSettings[0].segmentColor])?.color.replace('##', '#'),
                segmentValue: 0,
                segmentLength: 0
            }))
            .filter(({ segmentId }) => showSegmentsByMetrics || catalogIds.includes(String(segmentId)));
        const segmentKeys = segments.map(({ segmentId }) => segmentId);

        const {
            tooltipMetric: { factMetric, planMetric }
        } = this._widget.props;
        const competencies = dremioDatasets[1].rows
            .map((row) => {
                const plan = parseFloat(row[dremioSettings[1]?.plan]) || null;

                // eslint-disable-next-line no-restricted-syntax
                const currentSegment = segments.find(
                    ({ segmentId }) => segmentId === Number(row[dremioSettings[1].segmentId])
                );
                if (currentSegment == null) {
                    return {};
                }

                const { segmentId, segmentTitle, segmentColor } = currentSegment;
                // eslint-disable-next-line no-restricted-syntax
                const currentCompetence = dremioDatasets[2].rows.find(
                    ({ [dremioSettings[2].competenceId]: competenceId }) =>
                        competenceId === row[dremioSettings[1].competenceId]
                );

                const segmentIndex = segmentKeys.indexOf(Number(row[dremioSettings[1].segmentId]));

                const fact = (currentCompetence && parseFloat(currentCompetence[dremioSettings[2]?.fact])) || null;
                if (currentCompetence != null) {
                    segments[segmentIndex].segmentValue += fact;
                    segments[segmentIndex].segmentLength += 1;
                }

                return {
                    segmentId,
                    segment: segmentIndex,
                    segmentTitle,
                    segmentColor,
                    segmentValue: null,
                    plan,
                    fact,
                    tooltipPlanValue: row[planMetric] ?? plan,
                    tooltipFactValue: currentCompetence != null ? currentCompetence[factMetric] : fact,
                    tooltipText: row[dremioSettings[1].tooltip] || 'Нет данных'
                };
            })
            .filter((point) => !isEmpty(point))
            .sort(({ segment: segmentA }, { segment: segmentB }) => segmentA - segmentB);

        return competencies.map((point) => {
            const { segmentValue, segmentLength } = segments.find(({ segmentId }) => segmentId === point.segmentId);

            return { ...point, segmentValue: segmentValue / segmentLength || null };
        });
    }

    reduceDatasets({ dremioDatasets, dremioSettings, catalogIds, showSegmentsByMetrics }) {
        return dremioDatasets
            .reduce((acc, dataset, idx) => {
                const lastSegmentNumber = Math.max(...acc.map(({ segment }) => segment));
                const segmentLength = Number.isFinite(lastSegmentNumber) ? lastSegmentNumber + 1 : 0;
                return acc.concat(
                    this.buildDataByDataset({
                        dataset,
                        settings: dremioSettings[idx],
                        catalogIds,
                        showSegmentsByMetrics,
                        segmentLength
                    })
                );
            }, [])
            .sort(({ segment: segmentA }, { segment: segmentB }) => segmentA - segmentB);
    }

    /**
     * @param {array} catalogIds
     * @param {number} taskTypeFieldId
     * @param {number} segmentLength
     * @param {array} dataset - dremio dataset
     */
    buildDataByDataset({ dataset, settings, catalogIds, segmentLength, showSegmentsByMetrics }) {
        const { segmentId, segmentTitle, segmentValue, name, plan, fact, segmentColor, numberOfLaps, tooltip } =
            settings;
        const segments = new Set();
        dataset.rows.forEach((row) => {
            segments.add(row[segmentId]);
        });

        const segmentsKeys = [...segments.keys()];
        if (segmentsKeys.length === 0) {
            this._state.isLoading = false;
            return [];
        }

        return dataset.rows
            .filter((row) => showSegmentsByMetrics || catalogIds.includes(row[segmentId]))
            .map((row) => ({
                segment: segmentsKeys.indexOf(row[segmentId]) + segmentLength,
                segmentTitle: row[segmentTitle],
                segmentColor: JSON.parse(row[segmentColor])?.color.replace('##', '#'),
                segmentValue: row[segmentValue],
                diagramNumberOfLaps: parseFloat(row[numberOfLaps]),
                competenceName: row[name],
                plan: parseFloat(row[plan]),
                fact: parseFloat(row[fact]),
                tooltipText: row[tooltip]
            }));
    }

    /**
     * @param {object} diagramSettings - custom settings from param panel
     *
     * @description builds diagram circles
     * */
    buildDiagramCircles(diagramSettings) {
        const { centerX, centerY, radius } = diagramSettings;
        this._state.circles = [];
        for (let i = 1; i <= this._state.numberOfLaps; i++) {
            this._state.circles.push({
                id: i,
                xCoordinate: centerX,
                yCoordinate: centerY,
                radius: radius * i
            });
        }
        this._state.circles = this._state.circles.reverse();
    }

    /**
     * @param {object} diagramSettings
     * @param {number} titleOffset - offset title coefficient
     *
     * @description builds diagram
     * */
    buildDiagram({ diagramSettings, titleOffset }) {
        this.destroyDiagram();
        this.buildDiagramCircles(diagramSettings);

        const { segmentCharacteristics, competences } = this._state;
        if (competences.length === 0) {
            return;
        }

        const segments = segmentCharacteristics.length;
        let index = 0;
        for (let i = 0; i < segments; i++) {
            const { segmentColor, length: itemsLength, segmentValue, segmentTitle } = segmentCharacteristics[i];
            this.buildSegmentTitle(i, segments, segmentTitle, diagramSettings, titleOffset);
            if (segments > 1) {
                this.buildSegmentLine(i, segments, diagramSettings);
            }
            this.buildSegmentValue(i, segments, segmentColor, segmentValue, diagramSettings);
            for (let j = 0; j < itemsLength; j++) {
                const competence = competences[index];
                if (competence !== undefined) {
                    this.buildDataCircle({
                        i,
                        j,
                        segmentColor,
                        competence,
                        itemsLength,
                        segments,
                        diagramSettings
                    });
                }
                index++;
            }
        }
    }

    /**
     * @param {number} i - the index of the segment value
     * @param {number} segments - segments length
     * @param {string} title - segment title
     * @param {object} diagramSettings - diagram settings
     * @param {number} titleOffset -  title offset coefficient
     *
     * @description builds segment title for the catalogs
     * */
    buildSegmentTitle(i, segments, title, diagramSettings, titleOffset) {
        const { centerX, centerY, radius } = diagramSettings;
        const sectorCenterSegment = (i * 2 + 1) / (segments * 2);
        const { xCoordinate, yCoordinate } = resolveCoordinatesForSegmentTitles(
            sectorCenterSegment,
            this._state.circles.length,
            radius,
            centerX,
            centerY,
            titleOffset
        );
        this._state.segmentTitles.push({
            id: `title-${i}`,
            xCoordinate,
            yCoordinate,
            title
            // textAnchor: resolveTextAnchorForTitle(i + 1, segments)
        });
    }

    /**
     * @param {number} i - the index of the segment value
     * @param {number} segments - segments length
     * @param {object} diagramSettings - diagram settings
     *
     * @description builds lines dividing the diagram into segments
     * */
    buildSegmentLine(i, segments, diagramSettings) {
        const { centerX, centerY, radius } = diagramSettings;
        const sectorSegment = (i + 1) / segments;
        const { xCoordinate2, yCoordinate2 } = resolveCoordinatesForSegmentLines(
            sectorSegment,
            this._state.circles.length,
            radius,
            centerX,
            centerY
        );
        this._state.segmentLines.push({
            id: `line-${i}`,
            xCoordinate1: centerX,
            yCoordinate1: centerY,
            xCoordinate2,
            yCoordinate2
        });
    }

    /**
     * @param {number} i - the index of the segment value
     * @param {number} segments - segments length
     * @param {string} fill - text color
     * @param {number} value - segment value
     * @param {object} diagramSettings - diagram settings
     *
     * @description builds segment values for the catalogs
     * */
    buildSegmentValue(i, segments, fill, value, diagramSettings) {
        const { centerX, centerY, radius } = diagramSettings;
        const sectorCenterSegment = (i * 2 + 1) / (segments * 2);
        const { xCoordinate, yCoordinate } = resolveCoordinatesForSegmentValues(
            sectorCenterSegment,
            radius,
            centerX,
            centerY
        );
        this._state.segmentValues.push({
            id: `text-${i}`,
            xCoordinate,
            yCoordinate,
            fill,
            value
        });
    }

    /**
     * @param {number} i - segment index
     * @param {number} j - segment items index
     * @param {object} competence - circle metrics
     * @param {number} itemsLength - count circles in the segment
     * @param {number} segments - number of segments
     * @param {object} diagramSettings - diagram settings
     *
     * @description builds circles by segment
     * */
    buildDataCircle({ i, j, competence, itemsLength, segments, diagramSettings }) {
        const { centerX, centerY, radius, circleRadius, multiplierCircleRadius } = diagramSettings;
        const { plan, fact, tooltipText, segmentColor, tooltipPlanValue, tooltipFactValue } = competence;
        if (plan == null && fact == null) {
            return;
        }
        const percentPlan = (plan * 100) / this._state.numberOfLaps;
        const percentFact = (fact * 100) / this._state.numberOfLaps;
        const circlePercent = 100 / this._state.numberOfLaps;
        let circlePlan = percentPlan / circlePercent;
        let circleFact = percentFact / circlePercent;
        if (circlePlan === circleFact && fact !== plan) {
            if (plan > fact) {
                circlePlan++;
            } else {
                circleFact++;
            }
        }

        const sector = (j + 1 + i * (itemsLength + 1)) / ((itemsLength + 1) * segments);
        const fullRadius = radius * Math.max(circlePlan, circleFact);
        const { x: xFact, y: yFact } = resolveCoordinatesForCircles(sector, radius * circleFact, centerX, centerY);
        const circleFactData = {
            id: `circle-data-fact-${xFact}-${yFact}`,
            xCoordinate: xFact,
            yCoordinate: yFact,
            name: tooltipText,
            plan,
            fact,
            tooltipPlanValue: tooltipPlanValue ?? plan,
            tooltipFactValue: tooltipFactValue ?? fact,
            type: 'outside',
            angle: sector,
            fill: segmentColor,
            mainColor: segmentColor,
            strokeWidth: 0,
            stroke: '',
            fullRadius,
            opacity: 1,
            radius: circleRadius + multiplierCircleRadius * circleFact
        };

        if (plan !== fact) {
            const { x: xPlan, y: yPlan } = resolveCoordinatesForCircles(sector, radius * circlePlan, centerX, centerY);
            const circlePlanData = {
                id: `circle-data-plan-${xPlan}-${yPlan}`,
                xCoordinate: xPlan,
                yCoordinate: yPlan,
                angle: sector,
                plan,
                fact,
                tooltipPlanValue: tooltipPlanValue ?? plan,
                tooltipFactValue: tooltipFactValue ?? fact,
                type: 'outside',
                name: tooltipText,
                mainColor: segmentColor,
                stroke: segmentColor,
                strokeWidth: CIRCLE_STROKE_WIDTH_FOR_PLAN,
                fullRadius,
                radius: circleRadius + multiplierCircleRadius * circlePlan,
                fill: '#ffffff',
                opacity: 1
            };

            if (plan > fact) {
                circleFactData.fill = hexToRgb(circleFactData.fill, CIRCLE_OPACITY_FOR_THE_FACT_LESS_PLAN);
            }

            if (plan == null) {
                this._state.competenceCircles.push(circleFactData);
                return;
            }

            if (fact == null) {
                this._state.competenceCircles.push(circlePlanData);
                return;
            }

            this._state.competenceCircles.push(circleFactData, circlePlanData);
            return;
        }

        const circleCompletedData = JSON.parse(JSON.stringify(circleFactData));

        // adds two circles when plan === fact
        if (fact == null) {
            this._state.competenceCircles.push({
                ...circleCompletedData,
                id: `circle-data-fact-${xFact + 1}-${yFact}`,
                type: 'inside',
                strokeWidth: CIRCLE_STROKE_WIDTH_FOR_FACT_EQ_PLAN,
                stroke: '#ffffff',
                radius: circleCompletedData.radius - CIRCLE_OFFSET_RADIUS_FOR_FACT_EQ_PLAN
            });
            return;
        }

        this._state.competenceCircles.push(circleFactData, {
            ...circleCompletedData,
            id: `circle-data-fact-${xFact + 1}-${yFact}`,
            type: 'inside',
            strokeWidth: CIRCLE_STROKE_WIDTH_FOR_FACT_EQ_PLAN,
            stroke: '#ffffff',
            radius: circleCompletedData.radius - CIRCLE_OFFSET_RADIUS_FOR_FACT_EQ_PLAN
        });
    }

    /**
     * @description calculates the number of items in the segment
     * */
    resolveSegmentCharacteristics() {
        const segments = uniqBy(this._state.competences, 'segment');
        this._state.segmentCharacteristics = segments.map(({ segment, segmentColor, segmentValue, segmentTitle }) => ({
            length: this._state.competences.filter((competence) => competence.segment === segment).length,
            segmentColor,
            segmentValue,
            segmentTitle
        }));
    }

    /**
     * @param {string} mode - widget mode 'api/dremio'
     * @param {array} catalogIds
     * @param {number} taskId
     * @param {number} taskTypeFieldId
     * @param {object} dremioSettings - metric names from param panels
     * @param {array} dremioDatasets - dremio datasets
     * @param {boolean} mergeDatasets
     * @param {number} numberOfLaps
     * @param {boolean} showSegmentsByMetrics
     * @description resolves data by mode
     * */
    async resolveDataByMode({
        mode,
        catalogIds,
        taskId,
        taskTypeFieldId,
        dremioSettings,
        dremioDatasets,
        mergeDatasets,
        numberOfLaps,
        showSegmentsByMetrics
    }) {
        this._state.competences = [];
        this._state.isLoading = true;

        if (mode === 'api') {
            await this.buildByApiMode({ taskId, taskTypeFieldId });
        } else {
            this.buildByDremioMode({
                dremioDatasets,
                dremioSettings,
                catalogIds,
                mergeDatasets,
                numberOfLaps,
                showSegmentsByMetrics
            });
        }
        this._state.isLoading = false;
    }

    resolveSegmentValue(segment) {
        const segmentCompetencies = this._state.competences.filter((competence) => competence.segment === segment);
        return segmentCompetencies.reduce((acc, { fact }) => acc + fact, 0) / segmentCompetencies.length;
    }

    resolvePlanValueFromScaleLevels(scaleLevels) {
        return scaleLevels
            .filter(({ date_to: dateTo }) => dateTo === null)
            .sort(({ date_from: dateFromA }, { date_from: dateFromB }) => new Date(dateFromB) - new Date(dateFromA))[0]
            .level_point;
    }

    resolveFactValueFromIndicators(indicators) {
        return (
            indicators.reduce(
                (acc, { indicator_levels: indicatorLevels }) => acc + this.resolveAverageSum(indicatorLevels),
                0
            ) / indicators.length
        );
    }

    resolveAverageSum(indicatorLevels) {
        return (
            indicatorLevels.reduce((acc, { scale_level: scaleLevel }) => acc + scaleLevel?.level_point, 0) /
            indicatorLevels.length
        );
    }

    /**
     * @param {string} name - competence name
     * @param {number} xCoordinate
     * @param {number} radius
     * @param {number} yCoordinate
     * @param {string} mainColor - catalog color
     * @param {number} offsetY - offset by Y axis
     *
     * @description resolves tooltip settings
     * */
    showTooltip({ name, xCoordinate, type, radius, yCoordinate, mainColor }) {
        const currentRadius = type === 'inside' ? radius + CIRCLE_OFFSET_RADIUS_FOR_FACT_EQ_PLAN : radius;
        this._state.tooltip = {
            width: name.length,
            xCoordinate,
            yCoordinate: yCoordinate - 1.5 * currentRadius,
            background: mainColor,
            name
        };
    }

    /**
     * @description clears tooltip settings
     * */
    hideTooltip() {
        this._state.tooltip = null;
    }

    /**
     * @param {number} angle
     * @param {number} fullRadius - distance for the last circle on the diagram
     * @param {object} diagramSettings - settings from param panel
     *
     * @description resolves hover line settings
     */
    showConnectingLine({ angle, fullRadius, diagramSettings }) {
        const { centerX, centerY } = diagramSettings;
        const { x, y } = resolveCoordinatesForLineByCircles(angle, fullRadius, centerX, centerY);
        this.currentLineByHoveredCircles.push({
            id: `line-hover`,
            xCoordinate1: centerX,
            yCoordinate1: centerY,
            xCoordinate2: x,
            yCoordinate2: y
        });
    }

    /**
     * @description clears hover live settings
     * */
    hideConnectingLine() {
        this._state.currentLineByHoveredCircles = [];
    }
}
