import { Query, SDK, Dremio } from 'goodt-dremio-sdk';
import { useDremio } from '@goodt-common/dremio';
import cloneDeep from 'lodash/cloneDeep';
import isEqual from 'lodash/isEqual';

const { mixin: initialDremioMixin } = useDremio();

class CancelToken {
    isCancelled = false;

    cancel() {
        this.isCancelled = true;
    }
}

/**
 * (dremio) Widget component mixin
 */
const DremioMixin = {
    mixins: [initialDremioMixin],
    data() {
        return {
            dremioVars: [],
            // eslint-disable-next-line no-restricted-syntax
            queryHelper: [],
            dimensions: []
        };
    },
    watch: {
        $storeState: {
            immediate: true,
            handler(state) {
                if (Object.keys(state).length === 0) {
                    return;
                }
                this.$nextTick(() => {
                    this.dimensions = this.queryHelper
                        .map((query) => Object.keys(query.dimensionList))
                        .reduce((a, b) => a.concat(b));
                    this.storeStateWatcher(state);
                    this.loadData();
                });
            }
        }
    },
    methods: {
        /**
         * Loads data via dremio sdk
         */
        loadData() {
            this.dremioSdk.cancelActiveRequests();

            const cancelToken = new CancelToken();
            this.loadDataHooks.before(cancelToken.cancel.bind(cancelToken));
            if (cancelToken.isCancelled === true) {
                return;
            }

            Promise.all(this.queryHelper.map((helper) => this.dremioSdk.getData(helper.buildQuery(), 0, 0)))
                .then((results) => {
                    this.result = results;
                    this.loadDataHooks.then(results);
                })
                .catch(this.loadDataHooks.catch)
                .finally(this.loadDataHooks.finally);
        },
        /**
         * Inits queryHelpher, injects dremio vars, reloads data
         */
        dremioHandler() {
            const { dremio } = this.props;
            if (dremio == null || dremio.length === 0) {
                this.result = null;
                return;
            }

            this.queryHelper = dremio.map((dataset) => new Query(cloneDeep(dataset)));
            this.setDremioVars();
            this.loadData();
        },
        /**
         * Applies dremio filters
         *
         * @param {Record<string, any>} params   params to be injected
         * @return {boolean}
         */
        applyDremioFilters(params) {
            if (this.queryHelper == null) {
                return false;
            }

            let { query } = this.queryHelper;
            const originalQuery = cloneDeep(query);
            const dremioParams = this.dremioVars;
            let anyFilterApplied = false;

            Object.entries(params).forEach(([name, paramVal]) => {
                if (dremioParams.includes(name)) {
                    if (paramVal !== null) {
                        const filter = this.createDremioFilter(name, paramVal);
                        query = Query.queryInsertUpdateFilter(query, filter);
                    } else {
                        query = Query.queryRemoveFilter(query, name);
                    }
                    anyFilterApplied = isEqual(query, originalQuery) == null;
                }
            });

            return anyFilterApplied;
        },
        /**
         * Creates a new dremio query filter
         *
         * @param {string} name         metric/dimension/field name
         * @param {string|string[]} value  value
         * @return {Record<string, any>} filter
         */
        createDremioFilter(name, value) {
            const isArray = Array.isArray(value);
            return Query.createFilter({
                name,
                type: isArray ? Query.FILTER_TYPE.IN : Query.FILTER_TYPE.EQ,
                value: [value].flat()
            });
        },
        setDremioVars() {
            const { fields, metrics, dimensions } = this.queryHelper.reduce(
                (obj, { query, dimensionList }) => {
                    obj.fields = [...obj.fields, ...Query.queryFieldNames(query)];
                    obj.metrics = [...obj.metrics, ...Query.queryMetricNames(query)];
                    obj.dimensions = [...obj.dimensions, ...Object.keys(dimensionList)];
                    return obj;
                },
                { fields: [], metrics: [], dimensions: [] }
            );

            const dremioParams = [...fields, ...metrics, ...dimensions];
            const deletingVars = this.dremioVars.filter((name) => dremioParams.includes(name) === false);
            const addingVars = dremioParams.filter((name) => this.dremioVars.includes(name) === false);

            deletingVars.forEach((name) => this.$delete(this.descriptor.vars, name));
            addingVars.forEach((name) => this.$set(this.descriptor.vars, name, { description: name }));

            this.dremioVars = dremioParams;
        },
        /**
         * Store state watcher handler
         * @param {object} state
         */
        storeStateWatcher(state) {
            if (this.queryHelper.length === 0) {
                return;
            }

            const filterValues = [];

            Object.entries(state).forEach(([name, paramValue]) => {
                if (this.dimensions.includes(name)) {
                    filterValues.push({ [name]: paramValue });
                    return;
                }
                if (paramValue == null) {
                    this.queryHelper.forEach((helper) => {
                        helper.query = Query.queryRemoveFilter(helper.query, name);
                    });

                    return;
                }

                const value = [paramValue].flat();
                const type = Query.FILTER_TYPE.IN;

                const filter = Query.createFilter({ name, type, value });
                this.queryHelper.forEach((helper) => {
                    helper.query = Query.queryInsertUpdateFilter(helper.query, filter);
                });
            });
        }
    }
};

export { Query, SDK, Dremio, DremioMixin };
