import dayjs from 'dayjs';
import isLeapYear from 'dayjs/plugin/isLeapYear';
import isoWeek from 'dayjs/plugin/isoWeek';
import utc from 'dayjs/plugin/utc';
import timezone from 'dayjs/plugin/timezone';
import weekOfYear from 'dayjs/plugin/weekOfYear';
import isoWeeksInYear from 'dayjs/plugin/isoWeeksInYear';
import minMax from 'dayjs/plugin/minMax';
import localeData from 'dayjs/plugin/localeData';
import dayOfYear from 'dayjs/plugin/dayOfYear';
import isBetween from 'dayjs/plugin/isBetween';
import isSameOrAfter from 'dayjs/plugin/isSameOrAfter';
import isSameOrBefore from 'dayjs/plugin/isSameOrBefore';
import { DEFAULT_DATE_FORMAT, HOUR, MICROTIME_MINUTE, SHORT_DATE_FORMAT, WEEK, WEEK_DATE_FORMAT } from './constants';
import { WeekNames } from '../config';

dayjs.extend(utc);
dayjs.extend(timezone);
dayjs.extend(minMax);
dayjs.extend(localeData);
dayjs.extend(isoWeek);
dayjs.extend(isoWeeksInYear);
dayjs.extend(isLeapYear);
dayjs.extend(weekOfYear);
dayjs.extend(dayOfYear);
dayjs.extend(isBetween);
dayjs.extend(isBetween);
dayjs.extend(isSameOrAfter);
dayjs.extend(isSameOrBefore);

export const getWeekNumber = (date) => dayjs(date).isoWeek();
export const formatDate = (date, format = DEFAULT_DATE_FORMAT) => dayjs(date).format(format);
export const resolveDayjsDate = (date) => (date != null ? dayjs(`${date}T00:00:00Z`) : date);
export const getDaysInMonth = ({ month, year }) => dayjs(`${year}-${month}-1`, SHORT_DATE_FORMAT).daysInMonth();
export const getWeeksInYear = (year) => dayjs(`${year}-01-01`, DEFAULT_DATE_FORMAT).isoWeeksInYear();
export const getCurrentMonth = (date) => dayjs(date).month();
export const getDayOfYear = (date) => dayjs(date).dayOfYear();
export const getNextDate = ({ date, format, value }) => dayjs(date).add(value, format).format(DEFAULT_DATE_FORMAT);
export const getCurrentWeekInYear = () => dayjs().isoWeek();
export const getDaysInYear = (year) => (dayjs(String(year)).isLeapYear() ? 366 : 365);

export const buildWeekDates = ({ week, year, format }) => ({
    dateFrom: dayjs().year(year).isoWeek(week).startOf('isoWeek').format(format),
    dateEnd: dayjs().year(year).isoWeek(week).endOf('isoWeek').format(format)
});

export const getCurrentWeek = ({ year, week }) => {
    const { dateFrom, dateEnd } = buildWeekDates({ year, week, format: DEFAULT_DATE_FORMAT });
    return {
        startOf: dateFrom,
        endOf: dateEnd
    };
};

export const getAllDaysUpToMonth = ({ date, year }) => {
    let day = 0;
    for (let month = 1; month <= date.getMonth(); month++) {
        day += getDaysInMonth({ month, year });
    }
    day += date.getDate();
    return day;
};

export const isCurrentDay = ({ day, filterSettings, isMonthFormat, isWeekFormat }) => {
    const currentDay = dayjs().format(DEFAULT_DATE_FORMAT);
    const { year, month, week } = filterSettings;
    if (isMonthFormat) {
        return currentDay === dayjs(`${year}-${month + 1}-${day}`, SHORT_DATE_FORMAT).format(DEFAULT_DATE_FORMAT);
    }

    if (isWeekFormat) {
        return (
            currentDay ===
            dayjs().year(year).isoWeek(week).startOf('isoWeek').add(day, 'day').format(DEFAULT_DATE_FORMAT)
        );
    }

    return false;
};

export const isInRangeTableFilters = ({
    dateFrom,
    dateEnd,
    year,
    month,
    week,
    isYearFilterFormat,
    isMonthFilterFormat
}) => {
    const isInYearRange = year >= dayjs(dateFrom).year() && year <= dayjs(dateEnd).year();
    const currentDate = dayjs().year(year).month(month);
    if (isYearFilterFormat) {
        if (dateEnd === null) {
            return currentDate.isSameOrAfter(dateFrom, 'Y');
        }

        if (dateFrom === null) {
            return currentDate.isSameOrBefore(dateFrom, 'Y');
        }

        return isInYearRange;
    }

    if (isMonthFilterFormat) {
        if (!isInYearRange) {
            return false;
        }

        if (dateEnd === null) {
            return currentDate.isSameOrAfter(dateFrom, 'M');
        }

        if (dateFrom === null) {
            return currentDate.isSameOrBefore(dateFrom, 'M');
        }

        return currentDate.isBetween(dayjs(dateFrom), dayjs(dateEnd), 'month', '[]');
    }

    if (!isInYearRange) {
        return false;
    }

    const currentWeekDate = currentDate.isoWeek(week);
    if (dateEnd === null) {
        return currentWeekDate.isSameOrAfter(dateFrom, 'isoWeek');
    }

    if (dateFrom === null) {
        return currentWeekDate.isSameOrBefore(dateFrom, 'isoWeek');
    }

    return currentWeekDate.isBetween(dayjs(dateFrom), dayjs(dateEnd), 'isoWeek', '[]');
};

export const buildWeeksByYear = (year) => {
    const weeks = [];
    for (let i = 1; i <= getWeeksInYear(year); i++) {
        const { dateFrom, dateEnd } = buildWeekDates({ year, week: i, format: DEFAULT_DATE_FORMAT });
        weeks.push({
            id: i,
            startOf: dateFrom,
            endOf: dateEnd,
            title: `${dayjs(dateFrom).format(WEEK_DATE_FORMAT)}-${dayjs(dateEnd).format(WEEK_DATE_FORMAT)}`
        });
    }
    return weeks;
};
export const buildWeeksOptions = (year) =>
    buildWeeksByYear(year)
        .map(({ id, title }) => ({ value: id, label: title }))
        .reverse();

export const buildWeeksPanelOptions = (year) => {
    const weeks = [];
    for (let i = 1; i <= getWeeksInYear(year); i++) {
        const { dateFrom, dateEnd } = buildWeekDates({ year, week: i, format: DEFAULT_DATE_FORMAT });
        weeks.push({
            value: i,
            label: `${dayjs(dateFrom).format(WEEK_DATE_FORMAT)}-${dayjs(dateEnd).format(WEEK_DATE_FORMAT)}`
        });
    }
    return weeks;
};

export const resolveMinMaxValues = ({ min, max }) => ({
    min,
    max,
    minWeek: 1,
    maxWeek: dayjs(max).isoWeeksInYear(),
    minYear: dayjs(min).year(),
    maxYear: dayjs(max).year(),
    minMonth: dayjs(min).month(),
    maxMonth: dayjs(max).month()
});

export const getVacationDays = ({ worker, dateMetrics }) => {
    const { metricsEnd, metricsStart } = dateMetrics;
    const dateEnd = dayjs(worker[metricsEnd]);
    const dateStart = dayjs(worker[metricsStart]);
    const timeDiff = Math.abs(dateStart.diff(dateEnd));
    const diffDays = Math.ceil(timeDiff / (1000 * MICROTIME_MINUTE * HOUR));
    return diffDays + 1;
};

export const buildWeekTitles = ({ year, week }) => {
    let startOf = dayjs().year(year).isoWeek(week).startOf('isoWeek');
    const days = [`${startOf.format(WEEK_DATE_FORMAT)}, ${WeekNames[0]}`];
    for (let i = 1; i < WEEK; i++) {
        startOf = startOf.add(1, 'day');
        days.push(`${startOf.format(WEEK_DATE_FORMAT)}, ${WeekNames[i]}`);
    }
    return days;
};

export const getNumberDayInWeek = ({ year, day }) => dayjs().year(year).dayOfYear(day).day();
