import Leaflet from 'leaflet';
import { MapGeo, MapPanel, MapPoint, MapUtils } from '../utils/geojson';
import {
    DEFAULT_LAN_CENTER_RUSSIAN,
    DEFAULT_LON_CENTER_RUSSIAN,
    GeoLayersNames,
    MIN_RADIUS,
    RECALCULATION_ZOOM_POINT
} from '../utils/constants';

export default {
    data: () => ({
        layerLevel: 1
    }),
    methods: {
        getMunicipalityLayers({ name, metric }) {
            MapGeo.removePoints(this.points);
            this.points = [];

            const { canvas, Map, result, props } = this;
            const { sortMunicipality, dataByRegion = null } = this.mapGeo.findMOLayerAndDataByName({ name, metric });

            if (sortMunicipality == null) {
                return;
            }

            if (dataByRegion != null) {
                this.$storeCommit(dataByRegion);
            }

            const {
                border_mo: { geo, options }
            } = sortMunicipality;

            if (dataByRegion != null) {
                this.$storeCommit(dataByRegion);
            }

            options.onEachFeature = this.listenersGeoLayers;

            const geoLayers = Leaflet.geoJSON(geo, {
                ...options,
                renderer: canvas,
                pane: GeoLayersNames.BORDER_MO
            });

            this.geoLayersIds.border_mo = [geoLayers];

            this.controlPanel.updateLegendLayer(GeoLayersNames.BORDER_MO, metric);

            geoLayers.addTo(Map);
            if (this.infoPanel) {
                this.infoPanel.destroyPanel();
                if (props.infoPanel === true) {
                    this.infoPanel = new MapPanel({
                        Map,
                        demographyDataset: result,
                        createContentHandler: this.createContentHandler,
                        props,
                        defaultValue: props.infoPanelTexts.secondLayer,
                        layerName: GeoLayersNames.BORDER_MO
                    });
                }
            }
            this.layerLevel = 2;
        },
        getRegionLayer(canvas) {
            this.currentFeature = null;
            MapGeo.removePoints(this.points);
            this.filteredAdditionalDatasets = [];
            this.points = [];
            if (this.dataFiltrationMode.layer) {
                return;
            }

            const { markersGroups, Map, result, props } = this;
            if (markersGroups != null) {
                MapGeo.managingLayers({ Map, markersGroups, method: 'removeLayer' });
            }

            if (this.infoPanel) {
                if (this.infoPanel.panel) {
                    this.infoPanel.destroyPanel();
                }

                if (props.infoPanel === true) {
                    this.infoPanel = new MapPanel({
                        Map,
                        demographyDataset: result,
                        createContentHandler: this.createContentHandler,
                        props,
                        defaultValue: props.infoPanelTexts.firstLayer,
                        layerName: GeoLayersNames.RUSSIAN
                    });
                }
            }

            this.controlPanel.updateLegendLayer(GeoLayersNames.RUSSIAN);

            const [
                {
                    russian: { geo, options }
                }
            ] = this.mapGeo.dataJson.filter(({ name }) => name === 'Россия');

            options.onEachFeature = this.listenersGeoLayers;

            this.dataFiltrationMode.layer = Leaflet.geoJSON(geo, {
                ...options,
                renderer: canvas,
                pane: GeoLayersNames.RUSSIAN
            }).addTo(Map);

            this.layerLevel = 1;
        },
        applyGeoLayers(canvas) {
            const geoLayer = this.mapGeo.dataJson.find(({ name }) => name === 'Россия');
            if (geoLayer == null) {
                return;
            }

            const { geo, options } = geoLayer.russian;
            options.onEachFeature = this.listenersGeoLayers;
            const geoLayers = Leaflet.geoJSON(geo, {
                ...options,
                renderer: canvas,
                pane: 'russian'
            });
            this.geoLayersIds.russian = [geoLayers];
        },
        listenersGeoLayers(feature, layer) {
            let listeners = {
                click: this.zoomToFeature
            };

            if ([GeoLayersNames.BORDER_MO, GeoLayersNames.RUSSIAN].includes(layer.options.pane)) {
                listeners = {
                    ...listeners,
                    mouseover: this.highlightFeature,
                    mouseout: this.resetHighlightFeature
                };
            }

            layer.on(listeners);
        },
        zoomToFeature(e) {
            this.currentFeature = e;
            const { additionalLayers, isFiltrationMode } = this.props;
            if (!isFiltrationMode) {
                return;
            }

            const { metricJSON } = additionalLayers.find((layer) =>
                this.layerLevel === 1
                    ? String(layer.value) === String(GeoLayersNames.RUSSIAN)
                    : String(layer.value) === String(GeoLayersNames.BORDER_MO)
            );

            const { metricJSON: filterPointsMetricJSON } = additionalLayers.find(
                (layer) => String(layer.value) === String(GeoLayersNames.RUSSIAN)
            );

            const {
                properties: {
                    name: regionNameLowerCase,
                    NAME: regionNameUpperCase,
                    [metricJSON]: regionRussianMetric,
                    [filterPointsMetricJSON]: pointsMetric
                }
            } = e.target.feature;

            const regionName = regionNameLowerCase || regionNameUpperCase;
            if (regionName == null && regionRussianMetric == null) {
                return;
            }

            if (this.layerLevel === 2) {
                const dataByRegion = this.result.find(
                    (dremioRow) => String(dremioRow[metricJSON]) === String(regionRussianMetric)
                );
                if (dataByRegion != null) {
                    this.$storeCommit(dataByRegion);
                }
            }

            this.Map.fitBounds(e.target.getBounds());

            const {
                Map,
                canvas,
                additionalDremioDatasets,
                props: {
                    hasFilterPointsByRegion,
                    range,
                    clusterFilterMetric,
                    rangeMetric,
                    clusters: { active: clusterActive, ...polygonOptions }
                }
            } = this;

            if (this.markersGroups != null) {
                MapGeo.managingLayers({ Map, markersGroups: this.markersGroups, method: 'removeLayer' });
            }

            this.getMunicipalityLayers({ name: regionName, metric: regionRussianMetric });

            if (this.dataFiltrationMode.layer != null) {
                this.dataFiltrationMode.layer.remove();
                this.dataFiltrationMode.layer = null;
            }

            const { metricsLan, metricsLon, pointsDataset } = this.props;
            if ([metricsLan, metricsLon].includes(null) || pointsDataset == null || pointsDataset === 'main') {
                return;
            }

            const filteredDatasets = hasFilterPointsByRegion
                ? additionalDremioDatasets.filter(
                      ({ [filterPointsMetricJSON]: filterMetric }) =>
                          String(filterMetric) === String(regionRussianMetric) || filterMetric === pointsMetric
                  )
                : additionalDremioDatasets;
            this.filteredAdditionalDatasets = filteredDatasets;
            this.markersGroups = MapPoint.buildClusterGroup({
                points: filteredDatasets,
                styleRange: range,
                isFiltrationMode,
                clusterFilterMetric,
                polygonOptions
            });

            if (isFiltrationMode) {
                this.points = this.buildPoints({
                    canvas,
                    dataset: filteredDatasets,
                    markersGroups: this.markersGroups,
                    rangeMetric,
                    manageLayer: true
                });
            }

            const zoomLevel = this.Map.getZoom();
            this.mapGeo.hideRangeOfValues({
                zoomLevel,
                points: this.points,
                markersGroups: this.markersGroups,
                Map
            });
        },
        highlightFeature(e) {
            const layer = e.target;
            const {
                props: { defaultStyleInHover, infoPanel }
            } = this;
            layer.setStyle({ ...defaultStyleInHover });
            layer.bringToBack();

            if (this.infoPanel) {
                this.infoPanel.updatePanel({ layer: layer.feature.properties, hasInfoPanel: infoPanel });
            }
        },
        resetHighlightFeature(e) {
            const {
                geoLayers = null,
                dataFiltrationMode: { layer = null },
                props: { infoPanel }
            } = this;

            if (this.infoPanel) {
                this.infoPanel.updatePanel({ hasInfoPanel: infoPanel });
            }

            const borderMoLayer = this.geoLayersIds?.border_mo?.[0];

            [geoLayers, borderMoLayer, layer].forEach((defaultStyleLayer) => {
                if (defaultStyleLayer == null) {
                    return;
                }
                defaultStyleLayer.resetStyle(e.target);
            });
        },
        changingPointSize(zoomLevel) {
            const {
                points,
                props: { metricsSizeIcon, sizePoint }
            } = this;

            if (RECALCULATION_ZOOM_POINT > zoomLevel || metricsSizeIcon === '') {
                return;
            }

            points
                .filter((item) => item != null)
                .forEach((point) => {
                    const { radius } = point.options;
                    if (point.options?.img) {
                        return;
                    }

                    let minRadius = radius / zoomLevel;
                    if (minRadius > MIN_RADIUS) {
                        minRadius = MIN_RADIUS;
                    }

                    point.setRadius(minRadius < sizePoint ? sizePoint : minRadius);
                });
        },
        onZoomMapEnd() {
            const zoomLevel = this.Map.getZoom();
            if (this.mapGeo == null) {
                return;
            }

            this.mapGeo.hideRangeOfValues({
                zoomLevel,
                points: this.points,
                markersGroups: this.markersGroups,
                Map: this.Map
            });

            if (!this.props.isFiltrationMode) {
                this.mapGeo.hideLayersOnZoom(zoomLevel);
            }

            if (this.props.isFiltrationMode) {
                this.showRegionLayer(zoomLevel);
            }

            this.changingPointSize(zoomLevel);
        },
        showRegionLayer(zoomLevel) {
            const {
                canvas,
                geoLayersIds,
                props: {
                    showRegionRangeMin,
                    showRegionRangeMax,
                    zoom: { minZoomMap, maxZoomMap }
                }
            } = this;

            if (
                (showRegionRangeMin === '' || showRegionRangeMax === '') &&
                (zoomLevel < minZoomMap || zoomLevel > maxZoomMap)
            ) {
                return;
            }

            if (zoomLevel < showRegionRangeMin || zoomLevel > showRegionRangeMax) {
                return;
            }

            MapGeo.removeLayers(geoLayersIds);

            this.getRegionLayer(canvas);
        },
        createContentHandler(el, metric, options = null) {
            const contentMetricLayer = options || this.props[metric];

            if (!Array.isArray(contentMetricLayer.metrics)) {
                return '';
            }

            if (Array.isArray(el)) {
                return contentMetricLayer.metrics.reduce(
                    (acc, item) =>
                        `${acc} <div class="mar-bot-2" style="color:${item.color}; font-size: ${
                            item.fontSize
                        }; text-align: ${item.textAlign}">${item.prefix} ${MapUtils.formatMetric(
                            MapUtils.isIntegerNumber(Number(el[0][item.metric])) && item.useSum
                                ? this.getTotalValueByMetric(el, item.metric)
                                : el[0][item.metric],
                            item.format
                        )} ${item.postfix}</div>`,
                    ''
                );
            }

            // for only points mode
            return contentMetricLayer.metrics.reduce(
                (acc, item) =>
                    `${acc} <div class="mar-bot-2" style="color:${item.color}; font-size: ${
                        item.fontSize
                    }; text-align: ${item.textAlign}">${item.prefix} ${MapUtils.formatMetric(
                        MapUtils.isIntegerNumber(Number(el[item.metric])) ? Number(el[item.metric]) : el[item.metric],
                        item.format
                    )} ${item.postfix}</div>`,
                ''
            );
        },
        getTotalValueByMetric(el, metric) {
            return el.reduce((acc, { [metric]: value }) => acc + Number(value), 0);
        },
        getCenterMap() {
            const defaultCenterMaps = [DEFAULT_LAN_CENTER_RUSSIAN, DEFAULT_LON_CENTER_RUSSIAN];
            const {
                props: { centerLongitude, centerLatitude, metricsLan, metricsLon },
                result
            } = this;

            if (result == null) {
                return defaultCenterMaps;
            }

            const [firstElement = {}] = result;

            if (centerLongitude && centerLatitude) {
                return [centerLongitude, centerLatitude];
            }

            if (metricsLan in firstElement && metricsLon in firstElement) {
                return [firstElement[metricsLan], firstElement[metricsLon]];
            }

            return defaultCenterMaps;
        },
        /**
         * Передача данных в шину, открытие попапа по клику на точку
         * @param {boolean} isModal
         * @param {string} content
         * @param {string} popupUrl
         * @param {object} data
         * @param {boolean} isStore
         */
        listenerPoint({ isModal = false, content = '', popupUrl = null, data, isStore = false }) {
            if (isModal) {
                this.isVisiblePopup = true;
                this.contentPopup = content;
            }

            if (popupUrl != null) {
                this.$storeCommit(data);
                this.$routeNavigate({ path: popupUrl });
                return;
            }

            if (isStore) {
                this.$storeCommit(data);
            }
        }
    }
};
