import Leaflet from 'leaflet';
import { formatNumber } from '@goodt-common/utils-number';
import { GeoLayersNames } from '../constants';

export class MapControl {
    legend;

    constructor(options) {
        this.Map = options.Map;
        this.additionalLayers = options.additionalLayers;
        this.controlScale = options.controlScale;
        this.outputZoom = options.outputZoom;
        this.result = options.result;
        this.centerLongitude = options.centerLongitude;
        this.centerLatitude = options.centerLatitude;
        this.zoomFlightOfTheBumblebee = options.zoomFlightOfTheBumblebee;
        this.gradient = options.gradient;
        this.flyToPointByCoordinates = options.flyToPointByCoordinates;
        this.isVisibleControlHome = options.isVisibleControlHome;
        this.infoZoomLevel = null;
        this.home = null;
        this.scale = null;

        this.initPanels();
    }

    showInfoZoom() {
        const { outputZoom: isOutputZoom } = this;

        if (isOutputZoom === false) {
            return;
        }

        this.infoZoomLevel = Leaflet.control({ position: 'topright' });
        this.infoZoomLevel.onAdd = () => {
            const container = Leaflet.DomUtil.create('div');
            const gauge = Leaflet.DomUtil.create('div');
            container.style.padding = '5px';
            container.style.background = 'rgba(255,255,255,1)';
            gauge.innerHTML = `Zoom level: ${this.Map.getZoom()}`;

            this.Map.on('zoomstart zoom zoomend', () => {
                gauge.innerHTML = `Zoom level: ${this.Map.getZoom()}`;
            });
            container.appendChild(gauge);

            return container;
        };

        this.infoZoomLevel.addTo(this.Map);
    }

    /**
     * Get style string from object for a dom node
     *
     * @param {object} style
     */
    getStyleForTitle({ fontFamily, textAlign, marginBottom, fontSize, color }) {
        return `font-family:${fontFamily};text-align:${textAlign};margin-bottom:${marginBottom};color:${color};font-size:${fontSize}`;
    }

    showLegendForAdditionalLayers(GeoLayerName, options = {}) {
        const {
            gradient: {
                options: { gradientColorUse, gradientFormat },
                data: gradientLayersColors
            },
            result,
            additionalLayers
        } = this;

        const { active: isActive, component, range } = additionalLayers.find((item) => item.value === GeoLayerName);
        if (component.options.activeLegend === false) {
            return;
        }

        const gradientLayerColor = gradientLayersColors.find(({ layer }) => layer === GeoLayerName);

        const { dataGradients = [] } = gradientLayerColor;
        let { gradienMax, gradienMin } = gradientLayerColor;

        const { metricsFilter = null, region = null, metricsValue = null } = options;
        if (metricsFilter != null && region != null) {
            const regions = result
                // eslint-disable-next-line
                .filter((item) => item[metricsFilter] == region)
                .map(({ [metricsValue]: value }) => Number(value));

            gradienMax = Math.max.apply(null, regions);
            gradienMin = Math.min.apply(null, regions);
        }

        if (!isActive) {
            return;
        }

        this.legend = Leaflet.control({ position: 'bottomright' });
        this.legend.onAdd = () => {
            const div = Leaflet.DomUtil.create('div', 'info legend');
            const { data: dataRange, title, styleTitle, style = 'margin-right:0', format } = range;
            const styleForTitle = this.getStyleForTitle(styleTitle);

            const labels = [`<div class="h4" style="${styleForTitle}">${title}</div>`];

            if (gradientColorUse === true && dataGradients.length > 0) {
                div.innerHTML += labels.push(`<div class="d-flex flex-h-space-between flex-v-center">
                    <span class="small">${formatNumber(gradienMin, gradientFormat)}</span>
                    <span class="small">${formatNumber(gradienMax, gradientFormat)}</span>
                </div>`);

                dataGradients.forEach((color) => {
                    div.innerHTML += labels.push(`<i class="info-color" style="background:#${color};${style}"></i>`);
                });
            }

            if (gradientColorUse === false) {
                dataRange.forEach((rangeData) => {
                    const { fillColor: color, to, from } = rangeData;
                    div.innerHTML += labels.push(
                        `<div class="info-item"></div><i class="info-color" style="background:${color}"></i>${formatNumber(
                            from,
                            format
                        )} - ${formatNumber(to, format)}</div>`
                    );
                });
            }

            div.innerHTML = labels.join('');
            return div;
        };
        this.legend.addTo(this.Map);
    }

    updateLegendLayer(geoLayerName, region = null) {
        if (this.legend) {
            this.legend.remove();
        }

        let additionalParams = {};

        const {
            range: { metricsFilter = null, metricsValue }
        } = this.additionalLayers.find((item) => item.value === GeoLayersNames.RUSSIAN);

        if (region != null && metricsFilter != null) {
            additionalParams = {
                metricsFilter,
                metricsValue,
                region
            };
        }

        this.showLegendForAdditionalLayers(geoLayerName, additionalParams);
    }

    showControlScale() {
        const { active: isControlScaleActive } = this.controlScale;

        if (isControlScaleActive) {
            this.scale = Leaflet.control.scale(this.controlScale).addTo(this.Map);
        }
    }

    showControlHome() {
        const { centerLongitude, centerLatitude, zoomFlightOfTheBumblebee, isVisibleControlHome } = this;
        if (isVisibleControlHome === false) {
            return;
        }

        this.home = Leaflet.control({
            position: 'bottomleft'
        });

        this.home.onAdd = (map) => {
            this._div = Leaflet.DomUtil.create('div', 'btn btn-small btn-primary mdi mdi-home map-search-icon');
            this._div.addEventListener('click', (event) => {
                this.flyToPointByCoordinates([centerLongitude, centerLatitude], zoomFlightOfTheBumblebee);
            });
            return this._div;
        };

        this.home.addTo(this.Map);
    }

    initPanels() {
        this.showInfoZoom();
        this.showLegendForAdditionalLayers(GeoLayersNames.RUSSIAN);
        this.showControlScale();
        this.showControlHome();
    }

    destroyPanels() {
        ['infoZoomLevel', 'legend', 'scale', 'home'].forEach(
            (control) => this[control] != null && this[control].remove()
        );
    }
}
