import { convertRgbaToHex } from '@goodt-common/utils';
import Leaflet from 'leaflet';
import { difference, cloneDeep, isEqual, minBy, toNumber, maxBy, filter, inRange } from 'lodash';
import { GeoLayersNames, MAX_COLOR_GRADIENT_DEFAULT, MIN_COLOR_GRADIENT_DEFAULT, FIX_MIN_RANGE } from '../constants';

export class MapGeo {
    gradientLayerColors = null;

    constructor(Map, result, data) {
        this.Map = Map;
        this.result = result;
        this.gradient = data.gradient;
        this.additionalLayers = data.additionalLayers;
        this.geoLayersIds = data.geoLayersIds;
        this.clusters = data.clusters;
        this.markersGroups = data.markersGroups;
        this.range = data.range;
        this.points = data.points;
        this.cloneRange = cloneDeep(data.range);
        this.dataGradients = this.buildGradientForLayers();
        this.gradientLayerColors = this.getGradientForLayer();
        this.dataGeoLayers = this.addingOptionsLayers(data.geoLayersJSON || []);

        this.previousStateOfLayers = null;
    }

    get dataJson() {
        return this.dataGeoLayers;
    }

    /**
     * Replacing old layers with new ones when changing data
     * @param borderMo
     * @returns {{border_mo: {geo: *, options: {style: *}}, name: *}[]}
     */
    replaceMunicipalityLayer({ borderMo, canvas, handler }) {
        const { dataJson } = this;

        return borderMo.map(({ options: { region = null } }) => {
            const {
                border_mo: { options, geo }
            } = dataJson.find(({ name }) => name === region);

            options.onEachFeature = handler;

            return Leaflet.geoJSON(geo, {
                ...options,
                renderer: canvas,
                pane: GeoLayersNames.BORDER_MO
            });
        });
    }

    /**
     * Adding options to a layer
     * @param {array} geoLayers
     * @returns {{border_mo: {geo: *, options: {style: *}}, name: *}[]}
     */
    addingOptionsLayers(geoLayers) {
        return geoLayers.map((layer) => {
            const { name: layerName } = layer;
            if (layerName === 'Россия') {
                return this.applyStyleLayer({
                    region: 'Россия',
                    geoJsonData: layer.russian.geo,
                    method: 'styleRegionHandler',
                    layer: 'russian'
                });
            }

            return this.applyStyleLayer({
                region: layerName,
                geoJsonData: layer.border_mo.geo
            });
        });
    }

    /**
     * Defining Styles
     * @param {string} region
     * @param {string} geoJsonData
     * @param {string} method
     * @param {string} layer
     * @returns {{border_mo: {geo: *, options: {style: *}}, name: *}}
     */
    applyStyleLayer({ region, geoJsonData, method = 'styleMoHandler', layer = 'border_mo' }) {
        return {
            name: region,
            [layer]: {
                geo: geoJsonData,
                options: {
                    region,
                    style: this[method].bind(this)
                }
            }
        };
    }

    /**
     * Раскраска регионов России на основании одно из выбранных числовых диапазонов
     * @param feature
     * @returns {{}|{fillColor, color, fillOpacity, opacity, weight}}
     */
    styleRegionHandler(feature) {
        const { properties } = feature;
        const {
            result,
            additionalLayers,
            gradient: { gradientColorUse: isGradientColorUse }
        } = this;
        const layer = additionalLayers.find((item) => item.value === GeoLayersNames.RUSSIAN);

        if (!('component' in layer)) {
            return {};
        }

        const {
            component: { options: defaultOptions },
            metricJSON
        } = layer;

        if (!('range' in layer)) {
            return defaultOptions;
        }

        const { metricsFilter = null, metricsValue = null, data: rangeStyles } = layer.range;

        if (metricsFilter == null && metricsValue == null) {
            return defaultOptions;
        }

        // eslint-disable-next-line eqeqeq
        const foundElem = result.find((item) => item[metricsFilter] == properties[metricJSON]);

        if (foundElem == null || !(metricsValue in foundElem)) {
            return defaultOptions;
        }

        const count = foundElem[metricsValue];

        if (isGradientColorUse) {
            const gradientColor = this.getColorFromGradient({
                item: foundElem,
                key: metricsValue,
                layerName: GeoLayersNames.RUSSIAN
            });

            return {
                ...defaultOptions,
                fillColor: gradientColor
            };
        }

        const foundRangeItem = [...rangeStyles].reverse().find(({ from, to }) => count >= from && count <= to);

        if (foundRangeItem != null) {
            const { color, fillColor, opacity, fillOpacity } = foundRangeItem;

            return {
                color,
                fillColor,
                opacity,
                fillOpacity,
                weight: defaultOptions.weight
            };
        }

        return defaultOptions;
    }

    /**
     * Раскраска муниципальных образований России на основании одно из выбранных числовых диапазонов
     * @param feature
     * @returns {{fillColor, color, fillOpacity, opacity, weight}|*}
     */
    styleMoHandler(feature) {
        const { properties } = feature;
        const {
            result,
            additionalLayers,
            gradient: { gradientColorUse: isGradientColorUse }
        } = this;
        const metrics = additionalLayers.find((item) => item.value === GeoLayersNames.BORDER_MO);
        const { options: defaultOptions } = metrics.component;
        const { metricsFilter, metricsValue, data: rangeStyles } = metrics.range;
        const { metricJSON } = metrics;

        const count = result
            // eslint-disable-next-line eqeqeq
            .filter((item) => item[metricsFilter] == properties[metricJSON])
            .reduce((acc, item) => acc + Number(item[metricsValue]), 0);

        if (isGradientColorUse) {
            const gradientColor = this.getColorFromGradient({
                count,
                key: metricsValue,
                layerName: GeoLayersNames.BORDER_MO
            });

            return {
                ...defaultOptions,
                fillColor: gradientColor
            };
        }

        const foundRangeItem = [...rangeStyles].reverse().find(({ from, to }) => count >= from && count <= to);

        if (foundRangeItem == null) {
            return defaultOptions;
        }

        const { color, fillColor, opacity, fillOpacity } = foundRangeItem;

        return {
            color,
            fillColor,
            opacity,
            fillOpacity,
            weight: defaultOptions.weight
        };
    }

    styleRoadHandler() {
        const {
            component: { options }
        } = this.additionalLayers.find((item) => item.value === GeoLayersNames.ROAD);

        return options;
    }

    getSortedKeys(item) {
        const keys = Array.isArray(item) ? item : Object.keys(item);
        const additionalSortAllKeys = this.additionalLayers.map(({ value }) => value);
        const differenceKeys = difference(additionalSortAllKeys, keys);
        const sortKeys = additionalSortAllKeys.filter((key) => differenceKeys.includes(key) === false);
        return sortKeys;
    }

    hideLayersOnZoom(zoomLevel) {
        const activeGeoLayers = this.additionalLayers
            .filter((layer) => layer.hideLayerOnZoom <= zoomLevel && layer.active)
            .map((layer) => layer.value);

        if (isEqual(activeGeoLayers, this.previousStateOfLayers)) {
            return;
        }

        this.previousStateOfLayers = cloneDeep(activeGeoLayers);

        MapGeo.removeLayers(this.geoLayersIds);

        this.getSortedKeys(activeGeoLayers).forEach((layerSort) => {
            if (layerSort in this.geoLayersIds) {
                this.geoLayersIds[layerSort].forEach((item) => item.addTo(this.Map));
            }
        });
    }

    hideRangeOfValues({ zoomLevel, points, markersGroups, Map }) {
        const { active: clusterActive } = this.clusters;
        if (points.length === 0) {
            return;
        }

        if (this.range.length === 0) {
            if (markersGroups != null) {
                MapGeo.managingLayers({ Map, markersGroups, method: 'addLayer' });
            }
            return;
        }

        this.range.forEach((line, idx) => {
            const { hideOnScale, color } = line;
            if (hideOnScale == null) {
                return;
            }

            this.managingVisibilityOfValueRanges({
                color,
                vector: zoomLevel < hideOnScale,
                cluster: clusterActive,
                line: this.cloneRange[idx],
                points,
                markersGroups,
                Map
            });
        });
    }

    managingVisibilityOfValueRanges({ color, vector, cluster, line, points, markersGroups, Map }) {
        line.vectorScroll = vector;
        if (cluster && color in markersGroups) {
            MapGeo.managingLayers({
                isOneGroup: true,
                markersGroups: markersGroups[color],
                method: vector ? 'removeLayer' : 'addLayer',
                Map
            });
            return;
        }

        points.forEach((point) => {
            let {
                options: { color: colorPoint }
            } = point;

            if (colorPoint == null) {
                const {
                    icon: {
                        options: { iconColor }
                    }
                } = point.options;

                colorPoint = iconColor;
            }

            if (colorPoint !== color) {
                return;
            }

            const method = vector && colorPoint === color ? 'removeLayer' : 'addLayer';
            Map[method](point);
        });
    }

    buildGradientForLayers() {
        const { additionalLayers } = this;
        return additionalLayers
            .filter(({ value }) => [GeoLayersNames.RUSSIAN, GeoLayersNames.BORDER_MO].includes(value))
            .map(({ range: { metricsValue = null }, value }) => ({
                layer: value,
                gradient: this.creatingGradientPalette(metricsValue)
            }));
    }

    creatingGradientPalette(metric) {
        const gradObj = {
            gradientLineCss: '',
            dataGradients: [],
            gradientDataInRanges: [],
            gradienMax: '',
            gradienMin: ''
        };
        const { gradient, result } = this;
        const { gradientColorMax, gradientColorMin, gradientSmoothing } = gradient;
        const data = cloneDeep(result).sort((a, b) => a[metric] - b[metric]);
        if (data.length === 0 || metric === 0 || metric === null) {
            return gradObj;
        }

        const gradientMinValue = parseFloat(minBy(data, (dataItem) => toNumber(dataItem[metric]))[metric]);
        const gradientMaxValue = parseFloat(maxBy(data, (dataItem) => toNumber(dataItem[metric]))[metric]);

        const gradientMinColor = convertRgbaToHex(gradientColorMin) || convertRgbaToHex(MIN_COLOR_GRADIENT_DEFAULT);
        const gradientMaxColor = convertRgbaToHex(gradientColorMax) || convertRgbaToHex(MAX_COLOR_GRADIENT_DEFAULT);
        const dataInRange = filter(data, (dataItem) =>
            inRange(dataItem[metric], gradientMinValue, gradientMaxValue + FIX_MIN_RANGE)
        );
        const gradSmooth = gradientSmoothing || 1;
        const gradientDataColors = this.generateGradient(gradientMinColor, gradientMaxColor, gradSmooth);
        const gradientLineCss = gradientDataColors.reduce(
            (acc, item, index, array) => (acc + index !== array.length - 1 ? `#${item},` : `#${item}`),
            ''
        );

        gradObj.gradientLineCss = `linear-gradient(90deg,${gradientLineCss})`;
        gradObj.dataGradients = gradientDataColors;
        gradObj.gradientDataInRanges = dataInRange;
        gradObj.gradienMax = gradientMaxValue;
        gradObj.gradienMin = gradientMinValue;
        return gradObj;
    }

    hexToRGB(hex, alpha) {
        const read = parseInt(hex.slice(1, 3), 16);
        const green = parseInt(hex.slice(3, 5), 16);
        const blue = parseInt(hex.slice(5, 7), 16);

        if (alpha != null) {
            return `rgb(${read}, ${green}, ${blue}, ${alpha})`;
        }

        return `rgb(${read}, ${green}, ${blue})`;
    }

    getColorFromGradient({ item = null, key, layerName, count = null }) {
        const { gradient: gradientLayer = null } = this.dataGradients.find(({ layer }) => layer === layerName);

        if (gradientLayer == null) {
            return null;
        }

        const { dataGradients, gradientDataInRanges } = gradientLayer;
        // eslint-disable-next-line eqeqeq
        const itemIndex = gradientDataInRanges.findIndex((x) => x[key] == (item != null ? item[key] : count));

        let resultColor = null;
        if (itemIndex !== -1 && dataGradients.length > 0) {
            const colorIndex = Math.floor(itemIndex / (gradientDataInRanges.length / dataGradients.length));
            resultColor = this.hexToRGB(`#${dataGradients[colorIndex]}`, 1.0);
        }

        return resultColor;
    }

    generateGradient(min, max, smth) {
        const hex = (color) => {
            const str = '0123456789abcdef';
            // eslint-disable-next-line
            let ii = parseInt(color);
            if (ii === 0 || Number.isNaN(Number(color))) {
                return '00';
            }
            ii = Math.round(Math.min(Math.max(0, ii), 255));
            return str.charAt((ii - (ii % 16)) / 16) + str.charAt(ii % 16);
        };

        const convertToHex = ([red, green, blue]) => hex(red) + hex(green) + hex(blue);

        const trim = (str) => (str.charAt(0) === '#' ? str.substring(1, 7) : str);

        const convertToRGB = (hexVal) => {
            const colors = [];
            colors[0] = parseInt(trim(hexVal).substring(0, 2), 16);
            colors[1] = parseInt(trim(hexVal).substring(2, 4), 16);
            colors[2] = parseInt(trim(hexVal).substring(4, 6), 16);
            return colors;
        };

        const generateColor = (colorStart, colorEnd, colorCount) => {
            const start = convertToRGB(colorStart);
            const end = convertToRGB(colorEnd);
            const len = colorCount;
            let alpha = 0.0;
            const listOfColors = [];
            for (let i = 0; i < len; i++) {
                const colors = [];
                alpha += 1.0 / len;
                colors[0] = start[0] * alpha + (1 - alpha) * end[0];
                colors[1] = start[1] * alpha + (1 - alpha) * end[1];
                colors[2] = start[2] * alpha + (1 - alpha) * end[2];
                listOfColors.push(convertToHex(colors));
            }
            return listOfColors;
        };

        return generateColor(min, max, smth);
    }

    findMOLayerAndDataByName({ name, metric: metricJSON }) {
        const { additionalLayers, dataJson, result } = this;
        const {
            range: { metricsFilter: metricFilterRussian }
        } = additionalLayers.find((item) => item.value === GeoLayersNames.RUSSIAN);

        const { metricJSON: metricsBorderMoJSON } = additionalLayers.find(
            (item) => item.value === GeoLayersNames.BORDER_MO
        );

        const sortMunicipality = dataJson.find((layer) => {
            const { border_mo: borderMo = null } = layer;

            if (borderMo == null) {
                return null;
            }

            const {
                geo: {
                    features: [feature = null]
                }
            } = borderMo;

            if (feature == null) {
                return null;
            }

            const {
                properties: {
                    addr_regio: region = null,
                    state_name: stateName = null,
                    [metricsBorderMoJSON]: jsonValue = null
                }
            } = feature;

            return (region || stateName || String(jsonValue)) === name || String(jsonValue) === metricJSON;
        });
        // eslint-disable-next-line eqeqeq
        const dataByRegion = result.find((item) => item[metricFilterRussian] == metricJSON);

        return {
            sortMunicipality,
            dataByRegion
        };
    }

    getGradientForLayer() {
        return this.dataGradients.map(({ layer, gradient: { dataGradients, gradienMax, gradienMin } }) => ({
            layer,
            dataGradients,
            gradienMax,
            gradienMin
        }));
    }

    /**
     * remove layer from  map
     * @param layers
     */
    static removeLayers(layers) {
        Object.values(layers).forEach((layer) => {
            layer.forEach((item) => item.remove());
        });
    }

    /**
     * remove points from map
     * @param points
     */
    static removePoints(points) {
        points.forEach((point) => point.remove());
    }

    /**
     * Добавление или удаление сгруппированных слоев или слоя с карты
     *
     * @param {string} method
     * @param {object} Map
     * @param {object} markersGroups
     * @param {boolean} isOneGroup
     */

    static managingLayers({ method, Map, markersGroups, isOneGroup = false }) {
        if (isOneGroup) {
            Map[method](markersGroups);
            return;
        }

        Object.values(markersGroups).forEach((marker) => {
            Map[method](marker);
        });
    }
}
