import Leaflet from 'leaflet';

export class MapPanel {
    panel;

    constructor({ demographyDataset, Map, layerName, props, createContentHandler, defaultValue = 'Выберите область' }) {
        this.demographyDataset = demographyDataset;
        this.Map = Map;
        this.createContentHandler = createContentHandler;
        this.defaultValue = defaultValue;
        this.props = props;
        this.layerOption = this.findLayerOption({ layerName, panelTemplates: props.panelTemplates });

        this.initPanel();
    }

    findLayerOption({ panelTemplates, layerName }) {
        return panelTemplates.find(({ layers }) => layers === layerName);
    }

    onAdd() {
        this._div = Leaflet.DomUtil.create('div', 'info');
        this.update();
        return this._div;
    }

    update(data) {
        const { layerOption, createContentHandler, demographyDataset, defaultValue, props } = this;

        this._div.innerHTML = `${
            data != null
                ? this.getInfoData({
                      ...data,
                      props,
                      layerOption,
                      demographyDataset,
                      defaultValue,
                      createContentHandler
                  })
                : defaultValue
        }`;
    }

    getInfoData(data = null) {
        if (data == null) {
            return 'Нет данных';
        }

        const { layerOption = {}, demographyDataset, createContentHandler } = this;
        const { metricJSON = null, metricFilter = null } = layerOption;

        if ([metricJSON, metricFilter].includes(null)) {
            return 'Нет данных';
        }

        const metric = data[metricJSON];
        // eslint-disable-next-line eqeqeq
        const regionRows = demographyDataset.filter((elem) => elem[metricFilter] === String(metric));
        if (regionRows.length === 0) {
            return 'Нет данных';
        }

        return createContentHandler(regionRows, 'panelTemplates', layerOption);
    }

    initPanel() {
        if (this.panel != null) {
            return;
        }

        this.panel = Leaflet.control();
        this.panel.onAdd = this.onAdd;
        this.panel.update = this.update;
        this.panel.getInfoData = this.getInfoData.bind(this);
        this.panel.defaultValue = this.defaultValue;

        this.panel.addTo(this.Map);
    }

    /**
     * Обновление информации на инфо-панели
     * @param {object} layer
     * @param {boolean} hasInfoPanel
     */
    updatePanel({ layer = null, hasInfoPanel }) {
        if (!hasInfoPanel || this.panel == null) {
            return;
        }

        this.panel.update(layer);
    }

    destroyPanel() {
        if (this.panel == null) {
            return;
        }

        this.panel.remove();
        this.panel = null;
    }
}
