import Leaflet from 'leaflet';
import { FILL_OPACITY_DEFAULT, TooltipViewNames } from '../constants';

export class MapPoint {
    /**
     * Добавление маркера, точки на карту, либо в кластер по (по заданому из метрики признаку)
     *
     * @param {object} point
     * @param {object} Map
     * @param {boolean} isFiltrationMode
     * @param clusterActive
     * @param {string} clusterFilterMetric
     * @param {string} color
     * @param {object} markersGroups
     * @returns {object}
     */
    static addPointToMap(point, Map, { isFiltrationMode, clusterActive, clusterFilterMetric, color, markersGroups }) {
        if (isFiltrationMode && clusterActive && clusterFilterMetric in markersGroups) {
            return markersGroups[clusterFilterMetric].addLayer(point);
        }

        if (markersGroups && color in markersGroups && clusterActive) {
            return markersGroups[color].addLayer(point);
        }

        if (clusterActive && Object.keys(markersGroups).length === 1) {
            return markersGroups.defaultCluster.addLayer(point);
        }

        return point.addTo(Map);
    }

    /**
     * Группировки маркеров, точек по териториальному либо по цветовому диапазону.
     *
     * @param {object[]} points
     * @param {string[]} styleRange
     * @param {boolean} isFiltrationMode
     * @param {string} clusterFilterMetric
     * @param {object} polygonOptions - настройка стилей для агрегированной области
     * @returns {object}
     */
    static buildClusterGroup({ points, styleRange, isFiltrationMode, clusterFilterMetric, polygonOptions }) {
        const iconCreateFunction = this.iconCreateFunction(polygonOptions);

        const defaultCluster = {
            defaultCluster: Leaflet.markerClusterGroup({ iconCreateFunction, polygonOptions })
        };

        if (styleRange.length > 0) {
            return styleRange.reduce(
                (acc, { color }) => ({
                    ...acc,
                    [color]: Leaflet.markerClusterGroup({ iconCreateFunction, polygonOptions })
                }),
                defaultCluster
            );
        }

        return points.reduce((acc, point) => {
            const name = point[clusterFilterMetric];

            if (name == null) {
                return acc;
            }

            return {
                ...acc,
                [name]: Leaflet.markerClusterGroup({ iconCreateFunction, polygonOptions })
            };
        }, defaultCluster);
    }

    static iconCreateFunction(polygonOptions) {
        return (cluster) => {
            const childCount = cluster.getChildCount();
            if (polygonOptions.useRules === true) {
                const rule = polygonOptions.rules.find(({ from, to }) => childCount >= from && childCount < to);
                const customBackground = rule?.background || polygonOptions.defaultBackground;
                const customColor = rule?.color || polygonOptions.defaultColor;

                return new Leaflet.DivIcon({
                    html: `<div style="--w-background: ${customBackground}; --w-text-color: ${customColor}"><span>${childCount}</span></div>`,
                    className: `marker-custom-cluster leaflet-marker-icon marker-cluster leaflet-zoom-animated leaflet-interactive`,
                    iconSize: new Leaflet.Point(40, 40)
                });
            }

            let className = 'marker-cluster-';
            if (childCount < 10) {
                className += 'small';
            } else if (childCount < 100) {
                className += 'medium';
            } else {
                className += 'large';
            }
            return new Leaflet.DivIcon({
                html: `<div><span>${childCount}</span></div>`,
                className: `${className} leaflet-marker-icon marker-cluster leaflet-zoom-animated leaflet-interactive`,
                iconSize: new Leaflet.Point(40, 40)
            });
        };
    }

    /**
     * Расчет размера точки
     *
     * @param {number} sizePoint
     * @param {array<number,number>} sizeIcon
     * @param {number} optimizationSize
     * @param {number} maxSizePoint
     * @returns {number}
     */
    static calcPointSize({ sizePoint, sizeIcon, optimizationSize, maxSizePoint }) {
        if (sizeIcon == null) {
            return sizePoint;
        }

        return Math.min((sizePoint * sizeIcon) / optimizationSize, maxSizePoint);
    }

    /**
     * Создание точки
     *
     * @param {object} canvas
     * @param {number} radius
     * @param {string} color
     * @param {array.<number, number>} coordinatesPointed
     * @param {number} idx
     * @returns {object}
     */
    static buildCircle({ canvas, radius, color, coordinatesPointed, idx }) {
        return Leaflet.circle(coordinatesPointed, radius, {
            renderer: canvas,
            opacity: 1,
            fillColor: color,
            fillOpacity: FILL_OPACITY_DEFAULT,
            color,
            name: `markers-${idx}`
        });
    }

    /**
     * Создание маркера картинки
     *
     * @param {array.<number, number>} coordinatesPointed
     * @param color
     * @param {string} iconMarker
     * @param {array.<number,number>} iconSized
     * @returns {object}
     */
    static buildCanvasMarker({ coordinatesPointed, color, iconMarker, iconSized }) {
        return Leaflet.canvasMarker(coordinatesPointed, {
            img: {
                url: iconMarker,
                size: iconSized
            },
            color
        });
    }

    static buildPointTooltip({ viewMode, content, direction, popupUrl, listenerPoint, point, metrics }) {
        if (viewMode === TooltipViewNames.ALL) {
            point.bindTooltip(content, { permanent: true, direction });
            return;
        }

        if (viewMode === TooltipViewNames.CLICK) {
            point.bindTooltip(content, { direction, className: 'custom-tooltip' });
            point.off();
            point.on({
                click: () => {
                    point.openTooltip();
                }
            });
            return;
        }

        if (viewMode === TooltipViewNames.HOVER) {
            point.bindTooltip(content, { permanent: false, direction });
            return;
        }

        point.on(
            'click',
            listenerPoint.bind(this, {
                isModal: viewMode === TooltipViewNames.POPUP,
                content,
                isStore: true,
                popupUrl,
                data: metrics
            })
        );
    }

    /**
     * Get an icon display option
     *
     * @param {object} item
     * @param {string|null} metric
     * @param {object[]} variants
     * @param {string} defaultVariant
     */
    static receiveVariantIcon(item, { metric = null, variants = [], defaultVariant = '' }) {
        if (metric == null || variants.length === 0) {
            return defaultVariant;
        }

        const variant = variants.find(({ value }) => value.toLowerCase() === String(item[metric]).toLowerCase());

        if (variant == null) {
            return defaultVariant;
        }

        return variant?.variant;
    }

    static searchFlyCoordinated({ dataset, metricsFlightOfTheBumblebee, metricsLon, metricsLan, searchValue }) {
        const point = dataset.find((obj) => obj[metricsFlightOfTheBumblebee] === searchValue);

        if (metricsLon != null && metricsLan != null && point != null) {
            return [point[metricsLan], point[metricsLon]];
        }

        return null;
    }

    /**
     * Color Detection
     *
     * @param {object} item
     * @param {string} rangeDefaultColor
     * @param {string} rangeMetric
     * @param {object[]} range
     * @returns {string}
     */

    static getCurrentColor(item, { rangeDefaultColor, rangeMetric, range }) {
        const rangeCount = Number(item[rangeMetric]);

        if (rangeCount == null) {
            return rangeDefaultColor;
        }

        const foundRangeItem = [...range].reverse().find(({ from, to }) => rangeCount >= from && rangeCount <= to);

        if (foundRangeItem != null) {
            const { color: rangeColor } = foundRangeItem;

            return rangeColor;
        }

        return rangeDefaultColor;
    }

    /**
     * Animation of the appearance of a point
     *
     * @param {array} points
     * @param {number} animateDuration
     */
    static displayPointByMap({ points, animateDuration }) {
        [...points].forEach(([_, pane]) => {
            pane.style.transition = '.5s ease-in';
            pane.style.opacity = 0;
            setTimeout(() => {
                pane.style.opacity = 1;
            }, animateDuration * 1000);
        });
    }

    /**
     * all points on the visible area of the map
     *
     * @param {array[]} points
     * @param {object} Map
     * @returns {array.<array>}
     */
    static visibleAreaOfTheMapPoints(points, Map) {
        const bounds = Map.getBounds();
        return points.filter(([marker]) => bounds.contains(marker.getLatLng()));
    }
}
