import Color from 'color';
import { SizeUnits as SizeUnitsOriginal } from '@goodt-wcore/panels';

const formatNum = (num, localeSep, frDigits) =>
    localeSep === 'default'
        ? num
        : new Intl.NumberFormat(localeSep, {
              minimumFractionDigits: frDigits
          }).format(num);
const calcPercentage = (value, sep, frDigits, separator) => {
    if (separator !== '0') {
        return value.toLocaleString(sep, {
            style: 'percent',
            minimumFractionDigits: frDigits
        });
    }
    return value.toLocaleString(sep, {
        style: 'percent',
        minimumFractionDigits: frDigits,
        useGrouping: false
    });
};

const getIntersection = (src1, src2) => src1.filter((x) => src2.includes(x));

const getLeftSet = (src, target) => src.filter((el) => !target.includes(el));

const fixNum = (num, frDigits) => (num != null ? Number(num).toFixed(frDigits) : num);

const buildDimensionMetrics = (array, dimension, metricsObject) =>
    array.reduce(
        (acc, row) => ({
            ...acc,
            [row[dimension]]: Object.keys(metricsObject).reduce(
                (accMetrics, metric) => ({
                    ...accMetrics,
                    [metric]:
                        acc[row[dimension]] === undefined
                            ? [row[metric]]
                            : [...acc[row[dimension]][metric], row[metric]]
                }),
                {}
            )
        }),
        {}
    );

const Separators = ['default', 'ru-RU', 'de-DE', 'en-EN'];

const SortedSizeUnits = [...SizeUnitsOriginal].sort((e) => (e === 'px' ? -1 : +1));

const availableFormats = {
    0: ({ value, sep }) => formatNum(fixNum(value, 0), sep, 0),
    // eslint-disable-next-line no-magic-numbers
    1: ({ value, sep }) => formatNum(fixNum(value, 1), sep, 1).replace('.', ','),
    // eslint-disable-next-line no-magic-numbers
    2: ({ value, sep }) => formatNum(fixNum(value, 2), sep, 2).replace('.', ','),
    '0%': ({ value, sep, separator }) => calcPercentage(value, sep, 0, separator),
    '1%': ({ value, sep, separator }) => calcPercentage(value, sep, 1, separator),
    // eslint-disable-next-line no-magic-numbers
    '2%': ({ value, sep, separator }) => calcPercentage(value, sep, 2, separator)
};

export default {
    getDremioDimensionMetrics({ rows, dimension, metrics }) {
        rows.reduce(
            (acc, row) => ({
                ...acc,
                [row[dimension]]: metrics.reduce(
                    (accMetrics, metric) => ({
                        ...accMetrics,
                        [metric]:
                            acc[row[dimension]] === undefined
                                ? [row[metric]]
                                : [...acc[row[dimension]][metric], row[metric]]
                    }),
                    {}
                )
            }),
            {}
        );
    },
    // eslint-disable-next-line no-magic-numbers
    shiftColorByNumber(color, number, startLightness = 20) {
        if (color == null) {
            return null;
        }

        const colorData = Color(color).hsl().lightness(startLightness).object();
        const { l: light } = colorData;
        return Color(colorData)
            .lightness(light + number)
            .string();
    },
    // eslint-disable-next-line no-restricted-syntax
    tooltipFormatter({ data }) {
        const { format, separator, prefix, postfix, indicators } = this;
        const sep = Separators[separator];
        const { name, value } = data;
        const tooltips = indicators.map((indicator, i) => ({
            indicatorName: indicator.name,
            metricValue: value[i],
            metricName: name
        }));
        const values = tooltips
            .reduce((arr, { indicatorName, metricValue }) => {
                const tooltipFormat = `${indicatorName}: `;
                const formattedValue = availableFormats[format]({ value: Number(metricValue), sep, separator });
                if (formattedValue === 0 || formattedValue === '0%') {
                    return arr;
                }
                arr.push(`${tooltipFormat}${prefix}${formattedValue}${postfix}<br/>`);
                return arr;
            }, [])
            .join('');
        return `${name}<br/>${values}`;
    },
    extractValue(someCssVarOrValue, computedStyles) {
        const regExps = {
            REMS: /(?<=\d+)(rem)$/,
            EMS: /(?<=\d+)(em)$/,
            PXS: /px/,
            PERCENTS: /%$/,
            VHS: /vh$/,
            VWS: /vw$/
        };
        const cssVarOrValue = someCssVarOrValue.toString();
        const cssVar = `${cssVarOrValue}`.match(/--[-_0-9a-zA-Z%]+/)?.[0];
        const containerWidth = parseInt(computedStyles.width, 10);
        const viewPortHeight = getComputedStyle(document.documentElement).height;
        const viewPortWidth = getComputedStyle(document.documentElement).width;
        const trimmedCssVar = computedStyles.getPropertyValue(cssVar).replace(/[a-z]/g, '');
        const convertRemsToPixels = (rems) => rems * parseFloat(getComputedStyle(document.documentElement).fontSize);
        const convertEmsToPixels = (ems) => ems * parseFloat(getComputedStyle(document.documentElement).fontSize);
        const convertPercentsToPixels = (percents) => (percents * containerWidth) / 100;
        const convertVhsToPixels = (vhs) => (vhs * parseInt(viewPortHeight, 10)) / 100;
        const convertVwsToPixels = (vws) => (vws * parseInt(viewPortWidth, 10)) / 100;

        if (cssVar != null) {
            const result = convertRemsToPixels(trimmedCssVar);
            return result;
        }
        const trimmedValue = String(cssVarOrValue).replace(/[a-z]/g, '');
        const computeSizeUnit = (sizeUnit) =>
            Object.keys(regExps)
                .map((e) => sizeUnit.match(regExps[e])?.[0])
                .reduce((acc, e) => (e === undefined ? acc : e), '');
        if (computeSizeUnit(cssVarOrValue) === 'px') {
            return cssVarOrValue;
        }
        if (computeSizeUnit(cssVarOrValue) === '%') {
            return convertPercentsToPixels(parseInt(trimmedValue, 10));
        }
        if (computeSizeUnit(cssVarOrValue) === 'em') {
            return convertEmsToPixels(trimmedValue);
        }
        if (computeSizeUnit(cssVarOrValue) === 'rem') {
            return convertRemsToPixels(trimmedValue);
        }
        if (computeSizeUnit(cssVarOrValue) === 'vh') {
            return convertVhsToPixels(trimmedValue);
        }
        if (computeSizeUnit(cssVarOrValue) === 'vw') {
            return convertVwsToPixels(trimmedValue);
        }

        return '';
    },
    getIntersection,
    getLeftSet,
    SortedSizeUnits,
    buildDimensionMetrics
};
