import { InfrastructureError } from '@goodt-wcore/errors';

const GraphQL = {
    Kind: { SCALAR: 'SCALAR', OBJECT: 'OBJECT', INPUT_OBJECT: 'INPUT_OBJECT', NON_NULL: 'NON_NULL', LIST: 'LIST' },
    Name: { ID: 'ID', QUERY: 'Query', MUTATION: 'Mutation' },
    Type: { ID: 'ID' }
};
const throwSchemaValidationError = (reason) => {
    throw new Error(`GraphQL schema is invalid: ${reason}`);
};

/**
 * @typedef {object} GraphQLFieldType
 * @property {string} name
 * @property {string} kind
 * @property {GraphQLFieldType} ofType
 */
/**
 * @typedef {object} FieldType
 * @property {string} type
 * @property {string[]} kind
 */
/**
 * @typedef {object} QueryOrMutationInfo
 * @property {string} name
 * @property {string} description
 * @property {Object.<string, FieldType>} args
 * @property {FieldType} returns
 */
class FormSchema {
    /**
     *
     */
    schema = null;

    /** @type {Record<string, QueryOrMutationInfo>} */
    queries = null;

    /** @type {Record<string, QueryOrMutationInfo>} */
    mutations = null;

    /**
     *
     * @param {} schema
     */
    parse(schema) {
        if ([null, undefined, ''].includes(schema)) {
            throw new InfrastructureError('"schema" argument could not be "", "null" or "undefined"');
        }
        const queries = this.parseQueryList(schema);
        const mutations = this.parseMutationList(schema);
        this.schema = schema;
        this.queries = queries;
        this.mutations = mutations;
    }
    /**
     * @param {GraphQLFieldType} type
     * @return {FieldType}
     */
    parseFieldType({ name, kind, ofType }) {
        const { Name, Kind } = GraphQL;
        const fName = name ? name : ofType.name;
        const fKind = [kind, ofType?.kind].filter((v) => v != null);
        return { type: fName, kind: fKind };
    }
    /**
     * @param {object} schema
     * @return {Object.<string, QueryOrMutationInfo>}
     */
    parseQueryList({ types }) {
        const { parseFieldType } = this;
        const { Kind, Name } = GraphQL;
        const queryType = types.find(({ kind, name }) => kind === Kind.OBJECT && name === Name.QUERY);
        if (!queryType) {
            throwSchemaValidationError('Query type not found');
        }
        return queryType.fields
            .filter(({ type: { kind } }) => kind === Kind.OBJECT || kind === Kind.LIST)
            .reduce((acc, { name, description, args, type }) => {
                const argsHash = args.reduce((acc, { name, type }) => ({ ...acc, [name]: parseFieldType(type) }), {});
                return { ...acc, [name]: { name, description, args: argsHash, returns: parseFieldType(type) } };
            }, {});
    }
    /**
     * @param {object} schema
     * @return {Object.<string, QueryOrMutationInfo>}
     */
    parseMutationList({ types }) {
        const { isCrudMutation, parseFieldType } = this;
        const { Kind, Name } = GraphQL;
        const mutationType = types.find(({ kind, name }) => kind === Kind.OBJECT && name === Name.MUTATION);
        if (!mutationType) {
            throwSchemaValidationError('Mutation type not found');
        }
        return mutationType.fields
            .filter(({ type: { kind } }) => kind === Kind.OBJECT)
            .reduce((acc, { name, description, args, type }) => {
                if (!isCrudMutation(name)) {
                    return;
                }
                const argsHash = args.reduce((acc, { name, type }) => ({ ...acc, [name]: parseFieldType(type) }), {});
                return { ...acc, [name]: { name, description, args: argsHash, returns: parseFieldType(type) } };
            }, {});
    }
    /**
     * @return {string[]}
     */
    getMutationNames() {
        return Object.keys(this.mutations);
    }
    /**
     * @param {string} mutationName
     * @return {{ input:string, returns:string }}
     */
    getMutationModelNames(mutationName) {
        const mutation = this.mutations[mutationName];
        if (!mutation) {
            throw new Error(`Mutation not found: ${mutationName}`);
        }
        const { type: inputType } = mutation.args.input;
        const { type: returnsType } = mutation.returns;
        return { input: inputType, returns: returnsType };
    }
    /**
     * @param {string} mutationName
     * @param {{ inputVarName:string, returnsName:string }} [options]
     * @return {string}
     */
    getMutationRequestString(
        mutationName,
        { inputVarName, returnsName } = { inputVarName: 'input', returnsName: 'model' }
    ) {
        const mutation = this.mutations[mutationName];
        if (!mutation) {
            throw new Error(`Mutation not found: ${mutationName}`);
        }
        const { type: inputType } = mutation.args.input;
        const { type: returnsType } = mutation.returns;
        const returnsRequestString = this.getModelRequestString(returnsType);
        return `mutation($${inputVarName}: ${inputType}!) {
    ${returnsName}: ${mutationName}(input: $${inputVarName}) {
        ${returnsRequestString}
    }
}`;
    }
    /**
     * @param {string} modelName
     * @param {{ idVarName:string, returnsName:string }} [options]
     * @return {string}
     */
    getQueryRequestString(modelName, { idVarName, returnsName } = { idVarName: 'id', returnsName: 'model' }) {
        const query = this.queries[modelName];
        if (!query) {
            throw new Error(`Query not found: ${modelName}`);
        }
        const { type: returnsType } = query.returns;
        const returnsRequestString = this.getModelRequestString(returnsType);
        return `query($${idVarName}: ID!) {
    ${returnsName}: ${modelName}(id: $${idVarName}) {
        ${returnsRequestString}
    }
}`;
    }
    /**
     * @param {string} modelName
     * @param {{ returnsName:string }} [options]
     * @return {string}
     */
    getQueryListRequestString(modelName, { returnsName } = { returnsName: 'model' }) {
        const queryName = `${modelName}List`;
        const query = this.queries[queryName];
        if (!query) {
            throw new Error(`Query not found: ${queryName}`);
        }
        const { type: returnsType } = query.returns;
        const returnsRequestString = this.getModelRequestString(returnsType);
        return `query {
    ${returnsName}: ${queryName} {
        ${returnsRequestString}
    }
}`;
    }
    /**
     * @param {string} modelName
     * @return {string}
     */
    getModelRequestString(modelName) {
        const { Kind } = GraphQL;
        const model = this.getModelSchema(modelName);
        return Object.entries(model)
            .reduce((acc, [fieldName, { kind, type }]) => {
                if (kind.includes(Kind.SCALAR)) {
                    return [...acc, fieldName];
                }
                return [...acc, `${fieldName} { ${this.getModelRequestString(type)} }`];
            }, [])
            .join(' ');
    }
    /**
     * @param {string} modelName
     * @return {Object.<string, FieldType>}
     */
    getModelSchema(modelName) {
        const { parseFieldType } = this;
        const { types } = this.schema;
        const model = types.find(({ name, kind }) => name === modelName);
        const { fields, inputFields } = model;
        const tagetFields = fields ?? inputFields;
        return tagetFields.reduce((acc, { name, type }) => ({ ...acc, [name]: parseFieldType(type) }), {});
    }
    /**
     * @param {string} name
     * @return {boolean}
     */
    isCrudMutation(name) {
        return name.match(/(Create|Update)$/) != null;
    }
    /**
     * @param {string} name
     * @return {boolean}
     */
    isCreateMutation(name) {
        return name.match(/(Create)$/) != null;
    }
}

// const schema = new FormSchema();
// schema.parse(demoTypesSchema.data.schema);
// const mutationName = 'UserUpdate';
// const modelNames = schema.getMutationModelNames(mutationName);
// const inputModelSchema = schema.getModelSchema(modelNames.input);
// const mutation = schema.getMutationRequestString(mutationName);
// const getQuery = schema.getQueryRequestString(modelNames.returns);
// const listQuery = schema.getQueryListRequestString(modelNames.returns);

export { FormSchema, GraphQL };
