// here deep import is ok,
// due to import from @apollo/client affects "react" package import
import { gql } from '@apollo/client/core';
import { getAuthToken, createClient } from '@goodt-common/graphql';

import { FormSchema, GraphQL } from './FormSchema';
/**
 * @param {object} obj
 * @return {object}
 */
const sanitizeResult = (obj) => {
    if (Array.isArray(obj)) {
        return obj.map(sanitizeResult);
    }
    return Object.entries(obj).reduce((acc, [key, val]) => {
        if (key.indexOf('_') === 0) {
            return acc;
        }
        if (Array.isArray(val)) {
            return { ...acc, [key]: val.map(sanitizeResult) };
        }
        if (typeof val === 'object' && val != null) {
            return { ...acc, [key]: sanitizeResult(val) };
        }
        return { ...acc, [key]: val };
    }, {});
};

/**
 * @return {import('graphql').DocumentNode}
 */
const createGraphQLSchemaQuery = () => gql`
    query IntrospectionQuery {
        schema: __schema {
            types {
                ...FullType
            }
        }
    }
    fragment FullType on __Type {
        kind
        name
        description
        inputFields {
            name
            type {
                ...TypeRef
            }
        }
        fields(includeDeprecated: true) {
            name
            description
            args {
                name
                type {
                    ...TypeRef
                }
            }
            description
            type {
                ...TypeRef
            }
        }
    }
    fragment TypeRef on __Type {
        kind
        name
        ofType {
            kind
            name
            ofType {
                kind
                name
                ofType {
                    kind
                    name
                    ofType {
                        kind
                        name
                        ofType {
                            kind
                            name
                            ofType {
                                kind
                                name
                                ofType {
                                    kind
                                    name
                                }
                            }
                        }
                    }
                }
            }
        }
    }
`;

class FormService {
    /**
     * @type {import('@apollo/client').ApolloClient}
     */
    client;

    /** @type {?string} */
    action = null;

    /** @type {?FormSchema} */
    schema = null;

    constructor({ client }) {
        this.client = client;
    }

    /**
     *
     */
    reset() {
        this.schema = null;
        this.action = null;
    }

    async init() {
        const { data } = await this.client.query({ query: createGraphQLSchemaQuery(), fetchPolicy: 'network-only' });
        const schema = new FormSchema();
        schema.parse(data.schema);
        this.schema = schema;
    }
    /**
     * @param {string} id
     * @return {Promise.<object>}
     */
    async get(id) {
        const { action, schema } = this;
        const modelNames = schema.getMutationModelNames(action);
        const requestString = schema.getQueryRequestString(modelNames.returns);
        const query = gql`
            ${requestString}
        `;
        const { data } = await this.client.query({ query, variables: { id }, fetchPolicy: 'network-only' });
        return sanitizeResult(data.model);
    }
    /**
     * @param {object} model
     * @return {Promise.<object>}
     */
    async save(model) {
        const { action, schema } = this;
        const requestString = schema.getMutationRequestString(action);
        const mutation = gql`
            ${requestString}
        `;
        const { data } = await this.client.mutate({
            mutation,
            variables: { input: model },
            fetchPolicy: 'network-only'
        });
        return sanitizeResult(data.model);
    }
    /**
     * @param {string} modelName
     * @return {Promise.<object[]>}
     */
    async list(modelName) {
        const { schema } = this;
        const requestString = schema.getQueryListRequestString(modelName);
        const query = gql`
            ${requestString}
        `;
        const { data } = await this.client.query({ query, fetchPolicy: 'network-only' });
        return sanitizeResult(data.model);
    }
    /**
     * @return {{ name:string, description:string }[]}
     */
    get actionList() {
        const { isInitialized, schema } = this;
        if (!isInitialized) {
            return [];
        }
        return Object.values(schema.mutations).map(({ name, description }) => ({ name, description }));
    }
    /**
     * @return {boolean}
     */
    get isInitialized() {
        return this.schema != null;
    }
    /**
     * @return {boolean}
     */
    get isCreateAction() {
        const { action, schema } = this;
        return schema.isCreateMutation(action);
    }
    /**
     * @return {Object.<string, FormModelFieldSchema>}
     */
    get modelSchema() {
        const { Kind, Type } = GraphQL;
        const { action, schema } = this;
        const { input } = schema.getMutationModelNames(action);
        const modelSchema = schema.getModelSchema(input);
        const hasOptions = (kind) => {
            const arr = [...kind, Kind.LIST, Kind.INPUT_OBJECT];
            return arr.length != new Set(arr).size;
        };
        return Object.entries(modelSchema).reduce((acc, [key, { kind, type }]) => {
            const options = hasOptions(kind) ? () => this.list(type) : null;
            const isMultiple = kind.includes(Kind.LIST);
            const isRequired = kind.includes(Kind.NON_NULL);
            const isReadOnly = type === Type.ID;
            return { ...acc, [key]: { type, options, isMultiple, isRequired, isReadOnly } };
        }, {});
    }
}

/**
 *
 * @param uri
 * @return {FormService}
 */
export const createService = ({ uri }) => {
    const client = createClient({
        uri,
        auth: {
            getToken: getAuthToken
        }
    });

    return new FormService({ client });
};
