/* eslint-disable max-classes-per-file */
import { ApiClientMethod, BaseApiService, buildRequest } from '@goodt-common/api';
import { SafeResult } from '@goodt-common/utils';

/**
 * Service actions metadata
 */
const ServiceAction = {
    GET_MODEL_NAMES: {
        url: 'dictionary',
        options: { method: ApiClientMethod.GET }
    },
    GET_MODEL_SCHEMA: {
        url: ':path/meta',
        options: { method: ApiClientMethod.GET }
    },
    GET_MODEL: {
        url: ':path/:id',
        options: { method: ApiClientMethod.GET }
    },
    CREATE_MODEL: {
        url: ':path',
        options: { method: ApiClientMethod.POST }
    },
    UPDATE_MODEL: {
        url: ':path/:id',
        options: { method: ApiClientMethod.PUT }
    },
    DELETE_MODEL: {
        url: ':path/:id',
        options: { method: ApiClientMethod.DELETE }
    },
    LIST_MODEL: {
        url: ':path',
        options: { method: ApiClientMethod.GET }
    }
};

const FieldMetaType = {
    ARRAY: 'array',
    STRING: 'string',
    BOOLEAN: 'boolean',
    INT: 'integer',
    NUMBER: 'number'
};

const dtoToOption = (dto, { label, value }) => ({ label: dto[label] ?? 'error', value: dto[value] ?? 'error' });

const strToOption = (str) => ({ label: str, value: str });

const extractPath = ({ url, baseUrl }) => {
    const urlRel = url.replace(/(.+\/api\/)(.+)/, '$2');
    return urlRel;
};

/**
 * FormService class
 */
export class FormService extends BaseApiService {
    modelsPathMap = new Map();

    /**
     * Get model by id
     *
     * @param {{ id:string, modelName:string }} options
     * @return {Promise<import('@goodt-common/utils').ISafeResult<*, Error>>} result
     */
    get({ id, modelName }) {
        const path = this.modelsPathMap.get(modelName);
        const pathParams = { id, path };
        const action = ServiceAction.GET_MODEL;
        const request = buildRequest({ action, pathParams });
        return this.request(request);
    }

    /**
     * Creates a new model
     *
     * @param {{ model:any, modelName:string }} options
     * @return {Promise<import('@goodt-common/utils').ISafeResult<*, Error>>} result
     */
    create({ model, modelName }) {
        const params = model;
        const path = this.modelsPathMap.get(modelName);
        const pathParams = { path };
        const action = ServiceAction.CREATE_MODEL;
        const request = buildRequest({ action, params, pathParams });
        return this.request(request);
    }

    /**
     * Updates a model
     *
     * @param {{ model:any, modelName:string }} options
     * @return {Promise<import('@goodt-common/utils').ISafeResult<*, Error>>} result
     */
    update({ model, modelName }) {
        const { id } = model;
        const params = model;
        const path = this.modelsPathMap.get(modelName);
        const pathParams = { id, path };
        const action = ServiceAction.UPDATE_MODEL;
        const request = buildRequest({ action, params, pathParams });
        return this.request(request);
    }

    /**
     * Deletes a model by id
     *
     * @param {{ id:any, modelName:string }} options
     * @return {Promise<import('@goodt-common/utils').ISafeResult<*, Error>>} result
     */
    delete({ id, modelName }) {
        const path = this.modelsPathMap.get(modelName);
        const pathParams = { id, path };
        const action = ServiceAction.DELETE_MODEL;
        const request = buildRequest({ action, pathParams });
        return this.request(request);
    }

    /**
     * Get options
     *
     * @param {{ path:string }} options
     * @return {Promise<import('@goodt-common/utils').ISafeResult<*, Error>>} result
     */
    async list({ path }) {
        const pathParams = {
            path
        };
        const action = ServiceAction.LIST_MODEL;
        const request = buildRequest({ action, pathParams });
        const safeResult = await this.request(request);
        if (safeResult.isError) {
            return safeResult;
        }
        return SafeResult.success(safeResult.result.content ?? []);
    }

    /**
     * Get model names
     *
     * @return {Promise<import('@goodt-common/utils').ISafeResult<*, Error>>} result
     */
    async getModelNames() {
        const action = ServiceAction.GET_MODEL_NAMES;
        const request = buildRequest({ action });
        const safeResult = await this.request(request);
        if (safeResult.isError) {
            return safeResult;
        }
        const { _links: linkedFields = {} } = safeResult.result;
        // save path map
        this.modelsPathMap = new Map(
            Object.entries(linkedFields).map(([key, { href }]) => [
                key,
                extractPath({ url: href, baseUrl: this.apiBaseURL })
            ])
        );
        return SafeResult.success(Object.keys(linkedFields));
    }

    /**
     * Get model schema by name
     *
     * @param {string} modelName    model name
     * @return {Promise<import('@goodt-common/utils').ISafeResult<*, Error>>} result
     */
    async getModelSchema(modelName) {
        const action = ServiceAction.GET_MODEL_SCHEMA;
        const path = this.modelsPathMap.get(modelName);
        const pathParams = {
            path
        };
        const request = buildRequest({ action, pathParams });
        const safeResult = await this.request(request);
        if (safeResult.isError) {
            return safeResult;
        }
        /** @type {import('../../shared/mixins/form').FormDtoSchema} */
        const dtoSchema = safeResult.result;
        const { properties, required } = dtoSchema.dto;
        const { _links: linkedFieldsMeta = {} } = dtoSchema;

        const schema = Object.entries(properties).reduce((acc, [name, meta]) => {
            const { enum: enumOptions, readOnly: isReadOnly = false, minLength = 0, maxLength = 0 } = meta;
            const linkedFieldMeta = linkedFieldsMeta[name];
            const isEnum = enumOptions != null;
            const hasLinkedField = linkedFieldMeta != null;
            const hasOptions = hasLinkedField || isEnum;
            const isRequired = required.includes(name);
            const isMultiple = hasOptions ? meta.type === FieldMetaType.ARRAY : false;
            let options = null;

            // linked type -> dict
            if (hasLinkedField) {
                options = async () => {
                    const optionsPath = extractPath({ url: linkedFieldMeta.href, baseUrl: this.apiBaseURL });
                    const res = await this.list({ path: optionsPath });
                    return res.isError
                        ? []
                        : res.result.map((dto) =>
                              dtoToOption(dto, { label: linkedFieldMeta.title, value: linkedFieldMeta.name })
                          );
                };
            }
            // simple enum
            else if (isEnum) {
                options = () => Promise.resolve(enumOptions.map(strToOption));
            }

            /** @type {import('../../shared/mixins/form').FormModelFieldSchema} */
            const fieldSchema = {
                type: meta.type,
                isRequired,
                isMultiple,
                isReadOnly,
                minLength,
                maxLength,
                options
            };
            return { ...acc, [name]: fieldSchema };
        }, {});

        return SafeResult.success(schema);
    }
}

/**
 * Service factory
 *
 * @param {import('@goodt-common/api').IApiServiceOptions} options service options
 * @param {import('@goodt-common/net').ITransport} [transport] transport
 * @return {FormService} service instance
 */
export const createApiService = (options, transport) => new FormService({ options });
