const express = require('express');
const { graphqlHTTP } = require('express-graphql');
const { buildSchema } = require('graphql');
const cors = require('cors');

const schema = buildSchema(`
type Query {
    "Получение сущности по id (необходимо для formUpdate операции)"
    User(id: ID!): User
    
    "Получение списка сущностей вложенного типа role"
    UserRoleList: [Role]
}

type Mutation {
    "Изменение сущности по id (необходимо для formUpdate операции)"
    UserUpdate(input: UserUpdate!): User
    
    "Создание сущности (необходимо для formCreate операции)"
    UserCreate(input: UserCreate!): User
}

"Целевой тип юзер"
type User {
    id: ID!
    login: String!
    email: String!
    fullname: String
    comment: String
    roles: [Role]
    isActive: Boolean!
    createdAt: String!
    updatedAt: String!
}

"связный тип role (user => role[])"
type Role {
    id: ID!
    name: String!
}

"input тип для создания юзера"
input UserCreate {
    login: String!
    email: String!
    fullname: String
    comment: String
    roles: [UserRole]
}

"input тип для изменения юзера"
input UserUpdate {
    id: ID!
    fullname: String
    comment: String
    updatedAt: String
    isActive: Boolean
    roles: [UserRole]
}

"input тип для связи input типов юзера с role"
input UserRole {
    id: ID!
    name: String!
}
`);

const errorNotFound = (msg) => {
    throw new Error(`Not found: ${msg}`);
};
const guid = (n) => () => n++;

const userGuid = guid(1);
const roleGuid = guid(1);
const createUser = (user) => ({
    id: userGuid(),
    login: null,
    email: null,
    fullname: null,
    comment: null,
    roles: null,
    isActive: true,
    createdAt: new Date().toISOString(),
    updatedAt: new Date().toISOString(),
    ...user,
});
const createRole = (role) => ({ id: roleGuid(), name: null, ...role });
const roles = [
    createRole({ name: 'admin' }),
    createRole({ name: 'moderator' }),
    createRole({ name: 'reporter' }),
    createRole({ name: 'support' }),
    createRole({ name: 'user' }),
];
const users = [
    createUser({
        login: 'john.wick',
        email: 'john.wick@fakemail.com',
        fullname: 'John Wick',
    }),
];

const root = {
    User: ({ id }) => {
        const user = users.find((item) => item.id == id);
        if (user) {
            return user;
        }
        errorNotFound(`user by id: ${id}`);
    },
    UserCreate: ({ input }) => {
        const user = createUser(input);
        users.push(user);
        console.log('----------------');
        console.log('UserCreate', users);
        return user;
    },
    UserUpdate: ({ input }) => {
        const userFound = users.find((item) => item.id == input.id);
        if (userFound) {
            Object.assign(userFound, input);
            console.log('----------------');
            console.log('UserUpdate', users);
            return userFound;
        }
        errorNotFound(`user by id: ${input.id}`);
    },
    UserRoleList: () => roles,
};

const corsOpts = {
    origin: '*',
    allowedHeaders: '*',
    methods: 'GET,HEAD,PUT,PATCH,POST,DELETE,OPTIONS',
    preflightContinue: false,
    optionsSuccessStatus: 200,
};

const app = express();
app.use(cors(corsOpts));
app.use(
    '/graphql',
    graphqlHTTP({
        schema: schema,
        rootValue: root,
        graphiql: true,
    })
);
app.listen(4000);
console.log('Running a GraphQL API server at http://localhost:4000/graphql');
