import dayjs from 'dayjs';
import ru from 'dayjs/locale/ru';
import customParseFormat from 'dayjs/plugin/customParseFormat';
import { parseNumber, upperFirst } from '../utils/text';
import { property } from '../utils/object';

dayjs.extend(customParseFormat);

/**
 * Получает тип индикатора по описанию иконки
 * @param {string} description 
 */
function getIconType(description) {
    return description.includes('down') ? 'down' : 'up';
}

export class ApiClient {
    constructor(root) {
        this.root = root;
        if (this.root.endsWith('/')) {
            this.root = this.root.slice(0, -1);
        }
    }

    request(url) {
        return fetch(`${url}?simple=1`)
            .then(response => response.json());
    }

    /**
     * Возвращает дату и день недели, для которых API присылает данные.
     * @returns {Object} Объект с полями date (Date) и weekday (string)
     */
    getActualDate() {
        const url = `${this.root}/date`;
        return this
            .request(url)
            .then(data => {
                const date = dayjs(data[0].date, 'DD.MM.YYYY').locale(ru);
                return {
                    date   : date.toDate(),
                    weekday: date.format('dddd') 
                };
            });
    }

    /**
     * 
     * @param {('dgp'|'dgi')} organization 
     * @param {('objects'|'area')} subject 
     * @param {('weeks'|'months'|'years')} period 
     */
    getTotalForPeriod(organization, subject, period) {
        const url = `${this.root}/${organization}_${subject}_${period}`;

        return fetch(url).then(response => response.json()).then(data => {
            // Цифры "123" в api означают первые три категории (строительство, подготовка, кадастровый учёт),
            // которыми занимается комплекс градостроительной политики, а "456" -- последние три
            // (регистрация, хоз. оборот, налоговый оборот), это для комплекса экономической политики
            const categories = (organization === 'dgp') ? '123' : '456';
            const subjectKey = upperFirst(subject);

            const fieldName = `${subjectKey}_sum(${categories})`;
            const sumData = data ? data[0][fieldName] : null;

            return sumData ? sumData.qNum : -1;
        });
    }

    getDynamicsForPeriod(organization, subject, period) {
        const url = `${this.root}/${organization}_${subject}_${period}_dynamics?simple=1`;

        return fetch(url)
            .then(response => response.json())
            .then(items => {
                const fieldName = upperFirst(subject);
                return items
                    .map(item => ({
                        value: parseNumber(item[fieldName]),
                        date: dayjs(item.date).toDate()
                    }))
                    .sort((a, b) => a.date - b.date);
            });
    }

    getStageData(stageNumber, period) {
        const organization = 'dgp';
        const subjects = ['objects', 'area'];
        const indicatorColors = ['red', 'orange', 'yellow'];
        
        const indicatorsPromise = Promise.all(
            indicatorColors
                .filter(color => (stageNumber === 1) || (color === 'red'))
                .map(color => 
                    this.request(`${this.root}/${organization}_stage${stageNumber}_deviation_${color}_${period}`)
                    .then(data => ({
                        color,
                        value: parseNumber(data[0][`Deviation_${color}`] || data[0][`Deviation`]),
                        type: data[0]['Icon'].includes('down') ? 'down' : 'up'
                    }))
                )
        );

        const objectsUrl = `${this.root}/${organization}_stage${stageNumber}_objects_${period}`;
        const areaUrl = `${this.root}/${organization}_stage${stageNumber}_area_${period}`;

        const objectsPromise = this.request(objectsUrl)
            .then(data => ({
                value: parseNumber(data[0]['Objects']),
                deviation: parseNumber(data[0]['Deviation']),
                type: data[0]['Icon'].includes('down') ? 'down' : 'up'
            }));
        
        const areaPromise = this.request(areaUrl)
            .then(data => ({
                value: parseNumber(data[0]['Area']),
                deviation: parseNumber(data[0]['Deviation']),
                type: data[0]['Icon'].includes('down') ? 'down' : 'up'
            }));

        return Promise
            .all([indicatorsPromise, objectsPromise, areaPromise])
            .then(([indicators, objects, area]) => ({ indicators, objects, area }));
    }

    getConstructionDistrictsData(period) {
        return this
            .request(`${this.root}/stage1_districts_${period}`)
            .then(items => items.map(data => ({
                title: data.districts || data.district || data.District,
                objects: {
                    value: data.Objects,
                    deviation: data.Objects_deviation,
                    type: property(data, 'objects_icon', getIconType)
                },
                area: {
                    value: data.Area,
                    deviation: data.Area_deviation,
                    type: property(data, 'area_icon', getIconType)
                },
                indicators: [
                    { color: 'red', value: data.Red, type: getIconType(data.Red_icon) },
                    { color: 'orange', value: data.Orange, type: getIconType(data.Orange_icon) },
                    { color: 'yellow', value: data.Yellow, type: getIconType(data.Yellow_icon) }
                ]
            })));
    }

    getConstructionProblematicData(period) {
        const objectsPromise = this
            .request(`${this.root}/dgp_stage1_objects_all_zone`)
            .then(items => items.filter(item => item.period.toLowerCase() === period.toLowerCase()))
            .then(items => items[0])
            .then(data => [
                {
                    color: 'red',
                    value: property(data, 'objects_red', parseNumber),
                    deviation: property(data, 'objects_deviation_red', parseNumber),
                    type: property(data, 'objects_icon_red', getIconType)
                },
                {
                    color: 'orange',
                    value: property(data, 'objects_orange', parseNumber),
                    deviation: property(data, 'objects_deviation_orange', parseNumber),
                    type: property(data, 'objects_icon_orange', getIconType)
                },
                {
                    color: 'yellow',
                    value: property(data, 'objects_yellow', parseNumber),
                    deviation: property(data, 'objects_deviation_yellow', parseNumber),
                    type: property(data, 'objects_icon_yellow', getIconType)
                }
            ]);

        const areaPromise = this
            .request(`${this.root}/stage1_area_red_orange_yellow_${period}`)
            .then(data => [
                {
                    color: 'red',
                    value: parseNumber(data[0].Area_red),
                    deviation: property(data[0], 'area_red_deviation', parseNumber),
                    type: getIconType(data[0].Area_red_icon)
                },
                {
                    color: 'orange',
                    value: parseNumber(data[0].Area_orange),
                    deviation: property(data[0], 'area_orange_deviation', parseNumber),
                    type: getIconType(data[0].Area_orange_icon)
                },
                {
                    color: 'yellow',
                    value: parseNumber(data[0].Area_yellow),
                    deviation: property(data[0], 'area_yellow_deviation', parseNumber),
                    type: getIconType(data[0].Area_yellow_icon)
                }
            ]);
        
        return Promise
            .all([objectsPromise, areaPromise])
            .then(([objectItems, areaItems]) =>
                objectItems.map((_, index) => ({
                    color: objectItems[index].color,
                    objects: objectItems[index],
                    area: areaItems[index]
                }))
            );
    }

    getConstructionTotalData(period) {
        const objectsPromise = this
            .request(`${this.root}/dgp_stage1_objects_${period}`)
            .then(data => ({
                total: parseNumber(data[0].Objects),
                deviation: parseNumber(data[0].Deviation),
                type: getIconType(data[0].Icon)
            }));
        
        const areaPromise = this
            .request(`${this.root}/dgp_stage1_area_${period}`)
            .then(data => ({
                total: parseNumber(data[0].Area),
                deviation: parseNumber(data[0].Deviation),
                type: getIconType(data[0].Icon)
            }));
        
        return Promise
            .all([objectsPromise, areaPromise])
            .then(([objects, area]) => ({ objects, area }));
    }

    getConstructionFinanceData(period) {
        return this
            .request(`${this.root}/stage1_finance_${period}`)
            .then(items => [...items].map(item => ({
                title: item.source,
                objects: {
                    value: parseNumber(item.Objects),
                    deviation: parseNumber(item.Objects_Deviation),
                    type: property(item, 'objects_icon', getIconType)
                },
                area: {
                    value: parseNumber(item.Area),
                    deviation: parseNumber(item.Area_Deviation),
                    type: property(item, 'objects_icon', getIconType)
                },
                indicators: [
                    { color: 'red', value: parseNumber(item.Red), type: getIconType(item.Red_icon) },
                    { color: 'orange', value: parseNumber(item.Orange), type: getIconType(item.Orange_icon) },
                    { color: 'yellow', value: parseNumber(item.Yellow), type: getIconType(item.Yellow_icon) }
                ]
            })));
    }

    getConstructionCategoriesData(period) {
        return this
            .request(`${this.root}/stage1_category_${period}`)
            .then(items => [...items].map(item => ({
                category: item.category.toLowerCase(),
                objects: {
                    value: parseNumber(item.Objects),
                    deviation: parseNumber(item.Objects_deviation),
                    type: property(item, 'objects_icon', getIconType)
                },
                area: {
                    value: property(item, 'area', parseNumber)
                },
                indicators: [
                    { color: 'red', value: parseNumber(item.Red), type: getIconType(item.Red_icon) },
                    { color: 'orange', value: parseNumber(item.Orange), type: getIconType(item.Orange_icon) },
                    { color: 'yellow', value: parseNumber(item.Yellow), type: getIconType(item.Yellow_icon) }
                ]
            })));
    }

    getPreparationDistrictsData(period) {
        return this
            .request(`${this.root}/stage2_districts_${period}`)
            .then(items => items.map(data => ({
                title: data.districts || data.district || data.District,
                objects: {
                    value: data.Objects,
                    deviation: data.Objects_deviation,
                    type: property(data, 'objects_icon', getIconType)
                },
                area: {
                    value: data.Area,
                    deviation: data.Area_deviation,
                    type: property(data, 'area_icon', getIconType)
                },
                indicators: [
                    { color: 'red', value: data.Red, type: getIconType(data.Red_icon) }
                ]
            })));
    }

    getPreparationProblematicData(period) {
        const objectsPromise = this
            .request(`${this.root}/stage2_objects_red`)
            .then(items => items.find(item => item.period.toLowerCase() === period.toLowerCase()))
            .then(data => ({
                color: 'red',
                value: property(data, 'objects_red', parseNumber),
                deviation: property(data, 'objects_deviation_red', parseNumber),
                type: property(data, 'objects_icon_red', getIconType)
            }));

        const areaPromise = this
            .request(`${this.root}/stage2_area_red_${period}`)
            .then(data => ({
                color: 'red',
                value: parseNumber(data[0].Red),
                deviation: property(data[0], 'Red_deviation', parseNumber),
                type: getIconType(data[0].Red_icon)
            }));
        
        return Promise
            .all([objectsPromise, areaPromise])
            .then(([objects, area]) => [{ color: 'red', objects, area }]);
    }

    getPreparationCategoriesData(period) {
        return this
            .request(`${this.root}/stage2_category_${period}`)
            .then(items => [...items].map(item => ({
                category: item.category.toLowerCase(),
                objects: {
                    value: parseNumber(item.Objects),
                    deviation: parseNumber(item.Objects_deviation),
                    type: property(item, 'objects_icon', getIconType)
                },
                area: {
                    value: property(item, 'area', parseNumber)
                },
                indicators: [
                    { color: 'red', value: parseNumber(item.Red), type: getIconType(item.Red_icon) }
                ]
            })));
    }

    getPreparationFinanceData(period) {
        return this
            .request(`${this.root}/stage2_finance_${period}`)
            .then(items => [...items].map(item => ({
                title: item.source,
                objects: {
                    value: property(item, 'objects', parseNumber),
                    deviation: property(item, 'objects_deviation', parseNumber),
                    type: property(item, 'objects_icon', getIconType)
                },
                area: {
                    value: property(item, 'area', parseNumber),
                    deviation: property(item, 'area_deviation', parseNumber),
                    type: property(item, 'objects_icon', getIconType)
                },
                indicators: [
                    { color: 'red', value: parseNumber(item.Red), type: getIconType(item.Red_icon) }
                ]
            })));
    }

    getCadasteringDistrictsData(period) {
        return this
            .request(`${this.root}/stage3_districts_${period}`)
            .then(items => items.map(data => ({
                title: data.districts || data.district || data.District,
                objects: {
                    value: data.Objects,
                    deviation: data.Objects_deviation,
                    type: property(data, 'objects_icon', getIconType)
                },
                area: {
                    value: data.Area,
                    deviation: data.Area_deviation,
                    type: property(data, 'area_icon', getIconType)
                },
                indicators: [
                    { color: 'red', value: data.Red, type: getIconType(data.Red_icon) }
                ]
            })));
    }

    getCadasteringProblematicData(period) {
        const objectsPromise = this
            .request(`${this.root}/stage3_objects_red`)
            .then(items => items.find(item => item.period.toLowerCase() === period.toLowerCase()))
            .then(data => ({
                color: 'red',
                value: property(data, 'objects_red', parseNumber),
                deviation: property(data, 'objects_red_deviation', parseNumber),
                type: property(data, 'objects_red_icon', getIconType)
            }));

        const areaPromise = this
            .request(`${this.root}/stage3_area_red`)
            .then(items => items.find(item => item.period.toLowerCase() === period.toLowerCase()))
            .then(data => ({
                color: 'red',
                value: property(data, 'area_red', parseNumber),
                deviation: property(data, 'area_red_deviation', parseNumber),
                type: property(data, 'area_red_icon', getIconType)
            }));
        
        return Promise
            .all([objectsPromise, areaPromise])
            .then(([objects, area]) => [{ color: 'red', objects, area }]);
    }

    getCadasteringCategoriesData(period) {
        return this
            .request(`${this.root}/stage3_category_${period}`)
            .then(items => [...items].map(item => ({
                category: item.category.toLowerCase(),
                objects: {
                    value: parseNumber(item.Objects),
                    deviation: parseNumber(item.Objects_deviation),
                    type: property(item, 'objects_icon', getIconType)
                },
                area: {
                    value: property(item, 'area', parseNumber)
                },
                indicators: [
                    { color: 'red', value: parseNumber(item.Red), type: getIconType(item.Red_icon) }
                ]
            })));
    }

    getCadasteringFinanceData(period) {
        return this
            .request(`${this.root}/stage3_finance_${period}`)
            .then(items => [...items].map(item => ({
                title: item.source,
                objects: {
                    value: property(item, 'objects', parseNumber),
                    deviation: property(item, 'objects_deviation', parseNumber),
                    type: property(item, 'objects_icon', getIconType)
                },
                area: {
                    value: property(item, 'area', parseNumber),
                    deviation: property(item, 'area_deviation', parseNumber),
                    type: property(item, 'objects_icon', getIconType)
                },
                indicators: [
                    { color: 'red', value: parseNumber(item.Red), type: getIconType(item.Red_icon) }
                ]
            })));
    }

    getRegistrationDistrictsData(period) {
        return this
            .request(`${this.root}/stage4_districts_${period}`)
            .then(items => items.filter(item => item && item.districts || item.district || item.District))
            .then(items => items.map(data => ({
                title: data.districts || data.district || data.District,
                objects: {
                    value: property(data, 'objects', parseNumber),
                    deviation: property(data, 'objects_deviaton', parseNumber),
                    type: property(data, 'objects_icon', getIconType)
                },
                area: {
                    value: property(data, 'area', parseNumber),
                    deviation: property(data, 'area_deviation', parseNumber),
                    type: property(data, 'area_icon', getIconType)
                },
                indicators: [
                    { color: 'red', value: data.Red, type: getIconType(data.Red_icon) }
                ]
            })));
    }

    getRegistrationProblematicData(period) {
        const objectsPromise = this
            .request(`${this.root}/stage4_objects_red`)
            .then(items => items.find(item => item.period.toLowerCase() === period.toLowerCase()))
            .then(data => ({
                color: 'red',
                value: property(data, 'objects_red', parseNumber),
                deviation: property(data, 'objects_red_deviation', parseNumber),
                type: property(data, 'objects_red_icon', getIconType)
            }));

        const areaPromise = this
            .request(`${this.root}/stage4_area_red`)
            .then(items => items.find(item => item.period.toLowerCase() === period.toLowerCase()))
            .then(data => ({
                color: 'red',
                value: property(data, 'area_red', parseNumber),
                deviation: property(data, 'area_red_deviation', parseNumber),
                type: property(data, 'area_red_icon', getIconType)
            }));
        
        return Promise
            .all([objectsPromise, areaPromise])
            .then(([objects, area]) => [{ color: 'red', objects, area }]);
    }

    getRegistrationTotalData(period) {
        const objectsPromise = this
            .request(`${this.root}/dgp_stage4_objects_${period}`)
            .then(data => ({
                total: parseNumber(data[0].Objects),
                deviation: parseNumber(data[0].Deviation),
                type: getIconType(data[0].Icon)
            }));
        
        const areaPromise = this
            .request(`${this.root}/dgp_stage4_area_${period}`)
            .then(data => ({
                total: parseNumber(data[0].Area),
                deviation: parseNumber(data[0].Deviation),
                type: getIconType(data[0].Icon)
            }));
        
        return Promise
            .all([objectsPromise, areaPromise])
            .then(([objects, area]) => ({ objects, area }));
    }

    getRegistrationCategoriesData(period) {
        return this
            .request(`${this.root}/stage4_category_${period}`)
            .then(items => [...items].map(item => ({
                category: item.category.toLowerCase(),
                objects: {
                    value: parseNumber(item.Objects),
                    deviation: parseNumber(item.Objects_deviation),
                    type: property(item, 'objects_icon', getIconType)
                },
                area: {
                    value: property(item, 'area', parseNumber)
                },
                indicators: [
                    { color: 'red', value: parseNumber(item.Red), type: getIconType(item.Red_icon) }
                ]
            })));
    }

    getRegistrationFinanceData(period) {
        return this
            .request(`${this.root}/stage4_finance_${period}`)
            .then(items => items.filter(item => item && item.source))
            .then(items => [...items].map(item => ({
                title: item.source,
                objects: {
                    value: property(item, 'objects', parseNumber),
                    deviation: property(item, 'objects_deviation', parseNumber),
                    type: property(item, 'objects_icon', getIconType)
                },
                area: {
                    value: property(item, 'area', parseNumber),
                    deviation: property(item, 'area_deviation', parseNumber),
                    type: property(item, 'objects_icon', getIconType)
                },
                indicators: [
                    { color: 'red', value: parseNumber(item.Red), type: getIconType(item.Red_icon) }
                ]
            })));
    }

    getEconomicInvolvementDistrictsData(period) {
        return this
            .request(`${this.root}/stage5_districts_${period}`)
            .then(items => items.filter(item => item && item.districts || item.district || item.District))
            .then(items => items.map(data => ({
                title: data.districts || data.district || data.District,
                objects: {
                    value: property(data, 'objects', parseNumber),
                    deviation: property(data, 'objects_deviaton', parseNumber),
                    type: property(data, 'objects_icon', getIconType)
                },
                area: {
                    value: property(data, 'area', parseNumber),
                    deviation: property(data, 'area_deviation', parseNumber),
                    type: property(data, 'area_icon', getIconType)
                },
                indicators: [
                    { color: 'red', value: data.Red, type: getIconType(data.Red_icon) }
                ]
            })));
    }

    getEconomicInvolvementProblematicData(period) {
        const objectsPromise = this
            .request(`${this.root}/stage5_objects_red`)
            .then(items => items.find(item => item.period.toLowerCase() === period.toLowerCase()))
            .then(data => ({
                color: 'red',
                value: property(data, 'objects_red', parseNumber),
                deviation: property(data, 'objects_red_deviation', parseNumber),
                type: property(data, 'objects_red_icon', getIconType)
            }));

        const areaPromise = this
            .request(`${this.root}/stage5_area_red`)
            .then(items => items.find(item => item.period.toLowerCase() === period.toLowerCase()))
            .then(data => ({
                color: 'red',
                value: property(data, 'area_red', parseNumber),
                deviation: property(data, 'area_red_deviation', parseNumber),
                type: property(data, 'area_red_icon', getIconType)
            }));
        
        return Promise
            .all([objectsPromise, areaPromise])
            .then(([objects, area]) => [{ color: 'red', objects, area }]);
    }

    getEconomicInvolvementCategoriesData(period) {
        return this
            .request(`${this.root}/stage5_category_${period}`)
            .then(items => [...items].map(item => ({
                category: item.category.toLowerCase(),
                objects: {
                    value: parseNumber(item.Objects),
                    deviation: parseNumber(item.Objects_deviation),
                    type: property(item, 'objects_icon', getIconType)
                },
                area: {
                    value: property(item, 'area', parseNumber)
                },
                indicators: [
                    { color: 'red', value: parseNumber(item.Red), type: getIconType(item.Red_icon) }
                ]
            })));
    }

    getTaxInvolvementDistrictsData(period) {
        return this
            .request(`${this.root}/stage6_districts_${period}`)
            .then(items => items.filter(item => item && item.districts || item.district || item.District))
            .then(items => items.map(data => ({
                title: data.districts || data.district || data.District,
                objects: {
                    value: property(data, 'objects', parseNumber),
                    deviation: property(data, 'objects_deviaton', parseNumber),
                    type: property(data, 'objects_icon', getIconType)
                },
                area: {
                    value: property(data, 'area', parseNumber),
                    deviation: property(data, 'area_deviation', parseNumber),
                    type: property(data, 'area_icon', getIconType)
                },
                indicators: [
                    { color: 'red', value: data.Red, type: getIconType(data.Red_icon) }
                ]
            })));
    }

    getTaxInvolvementProblematicData(period) {
        const objectsPromise = this
            .request(`${this.root}/stage6_objects_red`)
            .then(items => items.find(item => item.period.toLowerCase() === period.toLowerCase()))
            .then(data => ({
                color: 'red',
                value: property(data, 'objects_red', parseNumber),
                deviation: property(data, 'objects_red_deviation', parseNumber),
                type: property(data, 'objects_red_icon', getIconType)
            }));

        const areaPromise = this
            .request(`${this.root}/stage6_area_red`)
            .then(items => items.find(item => item.period.toLowerCase() === period.toLowerCase()))
            .then(data => ({
                color: 'red',
                value: property(data, 'area_red', parseNumber),
                deviation: property(data, 'area_red_deviation', parseNumber),
                type: property(data, 'area_red_icon', getIconType)
            }));
        
        return Promise
            .all([objectsPromise, areaPromise])
            .then(([objects, area]) => [{ color: 'red', objects, area }]);
    }

    getTaxInvolvementCategoriesData(period) {
        return this
            .request(`${this.root}/stage6_category`)
            .then(items => items.filter(item => item.period.toLowerCase() === period.toLowerCase()))
            .then(items => [...items].map(item => ({
                category: item.category.toLowerCase(),
                objects: {
                    value: parseNumber(item.Objects),
                    deviation: parseNumber(item.Objects_deviation),
                    type: property(item, 'objects_icon', getIconType)
                },
                area: {
                    value: property(item, 'area', parseNumber)
                },
                indicators: [
                    { color: 'red', value: parseNumber(item.Red), type: getIconType(item.Red_icon) }
                ]
            })));
    }

    getProblematicData(stageName) {
        switch (stageName.toLowerCase()) {
            case 'construction': return this.getConstructionProblematicData('weeks');
            case 'preparation': return this.getPreparationProblematicData('weeks');
            case 'cadastering': return this.getCadasteringProblematicData('weeks');
            case 'registration': return this.getRegistrationProblematicData('weeks');
            case 'taxinvolvement': return this.getTaxInvolvementProblematicData('weeks');
            case 'economicinvolvement': return this.getEconomicInvolvementProblematicData('weeks');
        }
        throw new Error(`Unrecognized stage name: ${stageName}`);
    }
}