/* eslint-disable eqeqeq */
import merge from 'lodash/merge';
import get from 'lodash/get';
import { DIMENSION_SAMPLE, METRICS, TITLE_SAMPLE, SELECT_SAMPLE, BACKGROUND_SAMPLE } from './constants';

export const getFontSize = (value, unit) => {
    if (value != null && unit != null) {
        return `${value}${unit}`;
    }

    if (value != null) {
        return `${value}px`;
    }

    return null;
};

const SAMPLE_PROPS = Object.freeze({
    dimensions: [DIMENSION_SAMPLE, { ...DIMENSION_SAMPLE, isMajor: false }],
    metrics: METRICS,
    title: TITLE_SAMPLE,
    select: SELECT_SAMPLE,
    background: BACKGROUND_SAMPLE,
    showTile: false
});

export const normalizeProps = (initProps) => {
    const merged = merge({}, SAMPLE_PROPS, initProps);

    merged.dimensions = SAMPLE_PROPS.dimensions.map((dimension, index) =>
        merge({}, dimension, get(initProps, `dimensions.[${index}]`, null))
    );

    return merged;
};

export const getElemProps = (rowItem, props, currentItemId) => {
    const {
        showTile: isTileVisible,
        background: { color, gradient },
        select: { type, url, metric, borderColor, borderWidth }
    } = props;

    const elemProps = {
        style: {}
    };

    if (isTileVisible && gradient != null) {
        const { show: isVisibleGradient, position, color1, color2 } = gradient;
        const background = isVisibleGradient
            ? { backgroundImage: `linear-gradient(${position},${color1},${color2})` }
            : { backgroundColor: `${color}` };

        Object.assign(elemProps.style, background, {
            boxShadow: '0 0.125rem 0.5rem -0.125rem rgb(0 0 0 / 25%)'
        });
    }

    if (type !== 'none') {
        Object.assign(
            elemProps.style,
            rowItem.id === currentItemId
                ? {
                      borderColor,
                      borderWidth: getFontSize(borderWidth),
                      borderStyle: 'solid'
                  }
                : null
        );
    }

    if (type === 'select' && metric != null) {
        const metricValue = rowItem[metric];

        elemProps.url = metricValue;
    } else if (type === 'link') {
        elemProps.url = url;
    }

    return elemProps;
};

export const getDimensionConfig = (dimension) => {
    const { style, name } = dimension;
    const { color, fontFamily, fontSize, fontWeight, sizeUnit } = style;

    return {
        name,
        style: {
            color,
            fontFamily,
            fontWeight,
            fontSize: getFontSize(fontSize, sizeUnit)
        }
    };
};

export const formatMetric = (metricValue, { format, separator }) => {
    const separators = ['default', 'ru-RU', 'de-DE', 'en-EN'];
    const toFixed = (num, numDigits) => num.toFixed(numDigits);
    const calcPercentage = (num, frDigits) =>
        num.toLocaleString(undefined, {
            style: 'percent',
            minimumFractionDigits: frDigits
        });
    const formatNum = (num, localeSep, frDigits) =>
        localeSep === 'default'
            ? num
            : new Intl.NumberFormat(localeSep, { minimumFractionDigits: frDigits }).format(num);
    const availableFormats = {
        0: ({ value, sep }) => formatNum(toFixed(value, 0), sep, 0),
        1: ({ value, sep }) => formatNum(toFixed(value, 1), sep, 1),
        2: ({ value, sep }) => formatNum(toFixed(value, 2), sep, 2),
        '0%': ({ value }) => calcPercentage(value, 0),
        '1%': ({ value }) => calcPercentage(value, 1),
        '2%': ({ value }) => calcPercentage(value, 2)
    };

    return availableFormats[format]({
        value: metricValue,
        sep: separators[separator]
    });
};

export const getCustomBackground = (customColors, rawValue, isUsedListCustomColors, elemIndex, backgroundColor) => {
    if (isUsedListCustomColors) {
        const colors = customColors.filter((item) => item.name == null);
        return colors[elemIndex % colors.length]?.color;
    }

    const valueColor = customColors.find((item) => item.name === rawValue);

    if (valueColor != null) {
        return valueColor.color;
    }

    return backgroundColor;
};

const getBaseElemMetric = (rowItem, baseMetric, elemIndex) => {
    const {
        customColors,
        name,
        view,
        prefix,
        postfix,
        separator,
        format,
        style,
        isUsedListCustomColors,
        isUsedCustomColors,
        isUsedConditionalColor,
        colorByMetric
    } = baseMetric;
    const {
        color,
        fontFamily,
        fontSize,
        fontWeight,
        sizeUnit,
        background,
        moreNumber,
        moreTextColor,
        lessTextColor,
        moreColor,
        lessColor
    } = style;
    const isBadge = view === 'badge';
    const rawValue = rowItem[name];
    const valueNum = typeof rawValue === 'string' && rawValue.includes(',') ? NaN : Number.parseFloat(rawValue);
    const isNumericValue = !Number.isNaN(valueNum);

    const value = isNumericValue
        ? formatMetric(valueNum, {
              format,
              separator
          })
        : rawValue;

    const elemStyles = {
        color,
        fontFamily,
        fontWeight,
        fontSize: getFontSize(fontSize, sizeUnit),
        background
    };

    const baseElemMetric = {
        name,
        view,
        value,
        prefix,
        postfix,
        isBadge,
        isUsedConditionalColor,
        style: elemStyles
    };

    elemStyles.background = 'transparent';

    if (isBadge) {
        elemStyles.background = background;

        if (isUsedCustomColors) {
            elemStyles.backgroundColor = getCustomBackground(
                customColors,
                rawValue,
                isUsedListCustomColors,
                elemIndex,
                background
            );
        }
    }

    if (isUsedConditionalColor) {
        const condValue = rowItem[colorByMetric];
        const condValueNum = Number.parseFloat(condValue);
        const isSameLength = Number.isInteger(condValueNum)
            ? condValue == condValueNum
            : condValue.length === condValueNum.toString().length;
        const isCondNumeric = !Number.isNaN(condValueNum) && isSameLength;

        if (isCondNumeric === false) {
            return baseElemMetric;
        }
        if (isBadge) {
            elemStyles.color = condValueNum >= moreNumber ? moreTextColor : lessTextColor;
        }
        const colorField = isBadge ? 'background' : 'color';
        elemStyles[colorField] = condValueNum >= moreNumber ? moreColor : lessColor;
    }

    return baseElemMetric;
};

const getAdditionalElemMetrics = (rowItem, additionalMetrics, elemIndex) =>
    additionalMetrics.reduce((res, metric, index) => {
        const { id, view, name } = metric;
        const isBadge = view === 'badge';
        const rawValue = rowItem[name];

        const useSplit = typeof rawValue === 'string' && isBadge;

        if (useSplit) {
            const splitted = rawValue.split(',');

            if (splitted.length > 1) {
                res.push(
                    ...splitted.map((splitValue, valueIndex) => ({
                        id: Math.random() * id * valueIndex,
                        ...getBaseElemMetric({ ...rowItem, [name]: splitValue }, metric, valueIndex)
                    }))
                );

                return res;
            }
        }

        res.push({ id, ...getBaseElemMetric(rowItem, metric, elemIndex) });

        return res;
    }, []);

const getBadgeValues = (rowItem, badgeMetric, elemIndex) => {
    const {
        customColors,
        name,
        style,
        position,
        iconSettings,
        isUsedCustomColors,
        isUsedListCustomColors,
        isUsedConditionalColor,
        colorByMetric
    } = badgeMetric;
    const { color, background, fontFamily, fontSize, fontWeight, sizeUnit } = style;
    const { hasPositioning, left, right, top, bottom } = position;
    const {
        hasIcon,
        iconName,
        iconSize,
        iconColor,
        iconUnit,
        iconPosition,
        rules: iconRules,
        metric: iconMetric,
        isConditional
    } = iconSettings;
    const badgeValue = rowItem[name];

    const separatedValues = typeof badgeValue === 'string' ? badgeValue.split(',') : [badgeValue];

    return separatedValues.reduce((res, value, index) => {
        const styles = {};

        if (isUsedCustomColors) {
            styles.background = getCustomBackground(customColors, value, isUsedListCustomColors, index, background);
        }

        if (isUsedConditionalColor) {
            const condValue = rowItem[colorByMetric];
            const condValueNum = Number.parseFloat(condValue);
            const isSameLength = Number.isInteger(condValueNum)
                ? condValue == condValueNum
                : condValue.length === condValueNum.toString().length;
            const isCondNumeric = !Number.isNaN(condValueNum) && isSameLength;

            if (isCondNumeric) {
                styles.backgroundColor = condValueNum >= style.moreNumber ? style.moreColor : style.lessColor;
                styles.color = condValueNum >= style.moreNumber ? style.moreTextColor : style.lessTextColor;
            }
        }

        res.push({
            id: Date.now() * Math.random(),
            value,
            style: styles
        });

        return res;
    }, []);
};

const getBadgeElemMetric = (rowItem, badgeMetric, elemIndex) => {
    const { customColors, name, style, position, iconSettings, isUsedCustomColors, isUsedListCustomColors } =
        badgeMetric;
    const { color, background, fontFamily, fontSize, fontWeight, sizeUnit } = style;
    const { hasPositioning, left, right, top, bottom } = position;
    const {
        hasIcon,
        iconName,
        iconSize,
        iconColor,
        iconUnit,
        iconPosition,
        rules: iconRules,
        metric: iconMetric,
        isConditional
    } = iconSettings;
    const getClass = (className) => `mdi mdi-${className}`;

    const elem = {
        name,
        positionStyles: {}
    };
    const elemStyles = {
        color,
        fontFamily,
        fontWeight,
        fontSize: getFontSize(fontSize, sizeUnit),
        backgroundColor: background
    };

    const iconConfig = {};

    if (hasIcon) {
        Object.assign(iconConfig, {
            display: true,
            class: getClass(iconName),
            style: {
                fontSize: getFontSize(iconSize, iconUnit),
                color: iconColor
            }
        });
    }

    if (isConditional) {
        const valueNum = Number.parseFloat(rowItem[iconMetric]);

        Object.assign(iconConfig, { display: false });

        if (!Number.isNaN(valueNum)) {
            const rulesConfig = iconRules.reduce((res, rule) => {
                if (valueNum >= rule.from && rule.to >= valueNum) {
                    return {
                        display: true,
                        class: getClass(rule.name),
                        style: {
                            fontSize: getFontSize(iconSize, iconUnit),
                            color: rule.color
                        }
                    };
                }

                return res;
            }, null);

            Object.assign(iconConfig, rulesConfig);
        }
    }

    if (hasPositioning) {
        Object.assign(elem.positionStyles, {
            marginLeft: left,
            marginRight: right,
            marginTop: top,
            marginBottom: bottom
        });
    }

    return Object.assign(elem, {
        values: getBadgeValues(rowItem, badgeMetric, elemIndex),
        style: elemStyles,
        isReverse: iconPosition === 'left',
        icon: hasIcon ? iconConfig : null
    });
};

const getBenchmarkElemMetric = (rowItem, benchmarkMetric) => {
    const { name, type, style } = benchmarkMetric;
    const { colorNegative, colorPositive, fontSize } = style;

    const valueNum = Number.parseFloat(rowItem[name]);
    const isNumeric = !Number.isNaN(valueNum);
    const isPositive = type === 'reverse' ? valueNum < 0 : valueNum >= 0;

    return isNumeric
        ? {
              value: valueNum,
              type,
              isPositive,
              style: {
                  color: isPositive ? colorPositive : colorNegative,
                  fontSize: getFontSize(fontSize)
              }
          }
        : null;
};

export const getElemMetric = (rowItem, metrics, elemIndex) => ({
    baseMetric: getBaseElemMetric(rowItem, metrics.baseMetric, elemIndex),
    additionalMetrics: getAdditionalElemMetrics(rowItem, metrics.additionalMetrics, elemIndex),
    badgeMetric: getBadgeElemMetric(rowItem, metrics.badgeMetric, elemIndex),
    benchmarkMetric: getBenchmarkElemMetric(rowItem, metrics.benchmark, elemIndex)
});

export const getMetricSeries = (dataRows, metricName) =>
    Array.from(
        dataRows.reduce((set, row) => {
            const rowKeys = Object.keys(row);
            const metricKey = rowKeys.find((key) => key === metricName);
            const value = row[metricKey];

            if (value != null) {
                const values = value.split(',');

                values.forEach((item) => set.add(item));
            }

            return set;
        }, new Set())
    );

export const getMetricNameOptions = (metricNames) =>
    metricNames.map((item) => ({
        label: item,
        value: item
    }));
