/**
 * @typedef {object} TreeOptions
 * @property {import('@goodt-common/dremio').Query} queryHelper
 * @property {import('@goodt-common/dremio').SDK} dremioSdk
 * @property {string} rootNodeId
 * @property {string} fieldId
 * @property {string} fieldParentId
 */
export class Tree {
    /**
     * @param {TreeOptions} options
     */
    constructor(options = {}) {
        const { queryHelperAdditional, dremioSdk, rootNodeId, fieldId, fieldParentId } = options;
        /** @type {Query} */
        this.queryHelperAdditional = queryHelperAdditional;
        /** @type {SDK} */
        this.dremioSdk = dremioSdk;

        this.rootNodeId = rootNodeId;
        this.fieldId = fieldId;
        this.fieldParentId = fieldParentId;
        /** @type {Map.<string, TreeNode>} */
        this.nodes = new Map();
        /** @type {Map.<string, string[]>} */
        this.nodeChildren = new Map();
        this.rows = [];
    }

    /**
     * @param {string} nodeId
     * @return {TreeNode}
     */
    getNodeById(nodeId = null) {
        const { nodes } = this;
        const cacheKey = this.getNodeIdCacheKey(nodeId);
        if (nodes.has(cacheKey)) {
            return nodes.get(cacheKey);
        }
        const node = this.findNodeById(nodeId);
        if (node == null) {
            return null;
        }
        nodes.set(cacheKey, node);
        return node;
    }

    /**
     * @param {string} nodeId
     * @return {TreeNode[]}
     */
    getNodePath(nodeId) {
        const node = this.getNodeById(nodeId);
        // eslint-disable-next-line
        const path = [node];
        let isRootNode = this.isAnyRootNode(node);
        let parentId = this.getNodeParentId(node);

        while (isRootNode === false) {
            // eslint-disable-next-line
            const nodeParent = this.getNodeById(parentId);
            if (nodeParent == null) {
                break;
            }
            parentId = this.getNodeParentId(nodeParent);
            isRootNode = this.isAnyRootNode(nodeParent);
            path.push(nodeParent);
        }

        return path.reverse();
    }

    /**
     * @param {string} nodeId
     * @return {TreeNode[]}
     */
    getNodeChildren(nodeId) {
        const { nodes, nodeChildren, fieldId } = this;
        const cacheKey = this.getNodeIdCacheKey(nodeId);

        if (nodeChildren.has(cacheKey)) {
            return [...nodeChildren.get(cacheKey).values()].map((id) => nodes.get(id));
        }
        const filteredNodes = this.filterChildrenNode(nodeId);
        // cache nodes
        filteredNodes
            .sort((a, b) => a[fieldId] - b[fieldId])
            .forEach((child, i) => {
                const childId = this.getNodeId(child);
                const ckey = this.getNodeIdCacheKey(childId);
                nodes.set(ckey, child);
            });
        // cache children ids
        const childrenIds = filteredNodes.map((child) => this.getNodeId(child));
        nodeChildren.set(cacheKey, childrenIds);
        return filteredNodes;
    }

    /**
     * @param {string} nodeId
     * @returns {boolean}
     */
    hasNodeChildren(nodeId) {
        const cacheKey = this.getNodeIdCacheKey(nodeId);
        const children = this.nodeChildren.get(cacheKey);
        return children ? children.length > 0 : true;
    }

    async loadAdditionalRootNode() {
        const { queryHelperAdditional, dremioSdk, fieldId, fieldParentId } = this;
        const query = queryHelperAdditional.buildQuery();
        try {
            const {
                rows: [node]
            } = await dremioSdk.getData(query, 0, 1);
            return { ...node, [fieldId]: Tree.ROOT_NODE_CACHE_KEY, [fieldParentId]: Tree.TOP_ROOT_NODE_PARENT_ID };
        } catch (e) {
            if (e.isCancel) {
                // @muted
            }
            return null;
        }
    }

    buildAdditionalNodes() {
        const { fieldParentId } = this;
        const nodes = this.rows.filter((row) => row[fieldParentId] === Tree.TOP_ROOT_NODE_PARENT_ID);
        if (nodes.length === 0) {
            return null;
        }

        return nodes.map((node) => ({
            ...node,
            [fieldParentId]: Tree.ROOT_NODE_CACHE_KEY
        }));
    }

    /**
     * @param {string} nodeId
     * @return {TreeNode}
     */
    findNodeById(nodeId = null) {
        const { fieldId, fieldParentId } = this;
        const isTopMostRootNode = nodeId == null;

        const currentValue = isTopMostRootNode ? Tree.TOP_ROOT_NODE_PARENT_ID : nodeId;
        const currentFieldName = isTopMostRootNode ? fieldParentId : fieldId;
        return this.rows.find((row) => String(row[currentFieldName]) === String(currentValue));
    }

    /**
     * @param {string} parentNodeId
     * @return {*[]}
     */
    filterChildrenNode(parentNodeId) {
        const { fieldParentId } = this;
        return this.rows.filter((row) => row[fieldParentId] === parentNodeId);
    }

    /**
     *
     * @param {string} [nodeId=null]
     * @return {string}
     */
    getNodeIdCacheKey(nodeId = null) {
        return nodeId ?? Tree.ROOT_NODE_CACHE_KEY;
    }

    /**
     * @param {TreeNode} node
     * @return {string}
     */
    getNodeId(node) {
        return node[this.fieldId];
    }

    /**
     * @param {TreeNode} node
     * @return {string}
     */
    getNodeParentId(node) {
        return node[this.fieldParentId];
    }

    /**
     * @param {TreeNode} node
     * @return {boolean}
     */
    isRootNode(node) {
        return this.getNodeId(node) === this.rootNodeId;
    }

    /**
     *
     * @param {TreeNode} node
     * @return {boolean}
     */
    isTopRootNode(node) {
        return this.getNodeParentId(node) === Tree.TOP_ROOT_NODE_PARENT_ID;
    }

    /**
     * @param {TreeNode} node
     * @return {boolean}
     */
    isAnyRootNode(node) {
        return this.isRootNode(node) || this.isTopRootNode(node);
    }
}
Tree.TOP_ROOT_NODE_PARENT_ID = null;
Tree.ROOT_NODE_CACHE_KEY = 'root';
Tree.MINIMUM_CHILDREN_LENGTH = 1;
