/* eslint-disable no-param-reassign */
import { Dremio } from 'goodt-wcore';
import { isNumber } from 'lodash';
import { MetricRoleTypes, Vector } from './constants';

const { Query } = Dremio;

export const migrateProps = ({ dynamicMetrics, ...props }) => ({
    ...props,
    dynamicMetrics: dynamicMetrics.map((dynamicMetric) =>
        dynamicMetric.role === MetricRoleTypes.METRIC
            ? {
                  ...dynamicMetric,
                  width: dynamicMetric.width ?? '100%',
                  icon: {
                      tooltipText: '',
                      name: '',
                      color: '#000',
                      'font-size': '14px',
                      ...dynamicMetric.icon
                  }
              }
            : dynamicMetric
    )
});

export const VIRTUAL_ID = 'VIRTUAL';

/**
 * @typedef {import('../types').NodeData} NodeData
 * @typedef {import('../types').Node} Node
 * @typedef {import('../types').NodeDataOptions} NodeDataOptions
 */

/**
 *
 * @param {number} value
 * @param {?string} [format]
 * @param {?string} [separator]
 * @return {string}
 */
const formatCardNumber = (value, format, separator) => {
    const fraction = parseInt(format, 10);
    let percentSymbol = '';
    if (format.length > 1) {
        value *= 100;
        percentSymbol = '%';
    }
    value = Number(value.toFixed(fraction));
    if (separator !== '0') {
        value = value.toLocaleString(separator, { minimumFractionDigits: fraction });
    } else {
        value = value.toLocaleString('ru-RU', { minimumFractionDigits: fraction });
        value = value.replace(/\s/g, '');
    }

    return `${value}${percentSymbol}`;
};

/**
 *
 * @param {string|number} value
 * @param {string} format
 * @param {string} separator
 * @return {string}
 */
const formatCardText = (value, format, separator) => {
    if (isNumber(value) === false) {
        return value;
    }
    if (format == null || separator == null) {
        return value;
    }

    return formatCardNumber(value, format, separator);
};

/**
 * Extract dremio metrics
 * @param dremio
 * @return {string[]}
 */
export const extractDremioMetrics = (dremio) => {
    const { query, dimensionList } = dremio;
    const metricNames = Query.queryMetricNames(query);
    const dimensionNames = Object.keys(dimensionList);
    return [...metricNames, ...dimensionNames];
};

/**
 * @param {NodeData[]} list
 * @param {string} id
 * @param {string} parentId
 * @return {NodeData[]}
 */
export const collectRoots = (list = [], { id = 'id', parentId = 'parentId' } = {}) => {
    const nodeIds = list.map((node) => node[id]);
    return list.filter((node) => nodeIds.includes(node[parentId]) === false);
};

/**
 *
 * @param node
 * @param {string} id
 * @param {string} parentId
 * @param {string} imageUrl
 * @param {string} title
 * @param {{ style: Object, format: string, separator: string, value: string }[]} options
 * @return {NodeData}
 */

export const buildNodeOptions = (node, { id, parentId, imageUrl, title, options }) => ({
    id: node[id],
    parentId: node[parentId],
    imageUrl: node[imageUrl],
    title: node[title],
    options: options.map(({ style, format, separator, value }) => ({
        value: node[value],
        style,
        format,
        separator
    }))
});

/**
 */
export const buildVirtualCardLayout = ({ height }) =>
    `<div class="org-structure__card" style="height: ${height}px">
        <div class="org-structure__card-content">
            <div class="org-structure__card-title">Корневой элемент</div>
        </div>
    </div>`;

/**
 *
 * @param {number} height
 * @param {NodeData} data
 * @param {string} optionsLayout
 * @return {string}
 */
export const buildRealCardLayout = ({ height, data }, optionsLayout = '') => `
    <div class="org-structure__card" style="height: ${height}px">
        <code>${data.data.id}</code>
        ${data.data.imageUrl ? `<img src="${data.data.imageUrl}" class="org-structure__pic"/>` : ''}
        <div class="org-structure__card-content">
            <div class="org-structure__card-title">${data.data.title || ''}</div>
            ${optionsLayout}
        </div>
    </div>`;

/**
 *
 * @param {NodeDataOptions[]} options
 * @return {string}
 */
const buildCardOptionsLayout = (options) =>
    options
        .map(({ style, format, separator, value }) => {
            const inlineStyle = Object.entries(style)
                .map(([cssProp, cssValue]) => `${cssProp}:${cssValue}`)
                .join(';');
            const formattedText = formatCardText(value, format, separator);

            return `<div class="org-structure__card-text" style="${inlineStyle}">${formattedText}</div>`;
        })
        .join('');

/**
 *
 * @param {string} icon
 * @return {string}
 */
export const buildButtonContentLayout = ({ icon }) =>
    `<div class="org-structure__arrow"><i class="mdi ${icon}"></i></div>`;

/**
 *
 * @param {number} height
 * @param {NodeData} data
 * @return {string}
 */
export const buildCardLayout = ({ height, data }) => {
    if (data.id === VIRTUAL_ID) {
        return buildVirtualCardLayout({ height });
    }

    const cardOptionsLayout = Array.isArray(data.options) ? buildCardOptionsLayout(data.options) : '';

    return buildRealCardLayout({ height, data }, cardOptionsLayout);
};

export const getValueMetric = (data, { dynamicMetrics, metricRole = null, option = null }) => {
    const metric = dynamicMetrics.find(({ value, role }) => value !== null && role === metricRole);

    if ([data].includes(null) || metric?.value == null) {
        return null;
    }

    if (option != null && option in metric) {
        return metric[option];
    }

    return data[metric.value];
};

/**
 * @param {document} node
 * @param {string} vector
 * @returns {null|number}
 */
export const checkWidthNearbyNode = (node, { vector, separator = 2 }) => {
    const { offsetWidth: nodeWidth, nextSibling, previousSibling } = node;

    const siblingNodes = {
        [Vector.RIGHT]: nextSibling,
        [Vector.LEFT]: previousSibling
    };

    if (vector in siblingNodes) {
        const { offsetWidth: siblingWidth = 0 } = siblingNodes[vector];
        return siblingWidth / separator + nodeWidth / separator;
    }

    return null;
};

export const receiveNodeCenterAxis = (parent, child, { vector = 'y', prefix = 'offset', separator = 2 } = {}) => {
    if (vector === 'xy') {
        return vector.split('').reduce((acc, axis) => {
            const {
                [axis]: result,
                sizeParent,
                sizeChild
            } = receiveNodeCenterAxis(parent, child, { vector: axis, prefix, separator });
            return {
                ...acc,
                sizeChild,
                sizeParent,
                [axis]: result
            };
        }, {});
    }

    const axis = vector === 'y' ? `${prefix}Width` : `${prefix}Height`;
    const { [axis]: sizeParent } = parent;
    const { [axis]: sizeChild } = child;

    return {
        [vector]: sizeParent / separator - sizeChild / separator,
        sizeParent,
        sizeChild
    };
};

export const moveToChild = (panZoom, { child, parent, zoom = null } = {}) => {
    const { clientWidth: parentWidth } = child;
    const { left, right } = parent.getBoundingClientRect();
    const { scale } = panZoom.getTransform();
    return parentWidth * scale - (left + right);
};

/**
 * @param {array} arr
 * @param {string} field
 */
export const arrayUniqBy = (arr, field) => {
    const found = {};
    return arr.filter((item) => {
        const val = item[field];
        const isFound = found[val] === true;
        found[val] = true;
        return !isFound;
    });
};
/**
 * @param {array} arr
 * @return {array}
 */
export const uniqueArray = (arr) => [...new Set(arr)];

/**
 *
 * @param {number} width
 * @param {number} height
 * @param parentNode
 * @returns {number}
 */
export const calculateZoom = ({ width, height }, parentNode) => {
    const parentRect = parentNode?.getBoundingClientRect();

    if (parentRect == null) {
        return 1;
    }

    const { width: widthParent, height: heightParent } = parentRect;

    const zoomWidth = width / widthParent;
    const zoomHeight = height / heightParent;
    const maxDimension = Math.max(zoomWidth, zoomHeight);

    return maxDimension > 1 ? Math.ceil(maxDimension) - maxDimension : 1;
};

/**
 * @param {HTMLDocument} rootElement
 * @param {{attr: string, id: string, classElem: string, timeRemove: number }} options
 */
export const createShadowZoomElement = (rootElement, { attr, id, classElem = 'tree-shadow', timeRemove = 0 }) => {
    const dom = document.createElement('div');
    dom.setAttribute(attr, id);
    dom.classList.add(classElem);
    rootElement.append(dom);

    setTimeout(() => dom.remove(), timeRemove);
};
