import { formatNumber, resolveStepValue } from '../utils';

export const SliderMixin = {
    props: {
        thresholdValues: { type: Array, default: () => [] },
        value: { type: Array, default: () => [0, 0] }
    },
    computed: {
        hasRangeMode() {
            const { hasRangeMode } = this.$widget.props;
            return hasRangeMode;
        },
        grades() {
            const [min, max] = this.thresholdValues;
            const {
                metrics: { displayedStep }
            } = this.$widget.props;
            return Math.ceil((max - min) / displayedStep);
        },
        scaleValues() {
            const [min, max] = this.thresholdValues;
            const {
                metrics: { gridStep }
            } = this.$widget.props;
            return Math.ceil((max - min) / gridStep);
        }
    },
    watch: {
        value: {
            handler() {
                this.setValues();
            }
        },
        startValue: {
            handler(value, oldValue) {
                this.updateValues({ value, oldValue });
            }
        },
        endValue: {
            handler(value, oldValue) {
                this.updateValues({ value, oldValue });
            }
        },
        thresholdValues: {
            handler() {
                this.setValues();
            }
        }
    },
    mounted() {
        this.setValues();
    },
    beforeDestroy() {
        window.removeEventListener('mousemove', this.onMouseMove);
        window.removeEventListener('mouseup', this.onMouseUp);
    },
    methods: {
        onMouseUp() {
            window.removeEventListener('mousemove', this.onMouseMove);
            window.removeEventListener('mouseup', this.onMouseUp);
            if (!this.isPressed) {
                return;
            }

            this.isPressed = false;
            this.lockValue = null;
        },

        onKeyDown(event) {
            event.preventDefault();
            const { key } = event;
            const value = this.endValue;
            const [min, max] = this.thresholdValues;
            const {
                metrics: { valueStep: step }
            } = this.$widget.props;

            const newValue = resolveStepValue({ value, key, step, min, max });

            if (newValue == null || newValue === value) {
                return;
            }

            if (newValue < this.startValue) {
                this.startValue = newValue;
                this.endValue = newValue;
            } else {
                this.endValue = newValue;
            }
            this.updateSliderElementsStyles([this.startValue, this.endValue]);
        },
        updateValues({ value, oldValue }) {
            if (value === oldValue) {
                return;
            }

            this.$nextTick().then(() => {
                this.$emit('change', [this.startValue, this.endValue]);
            });
        },
        setValues() {
            const [startValue, endValue] = [...this.value].sort((a, b) => a - b);
            const [min, max] = this.thresholdValues;

            const isEmptyStartValue = startValue == null || startValue === '';
            this.startValue = !isEmptyStartValue && startValue >= min && startValue <= max ? startValue : min;

            const isEmptyEndValue = endValue == null || endValue === '';
            this.endValue = !isEmptyEndValue && endValue >= min && endValue <= max ? endValue : min;

            this.updateSliderElementsStyles([this.startValue, this.endValue]);
        },
        resolveMetricValue(value) {
            const {
                metrics: { format, separator }
            } = this.$widget.props;
            return !Number.isNaN(parseFloat(value)) && Number.isFinite(value)
                ? formatNumber({ value, format, separator })
                : value;
        }
    }
};
