import { KeyboardKeys, SliderOrientation, TooltipPosition } from './constants';

export const clamp = (n, min, max) => Math.min(Math.max(n, min), max);
export const quantize = (val, min, max, step) => {
    if (max - min <= 0) {
        return min;
    }

    if (step <= 0) {
        return clamp(val, min, max);
    }

    const steps = Math.ceil((max - min) / step);
    const valEx = min + Math.round((steps * (val - min)) / (max - min)) * step;

    return clamp(valEx, min, max);
};
export const getValuesArray = (value, value1, value2, lockValue) => {
    if (value === 0 && lockValue === 0) {
        return [0, 0];
    }

    if (lockValue != null) {
        return (value1 === lockValue ? [value1, value] : [value2, value]).sort((a, b) => a - b);
    }

    return Math.abs(value - value1) < Math.abs(value - value2) ? [value, value2] : [value1, value];
};
export const getLockValue = (value, values) => {
    if (values[0] === values[1]) {
        return null;
    }

    return Math.abs(value - values[0]) < Math.abs(value - values[1]) ? values[1] : values[0];
};

export const resolveStepValue = ({ value, key, step, min, max }) => {
    if (key === KeyboardKeys.PAGE_UP) {
        let delta = Math.max((max - min) / 10, step);
        if (delta <= 0) {
            delta = 1;
        }

        return quantize(value + delta, min, max, step);
    }

    if (key === KeyboardKeys.PAGE_DOWN) {
        let delta = Math.max((max - min) / 10, step);
        if (delta <= 0) {
            delta = 1;
        }

        return quantize(value - delta, min, max, step);
    }

    if (key === KeyboardKeys.END) {
        return max;
    }

    if (key === KeyboardKeys.HOME) {
        return min;
    }

    if ([KeyboardKeys.ARROW_DOWN, KeyboardKeys.ARROW_LEFT].includes(key)) {
        const delta = step > 0 ? step : (max - min) / 10;
        return Math.max(value - delta, min);
    }

    if ([KeyboardKeys.ARROW_UP, KeyboardKeys.ARROW_RIGHT].includes(key)) {
        const delta = step > 0 ? step : (max - min) / 10;
        return Math.min(value + delta, max);
    }

    return null;
};

export const resolvePositionStyle = ({ value, min, max }) => `${100 * (1 - (value - min) / (max - min))}%`;

export const formatNumber = ({ value, format, separator }) => {
    const valueIn = Number(value);
    const separators = [undefined, 'ru-RU', 'de-DE', 'en-EN'];

    const fixNum = (num, frDigits) => num?.toFixed(frDigits);
    const formatNum = (num, localeSep, frDigits, style = 'decimal') =>
        new Intl.NumberFormat(localeSep, {
            style,
            minimumFractionDigits: frDigits
        }).format(num);

    const availableFormats = {
        0: ({ val, sep }) => formatNum(fixNum(val, 0), sep, 0),
        1: ({ val, sep }) => formatNum(fixNum(val, 1), sep, 1),
        2: ({ val, sep }) => formatNum(fixNum(val, 2), sep, 2),
        '0%': ({ val, sep }) => formatNum(val / 100, sep, 0, 'percent'),
        '1%': ({ val, sep }) => formatNum(val / 100, sep, 1, 'percent'),
        '2%': ({ val, sep }) => formatNum(val / 100, sep, 2, 'percent')
    };

    return availableFormats[format]({ val: valueIn, sep: separators[separator] });
};

export const sortValues = (values) => values.sort((a, b) => a - b);

export const resolveTooltipPosition = (tooltipPosition) => {
    if (tooltipPosition === TooltipPosition.TOP) {
        return `tooltip--top`;
    }

    if (tooltipPosition === TooltipPosition.RIGHT) {
        return `tooltip--right`;
    }

    if (tooltipPosition === TooltipPosition.BOTTOM) {
        return `tooltip--bot`;
    }

    return `tooltip--left`;
};

export const resolveTooltipLeft = ({ tooltipPosition, sliderOrientation }) =>
    sliderOrientation === SliderOrientation.VERTICAL &&
    (tooltipPosition === TooltipPosition.TOP || tooltipPosition === TooltipPosition.BOTTOM)
        ? `50%`
        : `auto`;

export const resolveTooltipTop = ({ tooltipPosition }) =>
    tooltipPosition === TooltipPosition.TOP || tooltipPosition === TooltipPosition.BOTTOM ? `0` : `0.5rem`;

export const resolveTooltipOffset = ({ tooltipPosition, tooltipOffset, sliderOrientation }) => {
    if (sliderOrientation === SliderOrientation.VERTICAL && tooltipPosition === TooltipPosition.TOP) {
        return `-50%, -${tooltipOffset}`;
    }

    if (sliderOrientation === SliderOrientation.VERTICAL && tooltipPosition === TooltipPosition.BOTTOM) {
        return `-50%, ${tooltipOffset}`;
    }

    if (tooltipPosition === TooltipPosition.TOP) {
        return `50%, -${tooltipOffset}`;
    }

    if (tooltipPosition === TooltipPosition.RIGHT) {
        return `${tooltipOffset}, -50%`;
    }

    if (tooltipPosition === TooltipPosition.BOTTOM) {
        return `50%, ${tooltipOffset}`;
    }

    return `-${tooltipOffset}, -50%`;
};

export const resolveSizeUnits = (value) => (typeof value === 'number' ? `${Math.trunc(value)}px` : value);

export const hasPercentageSymbol = (value) => {
    const char = '%';
    const regex = new RegExp(`^[^${char}]*${char}[^${char}]*$`);
    return regex.test(value);
};
