/* eslint-disable id-length */
import { cloneDeep, isNaN } from 'lodash';
import { formatNumber } from '@goodt-common/utils-number';
import { MetricDimensionIndex } from '../constants';
import { setShadowStyle, getMaxAndMinValues, calculateSymbolSize, splitStrByWord } from './utils';

function getAxisName(axios) {
    axios.name = axios.showName ? axios.calcName : '';
}

function rem2px(obj, remField, pxField) {
    const remValue = obj[remField];
    let sizePx = parseFloat(remValue);
    if (/rem/.test(remValue) || /var/.test(remValue)) {
        const elem = document.createElement('div');
        elem.style.fontSize = remValue;
        document.body.append(elem);
        sizePx = parseFloat(getComputedStyle(elem).getPropertyValue('font-size'));
        elem.remove();
    }
    obj[pxField] = sizePx;
}

const buildDataZoom = ({ dataZoom, settings }) => {
    const dataZoomModified = dataZoom.map((zoom, index) => {
        const { shouldChangePosition } = settings[index];

        const orientation = zoom.localName === 'X' ? 'horizontal' : 'vertical';
        const mouseWheelZoomTemplate = { type: 'inside', orient: orientation };

        if (dataZoom[index].show && zoom.zoomOnMouseWheel) {
            const zoomExtended = shouldChangePosition
                ? [{ ...zoom, ...settings[index] }, mouseWheelZoomTemplate]
                : [zoom, mouseWheelZoomTemplate];
            return zoomExtended;
        }

        return shouldChangePosition ? { ...zoom, ...settings[index] } : zoom;
    });

    return dataZoomModified.flat().map((zoom) => ({ ...zoom, moveHandleSize: 0 }));
};

const isIntegerNumber = (value) => typeof value === 'number' && !Number.isNaN(value);

const buildSeries = ({ rows, pEcharts, dataZoomSettings, pointsLabelSettings, toolbox, ripple }, takeUnit2Px) => {
    const pEchart = cloneDeep(pEcharts);
    const {
        legend,
        title,
        xMetric,
        yMetric,
        firstDimension,
        aliasFirstDimension,
        legendField,
        aliasLegendField,
        colorField,

        defaultSymbolSize: defaultPointSize,
        minSize: minimumSize,
        maxSize: maximumSize,
        dividerSize,
        symbolSizeMetric,

        dataZoom,
        tooltip,
        backgrObj,
        shadowSettings
    } = pEchart;

    const defaultSymbolSize = takeUnit2Px({ size: defaultPointSize });
    const minSize = takeUnit2Px({ size: minimumSize });
    const maxSize = takeUnit2Px({ size: maximumSize });
    const { isAnimate: isRippleEffectAnimate, data: rippleEffectData } = ripple;

    let { xAxis, yAxis } = pEchart;
    let tmpXmax = 0;
    let tmpYmax = 0;

    function numberFormatting(valStr, source) {
        if (isIntegerNumber(Number(valStr)) === false) {
            return valStr;
        }

        let tmpMax = tmpXmax;
        const { format, separator } = source;
        let val = Number(valStr);
        const fraction = parseInt(format, 10);
        let pSymbol = '';

        if ([yAxis, yMetric].includes(source)) {
            tmpMax = tmpYmax;
        }

        if (format.length > 1) {
            val = (val * 100) / tmpMax;
            pSymbol = '%';
        }

        val = Number(val.toFixed(fraction));
        if (separator !== '0') {
            val = val.toLocaleString(separator, { minimumFractionDigits: fraction });
        } else {
            val = val.toLocaleString('ru-RU', { minimumFractionDigits: fraction });
            val = val.replace(/\s/g, '');
        }
        return val + pSymbol;
    }

    function axisLimit(row, axis) {
        let calcValue = true;
        let metric = xMetric;
        let tmpMax = tmpXmax;

        if (axis === yAxis) {
            metric = yMetric;
            tmpMax = tmpYmax;
        }
        axis.scale = false;
        const { limit } = axis;
        if (limit.on) {
            let { minVal } = limit;
            let { maxVal } = limit;

            if (limit.type === 'percent') {
                minVal = (Number(minVal) * tmpMax) / 100;
                maxVal = (Number(maxVal) * tmpMax) / 100;
            }
            axis.scale = true;
            calcValue = row[metric.name] >= Number(minVal) && row[metric.name] <= Number(maxVal);
        }

        return calcValue;
    }

    xAxis = xAxis[0];
    xAxis.interval = xAxis.interval === '' ? null : xAxis.interval;
    xAxis.axisLabel.interval = xAxis.axisLabel.interval === '' ? null : xAxis.axisLabel.interval;
    getAxisName(xAxis);
    xAxis.nameTextStyle.fontSize = takeUnit2Px({ size: xAxis.nameTextStyle.fullFontSize });
    xAxis.axisLabel.fontSize = takeUnit2Px({ size: xAxis.axisLabel.fullFontSize });
    xAxis.axisLabel.formatter = (value) => numberFormatting(value, xAxis);
    xAxis.axisLabel.margin = Number(xAxis.axisLabel.margin);
    xAxis.alignTicks = true;
    xAxis.axisLine.onZero = false;

    yAxis = yAxis[0];
    yAxis.interval = yAxis.interval === '' ? null : yAxis.interval;
    yAxis.axisLabel.interval = yAxis.axisLabel.interval === '' ? null : yAxis.axisLabel.interval;
    getAxisName(yAxis);
    yAxis.nameTextStyle.fontSize = takeUnit2Px({ size: yAxis.nameTextStyle.fullFontSize });
    yAxis.axisLabel.fontSize = takeUnit2Px({ size: yAxis.axisLabel.fullFontSize });
    yAxis.axisLabel.formatter = (value) => numberFormatting(value, yAxis);
    yAxis.axisLabel.margin = Number(yAxis.axisLabel.margin);
    yAxis.alignTicks = true;
    yAxis.axisLine.onZero = false;

    let tmpObj = Object.create(null);
    rows.forEach((row) => {
        tmpObj[row[legendField]] = '';
        if (row[xMetric.name] * 1 > tmpXmax) {
            tmpXmax = row[xMetric.name] * 1;
        }
        if (row[yMetric.name] * 1 > tmpYmax) {
            tmpYmax = row[yMetric.name] * 1;
        }
    });

    rem2px(legend, 'borderWidth', 'borderWidth');

    const serNames = Object.keys(tmpObj);
    legend.data = serNames.map((name) => ({ name, itemStyle: { borderWidth: legend.borderWidth } }));
    delete legend.borderWidth;
    legend.type = 'scroll';

    const tmpSeries = [];
    serNames.forEach((serName) => {
        tmpObj = {};
        tmpObj.name = serName;
        tmpObj.type = isRippleEffectAnimate ? 'effectScatter' : 'scatter';

        const filteredRows = rows.filter(
            (row) => row[legendField] === serName && axisLimit(row, xAxis) && axisLimit(row, yAxis)
        );

        const getSymbolSizeFromMetric = (symbol) => {
            if (symbolSizeMetric == null) {
                return null;
            }

            if (isNaN(symbol[symbolSizeMetric])) {
                return Number(minSize);
            }

            const selectedMetricValues = rows.map((row) => Number(row[symbolSizeMetric]));
            if (selectedMetricValues.length === 0) {
                return null;
            }

            const { max, min } = getMaxAndMinValues(selectedMetricValues);
            const coefficient = max - min;
            const sizesRange = symbol[symbolSizeMetric] - min;
            const symbolSize = calculateSymbolSize(defaultSymbolSize, sizesRange, coefficient, maxSize, minSize);

            return symbolSize;
        };

        const buildLabel = () => {
            const { isVisible, labelStyles, offset, position } = pointsLabelSettings;
            const richReducer = (
                acc,
                { name, shouldSyncLabelColor, color, fontFamily, fullFontSize, align, fontWeight }
            ) => {
                acc[name] = {
                    shouldSyncLabelColor,
                    color: shouldSyncLabelColor ? 'inherit' : color,
                    fontFamily,
                    fontSize: takeUnit2Px({ size: fullFontSize }),
                    align,
                    fontWeight
                };
                return acc;
            };

            const label = {
                position,
                show: true,
                offset: offset.map((elem) => Number(elem)),
                rich: labelStyles.reduce(richReducer, {})
            };

            return isVisible ? label : {};
        };

        tmpObj.data = filteredRows.map((item) => ({
            // eslint-disable-next-line no-restricted-syntax
            value: [
                item[xMetric.name],
                item[yMetric.name],
                item[firstDimension],
                item[legendField],
                Number(maxSize),
                Number(minSize),
                tmpXmax,
                tmpYmax
            ],
            symbolSize: getSymbolSizeFromMetric(item)
        }));

        tmpObj.label = buildLabel();

        const formatLabel = (value, { name, show, longValuesLength, format }) => {
            const index = MetricDimensionIndex[name];
            const valueToFormat = parseFloat(value[index]);
            const labelTemplate = `{${name}|${
                isNaN(valueToFormat)
                    ? splitStrByWord({ longValuesLength, str: value[index] })
                    : formatNumber(valueToFormat, format)
            }}`;
            return show ? labelTemplate : '';
        };

        const formatLabels = (value) =>
            pointsLabelSettings.labelStyles
                .map((style) => formatLabel(value, style))
                .filter((label) => label !== '')
                .join('\n');

        tmpObj.label.formatter = ({ value }) => formatLabels(value);

        let istyle = {
            borderWidth: 0.5,
            borderColor: '#abc',
            flatColor: '#6598C7',
            isGradient: false,
            gradientColor: {
                x: 0.5,
                y: 0.5,
                r: 1,
                type: 'radial',
                global: false,
                colorStops: [
                    { offset: 0, color: 'rgb(255,237,204)' },
                    { offset: 0.6, color: 'rgb(255,165,0)' }
                ]
            }
        };

        const idxName = pEchart.series.findIndex((el) => el.name === serName);
        if (idxName > -1) {
            istyle = pEchart.series[idxName].itemStyle;
        }

        istyle.color = filteredRows.length > 0 && colorField != null ? filteredRows[0][colorField] : null;

        if (istyle.color == null) {
            istyle.color = istyle.isGradient ? istyle.gradientColor : istyle.flatColor;
        }

        istyle = setShadowStyle(istyle, shadowSettings);

        tmpObj.itemStyle = istyle;
        tmpObj.symbolSize = (data) =>
            defaultSymbolSize === 0
                ? ((data[0] * (data[4] - data[5])) / data[6] + data[5]) / takeUnit2Px({ size: dividerSize })
                : defaultSymbolSize;
        tmpSeries.push({ ...tmpObj, rippleEffect: rippleEffectData });
    });

    title.textStyle.fontSize = takeUnit2Px({ size: title.textStyle.fullFontSize });

    rem2px(tooltip, 'fullPaddingTop', 'paddTop');
    rem2px(tooltip, 'fullPaddingRight', 'paddRight');
    rem2px(tooltip, 'fullPaddingDown', 'paddDown');
    rem2px(tooltip, 'fullPaddingLeft', 'paddLeft');
    tooltip.padding = [tooltip.paddTop, tooltip.paddRight, tooltip.paddDown, tooltip.paddLeft];

    rem2px(tooltip, 'fullBorderWidth', 'borderWidth');
    tooltip.textStyle.fontSize = takeUnit2Px({ size: tooltip.textStyle.fullFontSize });
    rem2px(tooltip.textStyle, 'fullLineHeight', 'lineHeight');

    tooltip.formatter = ({ data }) => {
        const dataValues = data.value;
        return tooltip.customData
            .map(({ value, label }) => {
                if (value === MetricDimensionIndex.xAxis) {
                    return `${xMetric.alias}: ${numberFormatting(dataValues[value], xMetric)}`;
                }

                if (value === MetricDimensionIndex.yAxis) {
                    return `${yMetric.alias}: ${numberFormatting(dataValues[value], yMetric)}`;
                }

                if (value === MetricDimensionIndex.dimension) {
                    return `${aliasFirstDimension}: ${dataValues[value]}`;
                }

                if (value === MetricDimensionIndex.legend) {
                    return `${aliasLegendField}: ${dataValues[value]}`;
                }

                return label;
            })
            .join('<br>');
    };
    // tooltip.triggerOn = 'click'; /**************************** test ****************************/

    let backgroundColor = 'transparent';
    const grad = backgrObj.gradientColor;
    const gradPos = grad.direction.split(' ');
    grad.x = gradPos[0];
    grad.y = gradPos[1];
    grad.x2 = gradPos[2];
    grad.y2 = gradPos[3];

    if (backgrObj.show) {
        backgroundColor = backgrObj.isGradient ? grad : backgrObj.flatColor;
    }

    rem2px(legend, 'fullItemGap', 'itemGap');
    legend.textStyle.fontSize = takeUnit2Px({ size: legend.textStyle.fullFontSize });
    rem2px(legend.textStyle, 'fullBorderWidth', 'borderWidth');
    rem2px(legend.textStyle, 'fullBorderRadius', 'borderRadius');

    return {
        legend,
        title,
        tooltip,
        serNames,
        xAxis,
        yAxis,
        dataZoom: buildDataZoom({ dataZoom, settings: dataZoomSettings }),
        series: tmpSeries,
        backgroundColor,
        grid: { containLabel: true },
        toolbox
    };
};

export { buildSeries };
