import { reactive } from '@goodt-common/utils';
import { BaseViewModel } from '@goodt-common/layers';
import { isEmpty } from 'lodash';
import { SvgUtils } from '../utils/SvgUtils';
import { SVG_STYLES } from '../config';

export class SvgViewModel extends BaseViewModel {
    svgElement;

    svgUtils;

    widget;

    _state = reactive({
        selectedNode: null,
        textNodes: [],
        svgNodes: []
    });

    constructor({ svgElement, widget }) {
        super();
        this.svgElement = svgElement;
        this.svgUtils = new SvgUtils(svgElement);
        this.widget = widget;
        this.extendContentWithStyles();
    }

    get selectedNode() {
        return this._state.selectedNode;
    }

    get textNodes() {
        return this._state.textNodes;
    }

    setWidget(widget) {
        this.widget = widget;
    }

    extendContentWithStyles() {
        const styleTag = document.createElementNS('http://www.w3.org/2000/svg', 'style');
        styleTag.textContent = SVG_STYLES;
        this.svgElement.appendChild(styleTag);
    }

    buildNodeOptions() {
        const svg = this.svgElement;
        return [...svg.children]
            .flatMap((node) => {
                if (node.children.length > 0) {
                    return this.getChildrenIds(node);
                }

                return node.id;
            })
            .filter((id) => id !== '')
            .map((id, idx) => ({ label: id, value: idx }));
    }

    getChildrenIds(parentNode) {
        return [
            parentNode.id,
            [...parentNode.children].flatMap((node) => {
                if (node.children.length > 0) {
                    return this.getChildrenIds(node);
                }

                return node.id;
            })
        ].flat();
    }

    resolveSvgElementSize(svgElement) {
        const boxSize = svgElement.getBBox();
        return {
            width: Math.max(boxSize.width ?? 0, svgElement.getComputedTextLength() ?? 0),
            height: boxSize.height ?? 0
        };
    }

    updateSvg(result) {
        this._state.selectedNode = null;
        if (this.svgElement == null) {
            return;
        }

        this._state.textNodes.forEach((node) => {
            node.remove();
        });

        // reset event listeners
        this._state.svgNodes.forEach((node) => {
            node.replaceWith(node.cloneNode(true));
        });

        const { rows } = result;
        const {
            nodes,
            fields: { metricId },
            labels
        } = this.widget.props;
        const svg = this.svgElement;
        this._state.svgNodes = this.buildDefaultNodes({ svg, rows });
        this._state.textNodes = [];
        const filteredNodes = nodes.filter(({ id }) => id !== '');
        this.updateNodes({ nodes: filteredNodes, rows, labels, metricId, svg });
    }

    updateNodes({ nodes, rows, labels, metricId, svg }) {
        nodes.forEach(({ id, isVisible }) => {
            const element = svg.getElementById(id);
            if (element == null || element.style == null) {
                return;
            }

            if (!isVisible) {
                element.style.display = 'none';
                return;
            }

            element.style.display = 'inherit';
            const row = rows.find(({ [metricId]: rowId }) => rowId === element.getAttribute('data-id'));
            if (row == null) {
                const {
                    hoverSettings: { shouldHoverAllElements, shouldUseSettings },
                    selectSettings: { isEnabled, shouldSelectAllElements },
                    colorSettings: { defaultColor, defaultStroke }
                } = this.widget.props;
                element.style.cursor =
                    (shouldUseSettings && shouldHoverAllElements) || (isEnabled && shouldSelectAllElements)
                        ? 'pointer'
                        : 'default';

                if (isEmpty(defaultColor) === false) {
                    element.style.fill = this.svgUtils.resolveNodeStyle({ value: defaultColor });
                }

                if (isEmpty(defaultStroke) === false) {
                    element.style.stroke = this.svgUtils.resolveNodeStyle({ value: defaultStroke });
                }

                element.setAttribute('data-style', element.style.cssText);
                return;
            }

            element.style.cursor = 'pointer';
            if (labels.length > 0) {
                const { shouldDrawForeignObject } = this.widget.props;
                this._state.textNodes.push(
                    shouldDrawForeignObject
                        ? this.createForeignObjectNode({ element, row, svg })
                        : this.createTextNode({ element, row, svg })
                );
            }

            element.setAttribute('data-style', element.style.cssText);
        });
    }

    resolveLabelsSettings({ element }) {
        const { labelOffsetY, labelOffsetX, labelOffsetRight, labelsSettings, offsetBetweenLabels } = this.widget.props;
        const foundLabel = labelsSettings.find(({ id }) => id === element.id);
        if (foundLabel?.useCustomLabelSettings) {
            return foundLabel;
        }

        return { labelOffsetY, labelOffsetX, labelOffsetRight, offsetBetweenLabels };
    }

    createTextNode({ element, row, svg }) {
        const { labels } = this.widget.props;
        const { labelOffsetY, labelOffsetX, labelOffsetRight, offsetBetweenLabels } = this.resolveLabelsSettings({
            element
        });
        const { width, height, x: xCoordinate, y: yCoordinate } = element.getBBox();
        const offsetX = xCoordinate + width * 0.01 * labelOffsetX;
        const textNode = this.svgUtils.buildTextNode({
            y: yCoordinate + height * 0.01 * labelOffsetY,
            x: offsetX
        });
        textNode.setAttribute('id', element.getAttribute('data-id'));
        textNode.innerHTML = '<tspan id="PROCESSING"></tspan >';
        svg.appendChild(textNode);
        const titles = this.svgUtils.buildTitles({ labels, row, isOnlyTitleText: false, shouldSplitWords: true });
        let isFulled = false;
        let isFirstLine = true; // for offset by the first line
        let hasAddedPoints = false;
        let prevTitles = textNode.innerHTML;
        let isFirstLineInLabel = true; // for offset between labels
        titles.forEach(({ words, prefix, postfix, style }) => {
            if (isFulled) {
                return;
            }

            isFirstLineInLabel = true;
            let line = '';
            words.forEach((word, idx) => {
                if (isFulled) {
                    return;
                }

                const { tempTSpanWidth, testLine } = this.buildTextInfo({
                    line,
                    style,
                    xCoordinate,
                    postfix,
                    prefix,
                    idx,
                    word,
                    words,
                    offsetBetweenLabels
                });

                if (this.resolveSvgElementSize(textNode).height > height) {
                    isFulled = true;
                    hasAddedPoints = true;
                    textNode.innerHTML = prevTitles;
                    textNode.innerHTML += this.svgUtils.buildTreeDots({ style });
                    isFirstLine = false;
                    isFirstLineInLabel = false;
                    return;
                }

                if (tempTSpanWidth > width - (width / 100) * labelOffsetRight && idx > 0) {
                    if (line.trim() !== '') {
                        textNode.innerHTML += this.svgUtils.buildTitleTSpan({
                            text: line,
                            x: offsetX,
                            style,
                            isFirstLine,
                            isFirstLineInLabel,
                            offsetBetweenLabels
                        });
                    }
                    line = `${word} `;
                    isFirstLine = false;
                    isFirstLineInLabel = false;
                } else {
                    line = `${testLine} `;
                }

                if (idx === words.length - 1) {
                    textNode.innerHTML += this.svgUtils.buildTitleTSpan({
                        text:
                            postfix.text.trim() !== '' && line.trim() === postfix.text.trim()
                                ? this.svgUtils.buildTSpan({ text: postfix.text, style: postfix.style })
                                : line,
                        x: offsetX,
                        style,
                        isFirstLine,
                        isFirstLineInLabel,
                        offsetBetweenLabels
                    });

                    isFirstLine = false;
                    isFirstLineInLabel = false;
                }
                isFulled = this.resolveSvgElementSize(textNode).height > height;
                if (!isFulled) {
                    prevTitles = textNode.innerHTML;
                }
            });

            isFulled = this.resolveSvgElementSize(textNode).height > height;
            if (!hasAddedPoints && isFulled) {
                textNode.innerHTML = prevTitles;
                textNode.innerHTML += this.svgUtils.buildTreeDots({ style });
                hasAddedPoints = true;
                isFirstLine = false;
            }
        });

        this.svgElement.getElementById('PROCESSING')?.remove();
        return textNode;
    }

    createForeignObjectNode({ element, row, svg }) {
        const { labels, limitSettings } = this.widget.props;
        const { offsetBetweenLabels, labelOffsetRight, labelOffsetX, labelOffsetY } = this.resolveLabelsSettings({
            element
        });
        const divNode = this.svgUtils.buildDivNode({
            element,
            lineClamp: this.svgUtils.resolveLineClamp({ element, limitSettings }),
            labels,
            labelOffsetRight,
            labelOffsetX,
            labelOffsetY
        });

        const titles = this.svgUtils.buildTitles({ labels, row, isOnlyTitleText: true, shouldSplitWords: false });
        divNode.innerHTML = titles.reduce(
            (acc, { words, prefix, postfix, style }, idx) =>
                words === ''
                    ? acc
                    : acc +
                      this.svgUtils.buildDiv({
                          words,
                          style,
                          prefix,
                          postfix,
                          offsetBetweenLabels,
                          isLastLabel: titles.length - 1 === idx
                      }),
            ''
        );

        const foreignObject = this.svgUtils.buildForeignObjectNode(element);
        foreignObject.appendChild(divNode);
        svg.appendChild(foreignObject);

        return foreignObject;
    }

    buildDefaultNodes({ svg, rows }) {
        const { nodes } = this.widget.props;

        const svgNodes = [...svg.children].flatMap((node) => {
            if (node.children.length > 0) {
                return [node, ...this.getChildren(node)].flat();
            }

            return node;
        });
        return svgNodes
            .filter((node) => node.id != null && node.id !== '' && node.style != null)
            .map((node) => {
                // eslint-disable-next-line no-restricted-syntax
                const currentNode = nodes.find(({ id }) => id === node.id);
                if (currentNode == null) {
                    return node;
                }

                if (!currentNode.isVisible) {
                    return null;
                }

                const { id, newId, shouldChangeId } = currentNode;
                node.setAttribute('data-id', shouldChangeId ? newId : id);
                this.resolveNodeStyles({ node, rows });
                node.setAttribute('data-style', node.style.cssText);
                return node;
            })
            .filter(Boolean);
    }

    buildTextInfo({ line, word, postfix, prefix, idx, words, style, xCoordinate, offsetBetweenLabels }) {
        const tempTSpan = this.svgElement.getElementById('PROCESSING');
        const testLine = this.svgUtils.buildTestLine({
            line,
            word,
            postfix,
            idx,
            prefix,
            wordsLength: words.length
        });

        tempTSpan.innerHTML = this.svgUtils.buildTitleTSpan({
            text: `${testLine} ${this.svgUtils.buildTreeDots({ style })}`,
            style,
            isFirstLine: true,
            isFirstLineInLabel: false,
            x: xCoordinate,
            offsetBetweenLabels
        });

        const { width: tempTSpanWidth, height: tempTSpanHeight } = this.resolveSvgElementSize(tempTSpan);

        return { testLine, tempTSpanWidth, tempTSpanHeight };
    }

    setNodeStyleBySettings({ colorNodes, node }) {
        const colorNode = colorNodes.find(({ id: rowId }) => rowId === node.id);
        if (colorNode == null) {
            return;
        }

        const { fill } = colorNode;
        if (fill != null) {
            node.style.fill = this.svgUtils.resolveNodeStyle({ value: fill });
        }
    }

    setNodeStyleByRules({ node, rules, rows }) {
        const {
            fields: { metricColor, metricId }
        } = this.widget.props;
        const foundRow = rows.find(({ [metricId]: id }) => String(node.getAttribute('data-id')) === String(id));
        if (foundRow == null) {
            return;
        }

        const { [metricColor]: value } = foundRow;
        const foundRule = rules.find(({ from, to }) => Number(value) >= Number(from) && Number(value) < Number(to));
        if (foundRule == null) {
            node.style.fill = '';
            node.style.stroke = '';
            return;
        }

        node.style.fill = foundRule.fill;
        node.style.stroke = foundRule.stroke;
    }

    getChildren(parentNode) {
        // spread for specific svg elements
        return [...parentNode.children].flatMap((node) => {
            if (node.children.length > 0) {
                return [node, ...this.getChildren(node)].flat();
            }

            return node;
        });
    }

    onElementEnter({ element, settings }) {
        const { fill, stroke, strokeWidth, additionalFill, additionalStroke } = settings;
        if (isEmpty(additionalFill) === false && isEmpty(additionalFill) === false) {
            this._state.svgNodes.forEach((node) => {
                if (isEmpty(additionalFill) === false) {
                    node.style.fill = additionalFill;
                }

                if (isEmpty(additionalStroke) === false) {
                    node.style.stroke = additionalStroke;
                }
            });
        }

        element.style.fill = fill;
        element.style.stroke = stroke;
        element.style.strokeWidth = strokeWidth;
    }

    onElementLeave() {
        this._state.svgNodes.forEach((node) => {
            node.style.cssText = node.getAttribute('data-style');
        });
    }

    resetSelectedNode() {
        this._state.svgNodes.forEach((node) => {
            node.style.opacity = 1;
        });
        this._state.textNodes.forEach((textNode) => {
            textNode.style.opacity = 1;
        });
        this._state.selectedNode = null;
    }

    selectNewNode({ element, settings }) {
        const { blockOpacity, textOpacity } = settings;
        this._state.selectedNode = element.id;
        this._state.svgNodes.forEach((node) => {
            node.style.opacity = blockOpacity;
        });
        this._state.textNodes.forEach((textNode) => {
            textNode.style.opacity = textOpacity;
        });

        element.style.opacity = 1;
        const elementId = element.getAttribute('data-id');
        const textNode = this._state.textNodes.find((node) => node.id === elementId);
        if (textNode != null) {
            textNode.style.opacity = 1;
        }
    }

    resolveNodeStyles({ node, rows }) {
        const {
            generalSettings: { strokeWidth, stroke, boxShadow }
        } = this.widget.props;
        node.style.stroke = this.svgUtils.resolveNodeStyle({ value: stroke });
        node.style.strokeWidth = this.svgUtils.resolveNodeStyle({ value: strokeWidth, defaultValue: 0 });
        node.style.filter = this.svgUtils.resolveNodeStyle({
            prefix: 'drop-shadow(',
            postfix: ')',
            value: boxShadow
        });
        node.style.transition = 'all 0.5s';
        const {
            colorSettings: { nodes: colorNodes, shouldUseRules, rules },
            fields: { metricColor }
        } = this.widget.props;
        if (!shouldUseRules || metricColor == null) {
            this.setNodeStyleBySettings({ colorNodes, node });
            return;
        }

        this.setNodeStyleByRules({ node, rules, rows });
    }
}
