import { cloneDeep } from 'lodash';
import { formatNumber } from '@goodt-common/utils-number';
import { OTHER_NAME } from './constants';

const resolveColorByRules = ({ rules, value }) => {
    const numberValue = Number(value);
    if (Number.isNaN(numberValue)) {
        return null;
    }

    const foundRule = rules.find(({ from, to }) => from <= numberValue && to > numberValue);
    return foundRule != null ? foundRule.color : null;
};

const resolveColor = ({ useMetric, colorField, parent, colorsByRules }, otherElem = null) => {
    const { isEnabled, rules, field } = colorsByRules;
    if (isEnabled && rules.length > 0) {
        return otherElem
            ? resolveColorByRules({ rules, value: otherElem[field] })
            : resolveColorByRules({ rules, value: parent[field] });
    }

    if (otherElem !== null) {
        return useMetric && colorField ? otherElem[colorField] : null;
    }

    return useMetric && colorField ? parent[colorField] : null;
};

const selectInsideBorderColor = ({ levels, idx, sortedRowsByValue }) => {
    const { splitInsideBorderColorBySections, insideBorderColor } = levels;
    const newLevels = cloneDeep(levels);
    if (newLevels.borderColors == null) {
        newLevels.borderColors = [];
    }

    return splitInsideBorderColorBySections
        ? sortedRowsByValue.map((row, index) => newLevels.borderColors[index % newLevels.borderColors.length])[idx]
        : insideBorderColor;
};

const formatSeriesData = (data, numberOfDimensionsAfterMerge, parentField, levels, colorsByRules) => {
    const seriesData = data.sort((a, b) => b.value - a.value);
    if (numberOfDimensionsAfterMerge === 'all') {
        return seriesData;
    }

    const { useMetric, mainColorField } = levels;
    const metricsToShow = seriesData.slice(0, numberOfDimensionsAfterMerge);
    const other = seriesData.slice(numberOfDimensionsAfterMerge).reduce(
        (otherObj, series) =>
            Object.entries(series).reduce((acc, [key, value]) => {
                if (typeof value === 'number' || !Number.isNaN(parseFloat(value))) {
                    acc[key] = (acc[key] ?? 0) + parseFloat(value);
                    return acc;
                }
                if (typeof value === 'object') {
                    acc[key] = Array.isArray(value) ? [] : {};
                    return acc;
                }
                acc[key] = value;
                return acc;
            }, otherObj),
        {}
    );

    other[parentField] = OTHER_NAME;
    other.name = OTHER_NAME;
    other.itemStyle.color = resolveColor({ useMetric, colorField: mainColorField, parent: null, colorsByRules }, other);
    metricsToShow.push(other);
    return metricsToShow;
};

// eslint-disable-next-line import/no-mutable-exports
export let sortedRowsLength;

const buildTree = ({
    rows,
    treeMap,
    levels,
    colorsByRules,
    numberOfDimensionsAfterMerge,
    isUniteRestDimensions,
    drilldown
}) => {
    const { parentField, childField, valueField } = treeMap;
    if ([parentField, childField, valueField].includes('')) {
        return [];
    }

    const { useMetric, colorField, mainColorField } = levels;
    const parents = rows.map((row) => row[parentField]);

    const sortedRowsByValue = [...new Set(parents)]
        .map((parent) => {
            // eslint-disable-next-line no-restricted-syntax
            const foundParent = rows.find((row) => row[parentField] === parent);
            const filteredRowsByParents = rows.filter((row) => row[parentField] === parent);
            return {
                [parentField]: parent,
                ...foundParent,
                value: filteredRowsByParents.reduce((acc, record) => acc + Number(record[valueField]), 0),
                itemStyle: {
                    color: resolveColor({ useMetric, colorField: mainColorField, parent: foundParent, colorsByRules })
                },
                name: parent,
                children: filteredRowsByParents.map((record) => ({
                    ...record,
                    name: record[childField],
                    value: record[valueField],
                    children: [],
                    itemStyle: {
                        color: resolveColor({ useMetric, colorField, parent: foundParent, colorsByRules })
                    }
                }))
            };
        })
        .sort((prev, next) => next.value - prev.value);

    const updateItemStyleBorderColor = (item, idx) => ({
        ...item,
        itemStyle: {
            ...item.itemStyle,
            borderColor: selectInsideBorderColor({ levels, idx, sortedRowsByValue })
        }
    });

    const updateSortedRowsByValue = () =>
        sortedRowsByValue.map((row, idx) => ({
            ...row,
            children: row.children.map((child) => updateItemStyleBorderColor(child, idx))
        }));

    const updateSortedRowsByValueWithOther = () =>
        formatSeriesData(sortedRowsByValue, numberOfDimensionsAfterMerge, parentField, levels, colorsByRules).map(
            (row, idx) => ({
                ...row,
                children:
                    row.name === OTHER_NAME && !drilldown.isActive
                        ? [updateItemStyleBorderColor(row, idx)]
                        : row.children.map((child) => updateItemStyleBorderColor(child, idx))
            })
        );

    sortedRowsLength = sortedRowsByValue.length;
    return isUniteRestDimensions ? updateSortedRowsByValueWithOther() : updateSortedRowsByValue();
};

const getLevelOptions = ({ levels, emphasis, colorsByRules }) => {
    const defaultLevels = [
        { itemStyle: { borderWidth: 0 }, upperLabel: { show: false } },
        { itemStyle: { borderWidth: levels.outsideBorderWidth, borderColor: levels.outsideBorderColor } },
        {
            itemStyle: {
                borderWidth: levels.insideBorderWidth,
                borderColor: levels.insideBorderColor,
                borderColors: []
            }
        }
    ];

    if (emphasis.isActive) {
        defaultLevels[1].emphasis = emphasis;
        defaultLevels[2].emphasis = emphasis;
    }

    if (!levels.useMetric && !colorsByRules.isEnabled) {
        defaultLevels[1].colorSaturation = [Number(levels.minSaturation), Number(levels.maxSaturation)];
    }

    return defaultLevels;
};

const buildBreadcrumb = ({ breadcrumb, takeUnit2Px }) => {
    if (breadcrumb.show === false) {
        return {
            show: false
        };
    }

    const newBreadcrumb = cloneDeep(breadcrumb);
    const { itemStyle, height, top } = newBreadcrumb;
    itemStyle.textStyle.fontSize = takeUnit2Px({ size: itemStyle.textStyle.fontSize });

    return {
        ...newBreadcrumb,
        height: takeUnit2Px({ size: height }),
        top: `${top}%`
    };
};

const generatePadding = ({ field, takeUnit2Px }) =>
    ['paddingTop', 'paddingRight', 'paddingBottom', 'paddingLeft'].map((paddingType) =>
        takeUnit2Px({ size: field[paddingType] })
    );

const generateRichStyleRules = (style, rules, takeUnit2Px) =>
    rules.reduce((acc, rule, idx) => {
        const richStyle = {
            ...style,
            borderRadius: takeUnit2Px({ size: style.borderRadius }),
            padding: generatePadding({ field: style, takeUnit2Px }),
            fontSize: takeUnit2Px({ size: rule.fontSize || 16 }),
            lineHeight: takeUnit2Px({ size: rule.fontSize || 16 }),
            color: rule.textColor || '#000',
            fontWeight: rule.fontWeight,
            fontFamily: rule.fontFamily
        };
        acc[`rule_${idx}`] = richStyle;
        return acc;
    }, {});

const generateRichLabelByRules = ({ labels, colorsByRules, takeUnit2Px }) => {
    const { rules } = colorsByRules;
    return labels.reduce((acc, { style }) => {
        const richStyleRules = generateRichStyleRules(style, rules, takeUnit2Px);
        return {
            ...acc,
            ...richStyleRules
        };
    }, {});
};

const generateRichLabel = ({ labels, hoverColor = null, takeUnit2Px, colorsByRules, otherDimensionsStyles = {} }) => {
    const { isEnabled, isActiveLabelStyles = false } = colorsByRules;
    if (isEnabled && isActiveLabelStyles) {
        return generateRichLabelByRules({ labels, colorsByRules, takeUnit2Px });
    }

    return labels.reduce(
        (acc, { id, field, style, prefix, postfix }) => ({
            ...acc,
            [`${field}_${id}`]: {
                ...style,
                fontSize: takeUnit2Px({ size: style.fullFontSize }),
                borderRadius: takeUnit2Px({ size: style.borderRadius }),
                padding: generatePadding({ field: style, takeUnit2Px }),
                color: hoverColor || style.color
            },
            [`${field}_${id}Prefix`]: {
                ...prefix.style,
                fontSize: takeUnit2Px({ size: prefix.style.fullFontSize }),
                borderRadius: takeUnit2Px({ size: prefix.style.borderRadius }),
                padding: generatePadding({ field: prefix.style, takeUnit2Px }),
                color: hoverColor || prefix.style.color
            },
            [`${field}_${id}Postfix`]: {
                ...postfix.style,
                fontSize: takeUnit2Px({ size: postfix.style.fullFontSize }),
                borderRadius: takeUnit2Px({ size: postfix.style.borderRadius }),
                padding: generatePadding({ field: postfix.style, takeUnit2Px }),
                color: hoverColor || postfix.style.color
            },
            [`other`]: {
                ...otherDimensionsStyles,
                fontSize: takeUnit2Px({ size: otherDimensionsStyles.fullFontSize }),
                borderRadius: takeUnit2Px({ size: otherDimensionsStyles.borderRadius }),
                padding: generatePadding({ field: otherDimensionsStyles, takeUnit2Px }),
                color: hoverColor || otherDimensionsStyles.color
            }
        }),
        {}
    );
};

const formatMetric = ({ value, format }) => {
    if (!Number.isNaN(parseFloat(value)) && Number.isFinite(value)) {
        const numberValue = Number(value);
        return formatNumber(numberValue, format);
    }

    return value;
};

const isIntegerNumber = (value) => typeof value === 'number' && !Number.isNaN(value);

const buildLabelBySetting = ({ field, prefix, postfix, data, format, id, mainLabelField }) => {
    if (data.name === OTHER_NAME) {
        data[mainLabelField] = OTHER_NAME;
    }

    if (data[field] === OTHER_NAME) {
        return `{other|Другое}`;
    }
    let label = `{${field}_${id}|${
        formatMetric({ value: isIntegerNumber(Number(data[field])) ? Number(data[field]) : data[field], format }) || ''
    }}`;
    if (prefix.text.trim() !== '') {
        label = `{${field}_${id}Prefix|${prefix.text}}${label}`;
    }

    if (postfix.text.trim() !== '') {
        label = `${label}{${field}_${id}Postfix|${postfix.text}}`;
    }

    return label;
};

const buildLabelByRules = ({
    field,
    prefix,
    postfix,
    data,
    format,
    colorsByRules,
    mainData = null,
    parent,
    mainLabelField
}) => {
    const { text: prefixText } = prefix;
    const { text: postfixText } = postfix;
    const parentField = data[parent];
    let label = isIntegerNumber(Number(data[field])) ? Number(data[field]) : data[field];
    label = formatMetric({ value: label, format }) || '';

    const foundParent = mainData.find((option) => option[parent] === parentField);
    const ruleIndex = colorsByRules.rules.findIndex(
        ({ from, to }) => from <= foundParent[colorsByRules.field] && to > foundParent[colorsByRules.field]
    );

    if (ruleIndex < 0) {
        return null;
    }
    let modifiedLabel = label;
    if (prefixText.trim() !== '') {
        modifiedLabel = `${prefixText}${modifiedLabel}`;
    }
    if (postfixText.trim() !== '') {
        modifiedLabel = `${modifiedLabel}${postfixText}`;
    }

    if (data.name === OTHER_NAME) {
        data[mainLabelField] = OTHER_NAME;
    }

    return data[field] === OTHER_NAME ? `{rule_${ruleIndex}|${OTHER_NAME}}` : `{rule_${ruleIndex}|${modifiedLabel}}`;
};

const buildLabels = ({ labels, data, colorsByRules, mainData = null, parent }) =>
    labels
        .filter(({ isShow, field }) => isShow && field)
        .map(({ id, field, prefix, postfix, style: { format } }) => {
            const { isEnabled, isActiveLabelStyles = false } = colorsByRules;
            const mainLabelField = labels[0].field;
            const labelByRules =
                isEnabled && isActiveLabelStyles
                    ? buildLabelByRules({
                          field,
                          prefix,
                          postfix,
                          data,
                          format,
                          colorsByRules,
                          mainData,
                          parent,
                          mainLabelField
                      })
                    : null;

            return labelByRules ?? buildLabelBySetting({ id, field, prefix, postfix, data, format, mainLabelField });
        })
        .join('\n');

const buildLabel = ({
    mainLabel,
    labels,
    drilldown,
    colorsByRules,
    takeUnit2Px,
    mainData,
    parent,
    otherDimensionsStyles
}) => ({
    formatter({ data, treePathInfo }) {
        const firstLevelLabels = labels.filter(({ isDrilldownLabel }) => isDrilldownLabel);
        const defaultLabels = labels.filter(({ isDrilldownLabel }) => !isDrilldownLabel);

        if (data.name === OTHER_NAME) {
            const labelsForOtherDimensions = drilldown.isActive ? firstLevelLabels : defaultLabels;
            mainLabel.isShow = true;
            if (mainLabel.field === null) {
                mainLabel.field = parent;
            }
            return buildLabels({
                labels: [mainLabel, ...labelsForOtherDimensions],
                data,
                colorsByRules,
                mainData,
                parent
            });
        }

        if (mainLabel.isShow && treePathInfo.length === 2) {
            return buildLabels({ labels: [mainLabel, ...firstLevelLabels], data, colorsByRules, mainData, parent });
        }

        if (
            !drilldown.isActive ||
            (mainLabel.isShow && treePathInfo.length === 3 && !drilldown.shouldShowMainLabelOnlyFirstLevel)
        ) {
            defaultLabels.unshift(mainLabel);
        }

        if (drilldown.isActive && !mainLabel.isShow && treePathInfo.length === 2) {
            return buildLabels({ labels: firstLevelLabels, data, colorsByRules, mainData, parent });
        }
        return buildLabels({ labels: defaultLabels, data, colorsByRules, mainData, parent });
    },
    color: drilldown.color,
    rich: generateRichLabel({ labels: [mainLabel, ...labels], colorsByRules, takeUnit2Px, otherDimensionsStyles })
});

const buildSeriesLabel = ({ drilldown, mainLabel, takeUnit2Px }) => ({
    color: drilldown.color,
    fontSize: takeUnit2Px({ size: drilldown.fontSize }),
    position: mainLabel.position
});

const buildUpperLabel = ({ upperLabel, upperLabelIcon, mainLabel, takeUnit2Px }) => ({
    ...upperLabel,
    fontSize: takeUnit2Px({ size: upperLabel.fontSize }),
    emphasis: {
        ...upperLabel,
        fontSize: takeUnit2Px({ size: upperLabel.fontSize })
    },
    formatter({ data }) {
        const { field } = mainLabel;
        const { field: iconField, position } = upperLabelIcon;

        const icon = data[iconField] ? `${data[iconField]}` : null;
        const fieldData = data[field] || '';
        if (icon == null) {
            return `{${field}Icon|}${fieldData}`;
        }

        const iconWithOffset = position === 'row' ? `${icon} ` : ` ${icon}`;
        return position === 'row'
            ? `{${field}Icon|${iconWithOffset}}${fieldData}`
            : `${fieldData}{${field}Icon|${iconWithOffset}}`;
    },
    rich: {
        [mainLabel.field]: {
            ...upperLabel,
            fontSize: takeUnit2Px({ size: mainLabel.fontSize }),
            height: upperLabel.height,
            color: mainLabel.color
        },
        [`${mainLabel.field}Icon`]: {
            fontSize: takeUnit2Px({ size: upperLabelIcon.fontSize }),
            height: upperLabel.height,
            color: upperLabelIcon.color
        }
    }
});

export const buildEchartOptions = ({
    rows,
    treeMapEcharts,
    type,
    levels,
    breadcrumb,
    upperLabel,
    upperLabelIcon,
    drilldown,
    emphasis,
    labels,
    mainLabel,
    roam,
    colorsByRules,
    takeUnit2Px,
    numberOfDimensionsAfterMerge,
    isUniteRestDimensions,
    otherDimensionsStyles
}) => {
    const treeMap = cloneDeep(treeMapEcharts);
    // TODO: implement new tooltip component when it was ready
    // const formatUtil = echarts.format;

    // const tooltip = {
    //     formatter(info) {
    //         const { value } = info;
    //         const { treePathInfo } = info;
    //         const treePaths = treePathInfo.map((pathInfo) => pathInfo.name);
    //         return [
    //             `<div class="tooltip-title"> ${formatUtil.encodeHTML(treePaths.join('/'))} </div>`,
    //             `Цифра: ${formatUtil.addCommas(value)} KB`
    //         ].join('');
    //     }
    // };
    const options = {
        ...treeMap,
        color: levels.color,
        borderColors: levels.borderColors ?? []
    };

    const {
        title,
        title: { textStyle }
    } = options;
    textStyle.fontSize = takeUnit2Px({ size: textStyle.fullFontSize });
    title.borderRadius = takeUnit2Px({ size: textStyle.borderRadius });
    title.backgroundColor = textStyle.backgroundColor;
    title.textStyle = textStyle;
    title.padding = generatePadding({ field: textStyle, takeUnit2Px });

    options.series[0] = {
        ...options.series[0],
        name: treeMap.title.show ? treeMap.title.text : '',
        type,
        data: buildTree({
            rows,
            treeMap,
            levels,
            colorsByRules,
            numberOfDimensionsAfterMerge,
            isUniteRestDimensions,
            drilldown
        }),
        label: buildSeriesLabel({ drilldown, mainLabel, takeUnit2Px }),
        levels: getLevelOptions({ levels, emphasis, colorsByRules }),
        upperLabel: buildUpperLabel({ upperLabel, upperLabelIcon, mainLabel, takeUnit2Px }),
        breadcrumb: buildBreadcrumb({ breadcrumb, takeUnit2Px }),
        leafDepth: drilldown.isActive ? 1 : null,
        drillDownIcon: drilldown.icon,
        sort: false,
        emphasis: {
            upperLabel: {
                color: '#000'
            }
        },
        roam
    };

    options.label = buildLabel({
        mainLabel,
        labels,
        drilldown,
        colorsByRules,
        takeUnit2Px,
        mainData: options.series[0].data,
        parent: options.parentField,
        otherDimensionsStyles
    });

    if (emphasis.isActive) {
        const { color } = emphasis.label.rich;
        options.series[0].emphasis = {
            label: {
                color,
                rich: generateRichLabel({
                    labels: [mainLabel, ...labels],
                    colorsByRules,
                    hoverColor: color,
                    takeUnit2Px,
                    otherDimensionsStyles
                })
            }
        };
    }

    return options;
};
