import { formatNumber } from '@goodt-common/utils-number';
import dayjs from 'dayjs';
import { memoize as _memoize } from 'lodash';
import { transliterate, getDiagramState } from './utils';
import {
    SERIES_TYPE,
    SERIES_SILENT,
    MAIN_SILENT,
    STACK_NAME,
    DEFAULT_VOID_VALUE,
    DiagramStateNames,
    SeriesTypeNames,
    DiagramBarTypeNames,
    DiagramBarNames,
    createLabelOptions
} from './constants';

const memoTransliterate = _memoize(transliterate);

class DataBuilder {
    props = {};

    elemComputedStyles = {};

    constructor(props, elemComputedStyles) {
        this.props = props;
        this.elemComputedStyles = elemComputedStyles;
    }

    buildColorGradient = ({ firstOffset, secondOffset, firstColor, secondColor, position }) => {
        // eslint-disable-next-line id-length
        const [x, y, x2, y2] = position.split(' ');
        const result = {
            type: 'linear',
            x,
            y,
            // eslint-disable-next-line id-length
            x2,
            // eslint-disable-next-line id-length
            y2,
            colorStops: [
                {
                    offset: firstOffset,
                    color: firstColor
                },
                {
                    offset: secondOffset,
                    color: secondColor
                }
            ]
        };
        return result;
    };

    createSeriesDataStyleByType = (type) => {
        const {
            metricOptions: {
                colors: { [type]: color },
                gradient: { [type]: gradient }
            }
        } = this.props;

        const { isUsed: isUsedGradient } = gradient;
        return {
            color: isUsedGradient ? this.buildColorGradient(gradient) : color
        };
    };

    createLabelFormatterString = (values = {}) =>
        Object.keys(values)
            .filter((key) => values[key] != null && values[key] !== '')
            .map((key) => `{${key}|${values[key]}}`)
            .join('');

    createAdditionalBar = ({ value, name, ...data }) => {
        const {
            labelDataOptions: {
                types: {
                    [DiagramBarTypeNames.ADDITIONAL]: { position }
                }
            }
        } = this.props.metricOptions;
        return {
            ...data,
            value,
            name,
            itemStyle: this.createSeriesDataStyleByType(DiagramBarTypeNames.ADDITIONAL),
            label: this.createLabel({
                values: { value, name },
                type: DiagramBarTypeNames.ADDITIONAL,
                position,
                ...data
            })
        };
    };

    createSeriesDataStyleTransparent = () => ({
        borderColor: 'rgba(0,255,0,0)',
        color: 'rgba(0,255,0,0)'
    });

    createSeriesBaseOptionsSilent = () => ({
        name: '',
        type: SERIES_TYPE,
        silent: SERIES_SILENT,
        stack: STACK_NAME
    });

    createSeriesBaseOptionsMinus = () => {
        const {
            props: {
                metricOptions: {
                    labelDataOptions: {
                        types: {
                            minus: {
                                name: { text }
                            }
                        }
                    }
                }
            }
        } = this;
        return {
            name: text || DiagramBarNames.MINUS,
            type: SERIES_TYPE,
            silent: MAIN_SILENT,
            stack: STACK_NAME
        };
    };

    createSeriesBaseOptionsPlus = () => {
        const {
            props: {
                metricOptions: {
                    labelDataOptions: {
                        types: { plus }
                    }
                }
            }
        } = this;
        return {
            name: plus?.name?.text || DiagramBarNames.PLUS,
            type: SERIES_TYPE,
            silent: MAIN_SILENT,
            stack: STACK_NAME
        };
    };

    createSeriesLegend = () => {
        const {
            props: { legend, legendManualSize }
        } = this;
        legend.data = [DiagramBarNames.PLUS, DiagramBarNames.MINUS, DiagramBarNames.ADDITIONAL];
        legend.pageIconInactiveColor = '#cccccc';
        return {
            ...legend,
            ...(legendManualSize.isManualMode ? legendManualSize : {})
        };
    };

    createSeriesBaseOptionsAdditional = () => {
        const {
            props: {
                metricOptions: {
                    labelDataOptions: {
                        types: {
                            additional: {
                                name: { text }
                            }
                        }
                    }
                }
            }
        } = this;
        return {
            name: text || DiagramBarNames.ADDITIONAL,
            type: SERIES_TYPE,
            silent: MAIN_SILENT,
            stack: STACK_NAME
        };
    };

    createAdditionalSeriesOptionsAdditional = () => ({
        ...this.createSeriesBaseOptionsAdditional()
    });

    createSeriesOptionsPlus = () => ({
        ...this.createSeriesBaseOptionsPlus()
    });

    createSeriesOptionsMinus = () => ({
        ...this.createSeriesBaseOptionsMinus()
    });

    createMainSeriesOptions = () => ({
        ...this.createSeriesBaseOptionsSilent()
    });

    createLabel = ({ values = null, position = 'top', type }) => {
        const {
            labelDataOptions: {
                types: { [type]: options }
            }
        } = this.props.metricOptions;

        const { isShown, name, value } = options;
        const formatterString = Object.entries(values)
            .filter(([key, val]) => val !== null && val !== '' && options?.[key]?.isShown)
            .reduce(
                (acc, [key, val]) => [
                    ...acc,
                    this.createLabelFormatterString({
                        [`${key}Prefix`]: options[key].prefix.text,
                        [key]: key === 'value' ? formatNumber(val, value.format) : val,
                        [`${key}Postfix`]: options[key].postfix.text
                    })
                ],
                []
            )
            .join('\n');

        const result = {
            show: isShown,
            position,
            formatter: formatterString ?? null,
            rich: {
                name: createLabelOptions({
                    ...name,
                    prefix: null,
                    postfix: null,
                    fontSize: this.convertVar2Size(name.fontSize)
                }),
                value: createLabelOptions({
                    ...value,
                    prefix: null,
                    postfix: null,
                    fontSize: this.convertVar2Size(value.fontSize)
                }),
                namePrefix: createLabelOptions({
                    ...name.prefix,
                    fontSize: this.convertVar2Size(name.prefix.fontSize)
                }),
                namePostfix: createLabelOptions({
                    ...name.postfix,
                    fontSize: this.convertVar2Size(name.postfix.fontSize)
                }),
                valuePrefix: createLabelOptions({
                    ...value.prefix,
                    fontSize: this.convertVar2Size(value.prefix.fontSize)
                }),
                valuePostfix: createLabelOptions({
                    ...value.postfix,
                    fontSize: this.convertVar2Size(value.postfix.fontSize)
                })
            }
        };
        return result;
    };

    createPositiveSeriesStateBuilders = () => ({
        [DiagramStateNames.DEFAULT]: ({ current, ...data }) => ({
            ...data,
            value: Math.abs(current),
            itemStyle: this.createSeriesDataStyleTransparent()
        }),
        [DiagramStateNames.POSITIVE_TO_NEGATIVE]: ({ total, current, name, ...data }) => {
            const {
                labelDataOptions: {
                    types: {
                        [DiagramBarTypeNames.MINUS]: { position }
                    }
                }
            } = this.props.metricOptions;
            const isShownLabel = position === 'bottom';
            return {
                ...data,
                name,
                value: total,
                ...(isShownLabel
                    ? {
                          label: this.createLabel({
                              values: { value: current, name },
                              position,
                              type: DiagramBarTypeNames.MINUS
                          })
                      }
                    : {}),
                itemStyle: this.createSeriesDataStyleByType(DiagramBarTypeNames.MINUS)
            };
        },
        [DiagramStateNames.NEGATIVE_TO_POSITIVE]: ({ total, current, name, ...data }) => {
            const {
                labelDataOptions: {
                    types: {
                        [DiagramBarTypeNames.PLUS]: { position }
                    }
                }
            } = this.props.metricOptions;
            const isShownLabel = position === 'bottom';
            return {
                ...data,
                name,
                value: total - current,
                ...(isShownLabel
                    ? {
                          label: this.createLabel({
                              values: { value: total - current, name },
                              position,
                              type: DiagramBarTypeNames.PLUS
                          })
                      }
                    : {}),
                itemStyle: this.createSeriesDataStyleByType(DiagramBarTypeNames.PLUS)
            };
        },
        [DiagramStateNames.POSITIVE_PLUS]: ({ current, name, ...data }) => {
            const {
                labelDataOptions: {
                    types: {
                        [DiagramBarTypeNames.PLUS]: { position }
                    }
                }
            } = this.props.metricOptions;
            return {
                ...data,
                name,
                value: current,
                label: this.createLabel({ values: { value: current, name }, type: DiagramBarTypeNames.PLUS, position }),
                itemStyle: this.createSeriesDataStyleByType(DiagramBarTypeNames.PLUS)
            };
        },
        [DiagramStateNames.POSITIVE_MINUS]: ({ ...data }) => ({
            ...data,
            value: '-',
            itemStyle: this.createSeriesDataStyleTransparent()
        }),
        [DiagramStateNames.NEGATIVE_PLUS]: ({ current, name, ...data }) => {
            const {
                labelDataOptions: {
                    types: {
                        [DiagramBarTypeNames.PLUS]: { position }
                    }
                }
            } = this.props.metricOptions;
            return {
                ...data,
                name,
                value: -Math.abs(current),
                label: this.createLabel({ values: { value: current, name }, type: DiagramBarTypeNames.PLUS, position }),
                itemStyle: this.createSeriesDataStyleByType(DiagramBarTypeNames.PLUS)
            };
        },
        [DiagramStateNames.NEGATIVE_MINUS]: ({ total, current, ...data }) => ({
            ...data,
            value: total - current,
            itemStyle: this.createSeriesDataStyleTransparent()
        })
    });

    createNegativeSeriesStateBuilders = () => ({
        [DiagramStateNames.DEFAULT]: ({ current, ...data }) => ({
            ...data,
            value: Math.abs(current),
            itemStyle: this.createSeriesDataStyleTransparent()
        }),
        [DiagramStateNames.POSITIVE_TO_NEGATIVE]: ({ total, current, name, ...data }) => {
            const {
                labelDataOptions: {
                    types: {
                        [DiagramBarTypeNames.MINUS]: { position }
                    }
                }
            } = this.props.metricOptions;
            const isShownLabel = position === 'top';
            return {
                ...data,
                name,
                value: total - current,
                ...(isShownLabel
                    ? {
                          label: this.createLabel({
                              values: { value: total - current, name },
                              position,
                              type: DiagramBarTypeNames.MINUS
                          })
                      }
                    : {}),
                itemStyle: this.createSeriesDataStyleByType(DiagramBarTypeNames.MINUS)
            };
        },
        [DiagramStateNames.NEGATIVE_TO_POSITIVE]: ({ total, current, name, ...data }) => {
            const {
                labelDataOptions: {
                    types: {
                        [DiagramBarTypeNames.PLUS]: { position }
                    }
                }
            } = this.props.metricOptions;
            const isShownLabel = position === 'bottom';
            return {
                ...data,
                name,
                value: total - current,
                ...(isShownLabel
                    ? {
                          label: this.createLabel({
                              values: { value: total - current, name },
                              position,
                              type: DiagramBarTypeNames.PLUS
                          })
                      }
                    : {}),
                itemStyle: this.createSeriesDataStyleByType(DiagramBarTypeNames.PLUS)
            };
        },
        [DiagramStateNames.POSITIVE_PLUS]: ({ total, current, ...data }) => ({
            ...data,
            value: '-',
            itemStyle: this.createSeriesDataStyleTransparent()
        }),
        [DiagramStateNames.POSITIVE_MINUS]: ({ current, name, ...data }) => {
            const {
                labelDataOptions: {
                    types: {
                        [DiagramBarTypeNames.MINUS]: { position }
                    }
                }
            } = this.props.metricOptions;
            return {
                ...data,
                name,
                value: Math.abs(current),
                label: this.createLabel({
                    values: { value: -Math.abs(current), name },
                    position,
                    type: DiagramBarTypeNames.MINUS
                }),
                itemStyle: this.createSeriesDataStyleByType(DiagramBarTypeNames.MINUS)
            };
        },
        [DiagramStateNames.NEGATIVE_PLUS]: ({ total, ...data }) => ({
            ...data,
            value: total,
            itemStyle: this.createSeriesDataStyleTransparent()
        }),
        [DiagramStateNames.NEGATIVE_MINUS]: ({ current, name, ...data }) => {
            const {
                labelDataOptions: {
                    types: {
                        [DiagramBarTypeNames.MINUS]: { position }
                    }
                }
            } = this.props.metricOptions;
            return {
                ...data,
                name,
                value: current,
                label: this.createLabel({
                    values: { value: current, name },
                    position,
                    type: DiagramBarTypeNames.MINUS
                }),
                itemStyle: this.createSeriesDataStyleByType(DiagramBarTypeNames.MINUS)
            };
        }
    });

    createTotalSeriesStateBuilders = () => ({
        [DiagramStateNames.DEFAULT]: ({ total, ...data }) => ({
            ...data,
            value: total,
            itemStyle: this.createSeriesDataStyleTransparent()
        }),
        [DiagramStateNames.POSITIVE_TO_NEGATIVE]: ({ total, current, name, ...data }) => {
            const {
                labelDataOptions: {
                    types: {
                        [DiagramBarTypeNames.MINUS]: { position }
                    }
                }
            } = this.props.metricOptions;
            const isShownLabel = position === 'bottom';
            return {
                ...data,
                name,
                value: total,
                ...(isShownLabel
                    ? {
                          label: this.createLabel({
                              values: { value: current, name },
                              position,
                              type: DiagramBarTypeNames.MINUS
                          })
                      }
                    : {}),
                itemStyle: this.createSeriesDataStyleByType(DiagramBarTypeNames.MINUS)
            };
        },
        [DiagramStateNames.NEGATIVE_TO_POSITIVE]: ({ total, current, name, ...data }) => {
            const {
                labelDataOptions: {
                    types: {
                        [DiagramBarTypeNames.PLUS]: { position }
                    }
                }
            } = this.props.metricOptions;
            const isShownLabel = position === 'top';
            return {
                ...data,
                value: total,
                ...(isShownLabel
                    ? {
                          label: this.createLabel({
                              values: { value: current, name },
                              name,
                              position,
                              type: DiagramBarTypeNames.PLUS
                          })
                      }
                    : {}),
                itemStyle: this.createSeriesDataStyleByType(DiagramBarTypeNames.PLUS)
            };
        },
        [DiagramStateNames.POSITIVE_PLUS]: ({ total, current, ...data }) => ({
            ...data,
            value: total - current,
            itemStyle: this.createSeriesDataStyleTransparent()
        }),
        [DiagramStateNames.POSITIVE_MINUS]: ({ total, ...data }) => ({
            ...data,
            value: total,
            itemStyle: this.createSeriesDataStyleTransparent()
        }),
        [DiagramStateNames.NEGATIVE_PLUS]: ({ total, ...data }) => ({
            ...data,
            value: total,
            itemStyle: this.createSeriesDataStyleTransparent()
        }),
        [DiagramStateNames.NEGATIVE_MINUS]: ({ total, current, ...data }) => ({
            ...data,
            value: total - current,
            itemStyle: this.createSeriesDataStyleTransparent()
        })
    });

    createSeriesStateFunctions = () => ({
        [SeriesTypeNames.POSITIVE]: this.createPositiveSeriesStateBuilders(),
        [SeriesTypeNames.NEGATIVE]: this.createNegativeSeriesStateBuilders(),
        [SeriesTypeNames.TOTAL]: this.createTotalSeriesStateBuilders()
    });

    buildSeriesData = (totalData, deltaValues, valueFunctions) =>
        deltaValues.map(({ value, name }, i) =>
            valueFunctions[getDiagramState(totalData[i], value)]({
                total: totalData[i],
                current: value,
                name: this.BarLabelFormatter(name)
            })
        );

    buildSeriesDataByTypes = (types = [], deltaValues, totalData) => {
        const seriesStateFunctions = this.createSeriesStateFunctions();
        return types.map((type) => this.buildSeriesData(totalData, deltaValues, seriesStateFunctions[type]));
    };

    /**
     * @param { array } seriesData
     * @param { array } deltaValues
     * @param { array } totalData
     * @return {*[]}
     */
    appendAdditionalSeriesData(seriesData, deltaValues = [], totalData = []) {
        const {
            props: {
                metricOptions: {
                    additionalBars: { start, end, others }
                }
            }
        } = this;
        if ([start, end, others].every(({ isUsed }) => isUsed === false)) {
            return seriesData;
        }

        const additionalBars = [
            {
                ...start,
                isAfter: false,
                isTotal: false,
                dimension: deltaValues?.[0]?.name ?? ''
            },
            ...others,
            {
                ...end,
                isAfter: true,
                dimension: deltaValues?.[deltaValues.length - 1]?.name ?? ''
            }
        ].filter(
            ({ dimension, isUsed }) => isUsed && deltaValues.findIndex(({ name }) => `${name}` === `${dimension}`) > -1
        );

        const newSeries = [];
        const additionalData = totalData.reduce((acc, total, i) => {
            const { name: curName, value: current } = deltaValues[i];
            const formattedName = this.BarLabelFormatter(curName);
            const bar = additionalBars.find(({ dimension }) => `${curName}` === `${dimension}`);
            if (bar === undefined) {
                seriesData.forEach((data, j) => {
                    newSeries[j] = [...(newSeries[j] ?? []), data[i]];
                });
                return [
                    ...acc,
                    {
                        value: DEFAULT_VOID_VALUE,
                        name: formattedName
                    }
                ];
            }
            const { isAfter, isTotal, value: barValue, name } = bar;
            if (isAfter) {
                seriesData.forEach((data, j) => {
                    newSeries[j] = [...(newSeries[j] ?? []), data[i], { value: DEFAULT_VOID_VALUE, name }];
                });
                return [
                    ...acc,
                    {
                        value: DEFAULT_VOID_VALUE,
                        name: formattedName
                    },
                    this.createAdditionalBar({
                        value: isTotal ? total : barValue,
                        name
                    })
                ];
            }

            seriesData.forEach((data, j) => {
                newSeries[j] = [...(newSeries[j] ?? []), { value: DEFAULT_VOID_VALUE, name }, data[i]];
            });

            const results = [
                ...acc,
                this.createAdditionalBar({
                    value: isTotal ? total - current : barValue,
                    name
                }),
                {
                    name: curName,
                    value: DEFAULT_VOID_VALUE
                }
            ];
            return results;
        }, []);
        return [...newSeries, additionalData];
    }

    calcTotalData(deltaValues) {
        const {
            props: {
                metricOptions: {
                    additionalBars: {
                        start: { isUsed: isUsedStart, value: startVal }
                    }
                }
            }
        } = this;
        const startValue = isUsedStart ? Number(startVal) : 0;
        return deltaValues.reduce(
            (acc, { value }, i) => [...acc, i === 0 ? startValue + Number(value) : Number(acc[i - 1]) + Number(value)],
            []
        );
    }

    buildSeries(deltaValues, generalOptions) {
        const totalData = this.calcTotalData(deltaValues);

        const seriesData = this.buildSeriesDataByTypes(
            [SeriesTypeNames.TOTAL, SeriesTypeNames.POSITIVE, SeriesTypeNames.NEGATIVE],
            deltaValues,
            totalData
        );
        const [totalSeriesData, positiveSeriesData, negativeSeriesData, additionalSeriesData] =
            this.appendAdditionalSeriesData(seriesData, deltaValues, totalData);

        return [
            {
                ...this.createMainSeriesOptions(),
                data: totalSeriesData,
                generalOptions
            },
            {
                ...this.createSeriesOptionsMinus(),
                ...generalOptions,
                data: negativeSeriesData
            },
            {
                ...this.createSeriesOptionsPlus(),
                ...generalOptions,
                data: positiveSeriesData
            },
            {
                ...this.createAdditionalSeriesOptionsAdditional(),
                ...generalOptions,
                data: additionalSeriesData
            }
        ];
    }

    calcAxisMin({ min }) {
        const {
            format,
            isFullPercentageScale,
            calcMinMax,
            manualMinMax,
            dynamicMinValue,
            minValue,
            isRoundedDynamicMinValue
        } = this;

        if (format === 'percent') {
            return isFullPercentageScale ? 0 : minValue / 100;
        }
        if (calcMinMax === false) {
            return null;
        }
        if (isRoundedDynamicMinValue) {
            return manualMinMax ? minValue : Math.floor(min - (Math.abs(min) / 100) * dynamicMinValue);
        }
        return manualMinMax ? minValue : min - (min / 100) * dynamicMinValue;
    }

    calcAxisMax({ max }) {
        const {
            format,
            isFullPercentageScale,
            calcMinMax,
            manualMinMax,
            enableMax,
            dynamicMaxValue,
            maxValue,
            isRoundedDynamicMaxValue
        } = this;
        if (format === 'percent') {
            if (isFullPercentageScale) {
                return 1;
            }
            return enableMax ? maxValue / 100 : null;
        }
        if (calcMinMax === false) {
            return null;
        }
        if (manualMinMax === true) {
            return enableMax ? maxValue : null;
        }
        if (isRoundedDynamicMaxValue) {
            return Math.ceil(max + (Math.abs(max) / 100) * dynamicMaxValue);
        }
        return max + (max / 100) * dynamicMaxValue;
    }

    valueAxisLabelFormatter(value) {
        const {
            axisLabel: { format }
        } = this;
        return formatNumber(value, format);
    }

    categoryAxisLabelFormatter(dimFormat, value) {
        const {
            axisLabel: { type, longValuesLength }
        } = this;
        const formattedValue = dimFormat === 'string' ? value : dayjs(value).format(dimFormat);

        const axisLabelFormats = {
            text: () =>
                longValuesLength === 0
                    ? formattedValue
                    : formattedValue
                          .split(/\s+/g)
                          .reduce(
                              (acc, word) =>
                                  acc[acc.length - 1] === undefined ||
                                  acc[acc.length - 1].length >= longValuesLength ||
                                  acc[acc.length - 1].length + word.length + 1 > longValuesLength
                                      ? [...acc, word]
                                      : [...acc.slice(0, acc.length - 1), `${acc[acc.length - 1]} ${word}`],
                              []
                          )
                          .join('\n'),
            image: () => `{${memoTransliterate(formattedValue).replace(/\s/g, '_')}|}`,
            'text&image': () => `${axisLabelFormats.image(formattedValue)}\n${axisLabelFormats.text(formattedValue)}`
        };
        return axisLabelFormats[type]();
    }

    BarLabelFormatter(value) {
        const {
            props: {
                metricOptions: {
                    labelDataOptions: { maxLabelLength }
                }
            }
        } = this;
        const barLabelFormats = {
            text: () =>
                maxLabelLength === 0
                    ? value
                    : value
                          .split(/\s+/g)
                          .reduce(
                              (acc, word) =>
                                  acc[acc.length - 1] === undefined ||
                                  acc[acc.length - 1].length >= maxLabelLength ||
                                  acc[acc.length - 1].length + word.length + 1 > maxLabelLength
                                      ? [...acc, word]
                                      : [...acc.slice(0, acc.length - 1), `${acc[acc.length - 1]} ${word}`],
                              []
                          )
                          .join('\n')
        };
        return barLabelFormats.text();
    }

    convertVar2Size(size = '') {
        const cssVar = `${size}`.match(/--[-_0-9a-zA-Z%]+/)?.[0];
        const cssVarsSize = cssVar != null ? this.elemComputedStyles?.getPropertyValue?.(cssVar) ?? null : null;

        return cssVar != null ? cssVarsSize : size;
    }

    buildAxes = (categoryNames, format) =>
        this.props.axes.reduce(
            (axes, item, i) => ({
                ...axes,
                [item.typeAxis === 'x' ? 'xAxis' : 'yAxis']: [
                    ...axes[item.typeAxis === 'x' ? 'xAxis' : 'yAxis'],
                    {
                        ...item,
                        id: i,
                        axisLine: {
                            ...item.axisLine,
                            onZero: false
                        },
                        name: item.showAxisName ? item.name : '',
                        ...(item.type === 'category'
                            ? {
                                  boundaryGap: item.hasGap,
                                  data: categoryNames,
                                  axisLabel: {
                                      ...item.axisLabel,
                                      fontSize: this.convertVar2Size(item.valueFontSize),
                                      formatter: this.categoryAxisLabelFormatter.bind(item, format)
                                  }
                              }
                            : {
                                  axisLabel: {
                                      ...item.axisLabel,
                                      fontSize: this.convertVar2Size(item.valueFontSize),
                                      formatter: this.valueAxisLabelFormatter.bind(item)
                                  },
                                  boundaryGap: item.hasGap ? [0, '20%'] : [0, 0],
                                  min: this.calcAxisMin.bind(item),
                                  max: this.calcAxisMax.bind(item),
                                  interval: item.interval === 0 ? null : item.interval
                              })
                    }
                ]
            }),
            { xAxis: [], yAxis: [] }
        );
}

export default (settings, elemComputedStyles) => new DataBuilder(settings, elemComputedStyles);
