/* eslint-disable id-length */
import { dayjs } from '@goodt-common/utils';

/**
 * @typedef ConditionType
 * @enum {string}
 * @type {Readonly<Record<string, string>>}
 */
export const ConditionType = Object.freeze({
    ALL: 'all',
    EQ: 'eq',
    GT: 'gt',
    LT: 'lt'
});

/**
 * @typedef ConditionValueFormat
 * @enum {string}
 * @type {Readonly<Record<string, string>>}
 */
export const ConditionValueFormat = Object.freeze({
    NUMBER: 'number',
    DATE: 'date'
});

/**
 * @enum {string}
 * @type {Readonly<Record<ConditionType, {name: string, handler: () => boolean}>>}
 */
export const ConditionTypes = Object.freeze({
    [ConditionType.ALL]: {
        name: 'всегда',
        handler: () => true
    },
    [ConditionType.EQ]: {
        name: '=',
        handler: (paramValue, styleValue, format) => {
            if (format === ConditionValueFormat.NUMBER) {
                return Number.isFinite(Number(paramValue)) && Number.isFinite(Number(styleValue))
                    ? Number(paramValue) === Number(styleValue)
                    : paramValue === styleValue;
            }
            return dayjs(paramValue).isValid() && dayjs(styleValue).isValid()
                ? dayjs(paramValue).diff(dayjs(styleValue)) === 0
                : paramValue === styleValue;
        }
    },
    [ConditionType.GT]: {
        name: '>',
        handler: (paramValue, styleValue, format) => {
            if (format === ConditionValueFormat.NUMBER) {
                return Number(paramValue) > Number(styleValue);
            }
            return dayjs(paramValue).isValid() && dayjs(styleValue).isValid()
                ? dayjs(paramValue).diff(dayjs(styleValue)) > 0
                : paramValue > styleValue;
        }
    },
    [ConditionType.LT]: {
        name: '<',
        handler: (paramValue, styleValue, format) => {
            if (format === ConditionValueFormat.NUMBER) {
                return Number(paramValue) < Number(styleValue);
            }
            return dayjs(paramValue).isValid() && dayjs(styleValue).isValid()
                ? dayjs(paramValue).diff(dayjs(styleValue)) < 0
                : paramValue < styleValue;
        }
    }
});

/**
 * @typedef ValueType
 * @enum {string}
 * @type {Readonly<Record<string, string>>}
 */
export const ValueType = Object.freeze({
    MD: 'md',
    INPUT: 'input'
});

/**
 * @enum {string}
 * @type {Readonly<Record<ValueType, {name: string, handler: () => any}>>}
 */
export const ValueTypes = Object.freeze({
    [ValueType.MD]: {
        name: 'метрика/измерение',
        handler: (value, row) => row[value] ?? null
    },
    [ValueType.INPUT]: { name: 'ввод', handler: (value) => value }
});

/**
 * @param {Record<string, any>} applicable
 * @return {{
 *     type: ConditionType,
 *     format: ConditionValueFormat,
 *     valueCompared: string,
 *     valueComparedType: ValueType,
 *     valueToCompareType: string,
 *     valueToCompareType: ValueType
 * } & Record<string, any>}
 */
export const StyleConditionFactory = (applicable) => ({
    type: ConditionType.ALL,
    format: ConditionValueFormat.NUMBER,
    valueCompared: '',
    valueComparedType: ValueType.MD,
    valueToCompare: '',
    valueToCompareType: ValueType.MD,
    ...applicable
});

/**
 * @enum {string}
 * @type {Readonly<Record<string, string>>}
 */
export const ControlType = Object.freeze({
    CHECKBOX: 'checkbox',
    RADIO: 'radio'
});
