import { merge, isEmpty } from 'lodash';

/**
 * @return {Record<string, string>}
 */
const Events = Object.freeze({
    SELECT: 'select',
    UNSELECT: 'unselect'
});

/**
 * @typedef {Object} Actions
 * @property {{varsFromSource: Record<string, any>, varsFromUser: Record<string, any>}} varsToCommit
 * @property {string[]} events
 * @property {{url: string, query: Record<string, string>, isTargetBlank: boolean}} [link]
 */

/**
 * Actions mixin, used for widgets, that have access to the row data
 * @depends useActionsDescriptor()
 * @return {Record<string, any>}
 */
export const useActions = () => ({
    inject: {
        table: {
            default: null
        },
        row: {
            default: null
        }
    },
    computed: {
        /**
         * @return {Record<string, any>[]}
         */
        selectedRows() {
            return this.row?.selectedRows ?? [];
        },
        /**
         * @return {boolean}
         */
        isSelectedRow() {
            return this.row?.isSelectedRow ?? false;
        },
        /**
         * @return {string[]}
         */
        dimensionMetricNames() {
            return this.table?.dimensionMetricNames ?? [];
        },
        /**
         * @return {Actions}
         */
        actions() {
            const { isSelectedRow, rowData, props } = this;

            if (isSelectedRow) {
                const {
                    varsToReset,
                    events: { [Events.UNSELECT]: events }
                } = props;
                const { varsFromSource, varsFromUser } = varsToReset.reduce(
                    (acc, variable) => {
                        if (rowData[variable] === undefined) {
                            acc.varsFromUser[variable] = null;
                            return acc;
                        }

                        acc.varsFromSource[variable] = null;
                        return acc;
                    },
                    { varsFromSource: {}, varsFromUser: {} }
                );
                return { varsToCommit: { varsFromSource, varsFromUser }, events };
            }

            const {
                varsToCommit,
                events: { [Events.SELECT]: events },
                link
            } = props;
            const { fromSource, fromUser: varsFromUser } = varsToCommit;
            const varsFromSource = fromSource.reduce(
                (acc, variable) => ({
                    ...acc,
                    [variable]: rowData[variable]
                }),
                {}
            );
            return {
                varsToCommit: { varsFromSource, varsFromUser },
                events: events.reduce((acc, event) => [...acc, rowData[event] ?? event], []),
                link: { ...link, url: rowData[link.url] ?? link.url }
            };
        },
        hasActiveActions() {
            const { varsToCommit, events, link } = this.actions;
            return [varsToCommit.varsFromSource, varsToCommit.varsFromUser, events, link?.url].every(isEmpty) === false;
        }
    },
    methods: {
        /**
         * @param {Actions} [actions]
         * @return {Actions}
         */
        createActions(actions = {}) {
            const { link, ...restActions } = this.actions;

            if (['', null, undefined].includes(link?.url)) {
                return merge(actions, restActions);
            }

            return merge(actions, this.actions);
        },
        /**
         * @param {Actions} actions
         * @param {boolean} shouldSelectRow
         */
        dispatchActions(actions, shouldSelectRow) {
            const event = new CustomEvent('actions-send', {
                bubbles: true,
                detail: {
                    actions,
                    shouldSelectRow
                }
            });

            this.$el.dispatchEvent(event);
        }
    }
});

/**
 * Actions descriptor factory
 * @param {() => Record<string, any>} descriptor         custom descriptor factory to merge with
 * @return {() => Record<string, any>}
 */
export function useActionsDescriptor(descriptor) {
    return () =>
        merge(
            {
                props: {
                    varsToCommit: {
                        type: Object,
                        default: () => ({
                            // eslint-disable-next-line no-restricted-syntax
                            fromSource: [],
                            fromUser: {}
                        })
                    },
                    varsToReset: {
                        type: Array,
                        default: () => []
                    },
                    events: {
                        type: Object,
                        default: () => ({
                            [Events.SELECT]: [],
                            [Events.UNSELECT]: []
                        })
                    },
                    link: {
                        type: Object,
                        default: () => ({
                            url: '',
                            query: {},
                            isTargetBlank: false
                        })
                    },
                    canSelectRow: {
                        type: Boolean,
                        default: false
                    }
                }
            },
            descriptor()
        );
}

/**
 * Actions panel factory
 * @return {() => Promise<typeof ActionsPanel>}
 */
export function useActionsPanel() {
    return () => import('../panels/ActionsPanel.vue');
}
