import { merge } from 'lodash';
import { shouldUseCssRule } from '../utils';

/**
 * Style conditions mixin, used for widgets that can use style conditions
 * @depends useStyleConditionsDescriptor()
 * @return {Record<string, any>}
 */
export const useStyleConditions = () => ({
    computed: {
        /**
         * @return {{cssStyle: Record<string, any>, cssClass: Record<string, boolean>}}
         */
        computedStyleConditions() {
            const {
                rowData,
                props: { styleConditions }
            } = this;
            return styleConditions.reduce(
                (acc, condition) => {
                    if (shouldUseCssRule(condition, rowData)) {
                        acc.cssStyle = { ...acc.cssStyle, ...condition.cssStyle };
                        acc.cssClass = { ...acc.cssClass, [condition.cssClass]: true };
                    }
                    return acc;
                },
                { cssStyle: {}, cssClass: {} }
            );
        },
        /**
         * @return {Record<string, any>}
         */
        cellCssStyle() {
            return this.computedStyleConditions.cssStyle;
        },
        /**
         * @return {Record<string, boolean>}
         */
        cellCssClass() {
            return this.computedStyleConditions.cssClass;
        }
    }
});

/**
 * Style conditions descriptor factory
 * @param {() => Record<string, any>} descriptor         custom descriptor factory to merge with
 * @return {() => Record<string, any>}
 */
export function useStyleConditionsDescriptor(descriptor) {
    return () =>
        merge(
            {
                props: {
                    styleConditions: {
                        type: Array,
                        default: () => [],
                        applicable: { cssStyle: {}, cssClass: [] }
                    }
                }
            },
            descriptor()
        );
}

/**
 * Style conditions panel factory
 * @return {() => Promise<typeof StyleConditionsPanel>}
 */
export function useStyleConditionsPanel() {
    return () => import('../panels/StyleConditionsPanel.vue');
}
