package com.goodt.drive.report.application.controllers.api;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.goodt.drive.report.GlobalDefs;
import com.goodt.drive.report.application.asm.ReportAsm;
import com.goodt.drive.report.application.data.OperationResult;
import com.goodt.drive.report.application.dto.ReportCreateDto;
import com.goodt.drive.report.application.dto.ReportDto;
import com.goodt.drive.report.application.dto.ReportGenerateDto;
import com.goodt.drive.report.application.dto.keycloak.TokenDataDtoBase;
import com.goodt.drive.report.application.entities.ReportEntity;
import com.goodt.drive.report.application.exception.DremioResponseException;
import com.goodt.drive.report.application.exception.UserTokenDataException;
import com.goodt.drive.report.application.factories.ReportFactory;
import com.goodt.drive.report.application.services.ReportServiceImpl;
import com.goodt.drive.report.application.services.TemplateService;
import com.goodt.drive.report.application.services.YargService;
import com.haulmont.yarg.reporting.ReportOutputDocument;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.media.ArraySchema;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.responses.ApiResponses;
import javassist.NotFoundException;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.io.IOException;
import java.nio.file.Path;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.stream.Collectors;

@RestController
@RequestMapping("/api/report")
public class ReportController extends BaseController {

    private final YargService yargService;

    private final TemplateService templateService;

    private final ReportServiceImpl reportService;

    private final ReportAsm reportAsm;

    private final String templatePrefix = "template_";

    private final Logger logger = LoggerFactory.getLogger(this.getClass());

    @Autowired
    public ReportController(YargService yargService, TemplateService templateService, ReportServiceImpl reportService, ReportAsm reportAsm) {
        this.yargService = yargService;
        this.templateService = templateService;
        this.reportService = reportService;
        this.reportAsm = reportAsm;
    }

    @Operation(summary = "Список отчётов", description = "Список отчётов", tags = { "report" })
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "В случае если удалось построить DTO (все необходимые настройки заданы в yml-файле)",
                    content = @Content(array = @ArraySchema(schema = @Schema(implementation = ReportDto.class)))),
            @ApiResponse(responseCode = "401", description = "В случае если запрос отправлен без токена или с недействительным токеном возвращается ошибка 401 и строка с ошибкой в теле запроса",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "500", description = "В случае если: что-то сломалось или не полностью сконфигрурировано приложение",
                    content = @Content(schema = @Schema(implementation = String.class))),
    })
    @GetMapping
    public @ResponseBody List<ReportDto> list() {
        List<ReportEntity> reportEntities = reportService.getAll();
        return reportEntities.stream().map(ReportFactory::create).collect(Collectors.toList());
    }

    @Operation(summary = "Получить отчёт", description = "Получить отчёт", tags = { "report" })
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "В случае если удалось построить DTO (все необходимые настройки заданы в yml-файле)",
                    content = @Content(array = @ArraySchema(schema = @Schema(implementation = ReportDto.class)))),
            @ApiResponse(responseCode = "401", description = "В случае если запрос отправлен без токена или с недействительным токеном возвращается ошибка 401 и строка с ошибкой в теле запроса",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "500", description = "В случае если: что-то сломалось или не полностью сконфигрурировано приложение",
                    content = @Content(schema = @Schema(implementation = String.class))),
    })
    @GetMapping("/{id}")
    public @ResponseBody ReportDto get(@PathVariable(name = "id", required = false)
                                       @Parameter(name = "id", description = "Идентификатор директории (таблица report).", example = "1")
                                       Long id) {
        return reportService.getById(id, reportAsm::toRes);
    }

    @Operation(summary = "Добавление отчёта", description = "Добавление отчёта", tags = { "report" })
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "В случае если удалось построить DTO (все необходимые настройки заданы в yml-файле)",
                    content = @Content(schema = @Schema(implementation = ReportDto.class))),
            @ApiResponse(responseCode = "401", description = "В случае если запрос отправлен без токена или с недействительным токеном возвращается ошибка 401 и строка с ошибкой в теле запроса",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "500", description = "В случае если: что-то сломалось или не полностью сконфигрурировано приложение",
                    content = @Content(schema = @Schema(implementation = String.class))),
    })
    @PostMapping
    public @ResponseBody ReportDto add(@io.swagger.v3.oas.annotations.parameters.RequestBody(description = "DTO представление директорий (таблица report)")
                                       @RequestBody ReportDto dto) {
        return reportService.create(entity -> reportAsm.create(entity, dto), reportAsm::toRes);
    }

    @Operation(summary = "Обновление отчёта", description = "Обновление отчёта", tags = { "report" })
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "В случае если удалось построить DTO (все необходимые настройки заданы в yml-файле)",
                    content = @Content(schema = @Schema(implementation = ReportDto.class))),
            @ApiResponse(responseCode = "401", description = "В случае если запрос отправлен без токена или с недействительным токеном возвращается ошибка 401 и строка с ошибкой в теле запроса",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "500", description = "В случае если: что-то сломалось или не полностью сконфигрурировано приложение",
                    content = @Content(schema = @Schema(implementation = String.class))),
    })
    @PutMapping("/{id}")
    public @ResponseBody ReportDto update(@PathVariable(name = "id", required = false)
                                          @Parameter(name = "id", description = "Идентификатор директории (таблица report).", example = "1")
                                          Long id,
                                          @io.swagger.v3.oas.annotations.parameters.RequestBody(description = "DTO представление директорий (таблица report)")
                                          @RequestBody ReportDto dto) {
        ReportEntity update = reportService.update(id, entity -> reportAsm.update(entity, dto));
        return reportAsm.toRes(update);
    }

    @Operation(summary = "Удаление отчёта", description = "Удаление отчёта", tags = { "report" })
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "В случае если удалось построить DTO (все необходимые настройки заданы в yml-файле)",
                    content = @Content(schema = @Schema(implementation = OperationResult.class))),
            @ApiResponse(responseCode = "401", description = "В случае если запрос отправлен без токена или с недействительным токеном возвращается ошибка 401 и строка с ошибкой в теле запроса",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "500", description = "В случае если: что-то сломалось или не полностью сконфигрурировано приложение",
                    content = @Content(schema = @Schema(implementation = String.class))),
    })
    @DeleteMapping("/{id}")
    public OperationResult delete(@PathVariable(name = "id", required = false)
                                      @Parameter(name = "id", description = "Идентификатор директории (таблица report).", example = "1")
                                      Long id) {
        reportService.delete(id);
        return new OperationResult(true, "");
    }

    @Operation(summary = "Создание отчёта", description = "Создание отчёта", tags = { "report" })
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "В случае если удалось построить отчёт",
                    content = @Content(schema = @Schema(implementation = ResponseEntity.class))),
            @ApiResponse(responseCode = "401", description = "В случае если запрос отправлен без токена или с недействительным токеном возвращается ошибка 401 и строка с ошибкой в теле запроса",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "500", description = "В случае если: что-то сломалось или не полностью сконфигрурировано приложение",
                    content = @Content(schema = @Schema(implementation = String.class))),
    })
    @PostMapping("generate")
    public @ResponseBody ResponseEntity<byte[]> generate(@RequestBody ReportGenerateDto body) throws DremioResponseException {
        try {
            TokenDataDtoBase userTokenDataDtoBase = null;
            try {
                userTokenDataDtoBase = getUserTokenDataDtoBase();
            } catch (UserTokenDataException e) {
                logger.debug(e.getMessage());
            }
            ReportDto reportDto = reportService.getById(body.getReportId(), reportAsm::toRes);
            List<LinkedHashMap<String, Object>> data = new ObjectMapper().readValue(reportDto.getData(), new TypeReference<List<LinkedHashMap<String, Object>>>() {});
            ReportCreateDto reportCreateDto = new ReportCreateDto(reportDto.getTemplateId(), body.getTemplateType(), data);
            String correctedTemplateId = correctTemplateId(reportCreateDto.getTemplateId());
            reportCreateDto.setTemplateId(correctedTemplateId);
            ReportOutputDocument reportOutputDocument = yargService.create(reportCreateDto, false, userTokenDataDtoBase);
            return buildResponse(reportOutputDocument);
        } catch (IOException | NotFoundException e) {
            logger.error(String.format("An error occurred during generate report: %s", e.getMessage()));
        }
        return null;
    }

    @Operation(summary = "Создание отчёта", description = "Создание отчёта", tags = { "report" })
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "В случае если удалось построить отчёт",
                    content = @Content(schema = @Schema(implementation = ResponseEntity.class))),
            @ApiResponse(responseCode = "401", description = "В случае если запрос отправлен без токена или с недействительным токеном возвращается ошибка 401 и строка с ошибкой в теле запроса",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "500", description = "В случае если: что-то сломалось или не полностью сконфигрурировано приложение",
                    content = @Content(schema = @Schema(implementation = String.class))),
    })
    @PostMapping("create")
    public @ResponseBody ResponseEntity<byte[]> create(@RequestBody ReportCreateDto body) throws DremioResponseException {
        try {
            TokenDataDtoBase userTokenDataDtoBase = null;
            try {
                userTokenDataDtoBase = getUserTokenDataDtoBase();
            } catch (UserTokenDataException e) {
                logger.debug(e.getMessage());
            }
            String correctedTemplateId = correctTemplateId(body.getTemplateId());
            body.setTemplateId(correctedTemplateId);
            ReportOutputDocument reportOutputDocument = yargService.create(body, false, userTokenDataDtoBase);
            return buildResponse(reportOutputDocument);
        } catch (IOException | NotFoundException e) {
            logger.error(String.format("An error occurred during generate report: %s", e.getMessage()));
        }
        return null;
    }

    private String correctTemplateId(String templateId) {
        if (StringUtils.isNumeric(templateId)) {
            List<Path> templates = templateService.findAll().stream()
                    .filter(path -> path.getParent().getFileName().toString().equals(GlobalDefs.TEMPLATES_GLOBAL_DIR))
                    .filter(path -> path.getFileName().toString().contains(templatePrefix + templateId))
                    .collect(Collectors.toList());
            int templatesCount = templates.size();
            if (templatesCount > 1) {
                throw new IndexOutOfBoundsException(String.format("Multiple (%d) templates found with templateId \"%s\".", templatesCount, templateId));
            } else if (templatesCount == 1) {
                Path templatePath = templates.get(0);
                return templatePath.getParent().getFileName().toString() + "/" + templatePath.getFileName().toString();
            }
        }
        return templateId;
    }

    private ResponseEntity<byte[]> buildResponse(ReportOutputDocument reportOutputDocument) {
        byte[] contents = reportOutputDocument.getContent();
        HttpHeaders headers = new HttpHeaders();
        switch (reportOutputDocument.getReportOutputType().getId()) {
            case "xls":
                headers.setContentType(MediaType.parseMediaType("application/vnd.ms-excel"));
                break;
            case "xlsx":
                headers.setContentType(MediaType.parseMediaType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet"));
                break;
            case "doc":
                headers.setContentType(MediaType.parseMediaType("application/msword"));
                break;
            case "docx":
                headers.setContentType(MediaType.parseMediaType("application/vnd.openxmlformats-officedocument.wordprocessingml.document"));
                break;
            case "pdf":
                headers.setContentType(MediaType.parseMediaType("application/pdf"));
                break;
            default:
                headers.setContentType(MediaType.parseMediaType("application/octet-stream"));
        }
        headers.set("Content-Disposition", "attachment; filename=" + reportOutputDocument.getDocumentName());
        headers.setContentLength(contents.length);
        headers.setCacheControl("must-revalidate, post-check=0, pre-check=0");
        return new ResponseEntity<byte[]>(contents, headers, HttpStatus.OK);
    }
}
