package com.goodt.drive.report.application.controllers.api;

import com.goodt.drive.report.application.data.OperationResult;
import com.goodt.drive.report.application.dto.NamespaceDto;
import com.goodt.drive.report.application.dto.TemplateDto;
import com.goodt.drive.report.application.factories.TemplateFactory;
import com.goodt.drive.report.application.services.TemplateService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.ArraySchema;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.responses.ApiResponses;
import javassist.NotFoundException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.*;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.activation.MimetypesFileTypeMap;
import java.io.IOException;
import java.net.URLConnection;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.List;
import java.util.stream.Collectors;

@RestController
@RequestMapping("/api/template")
public class TemplateController extends BaseController {

    @Value("${yarg.templates.create}")
    private boolean createEnabled;

    private final TemplateService templateService;

    private final Logger logger = LoggerFactory.getLogger(this.getClass());

    @Autowired
    public TemplateController(TemplateService templateService) {
        this.templateService = templateService;
    }

    @Operation(summary = "Список шаблонов", description = "Список шаблонов", tags = { "template" })
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "В случае если удалось построить DTO (все необходимые настройки заданы в yml-файле)",
                    content = @Content(array = @ArraySchema(schema = @Schema(implementation = TemplateDto.class)))),
            @ApiResponse(responseCode = "401", description = "В случае если запрос отправлен без токена или с недействительным токеном возвращается ошибка 401 и строка с ошибкой в теле запроса",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "500", description = "В случае если: что-то сломалось или не полностью сконфигрурировано приложение",
                    content = @Content(schema = @Schema(implementation = String.class))),
    })
    @GetMapping("list")
    public @ResponseBody List<TemplateDto> templates() {
        List<Path> paths = templateService.findAll();
        return paths.stream().map(path -> TemplateFactory.create(path)).collect(Collectors.toList());
    }

    @Operation(summary = "Скачать шаблон", description = "Скачать шаблон", tags = { "template" })
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "В случае если удалось построить DTO (все необходимые настройки заданы в yml-файле)",
                    content = @Content(schema = @Schema(implementation = ResponseEntity.class))),
            @ApiResponse(responseCode = "401", description = "В случае если запрос отправлен без токена или с недействительным токеном возвращается ошибка 401 и строка с ошибкой в теле запроса",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "500", description = "В случае если: что-то сломалось или не полностью сконфигрурировано приложение",
                    content = @Content(schema = @Schema(implementation = String.class))),
    })
    @GetMapping("download/{folder}/{filename}")
    public ResponseEntity<byte[]> download(@PathVariable("folder") String folder,
                                           @PathVariable("filename") String filename) throws NotFoundException, IOException {
        String templateId = getTemplateId(folder, filename);
        Path file = templateService.find(templateId);
        byte[] bytes = new byte[0];
        try {
            bytes = Files.readAllBytes(file);
        } catch (IOException e) {
            logger.error("An error occurred during read file {} for templateId {}", file.toAbsolutePath(), templateId);
            throw new IOException(e);
        }
        HttpHeaders headers = new HttpHeaders();
        try {
            headers.setContentLength(Files.size(file));
        } catch (IOException e) {
            logger.error("An error occurred during read file size {} for templateId {}", file.toAbsolutePath(), templateId);
            throw new IOException(e);
        }
        String contentType = getContentType(file);
        headers.setContentType(MediaType.valueOf(contentType));
        headers.setContentDisposition(ContentDisposition.attachment().filename(file.getFileName().toString()).build());
        return new ResponseEntity<>(bytes, headers, HttpStatus.OK);
    }

    @Operation(summary = "Загрузить шаблон", description = "Загрузить шаблон", tags = { "template" })
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "В случае если удалось построить DTO (все необходимые настройки заданы в yml-файле)",
                    content = @Content(schema = @Schema(implementation = TemplateDto.class))),
            @ApiResponse(responseCode = "401", description = "В случае если запрос отправлен без токена или с недействительным токеном возвращается ошибка 401 и строка с ошибкой в теле запроса",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "500", description = "В случае если: что-то сломалось или не полностью сконфигрурировано приложение",
                    content = @Content(schema = @Schema(implementation = String.class))),
    })
    @PostMapping("create")
    public TemplateDto create(@RequestParam("template") MultipartFile file) throws IOException {
        if (!createEnabled) {
            throw new IllegalArgumentException("Create templates disabled");
        }
        return TemplateFactory.create(templateService.create(file));
    }

    @Operation(summary = "Удалить шаблон", description = "Удалить шаблон", tags = { "template" })
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "В случае если удалось построить DTO (все необходимые настройки заданы в yml-файле)",
                    content = @Content(schema = @Schema(implementation = OperationResult.class))),
            @ApiResponse(responseCode = "401", description = "В случае если запрос отправлен без токена или с недействительным токеном возвращается ошибка 401 и строка с ошибкой в теле запроса",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "500", description = "В случае если: что-то сломалось или не полностью сконфигрурировано приложение",
                    content = @Content(schema = @Schema(implementation = String.class))),
    })
    @DeleteMapping("delete/{folder}/{filename}")
    public OperationResult delete(@PathVariable("folder") String folder,
                                  @PathVariable("filename") String filename) throws NotFoundException, IOException {
        if (!createEnabled) {
            return new OperationResult(false, "Delete templates disabled");
        }
        try {
            String templateId = getTemplateId(folder, filename);
            boolean deleted = templateService.delete(templateId);
            return new OperationResult(deleted, null);
        } catch (IllegalArgumentException e) {
            return new OperationResult(false, e.getMessage());
        }
    }

    @Operation(summary = "Получить список пространств имён шаблонов", description = "Получить список пространств имён шаблонов", tags = { "template" })
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "В случае если удалось построить DTO (все необходимые настройки заданы в yml-файле)",
                    content = @Content(array = @ArraySchema(schema = @Schema(implementation = NamespaceDto.class)))),
            @ApiResponse(responseCode = "401", description = "В случае если запрос отправлен без токена или с недействительным токеном возвращается ошибка 401 и строка с ошибкой в теле запроса",
                    content = @Content(schema = @Schema(implementation = String.class))),
            @ApiResponse(responseCode = "500", description = "В случае если: что-то сломалось или не полностью сконфигрурировано приложение",
                    content = @Content(schema = @Schema(implementation = String.class))),
    })
    @GetMapping("namespaces/{folder}/{filename}")
    public List<NamespaceDto> namespaces(@PathVariable("folder") String folder,
                                         @PathVariable("filename") String filename) throws NotFoundException, IOException {
        String templateId = getTemplateId(folder, filename);
        return templateService.findNamespaces(templateId);
    }

    private static String getContentType(Path file) {
        String contentType = URLConnection.guessContentTypeFromName(file.getFileName().toString());
        if (contentType == null) {
            contentType = new MimetypesFileTypeMap().getContentType(file.toFile());
        }
        return contentType;
    }

    private static String getTemplateId(String folder, String filename) {
        return String.join("/", folder, filename);
    }
}
