package com.goodt.drive.report.application.repository;

import com.goodt.drive.report.application.entity.permissions.EmployeePermissions;
import com.goodt.drive.report.application.entity.permissions.TablePermissions;
import com.goodt.drive.report.application.entity.query.QueryResult;
import com.goodt.drive.report.application.enums.EClientDataType;
import com.goodt.drive.report.application.system.ExpiredConcurrentHashMap;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.support.rowset.SqlRowSet;
import org.springframework.jdbc.support.rowset.SqlRowSetMetaData;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Repository;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.*;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.atomic.AtomicReference;

@Repository
public class DremioQueryImpl implements IDremioQuery {

    private static Logger logger = LoggerFactory.getLogger(DremioQueryImpl.class);

    @Autowired
    JdbcTemplate jdbcTemplate;

    @Autowired
    @Qualifier("permission")
    ExpiredConcurrentHashMap<String, EmployeePermissions> employeeCache;

    @Autowired
    @Qualifier("table-permission")
    ExpiredConcurrentHashMap<String, List<TablePermissions>> tablePermissionCache;

    @Value("${system.data.catalog}")
    String permissionsCatalog;

    @Async("threadPoolTaskExecutor")
    @Override
    public CompletableFuture<QueryResult> execute(String sql, UUID logId) {
        logger.info("{}: Execute query({})", logId, sql);
        QueryResult queryResult = new QueryResult();

        AtomicReference<Long> rowCounter = new AtomicReference<>(0l);

        jdbcTemplate.query(sql, new Object[]{}, rs -> {
            if (rs.getRow() == 1) {
                generateSchema(queryResult, rs);
            }
            Map<String, String> values = new HashMap<>();
            for (int k = 1; k <= rs.getMetaData().getColumnCount(); k++) {
                values.put(rs.getMetaData().getColumnName(k), rs.getString(k));
            }
            queryResult.addRow(values);
            rowCounter.getAndUpdate(x -> x + 1);
        });
        logger.info("{}: query returned {} rows", logId, rowCounter.get());

        queryResult.setRowCount(rowCounter.get());
        return CompletableFuture.completedFuture(queryResult);
    }

    @Async("threadPoolTaskExecutor")
    @Override
    public CompletableFuture<Long> getRowCount(String sql) {
        String sqlCnt = "select count(*) rowCount from (" + sql + ")";
        logger.debug("getRowCount(): " + sqlCnt);

        SqlRowSet sqlRowSet = jdbcTemplate.queryForRowSet(sqlCnt);
        if (sqlRowSet.next())
            return CompletableFuture.completedFuture(sqlRowSet.getLong("rowCount"));

        return CompletableFuture.completedFuture(0L);
    }

    @Async("threadPoolTaskExecutor")
    @Override
    public CompletableFuture<EmployeePermissions> getEmployeePermissions(String employeeId, List<String> path, UUID logId) {
        logger.debug("{}: getEmployeePermissions({})", logId, employeeId);
        EmployeePermissions empPermissions = new EmployeePermissions(employeeId);

        String cacheKey;
        if (path.isEmpty())
            cacheKey = employeeId;
        else
            cacheKey = getTablePath(path) + "_" + employeeId;

        CompletableFuture<List<TablePermissions>> permissions = getTablePermissions(path, logId);
        if (employeeCache.get(cacheKey) != null) {
            logger.debug("{}: Get employee permissions data from cache by key \"{}\"", logId, cacheKey);
            empPermissions = employeeCache.get(cacheKey);
        }
        else {
            logger.debug("{}: Get employee permissions data from DB", logId);
            String tablePermissionsString = "";
            try {
                List<TablePermissions> listPermissions = permissions.get();
                for (int i = 0; i < listPermissions.size(); i++) {
                    TablePermissions tablePermissions = listPermissions.get(i);
                    if (i > 0)
                        tablePermissionsString += ",";
                    tablePermissionsString += tablePermissions.getPermission();
                }
            } catch (InterruptedException e) {
                e.printStackTrace();
            } catch (ExecutionException e) {
                e.printStackTrace();
            }

            String sql = "select distinct "+tablePermissionsString+ " from \"" + permissionsCatalog + "\".\"permission\" where \"$employeeId\" = '" + employeeId + "'";
            logger.debug("{}: \t---> {}", logId, sql);
            SqlRowSet sqlRowSet = jdbcTemplate.queryForRowSet(sql);
            SqlRowSetMetaData metaData = sqlRowSet.getMetaData();
            String[] columnNames = metaData.getColumnNames();
            int rowCounter = 0;
            while (sqlRowSet.next()) {
                for (int k = 0; k < columnNames.length; k++) {
                    String colName = columnNames[k];
                    if (colName.equals("$employeeId"))
                        continue;
                    if (!empPermissions.getPermissions().containsKey(columnNames[k])) {
                        List<String> values = new ArrayList<>();
                        empPermissions.getPermissions().put(colName, values);
                    }
                    empPermissions.getPermissions().get(colName).add(sqlRowSet.getString(colName));
                }
                rowCounter++;
            }
            empPermissions.setPermissionsCount(rowCounter);
            logger.debug("{}: getEmployeePermissions() complete. Select {} rows.", logId, rowCounter);

            employeeCache.put(cacheKey, empPermissions);
        }

        return CompletableFuture.completedFuture(empPermissions);
    }

    @Async("threadPoolTaskExecutor")
    @Override
    public CompletableFuture<List<TablePermissions>> getTablePermissions(List<String> path, UUID logId) {
        logger.debug("{}: getTablePermissions({})", logId, path);
        List<TablePermissions> permissionsList = new ArrayList<>();
        String table = "null";
        if (!path.isEmpty())
            table = getTablePath(path);
        if (tablePermissionCache.get(table) != null) {
            logger.debug("{}: Get table permissions data from cache by key \"{}\"", logId, table);
            permissionsList = tablePermissionCache.get(table);
        }
        else {
            logger.debug("{}Get table permissions data from DB", logId);
            String sql = path.isEmpty() ? "select \"permission\", \"column\" from " + permissionsCatalog + ".\"table-permission\"" :
                    "select \"permission\", \"column\" from " + permissionsCatalog + ".\"table-permission\" where \"table\" = '" + table + "'";
            logger.debug("{}: \t---> {}", logId, sql);
            SqlRowSet sqlRowSet = jdbcTemplate.queryForRowSet(sql);
            while (sqlRowSet.next()) {
                TablePermissions tablePermissions = new TablePermissions();
                tablePermissions.setPermission(sqlRowSet.getString("permission"));
                tablePermissions.setColumn(sqlRowSet.getString("column"));
                permissionsList.add(tablePermissions);
            }
            logger.debug("{}: DB returned {} table permissions", logId, permissionsList.size());
            tablePermissionCache.put(table, permissionsList);
        }

        return CompletableFuture.completedFuture(permissionsList);
    }

    @Async("threadPoolTaskExecutor")
    @Override
    public CompletableFuture<Boolean> getDatasetTablePermissions(String employeeId, List<String> path,
                                                                                      UUID logId) {
        logger.debug("{}: getDatasetTablePermissions({}, {})", logId, employeeId, path);
        String table = "null";
        if (!path.isEmpty())
            table = getTablePath(path);

        logger.debug("{}: Get dataset table permissions data from DB", logId);
        String sql = "select distinct \"dataset_name\" from " + permissionsCatalog + ".\"dataset-table-permission\" where \"$employeeId\" in ('" + employeeId + "', '*')";
        logger.debug("{}: \t---> {}", logId, sql);

        SqlRowSet sqlRowSet = jdbcTemplate.queryForRowSet(sql);
        while (sqlRowSet.next()) {
            String datasetName = sqlRowSet.getString("dataset_name");
            logger.debug("{}: Dataset name = {}", logId, datasetName);
            if (datasetName.equals("*") || table.equals(datasetName)) {
                logger.debug("{}: Dataset access granted. Dataset name {}", logId, table);
                return CompletableFuture.completedFuture(true);
            }
        }

        logger.debug("{}: Dataset access declined. Dataset name {}", logId, table);
        return CompletableFuture.completedFuture(false);
    }

    private void generateSchema(QueryResult queryResult, ResultSet rs) throws SQLException {
        for (int k = 1; k <= rs.getMetaData().getColumnCount(); k++) {
            queryResult.addSchema(rs.getMetaData().getColumnName(k), getTypeByCode(rs.getMetaData().getColumnType(k)));
        }
    }

    private String getTablePath(List<String> path) {
        return path.stream().reduce((x1, x2) -> x1 + "." + x2).get();
    }

    private String getTypeByCode(int code) {
        return EClientDataType.getByCode(code);
    }
}
