package com.goodt.drive.report.application.services;


import com.goodt.drive.report.GlobalDefs;
import com.goodt.drive.report.application.dto.NamespaceDto;
import javassist.NotFoundException;
import org.apache.commons.io.FilenameUtils;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.hwpf.HWPFDocument;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.apache.poi.xwpf.extractor.XWPFWordExtractor;
import org.apache.poi.xwpf.usermodel.XWPFDocument;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;
import java.util.stream.Stream;

@Service
public class TemplateServiceImpl implements TemplateService {

    @Override
    public List<Path> findAll() {
        List<Path> templates = new ArrayList<>();
        Path globalDir = getGlobalDir();
        try (Stream<Path> walk = Files.walk(globalDir, 1)) {
            Set<Path> paths = walk
                    .filter(path -> !Files.isDirectory(path))
                    .collect(Collectors.toSet());
            templates.addAll(paths);
        } catch (IOException e) {
            logger.debug("An error occurred during read directory with global templates {}", globalDir);
        }
        Path localDir = getLocalDir();
        try (Stream<Path> walk = Files.walk(localDir, 1)) {
            Set<Path> paths = walk
                    .filter(path -> !Files.isDirectory(path))
                    .collect(Collectors.toSet());
            templates.addAll(paths);
        } catch (IOException e) {
            logger.debug("An error occurred during read directory with local templates {}", localDir);
        }
        return templates;
    }

    @Override
    public Path find(String templateId) throws NotFoundException {
        return filterById(findAll(), templateId);
    }

    @Override
    public Path create(MultipartFile file) throws IllegalArgumentException, IOException {
        if (file.getOriginalFilename() == null) {
            throw new IllegalArgumentException("Original filename is empty");
        }
        Path localDir = getLocalDir();
        if (!Files.exists(localDir)) {
            try {
                Files.createDirectories(localDir);
            } catch (IOException e) {
                logger.error("An error occurred during create local dir {}", localDir.toAbsolutePath());
                throw new IOException(e);
            }
        }
        Path destinationFile = Paths.get(localDir.toAbsolutePath().toString()).resolve(file.getOriginalFilename());
        try {
            file.transferTo(destinationFile);
        } catch (IOException e) {
            logger.error("An error occurred during create local dir {}", localDir.toAbsolutePath());
            throw new IOException(e);
        }
        return destinationFile;
    }

    @Override
    public boolean delete(String templateId) throws NotFoundException, IOException {
        Path path = filterById(findAll(), templateId);
        if (path.getParent().getFileName().toString().equals(GlobalDefs.TEMPLATES_GLOBAL_DIR)) {
            throw new IllegalArgumentException("Can't delete file from global namespace");
        }
        try {
            Files.delete(path);
        } catch (IOException e) {
            logger.error("An error occurred during delete local template {}", path.toAbsolutePath());
            throw new IOException(e);
        }
        return true;
    }

    @Override
    public List<NamespaceDto> findNamespaces(String templateId) throws NotFoundException, IOException {
        List<NamespaceDto> namespaces = new ArrayList<>();
        FileInputStream fileInputStream = null;
        Path file = find(templateId);
        try {
            fileInputStream = new FileInputStream(file.toFile());
            switch (FilenameUtils.getExtension(file.getFileName().toString())) {
                case "xls":
                    namespaces = findXlsNamespaces(fileInputStream);
                    break;
                case "xlsx":
                    namespaces = findXlsxNamespaces(fileInputStream);
                    break;
                case "doc":
                    namespaces = findDocNamespaces(fileInputStream);
                    break;
                case "docx":
                    namespaces = findDocxNamespaces(fileInputStream);
                    break;
            }
        } catch (FileNotFoundException e) {
            logger.error("An error occurred during open file {} for templateId {}", file.toAbsolutePath(), templateId);
            throw new FileNotFoundException(e.getMessage());
        } catch (IOException e) {
            logger.error("An error occurred during find namespaces in file {} for templateId {}", file.toAbsolutePath(), templateId);
            throw new IOException(e);
        } finally {
            if (fileInputStream != null) {
                try {
                    fileInputStream.close();
                } catch (IOException e) {
                    logger.error(String.format("An error occurred during close file stream: %s", e.getMessage()));
                }
            }
        }
        return namespaces;
    }

    private Path filterById(List<Path> paths, String templateId) throws NotFoundException {
        return paths.stream()
                .filter(path -> path.toAbsolutePath().toString().endsWith(templateId))
                .findFirst()
                .orElseThrow(() -> new NotFoundException(String.format("An error occurred during find template with id \"%s\"", templateId)));
    }

    private List<NamespaceDto> findXlsNamespaces(FileInputStream fileInputStream) throws IOException {
        HSSFWorkbook workbook = new HSSFWorkbook(fileInputStream);
        return workbook.getAllNames().stream().map(hssfName -> new NamespaceDto(hssfName.getNameName())).collect(Collectors.toList());
    }

    private List<NamespaceDto> findXlsxNamespaces(FileInputStream fileInputStream) throws IOException {
        XSSFWorkbook workbook = new XSSFWorkbook(fileInputStream);
        return workbook.getAllNames().stream().map(hssfName -> new NamespaceDto(hssfName.getNameName())).collect(Collectors.toList());
    }

    private List<NamespaceDto> findDocNamespaces(FileInputStream fileInputStream) throws IOException {
        HWPFDocument document = new HWPFDocument(fileInputStream);
        StringBuilder text = document.getText();
        return findWordNamespaces(text.toString());
    }

    private List<NamespaceDto> findDocxNamespaces(FileInputStream fileInputStream) throws IOException {
        XWPFDocument document = new XWPFDocument(fileInputStream);
        XWPFWordExtractor xwpfWordExtractor = new XWPFWordExtractor(document);
        String text = xwpfWordExtractor.getText();
        return findWordNamespaces(text);
    }

    private List<NamespaceDto> findWordNamespaces(String text) {
        Set<String> patternnamespaces = new HashSet<>();
        Matcher matcherMarker = Pattern.compile("\\$\\{([a-zA-Z0-9_]+)\\.[a-zA-Z0-9_]+\\}").matcher(text);
        while (matcherMarker.find()) {
            patternnamespaces.add(matcherMarker.group(1));
        }
        Matcher matcherGroup = Pattern.compile("##band=([a-zA-Z0-9_]+)").matcher(text);
        while (matcherGroup.find()) {
            patternnamespaces.add(matcherGroup.group(1));
        }
        return patternnamespaces.stream().map(s -> new NamespaceDto(s)).collect(Collectors.toList());
    }

    private Path getGlobalDir() {
        return Paths.get(baseTemplatePath).resolve(GlobalDefs.TEMPLATES_GLOBAL_DIR);
    }

    private Path getLocalDir() {
        return Paths.get(baseTemplatePath).resolve(GlobalDefs.TEMPLATES_LOCAL_DIR);
    }

    @Value("${yarg.templates.path}")
    private String baseTemplatePath;

    private final Logger logger = LoggerFactory.getLogger(this.getClass());
}
