package com.goodt.drive.report.application.services;


import com.goodt.drive.report.application.dto.ReportCreateDto;
import com.goodt.drive.report.application.dto.YargBandDto;
import com.goodt.drive.report.application.dto.dremio.ResponseDto;
import com.goodt.drive.report.application.dto.keycloak.TokenDataDtoBase;
import com.goodt.drive.report.application.exception.DremioResponseException;
import com.goodt.drive.report.application.utils.yarg.JsonDataLoaderExtended;
import com.google.gson.Gson;
import com.haulmont.yarg.formatters.factory.DefaultFormatterFactory;
import com.haulmont.yarg.formatters.impl.doc.connector.OfficeIntegration;
import com.haulmont.yarg.loaders.factory.DefaultLoaderFactory;
import com.haulmont.yarg.reporting.ReportOutputDocument;
import com.haulmont.yarg.reporting.Reporting;
import com.haulmont.yarg.reporting.RunParams;
import com.haulmont.yarg.structure.BandOrientation;
import com.haulmont.yarg.structure.Report;
import com.haulmont.yarg.structure.ReportBand;
import com.haulmont.yarg.structure.ReportOutputType;
import com.haulmont.yarg.structure.impl.BandBuilder;
import com.haulmont.yarg.structure.impl.ReportBuilder;
import com.haulmont.yarg.structure.impl.ReportTemplateBuilder;
import javassist.NotFoundException;
import org.apache.commons.io.FilenameUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.io.*;
import java.nio.file.Path;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

@Service
public class YargServiceImpl implements YargService {

    @Override
    public ReportOutputDocument create(ReportCreateDto body, boolean fileOutput, TokenDataDtoBase token) throws IOException, NotFoundException, DremioResponseException {
        String templateId = body.getTemplateId();
        Path path = templateService.find(templateId);
        String outputFilename = path.getFileName().toString();
        ReportOutputType templateType;
        if (body.getTemplateType() != null && body.getTemplateType().equals("pdf")) {
            templateType = ReportOutputType.getOutputTypeById(body.getTemplateType());
            outputFilename = String.format("%s.%s", outputFilename, body.getTemplateType());
        } else {
            templateType = getTemplateType(templateId);
        }

        ReportBuilder reportBuilder = new ReportBuilder();
        ReportTemplateBuilder reportTemplateBuilder = new ReportTemplateBuilder()
                .documentPath(path.toAbsolutePath().toString())
                .documentName(path.getFileName().toString())
                .outputType(templateType)
                .outputNamePattern(outputFilename)
                .readFileFromPath();
        reportBuilder.template(reportTemplateBuilder.build());

        BandBuilder bandBuilder = new BandBuilder();
        Report report = reportBuilder.build();
        RunParams runParams = new RunParams(report);
        List<LinkedHashMap<String, Object>> data = body.getData();
        Set<YargBandDto> jsonDremios = getJsonDremio(data, token);
        for (YargBandDto jsonDremio : jsonDremios) {
            String namespace = jsonDremio.getNamespace();
            String orientation = jsonDremio.getOrientation();
            BandOrientation bandOrientation = orientation != null && orientation.equals(BandOrientation.VERTICAL.id) ? BandOrientation.VERTICAL : BandOrientation.HORIZONTAL;
            ReportBand reportBand = bandBuilder.name(namespace).query(namespace, String.format("parameter=%s rows", namespace), DefaultLoaderFactory.JSON_DATA_LOADER).orientation(bandOrientation).build();
            reportBuilder.band(reportBand);
            runParams.param(namespace, jsonDremio.getBand());
        }

        Reporting reporting = new Reporting();
        DefaultFormatterFactory formatterFactory = new DefaultFormatterFactory();
        formatterFactory.setOfficeIntegration(new OfficeIntegration(openOfficePath, 8100, 8101, 8102, 8103));
        reporting.setFormatterFactory(formatterFactory);
        reporting.setLoaderFactory(new DefaultLoaderFactory().setJsonDataLoader(new JsonDataLoaderExtended()));
        return getReportOutputDocument(fileOutput, outputFilename, reporting, runParams);
    }

    private ReportOutputDocument getReportOutputDocument(boolean fileOutput, String outputFilename, Reporting reporting, RunParams runParams) throws FileNotFoundException {
        ReportOutputDocument reportOutputDocument;
        if (fileOutput) {
            String reportsFolderPath = String.format("%s/%s", baseTemplatePath, reportsFolderName);
            File directory = new File(reportsFolderPath);
            if (! directory.exists()){
                directory.mkdirs();
            }
            String format = String.format("%s/%s", reportsFolderPath, outputFilename);
            reportOutputDocument = reporting.runReport(runParams, new FileOutputStream(format));
        } else {
            reportOutputDocument = reporting.runReport(runParams);
        }
        return reportOutputDocument;
    }

    private Set<YargBandDto> getJsonDremio(List<LinkedHashMap<String, Object>> bodyJson, TokenDataDtoBase token) throws DremioResponseException {
        Set<YargBandDto> jsons = new LinkedHashSet<>();
        Map<String, String> dremioResponses = new LinkedHashMap<>();
        for (LinkedHashMap<String, Object> bodyJsonItem : bodyJson) {
            String dataRawJson = new Gson().toJson(bodyJsonItem.get(bodyDataFieldname));
            String responseFromDremio = "";
            if (!dataRawJson.equals("null") && !dataRawJson.equals("{}")) {
                Pattern pattern = Pattern.compile("#([a-z0-9_]+)\\.([a-z0-9_]+)#", Pattern.CASE_INSENSITIVE);
                Matcher matcher = pattern.matcher(dataRawJson);
                StringBuffer sb = new StringBuffer();
                while (matcher.find()) {
                    String patternNamespace = matcher.group(1);
                    String patternDataRaw = matcher.group(2);
                    if (dremioResponses.containsKey(patternNamespace)) {
                        String patternDremioResponse = dremioResponses.get(patternNamespace);
                        ResponseDto patternDremioReportList = new Gson().fromJson(patternDremioResponse, ResponseDto.class);
                        String patternDremioValue = patternDremioReportList.getRows().get(0).get(patternDataRaw);
                        matcher.appendReplacement(sb, patternDremioValue);
                    }
                }
                matcher.appendTail(sb);
                responseFromDremio = dremioService.request(sb.toString(), token);
            }
            String namespace = (String) bodyJsonItem.get(bodyNamespaceFieldname);
            dremioResponses.put(namespace, responseFromDremio);
        }

        for (Map.Entry<String, String> dremioResponse : dremioResponses.entrySet()) {
            jsons.add(new YargBandDto(dremioResponse.getKey(), dremioResponse.getValue()));
        }
        return jsons;
    }



    private ReportOutputType getTemplateType(String templateId) throws NotFoundException, IOException {
        Path file = templateService.find(templateId);
        return ReportOutputType.getOutputTypeById(FilenameUtils.getExtension(file.getFileName().toString()));
    }

    private final String bodyNamespaceFieldname = "name";
    private final String bodyDataFieldname = "data";

    @Value("${yarg.templates.path}")
    private String baseTemplatePath;

    private final String reportsFolderName = "reports";

    @Value("${yarg.openoffice.path}")
    private String openOfficePath;

    @Autowired
    TemplateService templateService;

    @Autowired
    DremioService dremioService;
}
