package com.goodt.drive.report.application.services.orgstructure;

import com.goodt.drive.report.GlobalDefs;
import com.goodt.drive.report.application.configuration.AppConfig;
import com.goodt.drive.report.application.dto.keycloak.TokenDataDto;
import com.goodt.drive.report.application.dto.keycloak.TokenDataDtoBase;
import com.goodt.drive.report.application.dto.orgstructure.DivisionTeamAssignmentDto;
import com.goodt.drive.report.application.dto.orgstructure.EmployeeInfoDto;
import com.goodt.drive.report.application.dto.orgstructure.LegalEntityTeamAssignmentDto;
import io.netty.handler.ssl.SslContext;
import io.netty.handler.ssl.SslContextBuilder;
import io.netty.handler.ssl.util.InsecureTrustManagerFactory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.http.client.reactive.ReactorClientHttpConnector;
import org.springframework.stereotype.Service;
import org.springframework.web.reactive.function.BodyInserters;
import org.springframework.web.reactive.function.client.WebClient;
import org.springframework.web.util.UriBuilder;
import reactor.netty.http.client.HttpClient;

import javax.net.ssl.SSLException;
import java.net.URI;
import java.util.List;
import java.util.Objects;
import java.util.function.Function;

@Service
public class OrgstructureServiceClientImpl implements IOrgstructureServiceClient {

    @Override
    public Long getEmployeeTaskRole(Long userId, Long employeeIdTaskOpener, TokenDataDtoBase token) {
        try {
            if (employeeIdTaskOpener == null) {
                logger.debug(String.format("Current employee id (assocciated to KeyCloak session via Mapper) is null"));
                return null;
            }
            List<DivisionTeamAssignmentDto> assignments = getAssignments(userId, token);
            if (assignments == null || assignments.size() != 1) {
                logger.debug(String.format("Empty assignemnts for assignment id (task user id): %d", userId));
                return null;
            }
            Long employeeIdTaskHolder = assignments.get(0).getEmployee().getId();
            if (Objects.equals(employeeIdTaskHolder, employeeIdTaskOpener)) {
                logger.debug(String.format("Employee %d is a task %d owner", employeeIdTaskOpener, assignments.get(0).getId()));
                return GlobalDefs.TASK_OWNER_ROLE_CODE;
            }
            Long divisionTeamIdTaskHolder = assignments.get(0).getDivisionTeam().getId();
            DivisionTeamAssignmentDto employeeHeadTaskHolder = getEmployeeHead(employeeIdTaskHolder, divisionTeamIdTaskHolder, token);
            if (employeeHeadTaskHolder != null && employeeHeadTaskHolder.getEmployee() != null && Objects.equals(employeeHeadTaskHolder.getEmployee().getId(), employeeIdTaskOpener)) {
                logger.debug(String.format("Employee %d is a employee %d head, therefore it has access to task %d",
                        employeeHeadTaskHolder.getEmployee().getId(), employeeIdTaskOpener, userId));
                return GlobalDefs.TASK_EMPLOYEE_HEAD_ROLE_CODE;
            }
            DivisionTeamAssignmentDto employeeHeadTaskOpener = getEmployeeHead(employeeIdTaskOpener, divisionTeamIdTaskHolder, token);
            if (employeeHeadTaskOpener != null && employeeHeadTaskOpener.getEmployee() != null && Objects.equals(employeeHeadTaskOpener.getEmployee().getId(), employeeIdTaskHolder)) {
                logger.debug(String.format("Employee %d is a employee %d subordinate, therefore it has access to task %d",
                        employeeIdTaskOpener, employeeHeadTaskOpener.getEmployee().getId(), userId));
                return GlobalDefs.TASK_EMPLOYEE_SUBORDINATE_ROLE_CODE;
            }
            List<DivisionTeamAssignmentDto> employeeAssignments = getAssignments(null, token);
            DivisionTeamAssignmentDto divisionTeamAssignmentDto = employeeAssignments.get(0);
            if (userId.equals(divisionTeamAssignmentDto.getId())) {
                logger.debug(String.format("Employee %d is a task %d assignment", employeeIdTaskOpener, divisionTeamAssignmentDto.getId()));
                return GlobalDefs.TASK_ASSIGNMENT_ROLE_CODE;
            }
            Long employeeDivisionTeamId = divisionTeamAssignmentDto.getDivisionTeam().getId();
            List<DivisionTeamAssignmentDto> employeeSubordinates = getEmployeeSubordinates(employeeIdTaskOpener, employeeDivisionTeamId, token);
            if (employeeSubordinates.stream().anyMatch(item -> Objects.equals(userId, item.getId()))) {
                logger.debug(String.format("Employee %d is a employee %d subordinate head, therefore it has access to task %d",
                        employeeIdTaskOpener, divisionTeamIdTaskHolder, userId));
                return GlobalDefs.TASK_EMPLOYEE_SUBORDINATE_HEAD_ROLE_CODE;
            }
            logger.debug(String.format("Employee %d is NOT a task %d owner or is not a employee head (actual head is %d)",
                    employeeIdTaskHolder, userId, employeeHeadTaskHolder != null ? employeeHeadTaskHolder.getEmployee().getId() : null));
            return GlobalDefs.TASK_OTHER_ROLE_CODE;
        } catch (Exception e) {
            logger.error(String.format("An error occured during employee %d task role fetch: %s", employeeIdTaskOpener, e.getMessage()));
            return null;
        }
    }

    @Override
    public EmployeeInfoDto getEmployeeInfo(Long employeeId, TokenDataDtoBase token) {
        try {
            WebClient orgstructureClient = getOrgstructureWebClient(token);
            EmployeeInfoDto employeeInfo = orgstructureClient.get()
                    .uri(uriBuilder -> uriBuilder.
                    path("/api/employee/info").
                    queryParam("id", employeeId).
                    build())
                    .retrieve().bodyToMono(EmployeeInfoDto.class).block();
            return employeeInfo;
        } catch (Exception e) {
            logger.error(String.format("An error occured during employee %d info fetch: %s", employeeId, e.getMessage()));
            return null;
        }
    }

    @Override
    public DivisionTeamAssignmentDto getEmployeeHead(Long employeeId, Long divisionTeamId, TokenDataDtoBase token) {
        try {
            WebClient orgstructureClient = getOrgstructureWebClient(token);
            DivisionTeamAssignmentDto head = orgstructureClient.get()
                    .uri(uriBuilder -> uriBuilder.
                    path("/api/employee/teamdivisionhead").
                    queryParam("employee", employeeId).
                    queryParam("team", divisionTeamId).
                    build())
                    .retrieve().bodyToMono(DivisionTeamAssignmentDto.class).block();
            return head;
        } catch (Exception e) {
            logger.error(String.format("An error occured during employee %d head fetch: %s", employeeId, e.getMessage()));
            return null;
        }
    }

    @Override
    public List<DivisionTeamAssignmentDto> getAssignments(Long userId, TokenDataDtoBase token) {
        try {
            WebClient orgstructureClient = getOrgstructureWebClient(token);
            Function<UriBuilder, URI> function = userId == null ?
                    uriBuilder -> uriBuilder.path("/api/employee/teamdivisionassignments").
                            build() :
                    uriBuilder -> uriBuilder.path("/api/employee/teamdivisionassignments").
                            queryParam("id", userId).
                            build();
            List<DivisionTeamAssignmentDto> assignments = orgstructureClient.get()
                    .uri(function)
                    .retrieve().bodyToFlux(DivisionTeamAssignmentDto.class)
                    .collectList().block();
            return assignments;
        } catch (Exception e) {
            logger.error(String.format("An error occured during assignment %d fetch: %s", userId, e.getMessage()));
            return null;
        }
    }

    @Override
    public List<DivisionTeamAssignmentDto> getEmployeeSubordinates(Long employeeId, Long divisionTeamId, TokenDataDtoBase token) {
        try {
            WebClient orgstructureClient = getOrgstructureWebClient(token);
            List<DivisionTeamAssignmentDto> subordinates = orgstructureClient.get()
                    .uri(uriBuilder -> uriBuilder.path("/api/employee/teamdivisionsubordinates").
                            queryParam("id", employeeId).
                            queryParam("team", divisionTeamId).
                            build())
                    .retrieve().bodyToFlux(DivisionTeamAssignmentDto.class)
                    .collectList().block();
            return subordinates;
        } catch (Exception e) {
            logger.error(String.format("An error occured during employee %d division team %d fetch: %s", employeeId, divisionTeamId, e.getMessage()));
            return null;
        }
    }

    @Override
    public List<LegalEntityTeamAssignmentDto> getLegalEntityTeamAssignments(Long employeeId, TokenDataDtoBase token) {
        try {
            WebClient orgstructureClient = getOrgstructureWebClient(token);
            Function<UriBuilder, URI> function = employeeId == null ?
                    uriBuilder -> uriBuilder.path("/api/employee/teamlegalentityassignments").
                            build() :
                    uriBuilder -> uriBuilder.path("/api/employee/teamlegalentityassignments").
                            queryParam("employee", employeeId).
                            build();
            List<LegalEntityTeamAssignmentDto> assignments = orgstructureClient.get()
                    .uri(function)
                    .retrieve().bodyToFlux(LegalEntityTeamAssignmentDto.class)
                    .collectList().block();
            return assignments;
        } catch (Exception e) {
            logger.error(String.format("An error occured during employee %d fetch: %s", employeeId, e.getMessage()));
            return null;
        }
    }

    @Deprecated
    private TokenDataDto authenticate() {
        try {
            SslContext sslContext = SslContextBuilder
                    .forClient()
                    .trustManager(InsecureTrustManagerFactory.INSTANCE)
                    .build();

            HttpClient httpClient = HttpClient.create().secure(t -> t.sslContext(sslContext));
            showKeyCloakDiagnosticInfo();
            WebClient client = WebClient.builder().baseUrl(appConfig.getKeyCloakConfig().getBaseUrl()).clientConnector(new ReactorClientHttpConnector(httpClient)).build();
            TokenDataDto result = client.post().uri(appConfig.getKeyCloakConfig().getTokenUrl())
                    .header(HttpHeaders.CONTENT_TYPE, MediaType.APPLICATION_FORM_URLENCODED_VALUE)
                    .body(BodyInserters.fromFormData("client_id", appConfig.getKeyCloakConfig().getClientId())
                            .with("client_secret", appConfig.getKeyCloakConfig().getClientSecret())
                            .with("grant_type", "password")
                            .with("scope", "local")
                            .with("username", appConfig.getKeyCloakConfig().getUsername())
                            .with("password", appConfig.getKeyCloakConfig().getPassword()))
                    .retrieve().bodyToMono(TokenDataDto.class).block();
            return result;
        } catch (SSLException e) {
            logger.error(e.getMessage(), e);
            return null;
        }
    }

    private WebClient getOrgstructureWebClient(TokenDataDtoBase token) {
        logger.debug(String.format("Connection to orgstructure. Connecting host: %s.", appConfig.getOrgstructureConfig().getHost()));
        try {
            SslContext sslContext = SslContextBuilder
                    .forClient()
                    .trustManager(InsecureTrustManagerFactory.INSTANCE)
                    .build();

            HttpClient httpClient = HttpClient.create().secure(t -> t.sslContext(sslContext));
            WebClient client = WebClient.builder().baseUrl(appConfig.getOrgstructureConfig().getHost()).clientConnector(new ReactorClientHttpConnector(httpClient))
                    .defaultHeader(HttpHeaders.AUTHORIZATION, String.format("%s %s", token.getTokenType(),
                            token.getAccessToken()))
                    .build();
            return client;
        } catch (SSLException e) {
            logger.error(e.getMessage(), e);
            return null;
        }
    }

    private void showKeyCloakDiagnosticInfo() {
        logger.debug(String.format("Keycloak base URL: %s", appConfig.getKeyCloakConfig().getBaseUrl()));
        logger.debug(String.format("Keycloak token URL: %s", appConfig.getKeyCloakConfig().getTokenUrl()));
        logger.debug(String.format("Keycloak client id: %s", appConfig.getKeyCloakConfig().getClientId()));
        logger.debug(String.format("Keycloak client secret: %s", appConfig.getKeyCloakConfig().getClientSecret()));
        logger.debug(String.format("Keycloak client username: %s", appConfig.getKeyCloakConfig().getUsername()));
    }

    @Autowired
    private AppConfig appConfig;

    private final Logger logger = LoggerFactory.getLogger(this.getClass());
}
