package com.goodt.drive.report.application.system;

import com.goodt.drive.report.application.utils.ExeptionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

public class ExpiredConcurrentHashMap<K, V> extends ConcurrentHashMap<K, V> {

    private static Logger logger = LoggerFactory.getLogger(ExpiredConcurrentHashMap.class);
    private static final long serialVersionUID = 1L;

    private Map<K, Long> timeMap = new ConcurrentHashMap<K, Long>();

    private long expiryInMillis = 5000;

    private long cleanerInMillis = 1000;

    private static final SimpleDateFormat sdf = new SimpleDateFormat("HH:mm:ss:SSS");

    private String cacheName;

    public ExpiredConcurrentHashMap(String cacheName) {
        this.cacheName = cacheName;
        initialize();
    }

    public ExpiredConcurrentHashMap(String cacheName, long expiryInMillis, long cleanerInMillis) {
        logger.info("Initialize cache for \"{}\" with expire time \"{}\" sec and cleaner time \"{}\" sec", cacheName, expiryInMillis / 1000, cleanerInMillis / 1000);
        this.cacheName = cacheName;
        this.expiryInMillis = expiryInMillis;
        this.cleanerInMillis = cleanerInMillis;
        initialize();
    }

    private void initialize() {
        new CleanerThread().start();
    }

    @Override
    public V put(K key, V value) {
        Date date = new Date();
        timeMap.put(key, date.getTime());
        logger.debug("Inserting element to \"{}\": " + sdf.format(date) + " : " + key + " : " + value, ExpiredConcurrentHashMap.this.cacheName);
        V returnVal = super.put(key, value);
        return returnVal;
    }

    @Override
    public void putAll(Map<? extends K, ? extends V> m) {
        for (K key : m.keySet()) {
            put(key, m.get(key));
        }
    }

    @Override
    public V putIfAbsent(K key, V value) {
        if (!containsKey(key))
            return put(key, value);
        else
            return get(key);
    }

    private class CleanerThread extends Thread {
        @Override
        public void run() {
            logger.info("Initiating Cleaner Thread for \"{}\"", ExpiredConcurrentHashMap.this.cacheName);
            while (true) {
                cleanMap();
                try {
                    Thread.sleep(cleanerInMillis);
                } catch (InterruptedException ex) {
                    logger.error(ExeptionUtils.getStackTrace(ex));
                }
            }
        }

        private void cleanMap() {
            long currentTime = new Date().getTime();
            for (K key : timeMap.keySet()) {
                if (currentTime > (timeMap.get(key) + expiryInMillis)) {
                    V value = remove(key);
                    timeMap.remove(key);
                    logger.debug("Removing expired element from \"{}\": " + sdf.format(new Date()) + " : " + key + " : " + value, ExpiredConcurrentHashMap.this.cacheName);
                }
            }
        }
    }
}