package com.goodt.drive.report.application.rest;

import com.goodt.drive.report.application.configuration.AppConfig;
import com.goodt.drive.report.application.data.OperationResult;
import com.goodt.drive.report.application.dto.ReportCreateDto;
import com.goodt.drive.report.application.dto.ReportDto;
import com.goodt.drive.report.application.dto.ReportGenerateDto;
import com.goodt.drive.report.application.dto.TokenResponseDto;
import com.goodt.drive.report.utils.IntegrationalTestBase;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestInstance;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.web.client.TestRestTemplate;
import org.springframework.boot.web.server.LocalServerPort;
import org.springframework.http.*;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.web.reactive.function.BodyInserters;
import org.springframework.web.reactive.function.client.WebClient;

@SpringBootTest(webEnvironment = SpringBootTest.WebEnvironment.RANDOM_PORT)
@ActiveProfiles("integrationaltests")
@TestInstance(TestInstance.Lifecycle.PER_CLASS)
class TestReportController extends IntegrationalTestBase {

    @LocalServerPort
    private int port;

    @Autowired
    private TestRestTemplate restTemplate;

    @Autowired
    private AppConfig appConfig;

    private String hostName = "http://localhost:" + port;

    private HttpHeaders requestHeaders;


    @BeforeAll
    public void setup() {
        WebClient client = WebClient.builder()
                .baseUrl(appConfig.getKeyCloakConfig().getBaseUrl())
                .build();
        TokenResponseDto result = client.post().uri(appConfig.getKeyCloakConfig().getTokenUrl())
                .header(HttpHeaders.CONTENT_TYPE, MediaType.APPLICATION_FORM_URLENCODED_VALUE)
                .body(BodyInserters.fromFormData("client_id", appConfig.getKeyCloakConfig().getClientId())
                        .with("client_secret", appConfig.getKeyCloakConfig().getClientSecret())
                        .with("grant_type", "password")
.with("username", appConfig.getKeyCloakConfig().getUsername())
                        .with("password", appConfig.getKeyCloakConfig().getPassword()))
                .retrieve().bodyToMono(TokenResponseDto.class).block();

        hostName = "http://localhost:" + port;
        requestHeaders = new HttpHeaders();
        requestHeaders.add("authorization", String.format("%s %s", result.getTokenType(), result.getAccessToken()));
    }

    @Test
    void testGetAll() {
        HttpEntity request = new HttpEntity<>(requestHeaders);

        ResponseEntity<ReportDto[]> response = restTemplate.exchange(
                hostName + "/api/report",
                HttpMethod.GET,
                request,
                ReportDto[].class
        );
        Assertions.assertEquals(HttpStatus.OK, response.getStatusCode());
        Assertions.assertTrue(response.getBody().length > 0);
    }

    @Test
    void testGetById() {
        HttpEntity request = new HttpEntity<>(requestHeaders);

        ResponseEntity<ReportDto> response = restTemplate.exchange(
                hostName + "/api/report/{id}",
                HttpMethod.GET,
                request,
                ReportDto.class,
                1L
        );
        Assertions.assertEquals(HttpStatus.OK, response.getStatusCode());
        Assertions.assertEquals(1L, response.getBody().getId());
    }

    @Test
    void testAdd() {
        ReportDto reportDto = new ReportDto();
        reportDto.setName("created name 1 xls");
        reportDto.setTemplateId("global/new_template_1.xls");
        reportDto.setData("{}");

        HttpEntity request = new HttpEntity<>(reportDto, requestHeaders);

        ResponseEntity<ReportDto> response = restTemplate.exchange(
                hostName + "/api/report",
                HttpMethod.POST,
                request,
                ReportDto.class
        );
        Assertions.assertEquals(HttpStatus.OK, response.getStatusCode());
        Assertions.assertEquals("created name 1 xls", response.getBody().getName());
    }

    @Test
    void testUpdate() {
        ReportDto reportDto = new ReportDto();
        reportDto.setName("upd name 2 xls");
        reportDto.setTemplateId("global/upd_template_2.xls");
        reportDto.setData("{}");

        HttpEntity request = new HttpEntity<>(reportDto, requestHeaders);

        ResponseEntity<ReportDto> response = restTemplate.exchange(
                hostName + "/api/report/{id}",
                HttpMethod.PUT,
                request,
                ReportDto.class,
                2L
        );
        Assertions.assertEquals(HttpStatus.OK, response.getStatusCode());
        Assertions.assertEquals("upd name 2 xls", response.getBody().getName());
    }

    @Test
    void testDelete() {
        HttpEntity request = new HttpEntity<>(requestHeaders);

        ResponseEntity<OperationResult> response = restTemplate.exchange(
                hostName + "/api/report/{id}",
                HttpMethod.DELETE,
                request,
                OperationResult.class,
                4L
        );
        Assertions.assertEquals(HttpStatus.OK, response.getStatusCode());
    }

    @Test
    void testGenerate() {
        ReportGenerateDto requestBody = new ReportGenerateDto();
        requestBody.setReportId(1L);

        HttpEntity<ReportGenerateDto> requestExtra = new HttpEntity<>(requestBody, requestHeaders);

        ResponseEntity<byte[]> response = restTemplate.exchange(
                hostName + "/api/report/generate",
                HttpMethod.POST,
                requestExtra,
                byte[].class
        );
        Assertions.assertEquals(HttpStatus.OK, response.getStatusCode());
        Assertions.assertTrue(response.getBody().length > 0);
    }

    @Test
    void testCreate() {
        ReportCreateDto requestBody = new ReportCreateDto();
        requestBody.setTemplateId("global/template_12.xls");

        HttpEntity<ReportCreateDto> requestExtra = new HttpEntity<>(requestBody, requestHeaders);

        ResponseEntity<byte[]> response = restTemplate.exchange(
                hostName + "/api/report/create",
                HttpMethod.POST,
                requestExtra,
                byte[].class
        );
        Assertions.assertEquals(HttpStatus.OK, response.getStatusCode());
        Assertions.assertTrue(response.getBody().length > 0);
    }

    @Test
    void testCreate_ByNumericId() {
        ReportCreateDto requestBody = new ReportCreateDto();
        requestBody.setTemplateId("12");

        HttpEntity<ReportCreateDto> requestExtra = new HttpEntity<>(requestBody, requestHeaders);

        ResponseEntity<byte[]> response = restTemplate.exchange(
                hostName + "/api/report/create",
                HttpMethod.POST,
                requestExtra,
                byte[].class
        );
        Assertions.assertEquals(HttpStatus.OK, response.getStatusCode());
        Assertions.assertTrue(response.getBody().length > 0);
    }

    @Test
    void testCreate_Pdf() {
        ReportCreateDto requestBody = new ReportCreateDto();
        requestBody.setTemplateId("global/template_12.xls");
        requestBody.setTemplateType("pdf");

        HttpEntity<ReportCreateDto> requestExtra = new HttpEntity<>(requestBody, requestHeaders);

        ResponseEntity<byte[]> response = restTemplate.exchange(
                hostName + "/api/report/create",
                HttpMethod.POST,
                requestExtra,
                byte[].class
        );
        Assertions.assertEquals(HttpStatus.OK, response.getStatusCode());
        Assertions.assertTrue(response.getBody().length > 0);
        HttpHeaders headers = response.getHeaders();
        Assertions.assertEquals(headers.getFirst("Content-Disposition"), "attachment; filename=template_12.xls.pdf");
        Assertions.assertEquals(headers.getFirst("Content-Type"), "application/pdf");
    }
}
