package com.goodt.drive.report.application.rest;

import com.goodt.drive.report.GlobalDefs;
import com.goodt.drive.report.application.configuration.AppConfig;
import com.goodt.drive.report.application.dto.NamespaceDto;
import com.goodt.drive.report.application.dto.TemplateDto;
import com.goodt.drive.report.application.dto.TokenResponseDto;
import com.goodt.drive.report.utils.IntegrationalTestBase;
import org.junit.jupiter.api.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.web.client.TestRestTemplate;
import org.springframework.boot.web.server.LocalServerPort;
import org.springframework.http.*;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.reactive.function.BodyInserters;
import org.springframework.web.reactive.function.client.WebClient;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.List;

@SpringBootTest(webEnvironment = SpringBootTest.WebEnvironment.RANDOM_PORT)
@ActiveProfiles("integrationaltests")
@TestInstance(TestInstance.Lifecycle.PER_CLASS)
class TestTemplateController extends IntegrationalTestBase {

    @LocalServerPort
    private int port;

    @Autowired
    private TestRestTemplate restTemplate;

    @Value("${yarg.templates.path}")
    private String baseTemplatePath;

    @Autowired
    private AppConfig appConfig;

    private String hostName = "http://localhost:" + port;
    private HttpHeaders requestHeaders;

    @BeforeAll
    void setup() {
        WebClient client = WebClient.builder()
                .baseUrl(appConfig.getKeyCloakConfig().getBaseUrl())
                .build();
        TokenResponseDto result = client.post().uri(appConfig.getKeyCloakConfig().getTokenUrl())
                .header(HttpHeaders.CONTENT_TYPE, MediaType.APPLICATION_FORM_URLENCODED_VALUE)
                .body(BodyInserters.fromFormData("client_id", appConfig.getKeyCloakConfig().getClientId())
                        .with("client_secret", appConfig.getKeyCloakConfig().getClientSecret())
                        .with("grant_type", "password")
.with("username", appConfig.getKeyCloakConfig().getUsername())
                        .with("password", appConfig.getKeyCloakConfig().getPassword()))
                .retrieve().bodyToMono(TokenResponseDto.class).block();

        hostName = "http://localhost:" + port;
        requestHeaders = new HttpHeaders();
        requestHeaders.add("authorization", String.format("%s %s", result.getTokenType(), result.getAccessToken()));
    }

    @AfterEach
    void downContext() throws IOException {
        Path newFile = Paths.get(baseTemplatePath).resolve(GlobalDefs.TEMPLATES_LOCAL_DIR).resolve("text-file.txt");
        Files.deleteIfExists(newFile);
    }

    @Test
    void testGetAll() {
        HttpEntity request = new HttpEntity<>(requestHeaders);

        ResponseEntity<TemplateDto[]> response = restTemplate.exchange(
                hostName + "/api/template/list",
                HttpMethod.GET,
                request,
                TemplateDto[].class
        );
        Assertions.assertEquals(HttpStatus.OK, response.getStatusCode());
        Assertions.assertTrue(response.getBody().length > 0);
    }

    @Test
    void testDownload() {
        HttpEntity request = new HttpEntity<>(requestHeaders);

        ResponseEntity<byte[]> response = restTemplate.exchange(
                hostName + "/api/template/download/{folder}/{filename}",
                HttpMethod.GET,
                request,
                byte[].class,
                "global",
                "template_11.xls"
        );
        Assertions.assertEquals(HttpStatus.OK, response.getStatusCode());
        Assertions.assertTrue(response.getBody().length > 0);
        Assertions.assertEquals("application/vnd.ms-excel", response.getHeaders().get(HttpHeaders.CONTENT_TYPE).get(0));
        Assertions.assertEquals("attachment; filename=\"template_11.xls\"", response.getHeaders().get(HttpHeaders.CONTENT_DISPOSITION).get(0));
    }

    @Test
    void testCreate() throws IOException {
        Path newFile = Paths.get(baseTemplatePath).resolve(GlobalDefs.TEMPLATES_LOCAL_DIR).resolve("text-file.txt");
        Files.deleteIfExists(newFile);
        Assertions.assertFalse(Files.exists(newFile));
        InputStream inputStream = new ByteArrayInputStream("text file data".getBytes());
        MultipartFile file = new MockMultipartFile("text-file.txt", "text-file.txt", MediaType.MULTIPART_FORM_DATA_VALUE, inputStream);
        HttpEntity<?> archive = new HttpEntity<>(file.getResource());

        MultiValueMap<String, Object> requestBody = new LinkedMultiValueMap<>();

        requestBody.add("template", archive);

        HttpHeaders currentHeaders = new HttpHeaders();
        currentHeaders.putAll(requestHeaders);
        currentHeaders.setContentType(MediaType.MULTIPART_FORM_DATA);
        HttpEntity<MultiValueMap> requestExtra = new HttpEntity<>(requestBody, currentHeaders);

        ResponseEntity<byte[]> response = restTemplate.exchange(
                hostName + "/api/template/create",
                HttpMethod.POST,
                requestExtra,
                byte[].class
        );
        Assertions.assertEquals(HttpStatus.OK, response.getStatusCode());
        Assertions.assertTrue(Files.exists(newFile));
    }

    @Test
    void testNamespaces() {
        HttpEntity request = new HttpEntity<>(requestHeaders);

        ResponseEntity<NamespaceDto[]> response = restTemplate.exchange(
                hostName + "/api/template/namespaces/{folder}/{filename}",
                HttpMethod.GET,
                request,
                NamespaceDto[].class,
                "global",
                "template_11.xls"
        );
        Assertions.assertEquals(HttpStatus.OK, response.getStatusCode());
        List<NamespaceDto> dtos = List.of(response.getBody());
        Assertions.assertEquals(4, dtos.size());
    }
}
