package com.goodt.drive.report.application.services;

import com.goodt.drive.report.GlobalDefs;
import com.goodt.drive.report.application.dto.NamespaceDto;
import com.goodt.drive.report.application.dto.TemplateDto;
import javassist.NotFoundException;
import org.apache.commons.io.IOUtils;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.test.context.ActiveProfiles;
import org.junit.jupiter.api.Assertions;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.List;


@SpringBootTest
@ActiveProfiles("functests")
class TestTemplateServiceImpl {

    @Value("${yarg.templates.path}")
    private String baseTemplatePath;

    @Autowired
    private TemplateService templateService;

    @Test
    void testFindAll() {
        List<Path> paths = templateService.findAll();
        Assertions.assertTrue(paths.size() > 0);
        Assertions.assertTrue(paths.stream().anyMatch(path -> path.getParent().getFileName().toString().equals("global")));
        Assertions.assertTrue(paths.stream().anyMatch(path -> path.getParent().getFileName().toString().equals("local")));
    }

    @Test
    void testNamespacesXls() throws NotFoundException, IOException {
        String templateId = "global/template_11.xls";
        List<NamespaceDto> namespaceDtos = templateService.findNamespaces(templateId);
        Assertions.assertEquals(4, namespaceDtos.size());
    }

    @Test
    void testNamespacesXlsx() throws NotFoundException, IOException {
        String templateId = "global/template_15.xlsx";
        List<NamespaceDto> namespaceDtos = templateService.findNamespaces(templateId);
        Assertions.assertEquals(4, namespaceDtos.size());
    }

    @Test
    void testNamespacesDoc() throws NotFoundException, IOException {
        String templateId = "global/template_13.doc";
        List<NamespaceDto> namespaceDtos = templateService.findNamespaces(templateId);
        Assertions.assertEquals(2, namespaceDtos.size());
    }

    @Test
    void testNamespacesDocx() throws NotFoundException, IOException {
        String templateId = "global/template_16.docx";
        List<NamespaceDto> namespaceDtos = templateService.findNamespaces(templateId);
        Assertions.assertEquals(2, namespaceDtos.size());
    }

    @Test
    void testFindFile() throws NotFoundException, IOException {
        Path file = templateService.find("global/template_11.xls");
        Assertions.assertEquals("template_11.xls", file.getFileName().toString());
    }

    @Test
    void testFindFilePaths() throws NotFoundException, IOException {
        Path globalFile = templateService.find("global/template_11.xls");
        Assertions.assertEquals("template_11.xls", globalFile.getFileName().toString());
        Assertions.assertTrue(globalFile.toAbsolutePath().toString().endsWith("/global/template_11.xls"));

        Path localFile = templateService.find("local/template_12.xls");
        Assertions.assertEquals("template_12.xls", localFile.getFileName().toString());
        Assertions.assertTrue(localFile.toAbsolutePath().toString().endsWith("/local/template_12.xls"));
    }

    @Test
    void testCreate() throws IOException {
        String templateName = "template_99.xls";
        FileInputStream file = new FileInputStream(String.format("%s/%s/%s", baseTemplatePath, GlobalDefs.TEMPLATES_GLOBAL_DIR, "template_11.xls"));
        MultipartFile multipartFile = new MockMultipartFile(templateName, templateName, "application/vnd.ms-excel", IOUtils.toByteArray(file));
        Path path = templateService.create(multipartFile);
        File newFile = new File(baseTemplatePath, String.format("%s/%s", GlobalDefs.TEMPLATES_LOCAL_DIR, templateName));
        Assertions.assertEquals("local", path.getParent().getFileName().toString());
        Assertions.assertEquals("template_99.xls", path.getFileName().toString());
        Assertions.assertTrue(newFile.exists());
        newFile.delete();
    }

    @Test
    void testDelete() throws IOException {
        String templateId = "local/template_1083.xls";
        Path path = Paths.get(baseTemplatePath).resolve(templateId);
        Files.createFile(path);
        Assertions.assertDoesNotThrow(() -> templateService.delete(templateId));
        Assertions.assertFalse(Files.exists(path));
    }
}
